#if defined(__AVR__)
#include <avr/pgmspace.h>
#elif defined(ESP8266)
#include <pgmspace.h>
#else
#define PROGMEM
#endif
#include <Arduino.h>
#include "JaCode.h"

#if defined(__AVR__)
#define FLASH_BYTEREAD(pucDest, pucData, uiLocate) ((unsigned char*)(pucDest))[0] = pgm_read_byte_near(((unsigned char*)(pucData)) + uiLocate);
#elif defined(ESP8266)
#define FLASH_BYTEREAD(pucDest, pucData, uiLocate) ((unsigned char*)(pucDest))[0] = 0; memcpy_P((unsigned char*)(pucData) + uiLocate, pucDest, 1);
#else
#define FLASH_BYTEREAD(pucDest, pucData, uiLocate) ((unsigned char*)(pucDest))[0] = ((unsigned char*)(pucData))[uiLocate];
#endif

typedef struct JaCode {
	unsigned char ucSJISSize;
	unsigned char aucSJIS[2];
	unsigned char ucUTF8Size;
	unsigned char aucUTF8[6];
	unsigned char ucUTF16Size;
	unsigned char aucUTF16[4];
	unsigned char ucEUCSize;
	unsigned char aucEUC[3];
} JaCode_t;

const static JaCode_t tJaCodes[] PROGMEM = {
	{
		1, {0xA0},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		1, {0xA1},
		3, {0xEF, 0xBD, 0xA1},
		2, {0x61, 0xFF},
		2, {0x8E, 0xA1}
	},
	{
		1, {0xA2},
		3, {0xEF, 0xBD, 0xA2},
		2, {0x62, 0xFF},
		2, {0x8E, 0xA2}
	},
	{
		1, {0xA3},
		3, {0xEF, 0xBD, 0xA3},
		2, {0x63, 0xFF},
		2, {0x8E, 0xA3}
	},
	{
		1, {0xA4},
		3, {0xEF, 0xBD, 0xA4},
		2, {0x64, 0xFF},
		2, {0x8E, 0xA4}
	},
	{
		1, {0xA5},
		3, {0xEF, 0xBD, 0xA5},
		2, {0x65, 0xFF},
		2, {0x8E, 0xA5}
	},
	{
		1, {0xA6},
		3, {0xEF, 0xBD, 0xA6},
		2, {0x66, 0xFF},
		2, {0x8E, 0xA6}
	},
	{
		1, {0xA7},
		3, {0xEF, 0xBD, 0xA7},
		2, {0x67, 0xFF},
		2, {0x8E, 0xA7}
	},
	{
		1, {0xA8},
		3, {0xEF, 0xBD, 0xA8},
		2, {0x68, 0xFF},
		2, {0x8E, 0xA8}
	},
	{
		1, {0xA9},
		3, {0xEF, 0xBD, 0xA9},
		2, {0x69, 0xFF},
		2, {0x8E, 0xA9}
	},
	{
		1, {0xAA},
		3, {0xEF, 0xBD, 0xAA},
		2, {0x6A, 0xFF},
		2, {0x8E, 0xAA}
	},
	{
		1, {0xAB},
		3, {0xEF, 0xBD, 0xAB},
		2, {0x6B, 0xFF},
		2, {0x8E, 0xAB}
	},
	{
		1, {0xAC},
		3, {0xEF, 0xBD, 0xAC},
		2, {0x6C, 0xFF},
		2, {0x8E, 0xAC}
	},
	{
		1, {0xAD},
		3, {0xEF, 0xBD, 0xAD},
		2, {0x6D, 0xFF},
		2, {0x8E, 0xAD}
	},
	{
		1, {0xAE},
		3, {0xEF, 0xBD, 0xAE},
		2, {0x6E, 0xFF},
		2, {0x8E, 0xAE}
	},
	{
		1, {0xAF},
		3, {0xEF, 0xBD, 0xAF},
		2, {0x6F, 0xFF},
		2, {0x8E, 0xAF}
	},
	{
		1, {0xB0},
		3, {0xEF, 0xBD, 0xB0},
		2, {0x70, 0xFF},
		2, {0x8E, 0xB0}
	},
	{
		1, {0xB1},
		3, {0xEF, 0xBD, 0xB1},
		2, {0x71, 0xFF},
		2, {0x8E, 0xB1}
	},
	{
		1, {0xB2},
		3, {0xEF, 0xBD, 0xB2},
		2, {0x72, 0xFF},
		2, {0x8E, 0xB2}
	},
	{
		1, {0xB3},
		3, {0xEF, 0xBD, 0xB3},
		2, {0x73, 0xFF},
		2, {0x8E, 0xB3}
	},
	{
		1, {0xB4},
		3, {0xEF, 0xBD, 0xB4},
		2, {0x74, 0xFF},
		2, {0x8E, 0xB4}
	},
	{
		1, {0xB5},
		3, {0xEF, 0xBD, 0xB5},
		2, {0x75, 0xFF},
		2, {0x8E, 0xB5}
	},
	{
		1, {0xB6},
		3, {0xEF, 0xBD, 0xB6},
		2, {0x76, 0xFF},
		2, {0x8E, 0xB6}
	},
	{
		1, {0xB7},
		3, {0xEF, 0xBD, 0xB7},
		2, {0x77, 0xFF},
		2, {0x8E, 0xB7}
	},
	{
		1, {0xB8},
		3, {0xEF, 0xBD, 0xB8},
		2, {0x78, 0xFF},
		2, {0x8E, 0xB8}
	},
	{
		1, {0xB9},
		3, {0xEF, 0xBD, 0xB9},
		2, {0x79, 0xFF},
		2, {0x8E, 0xB9}
	},
	{
		1, {0xBA},
		3, {0xEF, 0xBD, 0xBA},
		2, {0x7A, 0xFF},
		2, {0x8E, 0xBA}
	},
	{
		1, {0xBB},
		3, {0xEF, 0xBD, 0xBB},
		2, {0x7B, 0xFF},
		2, {0x8E, 0xBB}
	},
	{
		1, {0xBC},
		3, {0xEF, 0xBD, 0xBC},
		2, {0x7C, 0xFF},
		2, {0x8E, 0xBC}
	},
	{
		1, {0xBD},
		3, {0xEF, 0xBD, 0xBD},
		2, {0x7D, 0xFF},
		2, {0x8E, 0xBD}
	},
	{
		1, {0xBE},
		3, {0xEF, 0xBD, 0xBE},
		2, {0x7E, 0xFF},
		2, {0x8E, 0xBE}
	},
	{
		1, {0xBF},
		3, {0xEF, 0xBD, 0xBF},
		2, {0x7F, 0xFF},
		2, {0x8E, 0xBF}
	},
	{
		1, {0xC0},
		3, {0xEF, 0xBE, 0x80},
		2, {0x80, 0xFF},
		2, {0x8E, 0xC0}
	},
	{
		1, {0xC1},
		3, {0xEF, 0xBE, 0x81},
		2, {0x81, 0xFF},
		2, {0x8E, 0xC1}
	},
	{
		1, {0xC2},
		3, {0xEF, 0xBE, 0x82},
		2, {0x82, 0xFF},
		2, {0x8E, 0xC2}
	},
	{
		1, {0xC3},
		3, {0xEF, 0xBE, 0x83},
		2, {0x83, 0xFF},
		2, {0x8E, 0xC3}
	},
	{
		1, {0xC4},
		3, {0xEF, 0xBE, 0x84},
		2, {0x84, 0xFF},
		2, {0x8E, 0xC4}
	},
	{
		1, {0xC5},
		3, {0xEF, 0xBE, 0x85},
		2, {0x85, 0xFF},
		2, {0x8E, 0xC5}
	},
	{
		1, {0xC6},
		3, {0xEF, 0xBE, 0x86},
		2, {0x86, 0xFF},
		2, {0x8E, 0xC6}
	},
	{
		1, {0xC7},
		3, {0xEF, 0xBE, 0x87},
		2, {0x87, 0xFF},
		2, {0x8E, 0xC7}
	},
	{
		1, {0xC8},
		3, {0xEF, 0xBE, 0x88},
		2, {0x88, 0xFF},
		2, {0x8E, 0xC8}
	},
	{
		1, {0xC9},
		3, {0xEF, 0xBE, 0x89},
		2, {0x89, 0xFF},
		2, {0x8E, 0xC9}
	},
	{
		1, {0xCA},
		3, {0xEF, 0xBE, 0x8A},
		2, {0x8A, 0xFF},
		2, {0x8E, 0xCA}
	},
	{
		1, {0xCB},
		3, {0xEF, 0xBE, 0x8B},
		2, {0x8B, 0xFF},
		2, {0x8E, 0xCB}
	},
	{
		1, {0xCC},
		3, {0xEF, 0xBE, 0x8C},
		2, {0x8C, 0xFF},
		2, {0x8E, 0xCC}
	},
	{
		1, {0xCD},
		3, {0xEF, 0xBE, 0x8D},
		2, {0x8D, 0xFF},
		2, {0x8E, 0xCD}
	},
	{
		1, {0xCE},
		3, {0xEF, 0xBE, 0x8E},
		2, {0x8E, 0xFF},
		2, {0x8E, 0xCE}
	},
	{
		1, {0xCF},
		3, {0xEF, 0xBE, 0x8F},
		2, {0x8F, 0xFF},
		2, {0x8E, 0xCF}
	},
	{
		1, {0xD0},
		3, {0xEF, 0xBE, 0x90},
		2, {0x90, 0xFF},
		2, {0x8E, 0xD0}
	},
	{
		1, {0xD1},
		3, {0xEF, 0xBE, 0x91},
		2, {0x91, 0xFF},
		2, {0x8E, 0xD1}
	},
	{
		1, {0xD2},
		3, {0xEF, 0xBE, 0x92},
		2, {0x92, 0xFF},
		2, {0x8E, 0xD2}
	},
	{
		1, {0xD3},
		3, {0xEF, 0xBE, 0x93},
		2, {0x93, 0xFF},
		2, {0x8E, 0xD3}
	},
	{
		1, {0xD4},
		3, {0xEF, 0xBE, 0x94},
		2, {0x94, 0xFF},
		2, {0x8E, 0xD4}
	},
	{
		1, {0xD5},
		3, {0xEF, 0xBE, 0x95},
		2, {0x95, 0xFF},
		2, {0x8E, 0xD5}
	},
	{
		1, {0xD6},
		3, {0xEF, 0xBE, 0x96},
		2, {0x96, 0xFF},
		2, {0x8E, 0xD6}
	},
	{
		1, {0xD7},
		3, {0xEF, 0xBE, 0x97},
		2, {0x97, 0xFF},
		2, {0x8E, 0xD7}
	},
	{
		1, {0xD8},
		3, {0xEF, 0xBE, 0x98},
		2, {0x98, 0xFF},
		2, {0x8E, 0xD8}
	},
	{
		1, {0xD9},
		3, {0xEF, 0xBE, 0x99},
		2, {0x99, 0xFF},
		2, {0x8E, 0xD9}
	},
	{
		1, {0xDA},
		3, {0xEF, 0xBE, 0x9A},
		2, {0x9A, 0xFF},
		2, {0x8E, 0xDA}
	},
	{
		1, {0xDB},
		3, {0xEF, 0xBE, 0x9B},
		2, {0x9B, 0xFF},
		2, {0x8E, 0xDB}
	},
	{
		1, {0xDC},
		3, {0xEF, 0xBE, 0x9C},
		2, {0x9C, 0xFF},
		2, {0x8E, 0xDC}
	},
	{
		1, {0xDD},
		3, {0xEF, 0xBE, 0x9D},
		2, {0x9D, 0xFF},
		2, {0x8E, 0xDD}
	},
	{
		1, {0xDE},
		3, {0xEF, 0xBE, 0x9E},
		2, {0x9E, 0xFF},
		2, {0x8E, 0xDE}
	},
	{
		1, {0xDF},
		3, {0xEF, 0xBE, 0x9F},
		2, {0x9F, 0xFF},
		2, {0x8E, 0xDF}
	},
	{
		2, {0x81, 0x40},
		3, {0xE3, 0x80, 0x80},
		2, {0x00, 0x30},
		2, {0xA1, 0xA1}
	},
	{
		2, {0x81, 0x41},
		3, {0xE3, 0x80, 0x81},
		2, {0x01, 0x30},
		2, {0xA1, 0xA2}
	},
	{
		2, {0x81, 0x42},
		3, {0xE3, 0x80, 0x82},
		2, {0x02, 0x30},
		2, {0xA1, 0xA3}
	},
	{
		2, {0x81, 0x43},
		3, {0xEF, 0xBC, 0x8C},
		2, {0x0C, 0xFF},
		2, {0xA1, 0xA4}
	},
	{
		2, {0x81, 0x44},
		3, {0xEF, 0xBC, 0x8E},
		2, {0x0E, 0xFF},
		2, {0xA1, 0xA5}
	},
	{
		2, {0x81, 0x45},
		3, {0xE3, 0x83, 0xBB},
		2, {0xFB, 0x30},
		2, {0xA1, 0xA6}
	},
	{
		2, {0x81, 0x46},
		3, {0xEF, 0xBC, 0x9A},
		2, {0x1A, 0xFF},
		2, {0xA1, 0xA7}
	},
	{
		2, {0x81, 0x47},
		3, {0xEF, 0xBC, 0x9B},
		2, {0x1B, 0xFF},
		2, {0xA1, 0xA8}
	},
	{
		2, {0x81, 0x48},
		3, {0xEF, 0xBC, 0x9F},
		2, {0x1F, 0xFF},
		2, {0xA1, 0xA9}
	},
	{
		2, {0x81, 0x49},
		3, {0xEF, 0xBC, 0x81},
		2, {0x01, 0xFF},
		2, {0xA1, 0xAA}
	},
	{
		2, {0x81, 0x4A},
		3, {0xE3, 0x82, 0x9B},
		2, {0x9B, 0x30},
		2, {0xA1, 0xAB}
	},
	{
		2, {0x81, 0x4B},
		3, {0xE3, 0x82, 0x9C},
		2, {0x9C, 0x30},
		2, {0xA1, 0xAC}
	},
	{
		2, {0x81, 0x4C},
		2, {0xC2, 0xB4},
		2, {0xB4, 0x00},
		2, {0xA1, 0xAD}
	},
	{
		2, {0x81, 0x4D},
		3, {0xEF, 0xBD, 0x80},
		2, {0x40, 0xFF},
		2, {0xA1, 0xAE}
	},
	{
		2, {0x81, 0x4E},
		2, {0xC2, 0xA8},
		2, {0xA8, 0x00},
		2, {0xA1, 0xAF}
	},
	{
		2, {0x81, 0x4F},
		3, {0xEF, 0xBC, 0xBE},
		2, {0x3E, 0xFF},
		2, {0xA1, 0xB0}
	},
	{
		2, {0x81, 0x50},
		3, {0xEF, 0xBF, 0xA3},
		2, {0xE3, 0xFF},
		2, {0xA1, 0xB1}
	},
	{
		2, {0x81, 0x51},
		3, {0xEF, 0xBC, 0xBF},
		2, {0x3F, 0xFF},
		2, {0xA1, 0xB2}
	},
	{
		2, {0x81, 0x52},
		3, {0xE3, 0x83, 0xBD},
		2, {0xFD, 0x30},
		2, {0xA1, 0xB3}
	},
	{
		2, {0x81, 0x53},
		3, {0xE3, 0x83, 0xBE},
		2, {0xFE, 0x30},
		2, {0xA1, 0xB4}
	},
	{
		2, {0x81, 0x54},
		3, {0xE3, 0x82, 0x9D},
		2, {0x9D, 0x30},
		2, {0xA1, 0xB5}
	},
	{
		2, {0x81, 0x55},
		3, {0xE3, 0x82, 0x9E},
		2, {0x9E, 0x30},
		2, {0xA1, 0xB6}
	},
	{
		2, {0x81, 0x56},
		3, {0xE3, 0x80, 0x83},
		2, {0x03, 0x30},
		2, {0xA1, 0xB7}
	},
	{
		2, {0x81, 0x57},
		3, {0xE4, 0xBB, 0x9D},
		2, {0xDD, 0x4E},
		2, {0xA1, 0xB8}
	},
	{
		2, {0x81, 0x58},
		3, {0xE3, 0x80, 0x85},
		2, {0x05, 0x30},
		2, {0xA1, 0xB9}
	},
	{
		2, {0x81, 0x59},
		3, {0xE3, 0x80, 0x86},
		2, {0x06, 0x30},
		2, {0xA1, 0xBA}
	},
	{
		2, {0x81, 0x5A},
		3, {0xE3, 0x80, 0x87},
		2, {0x07, 0x30},
		2, {0xA1, 0xBB}
	},
	{
		2, {0x81, 0x5B},
		3, {0xE3, 0x83, 0xBC},
		2, {0xFC, 0x30},
		2, {0xA1, 0xBC}
	},
	{
		2, {0x81, 0x5C},
		3, {0xE2, 0x80, 0x94},
		2, {0x14, 0x20},
		0, {0x00}
	},
	{
		2, {0x81, 0x5D},
		3, {0xE2, 0x80, 0x90},
		2, {0x10, 0x20},
		2, {0xA1, 0xBE}
	},
	{
		2, {0x81, 0x5E},
		3, {0xEF, 0xBC, 0x8F},
		2, {0x0F, 0xFF},
		2, {0xA1, 0xBF}
	},
	{
		2, {0x81, 0x5F},
		3, {0xEF, 0xBC, 0xBC},
		2, {0x3C, 0xFF},
		2, {0xA1, 0xC0}
	},
	{
		2, {0x81, 0x60},
		3, {0xE3, 0x80, 0x9C},
		2, {0x1C, 0x30},
		2, {0xA1, 0xC1}
	},
	{
		2, {0x81, 0x61},
		3, {0xE2, 0x80, 0x96},
		2, {0x16, 0x20},
		2, {0xA1, 0xC2}
	},
	{
		2, {0x81, 0x62},
		3, {0xEF, 0xBD, 0x9C},
		2, {0x5C, 0xFF},
		2, {0xA1, 0xC3}
	},
	{
		2, {0x81, 0x63},
		3, {0xE2, 0x80, 0xA6},
		2, {0x26, 0x20},
		2, {0xA1, 0xC4}
	},
	{
		2, {0x81, 0x64},
		3, {0xE2, 0x80, 0xA5},
		2, {0x25, 0x20},
		2, {0xA1, 0xC5}
	},
	{
		2, {0x81, 0x65},
		3, {0xE2, 0x80, 0x98},
		2, {0x18, 0x20},
		2, {0xA1, 0xC6}
	},
	{
		2, {0x81, 0x66},
		3, {0xE2, 0x80, 0x99},
		2, {0x19, 0x20},
		2, {0xA1, 0xC7}
	},
	{
		2, {0x81, 0x67},
		3, {0xE2, 0x80, 0x9C},
		2, {0x1C, 0x20},
		2, {0xA1, 0xC8}
	},
	{
		2, {0x81, 0x68},
		3, {0xE2, 0x80, 0x9D},
		2, {0x1D, 0x20},
		2, {0xA1, 0xC9}
	},
	{
		2, {0x81, 0x69},
		3, {0xEF, 0xBC, 0x88},
		2, {0x08, 0xFF},
		2, {0xA1, 0xCA}
	},
	{
		2, {0x81, 0x6A},
		3, {0xEF, 0xBC, 0x89},
		2, {0x09, 0xFF},
		2, {0xA1, 0xCB}
	},
	{
		2, {0x81, 0x6B},
		3, {0xE3, 0x80, 0x94},
		2, {0x14, 0x30},
		2, {0xA1, 0xCC}
	},
	{
		2, {0x81, 0x6C},
		3, {0xE3, 0x80, 0x95},
		2, {0x15, 0x30},
		2, {0xA1, 0xCD}
	},
	{
		2, {0x81, 0x6D},
		3, {0xEF, 0xBC, 0xBB},
		2, {0x3B, 0xFF},
		2, {0xA1, 0xCE}
	},
	{
		2, {0x81, 0x6E},
		3, {0xEF, 0xBC, 0xBD},
		2, {0x3D, 0xFF},
		2, {0xA1, 0xCF}
	},
	{
		2, {0x81, 0x6F},
		3, {0xEF, 0xBD, 0x9B},
		2, {0x5B, 0xFF},
		2, {0xA1, 0xD0}
	},
	{
		2, {0x81, 0x70},
		3, {0xEF, 0xBD, 0x9D},
		2, {0x5D, 0xFF},
		2, {0xA1, 0xD1}
	},
	{
		2, {0x81, 0x71},
		3, {0xE3, 0x80, 0x88},
		2, {0x08, 0x30},
		2, {0xA1, 0xD2}
	},
	{
		2, {0x81, 0x72},
		3, {0xE3, 0x80, 0x89},
		2, {0x09, 0x30},
		2, {0xA1, 0xD3}
	},
	{
		2, {0x81, 0x73},
		3, {0xE3, 0x80, 0x8A},
		2, {0x0A, 0x30},
		2, {0xA1, 0xD4}
	},
	{
		2, {0x81, 0x74},
		3, {0xE3, 0x80, 0x8B},
		2, {0x0B, 0x30},
		2, {0xA1, 0xD5}
	},
	{
		2, {0x81, 0x75},
		3, {0xE3, 0x80, 0x8C},
		2, {0x0C, 0x30},
		2, {0xA1, 0xD6}
	},
	{
		2, {0x81, 0x76},
		3, {0xE3, 0x80, 0x8D},
		2, {0x0D, 0x30},
		2, {0xA1, 0xD7}
	},
	{
		2, {0x81, 0x77},
		3, {0xE3, 0x80, 0x8E},
		2, {0x0E, 0x30},
		2, {0xA1, 0xD8}
	},
	{
		2, {0x81, 0x78},
		3, {0xE3, 0x80, 0x8F},
		2, {0x0F, 0x30},
		2, {0xA1, 0xD9}
	},
	{
		2, {0x81, 0x79},
		3, {0xE3, 0x80, 0x90},
		2, {0x10, 0x30},
		2, {0xA1, 0xDA}
	},
	{
		2, {0x81, 0x7A},
		3, {0xE3, 0x80, 0x91},
		2, {0x11, 0x30},
		2, {0xA1, 0xDB}
	},
	{
		2, {0x81, 0x7B},
		3, {0xEF, 0xBC, 0x8B},
		2, {0x0B, 0xFF},
		2, {0xA1, 0xDC}
	},
	{
		2, {0x81, 0x7C},
		3, {0xE2, 0x88, 0x92},
		2, {0x12, 0x22},
		2, {0xA1, 0xDD}
	},
	{
		2, {0x81, 0x7D},
		2, {0xC2, 0xB1},
		2, {0xB1, 0x00},
		2, {0xA1, 0xDE}
	},
	{
		2, {0x81, 0x7E},
		2, {0xC3, 0x97},
		2, {0xD7, 0x00},
		2, {0xA1, 0xDF}
	},
	{
		2, {0x81, 0x80},
		2, {0xC3, 0xB7},
		2, {0xF7, 0x00},
		2, {0xA1, 0xE0}
	},
	{
		2, {0x81, 0x81},
		3, {0xEF, 0xBC, 0x9D},
		2, {0x1D, 0xFF},
		2, {0xA1, 0xE1}
	},
	{
		2, {0x81, 0x82},
		3, {0xE2, 0x89, 0xA0},
		2, {0x60, 0x22},
		2, {0xA1, 0xE2}
	},
	{
		2, {0x81, 0x83},
		3, {0xEF, 0xBC, 0x9C},
		2, {0x1C, 0xFF},
		2, {0xA1, 0xE3}
	},
	{
		2, {0x81, 0x84},
		3, {0xEF, 0xBC, 0x9E},
		2, {0x1E, 0xFF},
		2, {0xA1, 0xE4}
	},
	{
		2, {0x81, 0x85},
		3, {0xE2, 0x89, 0xA6},
		2, {0x66, 0x22},
		2, {0xA1, 0xE5}
	},
	{
		2, {0x81, 0x86},
		3, {0xE2, 0x89, 0xA7},
		2, {0x67, 0x22},
		2, {0xA1, 0xE6}
	},
	{
		2, {0x81, 0x87},
		3, {0xE2, 0x88, 0x9E},
		2, {0x1E, 0x22},
		2, {0xA1, 0xE7}
	},
	{
		2, {0x81, 0x88},
		3, {0xE2, 0x88, 0xB4},
		2, {0x34, 0x22},
		2, {0xA1, 0xE8}
	},
	{
		2, {0x81, 0x89},
		3, {0xE2, 0x99, 0x82},
		2, {0x42, 0x26},
		2, {0xA1, 0xE9}
	},
	{
		2, {0x81, 0x8A},
		3, {0xE2, 0x99, 0x80},
		2, {0x40, 0x26},
		2, {0xA1, 0xEA}
	},
	{
		2, {0x81, 0x8B},
		2, {0xC2, 0xB0},
		2, {0xB0, 0x00},
		2, {0xA1, 0xEB}
	},
	{
		2, {0x81, 0x8C},
		3, {0xE2, 0x80, 0xB2},
		2, {0x32, 0x20},
		2, {0xA1, 0xEC}
	},
	{
		2, {0x81, 0x8D},
		3, {0xE2, 0x80, 0xB3},
		2, {0x33, 0x20},
		2, {0xA1, 0xED}
	},
	{
		2, {0x81, 0x8E},
		3, {0xE2, 0x84, 0x83},
		2, {0x03, 0x21},
		2, {0xA1, 0xEE}
	},
	{
		2, {0x81, 0x8F},
		3, {0xEF, 0xBF, 0xA5},
		2, {0xE5, 0xFF},
		2, {0xA1, 0xEF}
	},
	{
		2, {0x81, 0x90},
		3, {0xEF, 0xBC, 0x84},
		2, {0x04, 0xFF},
		2, {0xA1, 0xF0}
	},
	{
		2, {0x81, 0x91},
		2, {0xC2, 0xA2},
		2, {0xA2, 0x00},
		2, {0xA1, 0xF1}
	},
	{
		2, {0x81, 0x92},
		2, {0xC2, 0xA3},
		2, {0xA3, 0x00},
		2, {0xA1, 0xF2}
	},
	{
		2, {0x81, 0x93},
		3, {0xEF, 0xBC, 0x85},
		2, {0x05, 0xFF},
		2, {0xA1, 0xF3}
	},
	{
		2, {0x81, 0x94},
		3, {0xEF, 0xBC, 0x83},
		2, {0x03, 0xFF},
		2, {0xA1, 0xF4}
	},
	{
		2, {0x81, 0x95},
		3, {0xEF, 0xBC, 0x86},
		2, {0x06, 0xFF},
		2, {0xA1, 0xF5}
	},
	{
		2, {0x81, 0x96},
		3, {0xEF, 0xBC, 0x8A},
		2, {0x0A, 0xFF},
		2, {0xA1, 0xF6}
	},
	{
		2, {0x81, 0x97},
		3, {0xEF, 0xBC, 0xA0},
		2, {0x20, 0xFF},
		2, {0xA1, 0xF7}
	},
	{
		2, {0x81, 0x98},
		2, {0xC2, 0xA7},
		2, {0xA7, 0x00},
		2, {0xA1, 0xF8}
	},
	{
		2, {0x81, 0x99},
		3, {0xE2, 0x98, 0x86},
		2, {0x06, 0x26},
		2, {0xA1, 0xF9}
	},
	{
		2, {0x81, 0x9A},
		3, {0xE2, 0x98, 0x85},
		2, {0x05, 0x26},
		2, {0xA1, 0xFA}
	},
	{
		2, {0x81, 0x9B},
		3, {0xE2, 0x97, 0x8B},
		2, {0xCB, 0x25},
		2, {0xA1, 0xFB}
	},
	{
		2, {0x81, 0x9C},
		3, {0xE2, 0x97, 0x8F},
		2, {0xCF, 0x25},
		2, {0xA1, 0xFC}
	},
	{
		2, {0x81, 0x9D},
		3, {0xE2, 0x97, 0x8E},
		2, {0xCE, 0x25},
		2, {0xA1, 0xFD}
	},
	{
		2, {0x81, 0x9E},
		3, {0xE2, 0x97, 0x87},
		2, {0xC7, 0x25},
		2, {0xA1, 0xFE}
	},
	{
		2, {0x81, 0x9F},
		3, {0xE2, 0x97, 0x86},
		2, {0xC6, 0x25},
		2, {0xA2, 0xA1}
	},
	{
		2, {0x81, 0xA0},
		3, {0xE2, 0x96, 0xA1},
		2, {0xA1, 0x25},
		2, {0xA2, 0xA2}
	},
	{
		2, {0x81, 0xA1},
		3, {0xE2, 0x96, 0xA0},
		2, {0xA0, 0x25},
		2, {0xA2, 0xA3}
	},
	{
		2, {0x81, 0xA2},
		3, {0xE2, 0x96, 0xB3},
		2, {0xB3, 0x25},
		2, {0xA2, 0xA4}
	},
	{
		2, {0x81, 0xA3},
		3, {0xE2, 0x96, 0xB2},
		2, {0xB2, 0x25},
		2, {0xA2, 0xA5}
	},
	{
		2, {0x81, 0xA4},
		3, {0xE2, 0x96, 0xBD},
		2, {0xBD, 0x25},
		2, {0xA2, 0xA6}
	},
	{
		2, {0x81, 0xA5},
		3, {0xE2, 0x96, 0xBC},
		2, {0xBC, 0x25},
		2, {0xA2, 0xA7}
	},
	{
		2, {0x81, 0xA6},
		3, {0xE2, 0x80, 0xBB},
		2, {0x3B, 0x20},
		2, {0xA2, 0xA8}
	},
	{
		2, {0x81, 0xA7},
		3, {0xE3, 0x80, 0x92},
		2, {0x12, 0x30},
		2, {0xA2, 0xA9}
	},
	{
		2, {0x81, 0xA8},
		3, {0xE2, 0x86, 0x92},
		2, {0x92, 0x21},
		2, {0xA2, 0xAA}
	},
	{
		2, {0x81, 0xA9},
		3, {0xE2, 0x86, 0x90},
		2, {0x90, 0x21},
		2, {0xA2, 0xAB}
	},
	{
		2, {0x81, 0xAA},
		3, {0xE2, 0x86, 0x91},
		2, {0x91, 0x21},
		2, {0xA2, 0xAC}
	},
	{
		2, {0x81, 0xAB},
		3, {0xE2, 0x86, 0x93},
		2, {0x93, 0x21},
		2, {0xA2, 0xAD}
	},
	{
		2, {0x81, 0xAC},
		3, {0xE3, 0x80, 0x93},
		2, {0x13, 0x30},
		2, {0xA2, 0xAE}
	},
	{
		2, {0x81, 0xAD},
		3, {0xEF, 0xBC, 0x87},
		2, {0x07, 0xFF},
		0, {0x00}
	},
	{
		2, {0x81, 0xAE},
		3, {0xEF, 0xBC, 0x82},
		2, {0x02, 0xFF},
		0, {0x00}
	},
	{
		2, {0x81, 0xAF},
		3, {0xEF, 0xBC, 0x8D},
		2, {0x0D, 0xFF},
		0, {0x00}
	},
	{
		2, {0x81, 0xB0},
		3, {0xEF, 0xBD, 0x9E},
		2, {0x5E, 0xFF},
		3, {0x8F, 0xA2, 0xB7}
	},
	{
		2, {0x81, 0xB1},
		3, {0xE3, 0x80, 0xB3},
		2, {0x33, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xB2},
		3, {0xE3, 0x80, 0xB4},
		2, {0x34, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xB3},
		3, {0xE3, 0x80, 0xB5},
		2, {0x35, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xB4},
		3, {0xE3, 0x80, 0xBB},
		2, {0x3B, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xB5},
		3, {0xE3, 0x80, 0xBC},
		2, {0x3C, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xB6},
		3, {0xE3, 0x83, 0xBF},
		2, {0xFF, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xB7},
		3, {0xE3, 0x82, 0x9F},
		2, {0x9F, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xB8},
		3, {0xE2, 0x88, 0x88},
		2, {0x08, 0x22},
		2, {0xA2, 0xBA}
	},
	{
		2, {0x81, 0xB9},
		3, {0xE2, 0x88, 0x8B},
		2, {0x0B, 0x22},
		2, {0xA2, 0xBB}
	},
	{
		2, {0x81, 0xBA},
		3, {0xE2, 0x8A, 0x86},
		2, {0x86, 0x22},
		2, {0xA2, 0xBC}
	},
	{
		2, {0x81, 0xBB},
		3, {0xE2, 0x8A, 0x87},
		2, {0x87, 0x22},
		2, {0xA2, 0xBD}
	},
	{
		2, {0x81, 0xBC},
		3, {0xE2, 0x8A, 0x82},
		2, {0x82, 0x22},
		2, {0xA2, 0xBE}
	},
	{
		2, {0x81, 0xBD},
		3, {0xE2, 0x8A, 0x83},
		2, {0x83, 0x22},
		2, {0xA2, 0xBF}
	},
	{
		2, {0x81, 0xBE},
		3, {0xE2, 0x88, 0xAA},
		2, {0x2A, 0x22},
		2, {0xA2, 0xC0}
	},
	{
		2, {0x81, 0xBF},
		3, {0xE2, 0x88, 0xA9},
		2, {0x29, 0x22},
		2, {0xA2, 0xC1}
	},
	{
		2, {0x81, 0xC0},
		3, {0xE2, 0x8A, 0x84},
		2, {0x84, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xC1},
		3, {0xE2, 0x8A, 0x85},
		2, {0x85, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xC2},
		3, {0xE2, 0x8A, 0x8A},
		2, {0x8A, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xC3},
		3, {0xE2, 0x8A, 0x8B},
		2, {0x8B, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xC4},
		3, {0xE2, 0x88, 0x89},
		2, {0x09, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xC5},
		3, {0xE2, 0x88, 0x85},
		2, {0x05, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xC6},
		3, {0xE2, 0x8C, 0x85},
		2, {0x05, 0x23},
		0, {0x00}
	},
	{
		2, {0x81, 0xC7},
		3, {0xE2, 0x8C, 0x86},
		2, {0x06, 0x23},
		0, {0x00}
	},
	{
		2, {0x81, 0xC8},
		3, {0xE2, 0x88, 0xA7},
		2, {0x27, 0x22},
		2, {0xA2, 0xCA}
	},
	{
		2, {0x81, 0xC9},
		3, {0xE2, 0x88, 0xA8},
		2, {0x28, 0x22},
		2, {0xA2, 0xCB}
	},
	{
		2, {0x81, 0xCA},
		2, {0xC2, 0xAC},
		2, {0xAC, 0x00},
		2, {0xA2, 0xCC}
	},
	{
		2, {0x81, 0xCB},
		3, {0xE2, 0x87, 0x92},
		2, {0xD2, 0x21},
		2, {0xA2, 0xCD}
	},
	{
		2, {0x81, 0xCC},
		3, {0xE2, 0x87, 0x94},
		2, {0xD4, 0x21},
		2, {0xA2, 0xCE}
	},
	{
		2, {0x81, 0xCD},
		3, {0xE2, 0x88, 0x80},
		2, {0x00, 0x22},
		2, {0xA2, 0xCF}
	},
	{
		2, {0x81, 0xCE},
		3, {0xE2, 0x88, 0x83},
		2, {0x03, 0x22},
		2, {0xA2, 0xD0}
	},
	{
		2, {0x81, 0xCF},
		3, {0xE2, 0x8A, 0x95},
		2, {0x95, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xD0},
		3, {0xE2, 0x8A, 0x96},
		2, {0x96, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xD1},
		3, {0xE2, 0x8A, 0x97},
		2, {0x97, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xD2},
		3, {0xE2, 0x88, 0xA5},
		2, {0x25, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xD3},
		3, {0xE2, 0x88, 0xA6},
		2, {0x26, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xD4},
		3, {0xEF, 0xBD, 0x9F},
		2, {0x5F, 0xFF},
		0, {0x00}
	},
	{
		2, {0x81, 0xD5},
		3, {0xEF, 0xBD, 0xA0},
		2, {0x60, 0xFF},
		0, {0x00}
	},
	{
		2, {0x81, 0xD6},
		3, {0xE3, 0x80, 0x98},
		2, {0x18, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xD7},
		3, {0xE3, 0x80, 0x99},
		2, {0x19, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xD8},
		3, {0xE3, 0x80, 0x96},
		2, {0x16, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xD9},
		3, {0xE3, 0x80, 0x97},
		2, {0x17, 0x30},
		0, {0x00}
	},
	{
		2, {0x81, 0xDA},
		3, {0xE2, 0x88, 0xA0},
		2, {0x20, 0x22},
		2, {0xA2, 0xDC}
	},
	{
		2, {0x81, 0xDB},
		3, {0xE2, 0x8A, 0xA5},
		2, {0xA5, 0x22},
		2, {0xA2, 0xDD}
	},
	{
		2, {0x81, 0xDC},
		3, {0xE2, 0x8C, 0x92},
		2, {0x12, 0x23},
		2, {0xA2, 0xDE}
	},
	{
		2, {0x81, 0xDD},
		3, {0xE2, 0x88, 0x82},
		2, {0x02, 0x22},
		2, {0xA2, 0xDF}
	},
	{
		2, {0x81, 0xDE},
		3, {0xE2, 0x88, 0x87},
		2, {0x07, 0x22},
		2, {0xA2, 0xE0}
	},
	{
		2, {0x81, 0xDF},
		3, {0xE2, 0x89, 0xA1},
		2, {0x61, 0x22},
		2, {0xA2, 0xE1}
	},
	{
		2, {0x81, 0xE0},
		3, {0xE2, 0x89, 0x92},
		2, {0x52, 0x22},
		2, {0xA2, 0xE2}
	},
	{
		2, {0x81, 0xE1},
		3, {0xE2, 0x89, 0xAA},
		2, {0x6A, 0x22},
		2, {0xA2, 0xE3}
	},
	{
		2, {0x81, 0xE2},
		3, {0xE2, 0x89, 0xAB},
		2, {0x6B, 0x22},
		2, {0xA2, 0xE4}
	},
	{
		2, {0x81, 0xE3},
		3, {0xE2, 0x88, 0x9A},
		2, {0x1A, 0x22},
		2, {0xA2, 0xE5}
	},
	{
		2, {0x81, 0xE4},
		3, {0xE2, 0x88, 0xBD},
		2, {0x3D, 0x22},
		2, {0xA2, 0xE6}
	},
	{
		2, {0x81, 0xE5},
		3, {0xE2, 0x88, 0x9D},
		2, {0x1D, 0x22},
		2, {0xA2, 0xE7}
	},
	{
		2, {0x81, 0xE6},
		3, {0xE2, 0x88, 0xB5},
		2, {0x35, 0x22},
		2, {0xA2, 0xE8}
	},
	{
		2, {0x81, 0xE7},
		3, {0xE2, 0x88, 0xAB},
		2, {0x2B, 0x22},
		2, {0xA2, 0xE9}
	},
	{
		2, {0x81, 0xE8},
		3, {0xE2, 0x88, 0xAC},
		2, {0x2C, 0x22},
		2, {0xA2, 0xEA}
	},
	{
		2, {0x81, 0xE9},
		3, {0xE2, 0x89, 0xA2},
		2, {0x62, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xEA},
		3, {0xE2, 0x89, 0x83},
		2, {0x43, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xEB},
		3, {0xE2, 0x89, 0x85},
		2, {0x45, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xEC},
		3, {0xE2, 0x89, 0x88},
		2, {0x48, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xED},
		3, {0xE2, 0x89, 0xB6},
		2, {0x76, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xEE},
		3, {0xE2, 0x89, 0xB7},
		2, {0x77, 0x22},
		0, {0x00}
	},
	{
		2, {0x81, 0xEF},
		3, {0xE2, 0x86, 0x94},
		2, {0x94, 0x21},
		0, {0x00}
	},
	{
		2, {0x81, 0xF0},
		3, {0xE2, 0x84, 0xAB},
		2, {0x2B, 0x21},
		2, {0xA2, 0xF2}
	},
	{
		2, {0x81, 0xF1},
		3, {0xE2, 0x80, 0xB0},
		2, {0x30, 0x20},
		2, {0xA2, 0xF3}
	},
	{
		2, {0x81, 0xF2},
		3, {0xE2, 0x99, 0xAF},
		2, {0x6F, 0x26},
		2, {0xA2, 0xF4}
	},
	{
		2, {0x81, 0xF3},
		3, {0xE2, 0x99, 0xAD},
		2, {0x6D, 0x26},
		2, {0xA2, 0xF5}
	},
	{
		2, {0x81, 0xF4},
		3, {0xE2, 0x99, 0xAA},
		2, {0x6A, 0x26},
		2, {0xA2, 0xF6}
	},
	{
		2, {0x81, 0xF5},
		3, {0xE2, 0x80, 0xA0},
		2, {0x20, 0x20},
		2, {0xA2, 0xF7}
	},
	{
		2, {0x81, 0xF6},
		3, {0xE2, 0x80, 0xA1},
		2, {0x21, 0x20},
		2, {0xA2, 0xF8}
	},
	{
		2, {0x81, 0xF7},
		2, {0xC2, 0xB6},
		2, {0xB6, 0x00},
		2, {0xA2, 0xF9}
	},
	{
		2, {0x81, 0xF8},
		3, {0xE2, 0x99, 0xAE},
		2, {0x6E, 0x26},
		0, {0x00}
	},
	{
		2, {0x81, 0xF9},
		3, {0xE2, 0x99, 0xAB},
		2, {0x6B, 0x26},
		0, {0x00}
	},
	{
		2, {0x81, 0xFA},
		3, {0xE2, 0x99, 0xAC},
		2, {0x6C, 0x26},
		0, {0x00}
	},
	{
		2, {0x81, 0xFB},
		3, {0xE2, 0x99, 0xA9},
		2, {0x69, 0x26},
		0, {0x00}
	},
	{
		2, {0x81, 0xFC},
		3, {0xE2, 0x97, 0xAF},
		2, {0xEF, 0x25},
		2, {0xA2, 0xFE}
	},
	{
		2, {0x82, 0x40},
		3, {0xE2, 0x96, 0xB7},
		2, {0xB7, 0x25},
		0, {0x00}
	},
	{
		2, {0x82, 0x41},
		3, {0xE2, 0x96, 0xB6},
		2, {0xB6, 0x25},
		0, {0x00}
	},
	{
		2, {0x82, 0x42},
		3, {0xE2, 0x97, 0x81},
		2, {0xC1, 0x25},
		0, {0x00}
	},
	{
		2, {0x82, 0x43},
		3, {0xE2, 0x97, 0x80},
		2, {0xC0, 0x25},
		0, {0x00}
	},
	{
		2, {0x82, 0x44},
		3, {0xE2, 0x86, 0x97},
		2, {0x97, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x45},
		3, {0xE2, 0x86, 0x98},
		2, {0x98, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x46},
		3, {0xE2, 0x86, 0x96},
		2, {0x96, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x47},
		3, {0xE2, 0x86, 0x99},
		2, {0x99, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x48},
		3, {0xE2, 0x87, 0x84},
		2, {0xC4, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x49},
		3, {0xE2, 0x87, 0xA8},
		2, {0xE8, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x4A},
		3, {0xE2, 0x87, 0xA6},
		2, {0xE6, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x4B},
		3, {0xE2, 0x87, 0xA7},
		2, {0xE7, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x4C},
		3, {0xE2, 0x87, 0xA9},
		2, {0xE9, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x4D},
		3, {0xE2, 0xA4, 0xB4},
		2, {0x34, 0x29},
		0, {0x00}
	},
	{
		2, {0x82, 0x4E},
		3, {0xE2, 0xA4, 0xB5},
		2, {0x35, 0x29},
		0, {0x00}
	},
	{
		2, {0x82, 0x4F},
		3, {0xEF, 0xBC, 0x90},
		2, {0x10, 0xFF},
		2, {0xA3, 0xB0}
	},
	{
		2, {0x82, 0x50},
		3, {0xEF, 0xBC, 0x91},
		2, {0x11, 0xFF},
		2, {0xA3, 0xB1}
	},
	{
		2, {0x82, 0x51},
		3, {0xEF, 0xBC, 0x92},
		2, {0x12, 0xFF},
		2, {0xA3, 0xB2}
	},
	{
		2, {0x82, 0x52},
		3, {0xEF, 0xBC, 0x93},
		2, {0x13, 0xFF},
		2, {0xA3, 0xB3}
	},
	{
		2, {0x82, 0x53},
		3, {0xEF, 0xBC, 0x94},
		2, {0x14, 0xFF},
		2, {0xA3, 0xB4}
	},
	{
		2, {0x82, 0x54},
		3, {0xEF, 0xBC, 0x95},
		2, {0x15, 0xFF},
		2, {0xA3, 0xB5}
	},
	{
		2, {0x82, 0x55},
		3, {0xEF, 0xBC, 0x96},
		2, {0x16, 0xFF},
		2, {0xA3, 0xB6}
	},
	{
		2, {0x82, 0x56},
		3, {0xEF, 0xBC, 0x97},
		2, {0x17, 0xFF},
		2, {0xA3, 0xB7}
	},
	{
		2, {0x82, 0x57},
		3, {0xEF, 0xBC, 0x98},
		2, {0x18, 0xFF},
		2, {0xA3, 0xB8}
	},
	{
		2, {0x82, 0x58},
		3, {0xEF, 0xBC, 0x99},
		2, {0x19, 0xFF},
		2, {0xA3, 0xB9}
	},
	{
		2, {0x82, 0x59},
		3, {0xE2, 0xA6, 0xBF},
		2, {0xBF, 0x29},
		0, {0x00}
	},
	{
		2, {0x82, 0x5A},
		3, {0xE2, 0x97, 0x89},
		2, {0xC9, 0x25},
		0, {0x00}
	},
	{
		2, {0x82, 0x5B},
		3, {0xE3, 0x80, 0xBD},
		2, {0x3D, 0x30},
		0, {0x00}
	},
	{
		2, {0x82, 0x5C},
		3, {0xEF, 0xB9, 0x86},
		2, {0x46, 0xFE},
		0, {0x00}
	},
	{
		2, {0x82, 0x5D},
		3, {0xEF, 0xB9, 0x85},
		2, {0x45, 0xFE},
		0, {0x00}
	},
	{
		2, {0x82, 0x5E},
		3, {0xE2, 0x97, 0xA6},
		2, {0xE6, 0x25},
		0, {0x00}
	},
	{
		2, {0x82, 0x5F},
		3, {0xE2, 0x80, 0xA2},
		2, {0x22, 0x20},
		0, {0x00}
	},
	{
		2, {0x82, 0x60},
		3, {0xEF, 0xBC, 0xA1},
		2, {0x21, 0xFF},
		2, {0xA3, 0xC1}
	},
	{
		2, {0x82, 0x61},
		3, {0xEF, 0xBC, 0xA2},
		2, {0x22, 0xFF},
		2, {0xA3, 0xC2}
	},
	{
		2, {0x82, 0x62},
		3, {0xEF, 0xBC, 0xA3},
		2, {0x23, 0xFF},
		2, {0xA3, 0xC3}
	},
	{
		2, {0x82, 0x63},
		3, {0xEF, 0xBC, 0xA4},
		2, {0x24, 0xFF},
		2, {0xA3, 0xC4}
	},
	{
		2, {0x82, 0x64},
		3, {0xEF, 0xBC, 0xA5},
		2, {0x25, 0xFF},
		2, {0xA3, 0xC5}
	},
	{
		2, {0x82, 0x65},
		3, {0xEF, 0xBC, 0xA6},
		2, {0x26, 0xFF},
		2, {0xA3, 0xC6}
	},
	{
		2, {0x82, 0x66},
		3, {0xEF, 0xBC, 0xA7},
		2, {0x27, 0xFF},
		2, {0xA3, 0xC7}
	},
	{
		2, {0x82, 0x67},
		3, {0xEF, 0xBC, 0xA8},
		2, {0x28, 0xFF},
		2, {0xA3, 0xC8}
	},
	{
		2, {0x82, 0x68},
		3, {0xEF, 0xBC, 0xA9},
		2, {0x29, 0xFF},
		2, {0xA3, 0xC9}
	},
	{
		2, {0x82, 0x69},
		3, {0xEF, 0xBC, 0xAA},
		2, {0x2A, 0xFF},
		2, {0xA3, 0xCA}
	},
	{
		2, {0x82, 0x6A},
		3, {0xEF, 0xBC, 0xAB},
		2, {0x2B, 0xFF},
		2, {0xA3, 0xCB}
	},
	{
		2, {0x82, 0x6B},
		3, {0xEF, 0xBC, 0xAC},
		2, {0x2C, 0xFF},
		2, {0xA3, 0xCC}
	},
	{
		2, {0x82, 0x6C},
		3, {0xEF, 0xBC, 0xAD},
		2, {0x2D, 0xFF},
		2, {0xA3, 0xCD}
	},
	{
		2, {0x82, 0x6D},
		3, {0xEF, 0xBC, 0xAE},
		2, {0x2E, 0xFF},
		2, {0xA3, 0xCE}
	},
	{
		2, {0x82, 0x6E},
		3, {0xEF, 0xBC, 0xAF},
		2, {0x2F, 0xFF},
		2, {0xA3, 0xCF}
	},
	{
		2, {0x82, 0x6F},
		3, {0xEF, 0xBC, 0xB0},
		2, {0x30, 0xFF},
		2, {0xA3, 0xD0}
	},
	{
		2, {0x82, 0x70},
		3, {0xEF, 0xBC, 0xB1},
		2, {0x31, 0xFF},
		2, {0xA3, 0xD1}
	},
	{
		2, {0x82, 0x71},
		3, {0xEF, 0xBC, 0xB2},
		2, {0x32, 0xFF},
		2, {0xA3, 0xD2}
	},
	{
		2, {0x82, 0x72},
		3, {0xEF, 0xBC, 0xB3},
		2, {0x33, 0xFF},
		2, {0xA3, 0xD3}
	},
	{
		2, {0x82, 0x73},
		3, {0xEF, 0xBC, 0xB4},
		2, {0x34, 0xFF},
		2, {0xA3, 0xD4}
	},
	{
		2, {0x82, 0x74},
		3, {0xEF, 0xBC, 0xB5},
		2, {0x35, 0xFF},
		2, {0xA3, 0xD5}
	},
	{
		2, {0x82, 0x75},
		3, {0xEF, 0xBC, 0xB6},
		2, {0x36, 0xFF},
		2, {0xA3, 0xD6}
	},
	{
		2, {0x82, 0x76},
		3, {0xEF, 0xBC, 0xB7},
		2, {0x37, 0xFF},
		2, {0xA3, 0xD7}
	},
	{
		2, {0x82, 0x77},
		3, {0xEF, 0xBC, 0xB8},
		2, {0x38, 0xFF},
		2, {0xA3, 0xD8}
	},
	{
		2, {0x82, 0x78},
		3, {0xEF, 0xBC, 0xB9},
		2, {0x39, 0xFF},
		2, {0xA3, 0xD9}
	},
	{
		2, {0x82, 0x79},
		3, {0xEF, 0xBC, 0xBA},
		2, {0x3A, 0xFF},
		2, {0xA3, 0xDA}
	},
	{
		2, {0x82, 0x7A},
		3, {0xE2, 0x88, 0x93},
		2, {0x13, 0x22},
		0, {0x00}
	},
	{
		2, {0x82, 0x7B},
		3, {0xE2, 0x84, 0xB5},
		2, {0x35, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x7C},
		3, {0xE2, 0x84, 0x8F},
		2, {0x0F, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x7D},
		3, {0xE3, 0x8F, 0x8B},
		2, {0xCB, 0x33},
		0, {0x00}
	},
	{
		2, {0x82, 0x7E},
		3, {0xE2, 0x84, 0x93},
		2, {0x13, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x80},
		3, {0xE2, 0x84, 0xA7},
		2, {0x27, 0x21},
		0, {0x00}
	},
	{
		2, {0x82, 0x81},
		3, {0xEF, 0xBD, 0x81},
		2, {0x41, 0xFF},
		2, {0xA3, 0xE1}
	},
	{
		2, {0x82, 0x82},
		3, {0xEF, 0xBD, 0x82},
		2, {0x42, 0xFF},
		2, {0xA3, 0xE2}
	},
	{
		2, {0x82, 0x83},
		3, {0xEF, 0xBD, 0x83},
		2, {0x43, 0xFF},
		2, {0xA3, 0xE3}
	},
	{
		2, {0x82, 0x84},
		3, {0xEF, 0xBD, 0x84},
		2, {0x44, 0xFF},
		2, {0xA3, 0xE4}
	},
	{
		2, {0x82, 0x85},
		3, {0xEF, 0xBD, 0x85},
		2, {0x45, 0xFF},
		2, {0xA3, 0xE5}
	},
	{
		2, {0x82, 0x86},
		3, {0xEF, 0xBD, 0x86},
		2, {0x46, 0xFF},
		2, {0xA3, 0xE6}
	},
	{
		2, {0x82, 0x87},
		3, {0xEF, 0xBD, 0x87},
		2, {0x47, 0xFF},
		2, {0xA3, 0xE7}
	},
	{
		2, {0x82, 0x88},
		3, {0xEF, 0xBD, 0x88},
		2, {0x48, 0xFF},
		2, {0xA3, 0xE8}
	},
	{
		2, {0x82, 0x89},
		3, {0xEF, 0xBD, 0x89},
		2, {0x49, 0xFF},
		2, {0xA3, 0xE9}
	},
	{
		2, {0x82, 0x8A},
		3, {0xEF, 0xBD, 0x8A},
		2, {0x4A, 0xFF},
		2, {0xA3, 0xEA}
	},
	{
		2, {0x82, 0x8B},
		3, {0xEF, 0xBD, 0x8B},
		2, {0x4B, 0xFF},
		2, {0xA3, 0xEB}
	},
	{
		2, {0x82, 0x8C},
		3, {0xEF, 0xBD, 0x8C},
		2, {0x4C, 0xFF},
		2, {0xA3, 0xEC}
	},
	{
		2, {0x82, 0x8D},
		3, {0xEF, 0xBD, 0x8D},
		2, {0x4D, 0xFF},
		2, {0xA3, 0xED}
	},
	{
		2, {0x82, 0x8E},
		3, {0xEF, 0xBD, 0x8E},
		2, {0x4E, 0xFF},
		2, {0xA3, 0xEE}
	},
	{
		2, {0x82, 0x8F},
		3, {0xEF, 0xBD, 0x8F},
		2, {0x4F, 0xFF},
		2, {0xA3, 0xEF}
	},
	{
		2, {0x82, 0x90},
		3, {0xEF, 0xBD, 0x90},
		2, {0x50, 0xFF},
		2, {0xA3, 0xF0}
	},
	{
		2, {0x82, 0x91},
		3, {0xEF, 0xBD, 0x91},
		2, {0x51, 0xFF},
		2, {0xA3, 0xF1}
	},
	{
		2, {0x82, 0x92},
		3, {0xEF, 0xBD, 0x92},
		2, {0x52, 0xFF},
		2, {0xA3, 0xF2}
	},
	{
		2, {0x82, 0x93},
		3, {0xEF, 0xBD, 0x93},
		2, {0x53, 0xFF},
		2, {0xA3, 0xF3}
	},
	{
		2, {0x82, 0x94},
		3, {0xEF, 0xBD, 0x94},
		2, {0x54, 0xFF},
		2, {0xA3, 0xF4}
	},
	{
		2, {0x82, 0x95},
		3, {0xEF, 0xBD, 0x95},
		2, {0x55, 0xFF},
		2, {0xA3, 0xF5}
	},
	{
		2, {0x82, 0x96},
		3, {0xEF, 0xBD, 0x96},
		2, {0x56, 0xFF},
		2, {0xA3, 0xF6}
	},
	{
		2, {0x82, 0x97},
		3, {0xEF, 0xBD, 0x97},
		2, {0x57, 0xFF},
		2, {0xA3, 0xF7}
	},
	{
		2, {0x82, 0x98},
		3, {0xEF, 0xBD, 0x98},
		2, {0x58, 0xFF},
		2, {0xA3, 0xF8}
	},
	{
		2, {0x82, 0x99},
		3, {0xEF, 0xBD, 0x99},
		2, {0x59, 0xFF},
		2, {0xA3, 0xF9}
	},
	{
		2, {0x82, 0x9A},
		3, {0xEF, 0xBD, 0x9A},
		2, {0x5A, 0xFF},
		2, {0xA3, 0xFA}
	},
	{
		2, {0x82, 0x9B},
		3, {0xE3, 0x82, 0xA0},
		2, {0xA0, 0x30},
		0, {0x00}
	},
	{
		2, {0x82, 0x9C},
		3, {0xE2, 0x80, 0x93},
		2, {0x13, 0x20},
		0, {0x00}
	},
	{
		2, {0x82, 0x9D},
		3, {0xE2, 0xA7, 0xBA},
		2, {0xFA, 0x29},
		0, {0x00}
	},
	{
		2, {0x82, 0x9E},
		3, {0xE2, 0xA7, 0xBB},
		2, {0xFB, 0x29},
		0, {0x00}
	},
	{
		2, {0x82, 0x9F},
		3, {0xE3, 0x81, 0x81},
		2, {0x41, 0x30},
		2, {0xA4, 0xA1}
	},
	{
		2, {0x82, 0xA0},
		3, {0xE3, 0x81, 0x82},
		2, {0x42, 0x30},
		2, {0xA4, 0xA2}
	},
	{
		2, {0x82, 0xA1},
		3, {0xE3, 0x81, 0x83},
		2, {0x43, 0x30},
		2, {0xA4, 0xA3}
	},
	{
		2, {0x82, 0xA2},
		3, {0xE3, 0x81, 0x84},
		2, {0x44, 0x30},
		2, {0xA4, 0xA4}
	},
	{
		2, {0x82, 0xA3},
		3, {0xE3, 0x81, 0x85},
		2, {0x45, 0x30},
		2, {0xA4, 0xA5}
	},
	{
		2, {0x82, 0xA4},
		3, {0xE3, 0x81, 0x86},
		2, {0x46, 0x30},
		2, {0xA4, 0xA6}
	},
	{
		2, {0x82, 0xA5},
		3, {0xE3, 0x81, 0x87},
		2, {0x47, 0x30},
		2, {0xA4, 0xA7}
	},
	{
		2, {0x82, 0xA6},
		3, {0xE3, 0x81, 0x88},
		2, {0x48, 0x30},
		2, {0xA4, 0xA8}
	},
	{
		2, {0x82, 0xA7},
		3, {0xE3, 0x81, 0x89},
		2, {0x49, 0x30},
		2, {0xA4, 0xA9}
	},
	{
		2, {0x82, 0xA8},
		3, {0xE3, 0x81, 0x8A},
		2, {0x4A, 0x30},
		2, {0xA4, 0xAA}
	},
	{
		2, {0x82, 0xA9},
		3, {0xE3, 0x81, 0x8B},
		2, {0x4B, 0x30},
		2, {0xA4, 0xAB}
	},
	{
		2, {0x82, 0xAA},
		3, {0xE3, 0x81, 0x8C},
		2, {0x4C, 0x30},
		2, {0xA4, 0xAC}
	},
	{
		2, {0x82, 0xAB},
		3, {0xE3, 0x81, 0x8D},
		2, {0x4D, 0x30},
		2, {0xA4, 0xAD}
	},
	{
		2, {0x82, 0xAC},
		3, {0xE3, 0x81, 0x8E},
		2, {0x4E, 0x30},
		2, {0xA4, 0xAE}
	},
	{
		2, {0x82, 0xAD},
		3, {0xE3, 0x81, 0x8F},
		2, {0x4F, 0x30},
		2, {0xA4, 0xAF}
	},
	{
		2, {0x82, 0xAE},
		3, {0xE3, 0x81, 0x90},
		2, {0x50, 0x30},
		2, {0xA4, 0xB0}
	},
	{
		2, {0x82, 0xAF},
		3, {0xE3, 0x81, 0x91},
		2, {0x51, 0x30},
		2, {0xA4, 0xB1}
	},
	{
		2, {0x82, 0xB0},
		3, {0xE3, 0x81, 0x92},
		2, {0x52, 0x30},
		2, {0xA4, 0xB2}
	},
	{
		2, {0x82, 0xB1},
		3, {0xE3, 0x81, 0x93},
		2, {0x53, 0x30},
		2, {0xA4, 0xB3}
	},
	{
		2, {0x82, 0xB2},
		3, {0xE3, 0x81, 0x94},
		2, {0x54, 0x30},
		2, {0xA4, 0xB4}
	},
	{
		2, {0x82, 0xB3},
		3, {0xE3, 0x81, 0x95},
		2, {0x55, 0x30},
		2, {0xA4, 0xB5}
	},
	{
		2, {0x82, 0xB4},
		3, {0xE3, 0x81, 0x96},
		2, {0x56, 0x30},
		2, {0xA4, 0xB6}
	},
	{
		2, {0x82, 0xB5},
		3, {0xE3, 0x81, 0x97},
		2, {0x57, 0x30},
		2, {0xA4, 0xB7}
	},
	{
		2, {0x82, 0xB6},
		3, {0xE3, 0x81, 0x98},
		2, {0x58, 0x30},
		2, {0xA4, 0xB8}
	},
	{
		2, {0x82, 0xB7},
		3, {0xE3, 0x81, 0x99},
		2, {0x59, 0x30},
		2, {0xA4, 0xB9}
	},
	{
		2, {0x82, 0xB8},
		3, {0xE3, 0x81, 0x9A},
		2, {0x5A, 0x30},
		2, {0xA4, 0xBA}
	},
	{
		2, {0x82, 0xB9},
		3, {0xE3, 0x81, 0x9B},
		2, {0x5B, 0x30},
		2, {0xA4, 0xBB}
	},
	{
		2, {0x82, 0xBA},
		3, {0xE3, 0x81, 0x9C},
		2, {0x5C, 0x30},
		2, {0xA4, 0xBC}
	},
	{
		2, {0x82, 0xBB},
		3, {0xE3, 0x81, 0x9D},
		2, {0x5D, 0x30},
		2, {0xA4, 0xBD}
	},
	{
		2, {0x82, 0xBC},
		3, {0xE3, 0x81, 0x9E},
		2, {0x5E, 0x30},
		2, {0xA4, 0xBE}
	},
	{
		2, {0x82, 0xBD},
		3, {0xE3, 0x81, 0x9F},
		2, {0x5F, 0x30},
		2, {0xA4, 0xBF}
	},
	{
		2, {0x82, 0xBE},
		3, {0xE3, 0x81, 0xA0},
		2, {0x60, 0x30},
		2, {0xA4, 0xC0}
	},
	{
		2, {0x82, 0xBF},
		3, {0xE3, 0x81, 0xA1},
		2, {0x61, 0x30},
		2, {0xA4, 0xC1}
	},
	{
		2, {0x82, 0xC0},
		3, {0xE3, 0x81, 0xA2},
		2, {0x62, 0x30},
		2, {0xA4, 0xC2}
	},
	{
		2, {0x82, 0xC1},
		3, {0xE3, 0x81, 0xA3},
		2, {0x63, 0x30},
		2, {0xA4, 0xC3}
	},
	{
		2, {0x82, 0xC2},
		3, {0xE3, 0x81, 0xA4},
		2, {0x64, 0x30},
		2, {0xA4, 0xC4}
	},
	{
		2, {0x82, 0xC3},
		3, {0xE3, 0x81, 0xA5},
		2, {0x65, 0x30},
		2, {0xA4, 0xC5}
	},
	{
		2, {0x82, 0xC4},
		3, {0xE3, 0x81, 0xA6},
		2, {0x66, 0x30},
		2, {0xA4, 0xC6}
	},
	{
		2, {0x82, 0xC5},
		3, {0xE3, 0x81, 0xA7},
		2, {0x67, 0x30},
		2, {0xA4, 0xC7}
	},
	{
		2, {0x82, 0xC6},
		3, {0xE3, 0x81, 0xA8},
		2, {0x68, 0x30},
		2, {0xA4, 0xC8}
	},
	{
		2, {0x82, 0xC7},
		3, {0xE3, 0x81, 0xA9},
		2, {0x69, 0x30},
		2, {0xA4, 0xC9}
	},
	{
		2, {0x82, 0xC8},
		3, {0xE3, 0x81, 0xAA},
		2, {0x6A, 0x30},
		2, {0xA4, 0xCA}
	},
	{
		2, {0x82, 0xC9},
		3, {0xE3, 0x81, 0xAB},
		2, {0x6B, 0x30},
		2, {0xA4, 0xCB}
	},
	{
		2, {0x82, 0xCA},
		3, {0xE3, 0x81, 0xAC},
		2, {0x6C, 0x30},
		2, {0xA4, 0xCC}
	},
	{
		2, {0x82, 0xCB},
		3, {0xE3, 0x81, 0xAD},
		2, {0x6D, 0x30},
		2, {0xA4, 0xCD}
	},
	{
		2, {0x82, 0xCC},
		3, {0xE3, 0x81, 0xAE},
		2, {0x6E, 0x30},
		2, {0xA4, 0xCE}
	},
	{
		2, {0x82, 0xCD},
		3, {0xE3, 0x81, 0xAF},
		2, {0x6F, 0x30},
		2, {0xA4, 0xCF}
	},
	{
		2, {0x82, 0xCE},
		3, {0xE3, 0x81, 0xB0},
		2, {0x70, 0x30},
		2, {0xA4, 0xD0}
	},
	{
		2, {0x82, 0xCF},
		3, {0xE3, 0x81, 0xB1},
		2, {0x71, 0x30},
		2, {0xA4, 0xD1}
	},
	{
		2, {0x82, 0xD0},
		3, {0xE3, 0x81, 0xB2},
		2, {0x72, 0x30},
		2, {0xA4, 0xD2}
	},
	{
		2, {0x82, 0xD1},
		3, {0xE3, 0x81, 0xB3},
		2, {0x73, 0x30},
		2, {0xA4, 0xD3}
	},
	{
		2, {0x82, 0xD2},
		3, {0xE3, 0x81, 0xB4},
		2, {0x74, 0x30},
		2, {0xA4, 0xD4}
	},
	{
		2, {0x82, 0xD3},
		3, {0xE3, 0x81, 0xB5},
		2, {0x75, 0x30},
		2, {0xA4, 0xD5}
	},
	{
		2, {0x82, 0xD4},
		3, {0xE3, 0x81, 0xB6},
		2, {0x76, 0x30},
		2, {0xA4, 0xD6}
	},
	{
		2, {0x82, 0xD5},
		3, {0xE3, 0x81, 0xB7},
		2, {0x77, 0x30},
		2, {0xA4, 0xD7}
	},
	{
		2, {0x82, 0xD6},
		3, {0xE3, 0x81, 0xB8},
		2, {0x78, 0x30},
		2, {0xA4, 0xD8}
	},
	{
		2, {0x82, 0xD7},
		3, {0xE3, 0x81, 0xB9},
		2, {0x79, 0x30},
		2, {0xA4, 0xD9}
	},
	{
		2, {0x82, 0xD8},
		3, {0xE3, 0x81, 0xBA},
		2, {0x7A, 0x30},
		2, {0xA4, 0xDA}
	},
	{
		2, {0x82, 0xD9},
		3, {0xE3, 0x81, 0xBB},
		2, {0x7B, 0x30},
		2, {0xA4, 0xDB}
	},
	{
		2, {0x82, 0xDA},
		3, {0xE3, 0x81, 0xBC},
		2, {0x7C, 0x30},
		2, {0xA4, 0xDC}
	},
	{
		2, {0x82, 0xDB},
		3, {0xE3, 0x81, 0xBD},
		2, {0x7D, 0x30},
		2, {0xA4, 0xDD}
	},
	{
		2, {0x82, 0xDC},
		3, {0xE3, 0x81, 0xBE},
		2, {0x7E, 0x30},
		2, {0xA4, 0xDE}
	},
	{
		2, {0x82, 0xDD},
		3, {0xE3, 0x81, 0xBF},
		2, {0x7F, 0x30},
		2, {0xA4, 0xDF}
	},
	{
		2, {0x82, 0xDE},
		3, {0xE3, 0x82, 0x80},
		2, {0x80, 0x30},
		2, {0xA4, 0xE0}
	},
	{
		2, {0x82, 0xDF},
		3, {0xE3, 0x82, 0x81},
		2, {0x81, 0x30},
		2, {0xA4, 0xE1}
	},
	{
		2, {0x82, 0xE0},
		3, {0xE3, 0x82, 0x82},
		2, {0x82, 0x30},
		2, {0xA4, 0xE2}
	},
	{
		2, {0x82, 0xE1},
		3, {0xE3, 0x82, 0x83},
		2, {0x83, 0x30},
		2, {0xA4, 0xE3}
	},
	{
		2, {0x82, 0xE2},
		3, {0xE3, 0x82, 0x84},
		2, {0x84, 0x30},
		2, {0xA4, 0xE4}
	},
	{
		2, {0x82, 0xE3},
		3, {0xE3, 0x82, 0x85},
		2, {0x85, 0x30},
		2, {0xA4, 0xE5}
	},
	{
		2, {0x82, 0xE4},
		3, {0xE3, 0x82, 0x86},
		2, {0x86, 0x30},
		2, {0xA4, 0xE6}
	},
	{
		2, {0x82, 0xE5},
		3, {0xE3, 0x82, 0x87},
		2, {0x87, 0x30},
		2, {0xA4, 0xE7}
	},
	{
		2, {0x82, 0xE6},
		3, {0xE3, 0x82, 0x88},
		2, {0x88, 0x30},
		2, {0xA4, 0xE8}
	},
	{
		2, {0x82, 0xE7},
		3, {0xE3, 0x82, 0x89},
		2, {0x89, 0x30},
		2, {0xA4, 0xE9}
	},
	{
		2, {0x82, 0xE8},
		3, {0xE3, 0x82, 0x8A},
		2, {0x8A, 0x30},
		2, {0xA4, 0xEA}
	},
	{
		2, {0x82, 0xE9},
		3, {0xE3, 0x82, 0x8B},
		2, {0x8B, 0x30},
		2, {0xA4, 0xEB}
	},
	{
		2, {0x82, 0xEA},
		3, {0xE3, 0x82, 0x8C},
		2, {0x8C, 0x30},
		2, {0xA4, 0xEC}
	},
	{
		2, {0x82, 0xEB},
		3, {0xE3, 0x82, 0x8D},
		2, {0x8D, 0x30},
		2, {0xA4, 0xED}
	},
	{
		2, {0x82, 0xEC},
		3, {0xE3, 0x82, 0x8E},
		2, {0x8E, 0x30},
		2, {0xA4, 0xEE}
	},
	{
		2, {0x82, 0xED},
		3, {0xE3, 0x82, 0x8F},
		2, {0x8F, 0x30},
		2, {0xA4, 0xEF}
	},
	{
		2, {0x82, 0xEE},
		3, {0xE3, 0x82, 0x90},
		2, {0x90, 0x30},
		2, {0xA4, 0xF0}
	},
	{
		2, {0x82, 0xEF},
		3, {0xE3, 0x82, 0x91},
		2, {0x91, 0x30},
		2, {0xA4, 0xF1}
	},
	{
		2, {0x82, 0xF0},
		3, {0xE3, 0x82, 0x92},
		2, {0x92, 0x30},
		2, {0xA4, 0xF2}
	},
	{
		2, {0x82, 0xF1},
		3, {0xE3, 0x82, 0x93},
		2, {0x93, 0x30},
		2, {0xA4, 0xF3}
	},
	{
		2, {0x82, 0xF2},
		3, {0xE3, 0x82, 0x94},
		2, {0x94, 0x30},
		0, {0x00}
	},
	{
		2, {0x82, 0xF3},
		3, {0xE3, 0x82, 0x95},
		2, {0x95, 0x30},
		0, {0x00}
	},
	{
		2, {0x82, 0xF4},
		3, {0xE3, 0x82, 0x96},
		2, {0x96, 0x30},
		0, {0x00}
	},
	{
		2, {0x82, 0xF5},
		6, {0xE3, 0x81, 0x8B, 0xE3, 0x82, 0x9A},
		4, {0x4B, 0x30, 0x9A, 0x30},
		2, {0xA4, 0xAB}
	},
	{
		2, {0x82, 0xF6},
		6, {0xE3, 0x81, 0x8D, 0xE3, 0x82, 0x9A},
		4, {0x4D, 0x30, 0x9A, 0x30},
		2, {0xA4, 0xAD}
	},
	{
		2, {0x82, 0xF7},
		6, {0xE3, 0x81, 0x8F, 0xE3, 0x82, 0x9A},
		4, {0x4F, 0x30, 0x9A, 0x30},
		2, {0xA4, 0xAF}
	},
	{
		2, {0x82, 0xF8},
		6, {0xE3, 0x81, 0x91, 0xE3, 0x82, 0x9A},
		4, {0x51, 0x30, 0x9A, 0x30},
		2, {0xA4, 0xB1}
	},
	{
		2, {0x82, 0xF9},
		6, {0xE3, 0x81, 0x93, 0xE3, 0x82, 0x9A},
		4, {0x53, 0x30, 0x9A, 0x30},
		2, {0xA4, 0xB3}
	},
	{
		2, {0x82, 0xFA},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x82, 0xFB},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x82, 0xFC},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x83, 0x40},
		3, {0xE3, 0x82, 0xA1},
		2, {0xA1, 0x30},
		2, {0xA5, 0xA1}
	},
	{
		2, {0x83, 0x41},
		3, {0xE3, 0x82, 0xA2},
		2, {0xA2, 0x30},
		2, {0xA5, 0xA2}
	},
	{
		2, {0x83, 0x42},
		3, {0xE3, 0x82, 0xA3},
		2, {0xA3, 0x30},
		2, {0xA5, 0xA3}
	},
	{
		2, {0x83, 0x43},
		3, {0xE3, 0x82, 0xA4},
		2, {0xA4, 0x30},
		2, {0xA5, 0xA4}
	},
	{
		2, {0x83, 0x44},
		3, {0xE3, 0x82, 0xA5},
		2, {0xA5, 0x30},
		2, {0xA5, 0xA5}
	},
	{
		2, {0x83, 0x45},
		3, {0xE3, 0x82, 0xA6},
		2, {0xA6, 0x30},
		2, {0xA5, 0xA6}
	},
	{
		2, {0x83, 0x46},
		3, {0xE3, 0x82, 0xA7},
		2, {0xA7, 0x30},
		2, {0xA5, 0xA7}
	},
	{
		2, {0x83, 0x47},
		3, {0xE3, 0x82, 0xA8},
		2, {0xA8, 0x30},
		2, {0xA5, 0xA8}
	},
	{
		2, {0x83, 0x48},
		3, {0xE3, 0x82, 0xA9},
		2, {0xA9, 0x30},
		2, {0xA5, 0xA9}
	},
	{
		2, {0x83, 0x49},
		3, {0xE3, 0x82, 0xAA},
		2, {0xAA, 0x30},
		2, {0xA5, 0xAA}
	},
	{
		2, {0x83, 0x4A},
		3, {0xE3, 0x82, 0xAB},
		2, {0xAB, 0x30},
		2, {0xA5, 0xAB}
	},
	{
		2, {0x83, 0x4B},
		3, {0xE3, 0x82, 0xAC},
		2, {0xAC, 0x30},
		2, {0xA5, 0xAC}
	},
	{
		2, {0x83, 0x4C},
		3, {0xE3, 0x82, 0xAD},
		2, {0xAD, 0x30},
		2, {0xA5, 0xAD}
	},
	{
		2, {0x83, 0x4D},
		3, {0xE3, 0x82, 0xAE},
		2, {0xAE, 0x30},
		2, {0xA5, 0xAE}
	},
	{
		2, {0x83, 0x4E},
		3, {0xE3, 0x82, 0xAF},
		2, {0xAF, 0x30},
		2, {0xA5, 0xAF}
	},
	{
		2, {0x83, 0x4F},
		3, {0xE3, 0x82, 0xB0},
		2, {0xB0, 0x30},
		2, {0xA5, 0xB0}
	},
	{
		2, {0x83, 0x50},
		3, {0xE3, 0x82, 0xB1},
		2, {0xB1, 0x30},
		2, {0xA5, 0xB1}
	},
	{
		2, {0x83, 0x51},
		3, {0xE3, 0x82, 0xB2},
		2, {0xB2, 0x30},
		2, {0xA5, 0xB2}
	},
	{
		2, {0x83, 0x52},
		3, {0xE3, 0x82, 0xB3},
		2, {0xB3, 0x30},
		2, {0xA5, 0xB3}
	},
	{
		2, {0x83, 0x53},
		3, {0xE3, 0x82, 0xB4},
		2, {0xB4, 0x30},
		2, {0xA5, 0xB4}
	},
	{
		2, {0x83, 0x54},
		3, {0xE3, 0x82, 0xB5},
		2, {0xB5, 0x30},
		2, {0xA5, 0xB5}
	},
	{
		2, {0x83, 0x55},
		3, {0xE3, 0x82, 0xB6},
		2, {0xB6, 0x30},
		2, {0xA5, 0xB6}
	},
	{
		2, {0x83, 0x56},
		3, {0xE3, 0x82, 0xB7},
		2, {0xB7, 0x30},
		2, {0xA5, 0xB7}
	},
	{
		2, {0x83, 0x57},
		3, {0xE3, 0x82, 0xB8},
		2, {0xB8, 0x30},
		2, {0xA5, 0xB8}
	},
	{
		2, {0x83, 0x58},
		3, {0xE3, 0x82, 0xB9},
		2, {0xB9, 0x30},
		2, {0xA5, 0xB9}
	},
	{
		2, {0x83, 0x59},
		3, {0xE3, 0x82, 0xBA},
		2, {0xBA, 0x30},
		2, {0xA5, 0xBA}
	},
	{
		2, {0x83, 0x5A},
		3, {0xE3, 0x82, 0xBB},
		2, {0xBB, 0x30},
		2, {0xA5, 0xBB}
	},
	{
		2, {0x83, 0x5B},
		3, {0xE3, 0x82, 0xBC},
		2, {0xBC, 0x30},
		2, {0xA5, 0xBC}
	},
	{
		2, {0x83, 0x5C},
		3, {0xE3, 0x82, 0xBD},
		2, {0xBD, 0x30},
		2, {0xA5, 0xBD}
	},
	{
		2, {0x83, 0x5D},
		3, {0xE3, 0x82, 0xBE},
		2, {0xBE, 0x30},
		2, {0xA5, 0xBE}
	},
	{
		2, {0x83, 0x5E},
		3, {0xE3, 0x82, 0xBF},
		2, {0xBF, 0x30},
		2, {0xA5, 0xBF}
	},
	{
		2, {0x83, 0x5F},
		3, {0xE3, 0x83, 0x80},
		2, {0xC0, 0x30},
		2, {0xA5, 0xC0}
	},
	{
		2, {0x83, 0x60},
		3, {0xE3, 0x83, 0x81},
		2, {0xC1, 0x30},
		2, {0xA5, 0xC1}
	},
	{
		2, {0x83, 0x61},
		3, {0xE3, 0x83, 0x82},
		2, {0xC2, 0x30},
		2, {0xA5, 0xC2}
	},
	{
		2, {0x83, 0x62},
		3, {0xE3, 0x83, 0x83},
		2, {0xC3, 0x30},
		2, {0xA5, 0xC3}
	},
	{
		2, {0x83, 0x63},
		3, {0xE3, 0x83, 0x84},
		2, {0xC4, 0x30},
		2, {0xA5, 0xC4}
	},
	{
		2, {0x83, 0x64},
		3, {0xE3, 0x83, 0x85},
		2, {0xC5, 0x30},
		2, {0xA5, 0xC5}
	},
	{
		2, {0x83, 0x65},
		3, {0xE3, 0x83, 0x86},
		2, {0xC6, 0x30},
		2, {0xA5, 0xC6}
	},
	{
		2, {0x83, 0x66},
		3, {0xE3, 0x83, 0x87},
		2, {0xC7, 0x30},
		2, {0xA5, 0xC7}
	},
	{
		2, {0x83, 0x67},
		3, {0xE3, 0x83, 0x88},
		2, {0xC8, 0x30},
		2, {0xA5, 0xC8}
	},
	{
		2, {0x83, 0x68},
		3, {0xE3, 0x83, 0x89},
		2, {0xC9, 0x30},
		2, {0xA5, 0xC9}
	},
	{
		2, {0x83, 0x69},
		3, {0xE3, 0x83, 0x8A},
		2, {0xCA, 0x30},
		2, {0xA5, 0xCA}
	},
	{
		2, {0x83, 0x6A},
		3, {0xE3, 0x83, 0x8B},
		2, {0xCB, 0x30},
		2, {0xA5, 0xCB}
	},
	{
		2, {0x83, 0x6B},
		3, {0xE3, 0x83, 0x8C},
		2, {0xCC, 0x30},
		2, {0xA5, 0xCC}
	},
	{
		2, {0x83, 0x6C},
		3, {0xE3, 0x83, 0x8D},
		2, {0xCD, 0x30},
		2, {0xA5, 0xCD}
	},
	{
		2, {0x83, 0x6D},
		3, {0xE3, 0x83, 0x8E},
		2, {0xCE, 0x30},
		2, {0xA5, 0xCE}
	},
	{
		2, {0x83, 0x6E},
		3, {0xE3, 0x83, 0x8F},
		2, {0xCF, 0x30},
		2, {0xA5, 0xCF}
	},
	{
		2, {0x83, 0x6F},
		3, {0xE3, 0x83, 0x90},
		2, {0xD0, 0x30},
		2, {0xA5, 0xD0}
	},
	{
		2, {0x83, 0x70},
		3, {0xE3, 0x83, 0x91},
		2, {0xD1, 0x30},
		2, {0xA5, 0xD1}
	},
	{
		2, {0x83, 0x71},
		3, {0xE3, 0x83, 0x92},
		2, {0xD2, 0x30},
		2, {0xA5, 0xD2}
	},
	{
		2, {0x83, 0x72},
		3, {0xE3, 0x83, 0x93},
		2, {0xD3, 0x30},
		2, {0xA5, 0xD3}
	},
	{
		2, {0x83, 0x73},
		3, {0xE3, 0x83, 0x94},
		2, {0xD4, 0x30},
		2, {0xA5, 0xD4}
	},
	{
		2, {0x83, 0x74},
		3, {0xE3, 0x83, 0x95},
		2, {0xD5, 0x30},
		2, {0xA5, 0xD5}
	},
	{
		2, {0x83, 0x75},
		3, {0xE3, 0x83, 0x96},
		2, {0xD6, 0x30},
		2, {0xA5, 0xD6}
	},
	{
		2, {0x83, 0x76},
		3, {0xE3, 0x83, 0x97},
		2, {0xD7, 0x30},
		2, {0xA5, 0xD7}
	},
	{
		2, {0x83, 0x77},
		3, {0xE3, 0x83, 0x98},
		2, {0xD8, 0x30},
		2, {0xA5, 0xD8}
	},
	{
		2, {0x83, 0x78},
		3, {0xE3, 0x83, 0x99},
		2, {0xD9, 0x30},
		2, {0xA5, 0xD9}
	},
	{
		2, {0x83, 0x79},
		3, {0xE3, 0x83, 0x9A},
		2, {0xDA, 0x30},
		2, {0xA5, 0xDA}
	},
	{
		2, {0x83, 0x7A},
		3, {0xE3, 0x83, 0x9B},
		2, {0xDB, 0x30},
		2, {0xA5, 0xDB}
	},
	{
		2, {0x83, 0x7B},
		3, {0xE3, 0x83, 0x9C},
		2, {0xDC, 0x30},
		2, {0xA5, 0xDC}
	},
	{
		2, {0x83, 0x7C},
		3, {0xE3, 0x83, 0x9D},
		2, {0xDD, 0x30},
		2, {0xA5, 0xDD}
	},
	{
		2, {0x83, 0x7D},
		3, {0xE3, 0x83, 0x9E},
		2, {0xDE, 0x30},
		2, {0xA5, 0xDE}
	},
	{
		2, {0x83, 0x7E},
		3, {0xE3, 0x83, 0x9F},
		2, {0xDF, 0x30},
		2, {0xA5, 0xDF}
	},
	{
		2, {0x83, 0x80},
		3, {0xE3, 0x83, 0xA0},
		2, {0xE0, 0x30},
		2, {0xA5, 0xE0}
	},
	{
		2, {0x83, 0x81},
		3, {0xE3, 0x83, 0xA1},
		2, {0xE1, 0x30},
		2, {0xA5, 0xE1}
	},
	{
		2, {0x83, 0x82},
		3, {0xE3, 0x83, 0xA2},
		2, {0xE2, 0x30},
		2, {0xA5, 0xE2}
	},
	{
		2, {0x83, 0x83},
		3, {0xE3, 0x83, 0xA3},
		2, {0xE3, 0x30},
		2, {0xA5, 0xE3}
	},
	{
		2, {0x83, 0x84},
		3, {0xE3, 0x83, 0xA4},
		2, {0xE4, 0x30},
		2, {0xA5, 0xE4}
	},
	{
		2, {0x83, 0x85},
		3, {0xE3, 0x83, 0xA5},
		2, {0xE5, 0x30},
		2, {0xA5, 0xE5}
	},
	{
		2, {0x83, 0x86},
		3, {0xE3, 0x83, 0xA6},
		2, {0xE6, 0x30},
		2, {0xA5, 0xE6}
	},
	{
		2, {0x83, 0x87},
		3, {0xE3, 0x83, 0xA7},
		2, {0xE7, 0x30},
		2, {0xA5, 0xE7}
	},
	{
		2, {0x83, 0x88},
		3, {0xE3, 0x83, 0xA8},
		2, {0xE8, 0x30},
		2, {0xA5, 0xE8}
	},
	{
		2, {0x83, 0x89},
		3, {0xE3, 0x83, 0xA9},
		2, {0xE9, 0x30},
		2, {0xA5, 0xE9}
	},
	{
		2, {0x83, 0x8A},
		3, {0xE3, 0x83, 0xAA},
		2, {0xEA, 0x30},
		2, {0xA5, 0xEA}
	},
	{
		2, {0x83, 0x8B},
		3, {0xE3, 0x83, 0xAB},
		2, {0xEB, 0x30},
		2, {0xA5, 0xEB}
	},
	{
		2, {0x83, 0x8C},
		3, {0xE3, 0x83, 0xAC},
		2, {0xEC, 0x30},
		2, {0xA5, 0xEC}
	},
	{
		2, {0x83, 0x8D},
		3, {0xE3, 0x83, 0xAD},
		2, {0xED, 0x30},
		2, {0xA5, 0xED}
	},
	{
		2, {0x83, 0x8E},
		3, {0xE3, 0x83, 0xAE},
		2, {0xEE, 0x30},
		2, {0xA5, 0xEE}
	},
	{
		2, {0x83, 0x8F},
		3, {0xE3, 0x83, 0xAF},
		2, {0xEF, 0x30},
		2, {0xA5, 0xEF}
	},
	{
		2, {0x83, 0x90},
		3, {0xE3, 0x83, 0xB0},
		2, {0xF0, 0x30},
		2, {0xA5, 0xF0}
	},
	{
		2, {0x83, 0x91},
		3, {0xE3, 0x83, 0xB1},
		2, {0xF1, 0x30},
		2, {0xA5, 0xF1}
	},
	{
		2, {0x83, 0x92},
		3, {0xE3, 0x83, 0xB2},
		2, {0xF2, 0x30},
		2, {0xA5, 0xF2}
	},
	{
		2, {0x83, 0x93},
		3, {0xE3, 0x83, 0xB3},
		2, {0xF3, 0x30},
		2, {0xA5, 0xF3}
	},
	{
		2, {0x83, 0x94},
		3, {0xE3, 0x83, 0xB4},
		2, {0xF4, 0x30},
		2, {0xA5, 0xF4}
	},
	{
		2, {0x83, 0x95},
		3, {0xE3, 0x83, 0xB5},
		2, {0xF5, 0x30},
		2, {0xA5, 0xF5}
	},
	{
		2, {0x83, 0x96},
		3, {0xE3, 0x83, 0xB6},
		2, {0xF6, 0x30},
		2, {0xA5, 0xF6}
	},
	{
		2, {0x83, 0x97},
		6, {0xE3, 0x82, 0xAB, 0xE3, 0x82, 0x9A},
		4, {0xAB, 0x30, 0x9A, 0x30},
		2, {0xA5, 0xAB}
	},
	{
		2, {0x83, 0x98},
		6, {0xE3, 0x82, 0xAD, 0xE3, 0x82, 0x9A},
		4, {0xAD, 0x30, 0x9A, 0x30},
		2, {0xA5, 0xAD}
	},
	{
		2, {0x83, 0x99},
		6, {0xE3, 0x82, 0xAF, 0xE3, 0x82, 0x9A},
		4, {0xAF, 0x30, 0x9A, 0x30},
		2, {0xA5, 0xAF}
	},
	{
		2, {0x83, 0x9A},
		6, {0xE3, 0x82, 0xB1, 0xE3, 0x82, 0x9A},
		4, {0xB1, 0x30, 0x9A, 0x30},
		2, {0xA5, 0xB1}
	},
	{
		2, {0x83, 0x9B},
		6, {0xE3, 0x82, 0xB3, 0xE3, 0x82, 0x9A},
		4, {0xB3, 0x30, 0x9A, 0x30},
		2, {0xA5, 0xB3}
	},
	{
		2, {0x83, 0x9C},
		6, {0xE3, 0x82, 0xBB, 0xE3, 0x82, 0x9A},
		4, {0xBB, 0x30, 0x9A, 0x30},
		2, {0xA5, 0xBB}
	},
	{
		2, {0x83, 0x9D},
		6, {0xE3, 0x83, 0x84, 0xE3, 0x82, 0x9A},
		4, {0xC4, 0x30, 0x9A, 0x30},
		2, {0xA5, 0xC4}
	},
	{
		2, {0x83, 0x9E},
		6, {0xE3, 0x83, 0x88, 0xE3, 0x82, 0x9A},
		4, {0xC8, 0x30, 0x9A, 0x30},
		2, {0xA5, 0xC8}
	},
	{
		2, {0x83, 0x9F},
		2, {0xCE, 0x91},
		2, {0x91, 0x03},
		2, {0xA6, 0xA1}
	},
	{
		2, {0x83, 0xA0},
		2, {0xCE, 0x92},
		2, {0x92, 0x03},
		2, {0xA6, 0xA2}
	},
	{
		2, {0x83, 0xA1},
		2, {0xCE, 0x93},
		2, {0x93, 0x03},
		2, {0xA6, 0xA3}
	},
	{
		2, {0x83, 0xA2},
		2, {0xCE, 0x94},
		2, {0x94, 0x03},
		2, {0xA6, 0xA4}
	},
	{
		2, {0x83, 0xA3},
		2, {0xCE, 0x95},
		2, {0x95, 0x03},
		2, {0xA6, 0xA5}
	},
	{
		2, {0x83, 0xA4},
		2, {0xCE, 0x96},
		2, {0x96, 0x03},
		2, {0xA6, 0xA6}
	},
	{
		2, {0x83, 0xA5},
		2, {0xCE, 0x97},
		2, {0x97, 0x03},
		2, {0xA6, 0xA7}
	},
	{
		2, {0x83, 0xA6},
		2, {0xCE, 0x98},
		2, {0x98, 0x03},
		2, {0xA6, 0xA8}
	},
	{
		2, {0x83, 0xA7},
		2, {0xCE, 0x99},
		2, {0x99, 0x03},
		2, {0xA6, 0xA9}
	},
	{
		2, {0x83, 0xA8},
		2, {0xCE, 0x9A},
		2, {0x9A, 0x03},
		2, {0xA6, 0xAA}
	},
	{
		2, {0x83, 0xA9},
		2, {0xCE, 0x9B},
		2, {0x9B, 0x03},
		2, {0xA6, 0xAB}
	},
	{
		2, {0x83, 0xAA},
		2, {0xCE, 0x9C},
		2, {0x9C, 0x03},
		2, {0xA6, 0xAC}
	},
	{
		2, {0x83, 0xAB},
		2, {0xCE, 0x9D},
		2, {0x9D, 0x03},
		2, {0xA6, 0xAD}
	},
	{
		2, {0x83, 0xAC},
		2, {0xCE, 0x9E},
		2, {0x9E, 0x03},
		2, {0xA6, 0xAE}
	},
	{
		2, {0x83, 0xAD},
		2, {0xCE, 0x9F},
		2, {0x9F, 0x03},
		2, {0xA6, 0xAF}
	},
	{
		2, {0x83, 0xAE},
		2, {0xCE, 0xA0},
		2, {0xA0, 0x03},
		2, {0xA6, 0xB0}
	},
	{
		2, {0x83, 0xAF},
		2, {0xCE, 0xA1},
		2, {0xA1, 0x03},
		2, {0xA6, 0xB1}
	},
	{
		2, {0x83, 0xB0},
		2, {0xCE, 0xA3},
		2, {0xA3, 0x03},
		2, {0xA6, 0xB2}
	},
	{
		2, {0x83, 0xB1},
		2, {0xCE, 0xA4},
		2, {0xA4, 0x03},
		2, {0xA6, 0xB3}
	},
	{
		2, {0x83, 0xB2},
		2, {0xCE, 0xA5},
		2, {0xA5, 0x03},
		2, {0xA6, 0xB4}
	},
	{
		2, {0x83, 0xB3},
		2, {0xCE, 0xA6},
		2, {0xA6, 0x03},
		2, {0xA6, 0xB5}
	},
	{
		2, {0x83, 0xB4},
		2, {0xCE, 0xA7},
		2, {0xA7, 0x03},
		2, {0xA6, 0xB6}
	},
	{
		2, {0x83, 0xB5},
		2, {0xCE, 0xA8},
		2, {0xA8, 0x03},
		2, {0xA6, 0xB7}
	},
	{
		2, {0x83, 0xB6},
		2, {0xCE, 0xA9},
		2, {0xA9, 0x03},
		2, {0xA6, 0xB8}
	},
	{
		2, {0x83, 0xB7},
		3, {0xE2, 0x99, 0xA4},
		2, {0x64, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xB8},
		3, {0xE2, 0x99, 0xA0},
		2, {0x60, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xB9},
		3, {0xE2, 0x99, 0xA2},
		2, {0x62, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xBA},
		3, {0xE2, 0x99, 0xA6},
		2, {0x66, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xBB},
		3, {0xE2, 0x99, 0xA1},
		2, {0x61, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xBC},
		3, {0xE2, 0x99, 0xA5},
		2, {0x65, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xBD},
		3, {0xE2, 0x99, 0xA7},
		2, {0x67, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xBE},
		3, {0xE2, 0x99, 0xA3},
		2, {0x63, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xBF},
		2, {0xCE, 0xB1},
		2, {0xB1, 0x03},
		2, {0xA6, 0xC1}
	},
	{
		2, {0x83, 0xC0},
		2, {0xCE, 0xB2},
		2, {0xB2, 0x03},
		2, {0xA6, 0xC2}
	},
	{
		2, {0x83, 0xC1},
		2, {0xCE, 0xB3},
		2, {0xB3, 0x03},
		2, {0xA6, 0xC3}
	},
	{
		2, {0x83, 0xC2},
		2, {0xCE, 0xB4},
		2, {0xB4, 0x03},
		2, {0xA6, 0xC4}
	},
	{
		2, {0x83, 0xC3},
		2, {0xCE, 0xB5},
		2, {0xB5, 0x03},
		2, {0xA6, 0xC5}
	},
	{
		2, {0x83, 0xC4},
		2, {0xCE, 0xB6},
		2, {0xB6, 0x03},
		2, {0xA6, 0xC6}
	},
	{
		2, {0x83, 0xC5},
		2, {0xCE, 0xB7},
		2, {0xB7, 0x03},
		2, {0xA6, 0xC7}
	},
	{
		2, {0x83, 0xC6},
		2, {0xCE, 0xB8},
		2, {0xB8, 0x03},
		2, {0xA6, 0xC8}
	},
	{
		2, {0x83, 0xC7},
		2, {0xCE, 0xB9},
		2, {0xB9, 0x03},
		2, {0xA6, 0xC9}
	},
	{
		2, {0x83, 0xC8},
		2, {0xCE, 0xBA},
		2, {0xBA, 0x03},
		2, {0xA6, 0xCA}
	},
	{
		2, {0x83, 0xC9},
		2, {0xCE, 0xBB},
		2, {0xBB, 0x03},
		2, {0xA6, 0xCB}
	},
	{
		2, {0x83, 0xCA},
		2, {0xCE, 0xBC},
		2, {0xBC, 0x03},
		2, {0xA6, 0xCC}
	},
	{
		2, {0x83, 0xCB},
		2, {0xCE, 0xBD},
		2, {0xBD, 0x03},
		2, {0xA6, 0xCD}
	},
	{
		2, {0x83, 0xCC},
		2, {0xCE, 0xBE},
		2, {0xBE, 0x03},
		2, {0xA6, 0xCE}
	},
	{
		2, {0x83, 0xCD},
		2, {0xCE, 0xBF},
		2, {0xBF, 0x03},
		2, {0xA6, 0xCF}
	},
	{
		2, {0x83, 0xCE},
		2, {0xCF, 0x80},
		2, {0xC0, 0x03},
		2, {0xA6, 0xD0}
	},
	{
		2, {0x83, 0xCF},
		2, {0xCF, 0x81},
		2, {0xC1, 0x03},
		2, {0xA6, 0xD1}
	},
	{
		2, {0x83, 0xD0},
		2, {0xCF, 0x83},
		2, {0xC3, 0x03},
		2, {0xA6, 0xD2}
	},
	{
		2, {0x83, 0xD1},
		2, {0xCF, 0x84},
		2, {0xC4, 0x03},
		2, {0xA6, 0xD3}
	},
	{
		2, {0x83, 0xD2},
		2, {0xCF, 0x85},
		2, {0xC5, 0x03},
		2, {0xA6, 0xD4}
	},
	{
		2, {0x83, 0xD3},
		2, {0xCF, 0x86},
		2, {0xC6, 0x03},
		2, {0xA6, 0xD5}
	},
	{
		2, {0x83, 0xD4},
		2, {0xCF, 0x87},
		2, {0xC7, 0x03},
		2, {0xA6, 0xD6}
	},
	{
		2, {0x83, 0xD5},
		2, {0xCF, 0x88},
		2, {0xC8, 0x03},
		2, {0xA6, 0xD7}
	},
	{
		2, {0x83, 0xD6},
		2, {0xCF, 0x89},
		2, {0xC9, 0x03},
		2, {0xA6, 0xD8}
	},
	{
		2, {0x83, 0xD7},
		2, {0xCF, 0x82},
		2, {0xC2, 0x03},
		3, {0x8F, 0xA6, 0xF8}
	},
	{
		2, {0x83, 0xD8},
		3, {0xE2, 0x93, 0xB5},
		2, {0xF5, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xD9},
		3, {0xE2, 0x93, 0xB6},
		2, {0xF6, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xDA},
		3, {0xE2, 0x93, 0xB7},
		2, {0xF7, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xDB},
		3, {0xE2, 0x93, 0xB8},
		2, {0xF8, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xDC},
		3, {0xE2, 0x93, 0xB9},
		2, {0xF9, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xDD},
		3, {0xE2, 0x93, 0xBA},
		2, {0xFA, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xDE},
		3, {0xE2, 0x93, 0xBB},
		2, {0xFB, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xDF},
		3, {0xE2, 0x93, 0xBC},
		2, {0xFC, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xE0},
		3, {0xE2, 0x93, 0xBD},
		2, {0xFD, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xE1},
		3, {0xE2, 0x93, 0xBE},
		2, {0xFE, 0x24},
		0, {0x00}
	},
	{
		2, {0x83, 0xE2},
		3, {0xE2, 0x98, 0x96},
		2, {0x16, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xE3},
		3, {0xE2, 0x98, 0x97},
		2, {0x17, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xE4},
		3, {0xE3, 0x80, 0xA0},
		2, {0x20, 0x30},
		0, {0x00}
	},
	{
		2, {0x83, 0xE5},
		3, {0xE2, 0x98, 0x8E},
		2, {0x0E, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xE6},
		3, {0xE2, 0x98, 0x80},
		2, {0x00, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xE7},
		3, {0xE2, 0x98, 0x81},
		2, {0x01, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xE8},
		3, {0xE2, 0x98, 0x82},
		2, {0x02, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xE9},
		3, {0xE2, 0x98, 0x83},
		2, {0x03, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xEA},
		3, {0xE2, 0x99, 0xA8},
		2, {0x68, 0x26},
		0, {0x00}
	},
	{
		2, {0x83, 0xEB},
		3, {0xE2, 0x96, 0xB1},
		2, {0xB1, 0x25},
		0, {0x00}
	},
	{
		2, {0x83, 0xEC},
		3, {0xE3, 0x87, 0xB0},
		2, {0xF0, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xED},
		3, {0xE3, 0x87, 0xB1},
		2, {0xF1, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xEE},
		3, {0xE3, 0x87, 0xB2},
		2, {0xF2, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xEF},
		3, {0xE3, 0x87, 0xB3},
		2, {0xF3, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xF0},
		3, {0xE3, 0x87, 0xB4},
		2, {0xF4, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xF1},
		3, {0xE3, 0x87, 0xB5},
		2, {0xF5, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xF2},
		3, {0xE3, 0x87, 0xB6},
		2, {0xF6, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xF3},
		3, {0xE3, 0x87, 0xB7},
		2, {0xF7, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xF4},
		3, {0xE3, 0x87, 0xB8},
		2, {0xF8, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xF5},
		3, {0xE3, 0x87, 0xB9},
		2, {0xF9, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xF6},
		6, {0xE3, 0x87, 0xB7, 0xE3, 0x82, 0x9A},
		4, {0xF7, 0x31, 0x9A, 0x30},
		0, {0x00}
	},
	{
		2, {0x83, 0xF7},
		3, {0xE3, 0x87, 0xBA},
		2, {0xFA, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xF8},
		3, {0xE3, 0x87, 0xBB},
		2, {0xFB, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xF9},
		3, {0xE3, 0x87, 0xBC},
		2, {0xFC, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xFA},
		3, {0xE3, 0x87, 0xBD},
		2, {0xFD, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xFB},
		3, {0xE3, 0x87, 0xBE},
		2, {0xFE, 0x31},
		0, {0x00}
	},
	{
		2, {0x83, 0xFC},
		3, {0xE3, 0x87, 0xBF},
		2, {0xFF, 0x31},
		0, {0x00}
	},
	{
		2, {0x84, 0x40},
		2, {0xD0, 0x90},
		2, {0x10, 0x04},
		2, {0xA7, 0xA1}
	},
	{
		2, {0x84, 0x41},
		2, {0xD0, 0x91},
		2, {0x11, 0x04},
		2, {0xA7, 0xA2}
	},
	{
		2, {0x84, 0x42},
		2, {0xD0, 0x92},
		2, {0x12, 0x04},
		2, {0xA7, 0xA3}
	},
	{
		2, {0x84, 0x43},
		2, {0xD0, 0x93},
		2, {0x13, 0x04},
		2, {0xA7, 0xA4}
	},
	{
		2, {0x84, 0x44},
		2, {0xD0, 0x94},
		2, {0x14, 0x04},
		2, {0xA7, 0xA5}
	},
	{
		2, {0x84, 0x45},
		2, {0xD0, 0x95},
		2, {0x15, 0x04},
		2, {0xA7, 0xA6}
	},
	{
		2, {0x84, 0x46},
		2, {0xD0, 0x81},
		2, {0x01, 0x04},
		2, {0xA7, 0xA7}
	},
	{
		2, {0x84, 0x47},
		2, {0xD0, 0x96},
		2, {0x16, 0x04},
		2, {0xA7, 0xA8}
	},
	{
		2, {0x84, 0x48},
		2, {0xD0, 0x97},
		2, {0x17, 0x04},
		2, {0xA7, 0xA9}
	},
	{
		2, {0x84, 0x49},
		2, {0xD0, 0x98},
		2, {0x18, 0x04},
		2, {0xA7, 0xAA}
	},
	{
		2, {0x84, 0x4A},
		2, {0xD0, 0x99},
		2, {0x19, 0x04},
		2, {0xA7, 0xAB}
	},
	{
		2, {0x84, 0x4B},
		2, {0xD0, 0x9A},
		2, {0x1A, 0x04},
		2, {0xA7, 0xAC}
	},
	{
		2, {0x84, 0x4C},
		2, {0xD0, 0x9B},
		2, {0x1B, 0x04},
		2, {0xA7, 0xAD}
	},
	{
		2, {0x84, 0x4D},
		2, {0xD0, 0x9C},
		2, {0x1C, 0x04},
		2, {0xA7, 0xAE}
	},
	{
		2, {0x84, 0x4E},
		2, {0xD0, 0x9D},
		2, {0x1D, 0x04},
		2, {0xA7, 0xAF}
	},
	{
		2, {0x84, 0x4F},
		2, {0xD0, 0x9E},
		2, {0x1E, 0x04},
		2, {0xA7, 0xB0}
	},
	{
		2, {0x84, 0x50},
		2, {0xD0, 0x9F},
		2, {0x1F, 0x04},
		2, {0xA7, 0xB1}
	},
	{
		2, {0x84, 0x51},
		2, {0xD0, 0xA0},
		2, {0x20, 0x04},
		2, {0xA7, 0xB2}
	},
	{
		2, {0x84, 0x52},
		2, {0xD0, 0xA1},
		2, {0x21, 0x04},
		2, {0xA7, 0xB3}
	},
	{
		2, {0x84, 0x53},
		2, {0xD0, 0xA2},
		2, {0x22, 0x04},
		2, {0xA7, 0xB4}
	},
	{
		2, {0x84, 0x54},
		2, {0xD0, 0xA3},
		2, {0x23, 0x04},
		2, {0xA7, 0xB5}
	},
	{
		2, {0x84, 0x55},
		2, {0xD0, 0xA4},
		2, {0x24, 0x04},
		2, {0xA7, 0xB6}
	},
	{
		2, {0x84, 0x56},
		2, {0xD0, 0xA5},
		2, {0x25, 0x04},
		2, {0xA7, 0xB7}
	},
	{
		2, {0x84, 0x57},
		2, {0xD0, 0xA6},
		2, {0x26, 0x04},
		2, {0xA7, 0xB8}
	},
	{
		2, {0x84, 0x58},
		2, {0xD0, 0xA7},
		2, {0x27, 0x04},
		2, {0xA7, 0xB9}
	},
	{
		2, {0x84, 0x59},
		2, {0xD0, 0xA8},
		2, {0x28, 0x04},
		2, {0xA7, 0xBA}
	},
	{
		2, {0x84, 0x5A},
		2, {0xD0, 0xA9},
		2, {0x29, 0x04},
		2, {0xA7, 0xBB}
	},
	{
		2, {0x84, 0x5B},
		2, {0xD0, 0xAA},
		2, {0x2A, 0x04},
		2, {0xA7, 0xBC}
	},
	{
		2, {0x84, 0x5C},
		2, {0xD0, 0xAB},
		2, {0x2B, 0x04},
		2, {0xA7, 0xBD}
	},
	{
		2, {0x84, 0x5D},
		2, {0xD0, 0xAC},
		2, {0x2C, 0x04},
		2, {0xA7, 0xBE}
	},
	{
		2, {0x84, 0x5E},
		2, {0xD0, 0xAD},
		2, {0x2D, 0x04},
		2, {0xA7, 0xBF}
	},
	{
		2, {0x84, 0x5F},
		2, {0xD0, 0xAE},
		2, {0x2E, 0x04},
		2, {0xA7, 0xC0}
	},
	{
		2, {0x84, 0x60},
		2, {0xD0, 0xAF},
		2, {0x2F, 0x04},
		2, {0xA7, 0xC1}
	},
	{
		2, {0x84, 0x61},
		3, {0xE2, 0x8E, 0xBE},
		2, {0xBE, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x62},
		3, {0xE2, 0x8E, 0xBF},
		2, {0xBF, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x63},
		3, {0xE2, 0x8F, 0x80},
		2, {0xC0, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x64},
		3, {0xE2, 0x8F, 0x81},
		2, {0xC1, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x65},
		3, {0xE2, 0x8F, 0x82},
		2, {0xC2, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x66},
		3, {0xE2, 0x8F, 0x83},
		2, {0xC3, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x67},
		3, {0xE2, 0x8F, 0x84},
		2, {0xC4, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x68},
		3, {0xE2, 0x8F, 0x85},
		2, {0xC5, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x69},
		3, {0xE2, 0x8F, 0x86},
		2, {0xC6, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x6A},
		3, {0xE2, 0x8F, 0x87},
		2, {0xC7, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x6B},
		3, {0xE2, 0x8F, 0x88},
		2, {0xC8, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x6C},
		3, {0xE2, 0x8F, 0x89},
		2, {0xC9, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x6D},
		3, {0xE2, 0x8F, 0x8A},
		2, {0xCA, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x6E},
		3, {0xE2, 0x8F, 0x8B},
		2, {0xCB, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x6F},
		3, {0xE2, 0x8F, 0x8C},
		2, {0xCC, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x70},
		2, {0xD0, 0xB0},
		2, {0x30, 0x04},
		2, {0xA7, 0xD1}
	},
	{
		2, {0x84, 0x71},
		2, {0xD0, 0xB1},
		2, {0x31, 0x04},
		2, {0xA7, 0xD2}
	},
	{
		2, {0x84, 0x72},
		2, {0xD0, 0xB2},
		2, {0x32, 0x04},
		2, {0xA7, 0xD3}
	},
	{
		2, {0x84, 0x73},
		2, {0xD0, 0xB3},
		2, {0x33, 0x04},
		2, {0xA7, 0xD4}
	},
	{
		2, {0x84, 0x74},
		2, {0xD0, 0xB4},
		2, {0x34, 0x04},
		2, {0xA7, 0xD5}
	},
	{
		2, {0x84, 0x75},
		2, {0xD0, 0xB5},
		2, {0x35, 0x04},
		2, {0xA7, 0xD6}
	},
	{
		2, {0x84, 0x76},
		2, {0xD1, 0x91},
		2, {0x51, 0x04},
		2, {0xA7, 0xD7}
	},
	{
		2, {0x84, 0x77},
		2, {0xD0, 0xB6},
		2, {0x36, 0x04},
		2, {0xA7, 0xD8}
	},
	{
		2, {0x84, 0x78},
		2, {0xD0, 0xB7},
		2, {0x37, 0x04},
		2, {0xA7, 0xD9}
	},
	{
		2, {0x84, 0x79},
		2, {0xD0, 0xB8},
		2, {0x38, 0x04},
		2, {0xA7, 0xDA}
	},
	{
		2, {0x84, 0x7A},
		2, {0xD0, 0xB9},
		2, {0x39, 0x04},
		2, {0xA7, 0xDB}
	},
	{
		2, {0x84, 0x7B},
		2, {0xD0, 0xBA},
		2, {0x3A, 0x04},
		2, {0xA7, 0xDC}
	},
	{
		2, {0x84, 0x7C},
		2, {0xD0, 0xBB},
		2, {0x3B, 0x04},
		2, {0xA7, 0xDD}
	},
	{
		2, {0x84, 0x7D},
		2, {0xD0, 0xBC},
		2, {0x3C, 0x04},
		2, {0xA7, 0xDE}
	},
	{
		2, {0x84, 0x7E},
		2, {0xD0, 0xBD},
		2, {0x3D, 0x04},
		2, {0xA7, 0xDF}
	},
	{
		2, {0x84, 0x80},
		2, {0xD0, 0xBE},
		2, {0x3E, 0x04},
		2, {0xA7, 0xE0}
	},
	{
		2, {0x84, 0x81},
		2, {0xD0, 0xBF},
		2, {0x3F, 0x04},
		2, {0xA7, 0xE1}
	},
	{
		2, {0x84, 0x82},
		2, {0xD1, 0x80},
		2, {0x40, 0x04},
		2, {0xA7, 0xE2}
	},
	{
		2, {0x84, 0x83},
		2, {0xD1, 0x81},
		2, {0x41, 0x04},
		2, {0xA7, 0xE3}
	},
	{
		2, {0x84, 0x84},
		2, {0xD1, 0x82},
		2, {0x42, 0x04},
		2, {0xA7, 0xE4}
	},
	{
		2, {0x84, 0x85},
		2, {0xD1, 0x83},
		2, {0x43, 0x04},
		2, {0xA7, 0xE5}
	},
	{
		2, {0x84, 0x86},
		2, {0xD1, 0x84},
		2, {0x44, 0x04},
		2, {0xA7, 0xE6}
	},
	{
		2, {0x84, 0x87},
		2, {0xD1, 0x85},
		2, {0x45, 0x04},
		2, {0xA7, 0xE7}
	},
	{
		2, {0x84, 0x88},
		2, {0xD1, 0x86},
		2, {0x46, 0x04},
		2, {0xA7, 0xE8}
	},
	{
		2, {0x84, 0x89},
		2, {0xD1, 0x87},
		2, {0x47, 0x04},
		2, {0xA7, 0xE9}
	},
	{
		2, {0x84, 0x8A},
		2, {0xD1, 0x88},
		2, {0x48, 0x04},
		2, {0xA7, 0xEA}
	},
	{
		2, {0x84, 0x8B},
		2, {0xD1, 0x89},
		2, {0x49, 0x04},
		2, {0xA7, 0xEB}
	},
	{
		2, {0x84, 0x8C},
		2, {0xD1, 0x8A},
		2, {0x4A, 0x04},
		2, {0xA7, 0xEC}
	},
	{
		2, {0x84, 0x8D},
		2, {0xD1, 0x8B},
		2, {0x4B, 0x04},
		2, {0xA7, 0xED}
	},
	{
		2, {0x84, 0x8E},
		2, {0xD1, 0x8C},
		2, {0x4C, 0x04},
		2, {0xA7, 0xEE}
	},
	{
		2, {0x84, 0x8F},
		2, {0xD1, 0x8D},
		2, {0x4D, 0x04},
		2, {0xA7, 0xEF}
	},
	{
		2, {0x84, 0x90},
		2, {0xD1, 0x8E},
		2, {0x4E, 0x04},
		2, {0xA7, 0xF0}
	},
	{
		2, {0x84, 0x91},
		2, {0xD1, 0x8F},
		2, {0x4F, 0x04},
		2, {0xA7, 0xF1}
	},
	{
		2, {0x84, 0x92},
		3, {0xE3, 0x83, 0xB7},
		2, {0xF7, 0x30},
		0, {0x00}
	},
	{
		2, {0x84, 0x93},
		3, {0xE3, 0x83, 0xB8},
		2, {0xF8, 0x30},
		0, {0x00}
	},
	{
		2, {0x84, 0x94},
		3, {0xE3, 0x83, 0xB9},
		2, {0xF9, 0x30},
		0, {0x00}
	},
	{
		2, {0x84, 0x95},
		3, {0xE3, 0x83, 0xBA},
		2, {0xFA, 0x30},
		0, {0x00}
	},
	{
		2, {0x84, 0x96},
		3, {0xE2, 0x8B, 0x9A},
		2, {0xDA, 0x22},
		0, {0x00}
	},
	{
		2, {0x84, 0x97},
		3, {0xE2, 0x8B, 0x9B},
		2, {0xDB, 0x22},
		0, {0x00}
	},
	{
		2, {0x84, 0x98},
		3, {0xE2, 0x85, 0x93},
		2, {0x53, 0x21},
		0, {0x00}
	},
	{
		2, {0x84, 0x99},
		3, {0xE2, 0x85, 0x94},
		2, {0x54, 0x21},
		0, {0x00}
	},
	{
		2, {0x84, 0x9A},
		3, {0xE2, 0x85, 0x95},
		2, {0x55, 0x21},
		0, {0x00}
	},
	{
		2, {0x84, 0x9B},
		3, {0xE2, 0x9C, 0x93},
		2, {0x13, 0x27},
		0, {0x00}
	},
	{
		2, {0x84, 0x9C},
		3, {0xE2, 0x8C, 0x98},
		2, {0x18, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x9D},
		3, {0xE2, 0x90, 0xA3},
		2, {0x23, 0x24},
		0, {0x00}
	},
	{
		2, {0x84, 0x9E},
		3, {0xE2, 0x8F, 0x8E},
		2, {0xCE, 0x23},
		0, {0x00}
	},
	{
		2, {0x84, 0x9F},
		3, {0xE2, 0x94, 0x80},
		2, {0x00, 0x25},
		2, {0xA8, 0xA1}
	},
	{
		2, {0x84, 0xA0},
		3, {0xE2, 0x94, 0x82},
		2, {0x02, 0x25},
		2, {0xA8, 0xA2}
	},
	{
		2, {0x84, 0xA1},
		3, {0xE2, 0x94, 0x8C},
		2, {0x0C, 0x25},
		2, {0xA8, 0xA3}
	},
	{
		2, {0x84, 0xA2},
		3, {0xE2, 0x94, 0x90},
		2, {0x10, 0x25},
		2, {0xA8, 0xA4}
	},
	{
		2, {0x84, 0xA3},
		3, {0xE2, 0x94, 0x98},
		2, {0x18, 0x25},
		2, {0xA8, 0xA5}
	},
	{
		2, {0x84, 0xA4},
		3, {0xE2, 0x94, 0x94},
		2, {0x14, 0x25},
		2, {0xA8, 0xA6}
	},
	{
		2, {0x84, 0xA5},
		3, {0xE2, 0x94, 0x9C},
		2, {0x1C, 0x25},
		2, {0xA8, 0xA7}
	},
	{
		2, {0x84, 0xA6},
		3, {0xE2, 0x94, 0xAC},
		2, {0x2C, 0x25},
		2, {0xA8, 0xA8}
	},
	{
		2, {0x84, 0xA7},
		3, {0xE2, 0x94, 0xA4},
		2, {0x24, 0x25},
		2, {0xA8, 0xA9}
	},
	{
		2, {0x84, 0xA8},
		3, {0xE2, 0x94, 0xB4},
		2, {0x34, 0x25},
		2, {0xA8, 0xAA}
	},
	{
		2, {0x84, 0xA9},
		3, {0xE2, 0x94, 0xBC},
		2, {0x3C, 0x25},
		2, {0xA8, 0xAB}
	},
	{
		2, {0x84, 0xAA},
		3, {0xE2, 0x94, 0x81},
		2, {0x01, 0x25},
		2, {0xA8, 0xAC}
	},
	{
		2, {0x84, 0xAB},
		3, {0xE2, 0x94, 0x83},
		2, {0x03, 0x25},
		2, {0xA8, 0xAD}
	},
	{
		2, {0x84, 0xAC},
		3, {0xE2, 0x94, 0x8F},
		2, {0x0F, 0x25},
		2, {0xA8, 0xAE}
	},
	{
		2, {0x84, 0xAD},
		3, {0xE2, 0x94, 0x93},
		2, {0x13, 0x25},
		2, {0xA8, 0xAF}
	},
	{
		2, {0x84, 0xAE},
		3, {0xE2, 0x94, 0x9B},
		2, {0x1B, 0x25},
		2, {0xA8, 0xB0}
	},
	{
		2, {0x84, 0xAF},
		3, {0xE2, 0x94, 0x97},
		2, {0x17, 0x25},
		2, {0xA8, 0xB1}
	},
	{
		2, {0x84, 0xB0},
		3, {0xE2, 0x94, 0xA3},
		2, {0x23, 0x25},
		2, {0xA8, 0xB2}
	},
	{
		2, {0x84, 0xB1},
		3, {0xE2, 0x94, 0xB3},
		2, {0x33, 0x25},
		2, {0xA8, 0xB3}
	},
	{
		2, {0x84, 0xB2},
		3, {0xE2, 0x94, 0xAB},
		2, {0x2B, 0x25},
		2, {0xA8, 0xB4}
	},
	{
		2, {0x84, 0xB3},
		3, {0xE2, 0x94, 0xBB},
		2, {0x3B, 0x25},
		2, {0xA8, 0xB5}
	},
	{
		2, {0x84, 0xB4},
		3, {0xE2, 0x95, 0x8B},
		2, {0x4B, 0x25},
		2, {0xA8, 0xB6}
	},
	{
		2, {0x84, 0xB5},
		3, {0xE2, 0x94, 0xA0},
		2, {0x20, 0x25},
		2, {0xA8, 0xB7}
	},
	{
		2, {0x84, 0xB6},
		3, {0xE2, 0x94, 0xAF},
		2, {0x2F, 0x25},
		2, {0xA8, 0xB8}
	},
	{
		2, {0x84, 0xB7},
		3, {0xE2, 0x94, 0xA8},
		2, {0x28, 0x25},
		2, {0xA8, 0xB9}
	},
	{
		2, {0x84, 0xB8},
		3, {0xE2, 0x94, 0xB7},
		2, {0x37, 0x25},
		2, {0xA8, 0xBA}
	},
	{
		2, {0x84, 0xB9},
		3, {0xE2, 0x94, 0xBF},
		2, {0x3F, 0x25},
		2, {0xA8, 0xBB}
	},
	{
		2, {0x84, 0xBA},
		3, {0xE2, 0x94, 0x9D},
		2, {0x1D, 0x25},
		2, {0xA8, 0xBC}
	},
	{
		2, {0x84, 0xBB},
		3, {0xE2, 0x94, 0xB0},
		2, {0x30, 0x25},
		2, {0xA8, 0xBD}
	},
	{
		2, {0x84, 0xBC},
		3, {0xE2, 0x94, 0xA5},
		2, {0x25, 0x25},
		2, {0xA8, 0xBE}
	},
	{
		2, {0x84, 0xBD},
		3, {0xE2, 0x94, 0xB8},
		2, {0x38, 0x25},
		2, {0xA8, 0xBF}
	},
	{
		2, {0x84, 0xBE},
		3, {0xE2, 0x95, 0x82},
		2, {0x42, 0x25},
		2, {0xA8, 0xC0}
	},
	{
		2, {0x84, 0xBF},
		3, {0xE3, 0x89, 0x91},
		2, {0x51, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC0},
		3, {0xE3, 0x89, 0x92},
		2, {0x52, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC1},
		3, {0xE3, 0x89, 0x93},
		2, {0x53, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC2},
		3, {0xE3, 0x89, 0x94},
		2, {0x54, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC3},
		3, {0xE3, 0x89, 0x95},
		2, {0x55, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC4},
		3, {0xE3, 0x89, 0x96},
		2, {0x56, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC5},
		3, {0xE3, 0x89, 0x97},
		2, {0x57, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC6},
		3, {0xE3, 0x89, 0x98},
		2, {0x58, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC7},
		3, {0xE3, 0x89, 0x99},
		2, {0x59, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC8},
		3, {0xE3, 0x89, 0x9A},
		2, {0x5A, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xC9},
		3, {0xE3, 0x89, 0x9B},
		2, {0x5B, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xCA},
		3, {0xE3, 0x89, 0x9C},
		2, {0x5C, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xCB},
		3, {0xE3, 0x89, 0x9D},
		2, {0x5D, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xCC},
		3, {0xE3, 0x89, 0x9E},
		2, {0x5E, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xCD},
		3, {0xE3, 0x89, 0x9F},
		2, {0x5F, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xCE},
		3, {0xE3, 0x8A, 0xB1},
		2, {0xB1, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xCF},
		3, {0xE3, 0x8A, 0xB2},
		2, {0xB2, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD0},
		3, {0xE3, 0x8A, 0xB3},
		2, {0xB3, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD1},
		3, {0xE3, 0x8A, 0xB4},
		2, {0xB4, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD2},
		3, {0xE3, 0x8A, 0xB5},
		2, {0xB5, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD3},
		3, {0xE3, 0x8A, 0xB6},
		2, {0xB6, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD4},
		3, {0xE3, 0x8A, 0xB7},
		2, {0xB7, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD5},
		3, {0xE3, 0x8A, 0xB8},
		2, {0xB8, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD6},
		3, {0xE3, 0x8A, 0xB9},
		2, {0xB9, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD7},
		3, {0xE3, 0x8A, 0xBA},
		2, {0xBA, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD8},
		3, {0xE3, 0x8A, 0xBB},
		2, {0xBB, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xD9},
		3, {0xE3, 0x8A, 0xBC},
		2, {0xBC, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xDA},
		3, {0xE3, 0x8A, 0xBD},
		2, {0xBD, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xDB},
		3, {0xE3, 0x8A, 0xBE},
		2, {0xBE, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xDC},
		3, {0xE3, 0x8A, 0xBF},
		2, {0xBF, 0x32},
		0, {0x00}
	},
	{
		2, {0x84, 0xDD},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x84, 0xDE},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x84, 0xDF},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x84, 0xE0},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x84, 0xE1},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x84, 0xE2},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x84, 0xE3},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x84, 0xE4},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x84, 0xE5},
		3, {0xE2, 0x97, 0x90},
		2, {0xD0, 0x25},
		0, {0x00}
	},
	{
		2, {0x84, 0xE6},
		3, {0xE2, 0x97, 0x91},
		2, {0xD1, 0x25},
		0, {0x00}
	},
	{
		2, {0x84, 0xE7},
		3, {0xE2, 0x97, 0x92},
		2, {0xD2, 0x25},
		0, {0x00}
	},
	{
		2, {0x84, 0xE8},
		3, {0xE2, 0x97, 0x93},
		2, {0xD3, 0x25},
		0, {0x00}
	},
	{
		2, {0x84, 0xE9},
		3, {0xE2, 0x80, 0xBC},
		2, {0x3C, 0x20},
		0, {0x00}
	},
	{
		2, {0x84, 0xEA},
		3, {0xE2, 0x81, 0x87},
		2, {0x47, 0x20},
		0, {0x00}
	},
	{
		2, {0x84, 0xEB},
		3, {0xE2, 0x81, 0x88},
		2, {0x48, 0x20},
		0, {0x00}
	},
	{
		2, {0x84, 0xEC},
		3, {0xE2, 0x81, 0x89},
		2, {0x49, 0x20},
		0, {0x00}
	},
	{
		2, {0x84, 0xED},
		2, {0xC7, 0x8D},
		2, {0xCD, 0x01},
		3, {0x8F, 0xAA, 0xA6}
	},
	{
		2, {0x84, 0xEE},
		2, {0xC7, 0x8E},
		2, {0xCE, 0x01},
		3, {0x8F, 0xAB, 0xA6}
	},
	{
		2, {0x84, 0xEF},
		2, {0xC7, 0x90},
		2, {0xD0, 0x01},
		3, {0x8F, 0xAB, 0xC3}
	},
	{
		2, {0x84, 0xF0},
		3, {0xE1, 0xB8, 0xBE},
		2, {0x3E, 0x1E},
		0, {0x00}
	},
	{
		2, {0x84, 0xF1},
		3, {0xE1, 0xB8, 0xBF},
		2, {0x3F, 0x1E},
		0, {0x00}
	},
	{
		2, {0x84, 0xF2},
		2, {0xC7, 0xB8},
		2, {0xF8, 0x01},
		0, {0x00}
	},
	{
		2, {0x84, 0xF3},
		2, {0xC7, 0xB9},
		2, {0xF9, 0x01},
		0, {0x00}
	},
	{
		2, {0x84, 0xF4},
		2, {0xC7, 0x91},
		2, {0xD1, 0x01},
		3, {0x8F, 0xAA, 0xD5}
	},
	{
		2, {0x84, 0xF5},
		2, {0xC7, 0x92},
		2, {0xD2, 0x01},
		3, {0x8F, 0xAB, 0xD5}
	},
	{
		2, {0x84, 0xF6},
		2, {0xC7, 0x94},
		2, {0xD4, 0x01},
		3, {0x8F, 0xAB, 0xE7}
	},
	{
		2, {0x84, 0xF7},
		2, {0xC7, 0x96},
		2, {0xD6, 0x01},
		3, {0x8F, 0xAB, 0xF0}
	},
	{
		2, {0x84, 0xF8},
		2, {0xC7, 0x98},
		2, {0xD8, 0x01},
		3, {0x8F, 0xAB, 0xED}
	},
	{
		2, {0x84, 0xF9},
		2, {0xC7, 0x9A},
		2, {0xDA, 0x01},
		3, {0x8F, 0xAB, 0xEF}
	},
	{
		2, {0x84, 0xFA},
		2, {0xC7, 0x9C},
		2, {0xDC, 0x01},
		3, {0x8F, 0xAB, 0xEE}
	},
	{
		2, {0x84, 0xFB},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x84, 0xFC},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x40},
		3, {0xE2, 0x82, 0xAC},
		2, {0xAC, 0x20},
		0, {0x00}
	},
	{
		2, {0x85, 0x41},
		2, {0xC2, 0xA0},
		2, {0xA0, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x42},
		2, {0xC2, 0xA1},
		2, {0xA1, 0x00},
		3, {0x8F, 0xA2, 0xC2}
	},
	{
		2, {0x85, 0x43},
		2, {0xC2, 0xA4},
		2, {0xA4, 0x00},
		3, {0x8F, 0xA2, 0xF0}
	},
	{
		2, {0x85, 0x44},
		2, {0xC2, 0xA6},
		2, {0xA6, 0x00},
		3, {0x8F, 0xA2, 0xC3}
	},
	{
		2, {0x85, 0x45},
		2, {0xC2, 0xA9},
		2, {0xA9, 0x00},
		3, {0x8F, 0xA2, 0xED}
	},
	{
		2, {0x85, 0x46},
		2, {0xC2, 0xAA},
		2, {0xAA, 0x00},
		3, {0x8F, 0xA2, 0xEC}
	},
	{
		2, {0x85, 0x47},
		2, {0xC2, 0xAB},
		2, {0xAB, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x48},
		2, {0xC2, 0xAD},
		2, {0xAD, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x49},
		2, {0xC2, 0xAE},
		2, {0xAE, 0x00},
		3, {0x8F, 0xA2, 0xEE}
	},
	{
		2, {0x85, 0x4A},
		2, {0xC2, 0xAF},
		2, {0xAF, 0x00},
		3, {0x8F, 0xA2, 0xB4}
	},
	{
		2, {0x85, 0x4B},
		2, {0xC2, 0xB2},
		2, {0xB2, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x4C},
		2, {0xC2, 0xB3},
		2, {0xB3, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x4D},
		2, {0xC2, 0xB7},
		2, {0xB7, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x4E},
		2, {0xC2, 0xB8},
		2, {0xB8, 0x00},
		3, {0x8F, 0xA2, 0xB1}
	},
	{
		2, {0x85, 0x4F},
		2, {0xC2, 0xB9},
		2, {0xB9, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x50},
		2, {0xC2, 0xBA},
		2, {0xBA, 0x00},
		3, {0x8F, 0xA2, 0xEB}
	},
	{
		2, {0x85, 0x51},
		2, {0xC2, 0xBB},
		2, {0xBB, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x52},
		2, {0xC2, 0xBC},
		2, {0xBC, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x53},
		2, {0xC2, 0xBD},
		2, {0xBD, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x54},
		2, {0xC2, 0xBE},
		2, {0xBE, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x55},
		2, {0xC2, 0xBF},
		2, {0xBF, 0x00},
		3, {0x8F, 0xA2, 0xC4}
	},
	{
		2, {0x85, 0x56},
		2, {0xC3, 0x80},
		2, {0xC0, 0x00},
		3, {0x8F, 0xAA, 0xA2}
	},
	{
		2, {0x85, 0x57},
		2, {0xC3, 0x81},
		2, {0xC1, 0x00},
		3, {0x8F, 0xAA, 0xA1}
	},
	{
		2, {0x85, 0x58},
		2, {0xC3, 0x82},
		2, {0xC2, 0x00},
		3, {0x8F, 0xAA, 0xA4}
	},
	{
		2, {0x85, 0x59},
		2, {0xC3, 0x83},
		2, {0xC3, 0x00},
		3, {0x8F, 0xAA, 0xAA}
	},
	{
		2, {0x85, 0x5A},
		2, {0xC3, 0x84},
		2, {0xC4, 0x00},
		3, {0x8F, 0xAA, 0xA3}
	},
	{
		2, {0x85, 0x5B},
		2, {0xC3, 0x85},
		2, {0xC5, 0x00},
		3, {0x8F, 0xAA, 0xA9}
	},
	{
		2, {0x85, 0x5C},
		2, {0xC3, 0x86},
		2, {0xC6, 0x00},
		3, {0x8F, 0xA9, 0xA1}
	},
	{
		2, {0x85, 0x5D},
		2, {0xC3, 0x87},
		2, {0xC7, 0x00},
		3, {0x8F, 0xAA, 0xAE}
	},
	{
		2, {0x85, 0x5E},
		2, {0xC3, 0x88},
		2, {0xC8, 0x00},
		3, {0x8F, 0xAA, 0xB2}
	},
	{
		2, {0x85, 0x5F},
		2, {0xC3, 0x89},
		2, {0xC9, 0x00},
		3, {0x8F, 0xAA, 0xB1}
	},
	{
		2, {0x85, 0x60},
		2, {0xC3, 0x8A},
		2, {0xCA, 0x00},
		3, {0x8F, 0xAA, 0xB4}
	},
	{
		2, {0x85, 0x61},
		2, {0xC3, 0x8B},
		2, {0xCB, 0x00},
		3, {0x8F, 0xAA, 0xB3}
	},
	{
		2, {0x85, 0x62},
		2, {0xC3, 0x8C},
		2, {0xCC, 0x00},
		3, {0x8F, 0xAA, 0xC0}
	},
	{
		2, {0x85, 0x63},
		2, {0xC3, 0x8D},
		2, {0xCD, 0x00},
		3, {0x8F, 0xAA, 0xBF}
	},
	{
		2, {0x85, 0x64},
		2, {0xC3, 0x8E},
		2, {0xCE, 0x00},
		3, {0x8F, 0xAA, 0xC2}
	},
	{
		2, {0x85, 0x65},
		2, {0xC3, 0x8F},
		2, {0xCF, 0x00},
		3, {0x8F, 0xAA, 0xC1}
	},
	{
		2, {0x85, 0x66},
		2, {0xC3, 0x90},
		2, {0xD0, 0x00},
		0, {0x00}
	},
	{
		2, {0x85, 0x67},
		2, {0xC3, 0x91},
		2, {0xD1, 0x00},
		3, {0x8F, 0xAA, 0xD0}
	},
	{
		2, {0x85, 0x68},
		2, {0xC3, 0x92},
		2, {0xD2, 0x00},
		3, {0x8F, 0xAA, 0xD2}
	},
	{
		2, {0x85, 0x69},
		2, {0xC3, 0x93},
		2, {0xD3, 0x00},
		3, {0x8F, 0xAA, 0xD1}
	},
	{
		2, {0x85, 0x6A},
		2, {0xC3, 0x94},
		2, {0xD4, 0x00},
		3, {0x8F, 0xAA, 0xD4}
	},
	{
		2, {0x85, 0x6B},
		2, {0xC3, 0x95},
		2, {0xD5, 0x00},
		3, {0x8F, 0xAA, 0xD8}
	},
	{
		2, {0x85, 0x6C},
		2, {0xC3, 0x96},
		2, {0xD6, 0x00},
		3, {0x8F, 0xAA, 0xD3}
	},
	{
		2, {0x85, 0x6D},
		2, {0xC3, 0x98},
		2, {0xD8, 0x00},
		3, {0x8F, 0xA9, 0xAC}
	},
	{
		2, {0x85, 0x6E},
		2, {0xC3, 0x99},
		2, {0xD9, 0x00},
		3, {0x8F, 0xAA, 0xE3}
	},
	{
		2, {0x85, 0x6F},
		2, {0xC3, 0x9A},
		2, {0xDA, 0x00},
		3, {0x8F, 0xAA, 0xE2}
	},
	{
		2, {0x85, 0x70},
		2, {0xC3, 0x9B},
		2, {0xDB, 0x00},
		3, {0x8F, 0xAA, 0xE5}
	},
	{
		2, {0x85, 0x71},
		2, {0xC3, 0x9C},
		2, {0xDC, 0x00},
		3, {0x8F, 0xAA, 0xE4}
	},
	{
		2, {0x85, 0x72},
		2, {0xC3, 0x9D},
		2, {0xDD, 0x00},
		3, {0x8F, 0xAA, 0xF2}
	},
	{
		2, {0x85, 0x73},
		2, {0xC3, 0x9E},
		2, {0xDE, 0x00},
		3, {0x8F, 0xA9, 0xB0}
	},
	{
		2, {0x85, 0x74},
		2, {0xC3, 0x9F},
		2, {0xDF, 0x00},
		3, {0x8F, 0xA9, 0xCE}
	},
	{
		2, {0x85, 0x75},
		2, {0xC3, 0xA0},
		2, {0xE0, 0x00},
		3, {0x8F, 0xAB, 0xA2}
	},
	{
		2, {0x85, 0x76},
		2, {0xC3, 0xA1},
		2, {0xE1, 0x00},
		3, {0x8F, 0xAB, 0xA1}
	},
	{
		2, {0x85, 0x77},
		2, {0xC3, 0xA2},
		2, {0xE2, 0x00},
		3, {0x8F, 0xAB, 0xA4}
	},
	{
		2, {0x85, 0x78},
		2, {0xC3, 0xA3},
		2, {0xE3, 0x00},
		3, {0x8F, 0xAB, 0xAA}
	},
	{
		2, {0x85, 0x79},
		2, {0xC3, 0xA4},
		2, {0xE4, 0x00},
		3, {0x8F, 0xAB, 0xA3}
	},
	{
		2, {0x85, 0x7A},
		2, {0xC3, 0xA5},
		2, {0xE5, 0x00},
		3, {0x8F, 0xAB, 0xA9}
	},
	{
		2, {0x85, 0x7B},
		2, {0xC3, 0xA6},
		2, {0xE6, 0x00},
		3, {0x8F, 0xA9, 0xC1}
	},
	{
		2, {0x85, 0x7C},
		2, {0xC3, 0xA7},
		2, {0xE7, 0x00},
		3, {0x8F, 0xAB, 0xAE}
	},
	{
		2, {0x85, 0x7D},
		2, {0xC3, 0xA8},
		2, {0xE8, 0x00},
		3, {0x8F, 0xAB, 0xB2}
	},
	{
		2, {0x85, 0x7E},
		2, {0xC3, 0xA9},
		2, {0xE9, 0x00},
		3, {0x8F, 0xAB, 0xB1}
	},
	{
		2, {0x85, 0x80},
		2, {0xC3, 0xAA},
		2, {0xEA, 0x00},
		3, {0x8F, 0xAB, 0xB4}
	},
	{
		2, {0x85, 0x81},
		2, {0xC3, 0xAB},
		2, {0xEB, 0x00},
		3, {0x8F, 0xAB, 0xB3}
	},
	{
		2, {0x85, 0x82},
		2, {0xC3, 0xAC},
		2, {0xEC, 0x00},
		3, {0x8F, 0xAB, 0xC0}
	},
	{
		2, {0x85, 0x83},
		2, {0xC3, 0xAD},
		2, {0xED, 0x00},
		3, {0x8F, 0xAB, 0xBF}
	},
	{
		2, {0x85, 0x84},
		2, {0xC3, 0xAE},
		2, {0xEE, 0x00},
		3, {0x8F, 0xAB, 0xC2}
	},
	{
		2, {0x85, 0x85},
		2, {0xC3, 0xAF},
		2, {0xEF, 0x00},
		3, {0x8F, 0xAB, 0xC1}
	},
	{
		2, {0x85, 0x86},
		2, {0xC3, 0xB0},
		2, {0xF0, 0x00},
		3, {0x8F, 0xA9, 0xC3}
	},
	{
		2, {0x85, 0x87},
		2, {0xC3, 0xB1},
		2, {0xF1, 0x00},
		3, {0x8F, 0xAB, 0xD0}
	},
	{
		2, {0x85, 0x88},
		2, {0xC3, 0xB2},
		2, {0xF2, 0x00},
		3, {0x8F, 0xAB, 0xD2}
	},
	{
		2, {0x85, 0x89},
		2, {0xC3, 0xB3},
		2, {0xF3, 0x00},
		3, {0x8F, 0xAB, 0xD1}
	},
	{
		2, {0x85, 0x8A},
		2, {0xC3, 0xB4},
		2, {0xF4, 0x00},
		3, {0x8F, 0xAB, 0xD4}
	},
	{
		2, {0x85, 0x8B},
		2, {0xC3, 0xB5},
		2, {0xF5, 0x00},
		3, {0x8F, 0xAB, 0xD8}
	},
	{
		2, {0x85, 0x8C},
		2, {0xC3, 0xB6},
		2, {0xF6, 0x00},
		3, {0x8F, 0xAB, 0xD3}
	},
	{
		2, {0x85, 0x8D},
		2, {0xC3, 0xB8},
		2, {0xF8, 0x00},
		3, {0x8F, 0xA9, 0xCC}
	},
	{
		2, {0x85, 0x8E},
		2, {0xC3, 0xB9},
		2, {0xF9, 0x00},
		3, {0x8F, 0xAB, 0xE3}
	},
	{
		2, {0x85, 0x8F},
		2, {0xC3, 0xBA},
		2, {0xFA, 0x00},
		3, {0x8F, 0xAB, 0xE2}
	},
	{
		2, {0x85, 0x90},
		2, {0xC3, 0xBB},
		2, {0xFB, 0x00},
		3, {0x8F, 0xAB, 0xE5}
	},
	{
		2, {0x85, 0x91},
		2, {0xC3, 0xBC},
		2, {0xFC, 0x00},
		3, {0x8F, 0xAB, 0xE4}
	},
	{
		2, {0x85, 0x92},
		2, {0xC3, 0xBD},
		2, {0xFD, 0x00},
		3, {0x8F, 0xAB, 0xF2}
	},
	{
		2, {0x85, 0x93},
		2, {0xC3, 0xBE},
		2, {0xFE, 0x00},
		3, {0x8F, 0xA9, 0xD0}
	},
	{
		2, {0x85, 0x94},
		2, {0xC3, 0xBF},
		2, {0xFF, 0x00},
		3, {0x8F, 0xAB, 0xF3}
	},
	{
		2, {0x85, 0x95},
		2, {0xC4, 0x80},
		2, {0x00, 0x01},
		3, {0x8F, 0xAA, 0xA7}
	},
	{
		2, {0x85, 0x96},
		2, {0xC4, 0xAA},
		2, {0x2A, 0x01},
		3, {0x8F, 0xAA, 0xC5}
	},
	{
		2, {0x85, 0x97},
		2, {0xC5, 0xAA},
		2, {0x6A, 0x01},
		3, {0x8F, 0xAA, 0xE9}
	},
	{
		2, {0x85, 0x98},
		2, {0xC4, 0x92},
		2, {0x12, 0x01},
		3, {0x8F, 0xAA, 0xB7}
	},
	{
		2, {0x85, 0x99},
		2, {0xC5, 0x8C},
		2, {0x4C, 0x01},
		3, {0x8F, 0xAA, 0xD7}
	},
	{
		2, {0x85, 0x9A},
		2, {0xC4, 0x81},
		2, {0x01, 0x01},
		3, {0x8F, 0xAB, 0xA7}
	},
	{
		2, {0x85, 0x9B},
		2, {0xC4, 0xAB},
		2, {0x2B, 0x01},
		3, {0x8F, 0xAB, 0xC5}
	},
	{
		2, {0x85, 0x9C},
		2, {0xC5, 0xAB},
		2, {0x6B, 0x01},
		3, {0x8F, 0xAB, 0xE9}
	},
	{
		2, {0x85, 0x9D},
		2, {0xC4, 0x93},
		2, {0x13, 0x01},
		3, {0x8F, 0xAB, 0xB7}
	},
	{
		2, {0x85, 0x9E},
		2, {0xC5, 0x8D},
		2, {0x4D, 0x01},
		3, {0x8F, 0xAB, 0xD7}
	},
	{
		2, {0x85, 0x9F},
		2, {0xC4, 0x84},
		2, {0x04, 0x01},
		3, {0x8F, 0xAA, 0xA8}
	},
	{
		2, {0x85, 0xA0},
		2, {0xCB, 0x98},
		2, {0xD8, 0x02},
		3, {0x8F, 0xA2, 0xAF}
	},
	{
		2, {0x85, 0xA1},
		2, {0xC5, 0x81},
		2, {0x41, 0x01},
		3, {0x8F, 0xA9, 0xA8}
	},
	{
		2, {0x85, 0xA2},
		2, {0xC4, 0xBD},
		2, {0x3D, 0x01},
		3, {0x8F, 0xAA, 0xCB}
	},
	{
		2, {0x85, 0xA3},
		2, {0xC5, 0x9A},
		2, {0x5A, 0x01},
		3, {0x8F, 0xAA, 0xDC}
	},
	{
		2, {0x85, 0xA4},
		2, {0xC5, 0xA0},
		2, {0x60, 0x01},
		3, {0x8F, 0xAA, 0xDE}
	},
	{
		2, {0x85, 0xA5},
		2, {0xC5, 0x9E},
		2, {0x5E, 0x01},
		3, {0x8F, 0xAA, 0xDF}
	},
	{
		2, {0x85, 0xA6},
		2, {0xC5, 0xA4},
		2, {0x64, 0x01},
		3, {0x8F, 0xAA, 0xE0}
	},
	{
		2, {0x85, 0xA7},
		2, {0xC5, 0xB9},
		2, {0x79, 0x01},
		3, {0x8F, 0xAA, 0xF5}
	},
	{
		2, {0x85, 0xA8},
		2, {0xC5, 0xBD},
		2, {0x7D, 0x01},
		3, {0x8F, 0xAA, 0xF6}
	},
	{
		2, {0x85, 0xA9},
		2, {0xC5, 0xBB},
		2, {0x7B, 0x01},
		3, {0x8F, 0xAA, 0xF7}
	},
	{
		2, {0x85, 0xAA},
		2, {0xC4, 0x85},
		2, {0x05, 0x01},
		3, {0x8F, 0xAB, 0xA8}
	},
	{
		2, {0x85, 0xAB},
		2, {0xCB, 0x9B},
		2, {0xDB, 0x02},
		3, {0x8F, 0xA2, 0xB5}
	},
	{
		2, {0x85, 0xAC},
		2, {0xC5, 0x82},
		2, {0x42, 0x01},
		3, {0x8F, 0xA9, 0xC8}
	},
	{
		2, {0x85, 0xAD},
		2, {0xC4, 0xBE},
		2, {0x3E, 0x01},
		3, {0x8F, 0xAB, 0xCB}
	},
	{
		2, {0x85, 0xAE},
		2, {0xC5, 0x9B},
		2, {0x5B, 0x01},
		3, {0x8F, 0xAB, 0xDC}
	},
	{
		2, {0x85, 0xAF},
		2, {0xCB, 0x87},
		2, {0xC7, 0x02},
		3, {0x8F, 0xA2, 0xB0}
	},
	{
		2, {0x85, 0xB0},
		2, {0xC5, 0xA1},
		2, {0x61, 0x01},
		3, {0x8F, 0xAB, 0xDE}
	},
	{
		2, {0x85, 0xB1},
		2, {0xC5, 0x9F},
		2, {0x5F, 0x01},
		3, {0x8F, 0xAB, 0xDF}
	},
	{
		2, {0x85, 0xB2},
		2, {0xC5, 0xA5},
		2, {0x65, 0x01},
		3, {0x8F, 0xAB, 0xE0}
	},
	{
		2, {0x85, 0xB3},
		2, {0xC5, 0xBA},
		2, {0x7A, 0x01},
		3, {0x8F, 0xAB, 0xF5}
	},
	{
		2, {0x85, 0xB4},
		2, {0xCB, 0x9D},
		2, {0xDD, 0x02},
		3, {0x8F, 0xA2, 0xB3}
	},
	{
		2, {0x85, 0xB5},
		2, {0xC5, 0xBE},
		2, {0x7E, 0x01},
		3, {0x8F, 0xAB, 0xF6}
	},
	{
		2, {0x85, 0xB6},
		2, {0xC5, 0xBC},
		2, {0x7C, 0x01},
		3, {0x8F, 0xAB, 0xF7}
	},
	{
		2, {0x85, 0xB7},
		2, {0xC5, 0x94},
		2, {0x54, 0x01},
		3, {0x8F, 0xAA, 0xD9}
	},
	{
		2, {0x85, 0xB8},
		2, {0xC4, 0x82},
		2, {0x02, 0x01},
		3, {0x8F, 0xAA, 0xA5}
	},
	{
		2, {0x85, 0xB9},
		2, {0xC4, 0xB9},
		2, {0x39, 0x01},
		3, {0x8F, 0xAA, 0xCA}
	},
	{
		2, {0x85, 0xBA},
		2, {0xC4, 0x86},
		2, {0x06, 0x01},
		3, {0x8F, 0xAA, 0xAB}
	},
	{
		2, {0x85, 0xBB},
		2, {0xC4, 0x8C},
		2, {0x0C, 0x01},
		3, {0x8F, 0xAA, 0xAD}
	},
	{
		2, {0x85, 0xBC},
		2, {0xC4, 0x98},
		2, {0x18, 0x01},
		3, {0x8F, 0xAA, 0xB8}
	},
	{
		2, {0x85, 0xBD},
		2, {0xC4, 0x9A},
		2, {0x1A, 0x01},
		3, {0x8F, 0xAA, 0xB5}
	},
	{
		2, {0x85, 0xBE},
		2, {0xC4, 0x8E},
		2, {0x0E, 0x01},
		3, {0x8F, 0xAA, 0xB0}
	},
	{
		2, {0x85, 0xBF},
		2, {0xC5, 0x83},
		2, {0x43, 0x01},
		3, {0x8F, 0xAA, 0xCD}
	},
	{
		2, {0x85, 0xC0},
		2, {0xC5, 0x87},
		2, {0x47, 0x01},
		3, {0x8F, 0xAA, 0xCE}
	},
	{
		2, {0x85, 0xC1},
		2, {0xC5, 0x90},
		2, {0x50, 0x01},
		3, {0x8F, 0xAA, 0xD6}
	},
	{
		2, {0x85, 0xC2},
		2, {0xC5, 0x98},
		2, {0x58, 0x01},
		3, {0x8F, 0xAA, 0xDA}
	},
	{
		2, {0x85, 0xC3},
		2, {0xC5, 0xAE},
		2, {0x6E, 0x01},
		3, {0x8F, 0xAA, 0xEB}
	},
	{
		2, {0x85, 0xC4},
		2, {0xC5, 0xB0},
		2, {0x70, 0x01},
		3, {0x8F, 0xAA, 0xE8}
	},
	{
		2, {0x85, 0xC5},
		2, {0xC5, 0xA2},
		2, {0x62, 0x01},
		3, {0x8F, 0xAA, 0xE1}
	},
	{
		2, {0x85, 0xC6},
		2, {0xC5, 0x95},
		2, {0x55, 0x01},
		3, {0x8F, 0xAB, 0xD9}
	},
	{
		2, {0x85, 0xC7},
		2, {0xC4, 0x83},
		2, {0x03, 0x01},
		3, {0x8F, 0xAB, 0xA5}
	},
	{
		2, {0x85, 0xC8},
		2, {0xC4, 0xBA},
		2, {0x3A, 0x01},
		3, {0x8F, 0xAB, 0xCA}
	},
	{
		2, {0x85, 0xC9},
		2, {0xC4, 0x87},
		2, {0x07, 0x01},
		3, {0x8F, 0xAB, 0xAB}
	},
	{
		2, {0x85, 0xCA},
		2, {0xC4, 0x8D},
		2, {0x0D, 0x01},
		3, {0x8F, 0xAB, 0xAD}
	},
	{
		2, {0x85, 0xCB},
		2, {0xC4, 0x99},
		2, {0x19, 0x01},
		3, {0x8F, 0xAB, 0xB8}
	},
	{
		2, {0x85, 0xCC},
		2, {0xC4, 0x9B},
		2, {0x1B, 0x01},
		3, {0x8F, 0xAB, 0xB5}
	},
	{
		2, {0x85, 0xCD},
		2, {0xC4, 0x8F},
		2, {0x0F, 0x01},
		3, {0x8F, 0xAB, 0xB0}
	},
	{
		2, {0x85, 0xCE},
		2, {0xC4, 0x91},
		2, {0x11, 0x01},
		3, {0x8F, 0xA9, 0xC2}
	},
	{
		2, {0x85, 0xCF},
		2, {0xC5, 0x84},
		2, {0x44, 0x01},
		3, {0x8F, 0xAB, 0xCD}
	},
	{
		2, {0x85, 0xD0},
		2, {0xC5, 0x88},
		2, {0x48, 0x01},
		3, {0x8F, 0xAB, 0xCE}
	},
	{
		2, {0x85, 0xD1},
		2, {0xC5, 0x91},
		2, {0x51, 0x01},
		3, {0x8F, 0xAB, 0xD6}
	},
	{
		2, {0x85, 0xD2},
		2, {0xC5, 0x99},
		2, {0x59, 0x01},
		3, {0x8F, 0xAB, 0xDA}
	},
	{
		2, {0x85, 0xD3},
		2, {0xC5, 0xAF},
		2, {0x6F, 0x01},
		3, {0x8F, 0xAB, 0xEB}
	},
	{
		2, {0x85, 0xD4},
		2, {0xC5, 0xB1},
		2, {0x71, 0x01},
		3, {0x8F, 0xAB, 0xE8}
	},
	{
		2, {0x85, 0xD5},
		2, {0xC5, 0xA3},
		2, {0x63, 0x01},
		3, {0x8F, 0xAB, 0xE1}
	},
	{
		2, {0x85, 0xD6},
		2, {0xCB, 0x99},
		2, {0xD9, 0x02},
		3, {0x8F, 0xA2, 0xB2}
	},
	{
		2, {0x85, 0xD7},
		2, {0xC4, 0x88},
		2, {0x08, 0x01},
		3, {0x8F, 0xAA, 0xAC}
	},
	{
		2, {0x85, 0xD8},
		2, {0xC4, 0x9C},
		2, {0x1C, 0x01},
		3, {0x8F, 0xAA, 0xBA}
	},
	{
		2, {0x85, 0xD9},
		2, {0xC4, 0xA4},
		2, {0x24, 0x01},
		3, {0x8F, 0xAA, 0xBE}
	},
	{
		2, {0x85, 0xDA},
		2, {0xC4, 0xB4},
		2, {0x34, 0x01},
		3, {0x8F, 0xAA, 0xC8}
	},
	{
		2, {0x85, 0xDB},
		2, {0xC5, 0x9C},
		2, {0x5C, 0x01},
		3, {0x8F, 0xAA, 0xDD}
	},
	{
		2, {0x85, 0xDC},
		2, {0xC5, 0xAC},
		2, {0x6C, 0x01},
		3, {0x8F, 0xAA, 0xE6}
	},
	{
		2, {0x85, 0xDD},
		2, {0xC4, 0x89},
		2, {0x09, 0x01},
		3, {0x8F, 0xAB, 0xAC}
	},
	{
		2, {0x85, 0xDE},
		2, {0xC4, 0x9D},
		2, {0x1D, 0x01},
		3, {0x8F, 0xAB, 0xBA}
	},
	{
		2, {0x85, 0xDF},
		2, {0xC4, 0xA5},
		2, {0x25, 0x01},
		3, {0x8F, 0xAB, 0xBE}
	},
	{
		2, {0x85, 0xE0},
		2, {0xC4, 0xB5},
		2, {0x35, 0x01},
		3, {0x8F, 0xAB, 0xC8}
	},
	{
		2, {0x85, 0xE1},
		2, {0xC5, 0x9D},
		2, {0x5D, 0x01},
		3, {0x8F, 0xAB, 0xDD}
	},
	{
		2, {0x85, 0xE2},
		2, {0xC5, 0xAD},
		2, {0x6D, 0x01},
		3, {0x8F, 0xAB, 0xE6}
	},
	{
		2, {0x85, 0xE3},
		2, {0xC9, 0xB1},
		2, {0x71, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xE4},
		2, {0xCA, 0x8B},
		2, {0x8B, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xE5},
		2, {0xC9, 0xBE},
		2, {0x7E, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xE6},
		2, {0xCA, 0x83},
		2, {0x83, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xE7},
		2, {0xCA, 0x92},
		2, {0x92, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xE8},
		2, {0xC9, 0xAC},
		2, {0x6C, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xE9},
		2, {0xC9, 0xAE},
		2, {0x6E, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xEA},
		2, {0xC9, 0xB9},
		2, {0x79, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xEB},
		2, {0xCA, 0x88},
		2, {0x88, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xEC},
		2, {0xC9, 0x96},
		2, {0x56, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xED},
		2, {0xC9, 0xB3},
		2, {0x73, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xEE},
		2, {0xC9, 0xBD},
		2, {0x7D, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xEF},
		2, {0xCA, 0x82},
		2, {0x82, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xF0},
		2, {0xCA, 0x90},
		2, {0x90, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xF1},
		2, {0xC9, 0xBB},
		2, {0x7B, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xF2},
		2, {0xC9, 0xAD},
		2, {0x6D, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xF3},
		2, {0xC9, 0x9F},
		2, {0x5F, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xF4},
		2, {0xC9, 0xB2},
		2, {0x72, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xF5},
		2, {0xCA, 0x9D},
		2, {0x9D, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xF6},
		2, {0xCA, 0x8E},
		2, {0x8E, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xF7},
		2, {0xC9, 0xA1},
		2, {0x61, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xF8},
		2, {0xC5, 0x8B},
		2, {0x4B, 0x01},
		3, {0x8F, 0xA9, 0xCB}
	},
	{
		2, {0x85, 0xF9},
		2, {0xC9, 0xB0},
		2, {0x70, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xFA},
		2, {0xCA, 0x81},
		2, {0x81, 0x02},
		0, {0x00}
	},
	{
		2, {0x85, 0xFB},
		2, {0xC4, 0xA7},
		2, {0x27, 0x01},
		3, {0x8F, 0xA9, 0xC4}
	},
	{
		2, {0x85, 0xFC},
		2, {0xCA, 0x95},
		2, {0x95, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x40},
		2, {0xCA, 0x94},
		2, {0x94, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x41},
		2, {0xC9, 0xA6},
		2, {0x66, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x42},
		2, {0xCA, 0x98},
		2, {0x98, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x43},
		2, {0xC7, 0x82},
		2, {0xC2, 0x01},
		0, {0x00}
	},
	{
		2, {0x86, 0x44},
		2, {0xC9, 0x93},
		2, {0x53, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x45},
		2, {0xC9, 0x97},
		2, {0x57, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x46},
		2, {0xCA, 0x84},
		2, {0x84, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x47},
		2, {0xC9, 0xA0},
		2, {0x60, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x48},
		2, {0xC6, 0x93},
		2, {0x93, 0x01},
		0, {0x00}
	},
	{
		2, {0x86, 0x49},
		2, {0xC5, 0x93},
		2, {0x53, 0x01},
		3, {0x8F, 0xA9, 0xCD}
	},
	{
		2, {0x86, 0x4A},
		2, {0xC5, 0x92},
		2, {0x52, 0x01},
		3, {0x8F, 0xA9, 0xAD}
	},
	{
		2, {0x86, 0x4B},
		2, {0xC9, 0xA8},
		2, {0x68, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x4C},
		2, {0xCA, 0x89},
		2, {0x89, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x4D},
		2, {0xC9, 0x98},
		2, {0x58, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x4E},
		2, {0xC9, 0xB5},
		2, {0x75, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x4F},
		2, {0xC9, 0x99},
		2, {0x59, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x50},
		2, {0xC9, 0x9C},
		2, {0x5C, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x51},
		2, {0xC9, 0x9E},
		2, {0x5E, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x52},
		2, {0xC9, 0x90},
		2, {0x50, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x53},
		2, {0xC9, 0xAF},
		2, {0x6F, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x54},
		2, {0xCA, 0x8A},
		2, {0x8A, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x55},
		2, {0xC9, 0xA4},
		2, {0x64, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x56},
		2, {0xCA, 0x8C},
		2, {0x8C, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x57},
		2, {0xC9, 0x94},
		2, {0x54, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x58},
		2, {0xC9, 0x91},
		2, {0x51, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x59},
		2, {0xC9, 0x92},
		2, {0x52, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x5A},
		2, {0xCA, 0x8D},
		2, {0x8D, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x5B},
		2, {0xC9, 0xA5},
		2, {0x65, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x5C},
		2, {0xCA, 0xA2},
		2, {0xA2, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x5D},
		2, {0xCA, 0xA1},
		2, {0xA1, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x5E},
		2, {0xC9, 0x95},
		2, {0x55, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x5F},
		2, {0xCA, 0x91},
		2, {0x91, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x60},
		2, {0xC9, 0xBA},
		2, {0x7A, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x61},
		2, {0xC9, 0xA7},
		2, {0x67, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x62},
		2, {0xC9, 0x9A},
		2, {0x5A, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x63},
		4, {0xC3, 0xA6, 0xCC, 0x80},
		4, {0xE6, 0x00, 0x00, 0x03},
		3, {0x8F, 0xA9, 0xC1}
	},
	{
		2, {0x86, 0x64},
		2, {0xC7, 0xBD},
		2, {0xFD, 0x01},
		0, {0x00}
	},
	{
		2, {0x86, 0x65},
		3, {0xE1, 0xBD, 0xB0},
		2, {0x70, 0x1F},
		0, {0x00}
	},
	{
		2, {0x86, 0x66},
		3, {0xE1, 0xBD, 0xB1},
		2, {0x71, 0x1F},
		0, {0x00}
	},
	{
		2, {0x86, 0x67},
		4, {0xC9, 0x94, 0xCC, 0x80},
		4, {0x54, 0x02, 0x00, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x68},
		4, {0xC9, 0x94, 0xCC, 0x81},
		4, {0x54, 0x02, 0x01, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x69},
		4, {0xCA, 0x8C, 0xCC, 0x80},
		4, {0x8C, 0x02, 0x00, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x6A},
		4, {0xCA, 0x8C, 0xCC, 0x81},
		4, {0x8C, 0x02, 0x01, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x6B},
		4, {0xC9, 0x99, 0xCC, 0x80},
		4, {0x59, 0x02, 0x00, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x6C},
		4, {0xC9, 0x99, 0xCC, 0x81},
		4, {0x59, 0x02, 0x01, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x6D},
		4, {0xC9, 0x9A, 0xCC, 0x80},
		4, {0x5A, 0x02, 0x00, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x6E},
		4, {0xC9, 0x9A, 0xCC, 0x81},
		4, {0x5A, 0x02, 0x01, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x6F},
		3, {0xE1, 0xBD, 0xB2},
		2, {0x72, 0x1F},
		0, {0x00}
	},
	{
		2, {0x86, 0x70},
		3, {0xE1, 0xBD, 0xB3},
		2, {0x73, 0x1F},
		0, {0x00}
	},
	{
		2, {0x86, 0x71},
		2, {0xCD, 0xA1},
		2, {0x61, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x72},
		2, {0xCB, 0x88},
		2, {0xC8, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x73},
		2, {0xCB, 0x8C},
		2, {0xCC, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x74},
		2, {0xCB, 0x90},
		2, {0xD0, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x75},
		2, {0xCB, 0x91},
		2, {0xD1, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x76},
		2, {0xCC, 0x86},
		2, {0x06, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x77},
		3, {0xE2, 0x80, 0xBF},
		2, {0x3F, 0x20},
		0, {0x00}
	},
	{
		2, {0x86, 0x78},
		2, {0xCC, 0x8B},
		2, {0x0B, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x79},
		2, {0xCC, 0x81},
		2, {0x01, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x7A},
		2, {0xCC, 0x84},
		2, {0x04, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x7B},
		2, {0xCC, 0x80},
		2, {0x00, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x7C},
		2, {0xCC, 0x8F},
		2, {0x0F, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x7D},
		2, {0xCC, 0x8C},
		2, {0x0C, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x7E},
		2, {0xCC, 0x82},
		2, {0x02, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x80},
		2, {0xCB, 0xA5},
		2, {0xE5, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x81},
		2, {0xCB, 0xA6},
		2, {0xE6, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x82},
		2, {0xCB, 0xA7},
		2, {0xE7, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x83},
		2, {0xCB, 0xA8},
		2, {0xE8, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x84},
		2, {0xCB, 0xA9},
		2, {0xE9, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x85},
		4, {0xCB, 0xA9, 0xCB, 0xA5},
		4, {0xE9, 0x02, 0xE5, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x86},
		4, {0xCB, 0xA5, 0xCB, 0xA9},
		4, {0xE5, 0x02, 0xE9, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x87},
		2, {0xCC, 0xA5},
		2, {0x25, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x88},
		2, {0xCC, 0xAC},
		2, {0x2C, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x89},
		2, {0xCC, 0xB9},
		2, {0x39, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x8A},
		2, {0xCC, 0x9C},
		2, {0x1C, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x8B},
		2, {0xCC, 0x9F},
		2, {0x1F, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x8C},
		2, {0xCC, 0xA0},
		2, {0x20, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x8D},
		2, {0xCC, 0x88},
		2, {0x08, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x8E},
		2, {0xCC, 0xBD},
		2, {0x3D, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x8F},
		2, {0xCC, 0xA9},
		2, {0x29, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x90},
		2, {0xCC, 0xAF},
		2, {0x2F, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x91},
		2, {0xCB, 0x9E},
		2, {0xDE, 0x02},
		0, {0x00}
	},
	{
		2, {0x86, 0x92},
		2, {0xCC, 0xA4},
		2, {0x24, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x93},
		2, {0xCC, 0xB0},
		2, {0x30, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x94},
		2, {0xCC, 0xBC},
		2, {0x3C, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x95},
		2, {0xCC, 0xB4},
		2, {0x34, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x96},
		2, {0xCC, 0x9D},
		2, {0x1D, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x97},
		2, {0xCC, 0x9E},
		2, {0x1E, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x98},
		2, {0xCC, 0x98},
		2, {0x18, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x99},
		2, {0xCC, 0x99},
		2, {0x19, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x9A},
		2, {0xCC, 0xAA},
		2, {0x2A, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x9B},
		2, {0xCC, 0xBA},
		2, {0x3A, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x9C},
		2, {0xCC, 0xBB},
		2, {0x3B, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x9D},
		2, {0xCC, 0x83},
		2, {0x03, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x9E},
		2, {0xCC, 0x9A},
		2, {0x1A, 0x03},
		0, {0x00}
	},
	{
		2, {0x86, 0x9F},
		3, {0xE2, 0x9D, 0xB6},
		2, {0x76, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA0},
		3, {0xE2, 0x9D, 0xB7},
		2, {0x77, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA1},
		3, {0xE2, 0x9D, 0xB8},
		2, {0x78, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA2},
		3, {0xE2, 0x9D, 0xB9},
		2, {0x79, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA3},
		3, {0xE2, 0x9D, 0xBA},
		2, {0x7A, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA4},
		3, {0xE2, 0x9D, 0xBB},
		2, {0x7B, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA5},
		3, {0xE2, 0x9D, 0xBC},
		2, {0x7C, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA6},
		3, {0xE2, 0x9D, 0xBD},
		2, {0x7D, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA7},
		3, {0xE2, 0x9D, 0xBE},
		2, {0x7E, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA8},
		3, {0xE2, 0x9D, 0xBF},
		2, {0x7F, 0x27},
		0, {0x00}
	},
	{
		2, {0x86, 0xA9},
		3, {0xE2, 0x93, 0xAB},
		2, {0xEB, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xAA},
		3, {0xE2, 0x93, 0xAC},
		2, {0xEC, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xAB},
		3, {0xE2, 0x93, 0xAD},
		2, {0xED, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xAC},
		3, {0xE2, 0x93, 0xAE},
		2, {0xEE, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xAD},
		3, {0xE2, 0x93, 0xAF},
		2, {0xEF, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xAE},
		3, {0xE2, 0x93, 0xB0},
		2, {0xF0, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xAF},
		3, {0xE2, 0x93, 0xB1},
		2, {0xF1, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xB0},
		3, {0xE2, 0x93, 0xB2},
		2, {0xF2, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xB1},
		3, {0xE2, 0x93, 0xB3},
		2, {0xF3, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xB2},
		3, {0xE2, 0x93, 0xB4},
		2, {0xF4, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xB3},
		3, {0xE2, 0x85, 0xB0},
		2, {0x70, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xB4},
		3, {0xE2, 0x85, 0xB1},
		2, {0x71, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xB5},
		3, {0xE2, 0x85, 0xB2},
		2, {0x72, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xB6},
		3, {0xE2, 0x85, 0xB3},
		2, {0x73, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xB7},
		3, {0xE2, 0x85, 0xB4},
		2, {0x74, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xB8},
		3, {0xE2, 0x85, 0xB5},
		2, {0x75, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xB9},
		3, {0xE2, 0x85, 0xB6},
		2, {0x76, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xBA},
		3, {0xE2, 0x85, 0xB7},
		2, {0x77, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xBB},
		3, {0xE2, 0x85, 0xB8},
		2, {0x78, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xBC},
		3, {0xE2, 0x85, 0xB9},
		2, {0x79, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xBD},
		3, {0xE2, 0x85, 0xBA},
		2, {0x7A, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xBE},
		3, {0xE2, 0x85, 0xBB},
		2, {0x7B, 0x21},
		0, {0x00}
	},
	{
		2, {0x86, 0xBF},
		3, {0xE2, 0x93, 0x90},
		2, {0xD0, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC0},
		3, {0xE2, 0x93, 0x91},
		2, {0xD1, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC1},
		3, {0xE2, 0x93, 0x92},
		2, {0xD2, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC2},
		3, {0xE2, 0x93, 0x93},
		2, {0xD3, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC3},
		3, {0xE2, 0x93, 0x94},
		2, {0xD4, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC4},
		3, {0xE2, 0x93, 0x95},
		2, {0xD5, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC5},
		3, {0xE2, 0x93, 0x96},
		2, {0xD6, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC6},
		3, {0xE2, 0x93, 0x97},
		2, {0xD7, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC7},
		3, {0xE2, 0x93, 0x98},
		2, {0xD8, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC8},
		3, {0xE2, 0x93, 0x99},
		2, {0xD9, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xC9},
		3, {0xE2, 0x93, 0x9A},
		2, {0xDA, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xCA},
		3, {0xE2, 0x93, 0x9B},
		2, {0xDB, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xCB},
		3, {0xE2, 0x93, 0x9C},
		2, {0xDC, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xCC},
		3, {0xE2, 0x93, 0x9D},
		2, {0xDD, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xCD},
		3, {0xE2, 0x93, 0x9E},
		2, {0xDE, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xCE},
		3, {0xE2, 0x93, 0x9F},
		2, {0xDF, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xCF},
		3, {0xE2, 0x93, 0xA0},
		2, {0xE0, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD0},
		3, {0xE2, 0x93, 0xA1},
		2, {0xE1, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD1},
		3, {0xE2, 0x93, 0xA2},
		2, {0xE2, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD2},
		3, {0xE2, 0x93, 0xA3},
		2, {0xE3, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD3},
		3, {0xE2, 0x93, 0xA4},
		2, {0xE4, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD4},
		3, {0xE2, 0x93, 0xA5},
		2, {0xE5, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD5},
		3, {0xE2, 0x93, 0xA6},
		2, {0xE6, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD6},
		3, {0xE2, 0x93, 0xA7},
		2, {0xE7, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD7},
		3, {0xE2, 0x93, 0xA8},
		2, {0xE8, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD8},
		3, {0xE2, 0x93, 0xA9},
		2, {0xE9, 0x24},
		0, {0x00}
	},
	{
		2, {0x86, 0xD9},
		3, {0xE3, 0x8B, 0x90},
		2, {0xD0, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xDA},
		3, {0xE3, 0x8B, 0x91},
		2, {0xD1, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xDB},
		3, {0xE3, 0x8B, 0x92},
		2, {0xD2, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xDC},
		3, {0xE3, 0x8B, 0x93},
		2, {0xD3, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xDD},
		3, {0xE3, 0x8B, 0x94},
		2, {0xD4, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xDE},
		3, {0xE3, 0x8B, 0x95},
		2, {0xD5, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xDF},
		3, {0xE3, 0x8B, 0x96},
		2, {0xD6, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE0},
		3, {0xE3, 0x8B, 0x97},
		2, {0xD7, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE1},
		3, {0xE3, 0x8B, 0x98},
		2, {0xD8, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE2},
		3, {0xE3, 0x8B, 0x99},
		2, {0xD9, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE3},
		3, {0xE3, 0x8B, 0x9A},
		2, {0xDA, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE4},
		3, {0xE3, 0x8B, 0x9B},
		2, {0xDB, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE5},
		3, {0xE3, 0x8B, 0x9C},
		2, {0xDC, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE6},
		3, {0xE3, 0x8B, 0x9D},
		2, {0xDD, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE7},
		3, {0xE3, 0x8B, 0x9E},
		2, {0xDE, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE8},
		3, {0xE3, 0x8B, 0x9F},
		2, {0xDF, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xE9},
		3, {0xE3, 0x8B, 0xA0},
		2, {0xE0, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xEA},
		3, {0xE3, 0x8B, 0xA1},
		2, {0xE1, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xEB},
		3, {0xE3, 0x8B, 0xA2},
		2, {0xE2, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xEC},
		3, {0xE3, 0x8B, 0xA3},
		2, {0xE3, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xED},
		3, {0xE3, 0x8B, 0xBA},
		2, {0xFA, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xEE},
		3, {0xE3, 0x8B, 0xA9},
		2, {0xE9, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xEF},
		3, {0xE3, 0x8B, 0xA5},
		2, {0xE5, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xF0},
		3, {0xE3, 0x8B, 0xAD},
		2, {0xED, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xF1},
		3, {0xE3, 0x8B, 0xAC},
		2, {0xEC, 0x32},
		0, {0x00}
	},
	{
		2, {0x86, 0xF2},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x86, 0xF3},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x86, 0xF4},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x86, 0xF5},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x86, 0xF6},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x86, 0xF7},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x86, 0xF8},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x86, 0xF9},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x86, 0xFA},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x86, 0xFB},
		3, {0xE2, 0x81, 0x91},
		2, {0x51, 0x20},
		0, {0x00}
	},
	{
		2, {0x86, 0xFC},
		3, {0xE2, 0x81, 0x82},
		2, {0x42, 0x20},
		0, {0x00}
	},
	{
		2, {0x87, 0x40},
		3, {0xE2, 0x91, 0xA0},
		2, {0x60, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x41},
		3, {0xE2, 0x91, 0xA1},
		2, {0x61, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x42},
		3, {0xE2, 0x91, 0xA2},
		2, {0x62, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x43},
		3, {0xE2, 0x91, 0xA3},
		2, {0x63, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x44},
		3, {0xE2, 0x91, 0xA4},
		2, {0x64, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x45},
		3, {0xE2, 0x91, 0xA5},
		2, {0x65, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x46},
		3, {0xE2, 0x91, 0xA6},
		2, {0x66, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x47},
		3, {0xE2, 0x91, 0xA7},
		2, {0x67, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x48},
		3, {0xE2, 0x91, 0xA8},
		2, {0x68, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x49},
		3, {0xE2, 0x91, 0xA9},
		2, {0x69, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x4A},
		3, {0xE2, 0x91, 0xAA},
		2, {0x6A, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x4B},
		3, {0xE2, 0x91, 0xAB},
		2, {0x6B, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x4C},
		3, {0xE2, 0x91, 0xAC},
		2, {0x6C, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x4D},
		3, {0xE2, 0x91, 0xAD},
		2, {0x6D, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x4E},
		3, {0xE2, 0x91, 0xAE},
		2, {0x6E, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x4F},
		3, {0xE2, 0x91, 0xAF},
		2, {0x6F, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x50},
		3, {0xE2, 0x91, 0xB0},
		2, {0x70, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x51},
		3, {0xE2, 0x91, 0xB1},
		2, {0x71, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x52},
		3, {0xE2, 0x91, 0xB2},
		2, {0x72, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x53},
		3, {0xE2, 0x91, 0xB3},
		2, {0x73, 0x24},
		0, {0x00}
	},
	{
		2, {0x87, 0x54},
		3, {0xE2, 0x85, 0xA0},
		2, {0x60, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x55},
		3, {0xE2, 0x85, 0xA1},
		2, {0x61, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x56},
		3, {0xE2, 0x85, 0xA2},
		2, {0x62, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x57},
		3, {0xE2, 0x85, 0xA3},
		2, {0x63, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x58},
		3, {0xE2, 0x85, 0xA4},
		2, {0x64, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x59},
		3, {0xE2, 0x85, 0xA5},
		2, {0x65, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x5A},
		3, {0xE2, 0x85, 0xA6},
		2, {0x66, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x5B},
		3, {0xE2, 0x85, 0xA7},
		2, {0x67, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x5C},
		3, {0xE2, 0x85, 0xA8},
		2, {0x68, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x5D},
		3, {0xE2, 0x85, 0xA9},
		2, {0x69, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x5E},
		3, {0xE2, 0x85, 0xAA},
		2, {0x6A, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x5F},
		3, {0xE3, 0x8D, 0x89},
		2, {0x49, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x60},
		3, {0xE3, 0x8C, 0x94},
		2, {0x14, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x61},
		3, {0xE3, 0x8C, 0xA2},
		2, {0x22, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x62},
		3, {0xE3, 0x8D, 0x8D},
		2, {0x4D, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x63},
		3, {0xE3, 0x8C, 0x98},
		2, {0x18, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x64},
		3, {0xE3, 0x8C, 0xA7},
		2, {0x27, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x65},
		3, {0xE3, 0x8C, 0x83},
		2, {0x03, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x66},
		3, {0xE3, 0x8C, 0xB6},
		2, {0x36, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x67},
		3, {0xE3, 0x8D, 0x91},
		2, {0x51, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x68},
		3, {0xE3, 0x8D, 0x97},
		2, {0x57, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x69},
		3, {0xE3, 0x8C, 0x8D},
		2, {0x0D, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x6A},
		3, {0xE3, 0x8C, 0xA6},
		2, {0x26, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x6B},
		3, {0xE3, 0x8C, 0xA3},
		2, {0x23, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x6C},
		3, {0xE3, 0x8C, 0xAB},
		2, {0x2B, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x6D},
		3, {0xE3, 0x8D, 0x8A},
		2, {0x4A, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x6E},
		3, {0xE3, 0x8C, 0xBB},
		2, {0x3B, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x6F},
		3, {0xE3, 0x8E, 0x9C},
		2, {0x9C, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x70},
		3, {0xE3, 0x8E, 0x9D},
		2, {0x9D, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x71},
		3, {0xE3, 0x8E, 0x9E},
		2, {0x9E, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x72},
		3, {0xE3, 0x8E, 0x8E},
		2, {0x8E, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x73},
		3, {0xE3, 0x8E, 0x8F},
		2, {0x8F, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x74},
		3, {0xE3, 0x8F, 0x84},
		2, {0xC4, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x75},
		3, {0xE3, 0x8E, 0xA1},
		2, {0xA1, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x76},
		3, {0xE2, 0x85, 0xAB},
		2, {0x6B, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x77},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x78},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x79},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x7A},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x7B},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x7C},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x7D},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x7E},
		3, {0xE3, 0x8D, 0xBB},
		2, {0x7B, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x80},
		3, {0xE3, 0x80, 0x9D},
		2, {0x1D, 0x30},
		0, {0x00}
	},
	{
		2, {0x87, 0x81},
		3, {0xE3, 0x80, 0x9F},
		2, {0x1F, 0x30},
		0, {0x00}
	},
	{
		2, {0x87, 0x82},
		3, {0xE2, 0x84, 0x96},
		2, {0x16, 0x21},
		3, {0x8F, 0xA2, 0xF1}
	},
	{
		2, {0x87, 0x83},
		3, {0xE3, 0x8F, 0x8D},
		2, {0xCD, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x84},
		3, {0xE2, 0x84, 0xA1},
		2, {0x21, 0x21},
		0, {0x00}
	},
	{
		2, {0x87, 0x85},
		3, {0xE3, 0x8A, 0xA4},
		2, {0xA4, 0x32},
		0, {0x00}
	},
	{
		2, {0x87, 0x86},
		3, {0xE3, 0x8A, 0xA5},
		2, {0xA5, 0x32},
		0, {0x00}
	},
	{
		2, {0x87, 0x87},
		3, {0xE3, 0x8A, 0xA6},
		2, {0xA6, 0x32},
		0, {0x00}
	},
	{
		2, {0x87, 0x88},
		3, {0xE3, 0x8A, 0xA7},
		2, {0xA7, 0x32},
		0, {0x00}
	},
	{
		2, {0x87, 0x89},
		3, {0xE3, 0x8A, 0xA8},
		2, {0xA8, 0x32},
		0, {0x00}
	},
	{
		2, {0x87, 0x8A},
		3, {0xE3, 0x88, 0xB1},
		2, {0x31, 0x32},
		0, {0x00}
	},
	{
		2, {0x87, 0x8B},
		3, {0xE3, 0x88, 0xB2},
		2, {0x32, 0x32},
		0, {0x00}
	},
	{
		2, {0x87, 0x8C},
		3, {0xE3, 0x88, 0xB9},
		2, {0x39, 0x32},
		0, {0x00}
	},
	{
		2, {0x87, 0x8D},
		3, {0xE3, 0x8D, 0xBE},
		2, {0x7E, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x8E},
		3, {0xE3, 0x8D, 0xBD},
		2, {0x7D, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x8F},
		3, {0xE3, 0x8D, 0xBC},
		2, {0x7C, 0x33},
		0, {0x00}
	},
	{
		2, {0x87, 0x90},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x91},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x92},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x93},
		3, {0xE2, 0x88, 0xAE},
		2, {0x2E, 0x22},
		0, {0x00}
	},
	{
		2, {0x87, 0x94},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x95},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x96},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x97},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x98},
		3, {0xE2, 0x88, 0x9F},
		2, {0x1F, 0x22},
		0, {0x00}
	},
	{
		2, {0x87, 0x99},
		3, {0xE2, 0x8A, 0xBF},
		2, {0xBF, 0x22},
		0, {0x00}
	},
	{
		2, {0x87, 0x9A},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x9B},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x9C},
		0, {0x00},
		0, {0x00},
		0, {0x00}
	},
	{
		2, {0x87, 0x9D},
		3, {0xE2, 0x9D, 0x96},
		2, {0x56, 0x27},
		0, {0x00}
	},
	{
		2, {0x87, 0x9E},
		3, {0xE2, 0x98, 0x9E},
		2, {0x1E, 0x26},
		0, {0x00}
	},
	{
		2, {0x87, 0x9F},
		3, {0xE4, 0xBF, 0xB1},
		2, {0xF1, 0x4F},
		0, {0x00}
	},
	{
		2, {0x87, 0xA0},
		4, {0xF0, 0xA0, 0x80, 0x8B},
		4, {0x40, 0xD8, 0x0B, 0xDC},
		0, {0x00}
	},
	{
		2, {0x87, 0xA1},
		3, {0xE3, 0x90, 0x82},
		2, {0x02, 0x34},
		0, {0x00}
	},
	{
		2, {0x87, 0xA2},
		3, {0xE4, 0xB8, 0xA8},
		2, {0x28, 0x4E},
		3, {0x8F, 0xB0, 0xA9}
	},
	{
		2, {0x87, 0xA3},
		3, {0xE4, 0xB8, 0xAF},
		2, {0x2F, 0x4E},
		3, {0x8F, 0xB0, 0xAC}
	},
	{
		2, {0x87, 0xA4},
		3, {0xE4, 0xB8, 0xB0},
		2, {0x30, 0x4E},
		3, {0x8F, 0xB0, 0xAD}
	},
	{
		2, {0x87, 0xA5},
		3, {0xE4, 0xBA, 0x8D},
		2, {0x8D, 0x4E},
		3, {0x8F, 0xB0, 0xBD}
	},
	{
		2, {0x87, 0xA6},
		3, {0xE4, 0xBB, 0xA1},
		2, {0xE1, 0x4E},
		3, {0x8F, 0xB0, 0xC8}
	},
	{
		2, {0x87, 0xA7},
		3, {0xE4, 0xBB, 0xBD},
		2, {0xFD, 0x4E},
		3, {0x8F, 0xB0, 0xCF}
	},
	{
		2, {0x87, 0xA8},
		3, {0xE4, 0xBB, 0xBF},
		2, {0xFF, 0x4E},
		3, {0x8F, 0xB0, 0xD1}
	},
	{
		2, {0x87, 0xA9},
		3, {0xE4, 0xBC, 0x83},
		2, {0x03, 0x4F},
		3, {0x8F, 0xB0, 0xD4}
	},
	{
		2, {0x87, 0xAA},
		3, {0xE4, 0xBC, 0x8B},
		2, {0x0B, 0x4F},
		3, {0x8F, 0xB0, 0xD6}
	},
	{
		2, {0x87, 0xAB},
		3, {0xE4, 0xBD, 0xA0},
		2, {0x60, 0x4F},
		3, {0x8F, 0xB0, 0xDF}
	},
	{
		2, {0x87, 0xAC},
		3, {0xE4, 0xBD, 0x88},
		2, {0x48, 0x4F},
		3, {0x8F, 0xB0, 0xE8}
	},
	{
		2, {0x87, 0xAD},
		3, {0xE4, 0xBD, 0x89},
		2, {0x49, 0x4F},
		3, {0x8F, 0xB0, 0xE9}
	},
	{
		2, {0x87, 0xAE},
		3, {0xE4, 0xBD, 0x96},
		2, {0x56, 0x4F},
		3, {0x8F, 0xB0, 0xEE}
	},
	{
		2, {0x87, 0xAF},
		3, {0xE4, 0xBD, 0x9F},
		2, {0x5F, 0x4F},
		3, {0x8F, 0xB0, 0xF0}
	},
	{
		2, {0x87, 0xB0},
		3, {0xE4, 0xBD, 0xAA},
		2, {0x6A, 0x4F},
		3, {0x8F, 0xB0, 0xF2}
	},
	{
		2, {0x87, 0xB1},
		3, {0xE4, 0xBD, 0xAC},
		2, {0x6C, 0x4F},
		3, {0x8F, 0xB0, 0xF3}
	},
	{
		2, {0x87, 0xB2},
		3, {0xE4, 0xBD, 0xBE},
		2, {0x7E, 0x4F},
		3, {0x8F, 0xB0, 0xFB}
	},
	{
		2, {0x87, 0xB3},
		3, {0xE4, 0xBE, 0x8A},
		2, {0x8A, 0x4F},
		3, {0x8F, 0xB1, 0xA3}
	},
	{
		2, {0x87, 0xB4},
		3, {0xE4, 0xBE, 0x94},
		2, {0x94, 0x4F},
		3, {0x8F, 0xB1, 0xA9}
	},
	{
		2, {0x87, 0xB5},
		3, {0xE4, 0xBE, 0x97},
		2, {0x97, 0x4F},
		3, {0x8F, 0xB1, 0xAA}
	},
	{
		2, {0x87, 0xB6},
		3, {0xEF, 0xA8, 0xB0},
		2, {0x30, 0xFA},
		0, {0x00}
	},
	{
		2, {0x87, 0xB7},
		3, {0xE4, 0xBF, 0x89},
		2, {0xC9, 0x4F},
		3, {0x8F, 0xB1, 0xBB}
	},
	{
		2, {0x87, 0xB8},
		3, {0xE4, 0xBF, 0xA0},
		2, {0xE0, 0x4F},
		3, {0x8F, 0xB1, 0xC2}
	},
	{
		2, {0x87, 0xB9},
		3, {0xE5, 0x80, 0x81},
		2, {0x01, 0x50},
		3, {0x8F, 0xB1, 0xCA}
	},
	{
		2, {0x87, 0xBA},
		3, {0xE5, 0x80, 0x82},
		2, {0x02, 0x50},
		0, {0x00}
	},
	{
		2, {0x87, 0xBB},
		3, {0xE5, 0x80, 0x8E},
		2, {0x0E, 0x50},
		3, {0x8F, 0xB1, 0xCF}
	},
	{
		2, {0x87, 0xBC},
		3, {0xE5, 0x80, 0x98},
		2, {0x18, 0x50},
		3, {0x8F, 0xB1, 0xD3}
	},
	{
		2, {0x87, 0xBD},
		3, {0xE5, 0x80, 0xA7},
		2, {0x27, 0x50},
		3, {0x8F, 0xB1, 0xD9}
	},
	{
		2, {0x87, 0xBE},
		3, {0xE5, 0x80, 0xAE},
		2, {0x2E, 0x50},
		3, {0x8F, 0xB1, 0xDA}
	},
	{
		2, {0x87, 0xBF},
		3, {0xE5, 0x81, 0x80},
		2, {0x40, 0x50},
		3, {0x8F, 0xB1, 0xDF}
	},
	{
		2, {0x87, 0xC0},
		3, {0xE5, 0x80, 0xBB},
		2, {0x3B, 0x50},
		3, {0x8F, 0xB1, 0xF6}
	},
	{
		2, {0x87, 0xC1},
		3, {0xE5, 0x81, 0x81},
		2, {0x41, 0x50},
		3, {0x8F, 0xB1, 0xE0}
	},
	{
		2, {0x87, 0xC2},
		3, {0xE5, 0x82, 0x94},
		2, {0x94, 0x50},
		3, {0x8F, 0xB2, 0xA3}
	},
	{
		2, {0x87, 0xC3},
		3, {0xE5, 0x83, 0x8C},
		2, {0xCC, 0x50},
		3, {0x8F, 0xB2, 0xB6}
	},
	{
		2, {0x87, 0xC4},
		3, {0xE5, 0x83, 0xB2},
		2, {0xF2, 0x50},
		3, {0x8F, 0xB2, 0xD1}
	},
	{
		2, {0x87, 0xC5},
		3, {0xE5, 0x83, 0x90},
		2, {0xD0, 0x50},
		3, {0x8F, 0xB2, 0xB8}
	},
	{
		2, {0x87, 0xC6},
		3, {0xE5, 0x83, 0xA6},
		2, {0xE6, 0x50},
		3, {0x8F, 0xB2, 0xC1}
	},
	{
		2, {0x87, 0xC7},
		3, {0xEF, 0xA8, 0xB1},
		2, {0x31, 0xFA},
		0, {0x00}
	},
	{
		2, {0x87, 0xC8},
		3, {0xE5, 0x84, 0x86},
		2, {0x06, 0x51},
		3, {0x8F, 0xB2, 0xCA}
	},
	{
		2, {0x87, 0xC9},
		3, {0xE5, 0x84, 0x83},
		2, {0x03, 0x51},
		3, {0x8F, 0xB2, 0xC9}
	},
	{
		2, {0x87, 0xCA},
		3, {0xE5, 0x84, 0x8B},
		2, {0x0B, 0x51},
		3, {0x8F, 0xB2, 0xCD}
	},
	{
		2, {0x87, 0xCB},
		3, {0xE5, 0x84, 0x9E},
		2, {0x1E, 0x51},
		3, {0x8F, 0xB2, 0xD8}
	},
	{
		2, {0x87, 0xCC},
		3, {0xE5, 0x84, 0xB5},
		2, {0x35, 0x51},
		3, {0x8F, 0xB2, 0xE2}
	},
	{
		2, {0x87, 0xCD},
		3, {0xE5, 0x85, 0x8A},
		2, {0x4A, 0x51},
		3, {0x8F, 0xB2, 0xE6}
	},
	{
		2, {0x87, 0xCE},
		3, {0xEF, 0xA8, 0xB2},
		2, {0x32, 0xFA},
		0, {0x00}
	},
	{
		2, {0x87, 0xCF},
		3, {0xE5, 0x85, 0x95},
		2, {0x55, 0x51},
		3, {0x8F, 0xB2, 0xE9}
	},
	{
		2, {0x87, 0xD0},
		3, {0xE5, 0x85, 0x97},
		2, {0x57, 0x51},
		3, {0x8F, 0xB2, 0xEA}
	},
	{
		2, {0x87, 0xD1},
		3, {0xE3, 0x92, 0xB5},
		2, {0xB5, 0x34},
		0, {0x00}
	},
	{
		2, {0x87, 0xD2},
		3, {0xE5, 0x86, 0x9D},
		2, {0x9D, 0x51},
		3, {0x8F, 0xB2, 0xF5}
	},
	{
		2, {0x87, 0xD3},
		3, {0xE5, 0x87, 0x83},
		2, {0xC3, 0x51},
		0, {0x00}
	},
	{
		2, {0x87, 0xD4},
		3, {0xE5, 0x87, 0x8A},
		2, {0xCA, 0x51},
		0, {0x00}
	},
	{
		2, {0x87, 0xD5},
		3, {0xE5, 0x87, 0x9E},
		2, {0xDE, 0x51},
		3, {0x8F, 0xB3, 0xA8}
	},
	{
		2, {0x87, 0xD6},
		3, {0xE5, 0x87, 0xA2},
		2, {0xE2, 0x51},
		3, {0x8F, 0xB3, 0xA9}
	},
	{
		2, {0x87, 0xD7},
		3, {0xE5, 0x87, 0xAE},
		2, {0xEE, 0x51},
		3, {0x8F, 0xB3, 0xAB}
	},
	{
		2, {0x87, 0xD8},
		3, {0xE5, 0x88, 0x81},
		2, {0x01, 0x52},
		3, {0x8F, 0xB3, 0xB0}
	},
	{
		2, {0x87, 0xD9},
		3, {0xE3, 0x93, 0x9B},
		2, {0xDB, 0x34},
		0, {0x00}
	},
	{
		2, {0x87, 0xDA},
		3, {0xE5, 0x88, 0x93},
		2, {0x13, 0x52},
		3, {0x8F, 0xB3, 0xB4}
	},
	{
		2, {0x87, 0xDB},
		3, {0xE5, 0x88, 0x95},
		2, {0x15, 0x52},
		3, {0x8F, 0xB3, 0xB5}
	},
	{
		2, {0x87, 0xDC},
		3, {0xE5, 0x89, 0x89},
		2, {0x49, 0x52},
		3, {0x8F, 0xB3, 0xBF}
	},
	{
		2, {0x87, 0xDD},
		3, {0xE5, 0x89, 0x97},
		2, {0x57, 0x52},
		3, {0x8F, 0xB3, 0xC1}
	},
	{
		2, {0x87, 0xDE},
		3, {0xE5, 0x89, 0xA1},
		2, {0x61, 0x52},
		3, {0x8F, 0xB3, 0xC7}
	},
	{
		2, {0x87, 0xDF},
		3, {0xE5, 0x8A, 0x93},
		2, {0x93, 0x52},
		3, {0x8F, 0xB3, 0xD2}
	},
	{
		2, {0x87, 0xE0},
		3, {0xE5, 0x8B, 0x88},
		2, {0xC8, 0x52},
		3, {0x8F, 0xB3, 0xE8}
	},
	{
		2, {0x87, 0xE1},
		3, {0xEF, 0xA8, 0xB3},
		2, {0x33, 0xFA},
		0, {0x00}
	},
	{
		2, {0x87, 0xE2},
		3, {0xE5, 0x8B, 0x8C},
		2, {0xCC, 0x52},
		3, {0x8F, 0xB3, 0xE9}
	},
	{
		2, {0x87, 0xE3},
		3, {0xE5, 0x8B, 0x90},
		2, {0xD0, 0x52},
		0, {0x00}
	},
	{
		2, {0x87, 0xE4},
		3, {0xE5, 0x8B, 0x96},
		2, {0xD6, 0x52},
		3, {0x8F, 0xB3, 0xED}
	},
	{
		2, {0x87, 0xE5},
		3, {0xE5, 0x8B, 0x9B},
		2, {0xDB, 0x52},
		3, {0x8F, 0xB3, 0xEE}
	},
	{
		2, {0x87, 0xE6},
		3, {0xEF, 0xA8, 0xB4},
		2, {0x34, 0xFA},
		0, {0x00}
	},
	{
		2, {0x87, 0xE7},
		3, {0xE5, 0x8B, 0xB0},
		2, {0xF0, 0x52},
		3, {0x8F, 0xB3, 0xF6}
	},
	{
		2, {0x87, 0xE8},
		3, {0xE5, 0x8B, 0xBB},
		2, {0xFB, 0x52},
		0, {0x00}
	},
	{
		2, {0x87, 0xE9},
		3, {0xE5, 0x8C, 0x80},
		2, {0x00, 0x53},
		3, {0x8F, 0xB3, 0xFB}
	},
	{
		2, {0x87, 0xEA},
		3, {0xE5, 0x8C, 0x87},
		2, {0x07, 0x53},
		0, {0x00}
	},
	{
		2, {0x87, 0xEB},
		3, {0xE5, 0x8C, 0x9C},
		2, {0x1C, 0x53},
		3, {0x8F, 0xB4, 0xA6}
	},
	{
		2, {0x87, 0xEC},
		3, {0xEF, 0xA8, 0xB5},
		2, {0x35, 0xFA},
		0, {0x00}
	},
	{
		2, {0x87, 0xED},
		3, {0xE5, 0x8D, 0xA1},
		2, {0x61, 0x53},
		3, {0x8F, 0xB4, 0xBB}
	},
	{
		2, {0x87, 0xEE},
		3, {0xE5, 0x8D, 0xA3},
		2, {0x63, 0x53},
		3, {0x8F, 0xB4, 0xBC}
	},
	{
		2, {0x87, 0xEF},
		3, {0xE5, 0x8D, 0xBD},
		2, {0x7D, 0x53},
		0, {0x00}
	},
	{
		2, {0x87, 0xF0},
		3, {0xE5, 0x8E, 0x93},
		2, {0x93, 0x53},
		3, {0x8F, 0xB4, 0xC7}
	},
	{
		2, {0x87, 0xF1},
		3, {0xE5, 0x8E, 0x9D},
		2, {0x9D, 0x53},
		3, {0x8F, 0xB4, 0xCA}
	},
	{
		2, {0x87, 0xF2},
		3, {0xE5, 0x8E, 0xB2},
		2, {0xB2, 0x53},
		3, {0x8F, 0xB4, 0xD0}
	},
	{
		2, {0x87, 0xF3},
		3, {0xE5, 0x90, 0x92},
		2, {0x12, 0x54},
		0, {0x00}
	},
	{
		2, {0x87, 0xF4},
		3, {0xE5, 0x90, 0xA7},
		2, {0x27, 0x54},
		3, {0x8F, 0xB4, 0xE8}
	},
	{
		2, {0x87, 0xF5},
		3, {0xE5, 0x91, 0x8D},
		2, {0x4D, 0x54},
		3, {0x8F, 0xB4, 0xF2}
	},
	{
		2, {0x87, 0xF6},
		3, {0xE5, 0x92, 0x9C},
		2, {0x9C, 0x54},
		3, {0x8F, 0xB5, 0xAA}
	},
	{
		2, {0x87, 0xF7},
		3, {0xE5, 0x91, 0xAB},
		2, {0x6B, 0x54},
		3, {0x8F, 0xB4, 0xFA}
	},
	{
		2, {0x87, 0xF8},
		3, {0xE5, 0x91, 0xB4},
		2, {0x74, 0x54},
		3, {0x8F, 0xB4, 0xFD}
	},
	{
		2, {0x87, 0xF9},
		3, {0xE5, 0x91, 0xBF},
		2, {0x7F, 0x54},
		3, {0x8F, 0xB4, 0xFE}
	},
	{
		2, {0x87, 0xFA},
		3, {0xE5, 0x92, 0x88},
		2, {0x88, 0x54},
		3, {0x8F, 0xB5, 0xA4}
	},
	{
		2, {0x87, 0xFB},
		3, {0xE5, 0x92, 0x96},
		2, {0x96, 0x54},
		3, {0x8F, 0xB5, 0xA9}
	},
	{
		2, {0x87, 0xFC},
		3, {0xE5, 0x92, 0xA1},
		2, {0xA1, 0x54},
		3, {0x8F, 0xB5, 0xAC}
	},
	{
		2, {0x88, 0x40},
		3, {0xE5, 0x92, 0xA9},
		2, {0xA9, 0x54},
		3, {0x8F, 0xB5, 0xAF}
	},
	{
		2, {0x88, 0x41},
		3, {0xE5, 0x93, 0x86},
		2, {0xC6, 0x54},
		3, {0x8F, 0xB5, 0xB9}
	},
	{
		2, {0x88, 0x42},
		3, {0xE5, 0x93, 0xBF},
		2, {0xFF, 0x54},
		3, {0x8F, 0xB5, 0xC4}
	},
	{
		2, {0x88, 0x43},
		3, {0xE5, 0x94, 0x8E},
		2, {0x0E, 0x55},
		3, {0x8F, 0xB5, 0xCC}
	},
	{
		2, {0x88, 0x44},
		3, {0xE5, 0x94, 0xAB},
		2, {0x2B, 0x55},
		3, {0x8F, 0xB5, 0xCF}
	},
	{
		2, {0x88, 0x45},
		3, {0xE5, 0x94, 0xB5},
		2, {0x35, 0x55},
		3, {0x8F, 0xB5, 0xD1}
	},
	{
		2, {0x88, 0x46},
		3, {0xE5, 0x95, 0x90},
		2, {0x50, 0x55},
		3, {0x8F, 0xB5, 0xDB}
	},
	{
		2, {0x88, 0x47},
		3, {0xE5, 0x95, 0x9E},
		2, {0x5E, 0x55},
		3, {0x8F, 0xB5, 0xE0}
	},
	{
		2, {0x88, 0x48},
		3, {0xE5, 0x96, 0x81},
		2, {0x81, 0x55},
		3, {0x8F, 0xB5, 0xE6}
	},
	{
		2, {0x88, 0x49},
		3, {0xE5, 0x96, 0x86},
		2, {0x86, 0x55},
		3, {0x8F, 0xB5, 0xE8}
	},
	{
		2, {0x88, 0x4A},
		3, {0xE5, 0x96, 0x8E},
		2, {0x8E, 0x55},
		3, {0x8F, 0xB5, 0xEA}
	},
	{
		2, {0x88, 0x4B},
		3, {0xEF, 0xA8, 0xB6},
		2, {0x36, 0xFA},
		0, {0x00}
	},
	{
		2, {0x88, 0x4C},
		3, {0xE5, 0x96, 0xAD},
		2, {0xAD, 0x55},
		3, {0x8F, 0xB5, 0xF3}
	},
	{
		2, {0x88, 0x4D},
		3, {0xE5, 0x97, 0x8E},
		2, {0xCE, 0x55},
		3, {0x8F, 0xB5, 0xFC}
	},
	{
		2, {0x88, 0x4E},
		3, {0xEF, 0xA8, 0xB7},
		2, {0x37, 0xFA},
		0, {0x00}
	},
	{
		2, {0x88, 0x4F},
		3, {0xE5, 0x98, 0x88},
		2, {0x08, 0x56},
		3, {0x8F, 0xB6, 0xAB}
	},
	{
		2, {0x88, 0x50},
		3, {0xE5, 0x98, 0x8E},
		2, {0x0E, 0x56},
		3, {0x8F, 0xB6, 0xAE}
	},
	{
		2, {0x88, 0x51},
		3, {0xE5, 0x98, 0xBB},
		2, {0x3B, 0x56},
		3, {0x8F, 0xB6, 0xBA}
	},
	{
		2, {0x88, 0x52},
		3, {0xE5, 0x99, 0x89},
		2, {0x49, 0x56},
		3, {0x8F, 0xB6, 0xC3}
	},
	{
		2, {0x88, 0x53},
		3, {0xE5, 0x99, 0xB6},
		2, {0x76, 0x56},
		0, {0x00}
	},
	{
		2, {0x88, 0x54},
		3, {0xE5, 0x99, 0xA6},
		2, {0x66, 0x56},
		3, {0x8F, 0xB6, 0xCD}
	},
	{
		2, {0x88, 0x55},
		3, {0xEF, 0xA8, 0xB8},
		2, {0x38, 0xFA},
		0, {0x00}
	},
	{
		2, {0x88, 0x56},
		3, {0xE5, 0x99, 0xAF},
		2, {0x6F, 0x56},
		3, {0x8F, 0xB6, 0xD0}
	},
	{
		2, {0x88, 0x57},
		3, {0xE5, 0x99, 0xB1},
		2, {0x71, 0x56},
		3, {0x8F, 0xB6, 0xD1}
	},
	{
		2, {0x88, 0x58},
		3, {0xE5, 0x99, 0xB2},
		2, {0x72, 0x56},
		3, {0x8F, 0xB6, 0xD2}
	},
	{
		2, {0x88, 0x59},
		3, {0xE5, 0x9A, 0x99},
		2, {0x99, 0x56},
		3, {0x8F, 0xB6, 0xDA}
	},
	{
		2, {0x88, 0x5A},
		3, {0xE5, 0x9A, 0x9E},
		2, {0x9E, 0x56},
		3, {0x8F, 0xB6, 0xDD}
	},
	{
		2, {0x88, 0x5B},
		3, {0xE5, 0x9A, 0xA9},
		2, {0xA9, 0x56},
		3, {0x8F, 0xB6, 0xE2}
	},
	{
		2, {0x88, 0x5C},
		3, {0xE5, 0x9A, 0xAC},
		2, {0xAC, 0x56},
		3, {0x8F, 0xB6, 0xE4}
	},
	{
		2, {0x88, 0x5D},
		3, {0xE5, 0x9A, 0xB3},
		2, {0xB3, 0x56},
		3, {0x8F, 0xB6, 0xE7}
	},
	{
		2, {0x88, 0x5E},
		3, {0xE5, 0x9B, 0x89},
		2, {0xC9, 0x56},
		3, {0x8F, 0xB6, 0xEB}
	},
	{
		2, {0x88, 0x5F},
		3, {0xE5, 0x9B, 0x8A},
		2, {0xCA, 0x56},
		3, {0x8F, 0xB6, 0xEC}
	},
	{
		2, {0x88, 0x60},
		3, {0xE5, 0x9C, 0x8A},
		2, {0x0A, 0x57},
		3, {0x8F, 0xB7, 0xA6}
	},
	{
		2, {0x88, 0x61},
		4, {0xF0, 0xA1, 0x88, 0xBD},
		4, {0x44, 0xD8, 0x3D, 0xDE},
		0, {0x00}
	},
	{
		2, {0x88, 0x62},
		3, {0xE5, 0x9C, 0xA1},
		2, {0x21, 0x57},
		0, {0x00}
	},
	{
		2, {0x88, 0x63},
		3, {0xE5, 0x9C, 0xAF},
		2, {0x2F, 0x57},
		3, {0x8F, 0xB7, 0xB6}
	},
	{
		2, {0x88, 0x64},
		3, {0xE5, 0x9C, 0xB3},
		2, {0x33, 0x57},
		3, {0x8F, 0xB7, 0xB7}
	},
	{
		2, {0x88, 0x65},
		3, {0xE5, 0x9C, 0xB4},
		2, {0x34, 0x57},
		3, {0x8F, 0xB7, 0xB8}
	},
	{
		2, {0x88, 0x66},
		3, {0xE5, 0x9D, 0xB0},
		2, {0x70, 0x57},
		3, {0x8F, 0xB7, 0xC9}
	},
	{
		2, {0x88, 0x67},
		3, {0xE5, 0x9D, 0xB7},
		2, {0x77, 0x57},
		3, {0x8F, 0xB7, 0xCE}
	},
	{
		2, {0x88, 0x68},
		3, {0xE5, 0x9D, 0xBC},
		2, {0x7C, 0x57},
		3, {0x8F, 0xB7, 0xD2}
	},
	{
		2, {0x88, 0x69},
		3, {0xE5, 0x9E, 0x9C},
		2, {0x9C, 0x57},
		3, {0x8F, 0xB7, 0xDB}
	},
	{
		2, {0x88, 0x6A},
		3, {0xEF, 0xA8, 0x8F},
		2, {0x0F, 0xFA},
		0, {0x00}
	},
	{
		2, {0x88, 0x6B},
		4, {0xF0, 0xA1, 0x8C, 0x9B},
		4, {0x44, 0xD8, 0x1B, 0xDF},
		0, {0x00}
	},
	{
		2, {0x88, 0x6C},
		3, {0xE5, 0x9E, 0xB8},
		2, {0xB8, 0x57},
		3, {0x8F, 0xB7, 0xE5}
	},
	{
		2, {0x88, 0x6D},
		3, {0xE5, 0x9F, 0x87},
		2, {0xC7, 0x57},
		3, {0x8F, 0xB7, 0xE7}
	},
	{
		2, {0x88, 0x6E},
		3, {0xE5, 0x9F, 0x88},
		2, {0xC8, 0x57},
		3, {0x8F, 0xB7, 0xE8}
	},
	{
		2, {0x88, 0x6F},
		3, {0xE5, 0x9F, 0x8F},
		2, {0xCF, 0x57},
		3, {0x8F, 0xB7, 0xEA}
	},
	{
		2, {0x88, 0x70},
		3, {0xE5, 0x9F, 0xA4},
		2, {0xE4, 0x57},
		3, {0x8F, 0xB7, 0xEE}
	},
	{
		2, {0x88, 0x71},
		3, {0xE5, 0x9F, 0xAD},
		2, {0xED, 0x57},
		3, {0x8F, 0xB7, 0xF2}
	},
	{
		2, {0x88, 0x72},
		3, {0xE5, 0x9F, 0xB5},
		2, {0xF5, 0x57},
		3, {0x8F, 0xB7, 0xF4}
	},
	{
		2, {0x88, 0x73},
		3, {0xE5, 0x9F, 0xB6},
		2, {0xF6, 0x57},
		3, {0x8F, 0xB7, 0xF5}
	},
	{
		2, {0x88, 0x74},
		3, {0xE5, 0x9F, 0xBF},
		2, {0xFF, 0x57},
		3, {0x8F, 0xB7, 0xF9}
	},
	{
		2, {0x88, 0x75},
		3, {0xE5, 0xA0, 0x89},
		2, {0x09, 0x58},
		3, {0x8F, 0xB7, 0xFD}
	},
	{
		2, {0x88, 0x76},
		3, {0xEF, 0xA8, 0x90},
		2, {0x10, 0xFA},
		0, {0x00}
	},
	{
		2, {0x88, 0x77},
		3, {0xE5, 0xA1, 0xA1},
		2, {0x61, 0x58},
		3, {0x8F, 0xB8, 0xB4}
	},
	{
		2, {0x88, 0x78},
		3, {0xE5, 0xA1, 0xA4},
		2, {0x64, 0x58},
		3, {0x8F, 0xB8, 0xB5}
	},
	{
		2, {0x88, 0x79},
		3, {0xEF, 0xA8, 0xB9},
		2, {0x39, 0xFA},
		0, {0x00}
	},
	{
		2, {0x88, 0x7A},
		3, {0xE5, 0xA1, 0xBC},
		2, {0x7C, 0x58},
		3, {0x8F, 0xB8, 0xB9}
	},
	{
		2, {0x88, 0x7B},
		3, {0xE5, 0xA2, 0x89},
		2, {0x89, 0x58},
		3, {0x8F, 0xB8, 0xBF}
	},
	{
		2, {0x88, 0x7C},
		3, {0xE5, 0xA2, 0x9E},
		2, {0x9E, 0x58},
		0, {0x00}
	},
	{
		2, {0x88, 0x7D},
		3, {0xEF, 0xA8, 0xBA},
		2, {0x3A, 0xFA},
		0, {0x00}
	},
	{
		2, {0x88, 0x7E},
		3, {0xE5, 0xA2, 0xA9},
		2, {0xA9, 0x58},
		3, {0x8F, 0xB8, 0xCC}
	},
	{
		2, {0x88, 0x80},
		4, {0xF0, 0xA1, 0x91, 0xAE},
		4, {0x45, 0xD8, 0x6E, 0xDC},
		0, {0x00}
	},
	{
		2, {0x88, 0x81},
		3, {0xE5, 0xA3, 0x92},
		2, {0xD2, 0x58},
		3, {0x8F, 0xB8, 0xD6}
	},
	{
		2, {0x88, 0x82},
		3, {0xE5, 0xA3, 0x8E},
		2, {0xCE, 0x58},
		3, {0x8F, 0xB8, 0xD4}
	},
	{
		2, {0x88, 0x83},
		3, {0xE5, 0xA3, 0x94},
		2, {0xD4, 0x58},
		3, {0x8F, 0xB8, 0xD7}
	},
	{
		2, {0x88, 0x84},
		3, {0xE5, 0xA3, 0x9A},
		2, {0xDA, 0x58},
		3, {0x8F, 0xB8, 0xD9}
	},
	{
		2, {0x88, 0x85},
		3, {0xE5, 0xA3, 0xA0},
		2, {0xE0, 0x58},
		0, {0x00}
	},
	{
		2, {0x88, 0x86},
		3, {0xE5, 0xA3, 0xA9},
		2, {0xE9, 0x58},
		3, {0x8F, 0xB8, 0xDD}
	},
	{
		2, {0x88, 0x87},
		3, {0xE5, 0xA4, 0x8C},
		2, {0x0C, 0x59},
		3, {0x8F, 0xB8, 0xE2}
	},
	{
		2, {0x88, 0x88},
		3, {0xE8, 0x99, 0x81},
		2, {0x41, 0x86},
		3, {0x8F, 0xB8, 0xE6}
	},
	{
		2, {0x88, 0x89},
		3, {0xE5, 0xA5, 0x9D},
		2, {0x5D, 0x59},
		3, {0x8F, 0xB8, 0xF8}
	},
	{
		2, {0x88, 0x8A},
		3, {0xE5, 0xA5, 0xAD},
		2, {0x6D, 0x59},
		3, {0x8F, 0xB8, 0xFE}
	},
	{
		2, {0x88, 0x8B},
		3, {0xE5, 0xA6, 0x8B},
		2, {0x8B, 0x59},
		3, {0x8F, 0xB9, 0xA8}
	},
	{
		2, {0x88, 0x8C},
		3, {0xE5, 0xA6, 0x92},
		2, {0x92, 0x59},
		3, {0x8F, 0xB9, 0xAB}
	},
	{
		2, {0x88, 0x8D},
		3, {0xE5, 0xA6, 0xA4},
		2, {0xA4, 0x59},
		3, {0x8F, 0xB9, 0xAF}
	},
	{
		2, {0x88, 0x8E},
		3, {0xE5, 0xA7, 0x83},
		2, {0xC3, 0x59},
		3, {0x8F, 0xB9, 0xBA}
	},
	{
		2, {0x88, 0x8F},
		3, {0xE5, 0xA7, 0x92},
		2, {0xD2, 0x59},
		3, {0x8F, 0xB9, 0xBF}
	},
	{
		2, {0x88, 0x90},
		3, {0xE5, 0xA7, 0x9D},
		2, {0xDD, 0x59},
		3, {0x8F, 0xB9, 0xC0}
	},
	{
		2, {0x88, 0x91},
		3, {0xE5, 0xA8, 0x93},
		2, {0x13, 0x5A},
		3, {0x8F, 0xB9, 0xD2}
	},
	{
		2, {0x88, 0x92},
		3, {0xE5, 0xA8, 0xA3},
		2, {0x23, 0x5A},
		3, {0x8F, 0xB9, 0xD4}
	},
	{
		2, {0x88, 0x93},
		3, {0xE5, 0xA9, 0xA7},
		2, {0x67, 0x5A},
		3, {0x8F, 0xB9, 0xE5}
	},
	{
		2, {0x88, 0x94},
		3, {0xE5, 0xA9, 0xAD},
		2, {0x6D, 0x5A},
		3, {0x8F, 0xB9, 0xE6}
	},
	{
		2, {0x88, 0x95},
		3, {0xE5, 0xA9, 0xB7},
		2, {0x77, 0x5A},
		3, {0x8F, 0xB9, 0xE7}
	},
	{
		2, {0x88, 0x96},
		3, {0xE5, 0xA9, 0xBE},
		2, {0x7E, 0x5A},
		3, {0x8F, 0xB9, 0xEA}
	},
	{
		2, {0x88, 0x97},
		3, {0xE5, 0xAA, 0x84},
		2, {0x84, 0x5A},
		0, {0x00}
	},
	{
		2, {0x88, 0x98},
		3, {0xE5, 0xAA, 0x9E},
		2, {0x9E, 0x5A},
		3, {0x8F, 0xB9, 0xF1}
	},
	{
		2, {0x88, 0x99},
		3, {0xE5, 0xAA, 0xA7},
		2, {0xA7, 0x5A},
		3, {0x8F, 0xB9, 0xF5}
	},
	{
		2, {0x88, 0x9A},
		3, {0xE5, 0xAB, 0x84},
		2, {0xC4, 0x5A},
		3, {0x8F, 0xBA, 0xA1}
	},
	{
		2, {0x88, 0x9B},
		4, {0xF0, 0xA1, 0xA2, 0xBD},
		4, {0x46, 0xD8, 0xBD, 0xDC},
		0, {0x00}
	},
	{
		2, {0x88, 0x9C},
		3, {0xE5, 0xAC, 0x99},
		2, {0x19, 0x5B},
		3, {0x8F, 0xBA, 0xB3}
	},
	{
		2, {0x88, 0x9D},
		3, {0xE5, 0xAC, 0xA5},
		2, {0x25, 0x5B},
		3, {0x8F, 0xBA, 0xB7}
	},
	{
		2, {0x88, 0x9E},
		3, {0xE5, 0x89, 0x9D},
		2, {0x5D, 0x52},
		0, {0x00}
	},
	{
		2, {0x88, 0x9F},
		3, {0xE4, 0xBA, 0x9C},
		2, {0x9C, 0x4E},
		2, {0xB0, 0xA1}
	},
	{
		2, {0x88, 0xA0},
		3, {0xE5, 0x94, 0x96},
		2, {0x16, 0x55},
		2, {0xB0, 0xA2}
	},
	{
		2, {0x88, 0xA1},
		3, {0xE5, 0xA8, 0x83},
		2, {0x03, 0x5A},
		2, {0xB0, 0xA3}
	},
	{
		2, {0x88, 0xA2},
		3, {0xE9, 0x98, 0xBF},
		2, {0x3F, 0x96},
		2, {0xB0, 0xA4}
	},
	{
		2, {0x88, 0xA3},
		3, {0xE5, 0x93, 0x80},
		2, {0xC0, 0x54},
		2, {0xB0, 0xA5}
	},
	{
		2, {0x88, 0xA4},
		3, {0xE6, 0x84, 0x9B},
		2, {0x1B, 0x61},
		2, {0xB0, 0xA6}
	},
	{
		2, {0x88, 0xA5},
		3, {0xE6, 0x8C, 0xA8},
		2, {0x28, 0x63},
		2, {0xB0, 0xA7}
	},
	{
		2, {0x88, 0xA6},
		3, {0xE5, 0xA7, 0xB6},
		2, {0xF6, 0x59},
		2, {0xB0, 0xA8}
	},
	{
		2, {0x88, 0xA7},
		3, {0xE9, 0x80, 0xA2},
		2, {0x22, 0x90},
		2, {0xB0, 0xA9}
	},
	{
		2, {0x88, 0xA8},
		3, {0xE8, 0x91, 0xB5},
		2, {0x75, 0x84},
		2, {0xB0, 0xAA}
	},
	{
		2, {0x88, 0xA9},
		3, {0xE8, 0x8C, 0x9C},
		2, {0x1C, 0x83},
		2, {0xB0, 0xAB}
	},
	{
		2, {0x88, 0xAA},
		3, {0xE7, 0xA9, 0x90},
		2, {0x50, 0x7A},
		2, {0xB0, 0xAC}
	},
	{
		2, {0x88, 0xAB},
		3, {0xE6, 0x82, 0xAA},
		2, {0xAA, 0x60},
		2, {0xB0, 0xAD}
	},
	{
		2, {0x88, 0xAC},
		3, {0xE6, 0x8F, 0xA1},
		2, {0xE1, 0x63},
		2, {0xB0, 0xAE}
	},
	{
		2, {0x88, 0xAD},
		3, {0xE6, 0xB8, 0xA5},
		2, {0x25, 0x6E},
		2, {0xB0, 0xAF}
	},
	{
		2, {0x88, 0xAE},
		3, {0xE6, 0x97, 0xAD},
		2, {0xED, 0x65},
		2, {0xB0, 0xB0}
	},
	{
		2, {0x88, 0xAF},
		3, {0xE8, 0x91, 0xA6},
		2, {0x66, 0x84},
		2, {0xB0, 0xB1}
	},
	{
		2, {0x88, 0xB0},
		3, {0xE8, 0x8A, 0xA6},
		2, {0xA6, 0x82},
		2, {0xB0, 0xB2}
	},
	{
		2, {0x88, 0xB1},
		3, {0xE9, 0xAF, 0xB5},
		2, {0xF5, 0x9B},
		2, {0xB0, 0xB3}
	},
	{
		2, {0x88, 0xB2},
		3, {0xE6, 0xA2, 0x93},
		2, {0x93, 0x68},
		2, {0xB0, 0xB4}
	},
	{
		2, {0x88, 0xB3},
		3, {0xE5, 0x9C, 0xA7},
		2, {0x27, 0x57},
		2, {0xB0, 0xB5}
	},
	{
		2, {0x88, 0xB4},
		3, {0xE6, 0x96, 0xA1},
		2, {0xA1, 0x65},
		2, {0xB0, 0xB6}
	},
	{
		2, {0x88, 0xB5},
		3, {0xE6, 0x89, 0xB1},
		2, {0x71, 0x62},
		2, {0xB0, 0xB7}
	},
	{
		2, {0x88, 0xB6},
		3, {0xE5, 0xAE, 0x9B},
		2, {0x9B, 0x5B},
		2, {0xB0, 0xB8}
	},
	{
		2, {0x88, 0xB7},
		3, {0xE5, 0xA7, 0x90},
		2, {0xD0, 0x59},
		2, {0xB0, 0xB9}
	},
	{
		2, {0x88, 0xB8},
		3, {0xE8, 0x99, 0xBB},
		2, {0x7B, 0x86},
		2, {0xB0, 0xBA}
	},
	{
		2, {0x88, 0xB9},
		3, {0xE9, 0xA3, 0xB4},
		2, {0xF4, 0x98},
		2, {0xB0, 0xBB}
	},
	{
		2, {0x88, 0xBA},
		3, {0xE7, 0xB5, 0xA2},
		2, {0x62, 0x7D},
		2, {0xB0, 0xBC}
	},
	{
		2, {0x88, 0xBB},
		3, {0xE7, 0xB6, 0xBE},
		2, {0xBE, 0x7D},
		2, {0xB0, 0xBD}
	},
	{
		2, {0x88, 0xBC},
		3, {0xE9, 0xAE, 0x8E},
		2, {0x8E, 0x9B},
		2, {0xB0, 0xBE}
	},
	{
		2, {0x88, 0xBD},
		3, {0xE6, 0x88, 0x96},
		2, {0x16, 0x62},
		2, {0xB0, 0xBF}
	},
	{
		2, {0x88, 0xBE},
		3, {0xE7, 0xB2, 0x9F},
		2, {0x9F, 0x7C},
		2, {0xB0, 0xC0}
	},
	{
		2, {0x88, 0xBF},
		3, {0xE8, 0xA2, 0xB7},
		2, {0xB7, 0x88},
		2, {0xB0, 0xC1}
	},
	{
		2, {0x88, 0xC0},
		3, {0xE5, 0xAE, 0x89},
		2, {0x89, 0x5B},
		2, {0xB0, 0xC2}
	},
	{
		2, {0x88, 0xC1},
		3, {0xE5, 0xBA, 0xB5},
		2, {0xB5, 0x5E},
		2, {0xB0, 0xC3}
	},
	{
		2, {0x88, 0xC2},
		3, {0xE6, 0x8C, 0x89},
		2, {0x09, 0x63},
		2, {0xB0, 0xC4}
	},
	{
		2, {0x88, 0xC3},
		3, {0xE6, 0x9A, 0x97},
		2, {0x97, 0x66},
		2, {0xB0, 0xC5}
	},
	{
		2, {0x88, 0xC4},
		3, {0xE6, 0xA1, 0x88},
		2, {0x48, 0x68},
		2, {0xB0, 0xC6}
	},
	{
		2, {0x88, 0xC5},
		3, {0xE9, 0x97, 0x87},
		2, {0xC7, 0x95},
		2, {0xB0, 0xC7}
	},
	{
		2, {0x88, 0xC6},
		3, {0xE9, 0x9E, 0x8D},
		2, {0x8D, 0x97},
		2, {0xB0, 0xC8}
	},
	{
		2, {0x88, 0xC7},
		3, {0xE6, 0x9D, 0x8F},
		2, {0x4F, 0x67},
		2, {0xB0, 0xC9}
	},
	{
		2, {0x88, 0xC8},
		3, {0xE4, 0xBB, 0xA5},
		2, {0xE5, 0x4E},
		2, {0xB0, 0xCA}
	},
	{
		2, {0x88, 0xC9},
		3, {0xE4, 0xBC, 0x8A},
		2, {0x0A, 0x4F},
		2, {0xB0, 0xCB}
	},
	{
		2, {0x88, 0xCA},
		3, {0xE4, 0xBD, 0x8D},
		2, {0x4D, 0x4F},
		2, {0xB0, 0xCC}
	},
	{
		2, {0x88, 0xCB},
		3, {0xE4, 0xBE, 0x9D},
		2, {0x9D, 0x4F},
		2, {0xB0, 0xCD}
	},
	{
		2, {0x88, 0xCC},
		3, {0xE5, 0x81, 0x89},
		2, {0x49, 0x50},
		2, {0xB0, 0xCE}
	},
	{
		2, {0x88, 0xCD},
		3, {0xE5, 0x9B, 0xB2},
		2, {0xF2, 0x56},
		2, {0xB0, 0xCF}
	},
	{
		2, {0x88, 0xCE},
		3, {0xE5, 0xA4, 0xB7},
		2, {0x37, 0x59},
		2, {0xB0, 0xD0}
	},
	{
		2, {0x88, 0xCF},
		3, {0xE5, 0xA7, 0x94},
		2, {0xD4, 0x59},
		2, {0xB0, 0xD1}
	},
	{
		2, {0x88, 0xD0},
		3, {0xE5, 0xA8, 0x81},
		2, {0x01, 0x5A},
		2, {0xB0, 0xD2}
	},
	{
		2, {0x88, 0xD1},
		3, {0xE5, 0xB0, 0x89},
		2, {0x09, 0x5C},
		2, {0xB0, 0xD3}
	},
	{
		2, {0x88, 0xD2},
		3, {0xE6, 0x83, 0x9F},
		2, {0xDF, 0x60},
		2, {0xB0, 0xD4}
	},
	{
		2, {0x88, 0xD3},
		3, {0xE6, 0x84, 0x8F},
		2, {0x0F, 0x61},
		2, {0xB0, 0xD5}
	},
	{
		2, {0x88, 0xD4},
		3, {0xE6, 0x85, 0xB0},
		2, {0x70, 0x61},
		2, {0xB0, 0xD6}
	},
	{
		2, {0x88, 0xD5},
		3, {0xE6, 0x98, 0x93},
		2, {0x13, 0x66},
		2, {0xB0, 0xD7}
	},
	{
		2, {0x88, 0xD6},
		3, {0xE6, 0xA4, 0x85},
		2, {0x05, 0x69},
		2, {0xB0, 0xD8}
	},
	{
		2, {0x88, 0xD7},
		3, {0xE7, 0x82, 0xBA},
		2, {0xBA, 0x70},
		2, {0xB0, 0xD9}
	},
	{
		2, {0x88, 0xD8},
		3, {0xE7, 0x95, 0x8F},
		2, {0x4F, 0x75},
		2, {0xB0, 0xDA}
	},
	{
		2, {0x88, 0xD9},
		3, {0xE7, 0x95, 0xB0},
		2, {0x70, 0x75},
		2, {0xB0, 0xDB}
	},
	{
		2, {0x88, 0xDA},
		3, {0xE7, 0xA7, 0xBB},
		2, {0xFB, 0x79},
		2, {0xB0, 0xDC}
	},
	{
		2, {0x88, 0xDB},
		3, {0xE7, 0xB6, 0xAD},
		2, {0xAD, 0x7D},
		2, {0xB0, 0xDD}
	},
	{
		2, {0x88, 0xDC},
		3, {0xE7, 0xB7, 0xAF},
		2, {0xEF, 0x7D},
		2, {0xB0, 0xDE}
	},
	{
		2, {0x88, 0xDD},
		3, {0xE8, 0x83, 0x83},
		2, {0xC3, 0x80},
		2, {0xB0, 0xDF}
	},
	{
		2, {0x88, 0xDE},
		3, {0xE8, 0x90, 0x8E},
		2, {0x0E, 0x84},
		2, {0xB0, 0xE0}
	},
	{
		2, {0x88, 0xDF},
		3, {0xE8, 0xA1, 0xA3},
		2, {0x63, 0x88},
		2, {0xB0, 0xE1}
	},
	{
		2, {0x88, 0xE0},
		3, {0xE8, 0xAC, 0x82},
		2, {0x02, 0x8B},
		2, {0xB0, 0xE2}
	},
	{
		2, {0x88, 0xE1},
		3, {0xE9, 0x81, 0x95},
		2, {0x55, 0x90},
		2, {0xB0, 0xE3}
	},
	{
		2, {0x88, 0xE2},
		3, {0xE9, 0x81, 0xBA},
		2, {0x7A, 0x90},
		2, {0xB0, 0xE4}
	},
	{
		2, {0x88, 0xE3},
		3, {0xE5, 0x8C, 0xBB},
		2, {0x3B, 0x53},
		2, {0xB0, 0xE5}
	},
	{
		2, {0x88, 0xE4},
		3, {0xE4, 0xBA, 0x95},
		2, {0x95, 0x4E},
		2, {0xB0, 0xE6}
	},
	{
		2, {0x88, 0xE5},
		3, {0xE4, 0xBA, 0xA5},
		2, {0xA5, 0x4E},
		2, {0xB0, 0xE7}
	},
	{
		2, {0x88, 0xE6},
		3, {0xE5, 0x9F, 0x9F},
		2, {0xDF, 0x57},
		2, {0xB0, 0xE8}
	},
	{
		2, {0x88, 0xE7},
		3, {0xE8, 0x82, 0xB2},
		2, {0xB2, 0x80},
		2, {0xB0, 0xE9}
	},
	{
		2, {0x88, 0xE8},
		3, {0xE9, 0x83, 0x81},
		2, {0xC1, 0x90},
		2, {0xB0, 0xEA}
	},
	{
		2, {0x88, 0xE9},
		3, {0xE7, 0xA3, 0xAF},
		2, {0xEF, 0x78},
		2, {0xB0, 0xEB}
	},
	{
		2, {0x88, 0xEA},
		3, {0xE4, 0xB8, 0x80},
		2, {0x00, 0x4E},
		2, {0xB0, 0xEC}
	},
	{
		2, {0x88, 0xEB},
		3, {0xE5, 0xA3, 0xB1},
		2, {0xF1, 0x58},
		2, {0xB0, 0xED}
	},
	{
		2, {0x88, 0xEC},
		3, {0xE6, 0xBA, 0xA2},
		2, {0xA2, 0x6E},
		2, {0xB0, 0xEE}
	},
	{
		2, {0x88, 0xED},
		3, {0xE9, 0x80, 0xB8},
		2, {0x38, 0x90},
		2, {0xB0, 0xEF}
	},
	{
		2, {0x88, 0xEE},
		3, {0xE7, 0xA8, 0xB2},
		2, {0x32, 0x7A},
		2, {0xB0, 0xF0}
	},
	{
		2, {0x88, 0xEF},
		3, {0xE8, 0x8C, 0xA8},
		2, {0x28, 0x83},
		2, {0xB0, 0xF1}
	},
	{
		2, {0x88, 0xF0},
		3, {0xE8, 0x8A, 0x8B},
		2, {0x8B, 0x82},
		2, {0xB0, 0xF2}
	},
	{
		2, {0x88, 0xF1},
		3, {0xE9, 0xB0, 0xAF},
		2, {0x2F, 0x9C},
		2, {0xB0, 0xF3}
	},
	{
		2, {0x88, 0xF2},
		3, {0xE5, 0x85, 0x81},
		2, {0x41, 0x51},
		2, {0xB0, 0xF4}
	},
	{
		2, {0x88, 0xF3},
		3, {0xE5, 0x8D, 0xB0},
		2, {0x70, 0x53},
		2, {0xB0, 0xF5}
	},
	{
		2, {0x88, 0xF4},
		3, {0xE5, 0x92, 0xBD},
		2, {0xBD, 0x54},
		2, {0xB0, 0xF6}
	},
	{
		2, {0x88, 0xF5},
		3, {0xE5, 0x93, 0xA1},
		2, {0xE1, 0x54},
		2, {0xB0, 0xF7}
	},
	{
		2, {0x88, 0xF6},
		3, {0xE5, 0x9B, 0xA0},
		2, {0xE0, 0x56},
		2, {0xB0, 0xF8}
	},
	{
		2, {0x88, 0xF7},
		3, {0xE5, 0xA7, 0xBB},
		2, {0xFB, 0x59},
		2, {0xB0, 0xF9}
	},
	{
		2, {0x88, 0xF8},
		3, {0xE5, 0xBC, 0x95},
		2, {0x15, 0x5F},
		2, {0xB0, 0xFA}
	},
	{
		2, {0x88, 0xF9},
		3, {0xE9, 0xA3, 0xB2},
		2, {0xF2, 0x98},
		2, {0xB0, 0xFB}
	},
	{
		2, {0x88, 0xFA},
		3, {0xE6, 0xB7, 0xAB},
		2, {0xEB, 0x6D},
		2, {0xB0, 0xFC}
	},
	{
		2, {0x88, 0xFB},
		3, {0xE8, 0x83, 0xA4},
		2, {0xE4, 0x80},
		2, {0xB0, 0xFD}
	},
	{
		2, {0x88, 0xFC},
		3, {0xE8, 0x94, 0xAD},
		2, {0x2D, 0x85},
		2, {0xB0, 0xFE}
	},
	{
		2, {0x89, 0x40},
		3, {0xE9, 0x99, 0xA2},
		2, {0x62, 0x96},
		2, {0xB1, 0xA1}
	},
	{
		2, {0x89, 0x41},
		3, {0xE9, 0x99, 0xB0},
		2, {0x70, 0x96},
		2, {0xB1, 0xA2}
	},
	{
		2, {0x89, 0x42},
		3, {0xE9, 0x9A, 0xA0},
		2, {0xA0, 0x96},
		2, {0xB1, 0xA3}
	},
	{
		2, {0x89, 0x43},
		3, {0xE9, 0x9F, 0xBB},
		2, {0xFB, 0x97},
		2, {0xB1, 0xA4}
	},
	{
		2, {0x89, 0x44},
		3, {0xE5, 0x90, 0x8B},
		2, {0x0B, 0x54},
		2, {0xB1, 0xA5}
	},
	{
		2, {0x89, 0x45},
		3, {0xE5, 0x8F, 0xB3},
		2, {0xF3, 0x53},
		2, {0xB1, 0xA6}
	},
	{
		2, {0x89, 0x46},
		3, {0xE5, 0xAE, 0x87},
		2, {0x87, 0x5B},
		2, {0xB1, 0xA7}
	},
	{
		2, {0x89, 0x47},
		3, {0xE7, 0x83, 0x8F},
		2, {0xCF, 0x70},
		2, {0xB1, 0xA8}
	},
	{
		2, {0x89, 0x48},
		3, {0xE7, 0xBE, 0xBD},
		2, {0xBD, 0x7F},
		2, {0xB1, 0xA9}
	},
	{
		2, {0x89, 0x49},
		3, {0xE8, 0xBF, 0x82},
		2, {0xC2, 0x8F},
		2, {0xB1, 0xAA}
	},
	{
		2, {0x89, 0x4A},
		3, {0xE9, 0x9B, 0xA8},
		2, {0xE8, 0x96},
		2, {0xB1, 0xAB}
	},
	{
		2, {0x89, 0x4B},
		3, {0xE5, 0x8D, 0xAF},
		2, {0x6F, 0x53},
		2, {0xB1, 0xAC}
	},
	{
		2, {0x89, 0x4C},
		3, {0xE9, 0xB5, 0x9C},
		2, {0x5C, 0x9D},
		2, {0xB1, 0xAD}
	},
	{
		2, {0x89, 0x4D},
		3, {0xE7, 0xAA, 0xBA},
		2, {0xBA, 0x7A},
		2, {0xB1, 0xAE}
	},
	{
		2, {0x89, 0x4E},
		3, {0xE4, 0xB8, 0x91},
		2, {0x11, 0x4E},
		2, {0xB1, 0xAF}
	},
	{
		2, {0x89, 0x4F},
		3, {0xE7, 0xA2, 0x93},
		2, {0x93, 0x78},
		2, {0xB1, 0xB0}
	},
	{
		2, {0x89, 0x50},
		3, {0xE8, 0x87, 0xBC},
		2, {0xFC, 0x81},
		2, {0xB1, 0xB1}
	},
	{
		2, {0x89, 0x51},
		3, {0xE6, 0xB8, 0xA6},
		2, {0x26, 0x6E},
		2, {0xB1, 0xB2}
	},
	{
		2, {0x89, 0x52},
		3, {0xE5, 0x98, 0x98},
		2, {0x18, 0x56},
		2, {0xB1, 0xB3}
	},
	{
		2, {0x89, 0x53},
		3, {0xE5, 0x94, 0x84},
		2, {0x04, 0x55},
		2, {0xB1, 0xB4}
	},
	{
		2, {0x89, 0x54},
		3, {0xE6, 0xAC, 0x9D},
		2, {0x1D, 0x6B},
		2, {0xB1, 0xB5}
	},
	{
		2, {0x89, 0x55},
		3, {0xE8, 0x94, 0x9A},
		2, {0x1A, 0x85},
		2, {0xB1, 0xB6}
	},
	{
		2, {0x89, 0x56},
		3, {0xE9, 0xB0, 0xBB},
		2, {0x3B, 0x9C},
		2, {0xB1, 0xB7}
	},
	{
		2, {0x89, 0x57},
		3, {0xE5, 0xA7, 0xA5},
		2, {0xE5, 0x59},
		2, {0xB1, 0xB8}
	},
	{
		2, {0x89, 0x58},
		3, {0xE5, 0x8E, 0xA9},
		2, {0xA9, 0x53},
		2, {0xB1, 0xB9}
	},
	{
		2, {0x89, 0x59},
		3, {0xE6, 0xB5, 0xA6},
		2, {0x66, 0x6D},
		2, {0xB1, 0xBA}
	},
	{
		2, {0x89, 0x5A},
		3, {0xE7, 0x93, 0x9C},
		2, {0xDC, 0x74},
		2, {0xB1, 0xBB}
	},
	{
		2, {0x89, 0x5B},
		3, {0xE9, 0x96, 0x8F},
		2, {0x8F, 0x95},
		2, {0xB1, 0xBC}
	},
	{
		2, {0x89, 0x5C},
		3, {0xE5, 0x99, 0x82},
		2, {0x42, 0x56},
		2, {0xB1, 0xBD}
	},
	{
		2, {0x89, 0x5D},
		3, {0xE4, 0xBA, 0x91},
		2, {0x91, 0x4E},
		2, {0xB1, 0xBE}
	},
	{
		2, {0x89, 0x5E},
		3, {0xE9, 0x81, 0x8B},
		2, {0x4B, 0x90},
		2, {0xB1, 0xBF}
	},
	{
		2, {0x89, 0x5F},
		3, {0xE9, 0x9B, 0xB2},
		2, {0xF2, 0x96},
		2, {0xB1, 0xC0}
	},
	{
		2, {0x89, 0x60},
		3, {0xE8, 0x8D, 0x8F},
		2, {0x4F, 0x83},
		2, {0xB1, 0xC1}
	},
	{
		2, {0x89, 0x61},
		3, {0xE9, 0xA4, 0x8C},
		2, {0x0C, 0x99},
		2, {0xB1, 0xC2}
	},
	{
		2, {0x89, 0x62},
		3, {0xE5, 0x8F, 0xA1},
		2, {0xE1, 0x53},
		2, {0xB1, 0xC3}
	},
	{
		2, {0x89, 0x63},
		3, {0xE5, 0x96, 0xB6},
		2, {0xB6, 0x55},
		2, {0xB1, 0xC4}
	},
	{
		2, {0x89, 0x64},
		3, {0xE5, 0xAC, 0xB0},
		2, {0x30, 0x5B},
		2, {0xB1, 0xC5}
	},
	{
		2, {0x89, 0x65},
		3, {0xE5, 0xBD, 0xB1},
		2, {0x71, 0x5F},
		2, {0xB1, 0xC6}
	},
	{
		2, {0x89, 0x66},
		3, {0xE6, 0x98, 0xA0},
		2, {0x20, 0x66},
		2, {0xB1, 0xC7}
	},
	{
		2, {0x89, 0x67},
		3, {0xE6, 0x9B, 0xB3},
		2, {0xF3, 0x66},
		2, {0xB1, 0xC8}
	},
	{
		2, {0x89, 0x68},
		3, {0xE6, 0xA0, 0x84},
		2, {0x04, 0x68},
		2, {0xB1, 0xC9}
	},
	{
		2, {0x89, 0x69},
		3, {0xE6, 0xB0, 0xB8},
		2, {0x38, 0x6C},
		2, {0xB1, 0xCA}
	},
	{
		2, {0x89, 0x6A},
		3, {0xE6, 0xB3, 0xB3},
		2, {0xF3, 0x6C},
		2, {0xB1, 0xCB}
	},
	{
		2, {0x89, 0x6B},
		3, {0xE6, 0xB4, 0xA9},
		2, {0x29, 0x6D},
		2, {0xB1, 0xCC}
	},
	{
		2, {0x89, 0x6C},
		3, {0xE7, 0x91, 0x9B},
		2, {0x5B, 0x74},
		2, {0xB1, 0xCD}
	},
	{
		2, {0x89, 0x6D},
		3, {0xE7, 0x9B, 0x88},
		2, {0xC8, 0x76},
		2, {0xB1, 0xCE}
	},
	{
		2, {0x89, 0x6E},
		3, {0xE7, 0xA9, 0x8E},
		2, {0x4E, 0x7A},
		2, {0xB1, 0xCF}
	},
	{
		2, {0x89, 0x6F},
		3, {0xE9, 0xA0, 0xB4},
		2, {0x34, 0x98},
		2, {0xB1, 0xD0}
	},
	{
		2, {0x89, 0x70},
		3, {0xE8, 0x8B, 0xB1},
		2, {0xF1, 0x82},
		2, {0xB1, 0xD1}
	},
	{
		2, {0x89, 0x71},
		3, {0xE8, 0xA1, 0x9B},
		2, {0x5B, 0x88},
		2, {0xB1, 0xD2}
	},
	{
		2, {0x89, 0x72},
		3, {0xE8, 0xA9, 0xA0},
		2, {0x60, 0x8A},
		2, {0xB1, 0xD3}
	},
	{
		2, {0x89, 0x73},
		3, {0xE9, 0x8B, 0xAD},
		2, {0xED, 0x92},
		2, {0xB1, 0xD4}
	},
	{
		2, {0x89, 0x74},
		3, {0xE6, 0xB6, 0xB2},
		2, {0xB2, 0x6D},
		2, {0xB1, 0xD5}
	},
	{
		2, {0x89, 0x75},
		3, {0xE7, 0x96, 0xAB},
		2, {0xAB, 0x75},
		2, {0xB1, 0xD6}
	},
	{
		2, {0x89, 0x76},
		3, {0xE7, 0x9B, 0x8A},
		2, {0xCA, 0x76},
		2, {0xB1, 0xD7}
	},
	{
		2, {0x89, 0x77},
		3, {0xE9, 0xA7, 0x85},
		2, {0xC5, 0x99},
		2, {0xB1, 0xD8}
	},
	{
		2, {0x89, 0x78},
		3, {0xE6, 0x82, 0xA6},
		2, {0xA6, 0x60},
		2, {0xB1, 0xD9}
	},
	{
		2, {0x89, 0x79},
		3, {0xE8, 0xAC, 0x81},
		2, {0x01, 0x8B},
		2, {0xB1, 0xDA}
	},
	{
		2, {0x89, 0x7A},
		3, {0xE8, 0xB6, 0x8A},
		2, {0x8A, 0x8D},
		2, {0xB1, 0xDB}
	},
	{
		2, {0x89, 0x7B},
		3, {0xE9, 0x96, 0xB2},
		2, {0xB2, 0x95},
		2, {0xB1, 0xDC}
	},
	{
		2, {0x89, 0x7C},
		3, {0xE6, 0xA6, 0x8E},
		2, {0x8E, 0x69},
		2, {0xB1, 0xDD}
	},
	{
		2, {0x89, 0x7D},
		3, {0xE5, 0x8E, 0xAD},
		2, {0xAD, 0x53},
		2, {0xB1, 0xDE}
	},
	{
		2, {0x89, 0x7E},
		3, {0xE5, 0x86, 0x86},
		2, {0x86, 0x51},
		2, {0xB1, 0xDF}
	},
	{
		2, {0x89, 0x80},
		3, {0xE5, 0x9C, 0x92},
		2, {0x12, 0x57},
		2, {0xB1, 0xE0}
	},
	{
		2, {0x89, 0x81},
		3, {0xE5, 0xA0, 0xB0},
		2, {0x30, 0x58},
		2, {0xB1, 0xE1}
	},
	{
		2, {0x89, 0x82},
		3, {0xE5, 0xA5, 0x84},
		2, {0x44, 0x59},
		2, {0xB1, 0xE2}
	},
	{
		2, {0x89, 0x83},
		3, {0xE5, 0xAE, 0xB4},
		2, {0xB4, 0x5B},
		2, {0xB1, 0xE3}
	},
	{
		2, {0x89, 0x84},
		3, {0xE5, 0xBB, 0xB6},
		2, {0xF6, 0x5E},
		2, {0xB1, 0xE4}
	},
	{
		2, {0x89, 0x85},
		3, {0xE6, 0x80, 0xA8},
		2, {0x28, 0x60},
		2, {0xB1, 0xE5}
	},
	{
		2, {0x89, 0x86},
		3, {0xE6, 0x8E, 0xA9},
		2, {0xA9, 0x63},
		2, {0xB1, 0xE6}
	},
	{
		2, {0x89, 0x87},
		3, {0xE6, 0x8F, 0xB4},
		2, {0xF4, 0x63},
		2, {0xB1, 0xE7}
	},
	{
		2, {0x89, 0x88},
		3, {0xE6, 0xB2, 0xBF},
		2, {0xBF, 0x6C},
		2, {0xB1, 0xE8}
	},
	{
		2, {0x89, 0x89},
		3, {0xE6, 0xBC, 0x94},
		2, {0x14, 0x6F},
		2, {0xB1, 0xE9}
	},
	{
		2, {0x89, 0x8A},
		3, {0xE7, 0x82, 0x8E},
		2, {0x8E, 0x70},
		2, {0xB1, 0xEA}
	},
	{
		2, {0x89, 0x8B},
		3, {0xE7, 0x84, 0x94},
		2, {0x14, 0x71},
		2, {0xB1, 0xEB}
	},
	{
		2, {0x89, 0x8C},
		3, {0xE7, 0x85, 0x99},
		2, {0x59, 0x71},
		2, {0xB1, 0xEC}
	},
	{
		2, {0x89, 0x8D},
		3, {0xE7, 0x87, 0x95},
		2, {0xD5, 0x71},
		2, {0xB1, 0xED}
	},
	{
		2, {0x89, 0x8E},
		3, {0xE7, 0x8C, 0xBF},
		2, {0x3F, 0x73},
		2, {0xB1, 0xEE}
	},
	{
		2, {0x89, 0x8F},
		3, {0xE7, 0xB8, 0x81},
		2, {0x01, 0x7E},
		2, {0xB1, 0xEF}
	},
	{
		2, {0x89, 0x90},
		3, {0xE8, 0x89, 0xB6},
		2, {0x76, 0x82},
		2, {0xB1, 0xF0}
	},
	{
		2, {0x89, 0x91},
		3, {0xE8, 0x8B, 0x91},
		2, {0xD1, 0x82},
		2, {0xB1, 0xF1}
	},
	{
		2, {0x89, 0x92},
		3, {0xE8, 0x96, 0x97},
		2, {0x97, 0x85},
		2, {0xB1, 0xF2}
	},
	{
		2, {0x89, 0x93},
		3, {0xE9, 0x81, 0xA0},
		2, {0x60, 0x90},
		2, {0xB1, 0xF3}
	},
	{
		2, {0x89, 0x94},
		3, {0xE9, 0x89, 0x9B},
		2, {0x5B, 0x92},
		2, {0xB1, 0xF4}
	},
	{
		2, {0x89, 0x95},
		3, {0xE9, 0xB4, 0x9B},
		2, {0x1B, 0x9D},
		2, {0xB1, 0xF5}
	},
	{
		2, {0x89, 0x96},
		3, {0xE5, 0xA1, 0xA9},
		2, {0x69, 0x58},
		2, {0xB1, 0xF6}
	},
	{
		2, {0x89, 0x97},
		3, {0xE6, 0x96, 0xBC},
		2, {0xBC, 0x65},
		2, {0xB1, 0xF7}
	},
	{
		2, {0x89, 0x98},
		3, {0xE6, 0xB1, 0x9A},
		2, {0x5A, 0x6C},
		2, {0xB1, 0xF8}
	},
	{
		2, {0x89, 0x99},
		3, {0xE7, 0x94, 0xA5},
		2, {0x25, 0x75},
		2, {0xB1, 0xF9}
	},
	{
		2, {0x89, 0x9A},
		3, {0xE5, 0x87, 0xB9},
		2, {0xF9, 0x51},
		2, {0xB1, 0xFA}
	},
	{
		2, {0x89, 0x9B},
		3, {0xE5, 0xA4, 0xAE},
		2, {0x2E, 0x59},
		2, {0xB1, 0xFB}
	},
	{
		2, {0x89, 0x9C},
		3, {0xE5, 0xA5, 0xA5},
		2, {0x65, 0x59},
		2, {0xB1, 0xFC}
	},
	{
		2, {0x89, 0x9D},
		3, {0xE5, 0xBE, 0x80},
		2, {0x80, 0x5F},
		2, {0xB1, 0xFD}
	},
	{
		2, {0x89, 0x9E},
		3, {0xE5, 0xBF, 0x9C},
		2, {0xDC, 0x5F},
		2, {0xB1, 0xFE}
	},
	{
		2, {0x89, 0x9F},
		3, {0xE6, 0x8A, 0xBC},
		2, {0xBC, 0x62},
		2, {0xB2, 0xA1}
	},
	{
		2, {0x89, 0xA0},
		3, {0xE6, 0x97, 0xBA},
		2, {0xFA, 0x65},
		2, {0xB2, 0xA2}
	},
	{
		2, {0x89, 0xA1},
		3, {0xE6, 0xA8, 0xAA},
		2, {0x2A, 0x6A},
		2, {0xB2, 0xA3}
	},
	{
		2, {0x89, 0xA2},
		3, {0xE6, 0xAC, 0xA7},
		2, {0x27, 0x6B},
		2, {0xB2, 0xA4}
	},
	{
		2, {0x89, 0xA3},
		3, {0xE6, 0xAE, 0xB4},
		2, {0xB4, 0x6B},
		2, {0xB2, 0xA5}
	},
	{
		2, {0x89, 0xA4},
		3, {0xE7, 0x8E, 0x8B},
		2, {0x8B, 0x73},
		2, {0xB2, 0xA6}
	},
	{
		2, {0x89, 0xA5},
		3, {0xE7, 0xBF, 0x81},
		2, {0xC1, 0x7F},
		2, {0xB2, 0xA7}
	},
	{
		2, {0x89, 0xA6},
		3, {0xE8, 0xA5, 0x96},
		2, {0x56, 0x89},
		2, {0xB2, 0xA8}
	},
	{
		2, {0x89, 0xA7},
		3, {0xE9, 0xB4, 0xAC},
		2, {0x2C, 0x9D},
		2, {0xB2, 0xA9}
	},
	{
		2, {0x89, 0xA8},
		3, {0xE9, 0xB4, 0x8E},
		2, {0x0E, 0x9D},
		2, {0xB2, 0xAA}
	},
	{
		2, {0x89, 0xA9},
		3, {0xE9, 0xBB, 0x84},
		2, {0xC4, 0x9E},
		2, {0xB2, 0xAB}
	},
	{
		2, {0x89, 0xAA},
		3, {0xE5, 0xB2, 0xA1},
		2, {0xA1, 0x5C},
		2, {0xB2, 0xAC}
	},
	{
		2, {0x89, 0xAB},
		3, {0xE6, 0xB2, 0x96},
		2, {0x96, 0x6C},
		2, {0xB2, 0xAD}
	},
	{
		2, {0x89, 0xAC},
		3, {0xE8, 0x8D, 0xBB},
		2, {0x7B, 0x83},
		2, {0xB2, 0xAE}
	},
	{
		2, {0x89, 0xAD},
		3, {0xE5, 0x84, 0x84},
		2, {0x04, 0x51},
		2, {0xB2, 0xAF}
	},
	{
		2, {0x89, 0xAE},
		3, {0xE5, 0xB1, 0x8B},
		2, {0x4B, 0x5C},
		2, {0xB2, 0xB0}
	},
	{
		2, {0x89, 0xAF},
		3, {0xE6, 0x86, 0xB6},
		2, {0xB6, 0x61},
		2, {0xB2, 0xB1}
	},
	{
		2, {0x89, 0xB0},
		3, {0xE8, 0x87, 0x86},
		2, {0xC6, 0x81},
		2, {0xB2, 0xB2}
	},
	{
		2, {0x89, 0xB1},
		3, {0xE6, 0xA1, 0xB6},
		2, {0x76, 0x68},
		2, {0xB2, 0xB3}
	},
	{
		2, {0x89, 0xB2},
		3, {0xE7, 0x89, 0xA1},
		2, {0x61, 0x72},
		2, {0xB2, 0xB4}
	},
	{
		2, {0x89, 0xB3},
		3, {0xE4, 0xB9, 0x99},
		2, {0x59, 0x4E},
		2, {0xB2, 0xB5}
	},
	{
		2, {0x89, 0xB4},
		3, {0xE4, 0xBF, 0xBA},
		2, {0xFA, 0x4F},
		2, {0xB2, 0xB6}
	},
	{
		2, {0x89, 0xB5},
		3, {0xE5, 0x8D, 0xB8},
		2, {0x78, 0x53},
		2, {0xB2, 0xB7}
	},
	{
		2, {0x89, 0xB6},
		3, {0xE6, 0x81, 0xA9},
		2, {0x69, 0x60},
		2, {0xB2, 0xB8}
	},
	{
		2, {0x89, 0xB7},
		3, {0xE6, 0xB8, 0xA9},
		2, {0x29, 0x6E},
		2, {0xB2, 0xB9}
	},
	{
		2, {0x89, 0xB8},
		3, {0xE7, 0xA9, 0x8F},
		2, {0x4F, 0x7A},
		2, {0xB2, 0xBA}
	},
	{
		2, {0x89, 0xB9},
		3, {0xE9, 0x9F, 0xB3},
		2, {0xF3, 0x97},
		2, {0xB2, 0xBB}
	},
	{
		2, {0x89, 0xBA},
		3, {0xE4, 0xB8, 0x8B},
		2, {0x0B, 0x4E},
		2, {0xB2, 0xBC}
	},
	{
		2, {0x89, 0xBB},
		3, {0xE5, 0x8C, 0x96},
		2, {0x16, 0x53},
		2, {0xB2, 0xBD}
	},
	{
		2, {0x89, 0xBC},
		3, {0xE4, 0xBB, 0xAE},
		2, {0xEE, 0x4E},
		2, {0xB2, 0xBE}
	},
	{
		2, {0x89, 0xBD},
		3, {0xE4, 0xBD, 0x95},
		2, {0x55, 0x4F},
		2, {0xB2, 0xBF}
	},
	{
		2, {0x89, 0xBE},
		3, {0xE4, 0xBC, 0xBD},
		2, {0x3D, 0x4F},
		2, {0xB2, 0xC0}
	},
	{
		2, {0x89, 0xBF},
		3, {0xE4, 0xBE, 0xA1},
		2, {0xA1, 0x4F},
		2, {0xB2, 0xC1}
	},
	{
		2, {0x89, 0xC0},
		3, {0xE4, 0xBD, 0xB3},
		2, {0x73, 0x4F},
		2, {0xB2, 0xC2}
	},
	{
		2, {0x89, 0xC1},
		3, {0xE5, 0x8A, 0xA0},
		2, {0xA0, 0x52},
		2, {0xB2, 0xC3}
	},
	{
		2, {0x89, 0xC2},
		3, {0xE5, 0x8F, 0xAF},
		2, {0xEF, 0x53},
		2, {0xB2, 0xC4}
	},
	{
		2, {0x89, 0xC3},
		3, {0xE5, 0x98, 0x89},
		2, {0x09, 0x56},
		2, {0xB2, 0xC5}
	},
	{
		2, {0x89, 0xC4},
		3, {0xE5, 0xA4, 0x8F},
		2, {0x0F, 0x59},
		2, {0xB2, 0xC6}
	},
	{
		2, {0x89, 0xC5},
		3, {0xE5, 0xAB, 0x81},
		2, {0xC1, 0x5A},
		2, {0xB2, 0xC7}
	},
	{
		2, {0x89, 0xC6},
		3, {0xE5, 0xAE, 0xB6},
		2, {0xB6, 0x5B},
		2, {0xB2, 0xC8}
	},
	{
		2, {0x89, 0xC7},
		3, {0xE5, 0xAF, 0xA1},
		2, {0xE1, 0x5B},
		2, {0xB2, 0xC9}
	},
	{
		2, {0x89, 0xC8},
		3, {0xE7, 0xA7, 0x91},
		2, {0xD1, 0x79},
		2, {0xB2, 0xCA}
	},
	{
		2, {0x89, 0xC9},
		3, {0xE6, 0x9A, 0x87},
		2, {0x87, 0x66},
		2, {0xB2, 0xCB}
	},
	{
		2, {0x89, 0xCA},
		3, {0xE6, 0x9E, 0x9C},
		2, {0x9C, 0x67},
		2, {0xB2, 0xCC}
	},
	{
		2, {0x89, 0xCB},
		3, {0xE6, 0x9E, 0xB6},
		2, {0xB6, 0x67},
		2, {0xB2, 0xCD}
	},
	{
		2, {0x89, 0xCC},
		3, {0xE6, 0xAD, 0x8C},
		2, {0x4C, 0x6B},
		2, {0xB2, 0xCE}
	},
	{
		2, {0x89, 0xCD},
		3, {0xE6, 0xB2, 0xB3},
		2, {0xB3, 0x6C},
		2, {0xB2, 0xCF}
	},
	{
		2, {0x89, 0xCE},
		3, {0xE7, 0x81, 0xAB},
		2, {0x6B, 0x70},
		2, {0xB2, 0xD0}
	},
	{
		2, {0x89, 0xCF},
		3, {0xE7, 0x8F, 0x82},
		2, {0xC2, 0x73},
		2, {0xB2, 0xD1}
	},
	{
		2, {0x89, 0xD0},
		3, {0xE7, 0xA6, 0x8D},
		2, {0x8D, 0x79},
		2, {0xB2, 0xD2}
	},
	{
		2, {0x89, 0xD1},
		3, {0xE7, 0xA6, 0xBE},
		2, {0xBE, 0x79},
		2, {0xB2, 0xD3}
	},
	{
		2, {0x89, 0xD2},
		3, {0xE7, 0xA8, 0xBC},
		2, {0x3C, 0x7A},
		2, {0xB2, 0xD4}
	},
	{
		2, {0x89, 0xD3},
		3, {0xE7, 0xAE, 0x87},
		2, {0x87, 0x7B},
		2, {0xB2, 0xD5}
	},
	{
		2, {0x89, 0xD4},
		3, {0xE8, 0x8A, 0xB1},
		2, {0xB1, 0x82},
		2, {0xB2, 0xD6}
	},
	{
		2, {0x89, 0xD5},
		3, {0xE8, 0x8B, 0x9B},
		2, {0xDB, 0x82},
		2, {0xB2, 0xD7}
	},
	{
		2, {0x89, 0xD6},
		3, {0xE8, 0x8C, 0x84},
		2, {0x04, 0x83},
		2, {0xB2, 0xD8}
	},
	{
		2, {0x89, 0xD7},
		3, {0xE8, 0x8D, 0xB7},
		2, {0x77, 0x83},
		2, {0xB2, 0xD9}
	},
	{
		2, {0x89, 0xD8},
		3, {0xE8, 0x8F, 0xAF},
		2, {0xEF, 0x83},
		2, {0xB2, 0xDA}
	},
	{
		2, {0x89, 0xD9},
		3, {0xE8, 0x8F, 0x93},
		2, {0xD3, 0x83},
		2, {0xB2, 0xDB}
	},
	{
		2, {0x89, 0xDA},
		3, {0xE8, 0x9D, 0xA6},
		2, {0x66, 0x87},
		2, {0xB2, 0xDC}
	},
	{
		2, {0x89, 0xDB},
		3, {0xE8, 0xAA, 0xB2},
		2, {0xB2, 0x8A},
		2, {0xB2, 0xDD}
	},
	{
		2, {0x89, 0xDC},
		3, {0xE5, 0x98, 0xA9},
		2, {0x29, 0x56},
		2, {0xB2, 0xDE}
	},
	{
		2, {0x89, 0xDD},
		3, {0xE8, 0xB2, 0xA8},
		2, {0xA8, 0x8C},
		2, {0xB2, 0xDF}
	},
	{
		2, {0x89, 0xDE},
		3, {0xE8, 0xBF, 0xA6},
		2, {0xE6, 0x8F},
		2, {0xB2, 0xE0}
	},
	{
		2, {0x89, 0xDF},
		3, {0xE9, 0x81, 0x8E},
		2, {0x4E, 0x90},
		2, {0xB2, 0xE1}
	},
	{
		2, {0x89, 0xE0},
		3, {0xE9, 0x9C, 0x9E},
		2, {0x1E, 0x97},
		2, {0xB2, 0xE2}
	},
	{
		2, {0x89, 0xE1},
		3, {0xE8, 0x9A, 0x8A},
		2, {0x8A, 0x86},
		2, {0xB2, 0xE3}
	},
	{
		2, {0x89, 0xE2},
		3, {0xE4, 0xBF, 0x84},
		2, {0xC4, 0x4F},
		2, {0xB2, 0xE4}
	},
	{
		2, {0x89, 0xE3},
		3, {0xE5, 0xB3, 0xA8},
		2, {0xE8, 0x5C},
		2, {0xB2, 0xE5}
	},
	{
		2, {0x89, 0xE4},
		3, {0xE6, 0x88, 0x91},
		2, {0x11, 0x62},
		2, {0xB2, 0xE6}
	},
	{
		2, {0x89, 0xE5},
		3, {0xE7, 0x89, 0x99},
		2, {0x59, 0x72},
		2, {0xB2, 0xE7}
	},
	{
		2, {0x89, 0xE6},
		3, {0xE7, 0x94, 0xBB},
		2, {0x3B, 0x75},
		2, {0xB2, 0xE8}
	},
	{
		2, {0x89, 0xE7},
		3, {0xE8, 0x87, 0xA5},
		2, {0xE5, 0x81},
		2, {0xB2, 0xE9}
	},
	{
		2, {0x89, 0xE8},
		3, {0xE8, 0x8A, 0xBD},
		2, {0xBD, 0x82},
		2, {0xB2, 0xEA}
	},
	{
		2, {0x89, 0xE9},
		3, {0xE8, 0x9B, 0xBE},
		2, {0xFE, 0x86},
		2, {0xB2, 0xEB}
	},
	{
		2, {0x89, 0xEA},
		3, {0xE8, 0xB3, 0x80},
		2, {0xC0, 0x8C},
		2, {0xB2, 0xEC}
	},
	{
		2, {0x89, 0xEB},
		3, {0xE9, 0x9B, 0x85},
		2, {0xC5, 0x96},
		2, {0xB2, 0xED}
	},
	{
		2, {0x89, 0xEC},
		3, {0xE9, 0xA4, 0x93},
		2, {0x13, 0x99},
		2, {0xB2, 0xEE}
	},
	{
		2, {0x89, 0xED},
		3, {0xE9, 0xA7, 0x95},
		2, {0xD5, 0x99},
		2, {0xB2, 0xEF}
	},
	{
		2, {0x89, 0xEE},
		3, {0xE4, 0xBB, 0x8B},
		2, {0xCB, 0x4E},
		2, {0xB2, 0xF0}
	},
	{
		2, {0x89, 0xEF},
		3, {0xE4, 0xBC, 0x9A},
		2, {0x1A, 0x4F},
		2, {0xB2, 0xF1}
	},
	{
		2, {0x89, 0xF0},
		3, {0xE8, 0xA7, 0xA3},
		2, {0xE3, 0x89},
		2, {0xB2, 0xF2}
	},
	{
		2, {0x89, 0xF1},
		3, {0xE5, 0x9B, 0x9E},
		2, {0xDE, 0x56},
		2, {0xB2, 0xF3}
	},
	{
		2, {0x89, 0xF2},
		3, {0xE5, 0xA1, 0x8A},
		2, {0x4A, 0x58},
		2, {0xB2, 0xF4}
	},
	{
		2, {0x89, 0xF3},
		3, {0xE5, 0xA3, 0x8A},
		2, {0xCA, 0x58},
		2, {0xB2, 0xF5}
	},
	{
		2, {0x89, 0xF4},
		3, {0xE5, 0xBB, 0xBB},
		2, {0xFB, 0x5E},
		2, {0xB2, 0xF6}
	},
	{
		2, {0x89, 0xF5},
		3, {0xE5, 0xBF, 0xAB},
		2, {0xEB, 0x5F},
		2, {0xB2, 0xF7}
	},
	{
		2, {0x89, 0xF6},
		3, {0xE6, 0x80, 0xAA},
		2, {0x2A, 0x60},
		2, {0xB2, 0xF8}
	},
	{
		2, {0x89, 0xF7},
		3, {0xE6, 0x82, 0x94},
		2, {0x94, 0x60},
		2, {0xB2, 0xF9}
	},
	{
		2, {0x89, 0xF8},
		3, {0xE6, 0x81, 0xA2},
		2, {0x62, 0x60},
		2, {0xB2, 0xFA}
	},
	{
		2, {0x89, 0xF9},
		3, {0xE6, 0x87, 0x90},
		2, {0xD0, 0x61},
		2, {0xB2, 0xFB}
	},
	{
		2, {0x89, 0xFA},
		3, {0xE6, 0x88, 0x92},
		2, {0x12, 0x62},
		2, {0xB2, 0xFC}
	},
	{
		2, {0x89, 0xFB},
		3, {0xE6, 0x8B, 0x90},
		2, {0xD0, 0x62},
		2, {0xB2, 0xFD}
	},
	{
		2, {0x89, 0xFC},
		3, {0xE6, 0x94, 0xB9},
		2, {0x39, 0x65},
		2, {0xB2, 0xFE}
	},
	{
		2, {0x8A, 0x40},
		3, {0xE9, 0xAD, 0x81},
		2, {0x41, 0x9B},
		2, {0xB3, 0xA1}
	},
	{
		2, {0x8A, 0x41},
		3, {0xE6, 0x99, 0xA6},
		2, {0x66, 0x66},
		2, {0xB3, 0xA2}
	},
	{
		2, {0x8A, 0x42},
		3, {0xE6, 0xA2, 0xB0},
		2, {0xB0, 0x68},
		2, {0xB3, 0xA3}
	},
	{
		2, {0x8A, 0x43},
		3, {0xE6, 0xB5, 0xB7},
		2, {0x77, 0x6D},
		2, {0xB3, 0xA4}
	},
	{
		2, {0x8A, 0x44},
		3, {0xE7, 0x81, 0xB0},
		2, {0x70, 0x70},
		2, {0xB3, 0xA5}
	},
	{
		2, {0x8A, 0x45},
		3, {0xE7, 0x95, 0x8C},
		2, {0x4C, 0x75},
		2, {0xB3, 0xA6}
	},
	{
		2, {0x8A, 0x46},
		3, {0xE7, 0x9A, 0x86},
		2, {0x86, 0x76},
		2, {0xB3, 0xA7}
	},
	{
		2, {0x8A, 0x47},
		3, {0xE7, 0xB5, 0xB5},
		2, {0x75, 0x7D},
		2, {0xB3, 0xA8}
	},
	{
		2, {0x8A, 0x48},
		3, {0xE8, 0x8A, 0xA5},
		2, {0xA5, 0x82},
		2, {0xB3, 0xA9}
	},
	{
		2, {0x8A, 0x49},
		3, {0xE8, 0x9F, 0xB9},
		2, {0xF9, 0x87},
		2, {0xB3, 0xAA}
	},
	{
		2, {0x8A, 0x4A},
		3, {0xE9, 0x96, 0x8B},
		2, {0x8B, 0x95},
		2, {0xB3, 0xAB}
	},
	{
		2, {0x8A, 0x4B},
		3, {0xE9, 0x9A, 0x8E},
		2, {0x8E, 0x96},
		2, {0xB3, 0xAC}
	},
	{
		2, {0x8A, 0x4C},
		3, {0xE8, 0xB2, 0x9D},
		2, {0x9D, 0x8C},
		2, {0xB3, 0xAD}
	},
	{
		2, {0x8A, 0x4D},
		3, {0xE5, 0x87, 0xB1},
		2, {0xF1, 0x51},
		2, {0xB3, 0xAE}
	},
	{
		2, {0x8A, 0x4E},
		3, {0xE5, 0x8A, 0xBE},
		2, {0xBE, 0x52},
		2, {0xB3, 0xAF}
	},
	{
		2, {0x8A, 0x4F},
		3, {0xE5, 0xA4, 0x96},
		2, {0x16, 0x59},
		2, {0xB3, 0xB0}
	},
	{
		2, {0x8A, 0x50},
		3, {0xE5, 0x92, 0xB3},
		2, {0xB3, 0x54},
		2, {0xB3, 0xB1}
	},
	{
		2, {0x8A, 0x51},
		3, {0xE5, 0xAE, 0xB3},
		2, {0xB3, 0x5B},
		2, {0xB3, 0xB2}
	},
	{
		2, {0x8A, 0x52},
		3, {0xE5, 0xB4, 0x96},
		2, {0x16, 0x5D},
		2, {0xB3, 0xB3}
	},
	{
		2, {0x8A, 0x53},
		3, {0xE6, 0x85, 0xA8},
		2, {0x68, 0x61},
		2, {0xB3, 0xB4}
	},
	{
		2, {0x8A, 0x54},
		3, {0xE6, 0xA6, 0x82},
		2, {0x82, 0x69},
		2, {0xB3, 0xB5}
	},
	{
		2, {0x8A, 0x55},
		3, {0xE6, 0xB6, 0xAF},
		2, {0xAF, 0x6D},
		2, {0xB3, 0xB6}
	},
	{
		2, {0x8A, 0x56},
		3, {0xE7, 0xA2, 0x8D},
		2, {0x8D, 0x78},
		2, {0xB3, 0xB7}
	},
	{
		2, {0x8A, 0x57},
		3, {0xE8, 0x93, 0x8B},
		2, {0xCB, 0x84},
		2, {0xB3, 0xB8}
	},
	{
		2, {0x8A, 0x58},
		3, {0xE8, 0xA1, 0x97},
		2, {0x57, 0x88},
		2, {0xB3, 0xB9}
	},
	{
		2, {0x8A, 0x59},
		3, {0xE8, 0xA9, 0xB2},
		2, {0x72, 0x8A},
		2, {0xB3, 0xBA}
	},
	{
		2, {0x8A, 0x5A},
		3, {0xE9, 0x8E, 0xA7},
		2, {0xA7, 0x93},
		2, {0xB3, 0xBB}
	},
	{
		2, {0x8A, 0x5B},
		3, {0xE9, 0xAA, 0xB8},
		2, {0xB8, 0x9A},
		2, {0xB3, 0xBC}
	},
	{
		2, {0x8A, 0x5C},
		3, {0xE6, 0xB5, 0xAC},
		2, {0x6C, 0x6D},
		2, {0xB3, 0xBD}
	},
	{
		2, {0x8A, 0x5D},
		3, {0xE9, 0xA6, 0xA8},
		2, {0xA8, 0x99},
		2, {0xB3, 0xBE}
	},
	{
		2, {0x8A, 0x5E},
		3, {0xE8, 0x9B, 0x99},
		2, {0xD9, 0x86},
		2, {0xB3, 0xBF}
	},
	{
		2, {0x8A, 0x5F},
		3, {0xE5, 0x9E, 0xA3},
		2, {0xA3, 0x57},
		2, {0xB3, 0xC0}
	},
	{
		2, {0x8A, 0x60},
		3, {0xE6, 0x9F, 0xBF},
		2, {0xFF, 0x67},
		2, {0xB3, 0xC1}
	},
	{
		2, {0x8A, 0x61},
		3, {0xE8, 0x9B, 0x8E},
		2, {0xCE, 0x86},
		2, {0xB3, 0xC2}
	},
	{
		2, {0x8A, 0x62},
		3, {0xE9, 0x88, 0x8E},
		2, {0x0E, 0x92},
		2, {0xB3, 0xC3}
	},
	{
		2, {0x8A, 0x63},
		3, {0xE5, 0x8A, 0x83},
		2, {0x83, 0x52},
		2, {0xB3, 0xC4}
	},
	{
		2, {0x8A, 0x64},
		3, {0xE5, 0x9A, 0x87},
		2, {0x87, 0x56},
		2, {0xB3, 0xC5}
	},
	{
		2, {0x8A, 0x65},
		3, {0xE5, 0x90, 0x84},
		2, {0x04, 0x54},
		2, {0xB3, 0xC6}
	},
	{
		2, {0x8A, 0x66},
		3, {0xE5, 0xBB, 0x93},
		2, {0xD3, 0x5E},
		2, {0xB3, 0xC7}
	},
	{
		2, {0x8A, 0x67},
		3, {0xE6, 0x8B, 0xA1},
		2, {0xE1, 0x62},
		2, {0xB3, 0xC8}
	},
	{
		2, {0x8A, 0x68},
		3, {0xE6, 0x92, 0xB9},
		2, {0xB9, 0x64},
		2, {0xB3, 0xC9}
	},
	{
		2, {0x8A, 0x69},
		3, {0xE6, 0xA0, 0xBC},
		2, {0x3C, 0x68},
		2, {0xB3, 0xCA}
	},
	{
		2, {0x8A, 0x6A},
		3, {0xE6, 0xA0, 0xB8},
		2, {0x38, 0x68},
		2, {0xB3, 0xCB}
	},
	{
		2, {0x8A, 0x6B},
		3, {0xE6, 0xAE, 0xBB},
		2, {0xBB, 0x6B},
		2, {0xB3, 0xCC}
	},
	{
		2, {0x8A, 0x6C},
		3, {0xE7, 0x8D, 0xB2},
		2, {0x72, 0x73},
		2, {0xB3, 0xCD}
	},
	{
		2, {0x8A, 0x6D},
		3, {0xE7, 0xA2, 0xBA},
		2, {0xBA, 0x78},
		2, {0xB3, 0xCE}
	},
	{
		2, {0x8A, 0x6E},
		3, {0xE7, 0xA9, 0xAB},
		2, {0x6B, 0x7A},
		2, {0xB3, 0xCF}
	},
	{
		2, {0x8A, 0x6F},
		3, {0xE8, 0xA6, 0x9A},
		2, {0x9A, 0x89},
		2, {0xB3, 0xD0}
	},
	{
		2, {0x8A, 0x70},
		3, {0xE8, 0xA7, 0x92},
		2, {0xD2, 0x89},
		2, {0xB3, 0xD1}
	},
	{
		2, {0x8A, 0x71},
		3, {0xE8, 0xB5, 0xAB},
		2, {0x6B, 0x8D},
		2, {0xB3, 0xD2}
	},
	{
		2, {0x8A, 0x72},
		3, {0xE8, 0xBC, 0x83},
		2, {0x03, 0x8F},
		2, {0xB3, 0xD3}
	},
	{
		2, {0x8A, 0x73},
		3, {0xE9, 0x83, 0xAD},
		2, {0xED, 0x90},
		2, {0xB3, 0xD4}
	},
	{
		2, {0x8A, 0x74},
		3, {0xE9, 0x96, 0xA3},
		2, {0xA3, 0x95},
		2, {0xB3, 0xD5}
	},
	{
		2, {0x8A, 0x75},
		3, {0xE9, 0x9A, 0x94},
		2, {0x94, 0x96},
		2, {0xB3, 0xD6}
	},
	{
		2, {0x8A, 0x76},
		3, {0xE9, 0x9D, 0xA9},
		2, {0x69, 0x97},
		2, {0xB3, 0xD7}
	},
	{
		2, {0x8A, 0x77},
		3, {0xE5, 0xAD, 0xA6},
		2, {0x66, 0x5B},
		2, {0xB3, 0xD8}
	},
	{
		2, {0x8A, 0x78},
		3, {0xE5, 0xB2, 0xB3},
		2, {0xB3, 0x5C},
		2, {0xB3, 0xD9}
	},
	{
		2, {0x8A, 0x79},
		3, {0xE6, 0xA5, 0xBD},
		2, {0x7D, 0x69},
		2, {0xB3, 0xDA}
	},
	{
		2, {0x8A, 0x7A},
		3, {0xE9, 0xA1, 0x8D},
		2, {0x4D, 0x98},
		2, {0xB3, 0xDB}
	},
	{
		2, {0x8A, 0x7B},
		3, {0xE9, 0xA1, 0x8E},
		2, {0x4E, 0x98},
		2, {0xB3, 0xDC}
	},
	{
		2, {0x8A, 0x7C},
		3, {0xE6, 0x8E, 0x9B},
		2, {0x9B, 0x63},
		2, {0xB3, 0xDD}
	},
	{
		2, {0x8A, 0x7D},
		3, {0xE7, 0xAC, 0xA0},
		2, {0x20, 0x7B},
		2, {0xB3, 0xDE}
	},
	{
		2, {0x8A, 0x7E},
		3, {0xE6, 0xA8, 0xAB},
		2, {0x2B, 0x6A},
		2, {0xB3, 0xDF}
	},
	{
		2, {0x8A, 0x80},
		3, {0xE6, 0xA9, 0xBF},
		2, {0x7F, 0x6A},
		2, {0xB3, 0xE0}
	},
	{
		2, {0x8A, 0x81},
		3, {0xE6, 0xA2, 0xB6},
		2, {0xB6, 0x68},
		2, {0xB3, 0xE1}
	},
	{
		2, {0x8A, 0x82},
		3, {0xE9, 0xB0, 0x8D},
		2, {0x0D, 0x9C},
		2, {0xB3, 0xE2}
	},
	{
		2, {0x8A, 0x83},
		3, {0xE6, 0xBD, 0x9F},
		2, {0x5F, 0x6F},
		2, {0xB3, 0xE3}
	},
	{
		2, {0x8A, 0x84},
		3, {0xE5, 0x89, 0xB2},
		2, {0x72, 0x52},
		2, {0xB3, 0xE4}
	},
	{
		2, {0x8A, 0x85},
		3, {0xE5, 0x96, 0x9D},
		2, {0x9D, 0x55},
		2, {0xB3, 0xE5}
	},
	{
		2, {0x8A, 0x86},
		3, {0xE6, 0x81, 0xB0},
		2, {0x70, 0x60},
		2, {0xB3, 0xE6}
	},
	{
		2, {0x8A, 0x87},
		3, {0xE6, 0x8B, 0xAC},
		2, {0xEC, 0x62},
		2, {0xB3, 0xE7}
	},
	{
		2, {0x8A, 0x88},
		3, {0xE6, 0xB4, 0xBB},
		2, {0x3B, 0x6D},
		2, {0xB3, 0xE8}
	},
	{
		2, {0x8A, 0x89},
		3, {0xE6, 0xB8, 0x87},
		2, {0x07, 0x6E},
		2, {0xB3, 0xE9}
	},
	{
		2, {0x8A, 0x8A},
		3, {0xE6, 0xBB, 0x91},
		2, {0xD1, 0x6E},
		2, {0xB3, 0xEA}
	},
	{
		2, {0x8A, 0x8B},
		3, {0xE8, 0x91, 0x9B},
		2, {0x5B, 0x84},
		2, {0xB3, 0xEB}
	},
	{
		2, {0x8A, 0x8C},
		3, {0xE8, 0xA4, 0x90},
		2, {0x10, 0x89},
		2, {0xB3, 0xEC}
	},
	{
		2, {0x8A, 0x8D},
		3, {0xE8, 0xBD, 0x84},
		2, {0x44, 0x8F},
		2, {0xB3, 0xED}
	},
	{
		2, {0x8A, 0x8E},
		3, {0xE4, 0xB8, 0x94},
		2, {0x14, 0x4E},
		2, {0xB3, 0xEE}
	},
	{
		2, {0x8A, 0x8F},
		3, {0xE9, 0xB0, 0xB9},
		2, {0x39, 0x9C},
		2, {0xB3, 0xEF}
	},
	{
		2, {0x8A, 0x90},
		3, {0xE5, 0x8F, 0xB6},
		2, {0xF6, 0x53},
		2, {0xB3, 0xF0}
	},
	{
		2, {0x8A, 0x91},
		3, {0xE6, 0xA4, 0x9B},
		2, {0x1B, 0x69},
		2, {0xB3, 0xF1}
	},
	{
		2, {0x8A, 0x92},
		3, {0xE6, 0xA8, 0xBA},
		2, {0x3A, 0x6A},
		2, {0xB3, 0xF2}
	},
	{
		2, {0x8A, 0x93},
		3, {0xE9, 0x9E, 0x84},
		2, {0x84, 0x97},
		2, {0xB3, 0xF3}
	},
	{
		2, {0x8A, 0x94},
		3, {0xE6, 0xA0, 0xAA},
		2, {0x2A, 0x68},
		2, {0xB3, 0xF4}
	},
	{
		2, {0x8A, 0x95},
		3, {0xE5, 0x85, 0x9C},
		2, {0x5C, 0x51},
		2, {0xB3, 0xF5}
	},
	{
		2, {0x8A, 0x96},
		3, {0xE7, 0xAB, 0x83},
		2, {0xC3, 0x7A},
		2, {0xB3, 0xF6}
	},
	{
		2, {0x8A, 0x97},
		3, {0xE8, 0x92, 0xB2},
		2, {0xB2, 0x84},
		2, {0xB3, 0xF7}
	},
	{
		2, {0x8A, 0x98},
		3, {0xE9, 0x87, 0x9C},
		2, {0xDC, 0x91},
		2, {0xB3, 0xF8}
	},
	{
		2, {0x8A, 0x99},
		3, {0xE9, 0x8E, 0x8C},
		2, {0x8C, 0x93},
		2, {0xB3, 0xF9}
	},
	{
		2, {0x8A, 0x9A},
		3, {0xE5, 0x99, 0x9B},
		2, {0x5B, 0x56},
		2, {0xB3, 0xFA}
	},
	{
		2, {0x8A, 0x9B},
		3, {0xE9, 0xB4, 0xA8},
		2, {0x28, 0x9D},
		2, {0xB3, 0xFB}
	},
	{
		2, {0x8A, 0x9C},
		3, {0xE6, 0xA0, 0xA2},
		2, {0x22, 0x68},
		2, {0xB3, 0xFC}
	},
	{
		2, {0x8A, 0x9D},
		3, {0xE8, 0x8C, 0x85},
		2, {0x05, 0x83},
		2, {0xB3, 0xFD}
	},
	{
		2, {0x8A, 0x9E},
		3, {0xE8, 0x90, 0xB1},
		2, {0x31, 0x84},
		2, {0xB3, 0xFE}
	},
	{
		2, {0x8A, 0x9F},
		3, {0xE7, 0xB2, 0xA5},
		2, {0xA5, 0x7C},
		2, {0xB4, 0xA1}
	},
	{
		2, {0x8A, 0xA0},
		3, {0xE5, 0x88, 0x88},
		2, {0x08, 0x52},
		2, {0xB4, 0xA2}
	},
	{
		2, {0x8A, 0xA1},
		3, {0xE8, 0x8B, 0x85},
		2, {0xC5, 0x82},
		2, {0xB4, 0xA3}
	},
	{
		2, {0x8A, 0xA2},
		3, {0xE7, 0x93, 0xA6},
		2, {0xE6, 0x74},
		2, {0xB4, 0xA4}
	},
	{
		2, {0x8A, 0xA3},
		3, {0xE4, 0xB9, 0xBE},
		2, {0x7E, 0x4E},
		2, {0xB4, 0xA5}
	},
	{
		2, {0x8A, 0xA4},
		3, {0xE4, 0xBE, 0x83},
		2, {0x83, 0x4F},
		2, {0xB4, 0xA6}
	},
	{
		2, {0x8A, 0xA5},
		3, {0xE5, 0x86, 0xA0},
		2, {0xA0, 0x51},
		2, {0xB4, 0xA7}
	},
	{
		2, {0x8A, 0xA6},
		3, {0xE5, 0xAF, 0x92},
		2, {0xD2, 0x5B},
		2, {0xB4, 0xA8}
	},
	{
		2, {0x8A, 0xA7},
		3, {0xE5, 0x88, 0x8A},
		2, {0x0A, 0x52},
		2, {0xB4, 0xA9}
	},
	{
		2, {0x8A, 0xA8},
		3, {0xE5, 0x8B, 0x98},
		2, {0xD8, 0x52},
		2, {0xB4, 0xAA}
	},
	{
		2, {0x8A, 0xA9},
		3, {0xE5, 0x8B, 0xA7},
		2, {0xE7, 0x52},
		2, {0xB4, 0xAB}
	},
	{
		2, {0x8A, 0xAA},
		3, {0xE5, 0xB7, 0xBB},
		2, {0xFB, 0x5D},
		2, {0xB4, 0xAC}
	},
	{
		2, {0x8A, 0xAB},
		3, {0xE5, 0x96, 0x9A},
		2, {0x9A, 0x55},
		2, {0xB4, 0xAD}
	},
	{
		2, {0x8A, 0xAC},
		3, {0xE5, 0xA0, 0xAA},
		2, {0x2A, 0x58},
		2, {0xB4, 0xAE}
	},
	{
		2, {0x8A, 0xAD},
		3, {0xE5, 0xA7, 0xA6},
		2, {0xE6, 0x59},
		2, {0xB4, 0xAF}
	},
	{
		2, {0x8A, 0xAE},
		3, {0xE5, 0xAE, 0x8C},
		2, {0x8C, 0x5B},
		2, {0xB4, 0xB0}
	},
	{
		2, {0x8A, 0xAF},
		3, {0xE5, 0xAE, 0x98},
		2, {0x98, 0x5B},
		2, {0xB4, 0xB1}
	},
	{
		2, {0x8A, 0xB0},
		3, {0xE5, 0xAF, 0x9B},
		2, {0xDB, 0x5B},
		2, {0xB4, 0xB2}
	},
	{
		2, {0x8A, 0xB1},
		3, {0xE5, 0xB9, 0xB2},
		2, {0x72, 0x5E},
		2, {0xB4, 0xB3}
	},
	{
		2, {0x8A, 0xB2},
		3, {0xE5, 0xB9, 0xB9},
		2, {0x79, 0x5E},
		2, {0xB4, 0xB4}
	},
	{
		2, {0x8A, 0xB3},
		3, {0xE6, 0x82, 0xA3},
		2, {0xA3, 0x60},
		2, {0xB4, 0xB5}
	},
	{
		2, {0x8A, 0xB4},
		3, {0xE6, 0x84, 0x9F},
		2, {0x1F, 0x61},
		2, {0xB4, 0xB6}
	},
	{
		2, {0x8A, 0xB5},
		3, {0xE6, 0x85, 0xA3},
		2, {0x63, 0x61},
		2, {0xB4, 0xB7}
	},
	{
		2, {0x8A, 0xB6},
		3, {0xE6, 0x86, 0xBE},
		2, {0xBE, 0x61},
		2, {0xB4, 0xB8}
	},
	{
		2, {0x8A, 0xB7},
		3, {0xE6, 0x8F, 0x9B},
		2, {0xDB, 0x63},
		2, {0xB4, 0xB9}
	},
	{
		2, {0x8A, 0xB8},
		3, {0xE6, 0x95, 0xA2},
		2, {0x62, 0x65},
		2, {0xB4, 0xBA}
	},
	{
		2, {0x8A, 0xB9},
		3, {0xE6, 0x9F, 0x91},
		2, {0xD1, 0x67},
		2, {0xB4, 0xBB}
	},
	{
		2, {0x8A, 0xBA},
		3, {0xE6, 0xA1, 0x93},
		2, {0x53, 0x68},
		2, {0xB4, 0xBC}
	},
	{
		2, {0x8A, 0xBB},
		3, {0xE6, 0xA3, 0xBA},
		2, {0xFA, 0x68},
		2, {0xB4, 0xBD}
	},
	{
		2, {0x8A, 0xBC},
		3, {0xE6, 0xAC, 0xBE},
		2, {0x3E, 0x6B},
		2, {0xB4, 0xBE}
	},
	{
		2, {0x8A, 0xBD},
		3, {0xE6, 0xAD, 0x93},
		2, {0x53, 0x6B},
		2, {0xB4, 0xBF}
	},
	{
		2, {0x8A, 0xBE},
		3, {0xE6, 0xB1, 0x97},
		2, {0x57, 0x6C},
		2, {0xB4, 0xC0}
	},
	{
		2, {0x8A, 0xBF},
		3, {0xE6, 0xBC, 0xA2},
		2, {0x22, 0x6F},
		2, {0xB4, 0xC1}
	},
	{
		2, {0x8A, 0xC0},
		3, {0xE6, 0xBE, 0x97},
		2, {0x97, 0x6F},
		2, {0xB4, 0xC2}
	},
	{
		2, {0x8A, 0xC1},
		3, {0xE6, 0xBD, 0x85},
		2, {0x45, 0x6F},
		2, {0xB4, 0xC3}
	},
	{
		2, {0x8A, 0xC2},
		3, {0xE7, 0x92, 0xB0},
		2, {0xB0, 0x74},
		2, {0xB4, 0xC4}
	},
	{
		2, {0x8A, 0xC3},
		3, {0xE7, 0x94, 0x98},
		2, {0x18, 0x75},
		2, {0xB4, 0xC5}
	},
	{
		2, {0x8A, 0xC4},
		3, {0xE7, 0x9B, 0xA3},
		2, {0xE3, 0x76},
		2, {0xB4, 0xC6}
	},
	{
		2, {0x8A, 0xC5},
		3, {0xE7, 0x9C, 0x8B},
		2, {0x0B, 0x77},
		2, {0xB4, 0xC7}
	},
	{
		2, {0x8A, 0xC6},
		3, {0xE7, 0xAB, 0xBF},
		2, {0xFF, 0x7A},
		2, {0xB4, 0xC8}
	},
	{
		2, {0x8A, 0xC7},
		3, {0xE7, 0xAE, 0xA1},
		2, {0xA1, 0x7B},
		2, {0xB4, 0xC9}
	},
	{
		2, {0x8A, 0xC8},
		3, {0xE7, 0xB0, 0xA1},
		2, {0x21, 0x7C},
		2, {0xB4, 0xCA}
	},
	{
		2, {0x8A, 0xC9},
		3, {0xE7, 0xB7, 0xA9},
		2, {0xE9, 0x7D},
		2, {0xB4, 0xCB}
	},
	{
		2, {0x8A, 0xCA},
		3, {0xE7, 0xBC, 0xB6},
		2, {0x36, 0x7F},
		2, {0xB4, 0xCC}
	},
	{
		2, {0x8A, 0xCB},
		3, {0xE7, 0xBF, 0xB0},
		2, {0xF0, 0x7F},
		2, {0xB4, 0xCD}
	},
	{
		2, {0x8A, 0xCC},
		3, {0xE8, 0x82, 0x9D},
		2, {0x9D, 0x80},
		2, {0xB4, 0xCE}
	},
	{
		2, {0x8A, 0xCD},
		3, {0xE8, 0x89, 0xA6},
		2, {0x66, 0x82},
		2, {0xB4, 0xCF}
	},
	{
		2, {0x8A, 0xCE},
		3, {0xE8, 0x8E, 0x9E},
		2, {0x9E, 0x83},
		2, {0xB4, 0xD0}
	},
	{
		2, {0x8A, 0xCF},
		3, {0xE8, 0xA6, 0xB3},
		2, {0xB3, 0x89},
		2, {0xB4, 0xD1}
	},
	{
		2, {0x8A, 0xD0},
		3, {0xE8, 0xAB, 0x8C},
		2, {0xCC, 0x8A},
		2, {0xB4, 0xD2}
	},
	{
		2, {0x8A, 0xD1},
		3, {0xE8, 0xB2, 0xAB},
		2, {0xAB, 0x8C},
		2, {0xB4, 0xD3}
	},
	{
		2, {0x8A, 0xD2},
		3, {0xE9, 0x82, 0x84},
		2, {0x84, 0x90},
		2, {0xB4, 0xD4}
	},
	{
		2, {0x8A, 0xD3},
		3, {0xE9, 0x91, 0x91},
		2, {0x51, 0x94},
		2, {0xB4, 0xD5}
	},
	{
		2, {0x8A, 0xD4},
		3, {0xE9, 0x96, 0x93},
		2, {0x93, 0x95},
		2, {0xB4, 0xD6}
	},
	{
		2, {0x8A, 0xD5},
		3, {0xE9, 0x96, 0x91},
		2, {0x91, 0x95},
		2, {0xB4, 0xD7}
	},
	{
		2, {0x8A, 0xD6},
		3, {0xE9, 0x96, 0xA2},
		2, {0xA2, 0x95},
		2, {0xB4, 0xD8}
	},
	{
		2, {0x8A, 0xD7},
		3, {0xE9, 0x99, 0xA5},
		2, {0x65, 0x96},
		2, {0xB4, 0xD9}
	},
	{
		2, {0x8A, 0xD8},
		3, {0xE9, 0x9F, 0x93},
		2, {0xD3, 0x97},
		2, {0xB4, 0xDA}
	},
	{
		2, {0x8A, 0xD9},
		3, {0xE9, 0xA4, 0xA8},
		2, {0x28, 0x99},
		2, {0xB4, 0xDB}
	},
	{
		2, {0x8A, 0xDA},
		3, {0xE8, 0x88, 0x98},
		2, {0x18, 0x82},
		2, {0xB4, 0xDC}
	},
	{
		2, {0x8A, 0xDB},
		3, {0xE4, 0xB8, 0xB8},
		2, {0x38, 0x4E},
		2, {0xB4, 0xDD}
	},
	{
		2, {0x8A, 0xDC},
		3, {0xE5, 0x90, 0xAB},
		2, {0x2B, 0x54},
		2, {0xB4, 0xDE}
	},
	{
		2, {0x8A, 0xDD},
		3, {0xE5, 0xB2, 0xB8},
		2, {0xB8, 0x5C},
		2, {0xB4, 0xDF}
	},
	{
		2, {0x8A, 0xDE},
		3, {0xE5, 0xB7, 0x8C},
		2, {0xCC, 0x5D},
		2, {0xB4, 0xE0}
	},
	{
		2, {0x8A, 0xDF},
		3, {0xE7, 0x8E, 0xA9},
		2, {0xA9, 0x73},
		2, {0xB4, 0xE1}
	},
	{
		2, {0x8A, 0xE0},
		3, {0xE7, 0x99, 0x8C},
		2, {0x4C, 0x76},
		2, {0xB4, 0xE2}
	},
	{
		2, {0x8A, 0xE1},
		3, {0xE7, 0x9C, 0xBC},
		2, {0x3C, 0x77},
		2, {0xB4, 0xE3}
	},
	{
		2, {0x8A, 0xE2},
		3, {0xE5, 0xB2, 0xA9},
		2, {0xA9, 0x5C},
		2, {0xB4, 0xE4}
	},
	{
		2, {0x8A, 0xE3},
		3, {0xE7, 0xBF, 0xAB},
		2, {0xEB, 0x7F},
		2, {0xB4, 0xE5}
	},
	{
		2, {0x8A, 0xE4},
		3, {0xE8, 0xB4, 0x8B},
		2, {0x0B, 0x8D},
		2, {0xB4, 0xE6}
	},
	{
		2, {0x8A, 0xE5},
		3, {0xE9, 0x9B, 0x81},
		2, {0xC1, 0x96},
		2, {0xB4, 0xE7}
	},
	{
		2, {0x8A, 0xE6},
		3, {0xE9, 0xA0, 0x91},
		2, {0x11, 0x98},
		2, {0xB4, 0xE8}
	},
	{
		2, {0x8A, 0xE7},
		3, {0xE9, 0xA1, 0x94},
		2, {0x54, 0x98},
		2, {0xB4, 0xE9}
	},
	{
		2, {0x8A, 0xE8},
		3, {0xE9, 0xA1, 0x98},
		2, {0x58, 0x98},
		2, {0xB4, 0xEA}
	},
	{
		2, {0x8A, 0xE9},
		3, {0xE4, 0xBC, 0x81},
		2, {0x01, 0x4F},
		2, {0xB4, 0xEB}
	},
	{
		2, {0x8A, 0xEA},
		3, {0xE4, 0xBC, 0x8E},
		2, {0x0E, 0x4F},
		2, {0xB4, 0xEC}
	},
	{
		2, {0x8A, 0xEB},
		3, {0xE5, 0x8D, 0xB1},
		2, {0x71, 0x53},
		2, {0xB4, 0xED}
	},
	{
		2, {0x8A, 0xEC},
		3, {0xE5, 0x96, 0x9C},
		2, {0x9C, 0x55},
		2, {0xB4, 0xEE}
	},
	{
		2, {0x8A, 0xED},
		3, {0xE5, 0x99, 0xA8},
		2, {0x68, 0x56},
		2, {0xB4, 0xEF}
	},
	{
		2, {0x8A, 0xEE},
		3, {0xE5, 0x9F, 0xBA},
		2, {0xFA, 0x57},
		2, {0xB4, 0xF0}
	},
	{
		2, {0x8A, 0xEF},
		3, {0xE5, 0xA5, 0x87},
		2, {0x47, 0x59},
		2, {0xB4, 0xF1}
	},
	{
		2, {0x8A, 0xF0},
		3, {0xE5, 0xAC, 0x89},
		2, {0x09, 0x5B},
		2, {0xB4, 0xF2}
	},
	{
		2, {0x8A, 0xF1},
		3, {0xE5, 0xAF, 0x84},
		2, {0xC4, 0x5B},
		2, {0xB4, 0xF3}
	},
	{
		2, {0x8A, 0xF2},
		3, {0xE5, 0xB2, 0x90},
		2, {0x90, 0x5C},
		2, {0xB4, 0xF4}
	},
	{
		2, {0x8A, 0xF3},
		3, {0xE5, 0xB8, 0x8C},
		2, {0x0C, 0x5E},
		2, {0xB4, 0xF5}
	},
	{
		2, {0x8A, 0xF4},
		3, {0xE5, 0xB9, 0xBE},
		2, {0x7E, 0x5E},
		2, {0xB4, 0xF6}
	},
	{
		2, {0x8A, 0xF5},
		3, {0xE5, 0xBF, 0x8C},
		2, {0xCC, 0x5F},
		2, {0xB4, 0xF7}
	},
	{
		2, {0x8A, 0xF6},
		3, {0xE6, 0x8F, 0xAE},
		2, {0xEE, 0x63},
		2, {0xB4, 0xF8}
	},
	{
		2, {0x8A, 0xF7},
		3, {0xE6, 0x9C, 0xBA},
		2, {0x3A, 0x67},
		2, {0xB4, 0xF9}
	},
	{
		2, {0x8A, 0xF8},
		3, {0xE6, 0x97, 0x97},
		2, {0xD7, 0x65},
		2, {0xB4, 0xFA}
	},
	{
		2, {0x8A, 0xF9},
		3, {0xE6, 0x97, 0xA2},
		2, {0xE2, 0x65},
		2, {0xB4, 0xFB}
	},
	{
		2, {0x8A, 0xFA},
		3, {0xE6, 0x9C, 0x9F},
		2, {0x1F, 0x67},
		2, {0xB4, 0xFC}
	},
	{
		2, {0x8A, 0xFB},
		3, {0xE6, 0xA3, 0x8B},
		2, {0xCB, 0x68},
		2, {0xB4, 0xFD}
	},
	{
		2, {0x8A, 0xFC},
		3, {0xE6, 0xA3, 0x84},
		2, {0xC4, 0x68},
		2, {0xB4, 0xFE}
	},
	{
		2, {0x8B, 0x40},
		3, {0xE6, 0xA9, 0x9F},
		2, {0x5F, 0x6A},
		2, {0xB5, 0xA1}
	},
	{
		2, {0x8B, 0x41},
		3, {0xE5, 0xB8, 0xB0},
		2, {0x30, 0x5E},
		2, {0xB5, 0xA2}
	},
	{
		2, {0x8B, 0x42},
		3, {0xE6, 0xAF, 0x85},
		2, {0xC5, 0x6B},
		2, {0xB5, 0xA3}
	},
	{
		2, {0x8B, 0x43},
		3, {0xE6, 0xB0, 0x97},
		2, {0x17, 0x6C},
		2, {0xB5, 0xA4}
	},
	{
		2, {0x8B, 0x44},
		3, {0xE6, 0xB1, 0xBD},
		2, {0x7D, 0x6C},
		2, {0xB5, 0xA5}
	},
	{
		2, {0x8B, 0x45},
		3, {0xE7, 0x95, 0xBF},
		2, {0x7F, 0x75},
		2, {0xB5, 0xA6}
	},
	{
		2, {0x8B, 0x46},
		3, {0xE7, 0xA5, 0x88},
		2, {0x48, 0x79},
		2, {0xB5, 0xA7}
	},
	{
		2, {0x8B, 0x47},
		3, {0xE5, 0xAD, 0xA3},
		2, {0x63, 0x5B},
		2, {0xB5, 0xA8}
	},
	{
		2, {0x8B, 0x48},
		3, {0xE7, 0xA8, 0x80},
		2, {0x00, 0x7A},
		2, {0xB5, 0xA9}
	},
	{
		2, {0x8B, 0x49},
		3, {0xE7, 0xB4, 0x80},
		2, {0x00, 0x7D},
		2, {0xB5, 0xAA}
	},
	{
		2, {0x8B, 0x4A},
		3, {0xE5, 0xBE, 0xBD},
		2, {0xBD, 0x5F},
		2, {0xB5, 0xAB}
	},
	{
		2, {0x8B, 0x4B},
		3, {0xE8, 0xA6, 0x8F},
		2, {0x8F, 0x89},
		2, {0xB5, 0xAC}
	},
	{
		2, {0x8B, 0x4C},
		3, {0xE8, 0xA8, 0x98},
		2, {0x18, 0x8A},
		2, {0xB5, 0xAD}
	},
	{
		2, {0x8B, 0x4D},
		3, {0xE8, 0xB2, 0xB4},
		2, {0xB4, 0x8C},
		2, {0xB5, 0xAE}
	},
	{
		2, {0x8B, 0x4E},
		3, {0xE8, 0xB5, 0xB7},
		2, {0x77, 0x8D},
		2, {0xB5, 0xAF}
	},
	{
		2, {0x8B, 0x4F},
		3, {0xE8, 0xBB, 0x8C},
		2, {0xCC, 0x8E},
		2, {0xB5, 0xB0}
	},
	{
		2, {0x8B, 0x50},
		3, {0xE8, 0xBC, 0x9D},
		2, {0x1D, 0x8F},
		2, {0xB5, 0xB1}
	},
	{
		2, {0x8B, 0x51},
		3, {0xE9, 0xA3, 0xA2},
		2, {0xE2, 0x98},
		2, {0xB5, 0xB2}
	},
	{
		2, {0x8B, 0x52},
		3, {0xE9, 0xA8, 0x8E},
		2, {0x0E, 0x9A},
		2, {0xB5, 0xB3}
	},
	{
		2, {0x8B, 0x53},
		3, {0xE9, 0xAC, 0xBC},
		2, {0x3C, 0x9B},
		2, {0xB5, 0xB4}
	},
	{
		2, {0x8B, 0x54},
		3, {0xE4, 0xBA, 0x80},
		2, {0x80, 0x4E},
		2, {0xB5, 0xB5}
	},
	{
		2, {0x8B, 0x55},
		3, {0xE5, 0x81, 0xBD},
		2, {0x7D, 0x50},
		2, {0xB5, 0xB6}
	},
	{
		2, {0x8B, 0x56},
		3, {0xE5, 0x84, 0x80},
		2, {0x00, 0x51},
		2, {0xB5, 0xB7}
	},
	{
		2, {0x8B, 0x57},
		3, {0xE5, 0xA6, 0x93},
		2, {0x93, 0x59},
		2, {0xB5, 0xB8}
	},
	{
		2, {0x8B, 0x58},
		3, {0xE5, 0xAE, 0x9C},
		2, {0x9C, 0x5B},
		2, {0xB5, 0xB9}
	},
	{
		2, {0x8B, 0x59},
		3, {0xE6, 0x88, 0xAF},
		2, {0x2F, 0x62},
		2, {0xB5, 0xBA}
	},
	{
		2, {0x8B, 0x5A},
		3, {0xE6, 0x8A, 0x80},
		2, {0x80, 0x62},
		2, {0xB5, 0xBB}
	},
	{
		2, {0x8B, 0x5B},
		3, {0xE6, 0x93, 0xAC},
		2, {0xEC, 0x64},
		2, {0xB5, 0xBC}
	},
	{
		2, {0x8B, 0x5C},
		3, {0xE6, 0xAC, 0xBA},
		2, {0x3A, 0x6B},
		2, {0xB5, 0xBD}
	},
	{
		2, {0x8B, 0x5D},
		3, {0xE7, 0x8A, 0xA0},
		2, {0xA0, 0x72},
		2, {0xB5, 0xBE}
	},
	{
		2, {0x8B, 0x5E},
		3, {0xE7, 0x96, 0x91},
		2, {0x91, 0x75},
		2, {0xB5, 0xBF}
	},
	{
		2, {0x8B, 0x5F},
		3, {0xE7, 0xA5, 0x87},
		2, {0x47, 0x79},
		2, {0xB5, 0xC0}
	},
	{
		2, {0x8B, 0x60},
		3, {0xE7, 0xBE, 0xA9},
		2, {0xA9, 0x7F},
		2, {0xB5, 0xC1}
	},
	{
		2, {0x8B, 0x61},
		3, {0xE8, 0x9F, 0xBB},
		2, {0xFB, 0x87},
		2, {0xB5, 0xC2}
	},
	{
		2, {0x8B, 0x62},
		3, {0xE8, 0xAA, 0xBC},
		2, {0xBC, 0x8A},
		2, {0xB5, 0xC3}
	},
	{
		2, {0x8B, 0x63},
		3, {0xE8, 0xAD, 0xB0},
		2, {0x70, 0x8B},
		2, {0xB5, 0xC4}
	},
	{
		2, {0x8B, 0x64},
		3, {0xE6, 0x8E, 0xAC},
		2, {0xAC, 0x63},
		2, {0xB5, 0xC5}
	},
	{
		2, {0x8B, 0x65},
		3, {0xE8, 0x8F, 0x8A},
		2, {0xCA, 0x83},
		2, {0xB5, 0xC6}
	},
	{
		2, {0x8B, 0x66},
		3, {0xE9, 0x9E, 0xA0},
		2, {0xA0, 0x97},
		2, {0xB5, 0xC7}
	},
	{
		2, {0x8B, 0x67},
		3, {0xE5, 0x90, 0x89},
		2, {0x09, 0x54},
		2, {0xB5, 0xC8}
	},
	{
		2, {0x8B, 0x68},
		3, {0xE5, 0x90, 0x83},
		2, {0x03, 0x54},
		2, {0xB5, 0xC9}
	},
	{
		2, {0x8B, 0x69},
		3, {0xE5, 0x96, 0xAB},
		2, {0xAB, 0x55},
		2, {0xB5, 0xCA}
	},
	{
		2, {0x8B, 0x6A},
		3, {0xE6, 0xA1, 0x94},
		2, {0x54, 0x68},
		2, {0xB5, 0xCB}
	},
	{
		2, {0x8B, 0x6B},
		3, {0xE6, 0xA9, 0x98},
		2, {0x58, 0x6A},
		2, {0xB5, 0xCC}
	},
	{
		2, {0x8B, 0x6C},
		3, {0xE8, 0xA9, 0xB0},
		2, {0x70, 0x8A},
		2, {0xB5, 0xCD}
	},
	{
		2, {0x8B, 0x6D},
		3, {0xE7, 0xA0, 0xA7},
		2, {0x27, 0x78},
		2, {0xB5, 0xCE}
	},
	{
		2, {0x8B, 0x6E},
		3, {0xE6, 0x9D, 0xB5},
		2, {0x75, 0x67},
		2, {0xB5, 0xCF}
	},
	{
		2, {0x8B, 0x6F},
		3, {0xE9, 0xBB, 0x8D},
		2, {0xCD, 0x9E},
		2, {0xB5, 0xD0}
	},
	{
		2, {0x8B, 0x70},
		3, {0xE5, 0x8D, 0xB4},
		2, {0x74, 0x53},
		2, {0xB5, 0xD1}
	},
	{
		2, {0x8B, 0x71},
		3, {0xE5, 0xAE, 0xA2},
		2, {0xA2, 0x5B},
		2, {0xB5, 0xD2}
	},
	{
		2, {0x8B, 0x72},
		3, {0xE8, 0x84, 0x9A},
		2, {0x1A, 0x81},
		2, {0xB5, 0xD3}
	},
	{
		2, {0x8B, 0x73},
		3, {0xE8, 0x99, 0x90},
		2, {0x50, 0x86},
		2, {0xB5, 0xD4}
	},
	{
		2, {0x8B, 0x74},
		3, {0xE9, 0x80, 0x86},
		2, {0x06, 0x90},
		2, {0xB5, 0xD5}
	},
	{
		2, {0x8B, 0x75},
		3, {0xE4, 0xB8, 0x98},
		2, {0x18, 0x4E},
		2, {0xB5, 0xD6}
	},
	{
		2, {0x8B, 0x76},
		3, {0xE4, 0xB9, 0x85},
		2, {0x45, 0x4E},
		2, {0xB5, 0xD7}
	},
	{
		2, {0x8B, 0x77},
		3, {0xE4, 0xBB, 0x87},
		2, {0xC7, 0x4E},
		2, {0xB5, 0xD8}
	},
	{
		2, {0x8B, 0x78},
		3, {0xE4, 0xBC, 0x91},
		2, {0x11, 0x4F},
		2, {0xB5, 0xD9}
	},
	{
		2, {0x8B, 0x79},
		3, {0xE5, 0x8F, 0x8A},
		2, {0xCA, 0x53},
		2, {0xB5, 0xDA}
	},
	{
		2, {0x8B, 0x7A},
		3, {0xE5, 0x90, 0xB8},
		2, {0x38, 0x54},
		2, {0xB5, 0xDB}
	},
	{
		2, {0x8B, 0x7B},
		3, {0xE5, 0xAE, 0xAE},
		2, {0xAE, 0x5B},
		2, {0xB5, 0xDC}
	},
	{
		2, {0x8B, 0x7C},
		3, {0xE5, 0xBC, 0x93},
		2, {0x13, 0x5F},
		2, {0xB5, 0xDD}
	},
	{
		2, {0x8B, 0x7D},
		3, {0xE6, 0x80, 0xA5},
		2, {0x25, 0x60},
		2, {0xB5, 0xDE}
	},
	{
		2, {0x8B, 0x7E},
		3, {0xE6, 0x95, 0x91},
		2, {0x51, 0x65},
		2, {0xB5, 0xDF}
	},
	{
		2, {0x8B, 0x80},
		3, {0xE6, 0x9C, 0xBD},
		2, {0x3D, 0x67},
		2, {0xB5, 0xE0}
	},
	{
		2, {0x8B, 0x81},
		3, {0xE6, 0xB1, 0x82},
		2, {0x42, 0x6C},
		2, {0xB5, 0xE1}
	},
	{
		2, {0x8B, 0x82},
		3, {0xE6, 0xB1, 0xB2},
		2, {0x72, 0x6C},
		2, {0xB5, 0xE2}
	},
	{
		2, {0x8B, 0x83},
		3, {0xE6, 0xB3, 0xA3},
		2, {0xE3, 0x6C},
		2, {0xB5, 0xE3}
	},
	{
		2, {0x8B, 0x84},
		3, {0xE7, 0x81, 0xB8},
		2, {0x78, 0x70},
		2, {0xB5, 0xE4}
	},
	{
		2, {0x8B, 0x85},
		3, {0xE7, 0x90, 0x83},
		2, {0x03, 0x74},
		2, {0xB5, 0xE5}
	},
	{
		2, {0x8B, 0x86},
		3, {0xE7, 0xA9, 0xB6},
		2, {0x76, 0x7A},
		2, {0xB5, 0xE6}
	},
	{
		2, {0x8B, 0x87},
		3, {0xE7, 0xAA, 0xAE},
		2, {0xAE, 0x7A},
		2, {0xB5, 0xE7}
	},
	{
		2, {0x8B, 0x88},
		3, {0xE7, 0xAC, 0x88},
		2, {0x08, 0x7B},
		2, {0xB5, 0xE8}
	},
	{
		2, {0x8B, 0x89},
		3, {0xE7, 0xB4, 0x9A},
		2, {0x1A, 0x7D},
		2, {0xB5, 0xE9}
	},
	{
		2, {0x8B, 0x8A},
		3, {0xE7, 0xB3, 0xBE},
		2, {0xFE, 0x7C},
		2, {0xB5, 0xEA}
	},
	{
		2, {0x8B, 0x8B},
		3, {0xE7, 0xB5, 0xA6},
		2, {0x66, 0x7D},
		2, {0xB5, 0xEB}
	},
	{
		2, {0x8B, 0x8C},
		3, {0xE6, 0x97, 0xA7},
		2, {0xE7, 0x65},
		2, {0xB5, 0xEC}
	},
	{
		2, {0x8B, 0x8D},
		3, {0xE7, 0x89, 0x9B},
		2, {0x5B, 0x72},
		2, {0xB5, 0xED}
	},
	{
		2, {0x8B, 0x8E},
		3, {0xE5, 0x8E, 0xBB},
		2, {0xBB, 0x53},
		2, {0xB5, 0xEE}
	},
	{
		2, {0x8B, 0x8F},
		3, {0xE5, 0xB1, 0x85},
		2, {0x45, 0x5C},
		2, {0xB5, 0xEF}
	},
	{
		2, {0x8B, 0x90},
		3, {0xE5, 0xB7, 0xA8},
		2, {0xE8, 0x5D},
		2, {0xB5, 0xF0}
	},
	{
		2, {0x8B, 0x91},
		3, {0xE6, 0x8B, 0x92},
		2, {0xD2, 0x62},
		2, {0xB5, 0xF1}
	},
	{
		2, {0x8B, 0x92},
		3, {0xE6, 0x8B, 0xA0},
		2, {0xE0, 0x62},
		2, {0xB5, 0xF2}
	},
	{
		2, {0x8B, 0x93},
		3, {0xE6, 0x8C, 0x99},
		2, {0x19, 0x63},
		2, {0xB5, 0xF3}
	},
	{
		2, {0x8B, 0x94},
		3, {0xE6, 0xB8, 0xA0},
		2, {0x20, 0x6E},
		2, {0xB5, 0xF4}
	},
	{
		2, {0x8B, 0x95},
		3, {0xE8, 0x99, 0x9A},
		2, {0x5A, 0x86},
		2, {0xB5, 0xF5}
	},
	{
		2, {0x8B, 0x96},
		3, {0xE8, 0xA8, 0xB1},
		2, {0x31, 0x8A},
		2, {0xB5, 0xF6}
	},
	{
		2, {0x8B, 0x97},
		3, {0xE8, 0xB7, 0x9D},
		2, {0xDD, 0x8D},
		2, {0xB5, 0xF7}
	},
	{
		2, {0x8B, 0x98},
		3, {0xE9, 0x8B, 0xB8},
		2, {0xF8, 0x92},
		2, {0xB5, 0xF8}
	},
	{
		2, {0x8B, 0x99},
		3, {0xE6, 0xBC, 0x81},
		2, {0x01, 0x6F},
		2, {0xB5, 0xF9}
	},
	{
		2, {0x8B, 0x9A},
		3, {0xE7, 0xA6, 0xA6},
		2, {0xA6, 0x79},
		2, {0xB5, 0xFA}
	},
	{
		2, {0x8B, 0x9B},
		3, {0xE9, 0xAD, 0x9A},
		2, {0x5A, 0x9B},
		2, {0xB5, 0xFB}
	},
	{
		2, {0x8B, 0x9C},
		3, {0xE4, 0xBA, 0xA8},
		2, {0xA8, 0x4E},
		2, {0xB5, 0xFC}
	},
	{
		2, {0x8B, 0x9D},
		3, {0xE4, 0xBA, 0xAB},
		2, {0xAB, 0x4E},
		2, {0xB5, 0xFD}
	},
	{
		2, {0x8B, 0x9E},
		3, {0xE4, 0xBA, 0xAC},
		2, {0xAC, 0x4E},
		2, {0xB5, 0xFE}
	},
	{
		2, {0x8B, 0x9F},
		3, {0xE4, 0xBE, 0x9B},
		2, {0x9B, 0x4F},
		2, {0xB6, 0xA1}
	},
	{
		2, {0x8B, 0xA0},
		3, {0xE4, 0xBE, 0xA0},
		2, {0xA0, 0x4F},
		2, {0xB6, 0xA2}
	},
	{
		2, {0x8B, 0xA1},
		3, {0xE5, 0x83, 0x91},
		2, {0xD1, 0x50},
		2, {0xB6, 0xA3}
	},
	{
		2, {0x8B, 0xA2},
		3, {0xE5, 0x85, 0x87},
		2, {0x47, 0x51},
		2, {0xB6, 0xA4}
	},
	{
		2, {0x8B, 0xA3},
		3, {0xE7, 0xAB, 0xB6},
		2, {0xF6, 0x7A},
		2, {0xB6, 0xA5}
	},
	{
		2, {0x8B, 0xA4},
		3, {0xE5, 0x85, 0xB1},
		2, {0x71, 0x51},
		2, {0xB6, 0xA6}
	},
	{
		2, {0x8B, 0xA5},
		3, {0xE5, 0x87, 0xB6},
		2, {0xF6, 0x51},
		2, {0xB6, 0xA7}
	},
	{
		2, {0x8B, 0xA6},
		3, {0xE5, 0x8D, 0x94},
		2, {0x54, 0x53},
		2, {0xB6, 0xA8}
	},
	{
		2, {0x8B, 0xA7},
		3, {0xE5, 0x8C, 0xA1},
		2, {0x21, 0x53},
		2, {0xB6, 0xA9}
	},
	{
		2, {0x8B, 0xA8},
		3, {0xE5, 0x8D, 0xBF},
		2, {0x7F, 0x53},
		2, {0xB6, 0xAA}
	},
	{
		2, {0x8B, 0xA9},
		3, {0xE5, 0x8F, 0xAB},
		2, {0xEB, 0x53},
		2, {0xB6, 0xAB}
	},
	{
		2, {0x8B, 0xAA},
		3, {0xE5, 0x96, 0xAC},
		2, {0xAC, 0x55},
		2, {0xB6, 0xAC}
	},
	{
		2, {0x8B, 0xAB},
		3, {0xE5, 0xA2, 0x83},
		2, {0x83, 0x58},
		2, {0xB6, 0xAD}
	},
	{
		2, {0x8B, 0xAC},
		3, {0xE5, 0xB3, 0xA1},
		2, {0xE1, 0x5C},
		2, {0xB6, 0xAE}
	},
	{
		2, {0x8B, 0xAD},
		3, {0xE5, 0xBC, 0xB7},
		2, {0x37, 0x5F},
		2, {0xB6, 0xAF}
	},
	{
		2, {0x8B, 0xAE},
		3, {0xE5, 0xBD, 0x8A},
		2, {0x4A, 0x5F},
		2, {0xB6, 0xB0}
	},
	{
		2, {0x8B, 0xAF},
		3, {0xE6, 0x80, 0xAF},
		2, {0x2F, 0x60},
		2, {0xB6, 0xB1}
	},
	{
		2, {0x8B, 0xB0},
		3, {0xE6, 0x81, 0x90},
		2, {0x50, 0x60},
		2, {0xB6, 0xB2}
	},
	{
		2, {0x8B, 0xB1},
		3, {0xE6, 0x81, 0xAD},
		2, {0x6D, 0x60},
		2, {0xB6, 0xB3}
	},
	{
		2, {0x8B, 0xB2},
		3, {0xE6, 0x8C, 0x9F},
		2, {0x1F, 0x63},
		2, {0xB6, 0xB4}
	},
	{
		2, {0x8B, 0xB3},
		3, {0xE6, 0x95, 0x99},
		2, {0x59, 0x65},
		2, {0xB6, 0xB5}
	},
	{
		2, {0x8B, 0xB4},
		3, {0xE6, 0xA9, 0x8B},
		2, {0x4B, 0x6A},
		2, {0xB6, 0xB6}
	},
	{
		2, {0x8B, 0xB5},
		3, {0xE6, 0xB3, 0x81},
		2, {0xC1, 0x6C},
		2, {0xB6, 0xB7}
	},
	{
		2, {0x8B, 0xB6},
		3, {0xE7, 0x8B, 0x82},
		2, {0xC2, 0x72},
		2, {0xB6, 0xB8}
	},
	{
		2, {0x8B, 0xB7},
		3, {0xE7, 0x8B, 0xAD},
		2, {0xED, 0x72},
		2, {0xB6, 0xB9}
	},
	{
		2, {0x8B, 0xB8},
		3, {0xE7, 0x9F, 0xAF},
		2, {0xEF, 0x77},
		2, {0xB6, 0xBA}
	},
	{
		2, {0x8B, 0xB9},
		3, {0xE8, 0x83, 0xB8},
		2, {0xF8, 0x80},
		2, {0xB6, 0xBB}
	},
	{
		2, {0x8B, 0xBA},
		3, {0xE8, 0x84, 0x85},
		2, {0x05, 0x81},
		2, {0xB6, 0xBC}
	},
	{
		2, {0x8B, 0xBB},
		3, {0xE8, 0x88, 0x88},
		2, {0x08, 0x82},
		2, {0xB6, 0xBD}
	},
	{
		2, {0x8B, 0xBC},
		3, {0xE8, 0x95, 0x8E},
		2, {0x4E, 0x85},
		2, {0xB6, 0xBE}
	},
	{
		2, {0x8B, 0xBD},
		3, {0xE9, 0x83, 0xB7},
		2, {0xF7, 0x90},
		2, {0xB6, 0xBF}
	},
	{
		2, {0x8B, 0xBE},
		3, {0xE9, 0x8F, 0xA1},
		2, {0xE1, 0x93},
		2, {0xB6, 0xC0}
	},
	{
		2, {0x8B, 0xBF},
		3, {0xE9, 0x9F, 0xBF},
		2, {0xFF, 0x97},
		2, {0xB6, 0xC1}
	},
	{
		2, {0x8B, 0xC0},
		3, {0xE9, 0xA5, 0x97},
		2, {0x57, 0x99},
		2, {0xB6, 0xC2}
	},
	{
		2, {0x8B, 0xC1},
		3, {0xE9, 0xA9, 0x9A},
		2, {0x5A, 0x9A},
		2, {0xB6, 0xC3}
	},
	{
		2, {0x8B, 0xC2},
		3, {0xE4, 0xBB, 0xB0},
		2, {0xF0, 0x4E},
		2, {0xB6, 0xC4}
	},
	{
		2, {0x8B, 0xC3},
		3, {0xE5, 0x87, 0x9D},
		2, {0xDD, 0x51},
		2, {0xB6, 0xC5}
	},
	{
		2, {0x8B, 0xC4},
		3, {0xE5, 0xB0, 0xAD},
		2, {0x2D, 0x5C},
		2, {0xB6, 0xC6}
	},
	{
		2, {0x8B, 0xC5},
		3, {0xE6, 0x9A, 0x81},
		2, {0x81, 0x66},
		2, {0xB6, 0xC7}
	},
	{
		2, {0x8B, 0xC6},
		3, {0xE6, 0xA5, 0xAD},
		2, {0x6D, 0x69},
		2, {0xB6, 0xC8}
	},
	{
		2, {0x8B, 0xC7},
		3, {0xE5, 0xB1, 0x80},
		2, {0x40, 0x5C},
		2, {0xB6, 0xC9}
	},
	{
		2, {0x8B, 0xC8},
		3, {0xE6, 0x9B, 0xB2},
		2, {0xF2, 0x66},
		2, {0xB6, 0xCA}
	},
	{
		2, {0x8B, 0xC9},
		3, {0xE6, 0xA5, 0xB5},
		2, {0x75, 0x69},
		2, {0xB6, 0xCB}
	},
	{
		2, {0x8B, 0xCA},
		3, {0xE7, 0x8E, 0x89},
		2, {0x89, 0x73},
		2, {0xB6, 0xCC}
	},
	{
		2, {0x8B, 0xCB},
		3, {0xE6, 0xA1, 0x90},
		2, {0x50, 0x68},
		2, {0xB6, 0xCD}
	},
	{
		2, {0x8B, 0xCC},
		3, {0xE7, 0xB2, 0x81},
		2, {0x81, 0x7C},
		2, {0xB6, 0xCE}
	},
	{
		2, {0x8B, 0xCD},
		3, {0xE5, 0x83, 0x85},
		2, {0xC5, 0x50},
		2, {0xB6, 0xCF}
	},
	{
		2, {0x8B, 0xCE},
		3, {0xE5, 0x8B, 0xA4},
		2, {0xE4, 0x52},
		2, {0xB6, 0xD0}
	},
	{
		2, {0x8B, 0xCF},
		3, {0xE5, 0x9D, 0x87},
		2, {0x47, 0x57},
		2, {0xB6, 0xD1}
	},
	{
		2, {0x8B, 0xD0},
		3, {0xE5, 0xB7, 0xBE},
		2, {0xFE, 0x5D},
		2, {0xB6, 0xD2}
	},
	{
		2, {0x8B, 0xD1},
		3, {0xE9, 0x8C, 0xA6},
		2, {0x26, 0x93},
		2, {0xB6, 0xD3}
	},
	{
		2, {0x8B, 0xD2},
		3, {0xE6, 0x96, 0xA4},
		2, {0xA4, 0x65},
		2, {0xB6, 0xD4}
	},
	{
		2, {0x8B, 0xD3},
		3, {0xE6, 0xAC, 0xA3},
		2, {0x23, 0x6B},
		2, {0xB6, 0xD5}
	},
	{
		2, {0x8B, 0xD4},
		3, {0xE6, 0xAC, 0xBD},
		2, {0x3D, 0x6B},
		2, {0xB6, 0xD6}
	},
	{
		2, {0x8B, 0xD5},
		3, {0xE7, 0x90, 0xB4},
		2, {0x34, 0x74},
		2, {0xB6, 0xD7}
	},
	{
		2, {0x8B, 0xD6},
		3, {0xE7, 0xA6, 0x81},
		2, {0x81, 0x79},
		2, {0xB6, 0xD8}
	},
	{
		2, {0x8B, 0xD7},
		3, {0xE7, 0xA6, 0xBD},
		2, {0xBD, 0x79},
		2, {0xB6, 0xD9}
	},
	{
		2, {0x8B, 0xD8},
		3, {0xE7, 0xAD, 0x8B},
		2, {0x4B, 0x7B},
		2, {0xB6, 0xDA}
	},
	{
		2, {0x8B, 0xD9},
		3, {0xE7, 0xB7, 0x8A},
		2, {0xCA, 0x7D},
		2, {0xB6, 0xDB}
	},
	{
		2, {0x8B, 0xDA},
		3, {0xE8, 0x8A, 0xB9},
		2, {0xB9, 0x82},
		2, {0xB6, 0xDC}
	},
	{
		2, {0x8B, 0xDB},
		3, {0xE8, 0x8F, 0x8C},
		2, {0xCC, 0x83},
		2, {0xB6, 0xDD}
	},
	{
		2, {0x8B, 0xDC},
		3, {0xE8, 0xA1, 0xBF},
		2, {0x7F, 0x88},
		2, {0xB6, 0xDE}
	},
	{
		2, {0x8B, 0xDD},
		3, {0xE8, 0xA5, 0x9F},
		2, {0x5F, 0x89},
		2, {0xB6, 0xDF}
	},
	{
		2, {0x8B, 0xDE},
		3, {0xE8, 0xAC, 0xB9},
		2, {0x39, 0x8B},
		2, {0xB6, 0xE0}
	},
	{
		2, {0x8B, 0xDF},
		3, {0xE8, 0xBF, 0x91},
		2, {0xD1, 0x8F},
		2, {0xB6, 0xE1}
	},
	{
		2, {0x8B, 0xE0},
		3, {0xE9, 0x87, 0x91},
		2, {0xD1, 0x91},
		2, {0xB6, 0xE2}
	},
	{
		2, {0x8B, 0xE1},
		3, {0xE5, 0x90, 0x9F},
		2, {0x1F, 0x54},
		2, {0xB6, 0xE3}
	},
	{
		2, {0x8B, 0xE2},
		3, {0xE9, 0x8A, 0x80},
		2, {0x80, 0x92},
		2, {0xB6, 0xE4}
	},
	{
		2, {0x8B, 0xE3},
		3, {0xE4, 0xB9, 0x9D},
		2, {0x5D, 0x4E},
		2, {0xB6, 0xE5}
	},
	{
		2, {0x8B, 0xE4},
		3, {0xE5, 0x80, 0xB6},
		2, {0x36, 0x50},
		2, {0xB6, 0xE6}
	},
	{
		2, {0x8B, 0xE5},
		3, {0xE5, 0x8F, 0xA5},
		2, {0xE5, 0x53},
		2, {0xB6, 0xE7}
	},
	{
		2, {0x8B, 0xE6},
		3, {0xE5, 0x8C, 0xBA},
		2, {0x3A, 0x53},
		2, {0xB6, 0xE8}
	},
	{
		2, {0x8B, 0xE7},
		3, {0xE7, 0x8B, 0x97},
		2, {0xD7, 0x72},
		2, {0xB6, 0xE9}
	},
	{
		2, {0x8B, 0xE8},
		3, {0xE7, 0x8E, 0x96},
		2, {0x96, 0x73},
		2, {0xB6, 0xEA}
	},
	{
		2, {0x8B, 0xE9},
		3, {0xE7, 0x9F, 0xA9},
		2, {0xE9, 0x77},
		2, {0xB6, 0xEB}
	},
	{
		2, {0x8B, 0xEA},
		3, {0xE8, 0x8B, 0xA6},
		2, {0xE6, 0x82},
		2, {0xB6, 0xEC}
	},
	{
		2, {0x8B, 0xEB},
		3, {0xE8, 0xBA, 0xAF},
		2, {0xAF, 0x8E},
		2, {0xB6, 0xED}
	},
	{
		2, {0x8B, 0xEC},
		3, {0xE9, 0xA7, 0x86},
		2, {0xC6, 0x99},
		2, {0xB6, 0xEE}
	},
	{
		2, {0x8B, 0xED},
		3, {0xE9, 0xA7, 0x88},
		2, {0xC8, 0x99},
		2, {0xB6, 0xEF}
	},
	{
		2, {0x8B, 0xEE},
		3, {0xE9, 0xA7, 0x92},
		2, {0xD2, 0x99},
		2, {0xB6, 0xF0}
	},
	{
		2, {0x8B, 0xEF},
		3, {0xE5, 0x85, 0xB7},
		2, {0x77, 0x51},
		2, {0xB6, 0xF1}
	},
	{
		2, {0x8B, 0xF0},
		3, {0xE6, 0x84, 0x9A},
		2, {0x1A, 0x61},
		2, {0xB6, 0xF2}
	},
	{
		2, {0x8B, 0xF1},
		3, {0xE8, 0x99, 0x9E},
		2, {0x5E, 0x86},
		2, {0xB6, 0xF3}
	},
	{
		2, {0x8B, 0xF2},
		3, {0xE5, 0x96, 0xB0},
		2, {0xB0, 0x55},
		2, {0xB6, 0xF4}
	},
	{
		2, {0x8B, 0xF3},
		3, {0xE7, 0xA9, 0xBA},
		2, {0x7A, 0x7A},
		2, {0xB6, 0xF5}
	},
	{
		2, {0x8B, 0xF4},
		3, {0xE5, 0x81, 0xB6},
		2, {0x76, 0x50},
		2, {0xB6, 0xF6}
	},
	{
		2, {0x8B, 0xF5},
		3, {0xE5, 0xAF, 0x93},
		2, {0xD3, 0x5B},
		2, {0xB6, 0xF7}
	},
	{
		2, {0x8B, 0xF6},
		3, {0xE9, 0x81, 0x87},
		2, {0x47, 0x90},
		2, {0xB6, 0xF8}
	},
	{
		2, {0x8B, 0xF7},
		3, {0xE9, 0x9A, 0x85},
		2, {0x85, 0x96},
		2, {0xB6, 0xF9}
	},
	{
		2, {0x8B, 0xF8},
		3, {0xE4, 0xB8, 0xB2},
		2, {0x32, 0x4E},
		2, {0xB6, 0xFA}
	},
	{
		2, {0x8B, 0xF9},
		3, {0xE6, 0xAB, 0x9B},
		2, {0xDB, 0x6A},
		2, {0xB6, 0xFB}
	},
	{
		2, {0x8B, 0xFA},
		3, {0xE9, 0x87, 0xA7},
		2, {0xE7, 0x91},
		2, {0xB6, 0xFC}
	},
	{
		2, {0x8B, 0xFB},
		3, {0xE5, 0xB1, 0x91},
		2, {0x51, 0x5C},
		2, {0xB6, 0xFD}
	},
	{
		2, {0x8B, 0xFC},
		3, {0xE5, 0xB1, 0x88},
		2, {0x48, 0x5C},
		2, {0xB6, 0xFE}
	},
	{
		2, {0x8C, 0x40},
		3, {0xE6, 0x8E, 0x98},
		2, {0x98, 0x63},
		2, {0xB7, 0xA1}
	},
	{
		2, {0x8C, 0x41},
		3, {0xE7, 0xAA, 0x9F},
		2, {0x9F, 0x7A},
		2, {0xB7, 0xA2}
	},
	{
		2, {0x8C, 0x42},
		3, {0xE6, 0xB2, 0x93},
		2, {0x93, 0x6C},
		2, {0xB7, 0xA3}
	},
	{
		2, {0x8C, 0x43},
		3, {0xE9, 0x9D, 0xB4},
		2, {0x74, 0x97},
		2, {0xB7, 0xA4}
	},
	{
		2, {0x8C, 0x44},
		3, {0xE8, 0xBD, 0xA1},
		2, {0x61, 0x8F},
		2, {0xB7, 0xA5}
	},
	{
		2, {0x8C, 0x45},
		3, {0xE7, 0xAA, 0xAA},
		2, {0xAA, 0x7A},
		2, {0xB7, 0xA6}
	},
	{
		2, {0x8C, 0x46},
		3, {0xE7, 0x86, 0x8A},
		2, {0x8A, 0x71},
		2, {0xB7, 0xA7}
	},
	{
		2, {0x8C, 0x47},
		3, {0xE9, 0x9A, 0x88},
		2, {0x88, 0x96},
		2, {0xB7, 0xA8}
	},
	{
		2, {0x8C, 0x48},
		3, {0xE7, 0xB2, 0x82},
		2, {0x82, 0x7C},
		2, {0xB7, 0xA9}
	},
	{
		2, {0x8C, 0x49},
		3, {0xE6, 0xA0, 0x97},
		2, {0x17, 0x68},
		2, {0xB7, 0xAA}
	},
	{
		2, {0x8C, 0x4A},
		3, {0xE7, 0xB9, 0xB0},
		2, {0x70, 0x7E},
		2, {0xB7, 0xAB}
	},
	{
		2, {0x8C, 0x4B},
		3, {0xE6, 0xA1, 0x91},
		2, {0x51, 0x68},
		2, {0xB7, 0xAC}
	},
	{
		2, {0x8C, 0x4C},
		3, {0xE9, 0x8D, 0xAC},
		2, {0x6C, 0x93},
		2, {0xB7, 0xAD}
	},
	{
		2, {0x8C, 0x4D},
		3, {0xE5, 0x8B, 0xB2},
		2, {0xF2, 0x52},
		2, {0xB7, 0xAE}
	},
	{
		2, {0x8C, 0x4E},
		3, {0xE5, 0x90, 0x9B},
		2, {0x1B, 0x54},
		2, {0xB7, 0xAF}
	},
	{
		2, {0x8C, 0x4F},
		3, {0xE8, 0x96, 0xAB},
		2, {0xAB, 0x85},
		2, {0xB7, 0xB0}
	},
	{
		2, {0x8C, 0x50},
		3, {0xE8, 0xA8, 0x93},
		2, {0x13, 0x8A},
		2, {0xB7, 0xB1}
	},
	{
		2, {0x8C, 0x51},
		3, {0xE7, 0xBE, 0xA4},
		2, {0xA4, 0x7F},
		2, {0xB7, 0xB2}
	},
	{
		2, {0x8C, 0x52},
		3, {0xE8, 0xBB, 0x8D},
		2, {0xCD, 0x8E},
		2, {0xB7, 0xB3}
	},
	{
		2, {0x8C, 0x53},
		3, {0xE9, 0x83, 0xA1},
		2, {0xE1, 0x90},
		2, {0xB7, 0xB4}
	},
	{
		2, {0x8C, 0x54},
		3, {0xE5, 0x8D, 0xA6},
		2, {0x66, 0x53},
		2, {0xB7, 0xB5}
	},
	{
		2, {0x8C, 0x55},
		3, {0xE8, 0xA2, 0x88},
		2, {0x88, 0x88},
		2, {0xB7, 0xB6}
	},
	{
		2, {0x8C, 0x56},
		3, {0xE7, 0xA5, 0x81},
		2, {0x41, 0x79},
		2, {0xB7, 0xB7}
	},
	{
		2, {0x8C, 0x57},
		3, {0xE4, 0xBF, 0x82},
		2, {0xC2, 0x4F},
		2, {0xB7, 0xB8}
	},
	{
		2, {0x8C, 0x58},
		3, {0xE5, 0x82, 0xBE},
		2, {0xBE, 0x50},
		2, {0xB7, 0xB9}
	},
	{
		2, {0x8C, 0x59},
		3, {0xE5, 0x88, 0x91},
		2, {0x11, 0x52},
		2, {0xB7, 0xBA}
	},
	{
		2, {0x8C, 0x5A},
		3, {0xE5, 0x85, 0x84},
		2, {0x44, 0x51},
		2, {0xB7, 0xBB}
	},
	{
		2, {0x8C, 0x5B},
		3, {0xE5, 0x95, 0x93},
		2, {0x53, 0x55},
		2, {0xB7, 0xBC}
	},
	{
		2, {0x8C, 0x5C},
		3, {0xE5, 0x9C, 0xAD},
		2, {0x2D, 0x57},
		2, {0xB7, 0xBD}
	},
	{
		2, {0x8C, 0x5D},
		3, {0xE7, 0x8F, 0xAA},
		2, {0xEA, 0x73},
		2, {0xB7, 0xBE}
	},
	{
		2, {0x8C, 0x5E},
		3, {0xE5, 0x9E, 0x8B},
		2, {0x8B, 0x57},
		2, {0xB7, 0xBF}
	},
	{
		2, {0x8C, 0x5F},
		3, {0xE5, 0xA5, 0x91},
		2, {0x51, 0x59},
		2, {0xB7, 0xC0}
	},
	{
		2, {0x8C, 0x60},
		3, {0xE5, 0xBD, 0xA2},
		2, {0x62, 0x5F},
		2, {0xB7, 0xC1}
	},
	{
		2, {0x8C, 0x61},
		3, {0xE5, 0xBE, 0x84},
		2, {0x84, 0x5F},
		2, {0xB7, 0xC2}
	},
	{
		2, {0x8C, 0x62},
		3, {0xE6, 0x81, 0xB5},
		2, {0x75, 0x60},
		2, {0xB7, 0xC3}
	},
	{
		2, {0x8C, 0x63},
		3, {0xE6, 0x85, 0xB6},
		2, {0x76, 0x61},
		2, {0xB7, 0xC4}
	},
	{
		2, {0x8C, 0x64},
		3, {0xE6, 0x85, 0xA7},
		2, {0x67, 0x61},
		2, {0xB7, 0xC5}
	},
	{
		2, {0x8C, 0x65},
		3, {0xE6, 0x86, 0xA9},
		2, {0xA9, 0x61},
		2, {0xB7, 0xC6}
	},
	{
		2, {0x8C, 0x66},
		3, {0xE6, 0x8E, 0xB2},
		2, {0xB2, 0x63},
		2, {0xB7, 0xC7}
	},
	{
		2, {0x8C, 0x67},
		3, {0xE6, 0x90, 0xBA},
		2, {0x3A, 0x64},
		2, {0xB7, 0xC8}
	},
	{
		2, {0x8C, 0x68},
		3, {0xE6, 0x95, 0xAC},
		2, {0x6C, 0x65},
		2, {0xB7, 0xC9}
	},
	{
		2, {0x8C, 0x69},
		3, {0xE6, 0x99, 0xAF},
		2, {0x6F, 0x66},
		2, {0xB7, 0xCA}
	},
	{
		2, {0x8C, 0x6A},
		3, {0xE6, 0xA1, 0x82},
		2, {0x42, 0x68},
		2, {0xB7, 0xCB}
	},
	{
		2, {0x8C, 0x6B},
		3, {0xE6, 0xB8, 0x93},
		2, {0x13, 0x6E},
		2, {0xB7, 0xCC}
	},
	{
		2, {0x8C, 0x6C},
		3, {0xE7, 0x95, 0xA6},
		2, {0x66, 0x75},
		2, {0xB7, 0xCD}
	},
	{
		2, {0x8C, 0x6D},
		3, {0xE7, 0xA8, 0xBD},
		2, {0x3D, 0x7A},
		2, {0xB7, 0xCE}
	},
	{
		2, {0x8C, 0x6E},
		3, {0xE7, 0xB3, 0xBB},
		2, {0xFB, 0x7C},
		2, {0xB7, 0xCF}
	},
	{
		2, {0x8C, 0x6F},
		3, {0xE7, 0xB5, 0x8C},
		2, {0x4C, 0x7D},
		2, {0xB7, 0xD0}
	},
	{
		2, {0x8C, 0x70},
		3, {0xE7, 0xB6, 0x99},
		2, {0x99, 0x7D},
		2, {0xB7, 0xD1}
	},
	{
		2, {0x8C, 0x71},
		3, {0xE7, 0xB9, 0x8B},
		2, {0x4B, 0x7E},
		2, {0xB7, 0xD2}
	},
	{
		2, {0x8C, 0x72},
		3, {0xE7, 0xBD, 0xAB},
		2, {0x6B, 0x7F},
		2, {0xB7, 0xD3}
	},
	{
		2, {0x8C, 0x73},
		3, {0xE8, 0x8C, 0x8E},
		2, {0x0E, 0x83},
		2, {0xB7, 0xD4}
	},
	{
		2, {0x8C, 0x74},
		3, {0xE8, 0x8D, 0x8A},
		2, {0x4A, 0x83},
		2, {0xB7, 0xD5}
	},
	{
		2, {0x8C, 0x75},
		3, {0xE8, 0x9B, 0x8D},
		2, {0xCD, 0x86},
		2, {0xB7, 0xD6}
	},
	{
		2, {0x8C, 0x76},
		3, {0xE8, 0xA8, 0x88},
		2, {0x08, 0x8A},
		2, {0xB7, 0xD7}
	},
	{
		2, {0x8C, 0x77},
		3, {0xE8, 0xA9, 0xA3},
		2, {0x63, 0x8A},
		2, {0xB7, 0xD8}
	},
	{
		2, {0x8C, 0x78},
		3, {0xE8, 0xAD, 0xA6},
		2, {0x66, 0x8B},
		2, {0xB7, 0xD9}
	},
	{
		2, {0x8C, 0x79},
		3, {0xE8, 0xBB, 0xBD},
		2, {0xFD, 0x8E},
		2, {0xB7, 0xDA}
	},
	{
		2, {0x8C, 0x7A},
		3, {0xE9, 0xA0, 0x9A},
		2, {0x1A, 0x98},
		2, {0xB7, 0xDB}
	},
	{
		2, {0x8C, 0x7B},
		3, {0xE9, 0xB6, 0x8F},
		2, {0x8F, 0x9D},
		2, {0xB7, 0xDC}
	},
	{
		2, {0x8C, 0x7C},
		3, {0xE8, 0x8A, 0xB8},
		2, {0xB8, 0x82},
		2, {0xB7, 0xDD}
	},
	{
		2, {0x8C, 0x7D},
		3, {0xE8, 0xBF, 0x8E},
		2, {0xCE, 0x8F},
		2, {0xB7, 0xDE}
	},
	{
		2, {0x8C, 0x7E},
		3, {0xE9, 0xAF, 0xA8},
		2, {0xE8, 0x9B},
		2, {0xB7, 0xDF}
	},
	{
		2, {0x8C, 0x80},
		3, {0xE5, 0x8A, 0x87},
		2, {0x87, 0x52},
		2, {0xB7, 0xE0}
	},
	{
		2, {0x8C, 0x81},
		3, {0xE6, 0x88, 0x9F},
		2, {0x1F, 0x62},
		2, {0xB7, 0xE1}
	},
	{
		2, {0x8C, 0x82},
		3, {0xE6, 0x92, 0x83},
		2, {0x83, 0x64},
		2, {0xB7, 0xE2}
	},
	{
		2, {0x8C, 0x83},
		3, {0xE6, 0xBF, 0x80},
		2, {0xC0, 0x6F},
		2, {0xB7, 0xE3}
	},
	{
		2, {0x8C, 0x84},
		3, {0xE9, 0x9A, 0x99},
		2, {0x99, 0x96},
		2, {0xB7, 0xE4}
	},
	{
		2, {0x8C, 0x85},
		3, {0xE6, 0xA1, 0x81},
		2, {0x41, 0x68},
		2, {0xB7, 0xE5}
	},
	{
		2, {0x8C, 0x86},
		3, {0xE5, 0x82, 0x91},
		2, {0x91, 0x50},
		2, {0xB7, 0xE6}
	},
	{
		2, {0x8C, 0x87},
		3, {0xE6, 0xAC, 0xA0},
		2, {0x20, 0x6B},
		2, {0xB7, 0xE7}
	},
	{
		2, {0x8C, 0x88},
		3, {0xE6, 0xB1, 0xBA},
		2, {0x7A, 0x6C},
		2, {0xB7, 0xE8}
	},
	{
		2, {0x8C, 0x89},
		3, {0xE6, 0xBD, 0x94},
		2, {0x54, 0x6F},
		2, {0xB7, 0xE9}
	},
	{
		2, {0x8C, 0x8A},
		3, {0xE7, 0xA9, 0xB4},
		2, {0x74, 0x7A},
		2, {0xB7, 0xEA}
	},
	{
		2, {0x8C, 0x8B},
		3, {0xE7, 0xB5, 0x90},
		2, {0x50, 0x7D},
		2, {0xB7, 0xEB}
	},
	{
		2, {0x8C, 0x8C},
		3, {0xE8, 0xA1, 0x80},
		2, {0x40, 0x88},
		2, {0xB7, 0xEC}
	},
	{
		2, {0x8C, 0x8D},
		3, {0xE8, 0xA8, 0xA3},
		2, {0x23, 0x8A},
		2, {0xB7, 0xED}
	},
	{
		2, {0x8C, 0x8E},
		3, {0xE6, 0x9C, 0x88},
		2, {0x08, 0x67},
		2, {0xB7, 0xEE}
	},
	{
		2, {0x8C, 0x8F},
		3, {0xE4, 0xBB, 0xB6},
		2, {0xF6, 0x4E},
		2, {0xB7, 0xEF}
	},
	{
		2, {0x8C, 0x90},
		3, {0xE5, 0x80, 0xB9},
		2, {0x39, 0x50},
		2, {0xB7, 0xF0}
	},
	{
		2, {0x8C, 0x91},
		3, {0xE5, 0x80, 0xA6},
		2, {0x26, 0x50},
		2, {0xB7, 0xF1}
	},
	{
		2, {0x8C, 0x92},
		3, {0xE5, 0x81, 0xA5},
		2, {0x65, 0x50},
		2, {0xB7, 0xF2}
	},
	{
		2, {0x8C, 0x93},
		3, {0xE5, 0x85, 0xBC},
		2, {0x7C, 0x51},
		2, {0xB7, 0xF3}
	},
	{
		2, {0x8C, 0x94},
		3, {0xE5, 0x88, 0xB8},
		2, {0x38, 0x52},
		2, {0xB7, 0xF4}
	},
	{
		2, {0x8C, 0x95},
		3, {0xE5, 0x89, 0xA3},
		2, {0x63, 0x52},
		2, {0xB7, 0xF5}
	},
	{
		2, {0x8C, 0x96},
		3, {0xE5, 0x96, 0xA7},
		2, {0xA7, 0x55},
		2, {0xB7, 0xF6}
	},
	{
		2, {0x8C, 0x97},
		3, {0xE5, 0x9C, 0x8F},
		2, {0x0F, 0x57},
		2, {0xB7, 0xF7}
	},
	{
		2, {0x8C, 0x98},
		3, {0xE5, 0xA0, 0x85},
		2, {0x05, 0x58},
		2, {0xB7, 0xF8}
	},
	{
		2, {0x8C, 0x99},
		3, {0xE5, 0xAB, 0x8C},
		2, {0xCC, 0x5A},
		2, {0xB7, 0xF9}
	},
	{
		2, {0x8C, 0x9A},
		3, {0xE5, 0xBB, 0xBA},
		2, {0xFA, 0x5E},
		2, {0xB7, 0xFA}
	},
	{
		2, {0x8C, 0x9B},
		3, {0xE6, 0x86, 0xB2},
		2, {0xB2, 0x61},
		2, {0xB7, 0xFB}
	},
	{
		2, {0x8C, 0x9C},
		3, {0xE6, 0x87, 0xB8},
		2, {0xF8, 0x61},
		2, {0xB7, 0xFC}
	},
	{
		2, {0x8C, 0x9D},
		3, {0xE6, 0x8B, 0xB3},
		2, {0xF3, 0x62},
		2, {0xB7, 0xFD}
	},
	{
		2, {0x8C, 0x9E},
		3, {0xE6, 0x8D, 0xB2},
		2, {0x72, 0x63},
		2, {0xB7, 0xFE}
	},
	{
		2, {0x8C, 0x9F},
		3, {0xE6, 0xA4, 0x9C},
		2, {0x1C, 0x69},
		2, {0xB8, 0xA1}
	},
	{
		2, {0x8C, 0xA0},
		3, {0xE6, 0xA8, 0xA9},
		2, {0x29, 0x6A},
		2, {0xB8, 0xA2}
	},
	{
		2, {0x8C, 0xA1},
		3, {0xE7, 0x89, 0xBD},
		2, {0x7D, 0x72},
		2, {0xB8, 0xA3}
	},
	{
		2, {0x8C, 0xA2},
		3, {0xE7, 0x8A, 0xAC},
		2, {0xAC, 0x72},
		2, {0xB8, 0xA4}
	},
	{
		2, {0x8C, 0xA3},
		3, {0xE7, 0x8C, 0xAE},
		2, {0x2E, 0x73},
		2, {0xB8, 0xA5}
	},
	{
		2, {0x8C, 0xA4},
		3, {0xE7, 0xA0, 0x94},
		2, {0x14, 0x78},
		2, {0xB8, 0xA6}
	},
	{
		2, {0x8C, 0xA5},
		3, {0xE7, 0xA1, 0xAF},
		2, {0x6F, 0x78},
		2, {0xB8, 0xA7}
	},
	{
		2, {0x8C, 0xA6},
		3, {0xE7, 0xB5, 0xB9},
		2, {0x79, 0x7D},
		2, {0xB8, 0xA8}
	},
	{
		2, {0x8C, 0xA7},
		3, {0xE7, 0x9C, 0x8C},
		2, {0x0C, 0x77},
		2, {0xB8, 0xA9}
	},
	{
		2, {0x8C, 0xA8},
		3, {0xE8, 0x82, 0xA9},
		2, {0xA9, 0x80},
		2, {0xB8, 0xAA}
	},
	{
		2, {0x8C, 0xA9},
		3, {0xE8, 0xA6, 0x8B},
		2, {0x8B, 0x89},
		2, {0xB8, 0xAB}
	},
	{
		2, {0x8C, 0xAA},
		3, {0xE8, 0xAC, 0x99},
		2, {0x19, 0x8B},
		2, {0xB8, 0xAC}
	},
	{
		2, {0x8C, 0xAB},
		3, {0xE8, 0xB3, 0xA2},
		2, {0xE2, 0x8C},
		2, {0xB8, 0xAD}
	},
	{
		2, {0x8C, 0xAC},
		3, {0xE8, 0xBB, 0x92},
		2, {0xD2, 0x8E},
		2, {0xB8, 0xAE}
	},
	{
		2, {0x8C, 0xAD},
		3, {0xE9, 0x81, 0xA3},
		2, {0x63, 0x90},
		2, {0xB8, 0xAF}
	},
	{
		2, {0x8C, 0xAE},
		3, {0xE9, 0x8D, 0xB5},
		2, {0x75, 0x93},
		2, {0xB8, 0xB0}
	},
	{
		2, {0x8C, 0xAF},
		3, {0xE9, 0x99, 0xBA},
		2, {0x7A, 0x96},
		2, {0xB8, 0xB1}
	},
	{
		2, {0x8C, 0xB0},
		3, {0xE9, 0xA1, 0x95},
		2, {0x55, 0x98},
		2, {0xB8, 0xB2}
	},
	{
		2, {0x8C, 0xB1},
		3, {0xE9, 0xA8, 0x93},
		2, {0x13, 0x9A},
		2, {0xB8, 0xB3}
	},
	{
		2, {0x8C, 0xB2},
		3, {0xE9, 0xB9, 0xB8},
		2, {0x78, 0x9E},
		2, {0xB8, 0xB4}
	},
	{
		2, {0x8C, 0xB3},
		3, {0xE5, 0x85, 0x83},
		2, {0x43, 0x51},
		2, {0xB8, 0xB5}
	},
	{
		2, {0x8C, 0xB4},
		3, {0xE5, 0x8E, 0x9F},
		2, {0x9F, 0x53},
		2, {0xB8, 0xB6}
	},
	{
		2, {0x8C, 0xB5},
		3, {0xE5, 0x8E, 0xB3},
		2, {0xB3, 0x53},
		2, {0xB8, 0xB7}
	},
	{
		2, {0x8C, 0xB6},
		3, {0xE5, 0xB9, 0xBB},
		2, {0x7B, 0x5E},
		2, {0xB8, 0xB8}
	},
	{
		2, {0x8C, 0xB7},
		3, {0xE5, 0xBC, 0xA6},
		2, {0x26, 0x5F},
		2, {0xB8, 0xB9}
	},
	{
		2, {0x8C, 0xB8},
		3, {0xE6, 0xB8, 0x9B},
		2, {0x1B, 0x6E},
		2, {0xB8, 0xBA}
	},
	{
		2, {0x8C, 0xB9},
		3, {0xE6, 0xBA, 0x90},
		2, {0x90, 0x6E},
		2, {0xB8, 0xBB}
	},
	{
		2, {0x8C, 0xBA},
		3, {0xE7, 0x8E, 0x84},
		2, {0x84, 0x73},
		2, {0xB8, 0xBC}
	},
	{
		2, {0x8C, 0xBB},
		3, {0xE7, 0x8F, 0xBE},
		2, {0xFE, 0x73},
		2, {0xB8, 0xBD}
	},
	{
		2, {0x8C, 0xBC},
		3, {0xE7, 0xB5, 0x83},
		2, {0x43, 0x7D},
		2, {0xB8, 0xBE}
	},
	{
		2, {0x8C, 0xBD},
		3, {0xE8, 0x88, 0xB7},
		2, {0x37, 0x82},
		2, {0xB8, 0xBF}
	},
	{
		2, {0x8C, 0xBE},
		3, {0xE8, 0xA8, 0x80},
		2, {0x00, 0x8A},
		2, {0xB8, 0xC0}
	},
	{
		2, {0x8C, 0xBF},
		3, {0xE8, 0xAB, 0xBA},
		2, {0xFA, 0x8A},
		2, {0xB8, 0xC1}
	},
	{
		2, {0x8C, 0xC0},
		3, {0xE9, 0x99, 0x90},
		2, {0x50, 0x96},
		2, {0xB8, 0xC2}
	},
	{
		2, {0x8C, 0xC1},
		3, {0xE4, 0xB9, 0x8E},
		2, {0x4E, 0x4E},
		2, {0xB8, 0xC3}
	},
	{
		2, {0x8C, 0xC2},
		3, {0xE5, 0x80, 0x8B},
		2, {0x0B, 0x50},
		2, {0xB8, 0xC4}
	},
	{
		2, {0x8C, 0xC3},
		3, {0xE5, 0x8F, 0xA4},
		2, {0xE4, 0x53},
		2, {0xB8, 0xC5}
	},
	{
		2, {0x8C, 0xC4},
		3, {0xE5, 0x91, 0xBC},
		2, {0x7C, 0x54},
		2, {0xB8, 0xC6}
	},
	{
		2, {0x8C, 0xC5},
		3, {0xE5, 0x9B, 0xBA},
		2, {0xFA, 0x56},
		2, {0xB8, 0xC7}
	},
	{
		2, {0x8C, 0xC6},
		3, {0xE5, 0xA7, 0x91},
		2, {0xD1, 0x59},
		2, {0xB8, 0xC8}
	},
	{
		2, {0x8C, 0xC7},
		3, {0xE5, 0xAD, 0xA4},
		2, {0x64, 0x5B},
		2, {0xB8, 0xC9}
	},
	{
		2, {0x8C, 0xC8},
		3, {0xE5, 0xB7, 0xB1},
		2, {0xF1, 0x5D},
		2, {0xB8, 0xCA}
	},
	{
		2, {0x8C, 0xC9},
		3, {0xE5, 0xBA, 0xAB},
		2, {0xAB, 0x5E},
		2, {0xB8, 0xCB}
	},
	{
		2, {0x8C, 0xCA},
		3, {0xE5, 0xBC, 0xA7},
		2, {0x27, 0x5F},
		2, {0xB8, 0xCC}
	},
	{
		2, {0x8C, 0xCB},
		3, {0xE6, 0x88, 0xB8},
		2, {0x38, 0x62},
		2, {0xB8, 0xCD}
	},
	{
		2, {0x8C, 0xCC},
		3, {0xE6, 0x95, 0x85},
		2, {0x45, 0x65},
		2, {0xB8, 0xCE}
	},
	{
		2, {0x8C, 0xCD},
		3, {0xE6, 0x9E, 0xAF},
		2, {0xAF, 0x67},
		2, {0xB8, 0xCF}
	},
	{
		2, {0x8C, 0xCE},
		3, {0xE6, 0xB9, 0x96},
		2, {0x56, 0x6E},
		2, {0xB8, 0xD0}
	},
	{
		2, {0x8C, 0xCF},
		3, {0xE7, 0x8B, 0x90},
		2, {0xD0, 0x72},
		2, {0xB8, 0xD1}
	},
	{
		2, {0x8C, 0xD0},
		3, {0xE7, 0xB3, 0x8A},
		2, {0xCA, 0x7C},
		2, {0xB8, 0xD2}
	},
	{
		2, {0x8C, 0xD1},
		3, {0xE8, 0xA2, 0xB4},
		2, {0xB4, 0x88},
		2, {0xB8, 0xD3}
	},
	{
		2, {0x8C, 0xD2},
		3, {0xE8, 0x82, 0xA1},
		2, {0xA1, 0x80},
		2, {0xB8, 0xD4}
	},
	{
		2, {0x8C, 0xD3},
		3, {0xE8, 0x83, 0xA1},
		2, {0xE1, 0x80},
		2, {0xB8, 0xD5}
	},
	{
		2, {0x8C, 0xD4},
		3, {0xE8, 0x8F, 0xB0},
		2, {0xF0, 0x83},
		2, {0xB8, 0xD6}
	},
	{
		2, {0x8C, 0xD5},
		3, {0xE8, 0x99, 0x8E},
		2, {0x4E, 0x86},
		2, {0xB8, 0xD7}
	},
	{
		2, {0x8C, 0xD6},
		3, {0xE8, 0xAA, 0x87},
		2, {0x87, 0x8A},
		2, {0xB8, 0xD8}
	},
	{
		2, {0x8C, 0xD7},
		3, {0xE8, 0xB7, 0xA8},
		2, {0xE8, 0x8D},
		2, {0xB8, 0xD9}
	},
	{
		2, {0x8C, 0xD8},
		3, {0xE9, 0x88, 0xB7},
		2, {0x37, 0x92},
		2, {0xB8, 0xDA}
	},
	{
		2, {0x8C, 0xD9},
		3, {0xE9, 0x9B, 0x87},
		2, {0xC7, 0x96},
		2, {0xB8, 0xDB}
	},
	{
		2, {0x8C, 0xDA},
		3, {0xE9, 0xA1, 0xA7},
		2, {0x67, 0x98},
		2, {0xB8, 0xDC}
	},
	{
		2, {0x8C, 0xDB},
		3, {0xE9, 0xBC, 0x93},
		2, {0x13, 0x9F},
		2, {0xB8, 0xDD}
	},
	{
		2, {0x8C, 0xDC},
		3, {0xE4, 0xBA, 0x94},
		2, {0x94, 0x4E},
		2, {0xB8, 0xDE}
	},
	{
		2, {0x8C, 0xDD},
		3, {0xE4, 0xBA, 0x92},
		2, {0x92, 0x4E},
		2, {0xB8, 0xDF}
	},
	{
		2, {0x8C, 0xDE},
		3, {0xE4, 0xBC, 0x8D},
		2, {0x0D, 0x4F},
		2, {0xB8, 0xE0}
	},
	{
		2, {0x8C, 0xDF},
		3, {0xE5, 0x8D, 0x88},
		2, {0x48, 0x53},
		2, {0xB8, 0xE1}
	},
	{
		2, {0x8C, 0xE0},
		3, {0xE5, 0x91, 0x89},
		2, {0x49, 0x54},
		2, {0xB8, 0xE2}
	},
	{
		2, {0x8C, 0xE1},
		3, {0xE5, 0x90, 0xBE},
		2, {0x3E, 0x54},
		2, {0xB8, 0xE3}
	},
	{
		2, {0x8C, 0xE2},
		3, {0xE5, 0xA8, 0xAF},
		2, {0x2F, 0x5A},
		2, {0xB8, 0xE4}
	},
	{
		2, {0x8C, 0xE3},
		3, {0xE5, 0xBE, 0x8C},
		2, {0x8C, 0x5F},
		2, {0xB8, 0xE5}
	},
	{
		2, {0x8C, 0xE4},
		3, {0xE5, 0xBE, 0xA1},
		2, {0xA1, 0x5F},
		2, {0xB8, 0xE6}
	},
	{
		2, {0x8C, 0xE5},
		3, {0xE6, 0x82, 0x9F},
		2, {0x9F, 0x60},
		2, {0xB8, 0xE7}
	},
	{
		2, {0x8C, 0xE6},
		3, {0xE6, 0xA2, 0xA7},
		2, {0xA7, 0x68},
		2, {0xB8, 0xE8}
	},
	{
		2, {0x8C, 0xE7},
		3, {0xE6, 0xAA, 0x8E},
		2, {0x8E, 0x6A},
		2, {0xB8, 0xE9}
	},
	{
		2, {0x8C, 0xE8},
		3, {0xE7, 0x91, 0x9A},
		2, {0x5A, 0x74},
		2, {0xB8, 0xEA}
	},
	{
		2, {0x8C, 0xE9},
		3, {0xE7, 0xA2, 0x81},
		2, {0x81, 0x78},
		2, {0xB8, 0xEB}
	},
	{
		2, {0x8C, 0xEA},
		3, {0xE8, 0xAA, 0x9E},
		2, {0x9E, 0x8A},
		2, {0xB8, 0xEC}
	},
	{
		2, {0x8C, 0xEB},
		3, {0xE8, 0xAA, 0xA4},
		2, {0xA4, 0x8A},
		2, {0xB8, 0xED}
	},
	{
		2, {0x8C, 0xEC},
		3, {0xE8, 0xAD, 0xB7},
		2, {0x77, 0x8B},
		2, {0xB8, 0xEE}
	},
	{
		2, {0x8C, 0xED},
		3, {0xE9, 0x86, 0x90},
		2, {0x90, 0x91},
		2, {0xB8, 0xEF}
	},
	{
		2, {0x8C, 0xEE},
		3, {0xE4, 0xB9, 0x9E},
		2, {0x5E, 0x4E},
		2, {0xB8, 0xF0}
	},
	{
		2, {0x8C, 0xEF},
		3, {0xE9, 0xAF, 0x89},
		2, {0xC9, 0x9B},
		2, {0xB8, 0xF1}
	},
	{
		2, {0x8C, 0xF0},
		3, {0xE4, 0xBA, 0xA4},
		2, {0xA4, 0x4E},
		2, {0xB8, 0xF2}
	},
	{
		2, {0x8C, 0xF1},
		3, {0xE4, 0xBD, 0xBC},
		2, {0x7C, 0x4F},
		2, {0xB8, 0xF3}
	},
	{
		2, {0x8C, 0xF2},
		3, {0xE4, 0xBE, 0xAF},
		2, {0xAF, 0x4F},
		2, {0xB8, 0xF4}
	},
	{
		2, {0x8C, 0xF3},
		3, {0xE5, 0x80, 0x99},
		2, {0x19, 0x50},
		2, {0xB8, 0xF5}
	},
	{
		2, {0x8C, 0xF4},
		3, {0xE5, 0x80, 0x96},
		2, {0x16, 0x50},
		2, {0xB8, 0xF6}
	},
	{
		2, {0x8C, 0xF5},
		3, {0xE5, 0x85, 0x89},
		2, {0x49, 0x51},
		2, {0xB8, 0xF7}
	},
	{
		2, {0x8C, 0xF6},
		3, {0xE5, 0x85, 0xAC},
		2, {0x6C, 0x51},
		2, {0xB8, 0xF8}
	},
	{
		2, {0x8C, 0xF7},
		3, {0xE5, 0x8A, 0x9F},
		2, {0x9F, 0x52},
		2, {0xB8, 0xF9}
	},
	{
		2, {0x8C, 0xF8},
		3, {0xE5, 0x8A, 0xB9},
		2, {0xB9, 0x52},
		2, {0xB8, 0xFA}
	},
	{
		2, {0x8C, 0xF9},
		3, {0xE5, 0x8B, 0xBE},
		2, {0xFE, 0x52},
		2, {0xB8, 0xFB}
	},
	{
		2, {0x8C, 0xFA},
		3, {0xE5, 0x8E, 0x9A},
		2, {0x9A, 0x53},
		2, {0xB8, 0xFC}
	},
	{
		2, {0x8C, 0xFB},
		3, {0xE5, 0x8F, 0xA3},
		2, {0xE3, 0x53},
		2, {0xB8, 0xFD}
	},
	{
		2, {0x8C, 0xFC},
		3, {0xE5, 0x90, 0x91},
		2, {0x11, 0x54},
		2, {0xB8, 0xFE}
	},
	{
		2, {0x8D, 0x40},
		3, {0xE5, 0x90, 0x8E},
		2, {0x0E, 0x54},
		2, {0xB9, 0xA1}
	},
	{
		2, {0x8D, 0x41},
		3, {0xE5, 0x96, 0x89},
		2, {0x89, 0x55},
		2, {0xB9, 0xA2}
	},
	{
		2, {0x8D, 0x42},
		3, {0xE5, 0x9D, 0x91},
		2, {0x51, 0x57},
		2, {0xB9, 0xA3}
	},
	{
		2, {0x8D, 0x43},
		3, {0xE5, 0x9E, 0xA2},
		2, {0xA2, 0x57},
		2, {0xB9, 0xA4}
	},
	{
		2, {0x8D, 0x44},
		3, {0xE5, 0xA5, 0xBD},
		2, {0x7D, 0x59},
		2, {0xB9, 0xA5}
	},
	{
		2, {0x8D, 0x45},
		3, {0xE5, 0xAD, 0x94},
		2, {0x54, 0x5B},
		2, {0xB9, 0xA6}
	},
	{
		2, {0x8D, 0x46},
		3, {0xE5, 0xAD, 0x9D},
		2, {0x5D, 0x5B},
		2, {0xB9, 0xA7}
	},
	{
		2, {0x8D, 0x47},
		3, {0xE5, 0xAE, 0x8F},
		2, {0x8F, 0x5B},
		2, {0xB9, 0xA8}
	},
	{
		2, {0x8D, 0x48},
		3, {0xE5, 0xB7, 0xA5},
		2, {0xE5, 0x5D},
		2, {0xB9, 0xA9}
	},
	{
		2, {0x8D, 0x49},
		3, {0xE5, 0xB7, 0xA7},
		2, {0xE7, 0x5D},
		2, {0xB9, 0xAA}
	},
	{
		2, {0x8D, 0x4A},
		3, {0xE5, 0xB7, 0xB7},
		2, {0xF7, 0x5D},
		2, {0xB9, 0xAB}
	},
	{
		2, {0x8D, 0x4B},
		3, {0xE5, 0xB9, 0xB8},
		2, {0x78, 0x5E},
		2, {0xB9, 0xAC}
	},
	{
		2, {0x8D, 0x4C},
		3, {0xE5, 0xBA, 0x83},
		2, {0x83, 0x5E},
		2, {0xB9, 0xAD}
	},
	{
		2, {0x8D, 0x4D},
		3, {0xE5, 0xBA, 0x9A},
		2, {0x9A, 0x5E},
		2, {0xB9, 0xAE}
	},
	{
		2, {0x8D, 0x4E},
		3, {0xE5, 0xBA, 0xB7},
		2, {0xB7, 0x5E},
		2, {0xB9, 0xAF}
	},
	{
		2, {0x8D, 0x4F},
		3, {0xE5, 0xBC, 0x98},
		2, {0x18, 0x5F},
		2, {0xB9, 0xB0}
	},
	{
		2, {0x8D, 0x50},
		3, {0xE6, 0x81, 0x92},
		2, {0x52, 0x60},
		2, {0xB9, 0xB1}
	},
	{
		2, {0x8D, 0x51},
		3, {0xE6, 0x85, 0x8C},
		2, {0x4C, 0x61},
		2, {0xB9, 0xB2}
	},
	{
		2, {0x8D, 0x52},
		3, {0xE6, 0x8A, 0x97},
		2, {0x97, 0x62},
		2, {0xB9, 0xB3}
	},
	{
		2, {0x8D, 0x53},
		3, {0xE6, 0x8B, 0x98},
		2, {0xD8, 0x62},
		2, {0xB9, 0xB4}
	},
	{
		2, {0x8D, 0x54},
		3, {0xE6, 0x8E, 0xA7},
		2, {0xA7, 0x63},
		2, {0xB9, 0xB5}
	},
	{
		2, {0x8D, 0x55},
		3, {0xE6, 0x94, 0xBB},
		2, {0x3B, 0x65},
		2, {0xB9, 0xB6}
	},
	{
		2, {0x8D, 0x56},
		3, {0xE6, 0x98, 0x82},
		2, {0x02, 0x66},
		2, {0xB9, 0xB7}
	},
	{
		2, {0x8D, 0x57},
		3, {0xE6, 0x99, 0x83},
		2, {0x43, 0x66},
		2, {0xB9, 0xB8}
	},
	{
		2, {0x8D, 0x58},
		3, {0xE6, 0x9B, 0xB4},
		2, {0xF4, 0x66},
		2, {0xB9, 0xB9}
	},
	{
		2, {0x8D, 0x59},
		3, {0xE6, 0x9D, 0xAD},
		2, {0x6D, 0x67},
		2, {0xB9, 0xBA}
	},
	{
		2, {0x8D, 0x5A},
		3, {0xE6, 0xA0, 0xA1},
		2, {0x21, 0x68},
		2, {0xB9, 0xBB}
	},
	{
		2, {0x8D, 0x5B},
		3, {0xE6, 0xA2, 0x97},
		2, {0x97, 0x68},
		2, {0xB9, 0xBC}
	},
	{
		2, {0x8D, 0x5C},
		3, {0xE6, 0xA7, 0x8B},
		2, {0xCB, 0x69},
		2, {0xB9, 0xBD}
	},
	{
		2, {0x8D, 0x5D},
		3, {0xE6, 0xB1, 0x9F},
		2, {0x5F, 0x6C},
		2, {0xB9, 0xBE}
	},
	{
		2, {0x8D, 0x5E},
		3, {0xE6, 0xB4, 0xAA},
		2, {0x2A, 0x6D},
		2, {0xB9, 0xBF}
	},
	{
		2, {0x8D, 0x5F},
		3, {0xE6, 0xB5, 0xA9},
		2, {0x69, 0x6D},
		2, {0xB9, 0xC0}
	},
	{
		2, {0x8D, 0x60},
		3, {0xE6, 0xB8, 0xAF},
		2, {0x2F, 0x6E},
		2, {0xB9, 0xC1}
	},
	{
		2, {0x8D, 0x61},
		3, {0xE6, 0xBA, 0x9D},
		2, {0x9D, 0x6E},
		2, {0xB9, 0xC2}
	},
	{
		2, {0x8D, 0x62},
		3, {0xE7, 0x94, 0xB2},
		2, {0x32, 0x75},
		2, {0xB9, 0xC3}
	},
	{
		2, {0x8D, 0x63},
		3, {0xE7, 0x9A, 0x87},
		2, {0x87, 0x76},
		2, {0xB9, 0xC4}
	},
	{
		2, {0x8D, 0x64},
		3, {0xE7, 0xA1, 0xAC},
		2, {0x6C, 0x78},
		2, {0xB9, 0xC5}
	},
	{
		2, {0x8D, 0x65},
		3, {0xE7, 0xA8, 0xBF},
		2, {0x3F, 0x7A},
		2, {0xB9, 0xC6}
	},
	{
		2, {0x8D, 0x66},
		3, {0xE7, 0xB3, 0xA0},
		2, {0xE0, 0x7C},
		2, {0xB9, 0xC7}
	},
	{
		2, {0x8D, 0x67},
		3, {0xE7, 0xB4, 0x85},
		2, {0x05, 0x7D},
		2, {0xB9, 0xC8}
	},
	{
		2, {0x8D, 0x68},
		3, {0xE7, 0xB4, 0x98},
		2, {0x18, 0x7D},
		2, {0xB9, 0xC9}
	},
	{
		2, {0x8D, 0x69},
		3, {0xE7, 0xB5, 0x9E},
		2, {0x5E, 0x7D},
		2, {0xB9, 0xCA}
	},
	{
		2, {0x8D, 0x6A},
		3, {0xE7, 0xB6, 0xB1},
		2, {0xB1, 0x7D},
		2, {0xB9, 0xCB}
	},
	{
		2, {0x8D, 0x6B},
		3, {0xE8, 0x80, 0x95},
		2, {0x15, 0x80},
		2, {0xB9, 0xCC}
	},
	{
		2, {0x8D, 0x6C},
		3, {0xE8, 0x80, 0x83},
		2, {0x03, 0x80},
		2, {0xB9, 0xCD}
	},
	{
		2, {0x8D, 0x6D},
		3, {0xE8, 0x82, 0xAF},
		2, {0xAF, 0x80},
		2, {0xB9, 0xCE}
	},
	{
		2, {0x8D, 0x6E},
		3, {0xE8, 0x82, 0xB1},
		2, {0xB1, 0x80},
		2, {0xB9, 0xCF}
	},
	{
		2, {0x8D, 0x6F},
		3, {0xE8, 0x85, 0x94},
		2, {0x54, 0x81},
		2, {0xB9, 0xD0}
	},
	{
		2, {0x8D, 0x70},
		3, {0xE8, 0x86, 0x8F},
		2, {0x8F, 0x81},
		2, {0xB9, 0xD1}
	},
	{
		2, {0x8D, 0x71},
		3, {0xE8, 0x88, 0xAA},
		2, {0x2A, 0x82},
		2, {0xB9, 0xD2}
	},
	{
		2, {0x8D, 0x72},
		3, {0xE8, 0x8D, 0x92},
		2, {0x52, 0x83},
		2, {0xB9, 0xD3}
	},
	{
		2, {0x8D, 0x73},
		3, {0xE8, 0xA1, 0x8C},
		2, {0x4C, 0x88},
		2, {0xB9, 0xD4}
	},
	{
		2, {0x8D, 0x74},
		3, {0xE8, 0xA1, 0xA1},
		2, {0x61, 0x88},
		2, {0xB9, 0xD5}
	},
	{
		2, {0x8D, 0x75},
		3, {0xE8, 0xAC, 0x9B},
		2, {0x1B, 0x8B},
		2, {0xB9, 0xD6}
	},
	{
		2, {0x8D, 0x76},
		3, {0xE8, 0xB2, 0xA2},
		2, {0xA2, 0x8C},
		2, {0xB9, 0xD7}
	},
	{
		2, {0x8D, 0x77},
		3, {0xE8, 0xB3, 0xBC},
		2, {0xFC, 0x8C},
		2, {0xB9, 0xD8}
	},
	{
		2, {0x8D, 0x78},
		3, {0xE9, 0x83, 0x8A},
		2, {0xCA, 0x90},
		2, {0xB9, 0xD9}
	},
	{
		2, {0x8D, 0x79},
		3, {0xE9, 0x85, 0xB5},
		2, {0x75, 0x91},
		2, {0xB9, 0xDA}
	},
	{
		2, {0x8D, 0x7A},
		3, {0xE9, 0x89, 0xB1},
		2, {0x71, 0x92},
		2, {0xB9, 0xDB}
	},
	{
		2, {0x8D, 0x7B},
		3, {0xE7, 0xA0, 0xBF},
		2, {0x3F, 0x78},
		2, {0xB9, 0xDC}
	},
	{
		2, {0x8D, 0x7C},
		3, {0xE9, 0x8B, 0xBC},
		2, {0xFC, 0x92},
		2, {0xB9, 0xDD}
	},
	{
		2, {0x8D, 0x7D},
		3, {0xE9, 0x96, 0xA4},
		2, {0xA4, 0x95},
		2, {0xB9, 0xDE}
	},
	{
		2, {0x8D, 0x7E},
		3, {0xE9, 0x99, 0x8D},
		2, {0x4D, 0x96},
		2, {0xB9, 0xDF}
	},
	{
		2, {0x8D, 0x80},
		3, {0xE9, 0xA0, 0x85},
		2, {0x05, 0x98},
		2, {0xB9, 0xE0}
	},
	{
		2, {0x8D, 0x81},
		3, {0xE9, 0xA6, 0x99},
		2, {0x99, 0x99},
		2, {0xB9, 0xE1}
	},
	{
		2, {0x8D, 0x82},
		3, {0xE9, 0xAB, 0x98},
		2, {0xD8, 0x9A},
		2, {0xB9, 0xE2}
	},
	{
		2, {0x8D, 0x83},
		3, {0xE9, 0xB4, 0xBB},
		2, {0x3B, 0x9D},
		2, {0xB9, 0xE3}
	},
	{
		2, {0x8D, 0x84},
		3, {0xE5, 0x89, 0x9B},
		2, {0x5B, 0x52},
		2, {0xB9, 0xE4}
	},
	{
		2, {0x8D, 0x85},
		3, {0xE5, 0x8A, 0xAB},
		2, {0xAB, 0x52},
		2, {0xB9, 0xE5}
	},
	{
		2, {0x8D, 0x86},
		3, {0xE5, 0x8F, 0xB7},
		2, {0xF7, 0x53},
		2, {0xB9, 0xE6}
	},
	{
		2, {0x8D, 0x87},
		3, {0xE5, 0x90, 0x88},
		2, {0x08, 0x54},
		2, {0xB9, 0xE7}
	},
	{
		2, {0x8D, 0x88},
		3, {0xE5, 0xA3, 0x95},
		2, {0xD5, 0x58},
		2, {0xB9, 0xE8}
	},
	{
		2, {0x8D, 0x89},
		3, {0xE6, 0x8B, 0xB7},
		2, {0xF7, 0x62},
		2, {0xB9, 0xE9}
	},
	{
		2, {0x8D, 0x8A},
		3, {0xE6, 0xBF, 0xA0},
		2, {0xE0, 0x6F},
		2, {0xB9, 0xEA}
	},
	{
		2, {0x8D, 0x8B},
		3, {0xE8, 0xB1, 0xAA},
		2, {0x6A, 0x8C},
		2, {0xB9, 0xEB}
	},
	{
		2, {0x8D, 0x8C},
		3, {0xE8, 0xBD, 0x9F},
		2, {0x5F, 0x8F},
		2, {0xB9, 0xEC}
	},
	{
		2, {0x8D, 0x8D},
		3, {0xE9, 0xBA, 0xB9},
		2, {0xB9, 0x9E},
		2, {0xB9, 0xED}
	},
	{
		2, {0x8D, 0x8E},
		3, {0xE5, 0x85, 0x8B},
		2, {0x4B, 0x51},
		2, {0xB9, 0xEE}
	},
	{
		2, {0x8D, 0x8F},
		3, {0xE5, 0x88, 0xBB},
		2, {0x3B, 0x52},
		2, {0xB9, 0xEF}
	},
	{
		2, {0x8D, 0x90},
		3, {0xE5, 0x91, 0x8A},
		2, {0x4A, 0x54},
		2, {0xB9, 0xF0}
	},
	{
		2, {0x8D, 0x91},
		3, {0xE5, 0x9B, 0xBD},
		2, {0xFD, 0x56},
		2, {0xB9, 0xF1}
	},
	{
		2, {0x8D, 0x92},
		3, {0xE7, 0xA9, 0x80},
		2, {0x40, 0x7A},
		2, {0xB9, 0xF2}
	},
	{
		2, {0x8D, 0x93},
		3, {0xE9, 0x85, 0xB7},
		2, {0x77, 0x91},
		2, {0xB9, 0xF3}
	},
	{
		2, {0x8D, 0x94},
		3, {0xE9, 0xB5, 0xA0},
		2, {0x60, 0x9D},
		2, {0xB9, 0xF4}
	},
	{
		2, {0x8D, 0x95},
		3, {0xE9, 0xBB, 0x92},
		2, {0xD2, 0x9E},
		2, {0xB9, 0xF5}
	},
	{
		2, {0x8D, 0x96},
		3, {0xE7, 0x8D, 0x84},
		2, {0x44, 0x73},
		2, {0xB9, 0xF6}
	},
	{
		2, {0x8D, 0x97},
		3, {0xE6, 0xBC, 0x89},
		2, {0x09, 0x6F},
		2, {0xB9, 0xF7}
	},
	{
		2, {0x8D, 0x98},
		3, {0xE8, 0x85, 0xB0},
		2, {0x70, 0x81},
		2, {0xB9, 0xF8}
	},
	{
		2, {0x8D, 0x99},
		3, {0xE7, 0x94, 0x91},
		2, {0x11, 0x75},
		2, {0xB9, 0xF9}
	},
	{
		2, {0x8D, 0x9A},
		3, {0xE5, 0xBF, 0xBD},
		2, {0xFD, 0x5F},
		2, {0xB9, 0xFA}
	},
	{
		2, {0x8D, 0x9B},
		3, {0xE6, 0x83, 0x9A},
		2, {0xDA, 0x60},
		2, {0xB9, 0xFB}
	},
	{
		2, {0x8D, 0x9C},
		3, {0xE9, 0xAA, 0xA8},
		2, {0xA8, 0x9A},
		2, {0xB9, 0xFC}
	},
	{
		2, {0x8D, 0x9D},
		3, {0xE7, 0x8B, 0x9B},
		2, {0xDB, 0x72},
		2, {0xB9, 0xFD}
	},
	{
		2, {0x8D, 0x9E},
		3, {0xE8, 0xBE, 0xBC},
		2, {0xBC, 0x8F},
		2, {0xB9, 0xFE}
	},
	{
		2, {0x8D, 0x9F},
		3, {0xE6, 0xAD, 0xA4},
		2, {0x64, 0x6B},
		2, {0xBA, 0xA1}
	},
	{
		2, {0x8D, 0xA0},
		3, {0xE9, 0xA0, 0x83},
		2, {0x03, 0x98},
		2, {0xBA, 0xA2}
	},
	{
		2, {0x8D, 0xA1},
		3, {0xE4, 0xBB, 0x8A},
		2, {0xCA, 0x4E},
		2, {0xBA, 0xA3}
	},
	{
		2, {0x8D, 0xA2},
		3, {0xE5, 0x9B, 0xB0},
		2, {0xF0, 0x56},
		2, {0xBA, 0xA4}
	},
	{
		2, {0x8D, 0xA3},
		3, {0xE5, 0x9D, 0xA4},
		2, {0x64, 0x57},
		2, {0xBA, 0xA5}
	},
	{
		2, {0x8D, 0xA4},
		3, {0xE5, 0xA2, 0xBE},
		2, {0xBE, 0x58},
		2, {0xBA, 0xA6}
	},
	{
		2, {0x8D, 0xA5},
		3, {0xE5, 0xA9, 0x9A},
		2, {0x5A, 0x5A},
		2, {0xBA, 0xA7}
	},
	{
		2, {0x8D, 0xA6},
		3, {0xE6, 0x81, 0xA8},
		2, {0x68, 0x60},
		2, {0xBA, 0xA8}
	},
	{
		2, {0x8D, 0xA7},
		3, {0xE6, 0x87, 0x87},
		2, {0xC7, 0x61},
		2, {0xBA, 0xA9}
	},
	{
		2, {0x8D, 0xA8},
		3, {0xE6, 0x98, 0x8F},
		2, {0x0F, 0x66},
		2, {0xBA, 0xAA}
	},
	{
		2, {0x8D, 0xA9},
		3, {0xE6, 0x98, 0x86},
		2, {0x06, 0x66},
		2, {0xBA, 0xAB}
	},
	{
		2, {0x8D, 0xAA},
		3, {0xE6, 0xA0, 0xB9},
		2, {0x39, 0x68},
		2, {0xBA, 0xAC}
	},
	{
		2, {0x8D, 0xAB},
		3, {0xE6, 0xA2, 0xB1},
		2, {0xB1, 0x68},
		2, {0xBA, 0xAD}
	},
	{
		2, {0x8D, 0xAC},
		3, {0xE6, 0xB7, 0xB7},
		2, {0xF7, 0x6D},
		2, {0xBA, 0xAE}
	},
	{
		2, {0x8D, 0xAD},
		3, {0xE7, 0x97, 0x95},
		2, {0xD5, 0x75},
		2, {0xBA, 0xAF}
	},
	{
		2, {0x8D, 0xAE},
		3, {0xE7, 0xB4, 0xBA},
		2, {0x3A, 0x7D},
		2, {0xBA, 0xB0}
	},
	{
		2, {0x8D, 0xAF},
		3, {0xE8, 0x89, 0xAE},
		2, {0x6E, 0x82},
		2, {0xBA, 0xB1}
	},
	{
		2, {0x8D, 0xB0},
		3, {0xE9, 0xAD, 0x82},
		2, {0x42, 0x9B},
		2, {0xBA, 0xB2}
	},
	{
		2, {0x8D, 0xB1},
		3, {0xE4, 0xBA, 0x9B},
		2, {0x9B, 0x4E},
		2, {0xBA, 0xB3}
	},
	{
		2, {0x8D, 0xB2},
		3, {0xE4, 0xBD, 0x90},
		2, {0x50, 0x4F},
		2, {0xBA, 0xB4}
	},
	{
		2, {0x8D, 0xB3},
		3, {0xE5, 0x8F, 0x89},
		2, {0xC9, 0x53},
		2, {0xBA, 0xB5}
	},
	{
		2, {0x8D, 0xB4},
		3, {0xE5, 0x94, 0x86},
		2, {0x06, 0x55},
		2, {0xBA, 0xB6}
	},
	{
		2, {0x8D, 0xB5},
		3, {0xE5, 0xB5, 0xAF},
		2, {0x6F, 0x5D},
		2, {0xBA, 0xB7}
	},
	{
		2, {0x8D, 0xB6},
		3, {0xE5, 0xB7, 0xA6},
		2, {0xE6, 0x5D},
		2, {0xBA, 0xB8}
	},
	{
		2, {0x8D, 0xB7},
		3, {0xE5, 0xB7, 0xAE},
		2, {0xEE, 0x5D},
		2, {0xBA, 0xB9}
	},
	{
		2, {0x8D, 0xB8},
		3, {0xE6, 0x9F, 0xBB},
		2, {0xFB, 0x67},
		2, {0xBA, 0xBA}
	},
	{
		2, {0x8D, 0xB9},
		3, {0xE6, 0xB2, 0x99},
		2, {0x99, 0x6C},
		2, {0xBA, 0xBB}
	},
	{
		2, {0x8D, 0xBA},
		3, {0xE7, 0x91, 0xB3},
		2, {0x73, 0x74},
		2, {0xBA, 0xBC}
	},
	{
		2, {0x8D, 0xBB},
		3, {0xE7, 0xA0, 0x82},
		2, {0x02, 0x78},
		2, {0xBA, 0xBD}
	},
	{
		2, {0x8D, 0xBC},
		3, {0xE8, 0xA9, 0x90},
		2, {0x50, 0x8A},
		2, {0xBA, 0xBE}
	},
	{
		2, {0x8D, 0xBD},
		3, {0xE9, 0x8E, 0x96},
		2, {0x96, 0x93},
		2, {0xBA, 0xBF}
	},
	{
		2, {0x8D, 0xBE},
		3, {0xE8, 0xA3, 0x9F},
		2, {0xDF, 0x88},
		2, {0xBA, 0xC0}
	},
	{
		2, {0x8D, 0xBF},
		3, {0xE5, 0x9D, 0x90},
		2, {0x50, 0x57},
		2, {0xBA, 0xC1}
	},
	{
		2, {0x8D, 0xC0},
		3, {0xE5, 0xBA, 0xA7},
		2, {0xA7, 0x5E},
		2, {0xBA, 0xC2}
	},
	{
		2, {0x8D, 0xC1},
		3, {0xE6, 0x8C, 0xAB},
		2, {0x2B, 0x63},
		2, {0xBA, 0xC3}
	},
	{
		2, {0x8D, 0xC2},
		3, {0xE5, 0x82, 0xB5},
		2, {0xB5, 0x50},
		2, {0xBA, 0xC4}
	},
	{
		2, {0x8D, 0xC3},
		3, {0xE5, 0x82, 0xAC},
		2, {0xAC, 0x50},
		2, {0xBA, 0xC5}
	},
	{
		2, {0x8D, 0xC4},
		3, {0xE5, 0x86, 0x8D},
		2, {0x8D, 0x51},
		2, {0xBA, 0xC6}
	},
	{
		2, {0x8D, 0xC5},
		3, {0xE6, 0x9C, 0x80},
		2, {0x00, 0x67},
		2, {0xBA, 0xC7}
	},
	{
		2, {0x8D, 0xC6},
		3, {0xE5, 0x93, 0x89},
		2, {0xC9, 0x54},
		2, {0xBA, 0xC8}
	},
	{
		2, {0x8D, 0xC7},
		3, {0xE5, 0xA1, 0x9E},
		2, {0x5E, 0x58},
		2, {0xBA, 0xC9}
	},
	{
		2, {0x8D, 0xC8},
		3, {0xE5, 0xA6, 0xBB},
		2, {0xBB, 0x59},
		2, {0xBA, 0xCA}
	},
	{
		2, {0x8D, 0xC9},
		3, {0xE5, 0xAE, 0xB0},
		2, {0xB0, 0x5B},
		2, {0xBA, 0xCB}
	},
	{
		2, {0x8D, 0xCA},
		3, {0xE5, 0xBD, 0xA9},
		2, {0x69, 0x5F},
		2, {0xBA, 0xCC}
	},
	{
		2, {0x8D, 0xCB},
		3, {0xE6, 0x89, 0x8D},
		2, {0x4D, 0x62},
		2, {0xBA, 0xCD}
	},
	{
		2, {0x8D, 0xCC},
		3, {0xE6, 0x8E, 0xA1},
		2, {0xA1, 0x63},
		2, {0xBA, 0xCE}
	},
	{
		2, {0x8D, 0xCD},
		3, {0xE6, 0xA0, 0xBD},
		2, {0x3D, 0x68},
		2, {0xBA, 0xCF}
	},
	{
		2, {0x8D, 0xCE},
		3, {0xE6, 0xAD, 0xB3},
		2, {0x73, 0x6B},
		2, {0xBA, 0xD0}
	},
	{
		2, {0x8D, 0xCF},
		3, {0xE6, 0xB8, 0x88},
		2, {0x08, 0x6E},
		2, {0xBA, 0xD1}
	},
	{
		2, {0x8D, 0xD0},
		3, {0xE7, 0x81, 0xBD},
		2, {0x7D, 0x70},
		2, {0xBA, 0xD2}
	},
	{
		2, {0x8D, 0xD1},
		3, {0xE9, 0x87, 0x87},
		2, {0xC7, 0x91},
		2, {0xBA, 0xD3}
	},
	{
		2, {0x8D, 0xD2},
		3, {0xE7, 0x8A, 0x80},
		2, {0x80, 0x72},
		2, {0xBA, 0xD4}
	},
	{
		2, {0x8D, 0xD3},
		3, {0xE7, 0xA0, 0x95},
		2, {0x15, 0x78},
		2, {0xBA, 0xD5}
	},
	{
		2, {0x8D, 0xD4},
		3, {0xE7, 0xA0, 0xA6},
		2, {0x26, 0x78},
		2, {0xBA, 0xD6}
	},
	{
		2, {0x8D, 0xD5},
		3, {0xE7, 0xA5, 0xAD},
		2, {0x6D, 0x79},
		2, {0xBA, 0xD7}
	},
	{
		2, {0x8D, 0xD6},
		3, {0xE6, 0x96, 0x8E},
		2, {0x8E, 0x65},
		2, {0xBA, 0xD8}
	},
	{
		2, {0x8D, 0xD7},
		3, {0xE7, 0xB4, 0xB0},
		2, {0x30, 0x7D},
		2, {0xBA, 0xD9}
	},
	{
		2, {0x8D, 0xD8},
		3, {0xE8, 0x8F, 0x9C},
		2, {0xDC, 0x83},
		2, {0xBA, 0xDA}
	},
	{
		2, {0x8D, 0xD9},
		3, {0xE8, 0xA3, 0x81},
		2, {0xC1, 0x88},
		2, {0xBA, 0xDB}
	},
	{
		2, {0x8D, 0xDA},
		3, {0xE8, 0xBC, 0x89},
		2, {0x09, 0x8F},
		2, {0xBA, 0xDC}
	},
	{
		2, {0x8D, 0xDB},
		3, {0xE9, 0x9A, 0x9B},
		2, {0x9B, 0x96},
		2, {0xBA, 0xDD}
	},
	{
		2, {0x8D, 0xDC},
		3, {0xE5, 0x89, 0xA4},
		2, {0x64, 0x52},
		2, {0xBA, 0xDE}
	},
	{
		2, {0x8D, 0xDD},
		3, {0xE5, 0x9C, 0xA8},
		2, {0x28, 0x57},
		2, {0xBA, 0xDF}
	},
	{
		2, {0x8D, 0xDE},
		3, {0xE6, 0x9D, 0x90},
		2, {0x50, 0x67},
		2, {0xBA, 0xE0}
	},
	{
		2, {0x8D, 0xDF},
		3, {0xE7, 0xBD, 0xAA},
		2, {0x6A, 0x7F},
		2, {0xBA, 0xE1}
	},
	{
		2, {0x8D, 0xE0},
		3, {0xE8, 0xB2, 0xA1},
		2, {0xA1, 0x8C},
		2, {0xBA, 0xE2}
	},
	{
		2, {0x8D, 0xE1},
		3, {0xE5, 0x86, 0xB4},
		2, {0xB4, 0x51},
		2, {0xBA, 0xE3}
	},
	{
		2, {0x8D, 0xE2},
		3, {0xE5, 0x9D, 0x82},
		2, {0x42, 0x57},
		2, {0xBA, 0xE4}
	},
	{
		2, {0x8D, 0xE3},
		3, {0xE9, 0x98, 0xAA},
		2, {0x2A, 0x96},
		2, {0xBA, 0xE5}
	},
	{
		2, {0x8D, 0xE4},
		3, {0xE5, 0xA0, 0xBA},
		2, {0x3A, 0x58},
		2, {0xBA, 0xE6}
	},
	{
		2, {0x8D, 0xE5},
		3, {0xE6, 0xA6, 0x8A},
		2, {0x8A, 0x69},
		2, {0xBA, 0xE7}
	},
	{
		2, {0x8D, 0xE6},
		3, {0xE8, 0x82, 0xB4},
		2, {0xB4, 0x80},
		2, {0xBA, 0xE8}
	},
	{
		2, {0x8D, 0xE7},
		3, {0xE5, 0x92, 0xB2},
		2, {0xB2, 0x54},
		2, {0xBA, 0xE9}
	},
	{
		2, {0x8D, 0xE8},
		3, {0xE5, 0xB4, 0x8E},
		2, {0x0E, 0x5D},
		2, {0xBA, 0xEA}
	},
	{
		2, {0x8D, 0xE9},
		3, {0xE5, 0x9F, 0xBC},
		2, {0xFC, 0x57},
		2, {0xBA, 0xEB}
	},
	{
		2, {0x8D, 0xEA},
		3, {0xE7, 0xA2, 0x95},
		2, {0x95, 0x78},
		2, {0xBA, 0xEC}
	},
	{
		2, {0x8D, 0xEB},
		3, {0xE9, 0xB7, 0xBA},
		2, {0xFA, 0x9D},
		2, {0xBA, 0xED}
	},
	{
		2, {0x8D, 0xEC},
		3, {0xE4, 0xBD, 0x9C},
		2, {0x5C, 0x4F},
		2, {0xBA, 0xEE}
	},
	{
		2, {0x8D, 0xED},
		3, {0xE5, 0x89, 0x8A},
		2, {0x4A, 0x52},
		2, {0xBA, 0xEF}
	},
	{
		2, {0x8D, 0xEE},
		3, {0xE5, 0x92, 0x8B},
		2, {0x8B, 0x54},
		2, {0xBA, 0xF0}
	},
	{
		2, {0x8D, 0xEF},
		3, {0xE6, 0x90, 0xBE},
		2, {0x3E, 0x64},
		2, {0xBA, 0xF1}
	},
	{
		2, {0x8D, 0xF0},
		3, {0xE6, 0x98, 0xA8},
		2, {0x28, 0x66},
		2, {0xBA, 0xF2}
	},
	{
		2, {0x8D, 0xF1},
		3, {0xE6, 0x9C, 0x94},
		2, {0x14, 0x67},
		2, {0xBA, 0xF3}
	},
	{
		2, {0x8D, 0xF2},
		3, {0xE6, 0x9F, 0xB5},
		2, {0xF5, 0x67},
		2, {0xBA, 0xF4}
	},
	{
		2, {0x8D, 0xF3},
		3, {0xE7, 0xAA, 0x84},
		2, {0x84, 0x7A},
		2, {0xBA, 0xF5}
	},
	{
		2, {0x8D, 0xF4},
		3, {0xE7, 0xAD, 0x96},
		2, {0x56, 0x7B},
		2, {0xBA, 0xF6}
	},
	{
		2, {0x8D, 0xF5},
		3, {0xE7, 0xB4, 0xA2},
		2, {0x22, 0x7D},
		2, {0xBA, 0xF7}
	},
	{
		2, {0x8D, 0xF6},
		3, {0xE9, 0x8C, 0xAF},
		2, {0x2F, 0x93},
		2, {0xBA, 0xF8}
	},
	{
		2, {0x8D, 0xF7},
		3, {0xE6, 0xA1, 0x9C},
		2, {0x5C, 0x68},
		2, {0xBA, 0xF9}
	},
	{
		2, {0x8D, 0xF8},
		3, {0xE9, 0xAE, 0xAD},
		2, {0xAD, 0x9B},
		2, {0xBA, 0xFA}
	},
	{
		2, {0x8D, 0xF9},
		3, {0xE7, 0xAC, 0xB9},
		2, {0x39, 0x7B},
		2, {0xBA, 0xFB}
	},
	{
		2, {0x8D, 0xFA},
		3, {0xE5, 0x8C, 0x99},
		2, {0x19, 0x53},
		2, {0xBA, 0xFC}
	},
	{
		2, {0x8D, 0xFB},
		3, {0xE5, 0x86, 0x8A},
		2, {0x8A, 0x51},
		2, {0xBA, 0xFD}
	},
	{
		2, {0x8D, 0xFC},
		3, {0xE5, 0x88, 0xB7},
		2, {0x37, 0x52},
		2, {0xBA, 0xFE}
	},
	{
		2, {0x8E, 0x40},
		3, {0xE5, 0xAF, 0x9F},
		2, {0xDF, 0x5B},
		2, {0xBB, 0xA1}
	},
	{
		2, {0x8E, 0x41},
		3, {0xE6, 0x8B, 0xB6},
		2, {0xF6, 0x62},
		2, {0xBB, 0xA2}
	},
	{
		2, {0x8E, 0x42},
		3, {0xE6, 0x92, 0xAE},
		2, {0xAE, 0x64},
		2, {0xBB, 0xA3}
	},
	{
		2, {0x8E, 0x43},
		3, {0xE6, 0x93, 0xA6},
		2, {0xE6, 0x64},
		2, {0xBB, 0xA4}
	},
	{
		2, {0x8E, 0x44},
		3, {0xE6, 0x9C, 0xAD},
		2, {0x2D, 0x67},
		2, {0xBB, 0xA5}
	},
	{
		2, {0x8E, 0x45},
		3, {0xE6, 0xAE, 0xBA},
		2, {0xBA, 0x6B},
		2, {0xBB, 0xA6}
	},
	{
		2, {0x8E, 0x46},
		3, {0xE8, 0x96, 0xA9},
		2, {0xA9, 0x85},
		2, {0xBB, 0xA7}
	},
	{
		2, {0x8E, 0x47},
		3, {0xE9, 0x9B, 0x91},
		2, {0xD1, 0x96},
		2, {0xBB, 0xA8}
	},
	{
		2, {0x8E, 0x48},
		3, {0xE7, 0x9A, 0x90},
		2, {0x90, 0x76},
		2, {0xBB, 0xA9}
	},
	{
		2, {0x8E, 0x49},
		3, {0xE9, 0xAF, 0x96},
		2, {0xD6, 0x9B},
		2, {0xBB, 0xAA}
	},
	{
		2, {0x8E, 0x4A},
		3, {0xE6, 0x8D, 0x8C},
		2, {0x4C, 0x63},
		2, {0xBB, 0xAB}
	},
	{
		2, {0x8E, 0x4B},
		3, {0xE9, 0x8C, 0x86},
		2, {0x06, 0x93},
		2, {0xBB, 0xAC}
	},
	{
		2, {0x8E, 0x4C},
		3, {0xE9, 0xAE, 0xAB},
		2, {0xAB, 0x9B},
		2, {0xBB, 0xAD}
	},
	{
		2, {0x8E, 0x4D},
		3, {0xE7, 0x9A, 0xBF},
		2, {0xBF, 0x76},
		2, {0xBB, 0xAE}
	},
	{
		2, {0x8E, 0x4E},
		3, {0xE6, 0x99, 0x92},
		2, {0x52, 0x66},
		2, {0xBB, 0xAF}
	},
	{
		2, {0x8E, 0x4F},
		3, {0xE4, 0xB8, 0x89},
		2, {0x09, 0x4E},
		2, {0xBB, 0xB0}
	},
	{
		2, {0x8E, 0x50},
		3, {0xE5, 0x82, 0x98},
		2, {0x98, 0x50},
		2, {0xBB, 0xB1}
	},
	{
		2, {0x8E, 0x51},
		3, {0xE5, 0x8F, 0x82},
		2, {0xC2, 0x53},
		2, {0xBB, 0xB2}
	},
	{
		2, {0x8E, 0x52},
		3, {0xE5, 0xB1, 0xB1},
		2, {0x71, 0x5C},
		2, {0xBB, 0xB3}
	},
	{
		2, {0x8E, 0x53},
		3, {0xE6, 0x83, 0xA8},
		2, {0xE8, 0x60},
		2, {0xBB, 0xB4}
	},
	{
		2, {0x8E, 0x54},
		3, {0xE6, 0x92, 0x92},
		2, {0x92, 0x64},
		2, {0xBB, 0xB5}
	},
	{
		2, {0x8E, 0x55},
		3, {0xE6, 0x95, 0xA3},
		2, {0x63, 0x65},
		2, {0xBB, 0xB6}
	},
	{
		2, {0x8E, 0x56},
		3, {0xE6, 0xA1, 0x9F},
		2, {0x5F, 0x68},
		2, {0xBB, 0xB7}
	},
	{
		2, {0x8E, 0x57},
		3, {0xE7, 0x87, 0xA6},
		2, {0xE6, 0x71},
		2, {0xBB, 0xB8}
	},
	{
		2, {0x8E, 0x58},
		3, {0xE7, 0x8F, 0x8A},
		2, {0xCA, 0x73},
		2, {0xBB, 0xB9}
	},
	{
		2, {0x8E, 0x59},
		3, {0xE7, 0x94, 0xA3},
		2, {0x23, 0x75},
		2, {0xBB, 0xBA}
	},
	{
		2, {0x8E, 0x5A},
		3, {0xE7, 0xAE, 0x97},
		2, {0x97, 0x7B},
		2, {0xBB, 0xBB}
	},
	{
		2, {0x8E, 0x5B},
		3, {0xE7, 0xBA, 0x82},
		2, {0x82, 0x7E},
		2, {0xBB, 0xBC}
	},
	{
		2, {0x8E, 0x5C},
		3, {0xE8, 0x9A, 0x95},
		2, {0x95, 0x86},
		2, {0xBB, 0xBD}
	},
	{
		2, {0x8E, 0x5D},
		3, {0xE8, 0xAE, 0x83},
		2, {0x83, 0x8B},
		2, {0xBB, 0xBE}
	},
	{
		2, {0x8E, 0x5E},
		3, {0xE8, 0xB3, 0x9B},
		2, {0xDB, 0x8C},
		2, {0xBB, 0xBF}
	},
	{
		2, {0x8E, 0x5F},
		3, {0xE9, 0x85, 0xB8},
		2, {0x78, 0x91},
		2, {0xBB, 0xC0}
	},
	{
		2, {0x8E, 0x60},
		3, {0xE9, 0xA4, 0x90},
		2, {0x10, 0x99},
		2, {0xBB, 0xC1}
	},
	{
		2, {0x8E, 0x61},
		3, {0xE6, 0x96, 0xAC},
		2, {0xAC, 0x65},
		2, {0xBB, 0xC2}
	},
	{
		2, {0x8E, 0x62},
		3, {0xE6, 0x9A, 0xAB},
		2, {0xAB, 0x66},
		2, {0xBB, 0xC3}
	},
	{
		2, {0x8E, 0x63},
		3, {0xE6, 0xAE, 0x8B},
		2, {0x8B, 0x6B},
		2, {0xBB, 0xC4}
	},
	{
		2, {0x8E, 0x64},
		3, {0xE4, 0xBB, 0x95},
		2, {0xD5, 0x4E},
		2, {0xBB, 0xC5}
	},
	{
		2, {0x8E, 0x65},
		3, {0xE4, 0xBB, 0x94},
		2, {0xD4, 0x4E},
		2, {0xBB, 0xC6}
	},
	{
		2, {0x8E, 0x66},
		3, {0xE4, 0xBC, 0xBA},
		2, {0x3A, 0x4F},
		2, {0xBB, 0xC7}
	},
	{
		2, {0x8E, 0x67},
		3, {0xE4, 0xBD, 0xBF},
		2, {0x7F, 0x4F},
		2, {0xBB, 0xC8}
	},
	{
		2, {0x8E, 0x68},
		3, {0xE5, 0x88, 0xBA},
		2, {0x3A, 0x52},
		2, {0xBB, 0xC9}
	},
	{
		2, {0x8E, 0x69},
		3, {0xE5, 0x8F, 0xB8},
		2, {0xF8, 0x53},
		2, {0xBB, 0xCA}
	},
	{
		2, {0x8E, 0x6A},
		3, {0xE5, 0x8F, 0xB2},
		2, {0xF2, 0x53},
		2, {0xBB, 0xCB}
	},
	{
		2, {0x8E, 0x6B},
		3, {0xE5, 0x97, 0xA3},
		2, {0xE3, 0x55},
		2, {0xBB, 0xCC}
	},
	{
		2, {0x8E, 0x6C},
		3, {0xE5, 0x9B, 0x9B},
		2, {0xDB, 0x56},
		2, {0xBB, 0xCD}
	},
	{
		2, {0x8E, 0x6D},
		3, {0xE5, 0xA3, 0xAB},
		2, {0xEB, 0x58},
		2, {0xBB, 0xCE}
	},
	{
		2, {0x8E, 0x6E},
		3, {0xE5, 0xA7, 0x8B},
		2, {0xCB, 0x59},
		2, {0xBB, 0xCF}
	},
	{
		2, {0x8E, 0x6F},
		3, {0xE5, 0xA7, 0x89},
		2, {0xC9, 0x59},
		2, {0xBB, 0xD0}
	},
	{
		2, {0x8E, 0x70},
		3, {0xE5, 0xA7, 0xBF},
		2, {0xFF, 0x59},
		2, {0xBB, 0xD1}
	},
	{
		2, {0x8E, 0x71},
		3, {0xE5, 0xAD, 0x90},
		2, {0x50, 0x5B},
		2, {0xBB, 0xD2}
	},
	{
		2, {0x8E, 0x72},
		3, {0xE5, 0xB1, 0x8D},
		2, {0x4D, 0x5C},
		2, {0xBB, 0xD3}
	},
	{
		2, {0x8E, 0x73},
		3, {0xE5, 0xB8, 0x82},
		2, {0x02, 0x5E},
		2, {0xBB, 0xD4}
	},
	{
		2, {0x8E, 0x74},
		3, {0xE5, 0xB8, 0xAB},
		2, {0x2B, 0x5E},
		2, {0xBB, 0xD5}
	},
	{
		2, {0x8E, 0x75},
		3, {0xE5, 0xBF, 0x97},
		2, {0xD7, 0x5F},
		2, {0xBB, 0xD6}
	},
	{
		2, {0x8E, 0x76},
		3, {0xE6, 0x80, 0x9D},
		2, {0x1D, 0x60},
		2, {0xBB, 0xD7}
	},
	{
		2, {0x8E, 0x77},
		3, {0xE6, 0x8C, 0x87},
		2, {0x07, 0x63},
		2, {0xBB, 0xD8}
	},
	{
		2, {0x8E, 0x78},
		3, {0xE6, 0x94, 0xAF},
		2, {0x2F, 0x65},
		2, {0xBB, 0xD9}
	},
	{
		2, {0x8E, 0x79},
		3, {0xE5, 0xAD, 0x9C},
		2, {0x5C, 0x5B},
		2, {0xBB, 0xDA}
	},
	{
		2, {0x8E, 0x7A},
		3, {0xE6, 0x96, 0xAF},
		2, {0xAF, 0x65},
		2, {0xBB, 0xDB}
	},
	{
		2, {0x8E, 0x7B},
		3, {0xE6, 0x96, 0xBD},
		2, {0xBD, 0x65},
		2, {0xBB, 0xDC}
	},
	{
		2, {0x8E, 0x7C},
		3, {0xE6, 0x97, 0xA8},
		2, {0xE8, 0x65},
		2, {0xBB, 0xDD}
	},
	{
		2, {0x8E, 0x7D},
		3, {0xE6, 0x9E, 0x9D},
		2, {0x9D, 0x67},
		2, {0xBB, 0xDE}
	},
	{
		2, {0x8E, 0x7E},
		3, {0xE6, 0xAD, 0xA2},
		2, {0x62, 0x6B},
		2, {0xBB, 0xDF}
	},
	{
		2, {0x8E, 0x80},
		3, {0xE6, 0xAD, 0xBB},
		2, {0x7B, 0x6B},
		2, {0xBB, 0xE0}
	},
	{
		2, {0x8E, 0x81},
		3, {0xE6, 0xB0, 0x8F},
		2, {0x0F, 0x6C},
		2, {0xBB, 0xE1}
	},
	{
		2, {0x8E, 0x82},
		3, {0xE7, 0x8D, 0x85},
		2, {0x45, 0x73},
		2, {0xBB, 0xE2}
	},
	{
		2, {0x8E, 0x83},
		3, {0xE7, 0xA5, 0x89},
		2, {0x49, 0x79},
		2, {0xBB, 0xE3}
	},
	{
		2, {0x8E, 0x84},
		3, {0xE7, 0xA7, 0x81},
		2, {0xC1, 0x79},
		2, {0xBB, 0xE4}
	},
	{
		2, {0x8E, 0x85},
		3, {0xE7, 0xB3, 0xB8},
		2, {0xF8, 0x7C},
		2, {0xBB, 0xE5}
	},
	{
		2, {0x8E, 0x86},
		3, {0xE7, 0xB4, 0x99},
		2, {0x19, 0x7D},
		2, {0xBB, 0xE6}
	},
	{
		2, {0x8E, 0x87},
		3, {0xE7, 0xB4, 0xAB},
		2, {0x2B, 0x7D},
		2, {0xBB, 0xE7}
	},
	{
		2, {0x8E, 0x88},
		3, {0xE8, 0x82, 0xA2},
		2, {0xA2, 0x80},
		2, {0xBB, 0xE8}
	},
	{
		2, {0x8E, 0x89},
		3, {0xE8, 0x84, 0x82},
		2, {0x02, 0x81},
		2, {0xBB, 0xE9}
	},
	{
		2, {0x8E, 0x8A},
		3, {0xE8, 0x87, 0xB3},
		2, {0xF3, 0x81},
		2, {0xBB, 0xEA}
	},
	{
		2, {0x8E, 0x8B},
		3, {0xE8, 0xA6, 0x96},
		2, {0x96, 0x89},
		2, {0xBB, 0xEB}
	},
	{
		2, {0x8E, 0x8C},
		3, {0xE8, 0xA9, 0x9E},
		2, {0x5E, 0x8A},
		2, {0xBB, 0xEC}
	},
	{
		2, {0x8E, 0x8D},
		3, {0xE8, 0xA9, 0xA9},
		2, {0x69, 0x8A},
		2, {0xBB, 0xED}
	},
	{
		2, {0x8E, 0x8E},
		3, {0xE8, 0xA9, 0xA6},
		2, {0x66, 0x8A},
		2, {0xBB, 0xEE}
	},
	{
		2, {0x8E, 0x8F},
		3, {0xE8, 0xAA, 0x8C},
		2, {0x8C, 0x8A},
		2, {0xBB, 0xEF}
	},
	{
		2, {0x8E, 0x90},
		3, {0xE8, 0xAB, 0xAE},
		2, {0xEE, 0x8A},
		2, {0xBB, 0xF0}
	},
	{
		2, {0x8E, 0x91},
		3, {0xE8, 0xB3, 0x87},
		2, {0xC7, 0x8C},
		2, {0xBB, 0xF1}
	},
	{
		2, {0x8E, 0x92},
		3, {0xE8, 0xB3, 0x9C},
		2, {0xDC, 0x8C},
		2, {0xBB, 0xF2}
	},
	{
		2, {0x8E, 0x93},
		3, {0xE9, 0x9B, 0x8C},
		2, {0xCC, 0x96},
		2, {0xBB, 0xF3}
	},
	{
		2, {0x8E, 0x94},
		3, {0xE9, 0xA3, 0xBC},
		2, {0xFC, 0x98},
		2, {0xBB, 0xF4}
	},
	{
		2, {0x8E, 0x95},
		3, {0xE6, 0xAD, 0xAF},
		2, {0x6F, 0x6B},
		2, {0xBB, 0xF5}
	},
	{
		2, {0x8E, 0x96},
		3, {0xE4, 0xBA, 0x8B},
		2, {0x8B, 0x4E},
		2, {0xBB, 0xF6}
	},
	{
		2, {0x8E, 0x97},
		3, {0xE4, 0xBC, 0xBC},
		2, {0x3C, 0x4F},
		2, {0xBB, 0xF7}
	},
	{
		2, {0x8E, 0x98},
		3, {0xE4, 0xBE, 0x8D},
		2, {0x8D, 0x4F},
		2, {0xBB, 0xF8}
	},
	{
		2, {0x8E, 0x99},
		3, {0xE5, 0x85, 0x90},
		2, {0x50, 0x51},
		2, {0xBB, 0xF9}
	},
	{
		2, {0x8E, 0x9A},
		3, {0xE5, 0xAD, 0x97},
		2, {0x57, 0x5B},
		2, {0xBB, 0xFA}
	},
	{
		2, {0x8E, 0x9B},
		3, {0xE5, 0xAF, 0xBA},
		2, {0xFA, 0x5B},
		2, {0xBB, 0xFB}
	},
	{
		2, {0x8E, 0x9C},
		3, {0xE6, 0x85, 0x88},
		2, {0x48, 0x61},
		2, {0xBB, 0xFC}
	},
	{
		2, {0x8E, 0x9D},
		3, {0xE6, 0x8C, 0x81},
		2, {0x01, 0x63},
		2, {0xBB, 0xFD}
	},
	{
		2, {0x8E, 0x9E},
		3, {0xE6, 0x99, 0x82},
		2, {0x42, 0x66},
		2, {0xBB, 0xFE}
	},
	{
		2, {0x8E, 0x9F},
		3, {0xE6, 0xAC, 0xA1},
		2, {0x21, 0x6B},
		2, {0xBC, 0xA1}
	},
	{
		2, {0x8E, 0xA0},
		3, {0xE6, 0xBB, 0x8B},
		2, {0xCB, 0x6E},
		2, {0xBC, 0xA2}
	},
	{
		2, {0x8E, 0xA1},
		3, {0xE6, 0xB2, 0xBB},
		2, {0xBB, 0x6C},
		2, {0xBC, 0xA3}
	},
	{
		2, {0x8E, 0xA2},
		3, {0xE7, 0x88, 0xBE},
		2, {0x3E, 0x72},
		2, {0xBC, 0xA4}
	},
	{
		2, {0x8E, 0xA3},
		3, {0xE7, 0x92, 0xBD},
		2, {0xBD, 0x74},
		2, {0xBC, 0xA5}
	},
	{
		2, {0x8E, 0xA4},
		3, {0xE7, 0x97, 0x94},
		2, {0xD4, 0x75},
		2, {0xBC, 0xA6}
	},
	{
		2, {0x8E, 0xA5},
		3, {0xE7, 0xA3, 0x81},
		2, {0xC1, 0x78},
		2, {0xBC, 0xA7}
	},
	{
		2, {0x8E, 0xA6},
		3, {0xE7, 0xA4, 0xBA},
		2, {0x3A, 0x79},
		2, {0xBC, 0xA8}
	},
	{
		2, {0x8E, 0xA7},
		3, {0xE8, 0x80, 0x8C},
		2, {0x0C, 0x80},
		2, {0xBC, 0xA9}
	},
	{
		2, {0x8E, 0xA8},
		3, {0xE8, 0x80, 0xB3},
		2, {0x33, 0x80},
		2, {0xBC, 0xAA}
	},
	{
		2, {0x8E, 0xA9},
		3, {0xE8, 0x87, 0xAA},
		2, {0xEA, 0x81},
		2, {0xBC, 0xAB}
	},
	{
		2, {0x8E, 0xAA},
		3, {0xE8, 0x92, 0x94},
		2, {0x94, 0x84},
		2, {0xBC, 0xAC}
	},
	{
		2, {0x8E, 0xAB},
		3, {0xE8, 0xBE, 0x9E},
		2, {0x9E, 0x8F},
		2, {0xBC, 0xAD}
	},
	{
		2, {0x8E, 0xAC},
		3, {0xE6, 0xB1, 0x90},
		2, {0x50, 0x6C},
		2, {0xBC, 0xAE}
	},
	{
		2, {0x8E, 0xAD},
		3, {0xE9, 0xB9, 0xBF},
		2, {0x7F, 0x9E},
		2, {0xBC, 0xAF}
	},
	{
		2, {0x8E, 0xAE},
		3, {0xE5, 0xBC, 0x8F},
		2, {0x0F, 0x5F},
		2, {0xBC, 0xB0}
	},
	{
		2, {0x8E, 0xAF},
		3, {0xE8, 0xAD, 0x98},
		2, {0x58, 0x8B},
		2, {0xBC, 0xB1}
	},
	{
		2, {0x8E, 0xB0},
		3, {0xE9, 0xB4, 0xAB},
		2, {0x2B, 0x9D},
		2, {0xBC, 0xB2}
	},
	{
		2, {0x8E, 0xB1},
		3, {0xE7, 0xAB, 0xBA},
		2, {0xFA, 0x7A},
		2, {0xBC, 0xB3}
	},
	{
		2, {0x8E, 0xB2},
		3, {0xE8, 0xBB, 0xB8},
		2, {0xF8, 0x8E},
		2, {0xBC, 0xB4}
	},
	{
		2, {0x8E, 0xB3},
		3, {0xE5, 0xAE, 0x8D},
		2, {0x8D, 0x5B},
		2, {0xBC, 0xB5}
	},
	{
		2, {0x8E, 0xB4},
		3, {0xE9, 0x9B, 0xAB},
		2, {0xEB, 0x96},
		2, {0xBC, 0xB6}
	},
	{
		2, {0x8E, 0xB5},
		3, {0xE4, 0xB8, 0x83},
		2, {0x03, 0x4E},
		2, {0xBC, 0xB7}
	},
	{
		2, {0x8E, 0xB6},
		3, {0xE5, 0x8F, 0xB1},
		2, {0xF1, 0x53},
		2, {0xBC, 0xB8}
	},
	{
		2, {0x8E, 0xB7},
		3, {0xE5, 0x9F, 0xB7},
		2, {0xF7, 0x57},
		2, {0xBC, 0xB9}
	},
	{
		2, {0x8E, 0xB8},
		3, {0xE5, 0xA4, 0xB1},
		2, {0x31, 0x59},
		2, {0xBC, 0xBA}
	},
	{
		2, {0x8E, 0xB9},
		3, {0xE5, 0xAB, 0x89},
		2, {0xC9, 0x5A},
		2, {0xBC, 0xBB}
	},
	{
		2, {0x8E, 0xBA},
		3, {0xE5, 0xAE, 0xA4},
		2, {0xA4, 0x5B},
		2, {0xBC, 0xBC}
	},
	{
		2, {0x8E, 0xBB},
		3, {0xE6, 0x82, 0x89},
		2, {0x89, 0x60},
		2, {0xBC, 0xBD}
	},
	{
		2, {0x8E, 0xBC},
		3, {0xE6, 0xB9, 0xBF},
		2, {0x7F, 0x6E},
		2, {0xBC, 0xBE}
	},
	{
		2, {0x8E, 0xBD},
		3, {0xE6, 0xBC, 0x86},
		2, {0x06, 0x6F},
		2, {0xBC, 0xBF}
	},
	{
		2, {0x8E, 0xBE},
		3, {0xE7, 0x96, 0xBE},
		2, {0xBE, 0x75},
		2, {0xBC, 0xC0}
	},
	{
		2, {0x8E, 0xBF},
		3, {0xE8, 0xB3, 0xAA},
		2, {0xEA, 0x8C},
		2, {0xBC, 0xC1}
	},
	{
		2, {0x8E, 0xC0},
		3, {0xE5, 0xAE, 0x9F},
		2, {0x9F, 0x5B},
		2, {0xBC, 0xC2}
	},
	{
		2, {0x8E, 0xC1},
		3, {0xE8, 0x94, 0x80},
		2, {0x00, 0x85},
		2, {0xBC, 0xC3}
	},
	{
		2, {0x8E, 0xC2},
		3, {0xE7, 0xAF, 0xA0},
		2, {0xE0, 0x7B},
		2, {0xBC, 0xC4}
	},
	{
		2, {0x8E, 0xC3},
		3, {0xE5, 0x81, 0xB2},
		2, {0x72, 0x50},
		2, {0xBC, 0xC5}
	},
	{
		2, {0x8E, 0xC4},
		3, {0xE6, 0x9F, 0xB4},
		2, {0xF4, 0x67},
		2, {0xBC, 0xC6}
	},
	{
		2, {0x8E, 0xC5},
		3, {0xE8, 0x8A, 0x9D},
		2, {0x9D, 0x82},
		2, {0xBC, 0xC7}
	},
	{
		2, {0x8E, 0xC6},
		3, {0xE5, 0xB1, 0xA1},
		2, {0x61, 0x5C},
		2, {0xBC, 0xC8}
	},
	{
		2, {0x8E, 0xC7},
		3, {0xE8, 0x95, 0x8A},
		2, {0x4A, 0x85},
		2, {0xBC, 0xC9}
	},
	{
		2, {0x8E, 0xC8},
		3, {0xE7, 0xB8, 0x9E},
		2, {0x1E, 0x7E},
		2, {0xBC, 0xCA}
	},
	{
		2, {0x8E, 0xC9},
		3, {0xE8, 0x88, 0x8E},
		2, {0x0E, 0x82},
		2, {0xBC, 0xCB}
	},
	{
		2, {0x8E, 0xCA},
		3, {0xE5, 0x86, 0x99},
		2, {0x99, 0x51},
		2, {0xBC, 0xCC}
	},
	{
		2, {0x8E, 0xCB},
		3, {0xE5, 0xB0, 0x84},
		2, {0x04, 0x5C},
		2, {0xBC, 0xCD}
	},
	{
		2, {0x8E, 0xCC},
		3, {0xE6, 0x8D, 0xA8},
		2, {0x68, 0x63},
		2, {0xBC, 0xCE}
	},
	{
		2, {0x8E, 0xCD},
		3, {0xE8, 0xB5, 0xA6},
		2, {0x66, 0x8D},
		2, {0xBC, 0xCF}
	},
	{
		2, {0x8E, 0xCE},
		3, {0xE6, 0x96, 0x9C},
		2, {0x9C, 0x65},
		2, {0xBC, 0xD0}
	},
	{
		2, {0x8E, 0xCF},
		3, {0xE7, 0x85, 0xAE},
		2, {0x6E, 0x71},
		2, {0xBC, 0xD1}
	},
	{
		2, {0x8E, 0xD0},
		3, {0xE7, 0xA4, 0xBE},
		2, {0x3E, 0x79},
		2, {0xBC, 0xD2}
	},
	{
		2, {0x8E, 0xD1},
		3, {0xE7, 0xB4, 0x97},
		2, {0x17, 0x7D},
		2, {0xBC, 0xD3}
	},
	{
		2, {0x8E, 0xD2},
		3, {0xE8, 0x80, 0x85},
		2, {0x05, 0x80},
		2, {0xBC, 0xD4}
	},
	{
		2, {0x8E, 0xD3},
		3, {0xE8, 0xAC, 0x9D},
		2, {0x1D, 0x8B},
		2, {0xBC, 0xD5}
	},
	{
		2, {0x8E, 0xD4},
		3, {0xE8, 0xBB, 0x8A},
		2, {0xCA, 0x8E},
		2, {0xBC, 0xD6}
	},
	{
		2, {0x8E, 0xD5},
		3, {0xE9, 0x81, 0xAE},
		2, {0x6E, 0x90},
		2, {0xBC, 0xD7}
	},
	{
		2, {0x8E, 0xD6},
		3, {0xE8, 0x9B, 0x87},
		2, {0xC7, 0x86},
		2, {0xBC, 0xD8}
	},
	{
		2, {0x8E, 0xD7},
		3, {0xE9, 0x82, 0xAA},
		2, {0xAA, 0x90},
		2, {0xBC, 0xD9}
	},
	{
		2, {0x8E, 0xD8},
		3, {0xE5, 0x80, 0x9F},
		2, {0x1F, 0x50},
		2, {0xBC, 0xDA}
	},
	{
		2, {0x8E, 0xD9},
		3, {0xE5, 0x8B, 0xBA},
		2, {0xFA, 0x52},
		2, {0xBC, 0xDB}
	},
	{
		2, {0x8E, 0xDA},
		3, {0xE5, 0xB0, 0xBA},
		2, {0x3A, 0x5C},
		2, {0xBC, 0xDC}
	},
	{
		2, {0x8E, 0xDB},
		3, {0xE6, 0x9D, 0x93},
		2, {0x53, 0x67},
		2, {0xBC, 0xDD}
	},
	{
		2, {0x8E, 0xDC},
		3, {0xE7, 0x81, 0xBC},
		2, {0x7C, 0x70},
		2, {0xBC, 0xDE}
	},
	{
		2, {0x8E, 0xDD},
		3, {0xE7, 0x88, 0xB5},
		2, {0x35, 0x72},
		2, {0xBC, 0xDF}
	},
	{
		2, {0x8E, 0xDE},
		3, {0xE9, 0x85, 0x8C},
		2, {0x4C, 0x91},
		2, {0xBC, 0xE0}
	},
	{
		2, {0x8E, 0xDF},
		3, {0xE9, 0x87, 0x88},
		2, {0xC8, 0x91},
		2, {0xBC, 0xE1}
	},
	{
		2, {0x8E, 0xE0},
		3, {0xE9, 0x8C, 0xAB},
		2, {0x2B, 0x93},
		2, {0xBC, 0xE2}
	},
	{
		2, {0x8E, 0xE1},
		3, {0xE8, 0x8B, 0xA5},
		2, {0xE5, 0x82},
		2, {0xBC, 0xE3}
	},
	{
		2, {0x8E, 0xE2},
		3, {0xE5, 0xAF, 0x82},
		2, {0xC2, 0x5B},
		2, {0xBC, 0xE4}
	},
	{
		2, {0x8E, 0xE3},
		3, {0xE5, 0xBC, 0xB1},
		2, {0x31, 0x5F},
		2, {0xBC, 0xE5}
	},
	{
		2, {0x8E, 0xE4},
		3, {0xE6, 0x83, 0xB9},
		2, {0xF9, 0x60},
		2, {0xBC, 0xE6}
	},
	{
		2, {0x8E, 0xE5},
		3, {0xE4, 0xB8, 0xBB},
		2, {0x3B, 0x4E},
		2, {0xBC, 0xE7}
	},
	{
		2, {0x8E, 0xE6},
		3, {0xE5, 0x8F, 0x96},
		2, {0xD6, 0x53},
		2, {0xBC, 0xE8}
	},
	{
		2, {0x8E, 0xE7},
		3, {0xE5, 0xAE, 0x88},
		2, {0x88, 0x5B},
		2, {0xBC, 0xE9}
	},
	{
		2, {0x8E, 0xE8},
		3, {0xE6, 0x89, 0x8B},
		2, {0x4B, 0x62},
		2, {0xBC, 0xEA}
	},
	{
		2, {0x8E, 0xE9},
		3, {0xE6, 0x9C, 0xB1},
		2, {0x31, 0x67},
		2, {0xBC, 0xEB}
	},
	{
		2, {0x8E, 0xEA},
		3, {0xE6, 0xAE, 0x8A},
		2, {0x8A, 0x6B},
		2, {0xBC, 0xEC}
	},
	{
		2, {0x8E, 0xEB},
		3, {0xE7, 0x8B, 0xA9},
		2, {0xE9, 0x72},
		2, {0xBC, 0xED}
	},
	{
		2, {0x8E, 0xEC},
		3, {0xE7, 0x8F, 0xA0},
		2, {0xE0, 0x73},
		2, {0xBC, 0xEE}
	},
	{
		2, {0x8E, 0xED},
		3, {0xE7, 0xA8, 0xAE},
		2, {0x2E, 0x7A},
		2, {0xBC, 0xEF}
	},
	{
		2, {0x8E, 0xEE},
		3, {0xE8, 0x85, 0xAB},
		2, {0x6B, 0x81},
		2, {0xBC, 0xF0}
	},
	{
		2, {0x8E, 0xEF},
		3, {0xE8, 0xB6, 0xA3},
		2, {0xA3, 0x8D},
		2, {0xBC, 0xF1}
	},
	{
		2, {0x8E, 0xF0},
		3, {0xE9, 0x85, 0x92},
		2, {0x52, 0x91},
		2, {0xBC, 0xF2}
	},
	{
		2, {0x8E, 0xF1},
		3, {0xE9, 0xA6, 0x96},
		2, {0x96, 0x99},
		2, {0xBC, 0xF3}
	},
	{
		2, {0x8E, 0xF2},
		3, {0xE5, 0x84, 0x92},
		2, {0x12, 0x51},
		2, {0xBC, 0xF4}
	},
	{
		2, {0x8E, 0xF3},
		3, {0xE5, 0x8F, 0x97},
		2, {0xD7, 0x53},
		2, {0xBC, 0xF5}
	},
	{
		2, {0x8E, 0xF4},
		3, {0xE5, 0x91, 0xAA},
		2, {0x6A, 0x54},
		2, {0xBC, 0xF6}
	},
	{
		2, {0x8E, 0xF5},
		3, {0xE5, 0xAF, 0xBF},
		2, {0xFF, 0x5B},
		2, {0xBC, 0xF7}
	},
	{
		2, {0x8E, 0xF6},
		3, {0xE6, 0x8E, 0x88},
		2, {0x88, 0x63},
		2, {0xBC, 0xF8}
	},
	{
		2, {0x8E, 0xF7},
		3, {0xE6, 0xA8, 0xB9},
		2, {0x39, 0x6A},
		2, {0xBC, 0xF9}
	},
	{
		2, {0x8E, 0xF8},
		3, {0xE7, 0xB6, 0xAC},
		2, {0xAC, 0x7D},
		2, {0xBC, 0xFA}
	},
	{
		2, {0x8E, 0xF9},
		3, {0xE9, 0x9C, 0x80},
		2, {0x00, 0x97},
		2, {0xBC, 0xFB}
	},
	{
		2, {0x8E, 0xFA},
		3, {0xE5, 0x9B, 0x9A},
		2, {0xDA, 0x56},
		2, {0xBC, 0xFC}
	},
	{
		2, {0x8E, 0xFB},
		3, {0xE5, 0x8F, 0x8E},
		2, {0xCE, 0x53},
		2, {0xBC, 0xFD}
	},
	{
		2, {0x8E, 0xFC},
		3, {0xE5, 0x91, 0xA8},
		2, {0x68, 0x54},
		2, {0xBC, 0xFE}
	},
	{
		2, {0x8F, 0x40},
		3, {0xE5, 0xAE, 0x97},
		2, {0x97, 0x5B},
		2, {0xBD, 0xA1}
	},
	{
		2, {0x8F, 0x41},
		3, {0xE5, 0xB0, 0xB1},
		2, {0x31, 0x5C},
		2, {0xBD, 0xA2}
	},
	{
		2, {0x8F, 0x42},
		3, {0xE5, 0xB7, 0x9E},
		2, {0xDE, 0x5D},
		2, {0xBD, 0xA3}
	},
	{
		2, {0x8F, 0x43},
		3, {0xE4, 0xBF, 0xAE},
		2, {0xEE, 0x4F},
		2, {0xBD, 0xA4}
	},
	{
		2, {0x8F, 0x44},
		3, {0xE6, 0x84, 0x81},
		2, {0x01, 0x61},
		2, {0xBD, 0xA5}
	},
	{
		2, {0x8F, 0x45},
		3, {0xE6, 0x8B, 0xBE},
		2, {0xFE, 0x62},
		2, {0xBD, 0xA6}
	},
	{
		2, {0x8F, 0x46},
		3, {0xE6, 0xB4, 0xB2},
		2, {0x32, 0x6D},
		2, {0xBD, 0xA7}
	},
	{
		2, {0x8F, 0x47},
		3, {0xE7, 0xA7, 0x80},
		2, {0xC0, 0x79},
		2, {0xBD, 0xA8}
	},
	{
		2, {0x8F, 0x48},
		3, {0xE7, 0xA7, 0x8B},
		2, {0xCB, 0x79},
		2, {0xBD, 0xA9}
	},
	{
		2, {0x8F, 0x49},
		3, {0xE7, 0xB5, 0x82},
		2, {0x42, 0x7D},
		2, {0xBD, 0xAA}
	},
	{
		2, {0x8F, 0x4A},
		3, {0xE7, 0xB9, 0x8D},
		2, {0x4D, 0x7E},
		2, {0xBD, 0xAB}
	},
	{
		2, {0x8F, 0x4B},
		3, {0xE7, 0xBF, 0x92},
		2, {0xD2, 0x7F},
		2, {0xBD, 0xAC}
	},
	{
		2, {0x8F, 0x4C},
		3, {0xE8, 0x87, 0xAD},
		2, {0xED, 0x81},
		2, {0xBD, 0xAD}
	},
	{
		2, {0x8F, 0x4D},
		3, {0xE8, 0x88, 0x9F},
		2, {0x1F, 0x82},
		2, {0xBD, 0xAE}
	},
	{
		2, {0x8F, 0x4E},
		3, {0xE8, 0x92, 0x90},
		2, {0x90, 0x84},
		2, {0xBD, 0xAF}
	},
	{
		2, {0x8F, 0x4F},
		3, {0xE8, 0xA1, 0x86},
		2, {0x46, 0x88},
		2, {0xBD, 0xB0}
	},
	{
		2, {0x8F, 0x50},
		3, {0xE8, 0xA5, 0xB2},
		2, {0x72, 0x89},
		2, {0xBD, 0xB1}
	},
	{
		2, {0x8F, 0x51},
		3, {0xE8, 0xAE, 0x90},
		2, {0x90, 0x8B},
		2, {0xBD, 0xB2}
	},
	{
		2, {0x8F, 0x52},
		3, {0xE8, 0xB9, 0xB4},
		2, {0x74, 0x8E},
		2, {0xBD, 0xB3}
	},
	{
		2, {0x8F, 0x53},
		3, {0xE8, 0xBC, 0xAF},
		2, {0x2F, 0x8F},
		2, {0xBD, 0xB4}
	},
	{
		2, {0x8F, 0x54},
		3, {0xE9, 0x80, 0xB1},
		2, {0x31, 0x90},
		2, {0xBD, 0xB5}
	},
	{
		2, {0x8F, 0x55},
		3, {0xE9, 0x85, 0x8B},
		2, {0x4B, 0x91},
		2, {0xBD, 0xB6}
	},
	{
		2, {0x8F, 0x56},
		3, {0xE9, 0x85, 0xAC},
		2, {0x6C, 0x91},
		2, {0xBD, 0xB7}
	},
	{
		2, {0x8F, 0x57},
		3, {0xE9, 0x9B, 0x86},
		2, {0xC6, 0x96},
		2, {0xBD, 0xB8}
	},
	{
		2, {0x8F, 0x58},
		3, {0xE9, 0x86, 0x9C},
		2, {0x9C, 0x91},
		2, {0xBD, 0xB9}
	},
	{
		2, {0x8F, 0x59},
		3, {0xE4, 0xBB, 0x80},
		2, {0xC0, 0x4E},
		2, {0xBD, 0xBA}
	},
	{
		2, {0x8F, 0x5A},
		3, {0xE4, 0xBD, 0x8F},
		2, {0x4F, 0x4F},
		2, {0xBD, 0xBB}
	},
	{
		2, {0x8F, 0x5B},
		3, {0xE5, 0x85, 0x85},
		2, {0x45, 0x51},
		2, {0xBD, 0xBC}
	},
	{
		2, {0x8F, 0x5C},
		3, {0xE5, 0x8D, 0x81},
		2, {0x41, 0x53},
		2, {0xBD, 0xBD}
	},
	{
		2, {0x8F, 0x5D},
		3, {0xE5, 0xBE, 0x93},
		2, {0x93, 0x5F},
		2, {0xBD, 0xBE}
	},
	{
		2, {0x8F, 0x5E},
		3, {0xE6, 0x88, 0x8E},
		2, {0x0E, 0x62},
		2, {0xBD, 0xBF}
	},
	{
		2, {0x8F, 0x5F},
		3, {0xE6, 0x9F, 0x94},
		2, {0xD4, 0x67},
		2, {0xBD, 0xC0}
	},
	{
		2, {0x8F, 0x60},
		3, {0xE6, 0xB1, 0x81},
		2, {0x41, 0x6C},
		2, {0xBD, 0xC1}
	},
	{
		2, {0x8F, 0x61},
		3, {0xE6, 0xB8, 0x8B},
		2, {0x0B, 0x6E},
		2, {0xBD, 0xC2}
	},
	{
		2, {0x8F, 0x62},
		3, {0xE7, 0x8D, 0xA3},
		2, {0x63, 0x73},
		2, {0xBD, 0xC3}
	},
	{
		2, {0x8F, 0x63},
		3, {0xE7, 0xB8, 0xA6},
		2, {0x26, 0x7E},
		2, {0xBD, 0xC4}
	},
	{
		2, {0x8F, 0x64},
		3, {0xE9, 0x87, 0x8D},
		2, {0xCD, 0x91},
		2, {0xBD, 0xC5}
	},
	{
		2, {0x8F, 0x65},
		3, {0xE9, 0x8A, 0x83},
		2, {0x83, 0x92},
		2, {0xBD, 0xC6}
	},
	{
		2, {0x8F, 0x66},
		3, {0xE5, 0x8F, 0x94},
		2, {0xD4, 0x53},
		2, {0xBD, 0xC7}
	},
	{
		2, {0x8F, 0x67},
		3, {0xE5, 0xA4, 0x99},
		2, {0x19, 0x59},
		2, {0xBD, 0xC8}
	},
	{
		2, {0x8F, 0x68},
		3, {0xE5, 0xAE, 0xBF},
		2, {0xBF, 0x5B},
		2, {0xBD, 0xC9}
	},
	{
		2, {0x8F, 0x69},
		3, {0xE6, 0xB7, 0x91},
		2, {0xD1, 0x6D},
		2, {0xBD, 0xCA}
	},
	{
		2, {0x8F, 0x6A},
		3, {0xE7, 0xA5, 0x9D},
		2, {0x5D, 0x79},
		2, {0xBD, 0xCB}
	},
	{
		2, {0x8F, 0x6B},
		3, {0xE7, 0xB8, 0xAE},
		2, {0x2E, 0x7E},
		2, {0xBD, 0xCC}
	},
	{
		2, {0x8F, 0x6C},
		3, {0xE7, 0xB2, 0x9B},
		2, {0x9B, 0x7C},
		2, {0xBD, 0xCD}
	},
	{
		2, {0x8F, 0x6D},
		3, {0xE5, 0xA1, 0xBE},
		2, {0x7E, 0x58},
		2, {0xBD, 0xCE}
	},
	{
		2, {0x8F, 0x6E},
		3, {0xE7, 0x86, 0x9F},
		2, {0x9F, 0x71},
		2, {0xBD, 0xCF}
	},
	{
		2, {0x8F, 0x6F},
		3, {0xE5, 0x87, 0xBA},
		2, {0xFA, 0x51},
		2, {0xBD, 0xD0}
	},
	{
		2, {0x8F, 0x70},
		3, {0xE8, 0xA1, 0x93},
		2, {0x53, 0x88},
		2, {0xBD, 0xD1}
	},
	{
		2, {0x8F, 0x71},
		3, {0xE8, 0xBF, 0xB0},
		2, {0xF0, 0x8F},
		2, {0xBD, 0xD2}
	},
	{
		2, {0x8F, 0x72},
		3, {0xE4, 0xBF, 0x8A},
		2, {0xCA, 0x4F},
		2, {0xBD, 0xD3}
	},
	{
		2, {0x8F, 0x73},
		3, {0xE5, 0xB3, 0xBB},
		2, {0xFB, 0x5C},
		2, {0xBD, 0xD4}
	},
	{
		2, {0x8F, 0x74},
		3, {0xE6, 0x98, 0xA5},
		2, {0x25, 0x66},
		2, {0xBD, 0xD5}
	},
	{
		2, {0x8F, 0x75},
		3, {0xE7, 0x9E, 0xAC},
		2, {0xAC, 0x77},
		2, {0xBD, 0xD6}
	},
	{
		2, {0x8F, 0x76},
		3, {0xE7, 0xAB, 0xA3},
		2, {0xE3, 0x7A},
		2, {0xBD, 0xD7}
	},
	{
		2, {0x8F, 0x77},
		3, {0xE8, 0x88, 0x9C},
		2, {0x1C, 0x82},
		2, {0xBD, 0xD8}
	},
	{
		2, {0x8F, 0x78},
		3, {0xE9, 0xA7, 0xBF},
		2, {0xFF, 0x99},
		2, {0xBD, 0xD9}
	},
	{
		2, {0x8F, 0x79},
		3, {0xE5, 0x87, 0x86},
		2, {0xC6, 0x51},
		2, {0xBD, 0xDA}
	},
	{
		2, {0x8F, 0x7A},
		3, {0xE5, 0xBE, 0xAA},
		2, {0xAA, 0x5F},
		2, {0xBD, 0xDB}
	},
	{
		2, {0x8F, 0x7B},
		3, {0xE6, 0x97, 0xAC},
		2, {0xEC, 0x65},
		2, {0xBD, 0xDC}
	},
	{
		2, {0x8F, 0x7C},
		3, {0xE6, 0xA5, 0xAF},
		2, {0x6F, 0x69},
		2, {0xBD, 0xDD}
	},
	{
		2, {0x8F, 0x7D},
		3, {0xE6, 0xAE, 0x89},
		2, {0x89, 0x6B},
		2, {0xBD, 0xDE}
	},
	{
		2, {0x8F, 0x7E},
		3, {0xE6, 0xB7, 0xB3},
		2, {0xF3, 0x6D},
		2, {0xBD, 0xDF}
	},
	{
		2, {0x8F, 0x80},
		3, {0xE6, 0xBA, 0x96},
		2, {0x96, 0x6E},
		2, {0xBD, 0xE0}
	},
	{
		2, {0x8F, 0x81},
		3, {0xE6, 0xBD, 0xA4},
		2, {0x64, 0x6F},
		2, {0xBD, 0xE1}
	},
	{
		2, {0x8F, 0x82},
		3, {0xE7, 0x9B, 0xBE},
		2, {0xFE, 0x76},
		2, {0xBD, 0xE2}
	},
	{
		2, {0x8F, 0x83},
		3, {0xE7, 0xB4, 0x94},
		2, {0x14, 0x7D},
		2, {0xBD, 0xE3}
	},
	{
		2, {0x8F, 0x84},
		3, {0xE5, 0xB7, 0xA1},
		2, {0xE1, 0x5D},
		2, {0xBD, 0xE4}
	},
	{
		2, {0x8F, 0x85},
		3, {0xE9, 0x81, 0xB5},
		2, {0x75, 0x90},
		2, {0xBD, 0xE5}
	},
	{
		2, {0x8F, 0x86},
		3, {0xE9, 0x86, 0x87},
		2, {0x87, 0x91},
		2, {0xBD, 0xE6}
	},
	{
		2, {0x8F, 0x87},
		3, {0xE9, 0xA0, 0x86},
		2, {0x06, 0x98},
		2, {0xBD, 0xE7}
	},
	{
		2, {0x8F, 0x88},
		3, {0xE5, 0x87, 0xA6},
		2, {0xE6, 0x51},
		2, {0xBD, 0xE8}
	},
	{
		2, {0x8F, 0x89},
		3, {0xE5, 0x88, 0x9D},
		2, {0x1D, 0x52},
		2, {0xBD, 0xE9}
	},
	{
		2, {0x8F, 0x8A},
		3, {0xE6, 0x89, 0x80},
		2, {0x40, 0x62},
		2, {0xBD, 0xEA}
	},
	{
		2, {0x8F, 0x8B},
		3, {0xE6, 0x9A, 0x91},
		2, {0x91, 0x66},
		2, {0xBD, 0xEB}
	},
	{
		2, {0x8F, 0x8C},
		3, {0xE6, 0x9B, 0x99},
		2, {0xD9, 0x66},
		2, {0xBD, 0xEC}
	},
	{
		2, {0x8F, 0x8D},
		3, {0xE6, 0xB8, 0x9A},
		2, {0x1A, 0x6E},
		2, {0xBD, 0xED}
	},
	{
		2, {0x8F, 0x8E},
		3, {0xE5, 0xBA, 0xB6},
		2, {0xB6, 0x5E},
		2, {0xBD, 0xEE}
	},
	{
		2, {0x8F, 0x8F},
		3, {0xE7, 0xB7, 0x92},
		2, {0xD2, 0x7D},
		2, {0xBD, 0xEF}
	},
	{
		2, {0x8F, 0x90},
		3, {0xE7, 0xBD, 0xB2},
		2, {0x72, 0x7F},
		2, {0xBD, 0xF0}
	},
	{
		2, {0x8F, 0x91},
		3, {0xE6, 0x9B, 0xB8},
		2, {0xF8, 0x66},
		2, {0xBD, 0xF1}
	},
	{
		2, {0x8F, 0x92},
		3, {0xE8, 0x96, 0xAF},
		2, {0xAF, 0x85},
		2, {0xBD, 0xF2}
	},
	{
		2, {0x8F, 0x93},
		3, {0xE8, 0x97, 0xB7},
		2, {0xF7, 0x85},
		2, {0xBD, 0xF3}
	},
	{
		2, {0x8F, 0x94},
		3, {0xE8, 0xAB, 0xB8},
		2, {0xF8, 0x8A},
		2, {0xBD, 0xF4}
	},
	{
		2, {0x8F, 0x95},
		3, {0xE5, 0x8A, 0xA9},
		2, {0xA9, 0x52},
		2, {0xBD, 0xF5}
	},
	{
		2, {0x8F, 0x96},
		3, {0xE5, 0x8F, 0x99},
		2, {0xD9, 0x53},
		2, {0xBD, 0xF6}
	},
	{
		2, {0x8F, 0x97},
		3, {0xE5, 0xA5, 0xB3},
		2, {0x73, 0x59},
		2, {0xBD, 0xF7}
	},
	{
		2, {0x8F, 0x98},
		3, {0xE5, 0xBA, 0x8F},
		2, {0x8F, 0x5E},
		2, {0xBD, 0xF8}
	},
	{
		2, {0x8F, 0x99},
		3, {0xE5, 0xBE, 0x90},
		2, {0x90, 0x5F},
		2, {0xBD, 0xF9}
	},
	{
		2, {0x8F, 0x9A},
		3, {0xE6, 0x81, 0x95},
		2, {0x55, 0x60},
		2, {0xBD, 0xFA}
	},
	{
		2, {0x8F, 0x9B},
		3, {0xE9, 0x8B, 0xA4},
		2, {0xE4, 0x92},
		2, {0xBD, 0xFB}
	},
	{
		2, {0x8F, 0x9C},
		3, {0xE9, 0x99, 0xA4},
		2, {0x64, 0x96},
		2, {0xBD, 0xFC}
	},
	{
		2, {0x8F, 0x9D},
		3, {0xE5, 0x82, 0xB7},
		2, {0xB7, 0x50},
		2, {0xBD, 0xFD}
	},
	{
		2, {0x8F, 0x9E},
		3, {0xE5, 0x84, 0x9F},
		2, {0x1F, 0x51},
		2, {0xBD, 0xFE}
	},
	{
		2, {0x8F, 0x9F},
		3, {0xE5, 0x8B, 0x9D},
		2, {0xDD, 0x52},
		2, {0xBE, 0xA1}
	},
	{
		2, {0x8F, 0xA0},
		3, {0xE5, 0x8C, 0xA0},
		2, {0x20, 0x53},
		2, {0xBE, 0xA2}
	},
	{
		2, {0x8F, 0xA1},
		3, {0xE5, 0x8D, 0x87},
		2, {0x47, 0x53},
		2, {0xBE, 0xA3}
	},
	{
		2, {0x8F, 0xA2},
		3, {0xE5, 0x8F, 0xAC},
		2, {0xEC, 0x53},
		2, {0xBE, 0xA4}
	},
	{
		2, {0x8F, 0xA3},
		3, {0xE5, 0x93, 0xA8},
		2, {0xE8, 0x54},
		2, {0xBE, 0xA5}
	},
	{
		2, {0x8F, 0xA4},
		3, {0xE5, 0x95, 0x86},
		2, {0x46, 0x55},
		2, {0xBE, 0xA6}
	},
	{
		2, {0x8F, 0xA5},
		3, {0xE5, 0x94, 0xB1},
		2, {0x31, 0x55},
		2, {0xBE, 0xA7}
	},
	{
		2, {0x8F, 0xA6},
		3, {0xE5, 0x98, 0x97},
		2, {0x17, 0x56},
		2, {0xBE, 0xA8}
	},
	{
		2, {0x8F, 0xA7},
		3, {0xE5, 0xA5, 0xA8},
		2, {0x68, 0x59},
		2, {0xBE, 0xA9}
	},
	{
		2, {0x8F, 0xA8},
		3, {0xE5, 0xA6, 0xBE},
		2, {0xBE, 0x59},
		2, {0xBE, 0xAA}
	},
	{
		2, {0x8F, 0xA9},
		3, {0xE5, 0xA8, 0xBC},
		2, {0x3C, 0x5A},
		2, {0xBE, 0xAB}
	},
	{
		2, {0x8F, 0xAA},
		3, {0xE5, 0xAE, 0xB5},
		2, {0xB5, 0x5B},
		2, {0xBE, 0xAC}
	},
	{
		2, {0x8F, 0xAB},
		3, {0xE5, 0xB0, 0x86},
		2, {0x06, 0x5C},
		2, {0xBE, 0xAD}
	},
	{
		2, {0x8F, 0xAC},
		3, {0xE5, 0xB0, 0x8F},
		2, {0x0F, 0x5C},
		2, {0xBE, 0xAE}
	},
	{
		2, {0x8F, 0xAD},
		3, {0xE5, 0xB0, 0x91},
		2, {0x11, 0x5C},
		2, {0xBE, 0xAF}
	},
	{
		2, {0x8F, 0xAE},
		3, {0xE5, 0xB0, 0x9A},
		2, {0x1A, 0x5C},
		2, {0xBE, 0xB0}
	},
	{
		2, {0x8F, 0xAF},
		3, {0xE5, 0xBA, 0x84},
		2, {0x84, 0x5E},
		2, {0xBE, 0xB1}
	},
	{
		2, {0x8F, 0xB0},
		3, {0xE5, 0xBA, 0x8A},
		2, {0x8A, 0x5E},
		2, {0xBE, 0xB2}
	},
	{
		2, {0x8F, 0xB1},
		3, {0xE5, 0xBB, 0xA0},
		2, {0xE0, 0x5E},
		2, {0xBE, 0xB3}
	},
	{
		2, {0x8F, 0xB2},
		3, {0xE5, 0xBD, 0xB0},
		2, {0x70, 0x5F},
		2, {0xBE, 0xB4}
	},
	{
		2, {0x8F, 0xB3},
		3, {0xE6, 0x89, 0xBF},
		2, {0x7F, 0x62},
		2, {0xBE, 0xB5}
	},
	{
		2, {0x8F, 0xB4},
		3, {0xE6, 0x8A, 0x84},
		2, {0x84, 0x62},
		2, {0xBE, 0xB6}
	},
	{
		2, {0x8F, 0xB5},
		3, {0xE6, 0x8B, 0x9B},
		2, {0xDB, 0x62},
		2, {0xBE, 0xB7}
	},
	{
		2, {0x8F, 0xB6},
		3, {0xE6, 0x8E, 0x8C},
		2, {0x8C, 0x63},
		2, {0xBE, 0xB8}
	},
	{
		2, {0x8F, 0xB7},
		3, {0xE6, 0x8D, 0xB7},
		2, {0x77, 0x63},
		2, {0xBE, 0xB9}
	},
	{
		2, {0x8F, 0xB8},
		3, {0xE6, 0x98, 0x87},
		2, {0x07, 0x66},
		2, {0xBE, 0xBA}
	},
	{
		2, {0x8F, 0xB9},
		3, {0xE6, 0x98, 0x8C},
		2, {0x0C, 0x66},
		2, {0xBE, 0xBB}
	},
	{
		2, {0x8F, 0xBA},
		3, {0xE6, 0x98, 0xAD},
		2, {0x2D, 0x66},
		2, {0xBE, 0xBC}
	},
	{
		2, {0x8F, 0xBB},
		3, {0xE6, 0x99, 0xB6},
		2, {0x76, 0x66},
		2, {0xBE, 0xBD}
	},
	{
		2, {0x8F, 0xBC},
		3, {0xE6, 0x9D, 0xBE},
		2, {0x7E, 0x67},
		2, {0xBE, 0xBE}
	},
	{
		2, {0x8F, 0xBD},
		3, {0xE6, 0xA2, 0xA2},
		2, {0xA2, 0x68},
		2, {0xBE, 0xBF}
	},
	{
		2, {0x8F, 0xBE},
		3, {0xE6, 0xA8, 0x9F},
		2, {0x1F, 0x6A},
		2, {0xBE, 0xC0}
	},
	{
		2, {0x8F, 0xBF},
		3, {0xE6, 0xA8, 0xB5},
		2, {0x35, 0x6A},
		2, {0xBE, 0xC1}
	},
	{
		2, {0x8F, 0xC0},
		3, {0xE6, 0xB2, 0xBC},
		2, {0xBC, 0x6C},
		2, {0xBE, 0xC2}
	},
	{
		2, {0x8F, 0xC1},
		3, {0xE6, 0xB6, 0x88},
		2, {0x88, 0x6D},
		2, {0xBE, 0xC3}
	},
	{
		2, {0x8F, 0xC2},
		3, {0xE6, 0xB8, 0x89},
		2, {0x09, 0x6E},
		2, {0xBE, 0xC4}
	},
	{
		2, {0x8F, 0xC3},
		3, {0xE6, 0xB9, 0x98},
		2, {0x58, 0x6E},
		2, {0xBE, 0xC5}
	},
	{
		2, {0x8F, 0xC4},
		3, {0xE7, 0x84, 0xBC},
		2, {0x3C, 0x71},
		2, {0xBE, 0xC6}
	},
	{
		2, {0x8F, 0xC5},
		3, {0xE7, 0x84, 0xA6},
		2, {0x26, 0x71},
		2, {0xBE, 0xC7}
	},
	{
		2, {0x8F, 0xC6},
		3, {0xE7, 0x85, 0xA7},
		2, {0x67, 0x71},
		2, {0xBE, 0xC8}
	},
	{
		2, {0x8F, 0xC7},
		3, {0xE7, 0x97, 0x87},
		2, {0xC7, 0x75},
		2, {0xBE, 0xC9}
	},
	{
		2, {0x8F, 0xC8},
		3, {0xE7, 0x9C, 0x81},
		2, {0x01, 0x77},
		2, {0xBE, 0xCA}
	},
	{
		2, {0x8F, 0xC9},
		3, {0xE7, 0xA1, 0x9D},
		2, {0x5D, 0x78},
		2, {0xBE, 0xCB}
	},
	{
		2, {0x8F, 0xCA},
		3, {0xE7, 0xA4, 0x81},
		2, {0x01, 0x79},
		2, {0xBE, 0xCC}
	},
	{
		2, {0x8F, 0xCB},
		3, {0xE7, 0xA5, 0xA5},
		2, {0x65, 0x79},
		2, {0xBE, 0xCD}
	},
	{
		2, {0x8F, 0xCC},
		3, {0xE7, 0xA7, 0xB0},
		2, {0xF0, 0x79},
		2, {0xBE, 0xCE}
	},
	{
		2, {0x8F, 0xCD},
		3, {0xE7, 0xAB, 0xA0},
		2, {0xE0, 0x7A},
		2, {0xBE, 0xCF}
	},
	{
		2, {0x8F, 0xCE},
		3, {0xE7, 0xAC, 0x91},
		2, {0x11, 0x7B},
		2, {0xBE, 0xD0}
	},
	{
		2, {0x8F, 0xCF},
		3, {0xE7, 0xB2, 0xA7},
		2, {0xA7, 0x7C},
		2, {0xBE, 0xD1}
	},
	{
		2, {0x8F, 0xD0},
		3, {0xE7, 0xB4, 0xB9},
		2, {0x39, 0x7D},
		2, {0xBE, 0xD2}
	},
	{
		2, {0x8F, 0xD1},
		3, {0xE8, 0x82, 0x96},
		2, {0x96, 0x80},
		2, {0xBE, 0xD3}
	},
	{
		2, {0x8F, 0xD2},
		3, {0xE8, 0x8F, 0x96},
		2, {0xD6, 0x83},
		2, {0xBE, 0xD4}
	},
	{
		2, {0x8F, 0xD3},
		3, {0xE8, 0x92, 0x8B},
		2, {0x8B, 0x84},
		2, {0xBE, 0xD5}
	},
	{
		2, {0x8F, 0xD4},
		3, {0xE8, 0x95, 0x89},
		2, {0x49, 0x85},
		2, {0xBE, 0xD6}
	},
	{
		2, {0x8F, 0xD5},
		3, {0xE8, 0xA1, 0x9D},
		2, {0x5D, 0x88},
		2, {0xBE, 0xD7}
	},
	{
		2, {0x8F, 0xD6},
		3, {0xE8, 0xA3, 0xB3},
		2, {0xF3, 0x88},
		2, {0xBE, 0xD8}
	},
	{
		2, {0x8F, 0xD7},
		3, {0xE8, 0xA8, 0x9F},
		2, {0x1F, 0x8A},
		2, {0xBE, 0xD9}
	},
	{
		2, {0x8F, 0xD8},
		3, {0xE8, 0xA8, 0xBC},
		2, {0x3C, 0x8A},
		2, {0xBE, 0xDA}
	},
	{
		2, {0x8F, 0xD9},
		3, {0xE8, 0xA9, 0x94},
		2, {0x54, 0x8A},
		2, {0xBE, 0xDB}
	},
	{
		2, {0x8F, 0xDA},
		3, {0xE8, 0xA9, 0xB3},
		2, {0x73, 0x8A},
		2, {0xBE, 0xDC}
	},
	{
		2, {0x8F, 0xDB},
		3, {0xE8, 0xB1, 0xA1},
		2, {0x61, 0x8C},
		2, {0xBE, 0xDD}
	},
	{
		2, {0x8F, 0xDC},
		3, {0xE8, 0xB3, 0x9E},
		2, {0xDE, 0x8C},
		2, {0xBE, 0xDE}
	},
	{
		2, {0x8F, 0xDD},
		3, {0xE9, 0x86, 0xA4},
		2, {0xA4, 0x91},
		2, {0xBE, 0xDF}
	},
	{
		2, {0x8F, 0xDE},
		3, {0xE9, 0x89, 0xA6},
		2, {0x66, 0x92},
		2, {0xBE, 0xE0}
	},
	{
		2, {0x8F, 0xDF},
		3, {0xE9, 0x8D, 0xBE},
		2, {0x7E, 0x93},
		2, {0xBE, 0xE1}
	},
	{
		2, {0x8F, 0xE0},
		3, {0xE9, 0x90, 0x98},
		2, {0x18, 0x94},
		2, {0xBE, 0xE2}
	},
	{
		2, {0x8F, 0xE1},
		3, {0xE9, 0x9A, 0x9C},
		2, {0x9C, 0x96},
		2, {0xBE, 0xE3}
	},
	{
		2, {0x8F, 0xE2},
		3, {0xE9, 0x9E, 0x98},
		2, {0x98, 0x97},
		2, {0xBE, 0xE4}
	},
	{
		2, {0x8F, 0xE3},
		3, {0xE4, 0xB8, 0x8A},
		2, {0x0A, 0x4E},
		2, {0xBE, 0xE5}
	},
	{
		2, {0x8F, 0xE4},
		3, {0xE4, 0xB8, 0x88},
		2, {0x08, 0x4E},
		2, {0xBE, 0xE6}
	},
	{
		2, {0x8F, 0xE5},
		3, {0xE4, 0xB8, 0x9E},
		2, {0x1E, 0x4E},
		2, {0xBE, 0xE7}
	},
	{
		2, {0x8F, 0xE6},
		3, {0xE4, 0xB9, 0x97},
		2, {0x57, 0x4E},
		2, {0xBE, 0xE8}
	},
	{
		2, {0x8F, 0xE7},
		3, {0xE5, 0x86, 0x97},
		2, {0x97, 0x51},
		2, {0xBE, 0xE9}
	},
	{
		2, {0x8F, 0xE8},
		3, {0xE5, 0x89, 0xB0},
		2, {0x70, 0x52},
		2, {0xBE, 0xEA}
	},
	{
		2, {0x8F, 0xE9},
		3, {0xE5, 0x9F, 0x8E},
		2, {0xCE, 0x57},
		2, {0xBE, 0xEB}
	},
	{
		2, {0x8F, 0xEA},
		3, {0xE5, 0xA0, 0xB4},
		2, {0x34, 0x58},
		2, {0xBE, 0xEC}
	},
	{
		2, {0x8F, 0xEB},
		3, {0xE5, 0xA3, 0x8C},
		2, {0xCC, 0x58},
		2, {0xBE, 0xED}
	},
	{
		2, {0x8F, 0xEC},
		3, {0xE5, 0xAC, 0xA2},
		2, {0x22, 0x5B},
		2, {0xBE, 0xEE}
	},
	{
		2, {0x8F, 0xED},
		3, {0xE5, 0xB8, 0xB8},
		2, {0x38, 0x5E},
		2, {0xBE, 0xEF}
	},
	{
		2, {0x8F, 0xEE},
		3, {0xE6, 0x83, 0x85},
		2, {0xC5, 0x60},
		2, {0xBE, 0xF0}
	},
	{
		2, {0x8F, 0xEF},
		3, {0xE6, 0x93, 0xBE},
		2, {0xFE, 0x64},
		2, {0xBE, 0xF1}
	},
	{
		2, {0x8F, 0xF0},
		3, {0xE6, 0x9D, 0xA1},
		2, {0x61, 0x67},
		2, {0xBE, 0xF2}
	},
	{
		2, {0x8F, 0xF1},
		3, {0xE6, 0x9D, 0x96},
		2, {0x56, 0x67},
		2, {0xBE, 0xF3}
	},
	{
		2, {0x8F, 0xF2},
		3, {0xE6, 0xB5, 0x84},
		2, {0x44, 0x6D},
		2, {0xBE, 0xF4}
	},
	{
		2, {0x8F, 0xF3},
		3, {0xE7, 0x8A, 0xB6},
		2, {0xB6, 0x72},
		2, {0xBE, 0xF5}
	},
	{
		2, {0x8F, 0xF4},
		3, {0xE7, 0x95, 0xB3},
		2, {0x73, 0x75},
		2, {0xBE, 0xF6}
	},
	{
		2, {0x8F, 0xF5},
		3, {0xE7, 0xA9, 0xA3},
		2, {0x63, 0x7A},
		2, {0xBE, 0xF7}
	},
	{
		2, {0x8F, 0xF6},
		3, {0xE8, 0x92, 0xB8},
		2, {0xB8, 0x84},
		2, {0xBE, 0xF8}
	},
	{
		2, {0x8F, 0xF7},
		3, {0xE8, 0xAD, 0xB2},
		2, {0x72, 0x8B},
		2, {0xBE, 0xF9}
	},
	{
		2, {0x8F, 0xF8},
		3, {0xE9, 0x86, 0xB8},
		2, {0xB8, 0x91},
		2, {0xBE, 0xFA}
	},
	{
		2, {0x8F, 0xF9},
		3, {0xE9, 0x8C, 0xA0},
		2, {0x20, 0x93},
		2, {0xBE, 0xFB}
	},
	{
		2, {0x8F, 0xFA},
		3, {0xE5, 0x98, 0xB1},
		2, {0x31, 0x56},
		2, {0xBE, 0xFC}
	},
	{
		2, {0x8F, 0xFB},
		3, {0xE5, 0x9F, 0xB4},
		2, {0xF4, 0x57},
		2, {0xBE, 0xFD}
	},
	{
		2, {0x8F, 0xFC},
		3, {0xE9, 0xA3, 0xBE},
		2, {0xFE, 0x98},
		2, {0xBE, 0xFE}
	},
	{
		2, {0x90, 0x40},
		3, {0xE6, 0x8B, 0xAD},
		2, {0xED, 0x62},
		2, {0xBF, 0xA1}
	},
	{
		2, {0x90, 0x41},
		3, {0xE6, 0xA4, 0x8D},
		2, {0x0D, 0x69},
		2, {0xBF, 0xA2}
	},
	{
		2, {0x90, 0x42},
		3, {0xE6, 0xAE, 0x96},
		2, {0x96, 0x6B},
		2, {0xBF, 0xA3}
	},
	{
		2, {0x90, 0x43},
		3, {0xE7, 0x87, 0xAD},
		2, {0xED, 0x71},
		2, {0xBF, 0xA4}
	},
	{
		2, {0x90, 0x44},
		3, {0xE7, 0xB9, 0x94},
		2, {0x54, 0x7E},
		2, {0xBF, 0xA5}
	},
	{
		2, {0x90, 0x45},
		3, {0xE8, 0x81, 0xB7},
		2, {0x77, 0x80},
		2, {0xBF, 0xA6}
	},
	{
		2, {0x90, 0x46},
		3, {0xE8, 0x89, 0xB2},
		2, {0x72, 0x82},
		2, {0xBF, 0xA7}
	},
	{
		2, {0x90, 0x47},
		3, {0xE8, 0xA7, 0xA6},
		2, {0xE6, 0x89},
		2, {0xBF, 0xA8}
	},
	{
		2, {0x90, 0x48},
		3, {0xE9, 0xA3, 0x9F},
		2, {0xDF, 0x98},
		2, {0xBF, 0xA9}
	},
	{
		2, {0x90, 0x49},
		3, {0xE8, 0x9D, 0x95},
		2, {0x55, 0x87},
		2, {0xBF, 0xAA}
	},
	{
		2, {0x90, 0x4A},
		3, {0xE8, 0xBE, 0xB1},
		2, {0xB1, 0x8F},
		2, {0xBF, 0xAB}
	},
	{
		2, {0x90, 0x4B},
		3, {0xE5, 0xB0, 0xBB},
		2, {0x3B, 0x5C},
		2, {0xBF, 0xAC}
	},
	{
		2, {0x90, 0x4C},
		3, {0xE4, 0xBC, 0xB8},
		2, {0x38, 0x4F},
		2, {0xBF, 0xAD}
	},
	{
		2, {0x90, 0x4D},
		3, {0xE4, 0xBF, 0xA1},
		2, {0xE1, 0x4F},
		2, {0xBF, 0xAE}
	},
	{
		2, {0x90, 0x4E},
		3, {0xE4, 0xBE, 0xB5},
		2, {0xB5, 0x4F},
		2, {0xBF, 0xAF}
	},
	{
		2, {0x90, 0x4F},
		3, {0xE5, 0x94, 0x87},
		2, {0x07, 0x55},
		2, {0xBF, 0xB0}
	},
	{
		2, {0x90, 0x50},
		3, {0xE5, 0xA8, 0xA0},
		2, {0x20, 0x5A},
		2, {0xBF, 0xB1}
	},
	{
		2, {0x90, 0x51},
		3, {0xE5, 0xAF, 0x9D},
		2, {0xDD, 0x5B},
		2, {0xBF, 0xB2}
	},
	{
		2, {0x90, 0x52},
		3, {0xE5, 0xAF, 0xA9},
		2, {0xE9, 0x5B},
		2, {0xBF, 0xB3}
	},
	{
		2, {0x90, 0x53},
		3, {0xE5, 0xBF, 0x83},
		2, {0xC3, 0x5F},
		2, {0xBF, 0xB4}
	},
	{
		2, {0x90, 0x54},
		3, {0xE6, 0x85, 0x8E},
		2, {0x4E, 0x61},
		2, {0xBF, 0xB5}
	},
	{
		2, {0x90, 0x55},
		3, {0xE6, 0x8C, 0xAF},
		2, {0x2F, 0x63},
		2, {0xBF, 0xB6}
	},
	{
		2, {0x90, 0x56},
		3, {0xE6, 0x96, 0xB0},
		2, {0xB0, 0x65},
		2, {0xBF, 0xB7}
	},
	{
		2, {0x90, 0x57},
		3, {0xE6, 0x99, 0x8B},
		2, {0x4B, 0x66},
		2, {0xBF, 0xB8}
	},
	{
		2, {0x90, 0x58},
		3, {0xE6, 0xA3, 0xAE},
		2, {0xEE, 0x68},
		2, {0xBF, 0xB9}
	},
	{
		2, {0x90, 0x59},
		3, {0xE6, 0xA6, 0x9B},
		2, {0x9B, 0x69},
		2, {0xBF, 0xBA}
	},
	{
		2, {0x90, 0x5A},
		3, {0xE6, 0xB5, 0xB8},
		2, {0x78, 0x6D},
		2, {0xBF, 0xBB}
	},
	{
		2, {0x90, 0x5B},
		3, {0xE6, 0xB7, 0xB1},
		2, {0xF1, 0x6D},
		2, {0xBF, 0xBC}
	},
	{
		2, {0x90, 0x5C},
		3, {0xE7, 0x94, 0xB3},
		2, {0x33, 0x75},
		2, {0xBF, 0xBD}
	},
	{
		2, {0x90, 0x5D},
		3, {0xE7, 0x96, 0xB9},
		2, {0xB9, 0x75},
		2, {0xBF, 0xBE}
	},
	{
		2, {0x90, 0x5E},
		3, {0xE7, 0x9C, 0x9F},
		2, {0x1F, 0x77},
		2, {0xBF, 0xBF}
	},
	{
		2, {0x90, 0x5F},
		3, {0xE7, 0xA5, 0x9E},
		2, {0x5E, 0x79},
		2, {0xBF, 0xC0}
	},
	{
		2, {0x90, 0x60},
		3, {0xE7, 0xA7, 0xA6},
		2, {0xE6, 0x79},
		2, {0xBF, 0xC1}
	},
	{
		2, {0x90, 0x61},
		3, {0xE7, 0xB4, 0xB3},
		2, {0x33, 0x7D},
		2, {0xBF, 0xC2}
	},
	{
		2, {0x90, 0x62},
		3, {0xE8, 0x87, 0xA3},
		2, {0xE3, 0x81},
		2, {0xBF, 0xC3}
	},
	{
		2, {0x90, 0x63},
		3, {0xE8, 0x8A, 0xAF},
		2, {0xAF, 0x82},
		2, {0xBF, 0xC4}
	},
	{
		2, {0x90, 0x64},
		3, {0xE8, 0x96, 0xAA},
		2, {0xAA, 0x85},
		2, {0xBF, 0xC5}
	},
	{
		2, {0x90, 0x65},
		3, {0xE8, 0xA6, 0xAA},
		2, {0xAA, 0x89},
		2, {0xBF, 0xC6}
	},
	{
		2, {0x90, 0x66},
		3, {0xE8, 0xA8, 0xBA},
		2, {0x3A, 0x8A},
		2, {0xBF, 0xC7}
	},
	{
		2, {0x90, 0x67},
		3, {0xE8, 0xBA, 0xAB},
		2, {0xAB, 0x8E},
		2, {0xBF, 0xC8}
	},
	{
		2, {0x90, 0x68},
		3, {0xE8, 0xBE, 0x9B},
		2, {0x9B, 0x8F},
		2, {0xBF, 0xC9}
	},
	{
		2, {0x90, 0x69},
		3, {0xE9, 0x80, 0xB2},
		2, {0x32, 0x90},
		2, {0xBF, 0xCA}
	},
	{
		2, {0x90, 0x6A},
		3, {0xE9, 0x87, 0x9D},
		2, {0xDD, 0x91},
		2, {0xBF, 0xCB}
	},
	{
		2, {0x90, 0x6B},
		3, {0xE9, 0x9C, 0x87},
		2, {0x07, 0x97},
		2, {0xBF, 0xCC}
	},
	{
		2, {0x90, 0x6C},
		3, {0xE4, 0xBA, 0xBA},
		2, {0xBA, 0x4E},
		2, {0xBF, 0xCD}
	},
	{
		2, {0x90, 0x6D},
		3, {0xE4, 0xBB, 0x81},
		2, {0xC1, 0x4E},
		2, {0xBF, 0xCE}
	},
	{
		2, {0x90, 0x6E},
		3, {0xE5, 0x88, 0x83},
		2, {0x03, 0x52},
		2, {0xBF, 0xCF}
	},
	{
		2, {0x90, 0x6F},
		3, {0xE5, 0xA1, 0xB5},
		2, {0x75, 0x58},
		2, {0xBF, 0xD0}
	},
	{
		2, {0x90, 0x70},
		3, {0xE5, 0xA3, 0xAC},
		2, {0xEC, 0x58},
		2, {0xBF, 0xD1}
	},
	{
		2, {0x90, 0x71},
		3, {0xE5, 0xB0, 0x8B},
		2, {0x0B, 0x5C},
		2, {0xBF, 0xD2}
	},
	{
		2, {0x90, 0x72},
		3, {0xE7, 0x94, 0x9A},
		2, {0x1A, 0x75},
		2, {0xBF, 0xD3}
	},
	{
		2, {0x90, 0x73},
		3, {0xE5, 0xB0, 0xBD},
		2, {0x3D, 0x5C},
		2, {0xBF, 0xD4}
	},
	{
		2, {0x90, 0x74},
		3, {0xE8, 0x85, 0x8E},
		2, {0x4E, 0x81},
		2, {0xBF, 0xD5}
	},
	{
		2, {0x90, 0x75},
		3, {0xE8, 0xA8, 0x8A},
		2, {0x0A, 0x8A},
		2, {0xBF, 0xD6}
	},
	{
		2, {0x90, 0x76},
		3, {0xE8, 0xBF, 0x85},
		2, {0xC5, 0x8F},
		2, {0xBF, 0xD7}
	},
	{
		2, {0x90, 0x77},
		3, {0xE9, 0x99, 0xA3},
		2, {0x63, 0x96},
		2, {0xBF, 0xD8}
	},
	{
		2, {0x90, 0x78},
		3, {0xE9, 0x9D, 0xAD},
		2, {0x6D, 0x97},
		2, {0xBF, 0xD9}
	},
	{
		2, {0x90, 0x79},
		3, {0xE7, 0xAC, 0xA5},
		2, {0x25, 0x7B},
		2, {0xBF, 0xDA}
	},
	{
		2, {0x90, 0x7A},
		3, {0xE8, 0xAB, 0x8F},
		2, {0xCF, 0x8A},
		2, {0xBF, 0xDB}
	},
	{
		2, {0x90, 0x7B},
		3, {0xE9, 0xA0, 0x88},
		2, {0x08, 0x98},
		2, {0xBF, 0xDC}
	},
	{
		2, {0x90, 0x7C},
		3, {0xE9, 0x85, 0xA2},
		2, {0x62, 0x91},
		2, {0xBF, 0xDD}
	},
	{
		2, {0x90, 0x7D},
		3, {0xE5, 0x9B, 0xB3},
		2, {0xF3, 0x56},
		2, {0xBF, 0xDE}
	},
	{
		2, {0x90, 0x7E},
		3, {0xE5, 0x8E, 0xA8},
		2, {0xA8, 0x53},
		2, {0xBF, 0xDF}
	},
	{
		2, {0x90, 0x80},
		3, {0xE9, 0x80, 0x97},
		2, {0x17, 0x90},
		2, {0xBF, 0xE0}
	},
	{
		2, {0x90, 0x81},
		3, {0xE5, 0x90, 0xB9},
		2, {0x39, 0x54},
		2, {0xBF, 0xE1}
	},
	{
		2, {0x90, 0x82},
		3, {0xE5, 0x9E, 0x82},
		2, {0x82, 0x57},
		2, {0xBF, 0xE2}
	},
	{
		2, {0x90, 0x83},
		3, {0xE5, 0xB8, 0xA5},
		2, {0x25, 0x5E},
		2, {0xBF, 0xE3}
	},
	{
		2, {0x90, 0x84},
		3, {0xE6, 0x8E, 0xA8},
		2, {0xA8, 0x63},
		2, {0xBF, 0xE4}
	},
	{
		2, {0x90, 0x85},
		3, {0xE6, 0xB0, 0xB4},
		2, {0x34, 0x6C},
		2, {0xBF, 0xE5}
	},
	{
		2, {0x90, 0x86},
		3, {0xE7, 0x82, 0x8A},
		2, {0x8A, 0x70},
		2, {0xBF, 0xE6}
	},
	{
		2, {0x90, 0x87},
		3, {0xE7, 0x9D, 0xA1},
		2, {0x61, 0x77},
		2, {0xBF, 0xE7}
	},
	{
		2, {0x90, 0x88},
		3, {0xE7, 0xB2, 0x8B},
		2, {0x8B, 0x7C},
		2, {0xBF, 0xE8}
	},
	{
		2, {0x90, 0x89},
		3, {0xE7, 0xBF, 0xA0},
		2, {0xE0, 0x7F},
		2, {0xBF, 0xE9}
	},
	{
		2, {0x90, 0x8A},
		3, {0xE8, 0xA1, 0xB0},
		2, {0x70, 0x88},
		2, {0xBF, 0xEA}
	},
	{
		2, {0x90, 0x8B},
		3, {0xE9, 0x81, 0x82},
		2, {0x42, 0x90},
		2, {0xBF, 0xEB}
	},
	{
		2, {0x90, 0x8C},
		3, {0xE9, 0x85, 0x94},
		2, {0x54, 0x91},
		2, {0xBF, 0xEC}
	},
	{
		2, {0x90, 0x8D},
		3, {0xE9, 0x8C, 0x90},
		2, {0x10, 0x93},
		2, {0xBF, 0xED}
	},
	{
		2, {0x90, 0x8E},
		3, {0xE9, 0x8C, 0x98},
		2, {0x18, 0x93},
		2, {0xBF, 0xEE}
	},
	{
		2, {0x90, 0x8F},
		3, {0xE9, 0x9A, 0x8F},
		2, {0x8F, 0x96},
		2, {0xBF, 0xEF}
	},
	{
		2, {0x90, 0x90},
		3, {0xE7, 0x91, 0x9E},
		2, {0x5E, 0x74},
		2, {0xBF, 0xF0}
	},
	{
		2, {0x90, 0x91},
		3, {0xE9, 0xAB, 0x84},
		2, {0xC4, 0x9A},
		2, {0xBF, 0xF1}
	},
	{
		2, {0x90, 0x92},
		3, {0xE5, 0xB4, 0x87},
		2, {0x07, 0x5D},
		2, {0xBF, 0xF2}
	},
	{
		2, {0x90, 0x93},
		3, {0xE5, 0xB5, 0xA9},
		2, {0x69, 0x5D},
		2, {0xBF, 0xF3}
	},
	{
		2, {0x90, 0x94},
		3, {0xE6, 0x95, 0xB0},
		2, {0x70, 0x65},
		2, {0xBF, 0xF4}
	},
	{
		2, {0x90, 0x95},
		3, {0xE6, 0x9E, 0xA2},
		2, {0xA2, 0x67},
		2, {0xBF, 0xF5}
	},
	{
		2, {0x90, 0x96},
		3, {0xE8, 0xB6, 0xA8},
		2, {0xA8, 0x8D},
		2, {0xBF, 0xF6}
	},
	{
		2, {0x90, 0x97},
		3, {0xE9, 0x9B, 0x9B},
		2, {0xDB, 0x96},
		2, {0xBF, 0xF7}
	},
	{
		2, {0x90, 0x98},
		3, {0xE6, 0x8D, 0xAE},
		2, {0x6E, 0x63},
		2, {0xBF, 0xF8}
	},
	{
		2, {0x90, 0x99},
		3, {0xE6, 0x9D, 0x89},
		2, {0x49, 0x67},
		2, {0xBF, 0xF9}
	},
	{
		2, {0x90, 0x9A},
		3, {0xE6, 0xA4, 0x99},
		2, {0x19, 0x69},
		2, {0xBF, 0xFA}
	},
	{
		2, {0x90, 0x9B},
		3, {0xE8, 0x8F, 0x85},
		2, {0xC5, 0x83},
		2, {0xBF, 0xFB}
	},
	{
		2, {0x90, 0x9C},
		3, {0xE9, 0xA0, 0x97},
		2, {0x17, 0x98},
		2, {0xBF, 0xFC}
	},
	{
		2, {0x90, 0x9D},
		3, {0xE9, 0x9B, 0x80},
		2, {0xC0, 0x96},
		2, {0xBF, 0xFD}
	},
	{
		2, {0x90, 0x9E},
		3, {0xE8, 0xA3, 0xBE},
		2, {0xFE, 0x88},
		2, {0xBF, 0xFE}
	},
	{
		2, {0x90, 0x9F},
		3, {0xE6, 0xBE, 0x84},
		2, {0x84, 0x6F},
		2, {0xC0, 0xA1}
	},
	{
		2, {0x90, 0xA0},
		3, {0xE6, 0x91, 0xBA},
		2, {0x7A, 0x64},
		2, {0xC0, 0xA2}
	},
	{
		2, {0x90, 0xA1},
		3, {0xE5, 0xAF, 0xB8},
		2, {0xF8, 0x5B},
		2, {0xC0, 0xA3}
	},
	{
		2, {0x90, 0xA2},
		3, {0xE4, 0xB8, 0x96},
		2, {0x16, 0x4E},
		2, {0xC0, 0xA4}
	},
	{
		2, {0x90, 0xA3},
		3, {0xE7, 0x80, 0xAC},
		2, {0x2C, 0x70},
		2, {0xC0, 0xA5}
	},
	{
		2, {0x90, 0xA4},
		3, {0xE7, 0x95, 0x9D},
		2, {0x5D, 0x75},
		2, {0xC0, 0xA6}
	},
	{
		2, {0x90, 0xA5},
		3, {0xE6, 0x98, 0xAF},
		2, {0x2F, 0x66},
		2, {0xC0, 0xA7}
	},
	{
		2, {0x90, 0xA6},
		3, {0xE5, 0x87, 0x84},
		2, {0xC4, 0x51},
		2, {0xC0, 0xA8}
	},
	{
		2, {0x90, 0xA7},
		3, {0xE5, 0x88, 0xB6},
		2, {0x36, 0x52},
		2, {0xC0, 0xA9}
	},
	{
		2, {0x90, 0xA8},
		3, {0xE5, 0x8B, 0xA2},
		2, {0xE2, 0x52},
		2, {0xC0, 0xAA}
	},
	{
		2, {0x90, 0xA9},
		3, {0xE5, 0xA7, 0x93},
		2, {0xD3, 0x59},
		2, {0xC0, 0xAB}
	},
	{
		2, {0x90, 0xAA},
		3, {0xE5, 0xBE, 0x81},
		2, {0x81, 0x5F},
		2, {0xC0, 0xAC}
	},
	{
		2, {0x90, 0xAB},
		3, {0xE6, 0x80, 0xA7},
		2, {0x27, 0x60},
		2, {0xC0, 0xAD}
	},
	{
		2, {0x90, 0xAC},
		3, {0xE6, 0x88, 0x90},
		2, {0x10, 0x62},
		2, {0xC0, 0xAE}
	},
	{
		2, {0x90, 0xAD},
		3, {0xE6, 0x94, 0xBF},
		2, {0x3F, 0x65},
		2, {0xC0, 0xAF}
	},
	{
		2, {0x90, 0xAE},
		3, {0xE6, 0x95, 0xB4},
		2, {0x74, 0x65},
		2, {0xC0, 0xB0}
	},
	{
		2, {0x90, 0xAF},
		3, {0xE6, 0x98, 0x9F},
		2, {0x1F, 0x66},
		2, {0xC0, 0xB1}
	},
	{
		2, {0x90, 0xB0},
		3, {0xE6, 0x99, 0xB4},
		2, {0x74, 0x66},
		2, {0xC0, 0xB2}
	},
	{
		2, {0x90, 0xB1},
		3, {0xE6, 0xA3, 0xB2},
		2, {0xF2, 0x68},
		2, {0xC0, 0xB3}
	},
	{
		2, {0x90, 0xB2},
		3, {0xE6, 0xA0, 0x96},
		2, {0x16, 0x68},
		2, {0xC0, 0xB4}
	},
	{
		2, {0x90, 0xB3},
		3, {0xE6, 0xAD, 0xA3},
		2, {0x63, 0x6B},
		2, {0xC0, 0xB5}
	},
	{
		2, {0x90, 0xB4},
		3, {0xE6, 0xB8, 0x85},
		2, {0x05, 0x6E},
		2, {0xC0, 0xB6}
	},
	{
		2, {0x90, 0xB5},
		3, {0xE7, 0x89, 0xB2},
		2, {0x72, 0x72},
		2, {0xC0, 0xB7}
	},
	{
		2, {0x90, 0xB6},
		3, {0xE7, 0x94, 0x9F},
		2, {0x1F, 0x75},
		2, {0xC0, 0xB8}
	},
	{
		2, {0x90, 0xB7},
		3, {0xE7, 0x9B, 0x9B},
		2, {0xDB, 0x76},
		2, {0xC0, 0xB9}
	},
	{
		2, {0x90, 0xB8},
		3, {0xE7, 0xB2, 0xBE},
		2, {0xBE, 0x7C},
		2, {0xC0, 0xBA}
	},
	{
		2, {0x90, 0xB9},
		3, {0xE8, 0x81, 0x96},
		2, {0x56, 0x80},
		2, {0xC0, 0xBB}
	},
	{
		2, {0x90, 0xBA},
		3, {0xE5, 0xA3, 0xB0},
		2, {0xF0, 0x58},
		2, {0xC0, 0xBC}
	},
	{
		2, {0x90, 0xBB},
		3, {0xE8, 0xA3, 0xBD},
		2, {0xFD, 0x88},
		2, {0xC0, 0xBD}
	},
	{
		2, {0x90, 0xBC},
		3, {0xE8, 0xA5, 0xBF},
		2, {0x7F, 0x89},
		2, {0xC0, 0xBE}
	},
	{
		2, {0x90, 0xBD},
		3, {0xE8, 0xAA, 0xA0},
		2, {0xA0, 0x8A},
		2, {0xC0, 0xBF}
	},
	{
		2, {0x90, 0xBE},
		3, {0xE8, 0xAA, 0x93},
		2, {0x93, 0x8A},
		2, {0xC0, 0xC0}
	},
	{
		2, {0x90, 0xBF},
		3, {0xE8, 0xAB, 0x8B},
		2, {0xCB, 0x8A},
		2, {0xC0, 0xC1}
	},
	{
		2, {0x90, 0xC0},
		3, {0xE9, 0x80, 0x9D},
		2, {0x1D, 0x90},
		2, {0xC0, 0xC2}
	},
	{
		2, {0x90, 0xC1},
		3, {0xE9, 0x86, 0x92},
		2, {0x92, 0x91},
		2, {0xC0, 0xC3}
	},
	{
		2, {0x90, 0xC2},
		3, {0xE9, 0x9D, 0x92},
		2, {0x52, 0x97},
		2, {0xC0, 0xC4}
	},
	{
		2, {0x90, 0xC3},
		3, {0xE9, 0x9D, 0x99},
		2, {0x59, 0x97},
		2, {0xC0, 0xC5}
	},
	{
		2, {0x90, 0xC4},
		3, {0xE6, 0x96, 0x89},
		2, {0x89, 0x65},
		2, {0xC0, 0xC6}
	},
	{
		2, {0x90, 0xC5},
		3, {0xE7, 0xA8, 0x8E},
		2, {0x0E, 0x7A},
		2, {0xC0, 0xC7}
	},
	{
		2, {0x90, 0xC6},
		3, {0xE8, 0x84, 0x86},
		2, {0x06, 0x81},
		2, {0xC0, 0xC8}
	},
	{
		2, {0x90, 0xC7},
		3, {0xE9, 0x9A, 0xBB},
		2, {0xBB, 0x96},
		2, {0xC0, 0xC9}
	},
	{
		2, {0x90, 0xC8},
		3, {0xE5, 0xB8, 0xAD},
		2, {0x2D, 0x5E},
		2, {0xC0, 0xCA}
	},
	{
		2, {0x90, 0xC9},
		3, {0xE6, 0x83, 0x9C},
		2, {0xDC, 0x60},
		2, {0xC0, 0xCB}
	},
	{
		2, {0x90, 0xCA},
		3, {0xE6, 0x88, 0x9A},
		2, {0x1A, 0x62},
		2, {0xC0, 0xCC}
	},
	{
		2, {0x90, 0xCB},
		3, {0xE6, 0x96, 0xA5},
		2, {0xA5, 0x65},
		2, {0xC0, 0xCD}
	},
	{
		2, {0x90, 0xCC},
		3, {0xE6, 0x98, 0x94},
		2, {0x14, 0x66},
		2, {0xC0, 0xCE}
	},
	{
		2, {0x90, 0xCD},
		3, {0xE6, 0x9E, 0x90},
		2, {0x90, 0x67},
		2, {0xC0, 0xCF}
	},
	{
		2, {0x90, 0xCE},
		3, {0xE7, 0x9F, 0xB3},
		2, {0xF3, 0x77},
		2, {0xC0, 0xD0}
	},
	{
		2, {0x90, 0xCF},
		3, {0xE7, 0xA9, 0x8D},
		2, {0x4D, 0x7A},
		2, {0xC0, 0xD1}
	},
	{
		2, {0x90, 0xD0},
		3, {0xE7, 0xB1, 0x8D},
		2, {0x4D, 0x7C},
		2, {0xC0, 0xD2}
	},
	{
		2, {0x90, 0xD1},
		3, {0xE7, 0xB8, 0xBE},
		2, {0x3E, 0x7E},
		2, {0xC0, 0xD3}
	},
	{
		2, {0x90, 0xD2},
		3, {0xE8, 0x84, 0x8A},
		2, {0x0A, 0x81},
		2, {0xC0, 0xD4}
	},
	{
		2, {0x90, 0xD3},
		3, {0xE8, 0xB2, 0xAC},
		2, {0xAC, 0x8C},
		2, {0xC0, 0xD5}
	},
	{
		2, {0x90, 0xD4},
		3, {0xE8, 0xB5, 0xA4},
		2, {0x64, 0x8D},
		2, {0xC0, 0xD6}
	},
	{
		2, {0x90, 0xD5},
		3, {0xE8, 0xB7, 0xA1},
		2, {0xE1, 0x8D},
		2, {0xC0, 0xD7}
	},
	{
		2, {0x90, 0xD6},
		3, {0xE8, 0xB9, 0x9F},
		2, {0x5F, 0x8E},
		2, {0xC0, 0xD8}
	},
	{
		2, {0x90, 0xD7},
		3, {0xE7, 0xA2, 0xA9},
		2, {0xA9, 0x78},
		2, {0xC0, 0xD9}
	},
	{
		2, {0x90, 0xD8},
		3, {0xE5, 0x88, 0x87},
		2, {0x07, 0x52},
		2, {0xC0, 0xDA}
	},
	{
		2, {0x90, 0xD9},
		3, {0xE6, 0x8B, 0x99},
		2, {0xD9, 0x62},
		2, {0xC0, 0xDB}
	},
	{
		2, {0x90, 0xDA},
		3, {0xE6, 0x8E, 0xA5},
		2, {0xA5, 0x63},
		2, {0xC0, 0xDC}
	},
	{
		2, {0x90, 0xDB},
		3, {0xE6, 0x91, 0x82},
		2, {0x42, 0x64},
		2, {0xC0, 0xDD}
	},
	{
		2, {0x90, 0xDC},
		3, {0xE6, 0x8A, 0x98},
		2, {0x98, 0x62},
		2, {0xC0, 0xDE}
	},
	{
		2, {0x90, 0xDD},
		3, {0xE8, 0xA8, 0xAD},
		2, {0x2D, 0x8A},
		2, {0xC0, 0xDF}
	},
	{
		2, {0x90, 0xDE},
		3, {0xE7, 0xAA, 0x83},
		2, {0x83, 0x7A},
		2, {0xC0, 0xE0}
	},
	{
		2, {0x90, 0xDF},
		3, {0xE7, 0xAF, 0x80},
		2, {0xC0, 0x7B},
		2, {0xC0, 0xE1}
	},
	{
		2, {0x90, 0xE0},
		3, {0xE8, 0xAA, 0xAC},
		2, {0xAC, 0x8A},
		2, {0xC0, 0xE2}
	},
	{
		2, {0x90, 0xE1},
		3, {0xE9, 0x9B, 0xAA},
		2, {0xEA, 0x96},
		2, {0xC0, 0xE3}
	},
	{
		2, {0x90, 0xE2},
		3, {0xE7, 0xB5, 0xB6},
		2, {0x76, 0x7D},
		2, {0xC0, 0xE4}
	},
	{
		2, {0x90, 0xE3},
		3, {0xE8, 0x88, 0x8C},
		2, {0x0C, 0x82},
		2, {0xC0, 0xE5}
	},
	{
		2, {0x90, 0xE4},
		3, {0xE8, 0x9D, 0x89},
		2, {0x49, 0x87},
		2, {0xC0, 0xE6}
	},
	{
		2, {0x90, 0xE5},
		3, {0xE4, 0xBB, 0x99},
		2, {0xD9, 0x4E},
		2, {0xC0, 0xE7}
	},
	{
		2, {0x90, 0xE6},
		3, {0xE5, 0x85, 0x88},
		2, {0x48, 0x51},
		2, {0xC0, 0xE8}
	},
	{
		2, {0x90, 0xE7},
		3, {0xE5, 0x8D, 0x83},
		2, {0x43, 0x53},
		2, {0xC0, 0xE9}
	},
	{
		2, {0x90, 0xE8},
		3, {0xE5, 0x8D, 0xA0},
		2, {0x60, 0x53},
		2, {0xC0, 0xEA}
	},
	{
		2, {0x90, 0xE9},
		3, {0xE5, 0xAE, 0xA3},
		2, {0xA3, 0x5B},
		2, {0xC0, 0xEB}
	},
	{
		2, {0x90, 0xEA},
		3, {0xE5, 0xB0, 0x82},
		2, {0x02, 0x5C},
		2, {0xC0, 0xEC}
	},
	{
		2, {0x90, 0xEB},
		3, {0xE5, 0xB0, 0x96},
		2, {0x16, 0x5C},
		2, {0xC0, 0xED}
	},
	{
		2, {0x90, 0xEC},
		3, {0xE5, 0xB7, 0x9D},
		2, {0xDD, 0x5D},
		2, {0xC0, 0xEE}
	},
	{
		2, {0x90, 0xED},
		3, {0xE6, 0x88, 0xA6},
		2, {0x26, 0x62},
		2, {0xC0, 0xEF}
	},
	{
		2, {0x90, 0xEE},
		3, {0xE6, 0x89, 0x87},
		2, {0x47, 0x62},
		2, {0xC0, 0xF0}
	},
	{
		2, {0x90, 0xEF},
		3, {0xE6, 0x92, 0xB0},
		2, {0xB0, 0x64},
		2, {0xC0, 0xF1}
	},
	{
		2, {0x90, 0xF0},
		3, {0xE6, 0xA0, 0x93},
		2, {0x13, 0x68},
		2, {0xC0, 0xF2}
	},
	{
		2, {0x90, 0xF1},
		3, {0xE6, 0xA0, 0xB4},
		2, {0x34, 0x68},
		2, {0xC0, 0xF3}
	},
	{
		2, {0x90, 0xF2},
		3, {0xE6, 0xB3, 0x89},
		2, {0xC9, 0x6C},
		2, {0xC0, 0xF4}
	},
	{
		2, {0x90, 0xF3},
		3, {0xE6, 0xB5, 0x85},
		2, {0x45, 0x6D},
		2, {0xC0, 0xF5}
	},
	{
		2, {0x90, 0xF4},
		3, {0xE6, 0xB4, 0x97},
		2, {0x17, 0x6D},
		2, {0xC0, 0xF6}
	},
	{
		2, {0x90, 0xF5},
		3, {0xE6, 0x9F, 0x93},
		2, {0xD3, 0x67},
		2, {0xC0, 0xF7}
	},
	{
		2, {0x90, 0xF6},
		3, {0xE6, 0xBD, 0x9C},
		2, {0x5C, 0x6F},
		2, {0xC0, 0xF8}
	},
	{
		2, {0x90, 0xF7},
		3, {0xE7, 0x85, 0x8E},
		2, {0x4E, 0x71},
		2, {0xC0, 0xF9}
	},
	{
		2, {0x90, 0xF8},
		3, {0xE7, 0x85, 0xBD},
		2, {0x7D, 0x71},
		2, {0xC0, 0xFA}
	},
	{
		2, {0x90, 0xF9},
		3, {0xE6, 0x97, 0x8B},
		2, {0xCB, 0x65},
		2, {0xC0, 0xFB}
	},
	{
		2, {0x90, 0xFA},
		3, {0xE7, 0xA9, 0xBF},
		2, {0x7F, 0x7A},
		2, {0xC0, 0xFC}
	},
	{
		2, {0x90, 0xFB},
		3, {0xE7, 0xAE, 0xAD},
		2, {0xAD, 0x7B},
		2, {0xC0, 0xFD}
	},
	{
		2, {0x90, 0xFC},
		3, {0xE7, 0xB7, 0x9A},
		2, {0xDA, 0x7D},
		2, {0xC0, 0xFE}
	},
	{
		2, {0x91, 0x40},
		3, {0xE7, 0xB9, 0x8A},
		2, {0x4A, 0x7E},
		2, {0xC1, 0xA1}
	},
	{
		2, {0x91, 0x41},
		3, {0xE7, 0xBE, 0xA8},
		2, {0xA8, 0x7F},
		2, {0xC1, 0xA2}
	},
	{
		2, {0x91, 0x42},
		3, {0xE8, 0x85, 0xBA},
		2, {0x7A, 0x81},
		2, {0xC1, 0xA3}
	},
	{
		2, {0x91, 0x43},
		3, {0xE8, 0x88, 0x9B},
		2, {0x1B, 0x82},
		2, {0xC1, 0xA4}
	},
	{
		2, {0x91, 0x44},
		3, {0xE8, 0x88, 0xB9},
		2, {0x39, 0x82},
		2, {0xC1, 0xA5}
	},
	{
		2, {0x91, 0x45},
		3, {0xE8, 0x96, 0xA6},
		2, {0xA6, 0x85},
		2, {0xC1, 0xA6}
	},
	{
		2, {0x91, 0x46},
		3, {0xE8, 0xA9, 0xAE},
		2, {0x6E, 0x8A},
		2, {0xC1, 0xA7}
	},
	{
		2, {0x91, 0x47},
		3, {0xE8, 0xB3, 0x8E},
		2, {0xCE, 0x8C},
		2, {0xC1, 0xA8}
	},
	{
		2, {0x91, 0x48},
		3, {0xE8, 0xB7, 0xB5},
		2, {0xF5, 0x8D},
		2, {0xC1, 0xA9}
	},
	{
		2, {0x91, 0x49},
		3, {0xE9, 0x81, 0xB8},
		2, {0x78, 0x90},
		2, {0xC1, 0xAA}
	},
	{
		2, {0x91, 0x4A},
		3, {0xE9, 0x81, 0xB7},
		2, {0x77, 0x90},
		2, {0xC1, 0xAB}
	},
	{
		2, {0x91, 0x4B},
		3, {0xE9, 0x8A, 0xAD},
		2, {0xAD, 0x92},
		2, {0xC1, 0xAC}
	},
	{
		2, {0x91, 0x4C},
		3, {0xE9, 0x8A, 0x91},
		2, {0x91, 0x92},
		2, {0xC1, 0xAD}
	},
	{
		2, {0x91, 0x4D},
		3, {0xE9, 0x96, 0x83},
		2, {0x83, 0x95},
		2, {0xC1, 0xAE}
	},
	{
		2, {0x91, 0x4E},
		3, {0xE9, 0xAE, 0xAE},
		2, {0xAE, 0x9B},
		2, {0xC1, 0xAF}
	},
	{
		2, {0x91, 0x4F},
		3, {0xE5, 0x89, 0x8D},
		2, {0x4D, 0x52},
		2, {0xC1, 0xB0}
	},
	{
		2, {0x91, 0x50},
		3, {0xE5, 0x96, 0x84},
		2, {0x84, 0x55},
		2, {0xC1, 0xB1}
	},
	{
		2, {0x91, 0x51},
		3, {0xE6, 0xBC, 0xB8},
		2, {0x38, 0x6F},
		2, {0xC1, 0xB2}
	},
	{
		2, {0x91, 0x52},
		3, {0xE7, 0x84, 0xB6},
		2, {0x36, 0x71},
		2, {0xC1, 0xB3}
	},
	{
		2, {0x91, 0x53},
		3, {0xE5, 0x85, 0xA8},
		2, {0x68, 0x51},
		2, {0xC1, 0xB4}
	},
	{
		2, {0x91, 0x54},
		3, {0xE7, 0xA6, 0x85},
		2, {0x85, 0x79},
		2, {0xC1, 0xB5}
	},
	{
		2, {0x91, 0x55},
		3, {0xE7, 0xB9, 0x95},
		2, {0x55, 0x7E},
		2, {0xC1, 0xB6}
	},
	{
		2, {0x91, 0x56},
		3, {0xE8, 0x86, 0xB3},
		2, {0xB3, 0x81},
		2, {0xC1, 0xB7}
	},
	{
		2, {0x91, 0x57},
		3, {0xE7, 0xB3, 0x8E},
		2, {0xCE, 0x7C},
		2, {0xC1, 0xB8}
	},
	{
		2, {0x91, 0x58},
		3, {0xE5, 0x99, 0x8C},
		2, {0x4C, 0x56},
		2, {0xC1, 0xB9}
	},
	{
		2, {0x91, 0x59},
		3, {0xE5, 0xA1, 0x91},
		2, {0x51, 0x58},
		2, {0xC1, 0xBA}
	},
	{
		2, {0x91, 0x5A},
		3, {0xE5, 0xB2, 0xA8},
		2, {0xA8, 0x5C},
		2, {0xC1, 0xBB}
	},
	{
		2, {0x91, 0x5B},
		3, {0xE6, 0x8E, 0xAA},
		2, {0xAA, 0x63},
		2, {0xC1, 0xBC}
	},
	{
		2, {0x91, 0x5C},
		3, {0xE6, 0x9B, 0xBE},
		2, {0xFE, 0x66},
		2, {0xC1, 0xBD}
	},
	{
		2, {0x91, 0x5D},
		3, {0xE6, 0x9B, 0xBD},
		2, {0xFD, 0x66},
		2, {0xC1, 0xBE}
	},
	{
		2, {0x91, 0x5E},
		3, {0xE6, 0xA5, 0x9A},
		2, {0x5A, 0x69},
		2, {0xC1, 0xBF}
	},
	{
		2, {0x91, 0x5F},
		3, {0xE7, 0x8B, 0x99},
		2, {0xD9, 0x72},
		2, {0xC1, 0xC0}
	},
	{
		2, {0x91, 0x60},
		3, {0xE7, 0x96, 0x8F},
		2, {0x8F, 0x75},
		2, {0xC1, 0xC1}
	},
	{
		2, {0x91, 0x61},
		3, {0xE7, 0x96, 0x8E},
		2, {0x8E, 0x75},
		2, {0xC1, 0xC2}
	},
	{
		2, {0x91, 0x62},
		3, {0xE7, 0xA4, 0x8E},
		2, {0x0E, 0x79},
		2, {0xC1, 0xC3}
	},
	{
		2, {0x91, 0x63},
		3, {0xE7, 0xA5, 0x96},
		2, {0x56, 0x79},
		2, {0xC1, 0xC4}
	},
	{
		2, {0x91, 0x64},
		3, {0xE7, 0xA7, 0x9F},
		2, {0xDF, 0x79},
		2, {0xC1, 0xC5}
	},
	{
		2, {0x91, 0x65},
		3, {0xE7, 0xB2, 0x97},
		2, {0x97, 0x7C},
		2, {0xC1, 0xC6}
	},
	{
		2, {0x91, 0x66},
		3, {0xE7, 0xB4, 0xA0},
		2, {0x20, 0x7D},
		2, {0xC1, 0xC7}
	},
	{
		2, {0x91, 0x67},
		3, {0xE7, 0xB5, 0x84},
		2, {0x44, 0x7D},
		2, {0xC1, 0xC8}
	},
	{
		2, {0x91, 0x68},
		3, {0xE8, 0x98, 0x87},
		2, {0x07, 0x86},
		2, {0xC1, 0xC9}
	},
	{
		2, {0x91, 0x69},
		3, {0xE8, 0xA8, 0xB4},
		2, {0x34, 0x8A},
		2, {0xC1, 0xCA}
	},
	{
		2, {0x91, 0x6A},
		3, {0xE9, 0x98, 0xBB},
		2, {0x3B, 0x96},
		2, {0xC1, 0xCB}
	},
	{
		2, {0x91, 0x6B},
		3, {0xE9, 0x81, 0xA1},
		2, {0x61, 0x90},
		2, {0xC1, 0xCC}
	},
	{
		2, {0x91, 0x6C},
		3, {0xE9, 0xBC, 0xA0},
		2, {0x20, 0x9F},
		2, {0xC1, 0xCD}
	},
	{
		2, {0x91, 0x6D},
		3, {0xE5, 0x83, 0xA7},
		2, {0xE7, 0x50},
		2, {0xC1, 0xCE}
	},
	{
		2, {0x91, 0x6E},
		3, {0xE5, 0x89, 0xB5},
		2, {0x75, 0x52},
		2, {0xC1, 0xCF}
	},
	{
		2, {0x91, 0x6F},
		3, {0xE5, 0x8F, 0x8C},
		2, {0xCC, 0x53},
		2, {0xC1, 0xD0}
	},
	{
		2, {0x91, 0x70},
		3, {0xE5, 0x8F, 0xA2},
		2, {0xE2, 0x53},
		2, {0xC1, 0xD1}
	},
	{
		2, {0x91, 0x71},
		3, {0xE5, 0x80, 0x89},
		2, {0x09, 0x50},
		2, {0xC1, 0xD2}
	},
	{
		2, {0x91, 0x72},
		3, {0xE5, 0x96, 0xAA},
		2, {0xAA, 0x55},
		2, {0xC1, 0xD3}
	},
	{
		2, {0x91, 0x73},
		3, {0xE5, 0xA3, 0xAE},
		2, {0xEE, 0x58},
		2, {0xC1, 0xD4}
	},
	{
		2, {0x91, 0x74},
		3, {0xE5, 0xA5, 0x8F},
		2, {0x4F, 0x59},
		2, {0xC1, 0xD5}
	},
	{
		2, {0x91, 0x75},
		3, {0xE7, 0x88, 0xBD},
		2, {0x3D, 0x72},
		2, {0xC1, 0xD6}
	},
	{
		2, {0x91, 0x76},
		3, {0xE5, 0xAE, 0x8B},
		2, {0x8B, 0x5B},
		2, {0xC1, 0xD7}
	},
	{
		2, {0x91, 0x77},
		3, {0xE5, 0xB1, 0xA4},
		2, {0x64, 0x5C},
		2, {0xC1, 0xD8}
	},
	{
		2, {0x91, 0x78},
		3, {0xE5, 0x8C, 0x9D},
		2, {0x1D, 0x53},
		2, {0xC1, 0xD9}
	},
	{
		2, {0x91, 0x79},
		3, {0xE6, 0x83, 0xA3},
		2, {0xE3, 0x60},
		2, {0xC1, 0xDA}
	},
	{
		2, {0x91, 0x7A},
		3, {0xE6, 0x83, 0xB3},
		2, {0xF3, 0x60},
		2, {0xC1, 0xDB}
	},
	{
		2, {0x91, 0x7B},
		3, {0xE6, 0x8D, 0x9C},
		2, {0x5C, 0x63},
		2, {0xC1, 0xDC}
	},
	{
		2, {0x91, 0x7C},
		3, {0xE6, 0x8E, 0x83},
		2, {0x83, 0x63},
		2, {0xC1, 0xDD}
	},
	{
		2, {0x91, 0x7D},
		3, {0xE6, 0x8C, 0xBF},
		2, {0x3F, 0x63},
		2, {0xC1, 0xDE}
	},
	{
		2, {0x91, 0x7E},
		3, {0xE6, 0x8E, 0xBB},
		2, {0xBB, 0x63},
		2, {0xC1, 0xDF}
	},
	{
		2, {0x91, 0x80},
		3, {0xE6, 0x93, 0x8D},
		2, {0xCD, 0x64},
		2, {0xC1, 0xE0}
	},
	{
		2, {0x91, 0x81},
		3, {0xE6, 0x97, 0xA9},
		2, {0xE9, 0x65},
		2, {0xC1, 0xE1}
	},
	{
		2, {0x91, 0x82},
		3, {0xE6, 0x9B, 0xB9},
		2, {0xF9, 0x66},
		2, {0xC1, 0xE2}
	},
	{
		2, {0x91, 0x83},
		3, {0xE5, 0xB7, 0xA3},
		2, {0xE3, 0x5D},
		2, {0xC1, 0xE3}
	},
	{
		2, {0x91, 0x84},
		3, {0xE6, 0xA7, 0x8D},
		2, {0xCD, 0x69},
		2, {0xC1, 0xE4}
	},
	{
		2, {0x91, 0x85},
		3, {0xE6, 0xA7, 0xBD},
		2, {0xFD, 0x69},
		2, {0xC1, 0xE5}
	},
	{
		2, {0x91, 0x86},
		3, {0xE6, 0xBC, 0x95},
		2, {0x15, 0x6F},
		2, {0xC1, 0xE6}
	},
	{
		2, {0x91, 0x87},
		3, {0xE7, 0x87, 0xA5},
		2, {0xE5, 0x71},
		2, {0xC1, 0xE7}
	},
	{
		2, {0x91, 0x88},
		3, {0xE4, 0xBA, 0x89},
		2, {0x89, 0x4E},
		2, {0xC1, 0xE8}
	},
	{
		2, {0x91, 0x89},
		3, {0xE7, 0x97, 0xA9},
		2, {0xE9, 0x75},
		2, {0xC1, 0xE9}
	},
	{
		2, {0x91, 0x8A},
		3, {0xE7, 0x9B, 0xB8},
		2, {0xF8, 0x76},
		2, {0xC1, 0xEA}
	},
	{
		2, {0x91, 0x8B},
		3, {0xE7, 0xAA, 0x93},
		2, {0x93, 0x7A},
		2, {0xC1, 0xEB}
	},
	{
		2, {0x91, 0x8C},
		3, {0xE7, 0xB3, 0x9F},
		2, {0xDF, 0x7C},
		2, {0xC1, 0xEC}
	},
	{
		2, {0x91, 0x8D},
		3, {0xE7, 0xB7, 0x8F},
		2, {0xCF, 0x7D},
		2, {0xC1, 0xED}
	},
	{
		2, {0x91, 0x8E},
		3, {0xE7, 0xB6, 0x9C},
		2, {0x9C, 0x7D},
		2, {0xC1, 0xEE}
	},
	{
		2, {0x91, 0x8F},
		3, {0xE8, 0x81, 0xA1},
		2, {0x61, 0x80},
		2, {0xC1, 0xEF}
	},
	{
		2, {0x91, 0x90},
		3, {0xE8, 0x8D, 0x89},
		2, {0x49, 0x83},
		2, {0xC1, 0xF0}
	},
	{
		2, {0x91, 0x91},
		3, {0xE8, 0x8D, 0x98},
		2, {0x58, 0x83},
		2, {0xC1, 0xF1}
	},
	{
		2, {0x91, 0x92},
		3, {0xE8, 0x91, 0xAC},
		2, {0x6C, 0x84},
		2, {0xC1, 0xF2}
	},
	{
		2, {0x91, 0x93},
		3, {0xE8, 0x92, 0xBC},
		2, {0xBC, 0x84},
		2, {0xC1, 0xF3}
	},
	{
		2, {0x91, 0x94},
		3, {0xE8, 0x97, 0xBB},
		2, {0xFB, 0x85},
		2, {0xC1, 0xF4}
	},
	{
		2, {0x91, 0x95},
		3, {0xE8, 0xA3, 0x85},
		2, {0xC5, 0x88},
		2, {0xC1, 0xF5}
	},
	{
		2, {0x91, 0x96},
		3, {0xE8, 0xB5, 0xB0},
		2, {0x70, 0x8D},
		2, {0xC1, 0xF6}
	},
	{
		2, {0x91, 0x97},
		3, {0xE9, 0x80, 0x81},
		2, {0x01, 0x90},
		2, {0xC1, 0xF7}
	},
	{
		2, {0x91, 0x98},
		3, {0xE9, 0x81, 0xAD},
		2, {0x6D, 0x90},
		2, {0xC1, 0xF8}
	},
	{
		2, {0x91, 0x99},
		3, {0xE9, 0x8E, 0x97},
		2, {0x97, 0x93},
		2, {0xC1, 0xF9}
	},
	{
		2, {0x91, 0x9A},
		3, {0xE9, 0x9C, 0x9C},
		2, {0x1C, 0x97},
		2, {0xC1, 0xFA}
	},
	{
		2, {0x91, 0x9B},
		3, {0xE9, 0xA8, 0x92},
		2, {0x12, 0x9A},
		2, {0xC1, 0xFB}
	},
	{
		2, {0x91, 0x9C},
		3, {0xE5, 0x83, 0x8F},
		2, {0xCF, 0x50},
		2, {0xC1, 0xFC}
	},
	{
		2, {0x91, 0x9D},
		3, {0xE5, 0xA2, 0x97},
		2, {0x97, 0x58},
		2, {0xC1, 0xFD}
	},
	{
		2, {0x91, 0x9E},
		3, {0xE6, 0x86, 0x8E},
		2, {0x8E, 0x61},
		2, {0xC1, 0xFE}
	},
	{
		2, {0x91, 0x9F},
		3, {0xE8, 0x87, 0x93},
		2, {0xD3, 0x81},
		2, {0xC2, 0xA1}
	},
	{
		2, {0x91, 0xA0},
		3, {0xE8, 0x94, 0xB5},
		2, {0x35, 0x85},
		2, {0xC2, 0xA2}
	},
	{
		2, {0x91, 0xA1},
		3, {0xE8, 0xB4, 0x88},
		2, {0x08, 0x8D},
		2, {0xC2, 0xA3}
	},
	{
		2, {0x91, 0xA2},
		3, {0xE9, 0x80, 0xA0},
		2, {0x20, 0x90},
		2, {0xC2, 0xA4}
	},
	{
		2, {0x91, 0xA3},
		3, {0xE4, 0xBF, 0x83},
		2, {0xC3, 0x4F},
		2, {0xC2, 0xA5}
	},
	{
		2, {0x91, 0xA4},
		3, {0xE5, 0x81, 0xB4},
		2, {0x74, 0x50},
		2, {0xC2, 0xA6}
	},
	{
		2, {0x91, 0xA5},
		3, {0xE5, 0x89, 0x87},
		2, {0x47, 0x52},
		2, {0xC2, 0xA7}
	},
	{
		2, {0x91, 0xA6},
		3, {0xE5, 0x8D, 0xB3},
		2, {0x73, 0x53},
		2, {0xC2, 0xA8}
	},
	{
		2, {0x91, 0xA7},
		3, {0xE6, 0x81, 0xAF},
		2, {0x6F, 0x60},
		2, {0xC2, 0xA9}
	},
	{
		2, {0x91, 0xA8},
		3, {0xE6, 0x8D, 0x89},
		2, {0x49, 0x63},
		2, {0xC2, 0xAA}
	},
	{
		2, {0x91, 0xA9},
		3, {0xE6, 0x9D, 0x9F},
		2, {0x5F, 0x67},
		2, {0xC2, 0xAB}
	},
	{
		2, {0x91, 0xAA},
		3, {0xE6, 0xB8, 0xAC},
		2, {0x2C, 0x6E},
		2, {0xC2, 0xAC}
	},
	{
		2, {0x91, 0xAB},
		3, {0xE8, 0xB6, 0xB3},
		2, {0xB3, 0x8D},
		2, {0xC2, 0xAD}
	},
	{
		2, {0x91, 0xAC},
		3, {0xE9, 0x80, 0x9F},
		2, {0x1F, 0x90},
		2, {0xC2, 0xAE}
	},
	{
		2, {0x91, 0xAD},
		3, {0xE4, 0xBF, 0x97},
		2, {0xD7, 0x4F},
		2, {0xC2, 0xAF}
	},
	{
		2, {0x91, 0xAE},
		3, {0xE5, 0xB1, 0x9E},
		2, {0x5E, 0x5C},
		2, {0xC2, 0xB0}
	},
	{
		2, {0x91, 0xAF},
		3, {0xE8, 0xB3, 0x8A},
		2, {0xCA, 0x8C},
		2, {0xC2, 0xB1}
	},
	{
		2, {0x91, 0xB0},
		3, {0xE6, 0x97, 0x8F},
		2, {0xCF, 0x65},
		2, {0xC2, 0xB2}
	},
	{
		2, {0x91, 0xB1},
		3, {0xE7, 0xB6, 0x9A},
		2, {0x9A, 0x7D},
		2, {0xC2, 0xB3}
	},
	{
		2, {0x91, 0xB2},
		3, {0xE5, 0x8D, 0x92},
		2, {0x52, 0x53},
		2, {0xC2, 0xB4}
	},
	{
		2, {0x91, 0xB3},
		3, {0xE8, 0xA2, 0x96},
		2, {0x96, 0x88},
		2, {0xC2, 0xB5}
	},
	{
		2, {0x91, 0xB4},
		3, {0xE5, 0x85, 0xB6},
		2, {0x76, 0x51},
		2, {0xC2, 0xB6}
	},
	{
		2, {0x91, 0xB5},
		3, {0xE6, 0x8F, 0x83},
		2, {0xC3, 0x63},
		2, {0xC2, 0xB7}
	},
	{
		2, {0x91, 0xB6},
		3, {0xE5, 0xAD, 0x98},
		2, {0x58, 0x5B},
		2, {0xC2, 0xB8}
	},
	{
		2, {0x91, 0xB7},
		3, {0xE5, 0xAD, 0xAB},
		2, {0x6B, 0x5B},
		2, {0xC2, 0xB9}
	},
	{
		2, {0x91, 0xB8},
		3, {0xE5, 0xB0, 0x8A},
		2, {0x0A, 0x5C},
		2, {0xC2, 0xBA}
	},
	{
		2, {0x91, 0xB9},
		3, {0xE6, 0x90, 0x8D},
		2, {0x0D, 0x64},
		2, {0xC2, 0xBB}
	},
	{
		2, {0x91, 0xBA},
		3, {0xE6, 0x9D, 0x91},
		2, {0x51, 0x67},
		2, {0xC2, 0xBC}
	},
	{
		2, {0x91, 0xBB},
		3, {0xE9, 0x81, 0x9C},
		2, {0x5C, 0x90},
		2, {0xC2, 0xBD}
	},
	{
		2, {0x91, 0xBC},
		3, {0xE4, 0xBB, 0x96},
		2, {0xD6, 0x4E},
		2, {0xC2, 0xBE}
	},
	{
		2, {0x91, 0xBD},
		3, {0xE5, 0xA4, 0x9A},
		2, {0x1A, 0x59},
		2, {0xC2, 0xBF}
	},
	{
		2, {0x91, 0xBE},
		3, {0xE5, 0xA4, 0xAA},
		2, {0x2A, 0x59},
		2, {0xC2, 0xC0}
	},
	{
		2, {0x91, 0xBF},
		3, {0xE6, 0xB1, 0xB0},
		2, {0x70, 0x6C},
		2, {0xC2, 0xC1}
	},
	{
		2, {0x91, 0xC0},
		3, {0xE8, 0xA9, 0x91},
		2, {0x51, 0x8A},
		2, {0xC2, 0xC2}
	},
	{
		2, {0x91, 0xC1},
		3, {0xE5, 0x94, 0xBE},
		2, {0x3E, 0x55},
		2, {0xC2, 0xC3}
	},
	{
		2, {0x91, 0xC2},
		3, {0xE5, 0xA0, 0x95},
		2, {0x15, 0x58},
		2, {0xC2, 0xC4}
	},
	{
		2, {0x91, 0xC3},
		3, {0xE5, 0xA6, 0xA5},
		2, {0xA5, 0x59},
		2, {0xC2, 0xC5}
	},
	{
		2, {0x91, 0xC4},
		3, {0xE6, 0x83, 0xB0},
		2, {0xF0, 0x60},
		2, {0xC2, 0xC6}
	},
	{
		2, {0x91, 0xC5},
		3, {0xE6, 0x89, 0x93},
		2, {0x53, 0x62},
		2, {0xC2, 0xC7}
	},
	{
		2, {0x91, 0xC6},
		3, {0xE6, 0x9F, 0x81},
		2, {0xC1, 0x67},
		2, {0xC2, 0xC8}
	},
	{
		2, {0x91, 0xC7},
		3, {0xE8, 0x88, 0xB5},
		2, {0x35, 0x82},
		2, {0xC2, 0xC9}
	},
	{
		2, {0x91, 0xC8},
		3, {0xE6, 0xA5, 0x95},
		2, {0x55, 0x69},
		2, {0xC2, 0xCA}
	},
	{
		2, {0x91, 0xC9},
		3, {0xE9, 0x99, 0x80},
		2, {0x40, 0x96},
		2, {0xC2, 0xCB}
	},
	{
		2, {0x91, 0xCA},
		3, {0xE9, 0xA7, 0x84},
		2, {0xC4, 0x99},
		2, {0xC2, 0xCC}
	},
	{
		2, {0x91, 0xCB},
		3, {0xE9, 0xA8, 0xA8},
		2, {0x28, 0x9A},
		2, {0xC2, 0xCD}
	},
	{
		2, {0x91, 0xCC},
		3, {0xE4, 0xBD, 0x93},
		2, {0x53, 0x4F},
		2, {0xC2, 0xCE}
	},
	{
		2, {0x91, 0xCD},
		3, {0xE5, 0xA0, 0x86},
		2, {0x06, 0x58},
		2, {0xC2, 0xCF}
	},
	{
		2, {0x91, 0xCE},
		3, {0xE5, 0xAF, 0xBE},
		2, {0xFE, 0x5B},
		2, {0xC2, 0xD0}
	},
	{
		2, {0x91, 0xCF},
		3, {0xE8, 0x80, 0x90},
		2, {0x10, 0x80},
		2, {0xC2, 0xD1}
	},
	{
		2, {0x91, 0xD0},
		3, {0xE5, 0xB2, 0xB1},
		2, {0xB1, 0x5C},
		2, {0xC2, 0xD2}
	},
	{
		2, {0x91, 0xD1},
		3, {0xE5, 0xB8, 0xAF},
		2, {0x2F, 0x5E},
		2, {0xC2, 0xD3}
	},
	{
		2, {0x91, 0xD2},
		3, {0xE5, 0xBE, 0x85},
		2, {0x85, 0x5F},
		2, {0xC2, 0xD4}
	},
	{
		2, {0x91, 0xD3},
		3, {0xE6, 0x80, 0xA0},
		2, {0x20, 0x60},
		2, {0xC2, 0xD5}
	},
	{
		2, {0x91, 0xD4},
		3, {0xE6, 0x85, 0x8B},
		2, {0x4B, 0x61},
		2, {0xC2, 0xD6}
	},
	{
		2, {0x91, 0xD5},
		3, {0xE6, 0x88, 0xB4},
		2, {0x34, 0x62},
		2, {0xC2, 0xD7}
	},
	{
		2, {0x91, 0xD6},
		3, {0xE6, 0x9B, 0xBF},
		2, {0xFF, 0x66},
		2, {0xC2, 0xD8}
	},
	{
		2, {0x91, 0xD7},
		3, {0xE6, 0xB3, 0xB0},
		2, {0xF0, 0x6C},
		2, {0xC2, 0xD9}
	},
	{
		2, {0x91, 0xD8},
		3, {0xE6, 0xBB, 0x9E},
		2, {0xDE, 0x6E},
		2, {0xC2, 0xDA}
	},
	{
		2, {0x91, 0xD9},
		3, {0xE8, 0x83, 0x8E},
		2, {0xCE, 0x80},
		2, {0xC2, 0xDB}
	},
	{
		2, {0x91, 0xDA},
		3, {0xE8, 0x85, 0xBF},
		2, {0x7F, 0x81},
		2, {0xC2, 0xDC}
	},
	{
		2, {0x91, 0xDB},
		3, {0xE8, 0x8B, 0x94},
		2, {0xD4, 0x82},
		2, {0xC2, 0xDD}
	},
	{
		2, {0x91, 0xDC},
		3, {0xE8, 0xA2, 0x8B},
		2, {0x8B, 0x88},
		2, {0xC2, 0xDE}
	},
	{
		2, {0x91, 0xDD},
		3, {0xE8, 0xB2, 0xB8},
		2, {0xB8, 0x8C},
		2, {0xC2, 0xDF}
	},
	{
		2, {0x91, 0xDE},
		3, {0xE9, 0x80, 0x80},
		2, {0x00, 0x90},
		2, {0xC2, 0xE0}
	},
	{
		2, {0x91, 0xDF},
		3, {0xE9, 0x80, 0xAE},
		2, {0x2E, 0x90},
		2, {0xC2, 0xE1}
	},
	{
		2, {0x91, 0xE0},
		3, {0xE9, 0x9A, 0x8A},
		2, {0x8A, 0x96},
		2, {0xC2, 0xE2}
	},
	{
		2, {0x91, 0xE1},
		3, {0xE9, 0xBB, 0x9B},
		2, {0xDB, 0x9E},
		2, {0xC2, 0xE3}
	},
	{
		2, {0x91, 0xE2},
		3, {0xE9, 0xAF, 0x9B},
		2, {0xDB, 0x9B},
		2, {0xC2, 0xE4}
	},
	{
		2, {0x91, 0xE3},
		3, {0xE4, 0xBB, 0xA3},
		2, {0xE3, 0x4E},
		2, {0xC2, 0xE5}
	},
	{
		2, {0x91, 0xE4},
		3, {0xE5, 0x8F, 0xB0},
		2, {0xF0, 0x53},
		2, {0xC2, 0xE6}
	},
	{
		2, {0x91, 0xE5},
		3, {0xE5, 0xA4, 0xA7},
		2, {0x27, 0x59},
		2, {0xC2, 0xE7}
	},
	{
		2, {0x91, 0xE6},
		3, {0xE7, 0xAC, 0xAC},
		2, {0x2C, 0x7B},
		2, {0xC2, 0xE8}
	},
	{
		2, {0x91, 0xE7},
		3, {0xE9, 0x86, 0x8D},
		2, {0x8D, 0x91},
		2, {0xC2, 0xE9}
	},
	{
		2, {0x91, 0xE8},
		3, {0xE9, 0xA1, 0x8C},
		2, {0x4C, 0x98},
		2, {0xC2, 0xEA}
	},
	{
		2, {0x91, 0xE9},
		3, {0xE9, 0xB7, 0xB9},
		2, {0xF9, 0x9D},
		2, {0xC2, 0xEB}
	},
	{
		2, {0x91, 0xEA},
		3, {0xE6, 0xBB, 0x9D},
		2, {0xDD, 0x6E},
		2, {0xC2, 0xEC}
	},
	{
		2, {0x91, 0xEB},
		3, {0xE7, 0x80, 0xA7},
		2, {0x27, 0x70},
		2, {0xC2, 0xED}
	},
	{
		2, {0x91, 0xEC},
		3, {0xE5, 0x8D, 0x93},
		2, {0x53, 0x53},
		2, {0xC2, 0xEE}
	},
	{
		2, {0x91, 0xED},
		3, {0xE5, 0x95, 0x84},
		2, {0x44, 0x55},
		2, {0xC2, 0xEF}
	},
	{
		2, {0x91, 0xEE},
		3, {0xE5, 0xAE, 0x85},
		2, {0x85, 0x5B},
		2, {0xC2, 0xF0}
	},
	{
		2, {0x91, 0xEF},
		3, {0xE6, 0x89, 0x98},
		2, {0x58, 0x62},
		2, {0xC2, 0xF1}
	},
	{
		2, {0x91, 0xF0},
		3, {0xE6, 0x8A, 0x9E},
		2, {0x9E, 0x62},
		2, {0xC2, 0xF2}
	},
	{
		2, {0x91, 0xF1},
		3, {0xE6, 0x8B, 0x93},
		2, {0xD3, 0x62},
		2, {0xC2, 0xF3}
	},
	{
		2, {0x91, 0xF2},
		3, {0xE6, 0xB2, 0xA2},
		2, {0xA2, 0x6C},
		2, {0xC2, 0xF4}
	},
	{
		2, {0x91, 0xF3},
		3, {0xE6, 0xBF, 0xAF},
		2, {0xEF, 0x6F},
		2, {0xC2, 0xF5}
	},
	{
		2, {0x91, 0xF4},
		3, {0xE7, 0x90, 0xA2},
		2, {0x22, 0x74},
		2, {0xC2, 0xF6}
	},
	{
		2, {0x91, 0xF5},
		3, {0xE8, 0xA8, 0x97},
		2, {0x17, 0x8A},
		2, {0xC2, 0xF7}
	},
	{
		2, {0x91, 0xF6},
		3, {0xE9, 0x90, 0xB8},
		2, {0x38, 0x94},
		2, {0xC2, 0xF8}
	},
	{
		2, {0x91, 0xF7},
		3, {0xE6, 0xBF, 0x81},
		2, {0xC1, 0x6F},
		2, {0xC2, 0xF9}
	},
	{
		2, {0x91, 0xF8},
		3, {0xE8, 0xAB, 0xBE},
		2, {0xFE, 0x8A},
		2, {0xC2, 0xFA}
	},
	{
		2, {0x91, 0xF9},
		3, {0xE8, 0x8C, 0xB8},
		2, {0x38, 0x83},
		2, {0xC2, 0xFB}
	},
	{
		2, {0x91, 0xFA},
		3, {0xE5, 0x87, 0xA7},
		2, {0xE7, 0x51},
		2, {0xC2, 0xFC}
	},
	{
		2, {0x91, 0xFB},
		3, {0xE8, 0x9B, 0xB8},
		2, {0xF8, 0x86},
		2, {0xC2, 0xFD}
	},
	{
		2, {0x91, 0xFC},
		3, {0xE5, 0x8F, 0xAA},
		2, {0xEA, 0x53},
		2, {0xC2, 0xFE}
	},
	{
		2, {0x92, 0x40},
		3, {0xE5, 0x8F, 0xA9},
		2, {0xE9, 0x53},
		2, {0xC3, 0xA1}
	},
	{
		2, {0x92, 0x41},
		3, {0xE4, 0xBD, 0x86},
		2, {0x46, 0x4F},
		2, {0xC3, 0xA2}
	},
	{
		2, {0x92, 0x42},
		3, {0xE9, 0x81, 0x94},
		2, {0x54, 0x90},
		2, {0xC3, 0xA3}
	},
	{
		2, {0x92, 0x43},
		3, {0xE8, 0xBE, 0xB0},
		2, {0xB0, 0x8F},
		2, {0xC3, 0xA4}
	},
	{
		2, {0x92, 0x44},
		3, {0xE5, 0xA5, 0xAA},
		2, {0x6A, 0x59},
		2, {0xC3, 0xA5}
	},
	{
		2, {0x92, 0x45},
		3, {0xE8, 0x84, 0xB1},
		2, {0x31, 0x81},
		2, {0xC3, 0xA6}
	},
	{
		2, {0x92, 0x46},
		3, {0xE5, 0xB7, 0xBD},
		2, {0xFD, 0x5D},
		2, {0xC3, 0xA7}
	},
	{
		2, {0x92, 0x47},
		3, {0xE7, 0xAB, 0xAA},
		2, {0xEA, 0x7A},
		2, {0xC3, 0xA8}
	},
	{
		2, {0x92, 0x48},
		3, {0xE8, 0xBE, 0xBF},
		2, {0xBF, 0x8F},
		2, {0xC3, 0xA9}
	},
	{
		2, {0x92, 0x49},
		3, {0xE6, 0xA3, 0x9A},
		2, {0xDA, 0x68},
		2, {0xC3, 0xAA}
	},
	{
		2, {0x92, 0x4A},
		3, {0xE8, 0xB0, 0xB7},
		2, {0x37, 0x8C},
		2, {0xC3, 0xAB}
	},
	{
		2, {0x92, 0x4B},
		3, {0xE7, 0x8B, 0xB8},
		2, {0xF8, 0x72},
		2, {0xC3, 0xAC}
	},
	{
		2, {0x92, 0x4C},
		3, {0xE9, 0xB1, 0x88},
		2, {0x48, 0x9C},
		2, {0xC3, 0xAD}
	},
	{
		2, {0x92, 0x4D},
		3, {0xE6, 0xA8, 0xBD},
		2, {0x3D, 0x6A},
		2, {0xC3, 0xAE}
	},
	{
		2, {0x92, 0x4E},
		3, {0xE8, 0xAA, 0xB0},
		2, {0xB0, 0x8A},
		2, {0xC3, 0xAF}
	},
	{
		2, {0x92, 0x4F},
		3, {0xE4, 0xB8, 0xB9},
		2, {0x39, 0x4E},
		2, {0xC3, 0xB0}
	},
	{
		2, {0x92, 0x50},
		3, {0xE5, 0x8D, 0x98},
		2, {0x58, 0x53},
		2, {0xC3, 0xB1}
	},
	{
		2, {0x92, 0x51},
		3, {0xE5, 0x98, 0x86},
		2, {0x06, 0x56},
		2, {0xC3, 0xB2}
	},
	{
		2, {0x92, 0x52},
		3, {0xE5, 0x9D, 0xA6},
		2, {0x66, 0x57},
		2, {0xC3, 0xB3}
	},
	{
		2, {0x92, 0x53},
		3, {0xE6, 0x8B, 0x85},
		2, {0xC5, 0x62},
		2, {0xC3, 0xB4}
	},
	{
		2, {0x92, 0x54},
		3, {0xE6, 0x8E, 0xA2},
		2, {0xA2, 0x63},
		2, {0xC3, 0xB5}
	},
	{
		2, {0x92, 0x55},
		3, {0xE6, 0x97, 0xA6},
		2, {0xE6, 0x65},
		2, {0xC3, 0xB6}
	},
	{
		2, {0x92, 0x56},
		3, {0xE6, 0xAD, 0x8E},
		2, {0x4E, 0x6B},
		2, {0xC3, 0xB7}
	},
	{
		2, {0x92, 0x57},
		3, {0xE6, 0xB7, 0xA1},
		2, {0xE1, 0x6D},
		2, {0xC3, 0xB8}
	},
	{
		2, {0x92, 0x58},
		3, {0xE6, 0xB9, 0x9B},
		2, {0x5B, 0x6E},
		2, {0xC3, 0xB9}
	},
	{
		2, {0x92, 0x59},
		3, {0xE7, 0x82, 0xAD},
		2, {0xAD, 0x70},
		2, {0xC3, 0xBA}
	},
	{
		2, {0x92, 0x5A},
		3, {0xE7, 0x9F, 0xAD},
		2, {0xED, 0x77},
		2, {0xC3, 0xBB}
	},
	{
		2, {0x92, 0x5B},
		3, {0xE7, 0xAB, 0xAF},
		2, {0xEF, 0x7A},
		2, {0xC3, 0xBC}
	},
	{
		2, {0x92, 0x5C},
		3, {0xE7, 0xAE, 0xAA},
		2, {0xAA, 0x7B},
		2, {0xC3, 0xBD}
	},
	{
		2, {0x92, 0x5D},
		3, {0xE7, 0xB6, 0xBB},
		2, {0xBB, 0x7D},
		2, {0xC3, 0xBE}
	},
	{
		2, {0x92, 0x5E},
		3, {0xE8, 0x80, 0xBD},
		2, {0x3D, 0x80},
		2, {0xC3, 0xBF}
	},
	{
		2, {0x92, 0x5F},
		3, {0xE8, 0x83, 0x86},
		2, {0xC6, 0x80},
		2, {0xC3, 0xC0}
	},
	{
		2, {0x92, 0x60},
		3, {0xE8, 0x9B, 0x8B},
		2, {0xCB, 0x86},
		2, {0xC3, 0xC1}
	},
	{
		2, {0x92, 0x61},
		3, {0xE8, 0xAA, 0x95},
		2, {0x95, 0x8A},
		2, {0xC3, 0xC2}
	},
	{
		2, {0x92, 0x62},
		3, {0xE9, 0x8D, 0x9B},
		2, {0x5B, 0x93},
		2, {0xC3, 0xC3}
	},
	{
		2, {0x92, 0x63},
		3, {0xE5, 0x9B, 0xA3},
		2, {0xE3, 0x56},
		2, {0xC3, 0xC4}
	},
	{
		2, {0x92, 0x64},
		3, {0xE5, 0xA3, 0x87},
		2, {0xC7, 0x58},
		2, {0xC3, 0xC5}
	},
	{
		2, {0x92, 0x65},
		3, {0xE5, 0xBC, 0xBE},
		2, {0x3E, 0x5F},
		2, {0xC3, 0xC6}
	},
	{
		2, {0x92, 0x66},
		3, {0xE6, 0x96, 0xAD},
		2, {0xAD, 0x65},
		2, {0xC3, 0xC7}
	},
	{
		2, {0x92, 0x67},
		3, {0xE6, 0x9A, 0x96},
		2, {0x96, 0x66},
		2, {0xC3, 0xC8}
	},
	{
		2, {0x92, 0x68},
		3, {0xE6, 0xAA, 0x80},
		2, {0x80, 0x6A},
		2, {0xC3, 0xC9}
	},
	{
		2, {0x92, 0x69},
		3, {0xE6, 0xAE, 0xB5},
		2, {0xB5, 0x6B},
		2, {0xC3, 0xCA}
	},
	{
		2, {0x92, 0x6A},
		3, {0xE7, 0x94, 0xB7},
		2, {0x37, 0x75},
		2, {0xC3, 0xCB}
	},
	{
		2, {0x92, 0x6B},
		3, {0xE8, 0xAB, 0x87},
		2, {0xC7, 0x8A},
		2, {0xC3, 0xCC}
	},
	{
		2, {0x92, 0x6C},
		3, {0xE5, 0x80, 0xA4},
		2, {0x24, 0x50},
		2, {0xC3, 0xCD}
	},
	{
		2, {0x92, 0x6D},
		3, {0xE7, 0x9F, 0xA5},
		2, {0xE5, 0x77},
		2, {0xC3, 0xCE}
	},
	{
		2, {0x92, 0x6E},
		3, {0xE5, 0x9C, 0xB0},
		2, {0x30, 0x57},
		2, {0xC3, 0xCF}
	},
	{
		2, {0x92, 0x6F},
		3, {0xE5, 0xBC, 0x9B},
		2, {0x1B, 0x5F},
		2, {0xC3, 0xD0}
	},
	{
		2, {0x92, 0x70},
		3, {0xE6, 0x81, 0xA5},
		2, {0x65, 0x60},
		2, {0xC3, 0xD1}
	},
	{
		2, {0x92, 0x71},
		3, {0xE6, 0x99, 0xBA},
		2, {0x7A, 0x66},
		2, {0xC3, 0xD2}
	},
	{
		2, {0x92, 0x72},
		3, {0xE6, 0xB1, 0xA0},
		2, {0x60, 0x6C},
		2, {0xC3, 0xD3}
	},
	{
		2, {0x92, 0x73},
		3, {0xE7, 0x97, 0xB4},
		2, {0xF4, 0x75},
		2, {0xC3, 0xD4}
	},
	{
		2, {0x92, 0x74},
		3, {0xE7, 0xA8, 0x9A},
		2, {0x1A, 0x7A},
		2, {0xC3, 0xD5}
	},
	{
		2, {0x92, 0x75},
		3, {0xE7, 0xBD, 0xAE},
		2, {0x6E, 0x7F},
		2, {0xC3, 0xD6}
	},
	{
		2, {0x92, 0x76},
		3, {0xE8, 0x87, 0xB4},
		2, {0xF4, 0x81},
		2, {0xC3, 0xD7}
	},
	{
		2, {0x92, 0x77},
		3, {0xE8, 0x9C, 0x98},
		2, {0x18, 0x87},
		2, {0xC3, 0xD8}
	},
	{
		2, {0x92, 0x78},
		3, {0xE9, 0x81, 0x85},
		2, {0x45, 0x90},
		2, {0xC3, 0xD9}
	},
	{
		2, {0x92, 0x79},
		3, {0xE9, 0xA6, 0xB3},
		2, {0xB3, 0x99},
		2, {0xC3, 0xDA}
	},
	{
		2, {0x92, 0x7A},
		3, {0xE7, 0xAF, 0x89},
		2, {0xC9, 0x7B},
		2, {0xC3, 0xDB}
	},
	{
		2, {0x92, 0x7B},
		3, {0xE7, 0x95, 0x9C},
		2, {0x5C, 0x75},
		2, {0xC3, 0xDC}
	},
	{
		2, {0x92, 0x7C},
		3, {0xE7, 0xAB, 0xB9},
		2, {0xF9, 0x7A},
		2, {0xC3, 0xDD}
	},
	{
		2, {0x92, 0x7D},
		3, {0xE7, 0xAD, 0x91},
		2, {0x51, 0x7B},
		2, {0xC3, 0xDE}
	},
	{
		2, {0x92, 0x7E},
		3, {0xE8, 0x93, 0x84},
		2, {0xC4, 0x84},
		2, {0xC3, 0xDF}
	},
	{
		2, {0x92, 0x80},
		3, {0xE9, 0x80, 0x90},
		2, {0x10, 0x90},
		2, {0xC3, 0xE0}
	},
	{
		2, {0x92, 0x81},
		3, {0xE7, 0xA7, 0xA9},
		2, {0xE9, 0x79},
		2, {0xC3, 0xE1}
	},
	{
		2, {0x92, 0x82},
		3, {0xE7, 0xAA, 0x92},
		2, {0x92, 0x7A},
		2, {0xC3, 0xE2}
	},
	{
		2, {0x92, 0x83},
		3, {0xE8, 0x8C, 0xB6},
		2, {0x36, 0x83},
		2, {0xC3, 0xE3}
	},
	{
		2, {0x92, 0x84},
		3, {0xE5, 0xAB, 0xA1},
		2, {0xE1, 0x5A},
		2, {0xC3, 0xE4}
	},
	{
		2, {0x92, 0x85},
		3, {0xE7, 0x9D, 0x80},
		2, {0x40, 0x77},
		2, {0xC3, 0xE5}
	},
	{
		2, {0x92, 0x86},
		3, {0xE4, 0xB8, 0xAD},
		2, {0x2D, 0x4E},
		2, {0xC3, 0xE6}
	},
	{
		2, {0x92, 0x87},
		3, {0xE4, 0xBB, 0xB2},
		2, {0xF2, 0x4E},
		2, {0xC3, 0xE7}
	},
	{
		2, {0x92, 0x88},
		3, {0xE5, 0xAE, 0x99},
		2, {0x99, 0x5B},
		2, {0xC3, 0xE8}
	},
	{
		2, {0x92, 0x89},
		3, {0xE5, 0xBF, 0xA0},
		2, {0xE0, 0x5F},
		2, {0xC3, 0xE9}
	},
	{
		2, {0x92, 0x8A},
		3, {0xE6, 0x8A, 0xBD},
		2, {0xBD, 0x62},
		2, {0xC3, 0xEA}
	},
	{
		2, {0x92, 0x8B},
		3, {0xE6, 0x98, 0xBC},
		2, {0x3C, 0x66},
		2, {0xC3, 0xEB}
	},
	{
		2, {0x92, 0x8C},
		3, {0xE6, 0x9F, 0xB1},
		2, {0xF1, 0x67},
		2, {0xC3, 0xEC}
	},
	{
		2, {0x92, 0x8D},
		3, {0xE6, 0xB3, 0xA8},
		2, {0xE8, 0x6C},
		2, {0xC3, 0xED}
	},
	{
		2, {0x92, 0x8E},
		3, {0xE8, 0x99, 0xAB},
		2, {0x6B, 0x86},
		2, {0xC3, 0xEE}
	},
	{
		2, {0x92, 0x8F},
		3, {0xE8, 0xA1, 0xB7},
		2, {0x77, 0x88},
		2, {0xC3, 0xEF}
	},
	{
		2, {0x92, 0x90},
		3, {0xE8, 0xA8, 0xBB},
		2, {0x3B, 0x8A},
		2, {0xC3, 0xF0}
	},
	{
		2, {0x92, 0x91},
		3, {0xE9, 0x85, 0x8E},
		2, {0x4E, 0x91},
		2, {0xC3, 0xF1}
	},
	{
		2, {0x92, 0x92},
		3, {0xE9, 0x8B, 0xB3},
		2, {0xF3, 0x92},
		2, {0xC3, 0xF2}
	},
	{
		2, {0x92, 0x93},
		3, {0xE9, 0xA7, 0x90},
		2, {0xD0, 0x99},
		2, {0xC3, 0xF3}
	},
	{
		2, {0x92, 0x94},
		3, {0xE6, 0xA8, 0x97},
		2, {0x17, 0x6A},
		2, {0xC3, 0xF4}
	},
	{
		2, {0x92, 0x95},
		3, {0xE7, 0x80, 0xA6},
		2, {0x26, 0x70},
		2, {0xC3, 0xF5}
	},
	{
		2, {0x92, 0x96},
		3, {0xE7, 0x8C, 0xAA},
		2, {0x2A, 0x73},
		2, {0xC3, 0xF6}
	},
	{
		2, {0x92, 0x97},
		3, {0xE8, 0x8B, 0xA7},
		2, {0xE7, 0x82},
		2, {0xC3, 0xF7}
	},
	{
		2, {0x92, 0x98},
		3, {0xE8, 0x91, 0x97},
		2, {0x57, 0x84},
		2, {0xC3, 0xF8}
	},
	{
		2, {0x92, 0x99},
		3, {0xE8, 0xB2, 0xAF},
		2, {0xAF, 0x8C},
		2, {0xC3, 0xF9}
	},
	{
		2, {0x92, 0x9A},
		3, {0xE4, 0xB8, 0x81},
		2, {0x01, 0x4E},
		2, {0xC3, 0xFA}
	},
	{
		2, {0x92, 0x9B},
		3, {0xE5, 0x85, 0x86},
		2, {0x46, 0x51},
		2, {0xC3, 0xFB}
	},
	{
		2, {0x92, 0x9C},
		3, {0xE5, 0x87, 0x8B},
		2, {0xCB, 0x51},
		2, {0xC3, 0xFC}
	},
	{
		2, {0x92, 0x9D},
		3, {0xE5, 0x96, 0x8B},
		2, {0x8B, 0x55},
		2, {0xC3, 0xFD}
	},
	{
		2, {0x92, 0x9E},
		3, {0xE5, 0xAF, 0xB5},
		2, {0xF5, 0x5B},
		2, {0xC3, 0xFE}
	},
	{
		2, {0x92, 0x9F},
		3, {0xE5, 0xB8, 0x96},
		2, {0x16, 0x5E},
		2, {0xC4, 0xA1}
	},
	{
		2, {0x92, 0xA0},
		3, {0xE5, 0xB8, 0xB3},
		2, {0x33, 0x5E},
		2, {0xC4, 0xA2}
	},
	{
		2, {0x92, 0xA1},
		3, {0xE5, 0xBA, 0x81},
		2, {0x81, 0x5E},
		2, {0xC4, 0xA3}
	},
	{
		2, {0x92, 0xA2},
		3, {0xE5, 0xBC, 0x94},
		2, {0x14, 0x5F},
		2, {0xC4, 0xA4}
	},
	{
		2, {0x92, 0xA3},
		3, {0xE5, 0xBC, 0xB5},
		2, {0x35, 0x5F},
		2, {0xC4, 0xA5}
	},
	{
		2, {0x92, 0xA4},
		3, {0xE5, 0xBD, 0xAB},
		2, {0x6B, 0x5F},
		2, {0xC4, 0xA6}
	},
	{
		2, {0x92, 0xA5},
		3, {0xE5, 0xBE, 0xB4},
		2, {0xB4, 0x5F},
		2, {0xC4, 0xA7}
	},
	{
		2, {0x92, 0xA6},
		3, {0xE6, 0x87, 0xB2},
		2, {0xF2, 0x61},
		2, {0xC4, 0xA8}
	},
	{
		2, {0x92, 0xA7},
		3, {0xE6, 0x8C, 0x91},
		2, {0x11, 0x63},
		2, {0xC4, 0xA9}
	},
	{
		2, {0x92, 0xA8},
		3, {0xE6, 0x9A, 0xA2},
		2, {0xA2, 0x66},
		2, {0xC4, 0xAA}
	},
	{
		2, {0x92, 0xA9},
		3, {0xE6, 0x9C, 0x9D},
		2, {0x1D, 0x67},
		2, {0xC4, 0xAB}
	},
	{
		2, {0x92, 0xAA},
		3, {0xE6, 0xBD, 0xAE},
		2, {0x6E, 0x6F},
		2, {0xC4, 0xAC}
	},
	{
		2, {0x92, 0xAB},
		3, {0xE7, 0x89, 0x92},
		2, {0x52, 0x72},
		2, {0xC4, 0xAD}
	},
	{
		2, {0x92, 0xAC},
		3, {0xE7, 0x94, 0xBA},
		2, {0x3A, 0x75},
		2, {0xC4, 0xAE}
	},
	{
		2, {0x92, 0xAD},
		3, {0xE7, 0x9C, 0xBA},
		2, {0x3A, 0x77},
		2, {0xC4, 0xAF}
	},
	{
		2, {0x92, 0xAE},
		3, {0xE8, 0x81, 0xB4},
		2, {0x74, 0x80},
		2, {0xC4, 0xB0}
	},
	{
		2, {0x92, 0xAF},
		3, {0xE8, 0x84, 0xB9},
		2, {0x39, 0x81},
		2, {0xC4, 0xB1}
	},
	{
		2, {0x92, 0xB0},
		3, {0xE8, 0x85, 0xB8},
		2, {0x78, 0x81},
		2, {0xC4, 0xB2}
	},
	{
		2, {0x92, 0xB1},
		3, {0xE8, 0x9D, 0xB6},
		2, {0x76, 0x87},
		2, {0xC4, 0xB3}
	},
	{
		2, {0x92, 0xB2},
		3, {0xE8, 0xAA, 0xBF},
		2, {0xBF, 0x8A},
		2, {0xC4, 0xB4}
	},
	{
		2, {0x92, 0xB3},
		3, {0xE8, 0xAB, 0x9C},
		2, {0xDC, 0x8A},
		2, {0xC4, 0xB5}
	},
	{
		2, {0x92, 0xB4},
		3, {0xE8, 0xB6, 0x85},
		2, {0x85, 0x8D},
		2, {0xC4, 0xB6}
	},
	{
		2, {0x92, 0xB5},
		3, {0xE8, 0xB7, 0xB3},
		2, {0xF3, 0x8D},
		2, {0xC4, 0xB7}
	},
	{
		2, {0x92, 0xB6},
		3, {0xE9, 0x8A, 0x9A},
		2, {0x9A, 0x92},
		2, {0xC4, 0xB8}
	},
	{
		2, {0x92, 0xB7},
		3, {0xE9, 0x95, 0xB7},
		2, {0x77, 0x95},
		2, {0xC4, 0xB9}
	},
	{
		2, {0x92, 0xB8},
		3, {0xE9, 0xA0, 0x82},
		2, {0x02, 0x98},
		2, {0xC4, 0xBA}
	},
	{
		2, {0x92, 0xB9},
		3, {0xE9, 0xB3, 0xA5},
		2, {0xE5, 0x9C},
		2, {0xC4, 0xBB}
	},
	{
		2, {0x92, 0xBA},
		3, {0xE5, 0x8B, 0x85},
		2, {0xC5, 0x52},
		2, {0xC4, 0xBC}
	},
	{
		2, {0x92, 0xBB},
		3, {0xE6, 0x8D, 0x97},
		2, {0x57, 0x63},
		2, {0xC4, 0xBD}
	},
	{
		2, {0x92, 0xBC},
		3, {0xE7, 0x9B, 0xB4},
		2, {0xF4, 0x76},
		2, {0xC4, 0xBE}
	},
	{
		2, {0x92, 0xBD},
		3, {0xE6, 0x9C, 0x95},
		2, {0x15, 0x67},
		2, {0xC4, 0xBF}
	},
	{
		2, {0x92, 0xBE},
		3, {0xE6, 0xB2, 0x88},
		2, {0x88, 0x6C},
		2, {0xC4, 0xC0}
	},
	{
		2, {0x92, 0xBF},
		3, {0xE7, 0x8F, 0x8D},
		2, {0xCD, 0x73},
		2, {0xC4, 0xC1}
	},
	{
		2, {0x92, 0xC0},
		3, {0xE8, 0xB3, 0x83},
		2, {0xC3, 0x8C},
		2, {0xC4, 0xC2}
	},
	{
		2, {0x92, 0xC1},
		3, {0xE9, 0x8E, 0xAE},
		2, {0xAE, 0x93},
		2, {0xC4, 0xC3}
	},
	{
		2, {0x92, 0xC2},
		3, {0xE9, 0x99, 0xB3},
		2, {0x73, 0x96},
		2, {0xC4, 0xC4}
	},
	{
		2, {0x92, 0xC3},
		3, {0xE6, 0xB4, 0xA5},
		2, {0x25, 0x6D},
		2, {0xC4, 0xC5}
	},
	{
		2, {0x92, 0xC4},
		3, {0xE5, 0xA2, 0x9C},
		2, {0x9C, 0x58},
		2, {0xC4, 0xC6}
	},
	{
		2, {0x92, 0xC5},
		3, {0xE6, 0xA4, 0x8E},
		2, {0x0E, 0x69},
		2, {0xC4, 0xC7}
	},
	{
		2, {0x92, 0xC6},
		3, {0xE6, 0xA7, 0x8C},
		2, {0xCC, 0x69},
		2, {0xC4, 0xC8}
	},
	{
		2, {0x92, 0xC7},
		3, {0xE8, 0xBF, 0xBD},
		2, {0xFD, 0x8F},
		2, {0xC4, 0xC9}
	},
	{
		2, {0x92, 0xC8},
		3, {0xE9, 0x8E, 0x9A},
		2, {0x9A, 0x93},
		2, {0xC4, 0xCA}
	},
	{
		2, {0x92, 0xC9},
		3, {0xE7, 0x97, 0x9B},
		2, {0xDB, 0x75},
		2, {0xC4, 0xCB}
	},
	{
		2, {0x92, 0xCA},
		3, {0xE9, 0x80, 0x9A},
		2, {0x1A, 0x90},
		2, {0xC4, 0xCC}
	},
	{
		2, {0x92, 0xCB},
		3, {0xE5, 0xA1, 0x9A},
		2, {0x5A, 0x58},
		2, {0xC4, 0xCD}
	},
	{
		2, {0x92, 0xCC},
		3, {0xE6, 0xA0, 0x82},
		2, {0x02, 0x68},
		2, {0xC4, 0xCE}
	},
	{
		2, {0x92, 0xCD},
		3, {0xE6, 0x8E, 0xB4},
		2, {0xB4, 0x63},
		2, {0xC4, 0xCF}
	},
	{
		2, {0x92, 0xCE},
		3, {0xE6, 0xA7, 0xBB},
		2, {0xFB, 0x69},
		2, {0xC4, 0xD0}
	},
	{
		2, {0x92, 0xCF},
		3, {0xE4, 0xBD, 0x83},
		2, {0x43, 0x4F},
		2, {0xC4, 0xD1}
	},
	{
		2, {0x92, 0xD0},
		3, {0xE6, 0xBC, 0xAC},
		2, {0x2C, 0x6F},
		2, {0xC4, 0xD2}
	},
	{
		2, {0x92, 0xD1},
		3, {0xE6, 0x9F, 0x98},
		2, {0xD8, 0x67},
		2, {0xC4, 0xD3}
	},
	{
		2, {0x92, 0xD2},
		3, {0xE8, 0xBE, 0xBB},
		2, {0xBB, 0x8F},
		2, {0xC4, 0xD4}
	},
	{
		2, {0x92, 0xD3},
		3, {0xE8, 0x94, 0xA6},
		2, {0x26, 0x85},
		2, {0xC4, 0xD5}
	},
	{
		2, {0x92, 0xD4},
		3, {0xE7, 0xB6, 0xB4},
		2, {0xB4, 0x7D},
		2, {0xC4, 0xD6}
	},
	{
		2, {0x92, 0xD5},
		3, {0xE9, 0x8D, 0x94},
		2, {0x54, 0x93},
		2, {0xC4, 0xD7}
	},
	{
		2, {0x92, 0xD6},
		3, {0xE6, 0xA4, 0xBF},
		2, {0x3F, 0x69},
		2, {0xC4, 0xD8}
	},
	{
		2, {0x92, 0xD7},
		3, {0xE6, 0xBD, 0xB0},
		2, {0x70, 0x6F},
		2, {0xC4, 0xD9}
	},
	{
		2, {0x92, 0xD8},
		3, {0xE5, 0x9D, 0xAA},
		2, {0x6A, 0x57},
		2, {0xC4, 0xDA}
	},
	{
		2, {0x92, 0xD9},
		3, {0xE5, 0xA3, 0xB7},
		2, {0xF7, 0x58},
		2, {0xC4, 0xDB}
	},
	{
		2, {0x92, 0xDA},
		3, {0xE5, 0xAC, 0xAC},
		2, {0x2C, 0x5B},
		2, {0xC4, 0xDC}
	},
	{
		2, {0x92, 0xDB},
		3, {0xE7, 0xB4, 0xAC},
		2, {0x2C, 0x7D},
		2, {0xC4, 0xDD}
	},
	{
		2, {0x92, 0xDC},
		3, {0xE7, 0x88, 0xAA},
		2, {0x2A, 0x72},
		2, {0xC4, 0xDE}
	},
	{
		2, {0x92, 0xDD},
		3, {0xE5, 0x90, 0x8A},
		2, {0x0A, 0x54},
		2, {0xC4, 0xDF}
	},
	{
		2, {0x92, 0xDE},
		3, {0xE9, 0x87, 0xA3},
		2, {0xE3, 0x91},
		2, {0xC4, 0xE0}
	},
	{
		2, {0x92, 0xDF},
		3, {0xE9, 0xB6, 0xB4},
		2, {0xB4, 0x9D},
		2, {0xC4, 0xE1}
	},
	{
		2, {0x92, 0xE0},
		3, {0xE4, 0xBA, 0xAD},
		2, {0xAD, 0x4E},
		2, {0xC4, 0xE2}
	},
	{
		2, {0x92, 0xE1},
		3, {0xE4, 0xBD, 0x8E},
		2, {0x4E, 0x4F},
		2, {0xC4, 0xE3}
	},
	{
		2, {0x92, 0xE2},
		3, {0xE5, 0x81, 0x9C},
		2, {0x5C, 0x50},
		2, {0xC4, 0xE4}
	},
	{
		2, {0x92, 0xE3},
		3, {0xE5, 0x81, 0xB5},
		2, {0x75, 0x50},
		2, {0xC4, 0xE5}
	},
	{
		2, {0x92, 0xE4},
		3, {0xE5, 0x89, 0x83},
		2, {0x43, 0x52},
		2, {0xC4, 0xE6}
	},
	{
		2, {0x92, 0xE5},
		3, {0xE8, 0xB2, 0x9E},
		2, {0x9E, 0x8C},
		2, {0xC4, 0xE7}
	},
	{
		2, {0x92, 0xE6},
		3, {0xE5, 0x91, 0x88},
		2, {0x48, 0x54},
		2, {0xC4, 0xE8}
	},
	{
		2, {0x92, 0xE7},
		3, {0xE5, 0xA0, 0xA4},
		2, {0x24, 0x58},
		2, {0xC4, 0xE9}
	},
	{
		2, {0x92, 0xE8},
		3, {0xE5, 0xAE, 0x9A},
		2, {0x9A, 0x5B},
		2, {0xC4, 0xEA}
	},
	{
		2, {0x92, 0xE9},
		3, {0xE5, 0xB8, 0x9D},
		2, {0x1D, 0x5E},
		2, {0xC4, 0xEB}
	},
	{
		2, {0x92, 0xEA},
		3, {0xE5, 0xBA, 0x95},
		2, {0x95, 0x5E},
		2, {0xC4, 0xEC}
	},
	{
		2, {0x92, 0xEB},
		3, {0xE5, 0xBA, 0xAD},
		2, {0xAD, 0x5E},
		2, {0xC4, 0xED}
	},
	{
		2, {0x92, 0xEC},
		3, {0xE5, 0xBB, 0xB7},
		2, {0xF7, 0x5E},
		2, {0xC4, 0xEE}
	},
	{
		2, {0x92, 0xED},
		3, {0xE5, 0xBC, 0x9F},
		2, {0x1F, 0x5F},
		2, {0xC4, 0xEF}
	},
	{
		2, {0x92, 0xEE},
		3, {0xE6, 0x82, 0x8C},
		2, {0x8C, 0x60},
		2, {0xC4, 0xF0}
	},
	{
		2, {0x92, 0xEF},
		3, {0xE6, 0x8A, 0xB5},
		2, {0xB5, 0x62},
		2, {0xC4, 0xF1}
	},
	{
		2, {0x92, 0xF0},
		3, {0xE6, 0x8C, 0xBA},
		2, {0x3A, 0x63},
		2, {0xC4, 0xF2}
	},
	{
		2, {0x92, 0xF1},
		3, {0xE6, 0x8F, 0x90},
		2, {0xD0, 0x63},
		2, {0xC4, 0xF3}
	},
	{
		2, {0x92, 0xF2},
		3, {0xE6, 0xA2, 0xAF},
		2, {0xAF, 0x68},
		2, {0xC4, 0xF4}
	},
	{
		2, {0x92, 0xF3},
		3, {0xE6, 0xB1, 0x80},
		2, {0x40, 0x6C},
		2, {0xC4, 0xF5}
	},
	{
		2, {0x92, 0xF4},
		3, {0xE7, 0xA2, 0x87},
		2, {0x87, 0x78},
		2, {0xC4, 0xF6}
	},
	{
		2, {0x92, 0xF5},
		3, {0xE7, 0xA6, 0x8E},
		2, {0x8E, 0x79},
		2, {0xC4, 0xF7}
	},
	{
		2, {0x92, 0xF6},
		3, {0xE7, 0xA8, 0x8B},
		2, {0x0B, 0x7A},
		2, {0xC4, 0xF8}
	},
	{
		2, {0x92, 0xF7},
		3, {0xE7, 0xB7, 0xA0},
		2, {0xE0, 0x7D},
		2, {0xC4, 0xF9}
	},
	{
		2, {0x92, 0xF8},
		3, {0xE8, 0x89, 0x87},
		2, {0x47, 0x82},
		2, {0xC4, 0xFA}
	},
	{
		2, {0x92, 0xF9},
		3, {0xE8, 0xA8, 0x82},
		2, {0x02, 0x8A},
		2, {0xC4, 0xFB}
	},
	{
		2, {0x92, 0xFA},
		3, {0xE8, 0xAB, 0xA6},
		2, {0xE6, 0x8A},
		2, {0xC4, 0xFC}
	},
	{
		2, {0x92, 0xFB},
		3, {0xE8, 0xB9, 0x84},
		2, {0x44, 0x8E},
		2, {0xC4, 0xFD}
	},
	{
		2, {0x92, 0xFC},
		3, {0xE9, 0x80, 0x93},
		2, {0x13, 0x90},
		2, {0xC4, 0xFE}
	},
	{
		2, {0x93, 0x40},
		3, {0xE9, 0x82, 0xB8},
		2, {0xB8, 0x90},
		2, {0xC5, 0xA1}
	},
	{
		2, {0x93, 0x41},
		3, {0xE9, 0x84, 0xAD},
		2, {0x2D, 0x91},
		2, {0xC5, 0xA2}
	},
	{
		2, {0x93, 0x42},
		3, {0xE9, 0x87, 0x98},
		2, {0xD8, 0x91},
		2, {0xC5, 0xA3}
	},
	{
		2, {0x93, 0x43},
		3, {0xE9, 0xBC, 0x8E},
		2, {0x0E, 0x9F},
		2, {0xC5, 0xA4}
	},
	{
		2, {0x93, 0x44},
		3, {0xE6, 0xB3, 0xA5},
		2, {0xE5, 0x6C},
		2, {0xC5, 0xA5}
	},
	{
		2, {0x93, 0x45},
		3, {0xE6, 0x91, 0x98},
		2, {0x58, 0x64},
		2, {0xC5, 0xA6}
	},
	{
		2, {0x93, 0x46},
		3, {0xE6, 0x93, 0xA2},
		2, {0xE2, 0x64},
		2, {0xC5, 0xA7}
	},
	{
		2, {0x93, 0x47},
		3, {0xE6, 0x95, 0xB5},
		2, {0x75, 0x65},
		2, {0xC5, 0xA8}
	},
	{
		2, {0x93, 0x48},
		3, {0xE6, 0xBB, 0xB4},
		2, {0xF4, 0x6E},
		2, {0xC5, 0xA9}
	},
	{
		2, {0x93, 0x49},
		3, {0xE7, 0x9A, 0x84},
		2, {0x84, 0x76},
		2, {0xC5, 0xAA}
	},
	{
		2, {0x93, 0x4A},
		3, {0xE7, 0xAC, 0x9B},
		2, {0x1B, 0x7B},
		2, {0xC5, 0xAB}
	},
	{
		2, {0x93, 0x4B},
		3, {0xE9, 0x81, 0xA9},
		2, {0x69, 0x90},
		2, {0xC5, 0xAC}
	},
	{
		2, {0x93, 0x4C},
		3, {0xE9, 0x8F, 0x91},
		2, {0xD1, 0x93},
		2, {0xC5, 0xAD}
	},
	{
		2, {0x93, 0x4D},
		3, {0xE6, 0xBA, 0xBA},
		2, {0xBA, 0x6E},
		2, {0xC5, 0xAE}
	},
	{
		2, {0x93, 0x4E},
		3, {0xE5, 0x93, 0xB2},
		2, {0xF2, 0x54},
		2, {0xC5, 0xAF}
	},
	{
		2, {0x93, 0x4F},
		3, {0xE5, 0xBE, 0xB9},
		2, {0xB9, 0x5F},
		2, {0xC5, 0xB0}
	},
	{
		2, {0x93, 0x50},
		3, {0xE6, 0x92, 0xA4},
		2, {0xA4, 0x64},
		2, {0xC5, 0xB1}
	},
	{
		2, {0x93, 0x51},
		3, {0xE8, 0xBD, 0x8D},
		2, {0x4D, 0x8F},
		2, {0xC5, 0xB2}
	},
	{
		2, {0x93, 0x52},
		3, {0xE8, 0xBF, 0xAD},
		2, {0xED, 0x8F},
		2, {0xC5, 0xB3}
	},
	{
		2, {0x93, 0x53},
		3, {0xE9, 0x89, 0x84},
		2, {0x44, 0x92},
		2, {0xC5, 0xB4}
	},
	{
		2, {0x93, 0x54},
		3, {0xE5, 0x85, 0xB8},
		2, {0x78, 0x51},
		2, {0xC5, 0xB5}
	},
	{
		2, {0x93, 0x55},
		3, {0xE5, 0xA1, 0xAB},
		2, {0x6B, 0x58},
		2, {0xC5, 0xB6}
	},
	{
		2, {0x93, 0x56},
		3, {0xE5, 0xA4, 0xA9},
		2, {0x29, 0x59},
		2, {0xC5, 0xB7}
	},
	{
		2, {0x93, 0x57},
		3, {0xE5, 0xB1, 0x95},
		2, {0x55, 0x5C},
		2, {0xC5, 0xB8}
	},
	{
		2, {0x93, 0x58},
		3, {0xE5, 0xBA, 0x97},
		2, {0x97, 0x5E},
		2, {0xC5, 0xB9}
	},
	{
		2, {0x93, 0x59},
		3, {0xE6, 0xB7, 0xBB},
		2, {0xFB, 0x6D},
		2, {0xC5, 0xBA}
	},
	{
		2, {0x93, 0x5A},
		3, {0xE7, 0xBA, 0x8F},
		2, {0x8F, 0x7E},
		2, {0xC5, 0xBB}
	},
	{
		2, {0x93, 0x5B},
		3, {0xE7, 0x94, 0x9C},
		2, {0x1C, 0x75},
		2, {0xC5, 0xBC}
	},
	{
		2, {0x93, 0x5C},
		3, {0xE8, 0xB2, 0xBC},
		2, {0xBC, 0x8C},
		2, {0xC5, 0xBD}
	},
	{
		2, {0x93, 0x5D},
		3, {0xE8, 0xBB, 0xA2},
		2, {0xE2, 0x8E},
		2, {0xC5, 0xBE}
	},
	{
		2, {0x93, 0x5E},
		3, {0xE9, 0xA1, 0x9B},
		2, {0x5B, 0x98},
		2, {0xC5, 0xBF}
	},
	{
		2, {0x93, 0x5F},
		3, {0xE7, 0x82, 0xB9},
		2, {0xB9, 0x70},
		2, {0xC5, 0xC0}
	},
	{
		2, {0x93, 0x60},
		3, {0xE4, 0xBC, 0x9D},
		2, {0x1D, 0x4F},
		2, {0xC5, 0xC1}
	},
	{
		2, {0x93, 0x61},
		3, {0xE6, 0xAE, 0xBF},
		2, {0xBF, 0x6B},
		2, {0xC5, 0xC2}
	},
	{
		2, {0x93, 0x62},
		3, {0xE6, 0xBE, 0xB1},
		2, {0xB1, 0x6F},
		2, {0xC5, 0xC3}
	},
	{
		2, {0x93, 0x63},
		3, {0xE7, 0x94, 0xB0},
		2, {0x30, 0x75},
		2, {0xC5, 0xC4}
	},
	{
		2, {0x93, 0x64},
		3, {0xE9, 0x9B, 0xBB},
		2, {0xFB, 0x96},
		2, {0xC5, 0xC5}
	},
	{
		2, {0x93, 0x65},
		3, {0xE5, 0x85, 0x8E},
		2, {0x4E, 0x51},
		2, {0xC5, 0xC6}
	},
	{
		2, {0x93, 0x66},
		3, {0xE5, 0x90, 0x90},
		2, {0x10, 0x54},
		2, {0xC5, 0xC7}
	},
	{
		2, {0x93, 0x67},
		3, {0xE5, 0xA0, 0xB5},
		2, {0x35, 0x58},
		2, {0xC5, 0xC8}
	},
	{
		2, {0x93, 0x68},
		3, {0xE5, 0xA1, 0x97},
		2, {0x57, 0x58},
		2, {0xC5, 0xC9}
	},
	{
		2, {0x93, 0x69},
		3, {0xE5, 0xA6, 0xAC},
		2, {0xAC, 0x59},
		2, {0xC5, 0xCA}
	},
	{
		2, {0x93, 0x6A},
		3, {0xE5, 0xB1, 0xA0},
		2, {0x60, 0x5C},
		2, {0xC5, 0xCB}
	},
	{
		2, {0x93, 0x6B},
		3, {0xE5, 0xBE, 0x92},
		2, {0x92, 0x5F},
		2, {0xC5, 0xCC}
	},
	{
		2, {0x93, 0x6C},
		3, {0xE6, 0x96, 0x97},
		2, {0x97, 0x65},
		2, {0xC5, 0xCD}
	},
	{
		2, {0x93, 0x6D},
		3, {0xE6, 0x9D, 0x9C},
		2, {0x5C, 0x67},
		2, {0xC5, 0xCE}
	},
	{
		2, {0x93, 0x6E},
		3, {0xE6, 0xB8, 0xA1},
		2, {0x21, 0x6E},
		2, {0xC5, 0xCF}
	},
	{
		2, {0x93, 0x6F},
		3, {0xE7, 0x99, 0xBB},
		2, {0x7B, 0x76},
		2, {0xC5, 0xD0}
	},
	{
		2, {0x93, 0x70},
		3, {0xE8, 0x8F, 0x9F},
		2, {0xDF, 0x83},
		2, {0xC5, 0xD1}
	},
	{
		2, {0x93, 0x71},
		3, {0xE8, 0xB3, 0xAD},
		2, {0xED, 0x8C},
		2, {0xC5, 0xD2}
	},
	{
		2, {0x93, 0x72},
		3, {0xE9, 0x80, 0x94},
		2, {0x14, 0x90},
		2, {0xC5, 0xD3}
	},
	{
		2, {0x93, 0x73},
		3, {0xE9, 0x83, 0xBD},
		2, {0xFD, 0x90},
		2, {0xC5, 0xD4}
	},
	{
		2, {0x93, 0x74},
		3, {0xE9, 0x8D, 0x8D},
		2, {0x4D, 0x93},
		2, {0xC5, 0xD5}
	},
	{
		2, {0x93, 0x75},
		3, {0xE7, 0xA0, 0xA5},
		2, {0x25, 0x78},
		2, {0xC5, 0xD6}
	},
	{
		2, {0x93, 0x76},
		3, {0xE7, 0xA0, 0xBA},
		2, {0x3A, 0x78},
		2, {0xC5, 0xD7}
	},
	{
		2, {0x93, 0x77},
		3, {0xE5, 0x8A, 0xAA},
		2, {0xAA, 0x52},
		2, {0xC5, 0xD8}
	},
	{
		2, {0x93, 0x78},
		3, {0xE5, 0xBA, 0xA6},
		2, {0xA6, 0x5E},
		2, {0xC5, 0xD9}
	},
	{
		2, {0x93, 0x79},
		3, {0xE5, 0x9C, 0x9F},
		2, {0x1F, 0x57},
		2, {0xC5, 0xDA}
	},
	{
		2, {0x93, 0x7A},
		3, {0xE5, 0xA5, 0xB4},
		2, {0x74, 0x59},
		2, {0xC5, 0xDB}
	},
	{
		2, {0x93, 0x7B},
		3, {0xE6, 0x80, 0x92},
		2, {0x12, 0x60},
		2, {0xC5, 0xDC}
	},
	{
		2, {0x93, 0x7C},
		3, {0xE5, 0x80, 0x92},
		2, {0x12, 0x50},
		2, {0xC5, 0xDD}
	},
	{
		2, {0x93, 0x7D},
		3, {0xE5, 0x85, 0x9A},
		2, {0x5A, 0x51},
		2, {0xC5, 0xDE}
	},
	{
		2, {0x93, 0x7E},
		3, {0xE5, 0x86, 0xAC},
		2, {0xAC, 0x51},
		2, {0xC5, 0xDF}
	},
	{
		2, {0x93, 0x80},
		3, {0xE5, 0x87, 0x8D},
		2, {0xCD, 0x51},
		2, {0xC5, 0xE0}
	},
	{
		2, {0x93, 0x81},
		3, {0xE5, 0x88, 0x80},
		2, {0x00, 0x52},
		2, {0xC5, 0xE1}
	},
	{
		2, {0x93, 0x82},
		3, {0xE5, 0x94, 0x90},
		2, {0x10, 0x55},
		2, {0xC5, 0xE2}
	},
	{
		2, {0x93, 0x83},
		3, {0xE5, 0xA1, 0x94},
		2, {0x54, 0x58},
		2, {0xC5, 0xE3}
	},
	{
		2, {0x93, 0x84},
		3, {0xE5, 0xA1, 0x98},
		2, {0x58, 0x58},
		2, {0xC5, 0xE4}
	},
	{
		2, {0x93, 0x85},
		3, {0xE5, 0xA5, 0x97},
		2, {0x57, 0x59},
		2, {0xC5, 0xE5}
	},
	{
		2, {0x93, 0x86},
		3, {0xE5, 0xAE, 0x95},
		2, {0x95, 0x5B},
		2, {0xC5, 0xE6}
	},
	{
		2, {0x93, 0x87},
		3, {0xE5, 0xB3, 0xB6},
		2, {0xF6, 0x5C},
		2, {0xC5, 0xE7}
	},
	{
		2, {0x93, 0x88},
		3, {0xE5, 0xB6, 0x8B},
		2, {0x8B, 0x5D},
		2, {0xC5, 0xE8}
	},
	{
		2, {0x93, 0x89},
		3, {0xE6, 0x82, 0xBC},
		2, {0xBC, 0x60},
		2, {0xC5, 0xE9}
	},
	{
		2, {0x93, 0x8A},
		3, {0xE6, 0x8A, 0x95},
		2, {0x95, 0x62},
		2, {0xC5, 0xEA}
	},
	{
		2, {0x93, 0x8B},
		3, {0xE6, 0x90, 0xAD},
		2, {0x2D, 0x64},
		2, {0xC5, 0xEB}
	},
	{
		2, {0x93, 0x8C},
		3, {0xE6, 0x9D, 0xB1},
		2, {0x71, 0x67},
		2, {0xC5, 0xEC}
	},
	{
		2, {0x93, 0x8D},
		3, {0xE6, 0xA1, 0x83},
		2, {0x43, 0x68},
		2, {0xC5, 0xED}
	},
	{
		2, {0x93, 0x8E},
		3, {0xE6, 0xA2, 0xBC},
		2, {0xBC, 0x68},
		2, {0xC5, 0xEE}
	},
	{
		2, {0x93, 0x8F},
		3, {0xE6, 0xA3, 0x9F},
		2, {0xDF, 0x68},
		2, {0xC5, 0xEF}
	},
	{
		2, {0x93, 0x90},
		3, {0xE7, 0x9B, 0x97},
		2, {0xD7, 0x76},
		2, {0xC5, 0xF0}
	},
	{
		2, {0x93, 0x91},
		3, {0xE6, 0xB7, 0x98},
		2, {0xD8, 0x6D},
		2, {0xC5, 0xF1}
	},
	{
		2, {0x93, 0x92},
		3, {0xE6, 0xB9, 0xAF},
		2, {0x6F, 0x6E},
		2, {0xC5, 0xF2}
	},
	{
		2, {0x93, 0x93},
		3, {0xE6, 0xB6, 0x9B},
		2, {0x9B, 0x6D},
		2, {0xC5, 0xF3}
	},
	{
		2, {0x93, 0x94},
		3, {0xE7, 0x81, 0xAF},
		2, {0x6F, 0x70},
		2, {0xC5, 0xF4}
	},
	{
		2, {0x93, 0x95},
		3, {0xE7, 0x87, 0x88},
		2, {0xC8, 0x71},
		2, {0xC5, 0xF5}
	},
	{
		2, {0x93, 0x96},
		3, {0xE5, 0xBD, 0x93},
		2, {0x53, 0x5F},
		2, {0xC5, 0xF6}
	},
	{
		2, {0x93, 0x97},
		3, {0xE7, 0x97, 0x98},
		2, {0xD8, 0x75},
		2, {0xC5, 0xF7}
	},
	{
		2, {0x93, 0x98},
		3, {0xE7, 0xA5, 0xB7},
		2, {0x77, 0x79},
		2, {0xC5, 0xF8}
	},
	{
		2, {0x93, 0x99},
		3, {0xE7, 0xAD, 0x89},
		2, {0x49, 0x7B},
		2, {0xC5, 0xF9}
	},
	{
		2, {0x93, 0x9A},
		3, {0xE7, 0xAD, 0x94},
		2, {0x54, 0x7B},
		2, {0xC5, 0xFA}
	},
	{
		2, {0x93, 0x9B},
		3, {0xE7, 0xAD, 0x92},
		2, {0x52, 0x7B},
		2, {0xC5, 0xFB}
	},
	{
		2, {0x93, 0x9C},
		3, {0xE7, 0xB3, 0x96},
		2, {0xD6, 0x7C},
		2, {0xC5, 0xFC}
	},
	{
		2, {0x93, 0x9D},
		3, {0xE7, 0xB5, 0xB1},
		2, {0x71, 0x7D},
		2, {0xC5, 0xFD}
	},
	{
		2, {0x93, 0x9E},
		3, {0xE5, 0x88, 0xB0},
		2, {0x30, 0x52},
		2, {0xC5, 0xFE}
	},
	{
		2, {0x93, 0x9F},
		3, {0xE8, 0x91, 0xA3},
		2, {0x63, 0x84},
		2, {0xC6, 0xA1}
	},
	{
		2, {0x93, 0xA0},
		3, {0xE8, 0x95, 0xA9},
		2, {0x69, 0x85},
		2, {0xC6, 0xA2}
	},
	{
		2, {0x93, 0xA1},
		3, {0xE8, 0x97, 0xA4},
		2, {0xE4, 0x85},
		2, {0xC6, 0xA3}
	},
	{
		2, {0x93, 0xA2},
		3, {0xE8, 0xA8, 0x8E},
		2, {0x0E, 0x8A},
		2, {0xC6, 0xA4}
	},
	{
		2, {0x93, 0xA3},
		3, {0xE8, 0xAC, 0x84},
		2, {0x04, 0x8B},
		2, {0xC6, 0xA5}
	},
	{
		2, {0x93, 0xA4},
		3, {0xE8, 0xB1, 0x86},
		2, {0x46, 0x8C},
		2, {0xC6, 0xA6}
	},
	{
		2, {0x93, 0xA5},
		3, {0xE8, 0xB8, 0x8F},
		2, {0x0F, 0x8E},
		2, {0xC6, 0xA7}
	},
	{
		2, {0x93, 0xA6},
		3, {0xE9, 0x80, 0x83},
		2, {0x03, 0x90},
		2, {0xC6, 0xA8}
	},
	{
		2, {0x93, 0xA7},
		3, {0xE9, 0x80, 0x8F},
		2, {0x0F, 0x90},
		2, {0xC6, 0xA9}
	},
	{
		2, {0x93, 0xA8},
		3, {0xE9, 0x90, 0x99},
		2, {0x19, 0x94},
		2, {0xC6, 0xAA}
	},
	{
		2, {0x93, 0xA9},
		3, {0xE9, 0x99, 0xB6},
		2, {0x76, 0x96},
		2, {0xC6, 0xAB}
	},
	{
		2, {0x93, 0xAA},
		3, {0xE9, 0xA0, 0xAD},
		2, {0x2D, 0x98},
		2, {0xC6, 0xAC}
	},
	{
		2, {0x93, 0xAB},
		3, {0xE9, 0xA8, 0xB0},
		2, {0x30, 0x9A},
		2, {0xC6, 0xAD}
	},
	{
		2, {0x93, 0xAC},
		3, {0xE9, 0x97, 0x98},
		2, {0xD8, 0x95},
		2, {0xC6, 0xAE}
	},
	{
		2, {0x93, 0xAD},
		3, {0xE5, 0x83, 0x8D},
		2, {0xCD, 0x50},
		2, {0xC6, 0xAF}
	},
	{
		2, {0x93, 0xAE},
		3, {0xE5, 0x8B, 0x95},
		2, {0xD5, 0x52},
		2, {0xC6, 0xB0}
	},
	{
		2, {0x93, 0xAF},
		3, {0xE5, 0x90, 0x8C},
		2, {0x0C, 0x54},
		2, {0xC6, 0xB1}
	},
	{
		2, {0x93, 0xB0},
		3, {0xE5, 0xA0, 0x82},
		2, {0x02, 0x58},
		2, {0xC6, 0xB2}
	},
	{
		2, {0x93, 0xB1},
		3, {0xE5, 0xB0, 0x8E},
		2, {0x0E, 0x5C},
		2, {0xC6, 0xB3}
	},
	{
		2, {0x93, 0xB2},
		3, {0xE6, 0x86, 0xA7},
		2, {0xA7, 0x61},
		2, {0xC6, 0xB4}
	},
	{
		2, {0x93, 0xB3},
		3, {0xE6, 0x92, 0x9E},
		2, {0x9E, 0x64},
		2, {0xC6, 0xB5}
	},
	{
		2, {0x93, 0xB4},
		3, {0xE6, 0xB4, 0x9E},
		2, {0x1E, 0x6D},
		2, {0xC6, 0xB6}
	},
	{
		2, {0x93, 0xB5},
		3, {0xE7, 0x9E, 0xB3},
		2, {0xB3, 0x77},
		2, {0xC6, 0xB7}
	},
	{
		2, {0x93, 0xB6},
		3, {0xE7, 0xAB, 0xA5},
		2, {0xE5, 0x7A},
		2, {0xC6, 0xB8}
	},
	{
		2, {0x93, 0xB7},
		3, {0xE8, 0x83, 0xB4},
		2, {0xF4, 0x80},
		2, {0xC6, 0xB9}
	},
	{
		2, {0x93, 0xB8},
		3, {0xE8, 0x90, 0x84},
		2, {0x04, 0x84},
		2, {0xC6, 0xBA}
	},
	{
		2, {0x93, 0xB9},
		3, {0xE9, 0x81, 0x93},
		2, {0x53, 0x90},
		2, {0xC6, 0xBB}
	},
	{
		2, {0x93, 0xBA},
		3, {0xE9, 0x8A, 0x85},
		2, {0x85, 0x92},
		2, {0xC6, 0xBC}
	},
	{
		2, {0x93, 0xBB},
		3, {0xE5, 0xB3, 0xA0},
		2, {0xE0, 0x5C},
		2, {0xC6, 0xBD}
	},
	{
		2, {0x93, 0xBC},
		3, {0xE9, 0xB4, 0x87},
		2, {0x07, 0x9D},
		2, {0xC6, 0xBE}
	},
	{
		2, {0x93, 0xBD},
		3, {0xE5, 0x8C, 0xBF},
		2, {0x3F, 0x53},
		2, {0xC6, 0xBF}
	},
	{
		2, {0x93, 0xBE},
		3, {0xE5, 0xBE, 0x97},
		2, {0x97, 0x5F},
		2, {0xC6, 0xC0}
	},
	{
		2, {0x93, 0xBF},
		3, {0xE5, 0xBE, 0xB3},
		2, {0xB3, 0x5F},
		2, {0xC6, 0xC1}
	},
	{
		2, {0x93, 0xC0},
		3, {0xE6, 0xB6, 0x9C},
		2, {0x9C, 0x6D},
		2, {0xC6, 0xC2}
	},
	{
		2, {0x93, 0xC1},
		3, {0xE7, 0x89, 0xB9},
		2, {0x79, 0x72},
		2, {0xC6, 0xC3}
	},
	{
		2, {0x93, 0xC2},
		3, {0xE7, 0x9D, 0xA3},
		2, {0x63, 0x77},
		2, {0xC6, 0xC4}
	},
	{
		2, {0x93, 0xC3},
		3, {0xE7, 0xA6, 0xBF},
		2, {0xBF, 0x79},
		2, {0xC6, 0xC5}
	},
	{
		2, {0x93, 0xC4},
		3, {0xE7, 0xAF, 0xA4},
		2, {0xE4, 0x7B},
		2, {0xC6, 0xC6}
	},
	{
		2, {0x93, 0xC5},
		3, {0xE6, 0xAF, 0x92},
		2, {0xD2, 0x6B},
		2, {0xC6, 0xC7}
	},
	{
		2, {0x93, 0xC6},
		3, {0xE7, 0x8B, 0xAC},
		2, {0xEC, 0x72},
		2, {0xC6, 0xC8}
	},
	{
		2, {0x93, 0xC7},
		3, {0xE8, 0xAA, 0xAD},
		2, {0xAD, 0x8A},
		2, {0xC6, 0xC9}
	},
	{
		2, {0x93, 0xC8},
		3, {0xE6, 0xA0, 0x83},
		2, {0x03, 0x68},
		2, {0xC6, 0xCA}
	},
	{
		2, {0x93, 0xC9},
		3, {0xE6, 0xA9, 0xA1},
		2, {0x61, 0x6A},
		2, {0xC6, 0xCB}
	},
	{
		2, {0x93, 0xCA},
		3, {0xE5, 0x87, 0xB8},
		2, {0xF8, 0x51},
		2, {0xC6, 0xCC}
	},
	{
		2, {0x93, 0xCB},
		3, {0xE7, 0xAA, 0x81},
		2, {0x81, 0x7A},
		2, {0xC6, 0xCD}
	},
	{
		2, {0x93, 0xCC},
		3, {0xE6, 0xA4, 0xB4},
		2, {0x34, 0x69},
		2, {0xC6, 0xCE}
	},
	{
		2, {0x93, 0xCD},
		3, {0xE5, 0xB1, 0x8A},
		2, {0x4A, 0x5C},
		2, {0xC6, 0xCF}
	},
	{
		2, {0x93, 0xCE},
		3, {0xE9, 0xB3, 0xB6},
		2, {0xF6, 0x9C},
		2, {0xC6, 0xD0}
	},
	{
		2, {0x93, 0xCF},
		3, {0xE8, 0x8B, 0xAB},
		2, {0xEB, 0x82},
		2, {0xC6, 0xD1}
	},
	{
		2, {0x93, 0xD0},
		3, {0xE5, 0xAF, 0x85},
		2, {0xC5, 0x5B},
		2, {0xC6, 0xD2}
	},
	{
		2, {0x93, 0xD1},
		3, {0xE9, 0x85, 0x89},
		2, {0x49, 0x91},
		2, {0xC6, 0xD3}
	},
	{
		2, {0x93, 0xD2},
		3, {0xE7, 0x80, 0x9E},
		2, {0x1E, 0x70},
		2, {0xC6, 0xD4}
	},
	{
		2, {0x93, 0xD3},
		3, {0xE5, 0x99, 0xB8},
		2, {0x78, 0x56},
		2, {0xC6, 0xD5}
	},
	{
		2, {0x93, 0xD4},
		3, {0xE5, 0xB1, 0xAF},
		2, {0x6F, 0x5C},
		2, {0xC6, 0xD6}
	},
	{
		2, {0x93, 0xD5},
		3, {0xE6, 0x83, 0x87},
		2, {0xC7, 0x60},
		2, {0xC6, 0xD7}
	},
	{
		2, {0x93, 0xD6},
		3, {0xE6, 0x95, 0xA6},
		2, {0x66, 0x65},
		2, {0xC6, 0xD8}
	},
	{
		2, {0x93, 0xD7},
		3, {0xE6, 0xB2, 0x8C},
		2, {0x8C, 0x6C},
		2, {0xC6, 0xD9}
	},
	{
		2, {0x93, 0xD8},
		3, {0xE8, 0xB1, 0x9A},
		2, {0x5A, 0x8C},
		2, {0xC6, 0xDA}
	},
	{
		2, {0x93, 0xD9},
		3, {0xE9, 0x81, 0x81},
		2, {0x41, 0x90},
		2, {0xC6, 0xDB}
	},
	{
		2, {0x93, 0xDA},
		3, {0xE9, 0xA0, 0x93},
		2, {0x13, 0x98},
		2, {0xC6, 0xDC}
	},
	{
		2, {0x93, 0xDB},
		3, {0xE5, 0x91, 0x91},
		2, {0x51, 0x54},
		2, {0xC6, 0xDD}
	},
	{
		2, {0x93, 0xDC},
		3, {0xE6, 0x9B, 0x87},
		2, {0xC7, 0x66},
		2, {0xC6, 0xDE}
	},
	{
		2, {0x93, 0xDD},
		3, {0xE9, 0x88, 0x8D},
		2, {0x0D, 0x92},
		2, {0xC6, 0xDF}
	},
	{
		2, {0x93, 0xDE},
		3, {0xE5, 0xA5, 0x88},
		2, {0x48, 0x59},
		2, {0xC6, 0xE0}
	},
	{
		2, {0x93, 0xDF},
		3, {0xE9, 0x82, 0xA3},
		2, {0xA3, 0x90},
		2, {0xC6, 0xE1}
	},
	{
		2, {0x93, 0xE0},
		3, {0xE5, 0x86, 0x85},
		2, {0x85, 0x51},
		2, {0xC6, 0xE2}
	},
	{
		2, {0x93, 0xE1},
		3, {0xE4, 0xB9, 0x8D},
		2, {0x4D, 0x4E},
		2, {0xC6, 0xE3}
	},
	{
		2, {0x93, 0xE2},
		3, {0xE5, 0x87, 0xAA},
		2, {0xEA, 0x51},
		2, {0xC6, 0xE4}
	},
	{
		2, {0x93, 0xE3},
		3, {0xE8, 0x96, 0x99},
		2, {0x99, 0x85},
		2, {0xC6, 0xE5}
	},
	{
		2, {0x93, 0xE4},
		3, {0xE8, 0xAC, 0x8E},
		2, {0x0E, 0x8B},
		2, {0xC6, 0xE6}
	},
	{
		2, {0x93, 0xE5},
		3, {0xE7, 0x81, 0x98},
		2, {0x58, 0x70},
		2, {0xC6, 0xE7}
	},
	{
		2, {0x93, 0xE6},
		3, {0xE6, 0x8D, 0xBA},
		2, {0x7A, 0x63},
		2, {0xC6, 0xE8}
	},
	{
		2, {0x93, 0xE7},
		3, {0xE9, 0x8D, 0x8B},
		2, {0x4B, 0x93},
		2, {0xC6, 0xE9}
	},
	{
		2, {0x93, 0xE8},
		3, {0xE6, 0xA5, 0xA2},
		2, {0x62, 0x69},
		2, {0xC6, 0xEA}
	},
	{
		2, {0x93, 0xE9},
		3, {0xE9, 0xA6, 0xB4},
		2, {0xB4, 0x99},
		2, {0xC6, 0xEB}
	},
	{
		2, {0x93, 0xEA},
		3, {0xE7, 0xB8, 0x84},
		2, {0x04, 0x7E},
		2, {0xC6, 0xEC}
	},
	{
		2, {0x93, 0xEB},
		3, {0xE7, 0x95, 0xB7},
		2, {0x77, 0x75},
		2, {0xC6, 0xED}
	},
	{
		2, {0x93, 0xEC},
		3, {0xE5, 0x8D, 0x97},
		2, {0x57, 0x53},
		2, {0xC6, 0xEE}
	},
	{
		2, {0x93, 0xED},
		3, {0xE6, 0xA5, 0xA0},
		2, {0x60, 0x69},
		2, {0xC6, 0xEF}
	},
	{
		2, {0x93, 0xEE},
		3, {0xE8, 0xBB, 0x9F},
		2, {0xDF, 0x8E},
		2, {0xC6, 0xF0}
	},
	{
		2, {0x93, 0xEF},
		3, {0xE9, 0x9B, 0xA3},
		2, {0xE3, 0x96},
		2, {0xC6, 0xF1}
	},
	{
		2, {0x93, 0xF0},
		3, {0xE6, 0xB1, 0x9D},
		2, {0x5D, 0x6C},
		2, {0xC6, 0xF2}
	},
	{
		2, {0x93, 0xF1},
		3, {0xE4, 0xBA, 0x8C},
		2, {0x8C, 0x4E},
		2, {0xC6, 0xF3}
	},
	{
		2, {0x93, 0xF2},
		3, {0xE5, 0xB0, 0xBC},
		2, {0x3C, 0x5C},
		2, {0xC6, 0xF4}
	},
	{
		2, {0x93, 0xF3},
		3, {0xE5, 0xBC, 0x90},
		2, {0x10, 0x5F},
		2, {0xC6, 0xF5}
	},
	{
		2, {0x93, 0xF4},
		3, {0xE8, 0xBF, 0xA9},
		2, {0xE9, 0x8F},
		2, {0xC6, 0xF6}
	},
	{
		2, {0x93, 0xF5},
		3, {0xE5, 0x8C, 0x82},
		2, {0x02, 0x53},
		2, {0xC6, 0xF7}
	},
	{
		2, {0x93, 0xF6},
		3, {0xE8, 0xB3, 0x91},
		2, {0xD1, 0x8C},
		2, {0xC6, 0xF8}
	},
	{
		2, {0x93, 0xF7},
		3, {0xE8, 0x82, 0x89},
		2, {0x89, 0x80},
		2, {0xC6, 0xF9}
	},
	{
		2, {0x93, 0xF8},
		3, {0xE8, 0x99, 0xB9},
		2, {0x79, 0x86},
		2, {0xC6, 0xFA}
	},
	{
		2, {0x93, 0xF9},
		3, {0xE5, 0xBB, 0xBF},
		2, {0xFF, 0x5E},
		2, {0xC6, 0xFB}
	},
	{
		2, {0x93, 0xFA},
		3, {0xE6, 0x97, 0xA5},
		2, {0xE5, 0x65},
		2, {0xC6, 0xFC}
	},
	{
		2, {0x93, 0xFB},
		3, {0xE4, 0xB9, 0xB3},
		2, {0x73, 0x4E},
		2, {0xC6, 0xFD}
	},
	{
		2, {0x93, 0xFC},
		3, {0xE5, 0x85, 0xA5},
		2, {0x65, 0x51},
		2, {0xC6, 0xFE}
	},
	{
		2, {0x94, 0x40},
		3, {0xE5, 0xA6, 0x82},
		2, {0x82, 0x59},
		2, {0xC7, 0xA1}
	},
	{
		2, {0x94, 0x41},
		3, {0xE5, 0xB0, 0xBF},
		2, {0x3F, 0x5C},
		2, {0xC7, 0xA2}
	},
	{
		2, {0x94, 0x42},
		3, {0xE9, 0x9F, 0xAE},
		2, {0xEE, 0x97},
		2, {0xC7, 0xA3}
	},
	{
		2, {0x94, 0x43},
		3, {0xE4, 0xBB, 0xBB},
		2, {0xFB, 0x4E},
		2, {0xC7, 0xA4}
	},
	{
		2, {0x94, 0x44},
		3, {0xE5, 0xA6, 0x8A},
		2, {0x8A, 0x59},
		2, {0xC7, 0xA5}
	},
	{
		2, {0x94, 0x45},
		3, {0xE5, 0xBF, 0x8D},
		2, {0xCD, 0x5F},
		2, {0xC7, 0xA6}
	},
	{
		2, {0x94, 0x46},
		3, {0xE8, 0xAA, 0x8D},
		2, {0x8D, 0x8A},
		2, {0xC7, 0xA7}
	},
	{
		2, {0x94, 0x47},
		3, {0xE6, 0xBF, 0xA1},
		2, {0xE1, 0x6F},
		2, {0xC7, 0xA8}
	},
	{
		2, {0x94, 0x48},
		3, {0xE7, 0xA6, 0xB0},
		2, {0xB0, 0x79},
		2, {0xC7, 0xA9}
	},
	{
		2, {0x94, 0x49},
		3, {0xE7, 0xA5, 0xA2},
		2, {0x62, 0x79},
		2, {0xC7, 0xAA}
	},
	{
		2, {0x94, 0x4A},
		3, {0xE5, 0xAF, 0xA7},
		2, {0xE7, 0x5B},
		2, {0xC7, 0xAB}
	},
	{
		2, {0x94, 0x4B},
		3, {0xE8, 0x91, 0xB1},
		2, {0x71, 0x84},
		2, {0xC7, 0xAC}
	},
	{
		2, {0x94, 0x4C},
		3, {0xE7, 0x8C, 0xAB},
		2, {0x2B, 0x73},
		2, {0xC7, 0xAD}
	},
	{
		2, {0x94, 0x4D},
		3, {0xE7, 0x86, 0xB1},
		2, {0xB1, 0x71},
		2, {0xC7, 0xAE}
	},
	{
		2, {0x94, 0x4E},
		3, {0xE5, 0xB9, 0xB4},
		2, {0x74, 0x5E},
		2, {0xC7, 0xAF}
	},
	{
		2, {0x94, 0x4F},
		3, {0xE5, 0xBF, 0xB5},
		2, {0xF5, 0x5F},
		2, {0xC7, 0xB0}
	},
	{
		2, {0x94, 0x50},
		3, {0xE6, 0x8D, 0xBB},
		2, {0x7B, 0x63},
		2, {0xC7, 0xB1}
	},
	{
		2, {0x94, 0x51},
		3, {0xE6, 0x92, 0x9A},
		2, {0x9A, 0x64},
		2, {0xC7, 0xB2}
	},
	{
		2, {0x94, 0x52},
		3, {0xE7, 0x87, 0x83},
		2, {0xC3, 0x71},
		2, {0xC7, 0xB3}
	},
	{
		2, {0x94, 0x53},
		3, {0xE7, 0xB2, 0x98},
		2, {0x98, 0x7C},
		2, {0xC7, 0xB4}
	},
	{
		2, {0x94, 0x54},
		3, {0xE4, 0xB9, 0x83},
		2, {0x43, 0x4E},
		2, {0xC7, 0xB5}
	},
	{
		2, {0x94, 0x55},
		3, {0xE5, 0xBB, 0xBC},
		2, {0xFC, 0x5E},
		2, {0xC7, 0xB6}
	},
	{
		2, {0x94, 0x56},
		3, {0xE4, 0xB9, 0x8B},
		2, {0x4B, 0x4E},
		2, {0xC7, 0xB7}
	},
	{
		2, {0x94, 0x57},
		3, {0xE5, 0x9F, 0x9C},
		2, {0xDC, 0x57},
		2, {0xC7, 0xB8}
	},
	{
		2, {0x94, 0x58},
		3, {0xE5, 0x9A, 0xA2},
		2, {0xA2, 0x56},
		2, {0xC7, 0xB9}
	},
	{
		2, {0x94, 0x59},
		3, {0xE6, 0x82, 0xA9},
		2, {0xA9, 0x60},
		2, {0xC7, 0xBA}
	},
	{
		2, {0x94, 0x5A},
		3, {0xE6, 0xBF, 0x83},
		2, {0xC3, 0x6F},
		2, {0xC7, 0xBB}
	},
	{
		2, {0x94, 0x5B},
		3, {0xE7, 0xB4, 0x8D},
		2, {0x0D, 0x7D},
		2, {0xC7, 0xBC}
	},
	{
		2, {0x94, 0x5C},
		3, {0xE8, 0x83, 0xBD},
		2, {0xFD, 0x80},
		2, {0xC7, 0xBD}
	},
	{
		2, {0x94, 0x5D},
		3, {0xE8, 0x84, 0xB3},
		2, {0x33, 0x81},
		2, {0xC7, 0xBE}
	},
	{
		2, {0x94, 0x5E},
		3, {0xE8, 0x86, 0xBF},
		2, {0xBF, 0x81},
		2, {0xC7, 0xBF}
	},
	{
		2, {0x94, 0x5F},
		3, {0xE8, 0xBE, 0xB2},
		2, {0xB2, 0x8F},
		2, {0xC7, 0xC0}
	},
	{
		2, {0x94, 0x60},
		3, {0xE8, 0xA6, 0x97},
		2, {0x97, 0x89},
		2, {0xC7, 0xC1}
	},
	{
		2, {0x94, 0x61},
		3, {0xE8, 0x9A, 0xA4},
		2, {0xA4, 0x86},
		2, {0xC7, 0xC2}
	},
	{
		2, {0x94, 0x62},
		3, {0xE5, 0xB7, 0xB4},
		2, {0xF4, 0x5D},
		2, {0xC7, 0xC3}
	},
	{
		2, {0x94, 0x63},
		3, {0xE6, 0x8A, 0x8A},
		2, {0x8A, 0x62},
		2, {0xC7, 0xC4}
	},
	{
		2, {0x94, 0x64},
		3, {0xE6, 0x92, 0xAD},
		2, {0xAD, 0x64},
		2, {0xC7, 0xC5}
	},
	{
		2, {0x94, 0x65},
		3, {0xE8, 0xA6, 0x87},
		2, {0x87, 0x89},
		2, {0xC7, 0xC6}
	},
	{
		2, {0x94, 0x66},
		3, {0xE6, 0x9D, 0xB7},
		2, {0x77, 0x67},
		2, {0xC7, 0xC7}
	},
	{
		2, {0x94, 0x67},
		3, {0xE6, 0xB3, 0xA2},
		2, {0xE2, 0x6C},
		2, {0xC7, 0xC8}
	},
	{
		2, {0x94, 0x68},
		3, {0xE6, 0xB4, 0xBE},
		2, {0x3E, 0x6D},
		2, {0xC7, 0xC9}
	},
	{
		2, {0x94, 0x69},
		3, {0xE7, 0x90, 0xB6},
		2, {0x36, 0x74},
		2, {0xC7, 0xCA}
	},
	{
		2, {0x94, 0x6A},
		3, {0xE7, 0xA0, 0xB4},
		2, {0x34, 0x78},
		2, {0xC7, 0xCB}
	},
	{
		2, {0x94, 0x6B},
		3, {0xE5, 0xA9, 0x86},
		2, {0x46, 0x5A},
		2, {0xC7, 0xCC}
	},
	{
		2, {0x94, 0x6C},
		3, {0xE7, 0xBD, 0xB5},
		2, {0x75, 0x7F},
		2, {0xC7, 0xCD}
	},
	{
		2, {0x94, 0x6D},
		3, {0xE8, 0x8A, 0xAD},
		2, {0xAD, 0x82},
		2, {0xC7, 0xCE}
	},
	{
		2, {0x94, 0x6E},
		3, {0xE9, 0xA6, 0xAC},
		2, {0xAC, 0x99},
		2, {0xC7, 0xCF}
	},
	{
		2, {0x94, 0x6F},
		3, {0xE4, 0xBF, 0xB3},
		2, {0xF3, 0x4F},
		2, {0xC7, 0xD0}
	},
	{
		2, {0x94, 0x70},
		3, {0xE5, 0xBB, 0x83},
		2, {0xC3, 0x5E},
		2, {0xC7, 0xD1}
	},
	{
		2, {0x94, 0x71},
		3, {0xE6, 0x8B, 0x9D},
		2, {0xDD, 0x62},
		2, {0xC7, 0xD2}
	},
	{
		2, {0x94, 0x72},
		3, {0xE6, 0x8E, 0x92},
		2, {0x92, 0x63},
		2, {0xC7, 0xD3}
	},
	{
		2, {0x94, 0x73},
		3, {0xE6, 0x95, 0x97},
		2, {0x57, 0x65},
		2, {0xC7, 0xD4}
	},
	{
		2, {0x94, 0x74},
		3, {0xE6, 0x9D, 0xAF},
		2, {0x6F, 0x67},
		2, {0xC7, 0xD5}
	},
	{
		2, {0x94, 0x75},
		3, {0xE7, 0x9B, 0x83},
		2, {0xC3, 0x76},
		2, {0xC7, 0xD6}
	},
	{
		2, {0x94, 0x76},
		3, {0xE7, 0x89, 0x8C},
		2, {0x4C, 0x72},
		2, {0xC7, 0xD7}
	},
	{
		2, {0x94, 0x77},
		3, {0xE8, 0x83, 0x8C},
		2, {0xCC, 0x80},
		2, {0xC7, 0xD8}
	},
	{
		2, {0x94, 0x78},
		3, {0xE8, 0x82, 0xBA},
		2, {0xBA, 0x80},
		2, {0xC7, 0xD9}
	},
	{
		2, {0x94, 0x79},
		3, {0xE8, 0xBC, 0xA9},
		2, {0x29, 0x8F},
		2, {0xC7, 0xDA}
	},
	{
		2, {0x94, 0x7A},
		3, {0xE9, 0x85, 0x8D},
		2, {0x4D, 0x91},
		2, {0xC7, 0xDB}
	},
	{
		2, {0x94, 0x7B},
		3, {0xE5, 0x80, 0x8D},
		2, {0x0D, 0x50},
		2, {0xC7, 0xDC}
	},
	{
		2, {0x94, 0x7C},
		3, {0xE5, 0x9F, 0xB9},
		2, {0xF9, 0x57},
		2, {0xC7, 0xDD}
	},
	{
		2, {0x94, 0x7D},
		3, {0xE5, 0xAA, 0x92},
		2, {0x92, 0x5A},
		2, {0xC7, 0xDE}
	},
	{
		2, {0x94, 0x7E},
		3, {0xE6, 0xA2, 0x85},
		2, {0x85, 0x68},
		2, {0xC7, 0xDF}
	},
	{
		2, {0x94, 0x80},
		3, {0xE6, 0xA5, 0xB3},
		2, {0x73, 0x69},
		2, {0xC7, 0xE0}
	},
	{
		2, {0x94, 0x81},
		3, {0xE7, 0x85, 0xA4},
		2, {0x64, 0x71},
		2, {0xC7, 0xE1}
	},
	{
		2, {0x94, 0x82},
		3, {0xE7, 0x8B, 0xBD},
		2, {0xFD, 0x72},
		2, {0xC7, 0xE2}
	},
	{
		2, {0x94, 0x83},
		3, {0xE8, 0xB2, 0xB7},
		2, {0xB7, 0x8C},
		2, {0xC7, 0xE3}
	},
	{
		2, {0x94, 0x84},
		3, {0xE5, 0xA3, 0xB2},
		2, {0xF2, 0x58},
		2, {0xC7, 0xE4}
	},
	{
		2, {0x94, 0x85},
		3, {0xE8, 0xB3, 0xA0},
		2, {0xE0, 0x8C},
		2, {0xC7, 0xE5}
	},
	{
		2, {0x94, 0x86},
		3, {0xE9, 0x99, 0xAA},
		2, {0x6A, 0x96},
		2, {0xC7, 0xE6}
	},
	{
		2, {0x94, 0x87},
		3, {0xE9, 0x80, 0x99},
		2, {0x19, 0x90},
		2, {0xC7, 0xE7}
	},
	{
		2, {0x94, 0x88},
		3, {0xE8, 0x9D, 0xBF},
		2, {0x7F, 0x87},
		2, {0xC7, 0xE8}
	},
	{
		2, {0x94, 0x89},
		3, {0xE7, 0xA7, 0xA4},
		2, {0xE4, 0x79},
		2, {0xC7, 0xE9}
	},
	{
		2, {0x94, 0x8A},
		3, {0xE7, 0x9F, 0xA7},
		2, {0xE7, 0x77},
		2, {0xC7, 0xEA}
	},
	{
		2, {0x94, 0x8B},
		3, {0xE8, 0x90, 0xA9},
		2, {0x29, 0x84},
		2, {0xC7, 0xEB}
	},
	{
		2, {0x94, 0x8C},
		3, {0xE4, 0xBC, 0xAF},
		2, {0x2F, 0x4F},
		2, {0xC7, 0xEC}
	},
	{
		2, {0x94, 0x8D},
		3, {0xE5, 0x89, 0xA5},
		2, {0x65, 0x52},
		2, {0xC7, 0xED}
	},
	{
		2, {0x94, 0x8E},
		3, {0xE5, 0x8D, 0x9A},
		2, {0x5A, 0x53},
		2, {0xC7, 0xEE}
	},
	{
		2, {0x94, 0x8F},
		3, {0xE6, 0x8B, 0x8D},
		2, {0xCD, 0x62},
		2, {0xC7, 0xEF}
	},
	{
		2, {0x94, 0x90},
		3, {0xE6, 0x9F, 0x8F},
		2, {0xCF, 0x67},
		2, {0xC7, 0xF0}
	},
	{
		2, {0x94, 0x91},
		3, {0xE6, 0xB3, 0x8A},
		2, {0xCA, 0x6C},
		2, {0xC7, 0xF1}
	},
	{
		2, {0x94, 0x92},
		3, {0xE7, 0x99, 0xBD},
		2, {0x7D, 0x76},
		2, {0xC7, 0xF2}
	},
	{
		2, {0x94, 0x93},
		3, {0xE7, 0xAE, 0x94},
		2, {0x94, 0x7B},
		2, {0xC7, 0xF3}
	},
	{
		2, {0x94, 0x94},
		3, {0xE7, 0xB2, 0x95},
		2, {0x95, 0x7C},
		2, {0xC7, 0xF4}
	},
	{
		2, {0x94, 0x95},
		3, {0xE8, 0x88, 0xB6},
		2, {0x36, 0x82},
		2, {0xC7, 0xF5}
	},
	{
		2, {0x94, 0x96},
		3, {0xE8, 0x96, 0x84},
		2, {0x84, 0x85},
		2, {0xC7, 0xF6}
	},
	{
		2, {0x94, 0x97},
		3, {0xE8, 0xBF, 0xAB},
		2, {0xEB, 0x8F},
		2, {0xC7, 0xF7}
	},
	{
		2, {0x94, 0x98},
		3, {0xE6, 0x9B, 0x9D},
		2, {0xDD, 0x66},
		2, {0xC7, 0xF8}
	},
	{
		2, {0x94, 0x99},
		3, {0xE6, 0xBC, 0xA0},
		2, {0x20, 0x6F},
		2, {0xC7, 0xF9}
	},
	{
		2, {0x94, 0x9A},
		3, {0xE7, 0x88, 0x86},
		2, {0x06, 0x72},
		2, {0xC7, 0xFA}
	},
	{
		2, {0x94, 0x9B},
		3, {0xE7, 0xB8, 0x9B},
		2, {0x1B, 0x7E},
		2, {0xC7, 0xFB}
	},
	{
		2, {0x94, 0x9C},
		3, {0xE8, 0x8E, 0xAB},
		2, {0xAB, 0x83},
		2, {0xC7, 0xFC}
	},
	{
		2, {0x94, 0x9D},
		3, {0xE9, 0xA7, 0x81},
		2, {0xC1, 0x99},
		2, {0xC7, 0xFD}
	},
	{
		2, {0x94, 0x9E},
		3, {0xE9, 0xBA, 0xA6},
		2, {0xA6, 0x9E},
		2, {0xC7, 0xFE}
	},
	{
		2, {0x94, 0x9F},
		3, {0xE5, 0x87, 0xBD},
		2, {0xFD, 0x51},
		2, {0xC8, 0xA1}
	},
	{
		2, {0x94, 0xA0},
		3, {0xE7, 0xAE, 0xB1},
		2, {0xB1, 0x7B},
		2, {0xC8, 0xA2}
	},
	{
		2, {0x94, 0xA1},
		3, {0xE7, 0xA1, 0xB2},
		2, {0x72, 0x78},
		2, {0xC8, 0xA3}
	},
	{
		2, {0x94, 0xA2},
		3, {0xE7, 0xAE, 0xB8},
		2, {0xB8, 0x7B},
		2, {0xC8, 0xA4}
	},
	{
		2, {0x94, 0xA3},
		3, {0xE8, 0x82, 0x87},
		2, {0x87, 0x80},
		2, {0xC8, 0xA5}
	},
	{
		2, {0x94, 0xA4},
		3, {0xE7, 0xAD, 0x88},
		2, {0x48, 0x7B},
		2, {0xC8, 0xA6}
	},
	{
		2, {0x94, 0xA5},
		3, {0xE6, 0xAB, 0xA8},
		2, {0xE8, 0x6A},
		2, {0xC8, 0xA7}
	},
	{
		2, {0x94, 0xA6},
		3, {0xE5, 0xB9, 0xA1},
		2, {0x61, 0x5E},
		2, {0xC8, 0xA8}
	},
	{
		2, {0x94, 0xA7},
		3, {0xE8, 0x82, 0x8C},
		2, {0x8C, 0x80},
		2, {0xC8, 0xA9}
	},
	{
		2, {0x94, 0xA8},
		3, {0xE7, 0x95, 0x91},
		2, {0x51, 0x75},
		2, {0xC8, 0xAA}
	},
	{
		2, {0x94, 0xA9},
		3, {0xE7, 0x95, 0xA0},
		2, {0x60, 0x75},
		2, {0xC8, 0xAB}
	},
	{
		2, {0x94, 0xAA},
		3, {0xE5, 0x85, 0xAB},
		2, {0x6B, 0x51},
		2, {0xC8, 0xAC}
	},
	{
		2, {0x94, 0xAB},
		3, {0xE9, 0x89, 0xA2},
		2, {0x62, 0x92},
		2, {0xC8, 0xAD}
	},
	{
		2, {0x94, 0xAC},
		3, {0xE6, 0xBA, 0x8C},
		2, {0x8C, 0x6E},
		2, {0xC8, 0xAE}
	},
	{
		2, {0x94, 0xAD},
		3, {0xE7, 0x99, 0xBA},
		2, {0x7A, 0x76},
		2, {0xC8, 0xAF}
	},
	{
		2, {0x94, 0xAE},
		3, {0xE9, 0x86, 0x97},
		2, {0x97, 0x91},
		2, {0xC8, 0xB0}
	},
	{
		2, {0x94, 0xAF},
		3, {0xE9, 0xAB, 0xAA},
		2, {0xEA, 0x9A},
		2, {0xC8, 0xB1}
	},
	{
		2, {0x94, 0xB0},
		3, {0xE4, 0xBC, 0x90},
		2, {0x10, 0x4F},
		2, {0xC8, 0xB2}
	},
	{
		2, {0x94, 0xB1},
		3, {0xE7, 0xBD, 0xB0},
		2, {0x70, 0x7F},
		2, {0xC8, 0xB3}
	},
	{
		2, {0x94, 0xB2},
		3, {0xE6, 0x8A, 0x9C},
		2, {0x9C, 0x62},
		2, {0xC8, 0xB4}
	},
	{
		2, {0x94, 0xB3},
		3, {0xE7, 0xAD, 0x8F},
		2, {0x4F, 0x7B},
		2, {0xC8, 0xB5}
	},
	{
		2, {0x94, 0xB4},
		3, {0xE9, 0x96, 0xA5},
		2, {0xA5, 0x95},
		2, {0xC8, 0xB6}
	},
	{
		2, {0x94, 0xB5},
		3, {0xE9, 0xB3, 0xA9},
		2, {0xE9, 0x9C},
		2, {0xC8, 0xB7}
	},
	{
		2, {0x94, 0xB6},
		3, {0xE5, 0x99, 0xBA},
		2, {0x7A, 0x56},
		2, {0xC8, 0xB8}
	},
	{
		2, {0x94, 0xB7},
		3, {0xE5, 0xA1, 0x99},
		2, {0x59, 0x58},
		2, {0xC8, 0xB9}
	},
	{
		2, {0x94, 0xB8},
		3, {0xE8, 0x9B, 0xA4},
		2, {0xE4, 0x86},
		2, {0xC8, 0xBA}
	},
	{
		2, {0x94, 0xB9},
		3, {0xE9, 0x9A, 0xBC},
		2, {0xBC, 0x96},
		2, {0xC8, 0xBB}
	},
	{
		2, {0x94, 0xBA},
		3, {0xE4, 0xBC, 0xB4},
		2, {0x34, 0x4F},
		2, {0xC8, 0xBC}
	},
	{
		2, {0x94, 0xBB},
		3, {0xE5, 0x88, 0xA4},
		2, {0x24, 0x52},
		2, {0xC8, 0xBD}
	},
	{
		2, {0x94, 0xBC},
		3, {0xE5, 0x8D, 0x8A},
		2, {0x4A, 0x53},
		2, {0xC8, 0xBE}
	},
	{
		2, {0x94, 0xBD},
		3, {0xE5, 0x8F, 0x8D},
		2, {0xCD, 0x53},
		2, {0xC8, 0xBF}
	},
	{
		2, {0x94, 0xBE},
		3, {0xE5, 0x8F, 0x9B},
		2, {0xDB, 0x53},
		2, {0xC8, 0xC0}
	},
	{
		2, {0x94, 0xBF},
		3, {0xE5, 0xB8, 0x86},
		2, {0x06, 0x5E},
		2, {0xC8, 0xC1}
	},
	{
		2, {0x94, 0xC0},
		3, {0xE6, 0x90, 0xAC},
		2, {0x2C, 0x64},
		2, {0xC8, 0xC2}
	},
	{
		2, {0x94, 0xC1},
		3, {0xE6, 0x96, 0x91},
		2, {0x91, 0x65},
		2, {0xC8, 0xC3}
	},
	{
		2, {0x94, 0xC2},
		3, {0xE6, 0x9D, 0xBF},
		2, {0x7F, 0x67},
		2, {0xC8, 0xC4}
	},
	{
		2, {0x94, 0xC3},
		3, {0xE6, 0xB0, 0xBE},
		2, {0x3E, 0x6C},
		2, {0xC8, 0xC5}
	},
	{
		2, {0x94, 0xC4},
		3, {0xE6, 0xB1, 0x8E},
		2, {0x4E, 0x6C},
		2, {0xC8, 0xC6}
	},
	{
		2, {0x94, 0xC5},
		3, {0xE7, 0x89, 0x88},
		2, {0x48, 0x72},
		2, {0xC8, 0xC7}
	},
	{
		2, {0x94, 0xC6},
		3, {0xE7, 0x8A, 0xAF},
		2, {0xAF, 0x72},
		2, {0xC8, 0xC8}
	},
	{
		2, {0x94, 0xC7},
		3, {0xE7, 0x8F, 0xAD},
		2, {0xED, 0x73},
		2, {0xC8, 0xC9}
	},
	{
		2, {0x94, 0xC8},
		3, {0xE7, 0x95, 0x94},
		2, {0x54, 0x75},
		2, {0xC8, 0xCA}
	},
	{
		2, {0x94, 0xC9},
		3, {0xE7, 0xB9, 0x81},
		2, {0x41, 0x7E},
		2, {0xC8, 0xCB}
	},
	{
		2, {0x94, 0xCA},
		3, {0xE8, 0x88, 0xAC},
		2, {0x2C, 0x82},
		2, {0xC8, 0xCC}
	},
	{
		2, {0x94, 0xCB},
		3, {0xE8, 0x97, 0xA9},
		2, {0xE9, 0x85},
		2, {0xC8, 0xCD}
	},
	{
		2, {0x94, 0xCC},
		3, {0xE8, 0xB2, 0xA9},
		2, {0xA9, 0x8C},
		2, {0xC8, 0xCE}
	},
	{
		2, {0x94, 0xCD},
		3, {0xE7, 0xAF, 0x84},
		2, {0xC4, 0x7B},
		2, {0xC8, 0xCF}
	},
	{
		2, {0x94, 0xCE},
		3, {0xE9, 0x87, 0x86},
		2, {0xC6, 0x91},
		2, {0xC8, 0xD0}
	},
	{
		2, {0x94, 0xCF},
		3, {0xE7, 0x85, 0xA9},
		2, {0x69, 0x71},
		2, {0xC8, 0xD1}
	},
	{
		2, {0x94, 0xD0},
		3, {0xE9, 0xA0, 0x92},
		2, {0x12, 0x98},
		2, {0xC8, 0xD2}
	},
	{
		2, {0x94, 0xD1},
		3, {0xE9, 0xA3, 0xAF},
		2, {0xEF, 0x98},
		2, {0xC8, 0xD3}
	},
	{
		2, {0x94, 0xD2},
		3, {0xE6, 0x8C, 0xBD},
		2, {0x3D, 0x63},
		2, {0xC8, 0xD4}
	},
	{
		2, {0x94, 0xD3},
		3, {0xE6, 0x99, 0xA9},
		2, {0x69, 0x66},
		2, {0xC8, 0xD5}
	},
	{
		2, {0x94, 0xD4},
		3, {0xE7, 0x95, 0xAA},
		2, {0x6A, 0x75},
		2, {0xC8, 0xD6}
	},
	{
		2, {0x94, 0xD5},
		3, {0xE7, 0x9B, 0xA4},
		2, {0xE4, 0x76},
		2, {0xC8, 0xD7}
	},
	{
		2, {0x94, 0xD6},
		3, {0xE7, 0xA3, 0x90},
		2, {0xD0, 0x78},
		2, {0xC8, 0xD8}
	},
	{
		2, {0x94, 0xD7},
		3, {0xE8, 0x95, 0x83},
		2, {0x43, 0x85},
		2, {0xC8, 0xD9}
	},
	{
		2, {0x94, 0xD8},
		3, {0xE8, 0x9B, 0xAE},
		2, {0xEE, 0x86},
		2, {0xC8, 0xDA}
	},
	{
		2, {0x94, 0xD9},
		3, {0xE5, 0x8C, 0xAA},
		2, {0x2A, 0x53},
		2, {0xC8, 0xDB}
	},
	{
		2, {0x94, 0xDA},
		3, {0xE5, 0x8D, 0x91},
		2, {0x51, 0x53},
		2, {0xC8, 0xDC}
	},
	{
		2, {0x94, 0xDB},
		3, {0xE5, 0x90, 0xA6},
		2, {0x26, 0x54},
		2, {0xC8, 0xDD}
	},
	{
		2, {0x94, 0xDC},
		3, {0xE5, 0xA6, 0x83},
		2, {0x83, 0x59},
		2, {0xC8, 0xDE}
	},
	{
		2, {0x94, 0xDD},
		3, {0xE5, 0xBA, 0x87},
		2, {0x87, 0x5E},
		2, {0xC8, 0xDF}
	},
	{
		2, {0x94, 0xDE},
		3, {0xE5, 0xBD, 0xBC},
		2, {0x7C, 0x5F},
		2, {0xC8, 0xE0}
	},
	{
		2, {0x94, 0xDF},
		3, {0xE6, 0x82, 0xB2},
		2, {0xB2, 0x60},
		2, {0xC8, 0xE1}
	},
	{
		2, {0x94, 0xE0},
		3, {0xE6, 0x89, 0x89},
		2, {0x49, 0x62},
		2, {0xC8, 0xE2}
	},
	{
		2, {0x94, 0xE1},
		3, {0xE6, 0x89, 0xB9},
		2, {0x79, 0x62},
		2, {0xC8, 0xE3}
	},
	{
		2, {0x94, 0xE2},
		3, {0xE6, 0x8A, 0xAB},
		2, {0xAB, 0x62},
		2, {0xC8, 0xE4}
	},
	{
		2, {0x94, 0xE3},
		3, {0xE6, 0x96, 0x90},
		2, {0x90, 0x65},
		2, {0xC8, 0xE5}
	},
	{
		2, {0x94, 0xE4},
		3, {0xE6, 0xAF, 0x94},
		2, {0xD4, 0x6B},
		2, {0xC8, 0xE6}
	},
	{
		2, {0x94, 0xE5},
		3, {0xE6, 0xB3, 0x8C},
		2, {0xCC, 0x6C},
		2, {0xC8, 0xE7}
	},
	{
		2, {0x94, 0xE6},
		3, {0xE7, 0x96, 0xB2},
		2, {0xB2, 0x75},
		2, {0xC8, 0xE8}
	},
	{
		2, {0x94, 0xE7},
		3, {0xE7, 0x9A, 0xAE},
		2, {0xAE, 0x76},
		2, {0xC8, 0xE9}
	},
	{
		2, {0x94, 0xE8},
		3, {0xE7, 0xA2, 0x91},
		2, {0x91, 0x78},
		2, {0xC8, 0xEA}
	},
	{
		2, {0x94, 0xE9},
		3, {0xE7, 0xA7, 0x98},
		2, {0xD8, 0x79},
		2, {0xC8, 0xEB}
	},
	{
		2, {0x94, 0xEA},
		3, {0xE7, 0xB7, 0x8B},
		2, {0xCB, 0x7D},
		2, {0xC8, 0xEC}
	},
	{
		2, {0x94, 0xEB},
		3, {0xE7, 0xBD, 0xB7},
		2, {0x77, 0x7F},
		2, {0xC8, 0xED}
	},
	{
		2, {0x94, 0xEC},
		3, {0xE8, 0x82, 0xA5},
		2, {0xA5, 0x80},
		2, {0xC8, 0xEE}
	},
	{
		2, {0x94, 0xED},
		3, {0xE8, 0xA2, 0xAB},
		2, {0xAB, 0x88},
		2, {0xC8, 0xEF}
	},
	{
		2, {0x94, 0xEE},
		3, {0xE8, 0xAA, 0xB9},
		2, {0xB9, 0x8A},
		2, {0xC8, 0xF0}
	},
	{
		2, {0x94, 0xEF},
		3, {0xE8, 0xB2, 0xBB},
		2, {0xBB, 0x8C},
		2, {0xC8, 0xF1}
	},
	{
		2, {0x94, 0xF0},
		3, {0xE9, 0x81, 0xBF},
		2, {0x7F, 0x90},
		2, {0xC8, 0xF2}
	},
	{
		2, {0x94, 0xF1},
		3, {0xE9, 0x9D, 0x9E},
		2, {0x5E, 0x97},
		2, {0xC8, 0xF3}
	},
	{
		2, {0x94, 0xF2},
		3, {0xE9, 0xA3, 0x9B},
		2, {0xDB, 0x98},
		2, {0xC8, 0xF4}
	},
	{
		2, {0x94, 0xF3},
		3, {0xE6, 0xA8, 0x8B},
		2, {0x0B, 0x6A},
		2, {0xC8, 0xF5}
	},
	{
		2, {0x94, 0xF4},
		3, {0xE7, 0xB0, 0xB8},
		2, {0x38, 0x7C},
		2, {0xC8, 0xF6}
	},
	{
		2, {0x94, 0xF5},
		3, {0xE5, 0x82, 0x99},
		2, {0x99, 0x50},
		2, {0xC8, 0xF7}
	},
	{
		2, {0x94, 0xF6},
		3, {0xE5, 0xB0, 0xBE},
		2, {0x3E, 0x5C},
		2, {0xC8, 0xF8}
	},
	{
		2, {0x94, 0xF7},
		3, {0xE5, 0xBE, 0xAE},
		2, {0xAE, 0x5F},
		2, {0xC8, 0xF9}
	},
	{
		2, {0x94, 0xF8},
		3, {0xE6, 0x9E, 0x87},
		2, {0x87, 0x67},
		2, {0xC8, 0xFA}
	},
	{
		2, {0x94, 0xF9},
		3, {0xE6, 0xAF, 0x98},
		2, {0xD8, 0x6B},
		2, {0xC8, 0xFB}
	},
	{
		2, {0x94, 0xFA},
		3, {0xE7, 0x90, 0xB5},
		2, {0x35, 0x74},
		2, {0xC8, 0xFC}
	},
	{
		2, {0x94, 0xFB},
		3, {0xE7, 0x9C, 0x89},
		2, {0x09, 0x77},
		2, {0xC8, 0xFD}
	},
	{
		2, {0x94, 0xFC},
		3, {0xE7, 0xBE, 0x8E},
		2, {0x8E, 0x7F},
		2, {0xC8, 0xFE}
	},
	{
		2, {0x95, 0x40},
		3, {0xE9, 0xBC, 0xBB},
		2, {0x3B, 0x9F},
		2, {0xC9, 0xA1}
	},
	{
		2, {0x95, 0x41},
		3, {0xE6, 0x9F, 0x8A},
		2, {0xCA, 0x67},
		2, {0xC9, 0xA2}
	},
	{
		2, {0x95, 0x42},
		3, {0xE7, 0xA8, 0x97},
		2, {0x17, 0x7A},
		2, {0xC9, 0xA3}
	},
	{
		2, {0x95, 0x43},
		3, {0xE5, 0x8C, 0xB9},
		2, {0x39, 0x53},
		2, {0xC9, 0xA4}
	},
	{
		2, {0x95, 0x44},
		3, {0xE7, 0x96, 0x8B},
		2, {0x8B, 0x75},
		2, {0xC9, 0xA5}
	},
	{
		2, {0x95, 0x45},
		3, {0xE9, 0xAB, 0xAD},
		2, {0xED, 0x9A},
		2, {0xC9, 0xA6}
	},
	{
		2, {0x95, 0x46},
		3, {0xE5, 0xBD, 0xA6},
		2, {0x66, 0x5F},
		2, {0xC9, 0xA7}
	},
	{
		2, {0x95, 0x47},
		3, {0xE8, 0x86, 0x9D},
		2, {0x9D, 0x81},
		2, {0xC9, 0xA8}
	},
	{
		2, {0x95, 0x48},
		3, {0xE8, 0x8F, 0xB1},
		2, {0xF1, 0x83},
		2, {0xC9, 0xA9}
	},
	{
		2, {0x95, 0x49},
		3, {0xE8, 0x82, 0x98},
		2, {0x98, 0x80},
		2, {0xC9, 0xAA}
	},
	{
		2, {0x95, 0x4A},
		3, {0xE5, 0xBC, 0xBC},
		2, {0x3C, 0x5F},
		2, {0xC9, 0xAB}
	},
	{
		2, {0x95, 0x4B},
		3, {0xE5, 0xBF, 0x85},
		2, {0xC5, 0x5F},
		2, {0xC9, 0xAC}
	},
	{
		2, {0x95, 0x4C},
		3, {0xE7, 0x95, 0xA2},
		2, {0x62, 0x75},
		2, {0xC9, 0xAD}
	},
	{
		2, {0x95, 0x4D},
		3, {0xE7, 0xAD, 0x86},
		2, {0x46, 0x7B},
		2, {0xC9, 0xAE}
	},
	{
		2, {0x95, 0x4E},
		3, {0xE9, 0x80, 0xBC},
		2, {0x3C, 0x90},
		2, {0xC9, 0xAF}
	},
	{
		2, {0x95, 0x4F},
		3, {0xE6, 0xA1, 0xA7},
		2, {0x67, 0x68},
		2, {0xC9, 0xB0}
	},
	{
		2, {0x95, 0x50},
		3, {0xE5, 0xA7, 0xAB},
		2, {0xEB, 0x59},
		2, {0xC9, 0xB1}
	},
	{
		2, {0x95, 0x51},
		3, {0xE5, 0xAA, 0x9B},
		2, {0x9B, 0x5A},
		2, {0xC9, 0xB2}
	},
	{
		2, {0x95, 0x52},
		3, {0xE7, 0xB4, 0x90},
		2, {0x10, 0x7D},
		2, {0xC9, 0xB3}
	},
	{
		2, {0x95, 0x53},
		3, {0xE7, 0x99, 0xBE},
		2, {0x7E, 0x76},
		2, {0xC9, 0xB4}
	},
	{
		2, {0x95, 0x54},
		3, {0xE8, 0xAC, 0xAC},
		2, {0x2C, 0x8B},
		2, {0xC9, 0xB5}
	},
	{
		2, {0x95, 0x55},
		3, {0xE4, 0xBF, 0xB5},
		2, {0xF5, 0x4F},
		2, {0xC9, 0xB6}
	},
	{
		2, {0x95, 0x56},
		3, {0xE5, 0xBD, 0xAA},
		2, {0x6A, 0x5F},
		2, {0xC9, 0xB7}
	},
	{
		2, {0x95, 0x57},
		3, {0xE6, 0xA8, 0x99},
		2, {0x19, 0x6A},
		2, {0xC9, 0xB8}
	},
	{
		2, {0x95, 0x58},
		3, {0xE6, 0xB0, 0xB7},
		2, {0x37, 0x6C},
		2, {0xC9, 0xB9}
	},
	{
		2, {0x95, 0x59},
		3, {0xE6, 0xBC, 0x82},
		2, {0x02, 0x6F},
		2, {0xC9, 0xBA}
	},
	{
		2, {0x95, 0x5A},
		3, {0xE7, 0x93, 0xA2},
		2, {0xE2, 0x74},
		2, {0xC9, 0xBB}
	},
	{
		2, {0x95, 0x5B},
		3, {0xE7, 0xA5, 0xA8},
		2, {0x68, 0x79},
		2, {0xC9, 0xBC}
	},
	{
		2, {0x95, 0x5C},
		3, {0xE8, 0xA1, 0xA8},
		2, {0x68, 0x88},
		2, {0xC9, 0xBD}
	},
	{
		2, {0x95, 0x5D},
		3, {0xE8, 0xA9, 0x95},
		2, {0x55, 0x8A},
		2, {0xC9, 0xBE}
	},
	{
		2, {0x95, 0x5E},
		3, {0xE8, 0xB1, 0xB9},
		2, {0x79, 0x8C},
		2, {0xC9, 0xBF}
	},
	{
		2, {0x95, 0x5F},
		3, {0xE5, 0xBB, 0x9F},
		2, {0xDF, 0x5E},
		2, {0xC9, 0xC0}
	},
	{
		2, {0x95, 0x60},
		3, {0xE6, 0x8F, 0x8F},
		2, {0xCF, 0x63},
		2, {0xC9, 0xC1}
	},
	{
		2, {0x95, 0x61},
		3, {0xE7, 0x97, 0x85},
		2, {0xC5, 0x75},
		2, {0xC9, 0xC2}
	},
	{
		2, {0x95, 0x62},
		3, {0xE7, 0xA7, 0x92},
		2, {0xD2, 0x79},
		2, {0xC9, 0xC3}
	},
	{
		2, {0x95, 0x63},
		3, {0xE8, 0x8B, 0x97},
		2, {0xD7, 0x82},
		2, {0xC9, 0xC4}
	},
	{
		2, {0x95, 0x64},
		3, {0xE9, 0x8C, 0xA8},
		2, {0x28, 0x93},
		2, {0xC9, 0xC5}
	},
	{
		2, {0x95, 0x65},
		3, {0xE9, 0x8B, 0xB2},
		2, {0xF2, 0x92},
		2, {0xC9, 0xC6}
	},
	{
		2, {0x95, 0x66},
		3, {0xE8, 0x92, 0x9C},
		2, {0x9C, 0x84},
		2, {0xC9, 0xC7}
	},
	{
		2, {0x95, 0x67},
		3, {0xE8, 0x9B, 0xAD},
		2, {0xED, 0x86},
		2, {0xC9, 0xC8}
	},
	{
		2, {0x95, 0x68},
		3, {0xE9, 0xB0, 0xAD},
		2, {0x2D, 0x9C},
		2, {0xC9, 0xC9}
	},
	{
		2, {0x95, 0x69},
		3, {0xE5, 0x93, 0x81},
		2, {0xC1, 0x54},
		2, {0xC9, 0xCA}
	},
	{
		2, {0x95, 0x6A},
		3, {0xE5, 0xBD, 0xAC},
		2, {0x6C, 0x5F},
		2, {0xC9, 0xCB}
	},
	{
		2, {0x95, 0x6B},
		3, {0xE6, 0x96, 0x8C},
		2, {0x8C, 0x65},
		2, {0xC9, 0xCC}
	},
	{
		2, {0x95, 0x6C},
		3, {0xE6, 0xB5, 0x9C},
		2, {0x5C, 0x6D},
		2, {0xC9, 0xCD}
	},
	{
		2, {0x95, 0x6D},
		3, {0xE7, 0x80, 0x95},
		2, {0x15, 0x70},
		2, {0xC9, 0xCE}
	},
	{
		2, {0x95, 0x6E},
		3, {0xE8, 0xB2, 0xA7},
		2, {0xA7, 0x8C},
		2, {0xC9, 0xCF}
	},
	{
		2, {0x95, 0x6F},
		3, {0xE8, 0xB3, 0x93},
		2, {0xD3, 0x8C},
		2, {0xC9, 0xD0}
	},
	{
		2, {0x95, 0x70},
		3, {0xE9, 0xA0, 0xBB},
		2, {0x3B, 0x98},
		2, {0xC9, 0xD1}
	},
	{
		2, {0x95, 0x71},
		3, {0xE6, 0x95, 0x8F},
		2, {0x4F, 0x65},
		2, {0xC9, 0xD2}
	},
	{
		2, {0x95, 0x72},
		3, {0xE7, 0x93, 0xB6},
		2, {0xF6, 0x74},
		2, {0xC9, 0xD3}
	},
	{
		2, {0x95, 0x73},
		3, {0xE4, 0xB8, 0x8D},
		2, {0x0D, 0x4E},
		2, {0xC9, 0xD4}
	},
	{
		2, {0x95, 0x74},
		3, {0xE4, 0xBB, 0x98},
		2, {0xD8, 0x4E},
		2, {0xC9, 0xD5}
	},
	{
		2, {0x95, 0x75},
		3, {0xE5, 0x9F, 0xA0},
		2, {0xE0, 0x57},
		2, {0xC9, 0xD6}
	},
	{
		2, {0x95, 0x76},
		3, {0xE5, 0xA4, 0xAB},
		2, {0x2B, 0x59},
		2, {0xC9, 0xD7}
	},
	{
		2, {0x95, 0x77},
		3, {0xE5, 0xA9, 0xA6},
		2, {0x66, 0x5A},
		2, {0xC9, 0xD8}
	},
	{
		2, {0x95, 0x78},
		3, {0xE5, 0xAF, 0x8C},
		2, {0xCC, 0x5B},
		2, {0xC9, 0xD9}
	},
	{
		2, {0x95, 0x79},
		3, {0xE5, 0x86, 0xA8},
		2, {0xA8, 0x51},
		2, {0xC9, 0xDA}
	},
	{
		2, {0x95, 0x7A},
		3, {0xE5, 0xB8, 0x83},
		2, {0x03, 0x5E},
		2, {0xC9, 0xDB}
	},
	{
		2, {0x95, 0x7B},
		3, {0xE5, 0xBA, 0x9C},
		2, {0x9C, 0x5E},
		2, {0xC9, 0xDC}
	},
	{
		2, {0x95, 0x7C},
		3, {0xE6, 0x80, 0x96},
		2, {0x16, 0x60},
		2, {0xC9, 0xDD}
	},
	{
		2, {0x95, 0x7D},
		3, {0xE6, 0x89, 0xB6},
		2, {0x76, 0x62},
		2, {0xC9, 0xDE}
	},
	{
		2, {0x95, 0x7E},
		3, {0xE6, 0x95, 0xB7},
		2, {0x77, 0x65},
		2, {0xC9, 0xDF}
	},
	{
		2, {0x95, 0x80},
		3, {0xE6, 0x96, 0xA7},
		2, {0xA7, 0x65},
		2, {0xC9, 0xE0}
	},
	{
		2, {0x95, 0x81},
		3, {0xE6, 0x99, 0xAE},
		2, {0x6E, 0x66},
		2, {0xC9, 0xE1}
	},
	{
		2, {0x95, 0x82},
		3, {0xE6, 0xB5, 0xAE},
		2, {0x6E, 0x6D},
		2, {0xC9, 0xE2}
	},
	{
		2, {0x95, 0x83},
		3, {0xE7, 0x88, 0xB6},
		2, {0x36, 0x72},
		2, {0xC9, 0xE3}
	},
	{
		2, {0x95, 0x84},
		3, {0xE7, 0xAC, 0xA6},
		2, {0x26, 0x7B},
		2, {0xC9, 0xE4}
	},
	{
		2, {0x95, 0x85},
		3, {0xE8, 0x85, 0x90},
		2, {0x50, 0x81},
		2, {0xC9, 0xE5}
	},
	{
		2, {0x95, 0x86},
		3, {0xE8, 0x86, 0x9A},
		2, {0x9A, 0x81},
		2, {0xC9, 0xE6}
	},
	{
		2, {0x95, 0x87},
		3, {0xE8, 0x8A, 0x99},
		2, {0x99, 0x82},
		2, {0xC9, 0xE7}
	},
	{
		2, {0x95, 0x88},
		3, {0xE8, 0xAD, 0x9C},
		2, {0x5C, 0x8B},
		2, {0xC9, 0xE8}
	},
	{
		2, {0x95, 0x89},
		3, {0xE8, 0xB2, 0xA0},
		2, {0xA0, 0x8C},
		2, {0xC9, 0xE9}
	},
	{
		2, {0x95, 0x8A},
		3, {0xE8, 0xB3, 0xA6},
		2, {0xE6, 0x8C},
		2, {0xC9, 0xEA}
	},
	{
		2, {0x95, 0x8B},
		3, {0xE8, 0xB5, 0xB4},
		2, {0x74, 0x8D},
		2, {0xC9, 0xEB}
	},
	{
		2, {0x95, 0x8C},
		3, {0xE9, 0x98, 0x9C},
		2, {0x1C, 0x96},
		2, {0xC9, 0xEC}
	},
	{
		2, {0x95, 0x8D},
		3, {0xE9, 0x99, 0x84},
		2, {0x44, 0x96},
		2, {0xC9, 0xED}
	},
	{
		2, {0x95, 0x8E},
		3, {0xE4, 0xBE, 0xAE},
		2, {0xAE, 0x4F},
		2, {0xC9, 0xEE}
	},
	{
		2, {0x95, 0x8F},
		3, {0xE6, 0x92, 0xAB},
		2, {0xAB, 0x64},
		2, {0xC9, 0xEF}
	},
	{
		2, {0x95, 0x90},
		3, {0xE6, 0xAD, 0xA6},
		2, {0x66, 0x6B},
		2, {0xC9, 0xF0}
	},
	{
		2, {0x95, 0x91},
		3, {0xE8, 0x88, 0x9E},
		2, {0x1E, 0x82},
		2, {0xC9, 0xF1}
	},
	{
		2, {0x95, 0x92},
		3, {0xE8, 0x91, 0xA1},
		2, {0x61, 0x84},
		2, {0xC9, 0xF2}
	},
	{
		2, {0x95, 0x93},
		3, {0xE8, 0x95, 0xAA},
		2, {0x6A, 0x85},
		2, {0xC9, 0xF3}
	},
	{
		2, {0x95, 0x94},
		3, {0xE9, 0x83, 0xA8},
		2, {0xE8, 0x90},
		2, {0xC9, 0xF4}
	},
	{
		2, {0x95, 0x95},
		3, {0xE5, 0xB0, 0x81},
		2, {0x01, 0x5C},
		2, {0xC9, 0xF5}
	},
	{
		2, {0x95, 0x96},
		3, {0xE6, 0xA5, 0x93},
		2, {0x53, 0x69},
		2, {0xC9, 0xF6}
	},
	{
		2, {0x95, 0x97},
		3, {0xE9, 0xA2, 0xA8},
		2, {0xA8, 0x98},
		2, {0xC9, 0xF7}
	},
	{
		2, {0x95, 0x98},
		3, {0xE8, 0x91, 0xBA},
		2, {0x7A, 0x84},
		2, {0xC9, 0xF8}
	},
	{
		2, {0x95, 0x99},
		3, {0xE8, 0x95, 0x97},
		2, {0x57, 0x85},
		2, {0xC9, 0xF9}
	},
	{
		2, {0x95, 0x9A},
		3, {0xE4, 0xBC, 0x8F},
		2, {0x0F, 0x4F},
		2, {0xC9, 0xFA}
	},
	{
		2, {0x95, 0x9B},
		3, {0xE5, 0x89, 0xAF},
		2, {0x6F, 0x52},
		2, {0xC9, 0xFB}
	},
	{
		2, {0x95, 0x9C},
		3, {0xE5, 0xBE, 0xA9},
		2, {0xA9, 0x5F},
		2, {0xC9, 0xFC}
	},
	{
		2, {0x95, 0x9D},
		3, {0xE5, 0xB9, 0x85},
		2, {0x45, 0x5E},
		2, {0xC9, 0xFD}
	},
	{
		2, {0x95, 0x9E},
		3, {0xE6, 0x9C, 0x8D},
		2, {0x0D, 0x67},
		2, {0xC9, 0xFE}
	},
	{
		2, {0x95, 0x9F},
		3, {0xE7, 0xA6, 0x8F},
		2, {0x8F, 0x79},
		2, {0xCA, 0xA1}
	},
	{
		2, {0x95, 0xA0},
		3, {0xE8, 0x85, 0xB9},
		2, {0x79, 0x81},
		2, {0xCA, 0xA2}
	},
	{
		2, {0x95, 0xA1},
		3, {0xE8, 0xA4, 0x87},
		2, {0x07, 0x89},
		2, {0xCA, 0xA3}
	},
	{
		2, {0x95, 0xA2},
		3, {0xE8, 0xA6, 0x86},
		2, {0x86, 0x89},
		2, {0xCA, 0xA4}
	},
	{
		2, {0x95, 0xA3},
		3, {0xE6, 0xB7, 0xB5},
		2, {0xF5, 0x6D},
		2, {0xCA, 0xA5}
	},
	{
		2, {0x95, 0xA4},
		3, {0xE5, 0xBC, 0x97},
		2, {0x17, 0x5F},
		2, {0xCA, 0xA6}
	},
	{
		2, {0x95, 0xA5},
		3, {0xE6, 0x89, 0x95},
		2, {0x55, 0x62},
		2, {0xCA, 0xA7}
	},
	{
		2, {0x95, 0xA6},
		3, {0xE6, 0xB2, 0xB8},
		2, {0xB8, 0x6C},
		2, {0xCA, 0xA8}
	},
	{
		2, {0x95, 0xA7},
		3, {0xE4, 0xBB, 0x8F},
		2, {0xCF, 0x4E},
		2, {0xCA, 0xA9}
	},
	{
		2, {0x95, 0xA8},
		3, {0xE7, 0x89, 0xA9},
		2, {0x69, 0x72},
		2, {0xCA, 0xAA}
	},
	{
		2, {0x95, 0xA9},
		3, {0xE9, 0xAE, 0x92},
		2, {0x92, 0x9B},
		2, {0xCA, 0xAB}
	},
	{
		2, {0x95, 0xAA},
		3, {0xE5, 0x88, 0x86},
		2, {0x06, 0x52},
		2, {0xCA, 0xAC}
	},
	{
		2, {0x95, 0xAB},
		3, {0xE5, 0x90, 0xBB},
		2, {0x3B, 0x54},
		2, {0xCA, 0xAD}
	},
	{
		2, {0x95, 0xAC},
		3, {0xE5, 0x99, 0xB4},
		2, {0x74, 0x56},
		2, {0xCA, 0xAE}
	},
	{
		2, {0x95, 0xAD},
		3, {0xE5, 0xA2, 0xB3},
		2, {0xB3, 0x58},
		2, {0xCA, 0xAF}
	},
	{
		2, {0x95, 0xAE},
		3, {0xE6, 0x86, 0xA4},
		2, {0xA4, 0x61},
		2, {0xCA, 0xB0}
	},
	{
		2, {0x95, 0xAF},
		3, {0xE6, 0x89, 0xAE},
		2, {0x6E, 0x62},
		2, {0xCA, 0xB1}
	},
	{
		2, {0x95, 0xB0},
		3, {0xE7, 0x84, 0x9A},
		2, {0x1A, 0x71},
		2, {0xCA, 0xB2}
	},
	{
		2, {0x95, 0xB1},
		3, {0xE5, 0xA5, 0xAE},
		2, {0x6E, 0x59},
		2, {0xCA, 0xB3}
	},
	{
		2, {0x95, 0xB2},
		3, {0xE7, 0xB2, 0x89},
		2, {0x89, 0x7C},
		2, {0xCA, 0xB4}
	},
	{
		2, {0x95, 0xB3},
		3, {0xE7, 0xB3, 0x9E},
		2, {0xDE, 0x7C},
		2, {0xCA, 0xB5}
	},
	{
		2, {0x95, 0xB4},
		3, {0xE7, 0xB4, 0x9B},
		2, {0x1B, 0x7D},
		2, {0xCA, 0xB6}
	},
	{
		2, {0x95, 0xB5},
		3, {0xE9, 0x9B, 0xB0},
		2, {0xF0, 0x96},
		2, {0xCA, 0xB7}
	},
	{
		2, {0x95, 0xB6},
		3, {0xE6, 0x96, 0x87},
		2, {0x87, 0x65},
		2, {0xCA, 0xB8}
	},
	{
		2, {0x95, 0xB7},
		3, {0xE8, 0x81, 0x9E},
		2, {0x5E, 0x80},
		2, {0xCA, 0xB9}
	},
	{
		2, {0x95, 0xB8},
		3, {0xE4, 0xB8, 0x99},
		2, {0x19, 0x4E},
		2, {0xCA, 0xBA}
	},
	{
		2, {0x95, 0xB9},
		3, {0xE4, 0xBD, 0xB5},
		2, {0x75, 0x4F},
		2, {0xCA, 0xBB}
	},
	{
		2, {0x95, 0xBA},
		3, {0xE5, 0x85, 0xB5},
		2, {0x75, 0x51},
		2, {0xCA, 0xBC}
	},
	{
		2, {0x95, 0xBB},
		3, {0xE5, 0xA1, 0x80},
		2, {0x40, 0x58},
		2, {0xCA, 0xBD}
	},
	{
		2, {0x95, 0xBC},
		3, {0xE5, 0xB9, 0xA3},
		2, {0x63, 0x5E},
		2, {0xCA, 0xBE}
	},
	{
		2, {0x95, 0xBD},
		3, {0xE5, 0xB9, 0xB3},
		2, {0x73, 0x5E},
		2, {0xCA, 0xBF}
	},
	{
		2, {0x95, 0xBE},
		3, {0xE5, 0xBC, 0x8A},
		2, {0x0A, 0x5F},
		2, {0xCA, 0xC0}
	},
	{
		2, {0x95, 0xBF},
		3, {0xE6, 0x9F, 0x84},
		2, {0xC4, 0x67},
		2, {0xCA, 0xC1}
	},
	{
		2, {0x95, 0xC0},
		3, {0xE4, 0xB8, 0xA6},
		2, {0x26, 0x4E},
		2, {0xCA, 0xC2}
	},
	{
		2, {0x95, 0xC1},
		3, {0xE8, 0x94, 0xBD},
		2, {0x3D, 0x85},
		2, {0xCA, 0xC3}
	},
	{
		2, {0x95, 0xC2},
		3, {0xE9, 0x96, 0x89},
		2, {0x89, 0x95},
		2, {0xCA, 0xC4}
	},
	{
		2, {0x95, 0xC3},
		3, {0xE9, 0x99, 0x9B},
		2, {0x5B, 0x96},
		2, {0xCA, 0xC5}
	},
	{
		2, {0x95, 0xC4},
		3, {0xE7, 0xB1, 0xB3},
		2, {0x73, 0x7C},
		2, {0xCA, 0xC6}
	},
	{
		2, {0x95, 0xC5},
		3, {0xE9, 0xA0, 0x81},
		2, {0x01, 0x98},
		2, {0xCA, 0xC7}
	},
	{
		2, {0x95, 0xC6},
		3, {0xE5, 0x83, 0xBB},
		2, {0xFB, 0x50},
		2, {0xCA, 0xC8}
	},
	{
		2, {0x95, 0xC7},
		3, {0xE5, 0xA3, 0x81},
		2, {0xC1, 0x58},
		2, {0xCA, 0xC9}
	},
	{
		2, {0x95, 0xC8},
		3, {0xE7, 0x99, 0x96},
		2, {0x56, 0x76},
		2, {0xCA, 0xCA}
	},
	{
		2, {0x95, 0xC9},
		3, {0xE7, 0xA2, 0xA7},
		2, {0xA7, 0x78},
		2, {0xCA, 0xCB}
	},
	{
		2, {0x95, 0xCA},
		3, {0xE5, 0x88, 0xA5},
		2, {0x25, 0x52},
		2, {0xCA, 0xCC}
	},
	{
		2, {0x95, 0xCB},
		3, {0xE7, 0x9E, 0xA5},
		2, {0xA5, 0x77},
		2, {0xCA, 0xCD}
	},
	{
		2, {0x95, 0xCC},
		3, {0xE8, 0x94, 0x91},
		2, {0x11, 0x85},
		2, {0xCA, 0xCE}
	},
	{
		2, {0x95, 0xCD},
		3, {0xE7, 0xAE, 0x86},
		2, {0x86, 0x7B},
		2, {0xCA, 0xCF}
	},
	{
		2, {0x95, 0xCE},
		3, {0xE5, 0x81, 0x8F},
		2, {0x4F, 0x50},
		2, {0xCA, 0xD0}
	},
	{
		2, {0x95, 0xCF},
		3, {0xE5, 0xA4, 0x89},
		2, {0x09, 0x59},
		2, {0xCA, 0xD1}
	},
	{
		2, {0x95, 0xD0},
		3, {0xE7, 0x89, 0x87},
		2, {0x47, 0x72},
		2, {0xCA, 0xD2}
	},
	{
		2, {0x95, 0xD1},
		3, {0xE7, 0xAF, 0x87},
		2, {0xC7, 0x7B},
		2, {0xCA, 0xD3}
	},
	{
		2, {0x95, 0xD2},
		3, {0xE7, 0xB7, 0xA8},
		2, {0xE8, 0x7D},
		2, {0xCA, 0xD4}
	},
	{
		2, {0x95, 0xD3},
		3, {0xE8, 0xBE, 0xBA},
		2, {0xBA, 0x8F},
		2, {0xCA, 0xD5}
	},
	{
		2, {0x95, 0xD4},
		3, {0xE8, 0xBF, 0x94},
		2, {0xD4, 0x8F},
		2, {0xCA, 0xD6}
	},
	{
		2, {0x95, 0xD5},
		3, {0xE9, 0x81, 0x8D},
		2, {0x4D, 0x90},
		2, {0xCA, 0xD7}
	},
	{
		2, {0x95, 0xD6},
		3, {0xE4, 0xBE, 0xBF},
		2, {0xBF, 0x4F},
		2, {0xCA, 0xD8}
	},
	{
		2, {0x95, 0xD7},
		3, {0xE5, 0x8B, 0x89},
		2, {0xC9, 0x52},
		2, {0xCA, 0xD9}
	},
	{
		2, {0x95, 0xD8},
		3, {0xE5, 0xA8, 0xA9},
		2, {0x29, 0x5A},
		2, {0xCA, 0xDA}
	},
	{
		2, {0x95, 0xD9},
		3, {0xE5, 0xBC, 0x81},
		2, {0x01, 0x5F},
		2, {0xCA, 0xDB}
	},
	{
		2, {0x95, 0xDA},
		3, {0xE9, 0x9E, 0xAD},
		2, {0xAD, 0x97},
		2, {0xCA, 0xDC}
	},
	{
		2, {0x95, 0xDB},
		3, {0xE4, 0xBF, 0x9D},
		2, {0xDD, 0x4F},
		2, {0xCA, 0xDD}
	},
	{
		2, {0x95, 0xDC},
		3, {0xE8, 0x88, 0x97},
		2, {0x17, 0x82},
		2, {0xCA, 0xDE}
	},
	{
		2, {0x95, 0xDD},
		3, {0xE9, 0x8B, 0xAA},
		2, {0xEA, 0x92},
		2, {0xCA, 0xDF}
	},
	{
		2, {0x95, 0xDE},
		3, {0xE5, 0x9C, 0x83},
		2, {0x03, 0x57},
		2, {0xCA, 0xE0}
	},
	{
		2, {0x95, 0xDF},
		3, {0xE6, 0x8D, 0x95},
		2, {0x55, 0x63},
		2, {0xCA, 0xE1}
	},
	{
		2, {0x95, 0xE0},
		3, {0xE6, 0xAD, 0xA9},
		2, {0x69, 0x6B},
		2, {0xCA, 0xE2}
	},
	{
		2, {0x95, 0xE1},
		3, {0xE7, 0x94, 0xAB},
		2, {0x2B, 0x75},
		2, {0xCA, 0xE3}
	},
	{
		2, {0x95, 0xE2},
		3, {0xE8, 0xA3, 0x9C},
		2, {0xDC, 0x88},
		2, {0xCA, 0xE4}
	},
	{
		2, {0x95, 0xE3},
		3, {0xE8, 0xBC, 0x94},
		2, {0x14, 0x8F},
		2, {0xCA, 0xE5}
	},
	{
		2, {0x95, 0xE4},
		3, {0xE7, 0xA9, 0x82},
		2, {0x42, 0x7A},
		2, {0xCA, 0xE6}
	},
	{
		2, {0x95, 0xE5},
		3, {0xE5, 0x8B, 0x9F},
		2, {0xDF, 0x52},
		2, {0xCA, 0xE7}
	},
	{
		2, {0x95, 0xE6},
		3, {0xE5, 0xA2, 0x93},
		2, {0x93, 0x58},
		2, {0xCA, 0xE8}
	},
	{
		2, {0x95, 0xE7},
		3, {0xE6, 0x85, 0x95},
		2, {0x55, 0x61},
		2, {0xCA, 0xE9}
	},
	{
		2, {0x95, 0xE8},
		3, {0xE6, 0x88, 0x8A},
		2, {0x0A, 0x62},
		2, {0xCA, 0xEA}
	},
	{
		2, {0x95, 0xE9},
		3, {0xE6, 0x9A, 0xAE},
		2, {0xAE, 0x66},
		2, {0xCA, 0xEB}
	},
	{
		2, {0x95, 0xEA},
		3, {0xE6, 0xAF, 0x8D},
		2, {0xCD, 0x6B},
		2, {0xCA, 0xEC}
	},
	{
		2, {0x95, 0xEB},
		3, {0xE7, 0xB0, 0xBF},
		2, {0x3F, 0x7C},
		2, {0xCA, 0xED}
	},
	{
		2, {0x95, 0xEC},
		3, {0xE8, 0x8F, 0xA9},
		2, {0xE9, 0x83},
		2, {0xCA, 0xEE}
	},
	{
		2, {0x95, 0xED},
		3, {0xE5, 0x80, 0xA3},
		2, {0x23, 0x50},
		2, {0xCA, 0xEF}
	},
	{
		2, {0x95, 0xEE},
		3, {0xE4, 0xBF, 0xB8},
		2, {0xF8, 0x4F},
		2, {0xCA, 0xF0}
	},
	{
		2, {0x95, 0xEF},
		3, {0xE5, 0x8C, 0x85},
		2, {0x05, 0x53},
		2, {0xCA, 0xF1}
	},
	{
		2, {0x95, 0xF0},
		3, {0xE5, 0x91, 0x86},
		2, {0x46, 0x54},
		2, {0xCA, 0xF2}
	},
	{
		2, {0x95, 0xF1},
		3, {0xE5, 0xA0, 0xB1},
		2, {0x31, 0x58},
		2, {0xCA, 0xF3}
	},
	{
		2, {0x95, 0xF2},
		3, {0xE5, 0xA5, 0x89},
		2, {0x49, 0x59},
		2, {0xCA, 0xF4}
	},
	{
		2, {0x95, 0xF3},
		3, {0xE5, 0xAE, 0x9D},
		2, {0x9D, 0x5B},
		2, {0xCA, 0xF5}
	},
	{
		2, {0x95, 0xF4},
		3, {0xE5, 0xB3, 0xB0},
		2, {0xF0, 0x5C},
		2, {0xCA, 0xF6}
	},
	{
		2, {0x95, 0xF5},
		3, {0xE5, 0xB3, 0xAF},
		2, {0xEF, 0x5C},
		2, {0xCA, 0xF7}
	},
	{
		2, {0x95, 0xF6},
		3, {0xE5, 0xB4, 0xA9},
		2, {0x29, 0x5D},
		2, {0xCA, 0xF8}
	},
	{
		2, {0x95, 0xF7},
		3, {0xE5, 0xBA, 0x96},
		2, {0x96, 0x5E},
		2, {0xCA, 0xF9}
	},
	{
		2, {0x95, 0xF8},
		3, {0xE6, 0x8A, 0xB1},
		2, {0xB1, 0x62},
		2, {0xCA, 0xFA}
	},
	{
		2, {0x95, 0xF9},
		3, {0xE6, 0x8D, 0xA7},
		2, {0x67, 0x63},
		2, {0xCA, 0xFB}
	},
	{
		2, {0x95, 0xFA},
		3, {0xE6, 0x94, 0xBE},
		2, {0x3E, 0x65},
		2, {0xCA, 0xFC}
	},
	{
		2, {0x95, 0xFB},
		3, {0xE6, 0x96, 0xB9},
		2, {0xB9, 0x65},
		2, {0xCA, 0xFD}
	},
	{
		2, {0x95, 0xFC},
		3, {0xE6, 0x9C, 0x8B},
		2, {0x0B, 0x67},
		2, {0xCA, 0xFE}
	},
	{
		2, {0x96, 0x40},
		3, {0xE6, 0xB3, 0x95},
		2, {0xD5, 0x6C},
		2, {0xCB, 0xA1}
	},
	{
		2, {0x96, 0x41},
		3, {0xE6, 0xB3, 0xA1},
		2, {0xE1, 0x6C},
		2, {0xCB, 0xA2}
	},
	{
		2, {0x96, 0x42},
		3, {0xE7, 0x83, 0xB9},
		2, {0xF9, 0x70},
		2, {0xCB, 0xA3}
	},
	{
		2, {0x96, 0x43},
		3, {0xE7, 0xA0, 0xB2},
		2, {0x32, 0x78},
		2, {0xCB, 0xA4}
	},
	{
		2, {0x96, 0x44},
		3, {0xE7, 0xB8, 0xAB},
		2, {0x2B, 0x7E},
		2, {0xCB, 0xA5}
	},
	{
		2, {0x96, 0x45},
		3, {0xE8, 0x83, 0x9E},
		2, {0xDE, 0x80},
		2, {0xCB, 0xA6}
	},
	{
		2, {0x96, 0x46},
		3, {0xE8, 0x8A, 0xB3},
		2, {0xB3, 0x82},
		2, {0xCB, 0xA7}
	},
	{
		2, {0x96, 0x47},
		3, {0xE8, 0x90, 0x8C},
		2, {0x0C, 0x84},
		2, {0xCB, 0xA8}
	},
	{
		2, {0x96, 0x48},
		3, {0xE8, 0x93, 0xAC},
		2, {0xEC, 0x84},
		2, {0xCB, 0xA9}
	},
	{
		2, {0x96, 0x49},
		3, {0xE8, 0x9C, 0x82},
		2, {0x02, 0x87},
		2, {0xCB, 0xAA}
	},
	{
		2, {0x96, 0x4A},
		3, {0xE8, 0xA4, 0x92},
		2, {0x12, 0x89},
		2, {0xCB, 0xAB}
	},
	{
		2, {0x96, 0x4B},
		3, {0xE8, 0xA8, 0xAA},
		2, {0x2A, 0x8A},
		2, {0xCB, 0xAC}
	},
	{
		2, {0x96, 0x4C},
		3, {0xE8, 0xB1, 0x8A},
		2, {0x4A, 0x8C},
		2, {0xCB, 0xAD}
	},
	{
		2, {0x96, 0x4D},
		3, {0xE9, 0x82, 0xA6},
		2, {0xA6, 0x90},
		2, {0xCB, 0xAE}
	},
	{
		2, {0x96, 0x4E},
		3, {0xE9, 0x8B, 0x92},
		2, {0xD2, 0x92},
		2, {0xCB, 0xAF}
	},
	{
		2, {0x96, 0x4F},
		3, {0xE9, 0xA3, 0xBD},
		2, {0xFD, 0x98},
		2, {0xCB, 0xB0}
	},
	{
		2, {0x96, 0x50},
		3, {0xE9, 0xB3, 0xB3},
		2, {0xF3, 0x9C},
		2, {0xCB, 0xB1}
	},
	{
		2, {0x96, 0x51},
		3, {0xE9, 0xB5, 0xAC},
		2, {0x6C, 0x9D},
		2, {0xCB, 0xB2}
	},
	{
		2, {0x96, 0x52},
		3, {0xE4, 0xB9, 0x8F},
		2, {0x4F, 0x4E},
		2, {0xCB, 0xB3}
	},
	{
		2, {0x96, 0x53},
		3, {0xE4, 0xBA, 0xA1},
		2, {0xA1, 0x4E},
		2, {0xCB, 0xB4}
	},
	{
		2, {0x96, 0x54},
		3, {0xE5, 0x82, 0x8D},
		2, {0x8D, 0x50},
		2, {0xCB, 0xB5}
	},
	{
		2, {0x96, 0x55},
		3, {0xE5, 0x89, 0x96},
		2, {0x56, 0x52},
		2, {0xCB, 0xB6}
	},
	{
		2, {0x96, 0x56},
		3, {0xE5, 0x9D, 0x8A},
		2, {0x4A, 0x57},
		2, {0xCB, 0xB7}
	},
	{
		2, {0x96, 0x57},
		3, {0xE5, 0xA6, 0xA8},
		2, {0xA8, 0x59},
		2, {0xCB, 0xB8}
	},
	{
		2, {0x96, 0x58},
		3, {0xE5, 0xB8, 0xBD},
		2, {0x3D, 0x5E},
		2, {0xCB, 0xB9}
	},
	{
		2, {0x96, 0x59},
		3, {0xE5, 0xBF, 0x98},
		2, {0xD8, 0x5F},
		2, {0xCB, 0xBA}
	},
	{
		2, {0x96, 0x5A},
		3, {0xE5, 0xBF, 0x99},
		2, {0xD9, 0x5F},
		2, {0xCB, 0xBB}
	},
	{
		2, {0x96, 0x5B},
		3, {0xE6, 0x88, 0xBF},
		2, {0x3F, 0x62},
		2, {0xCB, 0xBC}
	},
	{
		2, {0x96, 0x5C},
		3, {0xE6, 0x9A, 0xB4},
		2, {0xB4, 0x66},
		2, {0xCB, 0xBD}
	},
	{
		2, {0x96, 0x5D},
		3, {0xE6, 0x9C, 0x9B},
		2, {0x1B, 0x67},
		2, {0xCB, 0xBE}
	},
	{
		2, {0x96, 0x5E},
		3, {0xE6, 0x9F, 0x90},
		2, {0xD0, 0x67},
		2, {0xCB, 0xBF}
	},
	{
		2, {0x96, 0x5F},
		3, {0xE6, 0xA3, 0x92},
		2, {0xD2, 0x68},
		2, {0xCB, 0xC0}
	},
	{
		2, {0x96, 0x60},
		3, {0xE5, 0x86, 0x92},
		2, {0x92, 0x51},
		2, {0xCB, 0xC1}
	},
	{
		2, {0x96, 0x61},
		3, {0xE7, 0xB4, 0xA1},
		2, {0x21, 0x7D},
		2, {0xCB, 0xC2}
	},
	{
		2, {0x96, 0x62},
		3, {0xE8, 0x82, 0xAA},
		2, {0xAA, 0x80},
		2, {0xCB, 0xC3}
	},
	{
		2, {0x96, 0x63},
		3, {0xE8, 0x86, 0xA8},
		2, {0xA8, 0x81},
		2, {0xCB, 0xC4}
	},
	{
		2, {0x96, 0x64},
		3, {0xE8, 0xAC, 0x80},
		2, {0x00, 0x8B},
		2, {0xCB, 0xC5}
	},
	{
		2, {0x96, 0x65},
		3, {0xE8, 0xB2, 0x8C},
		2, {0x8C, 0x8C},
		2, {0xCB, 0xC6}
	},
	{
		2, {0x96, 0x66},
		3, {0xE8, 0xB2, 0xBF},
		2, {0xBF, 0x8C},
		2, {0xCB, 0xC7}
	},
	{
		2, {0x96, 0x67},
		3, {0xE9, 0x89, 0xBE},
		2, {0x7E, 0x92},
		2, {0xCB, 0xC8}
	},
	{
		2, {0x96, 0x68},
		3, {0xE9, 0x98, 0xB2},
		2, {0x32, 0x96},
		2, {0xCB, 0xC9}
	},
	{
		2, {0x96, 0x69},
		3, {0xE5, 0x90, 0xA0},
		2, {0x20, 0x54},
		2, {0xCB, 0xCA}
	},
	{
		2, {0x96, 0x6A},
		3, {0xE9, 0xA0, 0xAC},
		2, {0x2C, 0x98},
		2, {0xCB, 0xCB}
	},
	{
		2, {0x96, 0x6B},
		3, {0xE5, 0x8C, 0x97},
		2, {0x17, 0x53},
		2, {0xCB, 0xCC}
	},
	{
		2, {0x96, 0x6C},
		3, {0xE5, 0x83, 0x95},
		2, {0xD5, 0x50},
		2, {0xCB, 0xCD}
	},
	{
		2, {0x96, 0x6D},
		3, {0xE5, 0x8D, 0x9C},
		2, {0x5C, 0x53},
		2, {0xCB, 0xCE}
	},
	{
		2, {0x96, 0x6E},
		3, {0xE5, 0xA2, 0xA8},
		2, {0xA8, 0x58},
		2, {0xCB, 0xCF}
	},
	{
		2, {0x96, 0x6F},
		3, {0xE6, 0x92, 0xB2},
		2, {0xB2, 0x64},
		2, {0xCB, 0xD0}
	},
	{
		2, {0x96, 0x70},
		3, {0xE6, 0x9C, 0xB4},
		2, {0x34, 0x67},
		2, {0xCB, 0xD1}
	},
	{
		2, {0x96, 0x71},
		3, {0xE7, 0x89, 0xA7},
		2, {0x67, 0x72},
		2, {0xCB, 0xD2}
	},
	{
		2, {0x96, 0x72},
		3, {0xE7, 0x9D, 0xA6},
		2, {0x66, 0x77},
		2, {0xCB, 0xD3}
	},
	{
		2, {0x96, 0x73},
		3, {0xE7, 0xA9, 0x86},
		2, {0x46, 0x7A},
		2, {0xCB, 0xD4}
	},
	{
		2, {0x96, 0x74},
		3, {0xE9, 0x87, 0xA6},
		2, {0xE6, 0x91},
		2, {0xCB, 0xD5}
	},
	{
		2, {0x96, 0x75},
		3, {0xE5, 0x8B, 0x83},
		2, {0xC3, 0x52},
		2, {0xCB, 0xD6}
	},
	{
		2, {0x96, 0x76},
		3, {0xE6, 0xB2, 0xA1},
		2, {0xA1, 0x6C},
		2, {0xCB, 0xD7}
	},
	{
		2, {0x96, 0x77},
		3, {0xE6, 0xAE, 0x86},
		2, {0x86, 0x6B},
		2, {0xCB, 0xD8}
	},
	{
		2, {0x96, 0x78},
		3, {0xE5, 0xA0, 0x80},
		2, {0x00, 0x58},
		2, {0xCB, 0xD9}
	},
	{
		2, {0x96, 0x79},
		3, {0xE5, 0xB9, 0x8C},
		2, {0x4C, 0x5E},
		2, {0xCB, 0xDA}
	},
	{
		2, {0x96, 0x7A},
		3, {0xE5, 0xA5, 0x94},
		2, {0x54, 0x59},
		2, {0xCB, 0xDB}
	},
	{
		2, {0x96, 0x7B},
		3, {0xE6, 0x9C, 0xAC},
		2, {0x2C, 0x67},
		2, {0xCB, 0xDC}
	},
	{
		2, {0x96, 0x7C},
		3, {0xE7, 0xBF, 0xBB},
		2, {0xFB, 0x7F},
		2, {0xCB, 0xDD}
	},
	{
		2, {0x96, 0x7D},
		3, {0xE5, 0x87, 0xA1},
		2, {0xE1, 0x51},
		2, {0xCB, 0xDE}
	},
	{
		2, {0x96, 0x7E},
		3, {0xE7, 0x9B, 0x86},
		2, {0xC6, 0x76},
		2, {0xCB, 0xDF}
	},
	{
		2, {0x96, 0x80},
		3, {0xE6, 0x91, 0xA9},
		2, {0x69, 0x64},
		2, {0xCB, 0xE0}
	},
	{
		2, {0x96, 0x81},
		3, {0xE7, 0xA3, 0xA8},
		2, {0xE8, 0x78},
		2, {0xCB, 0xE1}
	},
	{
		2, {0x96, 0x82},
		3, {0xE9, 0xAD, 0x94},
		2, {0x54, 0x9B},
		2, {0xCB, 0xE2}
	},
	{
		2, {0x96, 0x83},
		3, {0xE9, 0xBA, 0xBB},
		2, {0xBB, 0x9E},
		2, {0xCB, 0xE3}
	},
	{
		2, {0x96, 0x84},
		3, {0xE5, 0x9F, 0x8B},
		2, {0xCB, 0x57},
		2, {0xCB, 0xE4}
	},
	{
		2, {0x96, 0x85},
		3, {0xE5, 0xA6, 0xB9},
		2, {0xB9, 0x59},
		2, {0xCB, 0xE5}
	},
	{
		2, {0x96, 0x86},
		3, {0xE6, 0x98, 0xA7},
		2, {0x27, 0x66},
		2, {0xCB, 0xE6}
	},
	{
		2, {0x96, 0x87},
		3, {0xE6, 0x9E, 0x9A},
		2, {0x9A, 0x67},
		2, {0xCB, 0xE7}
	},
	{
		2, {0x96, 0x88},
		3, {0xE6, 0xAF, 0x8E},
		2, {0xCE, 0x6B},
		2, {0xCB, 0xE8}
	},
	{
		2, {0x96, 0x89},
		3, {0xE5, 0x93, 0xA9},
		2, {0xE9, 0x54},
		2, {0xCB, 0xE9}
	},
	{
		2, {0x96, 0x8A},
		3, {0xE6, 0xA7, 0x99},
		2, {0xD9, 0x69},
		2, {0xCB, 0xEA}
	},
	{
		2, {0x96, 0x8B},
		3, {0xE5, 0xB9, 0x95},
		2, {0x55, 0x5E},
		2, {0xCB, 0xEB}
	},
	{
		2, {0x96, 0x8C},
		3, {0xE8, 0x86, 0x9C},
		2, {0x9C, 0x81},
		2, {0xCB, 0xEC}
	},
	{
		2, {0x96, 0x8D},
		3, {0xE6, 0x9E, 0x95},
		2, {0x95, 0x67},
		2, {0xCB, 0xED}
	},
	{
		2, {0x96, 0x8E},
		3, {0xE9, 0xAE, 0xAA},
		2, {0xAA, 0x9B},
		2, {0xCB, 0xEE}
	},
	{
		2, {0x96, 0x8F},
		3, {0xE6, 0x9F, 0xBE},
		2, {0xFE, 0x67},
		2, {0xCB, 0xEF}
	},
	{
		2, {0x96, 0x90},
		3, {0xE9, 0xB1, 0x92},
		2, {0x52, 0x9C},
		2, {0xCB, 0xF0}
	},
	{
		2, {0x96, 0x91},
		3, {0xE6, 0xA1, 0x9D},
		2, {0x5D, 0x68},
		2, {0xCB, 0xF1}
	},
	{
		2, {0x96, 0x92},
		3, {0xE4, 0xBA, 0xA6},
		2, {0xA6, 0x4E},
		2, {0xCB, 0xF2}
	},
	{
		2, {0x96, 0x93},
		3, {0xE4, 0xBF, 0xA3},
		2, {0xE3, 0x4F},
		2, {0xCB, 0xF3}
	},
	{
		2, {0x96, 0x94},
		3, {0xE5, 0x8F, 0x88},
		2, {0xC8, 0x53},
		2, {0xCB, 0xF4}
	},
	{
		2, {0x96, 0x95},
		3, {0xE6, 0x8A, 0xB9},
		2, {0xB9, 0x62},
		2, {0xCB, 0xF5}
	},
	{
		2, {0x96, 0x96},
		3, {0xE6, 0x9C, 0xAB},
		2, {0x2B, 0x67},
		2, {0xCB, 0xF6}
	},
	{
		2, {0x96, 0x97},
		3, {0xE6, 0xB2, 0xAB},
		2, {0xAB, 0x6C},
		2, {0xCB, 0xF7}
	},
	{
		2, {0x96, 0x98},
		3, {0xE8, 0xBF, 0x84},
		2, {0xC4, 0x8F},
		2, {0xCB, 0xF8}
	},
	{
		2, {0x96, 0x99},
		3, {0xE4, 0xBE, 0xAD},
		2, {0xAD, 0x4F},
		2, {0xCB, 0xF9}
	},
	{
		2, {0x96, 0x9A},
		3, {0xE7, 0xB9, 0xAD},
		2, {0x6D, 0x7E},
		2, {0xCB, 0xFA}
	},
	{
		2, {0x96, 0x9B},
		3, {0xE9, 0xBA, 0xBF},
		2, {0xBF, 0x9E},
		2, {0xCB, 0xFB}
	},
	{
		2, {0x96, 0x9C},
		3, {0xE4, 0xB8, 0x87},
		2, {0x07, 0x4E},
		2, {0xCB, 0xFC}
	},
	{
		2, {0x96, 0x9D},
		3, {0xE6, 0x85, 0xA2},
		2, {0x62, 0x61},
		2, {0xCB, 0xFD}
	},
	{
		2, {0x96, 0x9E},
		3, {0xE6, 0xBA, 0x80},
		2, {0x80, 0x6E},
		2, {0xCB, 0xFE}
	},
	{
		2, {0x96, 0x9F},
		3, {0xE6, 0xBC, 0xAB},
		2, {0x2B, 0x6F},
		2, {0xCC, 0xA1}
	},
	{
		2, {0x96, 0xA0},
		3, {0xE8, 0x94, 0x93},
		2, {0x13, 0x85},
		2, {0xCC, 0xA2}
	},
	{
		2, {0x96, 0xA1},
		3, {0xE5, 0x91, 0xB3},
		2, {0x73, 0x54},
		2, {0xCC, 0xA3}
	},
	{
		2, {0x96, 0xA2},
		3, {0xE6, 0x9C, 0xAA},
		2, {0x2A, 0x67},
		2, {0xCC, 0xA4}
	},
	{
		2, {0x96, 0xA3},
		3, {0xE9, 0xAD, 0x85},
		2, {0x45, 0x9B},
		2, {0xCC, 0xA5}
	},
	{
		2, {0x96, 0xA4},
		3, {0xE5, 0xB7, 0xB3},
		2, {0xF3, 0x5D},
		2, {0xCC, 0xA6}
	},
	{
		2, {0x96, 0xA5},
		3, {0xE7, 0xAE, 0x95},
		2, {0x95, 0x7B},
		2, {0xCC, 0xA7}
	},
	{
		2, {0x96, 0xA6},
		3, {0xE5, 0xB2, 0xAC},
		2, {0xAC, 0x5C},
		2, {0xCC, 0xA8}
	},
	{
		2, {0x96, 0xA7},
		3, {0xE5, 0xAF, 0x86},
		2, {0xC6, 0x5B},
		2, {0xCC, 0xA9}
	},
	{
		2, {0x96, 0xA8},
		3, {0xE8, 0x9C, 0x9C},
		2, {0x1C, 0x87},
		2, {0xCC, 0xAA}
	},
	{
		2, {0x96, 0xA9},
		3, {0xE6, 0xB9, 0x8A},
		2, {0x4A, 0x6E},
		2, {0xCC, 0xAB}
	},
	{
		2, {0x96, 0xAA},
		3, {0xE8, 0x93, 0x91},
		2, {0xD1, 0x84},
		2, {0xCC, 0xAC}
	},
	{
		2, {0x96, 0xAB},
		3, {0xE7, 0xA8, 0x94},
		2, {0x14, 0x7A},
		2, {0xCC, 0xAD}
	},
	{
		2, {0x96, 0xAC},
		3, {0xE8, 0x84, 0x88},
		2, {0x08, 0x81},
		2, {0xCC, 0xAE}
	},
	{
		2, {0x96, 0xAD},
		3, {0xE5, 0xA6, 0x99},
		2, {0x99, 0x59},
		2, {0xCC, 0xAF}
	},
	{
		2, {0x96, 0xAE},
		3, {0xE7, 0xB2, 0x8D},
		2, {0x8D, 0x7C},
		2, {0xCC, 0xB0}
	},
	{
		2, {0x96, 0xAF},
		3, {0xE6, 0xB0, 0x91},
		2, {0x11, 0x6C},
		2, {0xCC, 0xB1}
	},
	{
		2, {0x96, 0xB0},
		3, {0xE7, 0x9C, 0xA0},
		2, {0x20, 0x77},
		2, {0xCC, 0xB2}
	},
	{
		2, {0x96, 0xB1},
		3, {0xE5, 0x8B, 0x99},
		2, {0xD9, 0x52},
		2, {0xCC, 0xB3}
	},
	{
		2, {0x96, 0xB2},
		3, {0xE5, 0xA4, 0xA2},
		2, {0x22, 0x59},
		2, {0xCC, 0xB4}
	},
	{
		2, {0x96, 0xB3},
		3, {0xE7, 0x84, 0xA1},
		2, {0x21, 0x71},
		2, {0xCC, 0xB5}
	},
	{
		2, {0x96, 0xB4},
		3, {0xE7, 0x89, 0x9F},
		2, {0x5F, 0x72},
		2, {0xCC, 0xB6}
	},
	{
		2, {0x96, 0xB5},
		3, {0xE7, 0x9F, 0x9B},
		2, {0xDB, 0x77},
		2, {0xCC, 0xB7}
	},
	{
		2, {0x96, 0xB6},
		3, {0xE9, 0x9C, 0xA7},
		2, {0x27, 0x97},
		2, {0xCC, 0xB8}
	},
	{
		2, {0x96, 0xB7},
		3, {0xE9, 0xB5, 0xA1},
		2, {0x61, 0x9D},
		2, {0xCC, 0xB9}
	},
	{
		2, {0x96, 0xB8},
		3, {0xE6, 0xA4, 0x8B},
		2, {0x0B, 0x69},
		2, {0xCC, 0xBA}
	},
	{
		2, {0x96, 0xB9},
		3, {0xE5, 0xA9, 0xBF},
		2, {0x7F, 0x5A},
		2, {0xCC, 0xBB}
	},
	{
		2, {0x96, 0xBA},
		3, {0xE5, 0xA8, 0x98},
		2, {0x18, 0x5A},
		2, {0xCC, 0xBC}
	},
	{
		2, {0x96, 0xBB},
		3, {0xE5, 0x86, 0xA5},
		2, {0xA5, 0x51},
		2, {0xCC, 0xBD}
	},
	{
		2, {0x96, 0xBC},
		3, {0xE5, 0x90, 0x8D},
		2, {0x0D, 0x54},
		2, {0xCC, 0xBE}
	},
	{
		2, {0x96, 0xBD},
		3, {0xE5, 0x91, 0xBD},
		2, {0x7D, 0x54},
		2, {0xCC, 0xBF}
	},
	{
		2, {0x96, 0xBE},
		3, {0xE6, 0x98, 0x8E},
		2, {0x0E, 0x66},
		2, {0xCC, 0xC0}
	},
	{
		2, {0x96, 0xBF},
		3, {0xE7, 0x9B, 0x9F},
		2, {0xDF, 0x76},
		2, {0xCC, 0xC1}
	},
	{
		2, {0x96, 0xC0},
		3, {0xE8, 0xBF, 0xB7},
		2, {0xF7, 0x8F},
		2, {0xCC, 0xC2}
	},
	{
		2, {0x96, 0xC1},
		3, {0xE9, 0x8A, 0x98},
		2, {0x98, 0x92},
		2, {0xCC, 0xC3}
	},
	{
		2, {0x96, 0xC2},
		3, {0xE9, 0xB3, 0xB4},
		2, {0xF4, 0x9C},
		2, {0xCC, 0xC4}
	},
	{
		2, {0x96, 0xC3},
		3, {0xE5, 0xA7, 0xAA},
		2, {0xEA, 0x59},
		2, {0xCC, 0xC5}
	},
	{
		2, {0x96, 0xC4},
		3, {0xE7, 0x89, 0x9D},
		2, {0x5D, 0x72},
		2, {0xCC, 0xC6}
	},
	{
		2, {0x96, 0xC5},
		3, {0xE6, 0xBB, 0x85},
		2, {0xC5, 0x6E},
		2, {0xCC, 0xC7}
	},
	{
		2, {0x96, 0xC6},
		3, {0xE5, 0x85, 0x8D},
		2, {0x4D, 0x51},
		2, {0xCC, 0xC8}
	},
	{
		2, {0x96, 0xC7},
		3, {0xE6, 0xA3, 0x89},
		2, {0xC9, 0x68},
		2, {0xCC, 0xC9}
	},
	{
		2, {0x96, 0xC8},
		3, {0xE7, 0xB6, 0xBF},
		2, {0xBF, 0x7D},
		2, {0xCC, 0xCA}
	},
	{
		2, {0x96, 0xC9},
		3, {0xE7, 0xB7, 0xAC},
		2, {0xEC, 0x7D},
		2, {0xCC, 0xCB}
	},
	{
		2, {0x96, 0xCA},
		3, {0xE9, 0x9D, 0xA2},
		2, {0x62, 0x97},
		2, {0xCC, 0xCC}
	},
	{
		2, {0x96, 0xCB},
		3, {0xE9, 0xBA, 0xBA},
		2, {0xBA, 0x9E},
		2, {0xCC, 0xCD}
	},
	{
		2, {0x96, 0xCC},
		3, {0xE6, 0x91, 0xB8},
		2, {0x78, 0x64},
		2, {0xCC, 0xCE}
	},
	{
		2, {0x96, 0xCD},
		3, {0xE6, 0xA8, 0xA1},
		2, {0x21, 0x6A},
		2, {0xCC, 0xCF}
	},
	{
		2, {0x96, 0xCE},
		3, {0xE8, 0x8C, 0x82},
		2, {0x02, 0x83},
		2, {0xCC, 0xD0}
	},
	{
		2, {0x96, 0xCF},
		3, {0xE5, 0xA6, 0x84},
		2, {0x84, 0x59},
		2, {0xCC, 0xD1}
	},
	{
		2, {0x96, 0xD0},
		3, {0xE5, 0xAD, 0x9F},
		2, {0x5F, 0x5B},
		2, {0xCC, 0xD2}
	},
	{
		2, {0x96, 0xD1},
		3, {0xE6, 0xAF, 0x9B},
		2, {0xDB, 0x6B},
		2, {0xCC, 0xD3}
	},
	{
		2, {0x96, 0xD2},
		3, {0xE7, 0x8C, 0x9B},
		2, {0x1B, 0x73},
		2, {0xCC, 0xD4}
	},
	{
		2, {0x96, 0xD3},
		3, {0xE7, 0x9B, 0xB2},
		2, {0xF2, 0x76},
		2, {0xCC, 0xD5}
	},
	{
		2, {0x96, 0xD4},
		3, {0xE7, 0xB6, 0xB2},
		2, {0xB2, 0x7D},
		2, {0xCC, 0xD6}
	},
	{
		2, {0x96, 0xD5},
		3, {0xE8, 0x80, 0x97},
		2, {0x17, 0x80},
		2, {0xCC, 0xD7}
	},
	{
		2, {0x96, 0xD6},
		3, {0xE8, 0x92, 0x99},
		2, {0x99, 0x84},
		2, {0xCC, 0xD8}
	},
	{
		2, {0x96, 0xD7},
		3, {0xE5, 0x84, 0xB2},
		2, {0x32, 0x51},
		2, {0xCC, 0xD9}
	},
	{
		2, {0x96, 0xD8},
		3, {0xE6, 0x9C, 0xA8},
		2, {0x28, 0x67},
		2, {0xCC, 0xDA}
	},
	{
		2, {0x96, 0xD9},
		3, {0xE9, 0xBB, 0x99},
		2, {0xD9, 0x9E},
		2, {0xCC, 0xDB}
	},
	{
		2, {0x96, 0xDA},
		3, {0xE7, 0x9B, 0xAE},
		2, {0xEE, 0x76},
		2, {0xCC, 0xDC}
	},
	{
		2, {0x96, 0xDB},
		3, {0xE6, 0x9D, 0xA2},
		2, {0x62, 0x67},
		2, {0xCC, 0xDD}
	},
	{
		2, {0x96, 0xDC},
		3, {0xE5, 0x8B, 0xBF},
		2, {0xFF, 0x52},
		2, {0xCC, 0xDE}
	},
	{
		2, {0x96, 0xDD},
		3, {0xE9, 0xA4, 0x85},
		2, {0x05, 0x99},
		2, {0xCC, 0xDF}
	},
	{
		2, {0x96, 0xDE},
		3, {0xE5, 0xB0, 0xA4},
		2, {0x24, 0x5C},
		2, {0xCC, 0xE0}
	},
	{
		2, {0x96, 0xDF},
		3, {0xE6, 0x88, 0xBB},
		2, {0x3B, 0x62},
		2, {0xCC, 0xE1}
	},
	{
		2, {0x96, 0xE0},
		3, {0xE7, 0xB1, 0xBE},
		2, {0x7E, 0x7C},
		2, {0xCC, 0xE2}
	},
	{
		2, {0x96, 0xE1},
		3, {0xE8, 0xB2, 0xB0},
		2, {0xB0, 0x8C},
		2, {0xCC, 0xE3}
	},
	{
		2, {0x96, 0xE2},
		3, {0xE5, 0x95, 0x8F},
		2, {0x4F, 0x55},
		2, {0xCC, 0xE4}
	},
	{
		2, {0x96, 0xE3},
		3, {0xE6, 0x82, 0xB6},
		2, {0xB6, 0x60},
		2, {0xCC, 0xE5}
	},
	{
		2, {0x96, 0xE4},
		3, {0xE7, 0xB4, 0x8B},
		2, {0x0B, 0x7D},
		2, {0xCC, 0xE6}
	},
	{
		2, {0x96, 0xE5},
		3, {0xE9, 0x96, 0x80},
		2, {0x80, 0x95},
		2, {0xCC, 0xE7}
	},
	{
		2, {0x96, 0xE6},
		3, {0xE5, 0x8C, 0x81},
		2, {0x01, 0x53},
		2, {0xCC, 0xE8}
	},
	{
		2, {0x96, 0xE7},
		3, {0xE4, 0xB9, 0x9F},
		2, {0x5F, 0x4E},
		2, {0xCC, 0xE9}
	},
	{
		2, {0x96, 0xE8},
		3, {0xE5, 0x86, 0xB6},
		2, {0xB6, 0x51},
		2, {0xCC, 0xEA}
	},
	{
		2, {0x96, 0xE9},
		3, {0xE5, 0xA4, 0x9C},
		2, {0x1C, 0x59},
		2, {0xCC, 0xEB}
	},
	{
		2, {0x96, 0xEA},
		3, {0xE7, 0x88, 0xBA},
		2, {0x3A, 0x72},
		2, {0xCC, 0xEC}
	},
	{
		2, {0x96, 0xEB},
		3, {0xE8, 0x80, 0xB6},
		2, {0x36, 0x80},
		2, {0xCC, 0xED}
	},
	{
		2, {0x96, 0xEC},
		3, {0xE9, 0x87, 0x8E},
		2, {0xCE, 0x91},
		2, {0xCC, 0xEE}
	},
	{
		2, {0x96, 0xED},
		3, {0xE5, 0xBC, 0xA5},
		2, {0x25, 0x5F},
		2, {0xCC, 0xEF}
	},
	{
		2, {0x96, 0xEE},
		3, {0xE7, 0x9F, 0xA2},
		2, {0xE2, 0x77},
		2, {0xCC, 0xF0}
	},
	{
		2, {0x96, 0xEF},
		3, {0xE5, 0x8E, 0x84},
		2, {0x84, 0x53},
		2, {0xCC, 0xF1}
	},
	{
		2, {0x96, 0xF0},
		3, {0xE5, 0xBD, 0xB9},
		2, {0x79, 0x5F},
		2, {0xCC, 0xF2}
	},
	{
		2, {0x96, 0xF1},
		3, {0xE7, 0xB4, 0x84},
		2, {0x04, 0x7D},
		2, {0xCC, 0xF3}
	},
	{
		2, {0x96, 0xF2},
		3, {0xE8, 0x96, 0xAC},
		2, {0xAC, 0x85},
		2, {0xCC, 0xF4}
	},
	{
		2, {0x96, 0xF3},
		3, {0xE8, 0xA8, 0xB3},
		2, {0x33, 0x8A},
		2, {0xCC, 0xF5}
	},
	{
		2, {0x96, 0xF4},
		3, {0xE8, 0xBA, 0x8D},
		2, {0x8D, 0x8E},
		2, {0xCC, 0xF6}
	},
	{
		2, {0x96, 0xF5},
		3, {0xE9, 0x9D, 0x96},
		2, {0x56, 0x97},
		2, {0xCC, 0xF7}
	},
	{
		2, {0x96, 0xF6},
		3, {0xE6, 0x9F, 0xB3},
		2, {0xF3, 0x67},
		2, {0xCC, 0xF8}
	},
	{
		2, {0x96, 0xF7},
		3, {0xE8, 0x96, 0xAE},
		2, {0xAE, 0x85},
		2, {0xCC, 0xF9}
	},
	{
		2, {0x96, 0xF8},
		3, {0xE9, 0x91, 0x93},
		2, {0x53, 0x94},
		2, {0xCC, 0xFA}
	},
	{
		2, {0x96, 0xF9},
		3, {0xE6, 0x84, 0x89},
		2, {0x09, 0x61},
		2, {0xCC, 0xFB}
	},
	{
		2, {0x96, 0xFA},
		3, {0xE6, 0x84, 0x88},
		2, {0x08, 0x61},
		2, {0xCC, 0xFC}
	},
	{
		2, {0x96, 0xFB},
		3, {0xE6, 0xB2, 0xB9},
		2, {0xB9, 0x6C},
		2, {0xCC, 0xFD}
	},
	{
		2, {0x96, 0xFC},
		3, {0xE7, 0x99, 0x92},
		2, {0x52, 0x76},
		2, {0xCC, 0xFE}
	},
	{
		2, {0x97, 0x40},
		3, {0xE8, 0xAB, 0xAD},
		2, {0xED, 0x8A},
		2, {0xCD, 0xA1}
	},
	{
		2, {0x97, 0x41},
		3, {0xE8, 0xBC, 0xB8},
		2, {0x38, 0x8F},
		2, {0xCD, 0xA2}
	},
	{
		2, {0x97, 0x42},
		3, {0xE5, 0x94, 0xAF},
		2, {0x2F, 0x55},
		2, {0xCD, 0xA3}
	},
	{
		2, {0x97, 0x43},
		3, {0xE4, 0xBD, 0x91},
		2, {0x51, 0x4F},
		2, {0xCD, 0xA4}
	},
	{
		2, {0x97, 0x44},
		3, {0xE5, 0x84, 0xAA},
		2, {0x2A, 0x51},
		2, {0xCD, 0xA5}
	},
	{
		2, {0x97, 0x45},
		3, {0xE5, 0x8B, 0x87},
		2, {0xC7, 0x52},
		2, {0xCD, 0xA6}
	},
	{
		2, {0x97, 0x46},
		3, {0xE5, 0x8F, 0x8B},
		2, {0xCB, 0x53},
		2, {0xCD, 0xA7}
	},
	{
		2, {0x97, 0x47},
		3, {0xE5, 0xAE, 0xA5},
		2, {0xA5, 0x5B},
		2, {0xCD, 0xA8}
	},
	{
		2, {0x97, 0x48},
		3, {0xE5, 0xB9, 0xBD},
		2, {0x7D, 0x5E},
		2, {0xCD, 0xA9}
	},
	{
		2, {0x97, 0x49},
		3, {0xE6, 0x82, 0xA0},
		2, {0xA0, 0x60},
		2, {0xCD, 0xAA}
	},
	{
		2, {0x97, 0x4A},
		3, {0xE6, 0x86, 0x82},
		2, {0x82, 0x61},
		2, {0xCD, 0xAB}
	},
	{
		2, {0x97, 0x4B},
		3, {0xE6, 0x8F, 0x96},
		2, {0xD6, 0x63},
		2, {0xCD, 0xAC}
	},
	{
		2, {0x97, 0x4C},
		3, {0xE6, 0x9C, 0x89},
		2, {0x09, 0x67},
		2, {0xCD, 0xAD}
	},
	{
		2, {0x97, 0x4D},
		3, {0xE6, 0x9F, 0x9A},
		2, {0xDA, 0x67},
		2, {0xCD, 0xAE}
	},
	{
		2, {0x97, 0x4E},
		3, {0xE6, 0xB9, 0xA7},
		2, {0x67, 0x6E},
		2, {0xCD, 0xAF}
	},
	{
		2, {0x97, 0x4F},
		3, {0xE6, 0xB6, 0x8C},
		2, {0x8C, 0x6D},
		2, {0xCD, 0xB0}
	},
	{
		2, {0x97, 0x50},
		3, {0xE7, 0x8C, 0xB6},
		2, {0x36, 0x73},
		2, {0xCD, 0xB1}
	},
	{
		2, {0x97, 0x51},
		3, {0xE7, 0x8C, 0xB7},
		2, {0x37, 0x73},
		2, {0xCD, 0xB2}
	},
	{
		2, {0x97, 0x52},
		3, {0xE7, 0x94, 0xB1},
		2, {0x31, 0x75},
		2, {0xCD, 0xB3}
	},
	{
		2, {0x97, 0x53},
		3, {0xE7, 0xA5, 0x90},
		2, {0x50, 0x79},
		2, {0xCD, 0xB4}
	},
	{
		2, {0x97, 0x54},
		3, {0xE8, 0xA3, 0x95},
		2, {0xD5, 0x88},
		2, {0xCD, 0xB5}
	},
	{
		2, {0x97, 0x55},
		3, {0xE8, 0xAA, 0x98},
		2, {0x98, 0x8A},
		2, {0xCD, 0xB6}
	},
	{
		2, {0x97, 0x56},
		3, {0xE9, 0x81, 0x8A},
		2, {0x4A, 0x90},
		2, {0xCD, 0xB7}
	},
	{
		2, {0x97, 0x57},
		3, {0xE9, 0x82, 0x91},
		2, {0x91, 0x90},
		2, {0xCD, 0xB8}
	},
	{
		2, {0x97, 0x58},
		3, {0xE9, 0x83, 0xB5},
		2, {0xF5, 0x90},
		2, {0xCD, 0xB9}
	},
	{
		2, {0x97, 0x59},
		3, {0xE9, 0x9B, 0x84},
		2, {0xC4, 0x96},
		2, {0xCD, 0xBA}
	},
	{
		2, {0x97, 0x5A},
		3, {0xE8, 0x9E, 0x8D},
		2, {0x8D, 0x87},
		2, {0xCD, 0xBB}
	},
	{
		2, {0x97, 0x5B},
		3, {0xE5, 0xA4, 0x95},
		2, {0x15, 0x59},
		2, {0xCD, 0xBC}
	},
	{
		2, {0x97, 0x5C},
		3, {0xE4, 0xBA, 0x88},
		2, {0x88, 0x4E},
		2, {0xCD, 0xBD}
	},
	{
		2, {0x97, 0x5D},
		3, {0xE4, 0xBD, 0x99},
		2, {0x59, 0x4F},
		2, {0xCD, 0xBE}
	},
	{
		2, {0x97, 0x5E},
		3, {0xE4, 0xB8, 0x8E},
		2, {0x0E, 0x4E},
		2, {0xCD, 0xBF}
	},
	{
		2, {0x97, 0x5F},
		3, {0xE8, 0xAA, 0x89},
		2, {0x89, 0x8A},
		2, {0xCD, 0xC0}
	},
	{
		2, {0x97, 0x60},
		3, {0xE8, 0xBC, 0xBF},
		2, {0x3F, 0x8F},
		2, {0xCD, 0xC1}
	},
	{
		2, {0x97, 0x61},
		3, {0xE9, 0xA0, 0x90},
		2, {0x10, 0x98},
		2, {0xCD, 0xC2}
	},
	{
		2, {0x97, 0x62},
		3, {0xE5, 0x82, 0xAD},
		2, {0xAD, 0x50},
		2, {0xCD, 0xC3}
	},
	{
		2, {0x97, 0x63},
		3, {0xE5, 0xB9, 0xBC},
		2, {0x7C, 0x5E},
		2, {0xCD, 0xC4}
	},
	{
		2, {0x97, 0x64},
		3, {0xE5, 0xA6, 0x96},
		2, {0x96, 0x59},
		2, {0xCD, 0xC5}
	},
	{
		2, {0x97, 0x65},
		3, {0xE5, 0xAE, 0xB9},
		2, {0xB9, 0x5B},
		2, {0xCD, 0xC6}
	},
	{
		2, {0x97, 0x66},
		3, {0xE5, 0xBA, 0xB8},
		2, {0xB8, 0x5E},
		2, {0xCD, 0xC7}
	},
	{
		2, {0x97, 0x67},
		3, {0xE6, 0x8F, 0x9A},
		2, {0xDA, 0x63},
		2, {0xCD, 0xC8}
	},
	{
		2, {0x97, 0x68},
		3, {0xE6, 0x8F, 0xBA},
		2, {0xFA, 0x63},
		2, {0xCD, 0xC9}
	},
	{
		2, {0x97, 0x69},
		3, {0xE6, 0x93, 0x81},
		2, {0xC1, 0x64},
		2, {0xCD, 0xCA}
	},
	{
		2, {0x97, 0x6A},
		3, {0xE6, 0x9B, 0x9C},
		2, {0xDC, 0x66},
		2, {0xCD, 0xCB}
	},
	{
		2, {0x97, 0x6B},
		3, {0xE6, 0xA5, 0x8A},
		2, {0x4A, 0x69},
		2, {0xCD, 0xCC}
	},
	{
		2, {0x97, 0x6C},
		3, {0xE6, 0xA7, 0x98},
		2, {0xD8, 0x69},
		2, {0xCD, 0xCD}
	},
	{
		2, {0x97, 0x6D},
		3, {0xE6, 0xB4, 0x8B},
		2, {0x0B, 0x6D},
		2, {0xCD, 0xCE}
	},
	{
		2, {0x97, 0x6E},
		3, {0xE6, 0xBA, 0xB6},
		2, {0xB6, 0x6E},
		2, {0xCD, 0xCF}
	},
	{
		2, {0x97, 0x6F},
		3, {0xE7, 0x86, 0x94},
		2, {0x94, 0x71},
		2, {0xCD, 0xD0}
	},
	{
		2, {0x97, 0x70},
		3, {0xE7, 0x94, 0xA8},
		2, {0x28, 0x75},
		2, {0xCD, 0xD1}
	},
	{
		2, {0x97, 0x71},
		3, {0xE7, 0xAA, 0xAF},
		2, {0xAF, 0x7A},
		2, {0xCD, 0xD2}
	},
	{
		2, {0x97, 0x72},
		3, {0xE7, 0xBE, 0x8A},
		2, {0x8A, 0x7F},
		2, {0xCD, 0xD3}
	},
	{
		2, {0x97, 0x73},
		3, {0xE8, 0x80, 0x80},
		2, {0x00, 0x80},
		2, {0xCD, 0xD4}
	},
	{
		2, {0x97, 0x74},
		3, {0xE8, 0x91, 0x89},
		2, {0x49, 0x84},
		2, {0xCD, 0xD5}
	},
	{
		2, {0x97, 0x75},
		3, {0xE8, 0x93, 0x89},
		2, {0xC9, 0x84},
		2, {0xCD, 0xD6}
	},
	{
		2, {0x97, 0x76},
		3, {0xE8, 0xA6, 0x81},
		2, {0x81, 0x89},
		2, {0xCD, 0xD7}
	},
	{
		2, {0x97, 0x77},
		3, {0xE8, 0xAC, 0xA1},
		2, {0x21, 0x8B},
		2, {0xCD, 0xD8}
	},
	{
		2, {0x97, 0x78},
		3, {0xE8, 0xB8, 0x8A},
		2, {0x0A, 0x8E},
		2, {0xCD, 0xD9}
	},
	{
		2, {0x97, 0x79},
		3, {0xE9, 0x81, 0xA5},
		2, {0x65, 0x90},
		2, {0xCD, 0xDA}
	},
	{
		2, {0x97, 0x7A},
		3, {0xE9, 0x99, 0xBD},
		2, {0x7D, 0x96},
		2, {0xCD, 0xDB}
	},
	{
		2, {0x97, 0x7B},
		3, {0xE9, 0xA4, 0x8A},
		2, {0x0A, 0x99},
		2, {0xCD, 0xDC}
	},
	{
		2, {0x97, 0x7C},
		3, {0xE6, 0x85, 0xBE},
		2, {0x7E, 0x61},
		2, {0xCD, 0xDD}
	},
	{
		2, {0x97, 0x7D},
		3, {0xE6, 0x8A, 0x91},
		2, {0x91, 0x62},
		2, {0xCD, 0xDE}
	},
	{
		2, {0x97, 0x7E},
		3, {0xE6, 0xAC, 0xB2},
		2, {0x32, 0x6B},
		2, {0xCD, 0xDF}
	},
	{
		2, {0x97, 0x80},
		3, {0xE6, 0xB2, 0x83},
		2, {0x83, 0x6C},
		2, {0xCD, 0xE0}
	},
	{
		2, {0x97, 0x81},
		3, {0xE6, 0xB5, 0xB4},
		2, {0x74, 0x6D},
		2, {0xCD, 0xE1}
	},
	{
		2, {0x97, 0x82},
		3, {0xE7, 0xBF, 0x8C},
		2, {0xCC, 0x7F},
		2, {0xCD, 0xE2}
	},
	{
		2, {0x97, 0x83},
		3, {0xE7, 0xBF, 0xBC},
		2, {0xFC, 0x7F},
		2, {0xCD, 0xE3}
	},
	{
		2, {0x97, 0x84},
		3, {0xE6, 0xB7, 0x80},
		2, {0xC0, 0x6D},
		2, {0xCD, 0xE4}
	},
	{
		2, {0x97, 0x85},
		3, {0xE7, 0xBE, 0x85},
		2, {0x85, 0x7F},
		2, {0xCD, 0xE5}
	},
	{
		2, {0x97, 0x86},
		3, {0xE8, 0x9E, 0xBA},
		2, {0xBA, 0x87},
		2, {0xCD, 0xE6}
	},
	{
		2, {0x97, 0x87},
		3, {0xE8, 0xA3, 0xB8},
		2, {0xF8, 0x88},
		2, {0xCD, 0xE7}
	},
	{
		2, {0x97, 0x88},
		3, {0xE6, 0x9D, 0xA5},
		2, {0x65, 0x67},
		2, {0xCD, 0xE8}
	},
	{
		2, {0x97, 0x89},
		3, {0xE8, 0x8E, 0xB1},
		2, {0xB1, 0x83},
		2, {0xCD, 0xE9}
	},
	{
		2, {0x97, 0x8A},
		3, {0xE9, 0xA0, 0xBC},
		2, {0x3C, 0x98},
		2, {0xCD, 0xEA}
	},
	{
		2, {0x97, 0x8B},
		3, {0xE9, 0x9B, 0xB7},
		2, {0xF7, 0x96},
		2, {0xCD, 0xEB}
	},
	{
		2, {0x97, 0x8C},
		3, {0xE6, 0xB4, 0x9B},
		2, {0x1B, 0x6D},
		2, {0xCD, 0xEC}
	},
	{
		2, {0x97, 0x8D},
		3, {0xE7, 0xB5, 0xA1},
		2, {0x61, 0x7D},
		2, {0xCD, 0xED}
	},
	{
		2, {0x97, 0x8E},
		3, {0xE8, 0x90, 0xBD},
		2, {0x3D, 0x84},
		2, {0xCD, 0xEE}
	},
	{
		2, {0x97, 0x8F},
		3, {0xE9, 0x85, 0xAA},
		2, {0x6A, 0x91},
		2, {0xCD, 0xEF}
	},
	{
		2, {0x97, 0x90},
		3, {0xE4, 0xB9, 0xB1},
		2, {0x71, 0x4E},
		2, {0xCD, 0xF0}
	},
	{
		2, {0x97, 0x91},
		3, {0xE5, 0x8D, 0xB5},
		2, {0x75, 0x53},
		2, {0xCD, 0xF1}
	},
	{
		2, {0x97, 0x92},
		3, {0xE5, 0xB5, 0x90},
		2, {0x50, 0x5D},
		2, {0xCD, 0xF2}
	},
	{
		2, {0x97, 0x93},
		3, {0xE6, 0xAC, 0x84},
		2, {0x04, 0x6B},
		2, {0xCD, 0xF3}
	},
	{
		2, {0x97, 0x94},
		3, {0xE6, 0xBF, 0xAB},
		2, {0xEB, 0x6F},
		2, {0xCD, 0xF4}
	},
	{
		2, {0x97, 0x95},
		3, {0xE8, 0x97, 0x8D},
		2, {0xCD, 0x85},
		2, {0xCD, 0xF5}
	},
	{
		2, {0x97, 0x96},
		3, {0xE8, 0x98, 0xAD},
		2, {0x2D, 0x86},
		2, {0xCD, 0xF6}
	},
	{
		2, {0x97, 0x97},
		3, {0xE8, 0xA6, 0xA7},
		2, {0xA7, 0x89},
		2, {0xCD, 0xF7}
	},
	{
		2, {0x97, 0x98},
		3, {0xE5, 0x88, 0xA9},
		2, {0x29, 0x52},
		2, {0xCD, 0xF8}
	},
	{
		2, {0x97, 0x99},
		3, {0xE5, 0x90, 0x8F},
		2, {0x0F, 0x54},
		2, {0xCD, 0xF9}
	},
	{
		2, {0x97, 0x9A},
		3, {0xE5, 0xB1, 0xA5},
		2, {0x65, 0x5C},
		2, {0xCD, 0xFA}
	},
	{
		2, {0x97, 0x9B},
		3, {0xE6, 0x9D, 0x8E},
		2, {0x4E, 0x67},
		2, {0xCD, 0xFB}
	},
	{
		2, {0x97, 0x9C},
		3, {0xE6, 0xA2, 0xA8},
		2, {0xA8, 0x68},
		2, {0xCD, 0xFC}
	},
	{
		2, {0x97, 0x9D},
		3, {0xE7, 0x90, 0x86},
		2, {0x06, 0x74},
		2, {0xCD, 0xFD}
	},
	{
		2, {0x97, 0x9E},
		3, {0xE7, 0x92, 0x83},
		2, {0x83, 0x74},
		2, {0xCD, 0xFE}
	},
	{
		2, {0x97, 0x9F},
		3, {0xE7, 0x97, 0xA2},
		2, {0xE2, 0x75},
		2, {0xCE, 0xA1}
	},
	{
		2, {0x97, 0xA0},
		3, {0xE8, 0xA3, 0x8F},
		2, {0xCF, 0x88},
		2, {0xCE, 0xA2}
	},
	{
		2, {0x97, 0xA1},
		3, {0xE8, 0xA3, 0xA1},
		2, {0xE1, 0x88},
		2, {0xCE, 0xA3}
	},
	{
		2, {0x97, 0xA2},
		3, {0xE9, 0x87, 0x8C},
		2, {0xCC, 0x91},
		2, {0xCE, 0xA4}
	},
	{
		2, {0x97, 0xA3},
		3, {0xE9, 0x9B, 0xA2},
		2, {0xE2, 0x96},
		2, {0xCE, 0xA5}
	},
	{
		2, {0x97, 0xA4},
		3, {0xE9, 0x99, 0xB8},
		2, {0x78, 0x96},
		2, {0xCE, 0xA6}
	},
	{
		2, {0x97, 0xA5},
		3, {0xE5, 0xBE, 0x8B},
		2, {0x8B, 0x5F},
		2, {0xCE, 0xA7}
	},
	{
		2, {0x97, 0xA6},
		3, {0xE7, 0x8E, 0x87},
		2, {0x87, 0x73},
		2, {0xCE, 0xA8}
	},
	{
		2, {0x97, 0xA7},
		3, {0xE7, 0xAB, 0x8B},
		2, {0xCB, 0x7A},
		2, {0xCE, 0xA9}
	},
	{
		2, {0x97, 0xA8},
		3, {0xE8, 0x91, 0x8E},
		2, {0x4E, 0x84},
		2, {0xCE, 0xAA}
	},
	{
		2, {0x97, 0xA9},
		3, {0xE6, 0x8E, 0xA0},
		2, {0xA0, 0x63},
		2, {0xCE, 0xAB}
	},
	{
		2, {0x97, 0xAA},
		3, {0xE7, 0x95, 0xA5},
		2, {0x65, 0x75},
		2, {0xCE, 0xAC}
	},
	{
		2, {0x97, 0xAB},
		3, {0xE5, 0x8A, 0x89},
		2, {0x89, 0x52},
		2, {0xCE, 0xAD}
	},
	{
		2, {0x97, 0xAC},
		3, {0xE6, 0xB5, 0x81},
		2, {0x41, 0x6D},
		2, {0xCE, 0xAE}
	},
	{
		2, {0x97, 0xAD},
		3, {0xE6, 0xBA, 0x9C},
		2, {0x9C, 0x6E},
		2, {0xCE, 0xAF}
	},
	{
		2, {0x97, 0xAE},
		3, {0xE7, 0x90, 0x89},
		2, {0x09, 0x74},
		2, {0xCE, 0xB0}
	},
	{
		2, {0x97, 0xAF},
		3, {0xE7, 0x95, 0x99},
		2, {0x59, 0x75},
		2, {0xCE, 0xB1}
	},
	{
		2, {0x97, 0xB0},
		3, {0xE7, 0xA1, 0xAB},
		2, {0x6B, 0x78},
		2, {0xCE, 0xB2}
	},
	{
		2, {0x97, 0xB1},
		3, {0xE7, 0xB2, 0x92},
		2, {0x92, 0x7C},
		2, {0xCE, 0xB3}
	},
	{
		2, {0x97, 0xB2},
		3, {0xE9, 0x9A, 0x86},
		2, {0x86, 0x96},
		2, {0xCE, 0xB4}
	},
	{
		2, {0x97, 0xB3},
		3, {0xE7, 0xAB, 0x9C},
		2, {0xDC, 0x7A},
		2, {0xCE, 0xB5}
	},
	{
		2, {0x97, 0xB4},
		3, {0xE9, 0xBE, 0x8D},
		2, {0x8D, 0x9F},
		2, {0xCE, 0xB6}
	},
	{
		2, {0x97, 0xB5},
		3, {0xE4, 0xBE, 0xB6},
		2, {0xB6, 0x4F},
		2, {0xCE, 0xB7}
	},
	{
		2, {0x97, 0xB6},
		3, {0xE6, 0x85, 0xAE},
		2, {0x6E, 0x61},
		2, {0xCE, 0xB8}
	},
	{
		2, {0x97, 0xB7},
		3, {0xE6, 0x97, 0x85},
		2, {0xC5, 0x65},
		2, {0xCE, 0xB9}
	},
	{
		2, {0x97, 0xB8},
		3, {0xE8, 0x99, 0x9C},
		2, {0x5C, 0x86},
		2, {0xCE, 0xBA}
	},
	{
		2, {0x97, 0xB9},
		3, {0xE4, 0xBA, 0x86},
		2, {0x86, 0x4E},
		2, {0xCE, 0xBB}
	},
	{
		2, {0x97, 0xBA},
		3, {0xE4, 0xBA, 0xAE},
		2, {0xAE, 0x4E},
		2, {0xCE, 0xBC}
	},
	{
		2, {0x97, 0xBB},
		3, {0xE5, 0x83, 0x9A},
		2, {0xDA, 0x50},
		2, {0xCE, 0xBD}
	},
	{
		2, {0x97, 0xBC},
		3, {0xE4, 0xB8, 0xA1},
		2, {0x21, 0x4E},
		2, {0xCE, 0xBE}
	},
	{
		2, {0x97, 0xBD},
		3, {0xE5, 0x87, 0x8C},
		2, {0xCC, 0x51},
		2, {0xCE, 0xBF}
	},
	{
		2, {0x97, 0xBE},
		3, {0xE5, 0xAF, 0xAE},
		2, {0xEE, 0x5B},
		2, {0xCE, 0xC0}
	},
	{
		2, {0x97, 0xBF},
		3, {0xE6, 0x96, 0x99},
		2, {0x99, 0x65},
		2, {0xCE, 0xC1}
	},
	{
		2, {0x97, 0xC0},
		3, {0xE6, 0xA2, 0x81},
		2, {0x81, 0x68},
		2, {0xCE, 0xC2}
	},
	{
		2, {0x97, 0xC1},
		3, {0xE6, 0xB6, 0xBC},
		2, {0xBC, 0x6D},
		2, {0xCE, 0xC3}
	},
	{
		2, {0x97, 0xC2},
		3, {0xE7, 0x8C, 0x9F},
		2, {0x1F, 0x73},
		2, {0xCE, 0xC4}
	},
	{
		2, {0x97, 0xC3},
		3, {0xE7, 0x99, 0x82},
		2, {0x42, 0x76},
		2, {0xCE, 0xC5}
	},
	{
		2, {0x97, 0xC4},
		3, {0xE7, 0x9E, 0xAD},
		2, {0xAD, 0x77},
		2, {0xCE, 0xC6}
	},
	{
		2, {0x97, 0xC5},
		3, {0xE7, 0xA8, 0x9C},
		2, {0x1C, 0x7A},
		2, {0xCE, 0xC7}
	},
	{
		2, {0x97, 0xC6},
		3, {0xE7, 0xB3, 0xA7},
		2, {0xE7, 0x7C},
		2, {0xCE, 0xC8}
	},
	{
		2, {0x97, 0xC7},
		3, {0xE8, 0x89, 0xAF},
		2, {0x6F, 0x82},
		2, {0xCE, 0xC9}
	},
	{
		2, {0x97, 0xC8},
		3, {0xE8, 0xAB, 0x92},
		2, {0xD2, 0x8A},
		2, {0xCE, 0xCA}
	},
	{
		2, {0x97, 0xC9},
		3, {0xE9, 0x81, 0xBC},
		2, {0x7C, 0x90},
		2, {0xCE, 0xCB}
	},
	{
		2, {0x97, 0xCA},
		3, {0xE9, 0x87, 0x8F},
		2, {0xCF, 0x91},
		2, {0xCE, 0xCC}
	},
	{
		2, {0x97, 0xCB},
		3, {0xE9, 0x99, 0xB5},
		2, {0x75, 0x96},
		2, {0xCE, 0xCD}
	},
	{
		2, {0x97, 0xCC},
		3, {0xE9, 0xA0, 0x98},
		2, {0x18, 0x98},
		2, {0xCE, 0xCE}
	},
	{
		2, {0x97, 0xCD},
		3, {0xE5, 0x8A, 0x9B},
		2, {0x9B, 0x52},
		2, {0xCE, 0xCF}
	},
	{
		2, {0x97, 0xCE},
		3, {0xE7, 0xB7, 0x91},
		2, {0xD1, 0x7D},
		2, {0xCE, 0xD0}
	},
	{
		2, {0x97, 0xCF},
		3, {0xE5, 0x80, 0xAB},
		2, {0x2B, 0x50},
		2, {0xCE, 0xD1}
	},
	{
		2, {0x97, 0xD0},
		3, {0xE5, 0x8E, 0x98},
		2, {0x98, 0x53},
		2, {0xCE, 0xD2}
	},
	{
		2, {0x97, 0xD1},
		3, {0xE6, 0x9E, 0x97},
		2, {0x97, 0x67},
		2, {0xCE, 0xD3}
	},
	{
		2, {0x97, 0xD2},
		3, {0xE6, 0xB7, 0x8B},
		2, {0xCB, 0x6D},
		2, {0xCE, 0xD4}
	},
	{
		2, {0x97, 0xD3},
		3, {0xE7, 0x87, 0x90},
		2, {0xD0, 0x71},
		2, {0xCE, 0xD5}
	},
	{
		2, {0x97, 0xD4},
		3, {0xE7, 0x90, 0xB3},
		2, {0x33, 0x74},
		2, {0xCE, 0xD6}
	},
	{
		2, {0x97, 0xD5},
		3, {0xE8, 0x87, 0xA8},
		2, {0xE8, 0x81},
		2, {0xCE, 0xD7}
	},
	{
		2, {0x97, 0xD6},
		3, {0xE8, 0xBC, 0xAA},
		2, {0x2A, 0x8F},
		2, {0xCE, 0xD8}
	},
	{
		2, {0x97, 0xD7},
		3, {0xE9, 0x9A, 0xA3},
		2, {0xA3, 0x96},
		2, {0xCE, 0xD9}
	},
	{
		2, {0x97, 0xD8},
		3, {0xE9, 0xB1, 0x97},
		2, {0x57, 0x9C},
		2, {0xCE, 0xDA}
	},
	{
		2, {0x97, 0xD9},
		3, {0xE9, 0xBA, 0x9F},
		2, {0x9F, 0x9E},
		2, {0xCE, 0xDB}
	},
	{
		2, {0x97, 0xDA},
		3, {0xE7, 0x91, 0xA0},
		2, {0x60, 0x74},
		2, {0xCE, 0xDC}
	},
	{
		2, {0x97, 0xDB},
		3, {0xE5, 0xA1, 0x81},
		2, {0x41, 0x58},
		2, {0xCE, 0xDD}
	},
	{
		2, {0x97, 0xDC},
		3, {0xE6, 0xB6, 0x99},
		2, {0x99, 0x6D},
		2, {0xCE, 0xDE}
	},
	{
		2, {0x97, 0xDD},
		3, {0xE7, 0xB4, 0xAF},
		2, {0x2F, 0x7D},
		2, {0xCE, 0xDF}
	},
	{
		2, {0x97, 0xDE},
		3, {0xE9, 0xA1, 0x9E},
		2, {0x5E, 0x98},
		2, {0xCE, 0xE0}
	},
	{
		2, {0x97, 0xDF},
		3, {0xE4, 0xBB, 0xA4},
		2, {0xE4, 0x4E},
		2, {0xCE, 0xE1}
	},
	{
		2, {0x97, 0xE0},
		3, {0xE4, 0xBC, 0xB6},
		2, {0x36, 0x4F},
		2, {0xCE, 0xE2}
	},
	{
		2, {0x97, 0xE1},
		3, {0xE4, 0xBE, 0x8B},
		2, {0x8B, 0x4F},
		2, {0xCE, 0xE3}
	},
	{
		2, {0x97, 0xE2},
		3, {0xE5, 0x86, 0xB7},
		2, {0xB7, 0x51},
		2, {0xCE, 0xE4}
	},
	{
		2, {0x97, 0xE3},
		3, {0xE5, 0x8A, 0xB1},
		2, {0xB1, 0x52},
		2, {0xCE, 0xE5}
	},
	{
		2, {0x97, 0xE4},
		3, {0xE5, 0xB6, 0xBA},
		2, {0xBA, 0x5D},
		2, {0xCE, 0xE6}
	},
	{
		2, {0x97, 0xE5},
		3, {0xE6, 0x80, 0x9C},
		2, {0x1C, 0x60},
		2, {0xCE, 0xE7}
	},
	{
		2, {0x97, 0xE6},
		3, {0xE7, 0x8E, 0xB2},
		2, {0xB2, 0x73},
		2, {0xCE, 0xE8}
	},
	{
		2, {0x97, 0xE7},
		3, {0xE7, 0xA4, 0xBC},
		2, {0x3C, 0x79},
		2, {0xCE, 0xE9}
	},
	{
		2, {0x97, 0xE8},
		3, {0xE8, 0x8B, 0x93},
		2, {0xD3, 0x82},
		2, {0xCE, 0xEA}
	},
	{
		2, {0x97, 0xE9},
		3, {0xE9, 0x88, 0xB4},
		2, {0x34, 0x92},
		2, {0xCE, 0xEB}
	},
	{
		2, {0x97, 0xEA},
		3, {0xE9, 0x9A, 0xB7},
		2, {0xB7, 0x96},
		2, {0xCE, 0xEC}
	},
	{
		2, {0x97, 0xEB},
		3, {0xE9, 0x9B, 0xB6},
		2, {0xF6, 0x96},
		2, {0xCE, 0xED}
	},
	{
		2, {0x97, 0xEC},
		3, {0xE9, 0x9C, 0x8A},
		2, {0x0A, 0x97},
		2, {0xCE, 0xEE}
	},
	{
		2, {0x97, 0xED},
		3, {0xE9, 0xBA, 0x97},
		2, {0x97, 0x9E},
		2, {0xCE, 0xEF}
	},
	{
		2, {0x97, 0xEE},
		3, {0xE9, 0xBD, 0xA2},
		2, {0x62, 0x9F},
		2, {0xCE, 0xF0}
	},
	{
		2, {0x97, 0xEF},
		3, {0xE6, 0x9A, 0xA6},
		2, {0xA6, 0x66},
		2, {0xCE, 0xF1}
	},
	{
		2, {0x97, 0xF0},
		3, {0xE6, 0xAD, 0xB4},
		2, {0x74, 0x6B},
		2, {0xCE, 0xF2}
	},
	{
		2, {0x97, 0xF1},
		3, {0xE5, 0x88, 0x97},
		2, {0x17, 0x52},
		2, {0xCE, 0xF3}
	},
	{
		2, {0x97, 0xF2},
		3, {0xE5, 0x8A, 0xA3},
		2, {0xA3, 0x52},
		2, {0xCE, 0xF4}
	},
	{
		2, {0x97, 0xF3},
		3, {0xE7, 0x83, 0x88},
		2, {0xC8, 0x70},
		2, {0xCE, 0xF5}
	},
	{
		2, {0x97, 0xF4},
		3, {0xE8, 0xA3, 0x82},
		2, {0xC2, 0x88},
		2, {0xCE, 0xF6}
	},
	{
		2, {0x97, 0xF5},
		3, {0xE5, 0xBB, 0x89},
		2, {0xC9, 0x5E},
		2, {0xCE, 0xF7}
	},
	{
		2, {0x97, 0xF6},
		3, {0xE6, 0x81, 0x8B},
		2, {0x4B, 0x60},
		2, {0xCE, 0xF8}
	},
	{
		2, {0x97, 0xF7},
		3, {0xE6, 0x86, 0x90},
		2, {0x90, 0x61},
		2, {0xCE, 0xF9}
	},
	{
		2, {0x97, 0xF8},
		3, {0xE6, 0xBC, 0xA3},
		2, {0x23, 0x6F},
		2, {0xCE, 0xFA}
	},
	{
		2, {0x97, 0xF9},
		3, {0xE7, 0x85, 0x89},
		2, {0x49, 0x71},
		2, {0xCE, 0xFB}
	},
	{
		2, {0x97, 0xFA},
		3, {0xE7, 0xB0, 0xBE},
		2, {0x3E, 0x7C},
		2, {0xCE, 0xFC}
	},
	{
		2, {0x97, 0xFB},
		3, {0xE7, 0xB7, 0xB4},
		2, {0xF4, 0x7D},
		2, {0xCE, 0xFD}
	},
	{
		2, {0x97, 0xFC},
		3, {0xE8, 0x81, 0xAF},
		2, {0x6F, 0x80},
		2, {0xCE, 0xFE}
	},
	{
		2, {0x98, 0x40},
		3, {0xE8, 0x93, 0xAE},
		2, {0xEE, 0x84},
		2, {0xCF, 0xA1}
	},
	{
		2, {0x98, 0x41},
		3, {0xE9, 0x80, 0xA3},
		2, {0x23, 0x90},
		2, {0xCF, 0xA2}
	},
	{
		2, {0x98, 0x42},
		3, {0xE9, 0x8C, 0xAC},
		2, {0x2C, 0x93},
		2, {0xCF, 0xA3}
	},
	{
		2, {0x98, 0x43},
		3, {0xE5, 0x91, 0x82},
		2, {0x42, 0x54},
		2, {0xCF, 0xA4}
	},
	{
		2, {0x98, 0x44},
		3, {0xE9, 0xAD, 0xAF},
		2, {0x6F, 0x9B},
		2, {0xCF, 0xA5}
	},
	{
		2, {0x98, 0x45},
		3, {0xE6, 0xAB, 0x93},
		2, {0xD3, 0x6A},
		2, {0xCF, 0xA6}
	},
	{
		2, {0x98, 0x46},
		3, {0xE7, 0x82, 0x89},
		2, {0x89, 0x70},
		2, {0xCF, 0xA7}
	},
	{
		2, {0x98, 0x47},
		3, {0xE8, 0xB3, 0x82},
		2, {0xC2, 0x8C},
		2, {0xCF, 0xA8}
	},
	{
		2, {0x98, 0x48},
		3, {0xE8, 0xB7, 0xAF},
		2, {0xEF, 0x8D},
		2, {0xCF, 0xA9}
	},
	{
		2, {0x98, 0x49},
		3, {0xE9, 0x9C, 0xB2},
		2, {0x32, 0x97},
		2, {0xCF, 0xAA}
	},
	{
		2, {0x98, 0x4A},
		3, {0xE5, 0x8A, 0xB4},
		2, {0xB4, 0x52},
		2, {0xCF, 0xAB}
	},
	{
		2, {0x98, 0x4B},
		3, {0xE5, 0xA9, 0x81},
		2, {0x41, 0x5A},
		2, {0xCF, 0xAC}
	},
	{
		2, {0x98, 0x4C},
		3, {0xE5, 0xBB, 0x8A},
		2, {0xCA, 0x5E},
		2, {0xCF, 0xAD}
	},
	{
		2, {0x98, 0x4D},
		3, {0xE5, 0xBC, 0x84},
		2, {0x04, 0x5F},
		2, {0xCF, 0xAE}
	},
	{
		2, {0x98, 0x4E},
		3, {0xE6, 0x9C, 0x97},
		2, {0x17, 0x67},
		2, {0xCF, 0xAF}
	},
	{
		2, {0x98, 0x4F},
		3, {0xE6, 0xA5, 0xBC},
		2, {0x7C, 0x69},
		2, {0xCF, 0xB0}
	},
	{
		2, {0x98, 0x50},
		3, {0xE6, 0xA6, 0x94},
		2, {0x94, 0x69},
		2, {0xCF, 0xB1}
	},
	{
		2, {0x98, 0x51},
		3, {0xE6, 0xB5, 0xAA},
		2, {0x6A, 0x6D},
		2, {0xCF, 0xB2}
	},
	{
		2, {0x98, 0x52},
		3, {0xE6, 0xBC, 0x8F},
		2, {0x0F, 0x6F},
		2, {0xCF, 0xB3}
	},
	{
		2, {0x98, 0x53},
		3, {0xE7, 0x89, 0xA2},
		2, {0x62, 0x72},
		2, {0xCF, 0xB4}
	},
	{
		2, {0x98, 0x54},
		3, {0xE7, 0x8B, 0xBC},
		2, {0xFC, 0x72},
		2, {0xCF, 0xB5}
	},
	{
		2, {0x98, 0x55},
		3, {0xE7, 0xAF, 0xAD},
		2, {0xED, 0x7B},
		2, {0xCF, 0xB6}
	},
	{
		2, {0x98, 0x56},
		3, {0xE8, 0x80, 0x81},
		2, {0x01, 0x80},
		2, {0xCF, 0xB7}
	},
	{
		2, {0x98, 0x57},
		3, {0xE8, 0x81, 0xBE},
		2, {0x7E, 0x80},
		2, {0xCF, 0xB8}
	},
	{
		2, {0x98, 0x58},
		3, {0xE8, 0x9D, 0x8B},
		2, {0x4B, 0x87},
		2, {0xCF, 0xB9}
	},
	{
		2, {0x98, 0x59},
		3, {0xE9, 0x83, 0x8E},
		2, {0xCE, 0x90},
		2, {0xCF, 0xBA}
	},
	{
		2, {0x98, 0x5A},
		3, {0xE5, 0x85, 0xAD},
		2, {0x6D, 0x51},
		2, {0xCF, 0xBB}
	},
	{
		2, {0x98, 0x5B},
		3, {0xE9, 0xBA, 0x93},
		2, {0x93, 0x9E},
		2, {0xCF, 0xBC}
	},
	{
		2, {0x98, 0x5C},
		3, {0xE7, 0xA6, 0x84},
		2, {0x84, 0x79},
		2, {0xCF, 0xBD}
	},
	{
		2, {0x98, 0x5D},
		3, {0xE8, 0x82, 0x8B},
		2, {0x8B, 0x80},
		2, {0xCF, 0xBE}
	},
	{
		2, {0x98, 0x5E},
		3, {0xE9, 0x8C, 0xB2},
		2, {0x32, 0x93},
		2, {0xCF, 0xBF}
	},
	{
		2, {0x98, 0x5F},
		3, {0xE8, 0xAB, 0x96},
		2, {0xD6, 0x8A},
		2, {0xCF, 0xC0}
	},
	{
		2, {0x98, 0x60},
		3, {0xE5, 0x80, 0xAD},
		2, {0x2D, 0x50},
		2, {0xCF, 0xC1}
	},
	{
		2, {0x98, 0x61},
		3, {0xE5, 0x92, 0x8C},
		2, {0x8C, 0x54},
		2, {0xCF, 0xC2}
	},
	{
		2, {0x98, 0x62},
		3, {0xE8, 0xA9, 0xB1},
		2, {0x71, 0x8A},
		2, {0xCF, 0xC3}
	},
	{
		2, {0x98, 0x63},
		3, {0xE6, 0xAD, 0xAA},
		2, {0x6A, 0x6B},
		2, {0xCF, 0xC4}
	},
	{
		2, {0x98, 0x64},
		3, {0xE8, 0xB3, 0x84},
		2, {0xC4, 0x8C},
		2, {0xCF, 0xC5}
	},
	{
		2, {0x98, 0x65},
		3, {0xE8, 0x84, 0x87},
		2, {0x07, 0x81},
		2, {0xCF, 0xC6}
	},
	{
		2, {0x98, 0x66},
		3, {0xE6, 0x83, 0x91},
		2, {0xD1, 0x60},
		2, {0xCF, 0xC7}
	},
	{
		2, {0x98, 0x67},
		3, {0xE6, 0x9E, 0xA0},
		2, {0xA0, 0x67},
		2, {0xCF, 0xC8}
	},
	{
		2, {0x98, 0x68},
		3, {0xE9, 0xB7, 0xB2},
		2, {0xF2, 0x9D},
		2, {0xCF, 0xC9}
	},
	{
		2, {0x98, 0x69},
		3, {0xE4, 0xBA, 0x99},
		2, {0x99, 0x4E},
		2, {0xCF, 0xCA}
	},
	{
		2, {0x98, 0x6A},
		3, {0xE4, 0xBA, 0x98},
		2, {0x98, 0x4E},
		2, {0xCF, 0xCB}
	},
	{
		2, {0x98, 0x6B},
		3, {0xE9, 0xB0, 0x90},
		2, {0x10, 0x9C},
		2, {0xCF, 0xCC}
	},
	{
		2, {0x98, 0x6C},
		3, {0xE8, 0xA9, 0xAB},
		2, {0x6B, 0x8A},
		2, {0xCF, 0xCD}
	},
	{
		2, {0x98, 0x6D},
		3, {0xE8, 0x97, 0x81},
		2, {0xC1, 0x85},
		2, {0xCF, 0xCE}
	},
	{
		2, {0x98, 0x6E},
		3, {0xE8, 0x95, 0xA8},
		2, {0x68, 0x85},
		2, {0xCF, 0xCF}
	},
	{
		2, {0x98, 0x6F},
		3, {0xE6, 0xA4, 0x80},
		2, {0x00, 0x69},
		2, {0xCF, 0xD0}
	},
	{
		2, {0x98, 0x70},
		3, {0xE6, 0xB9, 0xBE},
		2, {0x7E, 0x6E},
		2, {0xCF, 0xD1}
	},
	{
		2, {0x98, 0x71},
		3, {0xE7, 0xA2, 0x97},
		2, {0x97, 0x78},
		2, {0xCF, 0xD2}
	},
	{
		2, {0x98, 0x72},
		3, {0xE8, 0x85, 0x95},
		2, {0x55, 0x81},
		2, {0xCF, 0xD3}
	},
	{
		2, {0x98, 0x73},
		4, {0xF0, 0xA0, 0xAE, 0x9F},
		4, {0x42, 0xD8, 0x9F, 0xDF},
		0, {0x00}
	},
	{
		2, {0x98, 0x74},
		3, {0xE5, 0xAD, 0x81},
		2, {0x41, 0x5B},
		3, {0x8F, 0xBA, 0xBA}
	},
	{
		2, {0x98, 0x75},
		3, {0xE5, 0xAD, 0x96},
		2, {0x56, 0x5B},
		3, {0x8F, 0xBA, 0xBE}
	},
	{
		2, {0x98, 0x76},
		3, {0xE5, 0xAD, 0xBD},
		2, {0x7D, 0x5B},
		3, {0x8F, 0xBA, 0xC4}
	},
	{
		2, {0x98, 0x77},
		3, {0xE5, 0xAE, 0x93},
		2, {0x93, 0x5B},
		3, {0x8F, 0xBA, 0xCE}
	},
	{
		2, {0x98, 0x78},
		3, {0xE5, 0xAF, 0x98},
		2, {0xD8, 0x5B},
		3, {0x8F, 0xBA, 0xE1}
	},
	{
		2, {0x98, 0x79},
		3, {0xE5, 0xAF, 0xAC},
		2, {0xEC, 0x5B},
		0, {0x00}
	},
	{
		2, {0x98, 0x7A},
		3, {0xE5, 0xB0, 0x92},
		2, {0x12, 0x5C},
		0, {0x00}
	},
	{
		2, {0x98, 0x7B},
		3, {0xE5, 0xB0, 0x9E},
		2, {0x1E, 0x5C},
		3, {0x8F, 0xBA, 0xEB}
	},
	{
		2, {0x98, 0x7C},
		3, {0xE5, 0xB0, 0xA3},
		2, {0x23, 0x5C},
		3, {0x8F, 0xBA, 0xED}
	},
	{
		2, {0x98, 0x7D},
		3, {0xE5, 0xB0, 0xAB},
		2, {0x2B, 0x5C},
		3, {0x8F, 0xBA, 0xF0}
	},
	{
		2, {0x98, 0x7E},
		3, {0xE3, 0x9E, 0x8D},
		2, {0x8D, 0x37},
		0, {0x00}
	},
	{
		2, {0x98, 0x80},
		3, {0xE5, 0xB1, 0xA2},
		2, {0x62, 0x5C},
		3, {0x8F, 0xBA, 0xFA}
	},
	{
		2, {0x98, 0x81},
		3, {0xEF, 0xA8, 0xBB},
		2, {0x3B, 0xFA},
		0, {0x00}
	},
	{
		2, {0x98, 0x82},
		3, {0xEF, 0xA8, 0xBC},
		2, {0x3C, 0xFA},
		0, {0x00}
	},
	{
		2, {0x98, 0x83},
		4, {0xF0, 0xA1, 0x9A, 0xB4},
		4, {0x45, 0xD8, 0xB4, 0xDE},
		0, {0x00}
	},
	{
		2, {0x98, 0x84},
		3, {0xE5, 0xB1, 0xBA},
		2, {0x7A, 0x5C},
		3, {0x8F, 0xBB, 0xA5}
	},
	{
		2, {0x98, 0x85},
		3, {0xE5, 0xB2, 0x8F},
		2, {0x8F, 0x5C},
		3, {0x8F, 0xBB, 0xAC}
	},
	{
		2, {0x98, 0x86},
		3, {0xE5, 0xB2, 0x9F},
		2, {0x9F, 0x5C},
		3, {0x8F, 0xBB, 0xAF}
	},
	{
		2, {0x98, 0x87},
		3, {0xE5, 0xB2, 0xA3},
		2, {0xA3, 0x5C},
		3, {0x8F, 0xBB, 0xB2}
	},
	{
		2, {0x98, 0x88},
		3, {0xE5, 0xB2, 0xAA},
		2, {0xAA, 0x5C},
		3, {0x8F, 0xBB, 0xB4}
	},
	{
		2, {0x98, 0x89},
		3, {0xE5, 0xB2, 0xBA},
		2, {0xBA, 0x5C},
		3, {0x8F, 0xBB, 0xB8}
	},
	{
		2, {0x98, 0x8A},
		3, {0xE5, 0xB3, 0x8B},
		2, {0xCB, 0x5C},
		3, {0x8F, 0xBB, 0xBA}
	},
	{
		2, {0x98, 0x8B},
		3, {0xE5, 0xB3, 0x90},
		2, {0xD0, 0x5C},
		0, {0x00}
	},
	{
		2, {0x98, 0x8C},
		3, {0xE5, 0xB3, 0x92},
		2, {0xD2, 0x5C},
		3, {0x8F, 0xBB, 0xBB}
	},
	{
		2, {0x98, 0x8D},
		3, {0xE5, 0xB3, 0xB4},
		2, {0xF4, 0x5C},
		3, {0x8F, 0xBB, 0xC1}
	},
	{
		2, {0x98, 0x8E},
		4, {0xF0, 0xA1, 0xB8, 0xB4},
		4, {0x47, 0xD8, 0x34, 0xDE},
		0, {0x00}
	},
	{
		2, {0x98, 0x8F},
		3, {0xE3, 0x9F, 0xA2},
		2, {0xE2, 0x37},
		0, {0x00}
	},
	{
		2, {0x98, 0x90},
		3, {0xE5, 0xB4, 0x8D},
		2, {0x0D, 0x5D},
		3, {0x8F, 0xBB, 0xC4}
	},
	{
		2, {0x98, 0x91},
		3, {0xE5, 0xB4, 0xA7},
		2, {0x27, 0x5D},
		3, {0x8F, 0xBB, 0xCA}
	},
	{
		2, {0x98, 0x92},
		3, {0xEF, 0xA8, 0x91},
		2, {0x11, 0xFA},
		0, {0x00}
	},
	{
		2, {0x98, 0x93},
		3, {0xE5, 0xB5, 0x86},
		2, {0x46, 0x5D},
		3, {0x8F, 0xBB, 0xD2}
	},
	{
		2, {0x98, 0x94},
		3, {0xE5, 0xB5, 0x87},
		2, {0x47, 0x5D},
		0, {0x00}
	},
	{
		2, {0x98, 0x95},
		3, {0xE5, 0xB5, 0x93},
		2, {0x53, 0x5D},
		0, {0x00}
	},
	{
		2, {0x98, 0x96},
		3, {0xE5, 0xB5, 0x8A},
		2, {0x4A, 0x5D},
		3, {0x8F, 0xBB, 0xD7}
	},
	{
		2, {0x98, 0x97},
		3, {0xE5, 0xB5, 0xAD},
		2, {0x6D, 0x5D},
		3, {0x8F, 0xBB, 0xDE}
	},
	{
		2, {0x98, 0x98},
		3, {0xE5, 0xB6, 0x81},
		2, {0x81, 0x5D},
		3, {0x8F, 0xBB, 0xE4}
	},
	{
		2, {0x98, 0x99},
		3, {0xE5, 0xB6, 0xA0},
		2, {0xA0, 0x5D},
		3, {0x8F, 0xBB, 0xEF}
	},
	{
		2, {0x98, 0x9A},
		3, {0xE5, 0xB6, 0xA4},
		2, {0xA4, 0x5D},
		0, {0x00}
	},
	{
		2, {0x98, 0x9B},
		3, {0xE5, 0xB6, 0xA7},
		2, {0xA7, 0x5D},
		3, {0x8F, 0xBB, 0xF0}
	},
	{
		2, {0x98, 0x9C},
		3, {0xE5, 0xB6, 0xB8},
		2, {0xB8, 0x5D},
		3, {0x8F, 0xBB, 0xF4}
	},
	{
		2, {0x98, 0x9D},
		3, {0xE5, 0xB7, 0x8B},
		2, {0xCB, 0x5D},
		3, {0x8F, 0xBB, 0xF8}
	},
	{
		2, {0x98, 0x9E},
		3, {0xE5, 0x90, 0x9E},
		2, {0x1E, 0x54},
		0, {0x00}
	},
	{
		2, {0x98, 0x9F},
		3, {0xE5, 0xBC, 0x8C},
		2, {0x0C, 0x5F},
		2, {0xD0, 0xA1}
	},
	{
		2, {0x98, 0xA0},
		3, {0xE4, 0xB8, 0x90},
		2, {0x10, 0x4E},
		2, {0xD0, 0xA2}
	},
	{
		2, {0x98, 0xA1},
		3, {0xE4, 0xB8, 0x95},
		2, {0x15, 0x4E},
		2, {0xD0, 0xA3}
	},
	{
		2, {0x98, 0xA2},
		3, {0xE4, 0xB8, 0xAA},
		2, {0x2A, 0x4E},
		2, {0xD0, 0xA4}
	},
	{
		2, {0x98, 0xA3},
		3, {0xE4, 0xB8, 0xB1},
		2, {0x31, 0x4E},
		2, {0xD0, 0xA5}
	},
	{
		2, {0x98, 0xA4},
		3, {0xE4, 0xB8, 0xB6},
		2, {0x36, 0x4E},
		2, {0xD0, 0xA6}
	},
	{
		2, {0x98, 0xA5},
		3, {0xE4, 0xB8, 0xBC},
		2, {0x3C, 0x4E},
		2, {0xD0, 0xA7}
	},
	{
		2, {0x98, 0xA6},
		3, {0xE4, 0xB8, 0xBF},
		2, {0x3F, 0x4E},
		2, {0xD0, 0xA8}
	},
	{
		2, {0x98, 0xA7},
		3, {0xE4, 0xB9, 0x82},
		2, {0x42, 0x4E},
		2, {0xD0, 0xA9}
	},
	{
		2, {0x98, 0xA8},
		3, {0xE4, 0xB9, 0x96},
		2, {0x56, 0x4E},
		2, {0xD0, 0xAA}
	},
	{
		2, {0x98, 0xA9},
		3, {0xE4, 0xB9, 0x98},
		2, {0x58, 0x4E},
		2, {0xD0, 0xAB}
	},
	{
		2, {0x98, 0xAA},
		3, {0xE4, 0xBA, 0x82},
		2, {0x82, 0x4E},
		2, {0xD0, 0xAC}
	},
	{
		2, {0x98, 0xAB},
		3, {0xE4, 0xBA, 0x85},
		2, {0x85, 0x4E},
		2, {0xD0, 0xAD}
	},
	{
		2, {0x98, 0xAC},
		3, {0xE8, 0xB1, 0xAB},
		2, {0x6B, 0x8C},
		2, {0xD0, 0xAE}
	},
	{
		2, {0x98, 0xAD},
		3, {0xE4, 0xBA, 0x8A},
		2, {0x8A, 0x4E},
		2, {0xD0, 0xAF}
	},
	{
		2, {0x98, 0xAE},
		3, {0xE8, 0x88, 0x92},
		2, {0x12, 0x82},
		2, {0xD0, 0xB0}
	},
	{
		2, {0x98, 0xAF},
		3, {0xE5, 0xBC, 0x8D},
		2, {0x0D, 0x5F},
		2, {0xD0, 0xB1}
	},
	{
		2, {0x98, 0xB0},
		3, {0xE4, 0xBA, 0x8E},
		2, {0x8E, 0x4E},
		2, {0xD0, 0xB2}
	},
	{
		2, {0x98, 0xB1},
		3, {0xE4, 0xBA, 0x9E},
		2, {0x9E, 0x4E},
		2, {0xD0, 0xB3}
	},
	{
		2, {0x98, 0xB2},
		3, {0xE4, 0xBA, 0x9F},
		2, {0x9F, 0x4E},
		2, {0xD0, 0xB4}
	},
	{
		2, {0x98, 0xB3},
		3, {0xE4, 0xBA, 0xA0},
		2, {0xA0, 0x4E},
		2, {0xD0, 0xB5}
	},
	{
		2, {0x98, 0xB4},
		3, {0xE4, 0xBA, 0xA2},
		2, {0xA2, 0x4E},
		2, {0xD0, 0xB6}
	},
	{
		2, {0x98, 0xB5},
		3, {0xE4, 0xBA, 0xB0},
		2, {0xB0, 0x4E},
		2, {0xD0, 0xB7}
	},
	{
		2, {0x98, 0xB6},
		3, {0xE4, 0xBA, 0xB3},
		2, {0xB3, 0x4E},
		2, {0xD0, 0xB8}
	},
	{
		2, {0x98, 0xB7},
		3, {0xE4, 0xBA, 0xB6},
		2, {0xB6, 0x4E},
		2, {0xD0, 0xB9}
	},
	{
		2, {0x98, 0xB8},
		3, {0xE4, 0xBB, 0x8E},
		2, {0xCE, 0x4E},
		2, {0xD0, 0xBA}
	},
	{
		2, {0x98, 0xB9},
		3, {0xE4, 0xBB, 0x8D},
		2, {0xCD, 0x4E},
		2, {0xD0, 0xBB}
	},
	{
		2, {0x98, 0xBA},
		3, {0xE4, 0xBB, 0x84},
		2, {0xC4, 0x4E},
		2, {0xD0, 0xBC}
	},
	{
		2, {0x98, 0xBB},
		3, {0xE4, 0xBB, 0x86},
		2, {0xC6, 0x4E},
		2, {0xD0, 0xBD}
	},
	{
		2, {0x98, 0xBC},
		3, {0xE4, 0xBB, 0x82},
		2, {0xC2, 0x4E},
		2, {0xD0, 0xBE}
	},
	{
		2, {0x98, 0xBD},
		3, {0xE4, 0xBB, 0x97},
		2, {0xD7, 0x4E},
		2, {0xD0, 0xBF}
	},
	{
		2, {0x98, 0xBE},
		3, {0xE4, 0xBB, 0x9E},
		2, {0xDE, 0x4E},
		2, {0xD0, 0xC0}
	},
	{
		2, {0x98, 0xBF},
		3, {0xE4, 0xBB, 0xAD},
		2, {0xED, 0x4E},
		2, {0xD0, 0xC1}
	},
	{
		2, {0x98, 0xC0},
		3, {0xE4, 0xBB, 0x9F},
		2, {0xDF, 0x4E},
		2, {0xD0, 0xC2}
	},
	{
		2, {0x98, 0xC1},
		3, {0xE4, 0xBB, 0xB7},
		2, {0xF7, 0x4E},
		2, {0xD0, 0xC3}
	},
	{
		2, {0x98, 0xC2},
		3, {0xE4, 0xBC, 0x89},
		2, {0x09, 0x4F},
		2, {0xD0, 0xC4}
	},
	{
		2, {0x98, 0xC3},
		3, {0xE4, 0xBD, 0x9A},
		2, {0x5A, 0x4F},
		2, {0xD0, 0xC5}
	},
	{
		2, {0x98, 0xC4},
		3, {0xE4, 0xBC, 0xB0},
		2, {0x30, 0x4F},
		2, {0xD0, 0xC6}
	},
	{
		2, {0x98, 0xC5},
		3, {0xE4, 0xBD, 0x9B},
		2, {0x5B, 0x4F},
		2, {0xD0, 0xC7}
	},
	{
		2, {0x98, 0xC6},
		3, {0xE4, 0xBD, 0x9D},
		2, {0x5D, 0x4F},
		2, {0xD0, 0xC8}
	},
	{
		2, {0x98, 0xC7},
		3, {0xE4, 0xBD, 0x97},
		2, {0x57, 0x4F},
		2, {0xD0, 0xC9}
	},
	{
		2, {0x98, 0xC8},
		3, {0xE4, 0xBD, 0x87},
		2, {0x47, 0x4F},
		2, {0xD0, 0xCA}
	},
	{
		2, {0x98, 0xC9},
		3, {0xE4, 0xBD, 0xB6},
		2, {0x76, 0x4F},
		2, {0xD0, 0xCB}
	},
	{
		2, {0x98, 0xCA},
		3, {0xE4, 0xBE, 0x88},
		2, {0x88, 0x4F},
		2, {0xD0, 0xCC}
	},
	{
		2, {0x98, 0xCB},
		3, {0xE4, 0xBE, 0x8F},
		2, {0x8F, 0x4F},
		2, {0xD0, 0xCD}
	},
	{
		2, {0x98, 0xCC},
		3, {0xE4, 0xBE, 0x98},
		2, {0x98, 0x4F},
		2, {0xD0, 0xCE}
	},
	{
		2, {0x98, 0xCD},
		3, {0xE4, 0xBD, 0xBB},
		2, {0x7B, 0x4F},
		2, {0xD0, 0xCF}
	},
	{
		2, {0x98, 0xCE},
		3, {0xE4, 0xBD, 0xA9},
		2, {0x69, 0x4F},
		2, {0xD0, 0xD0}
	},
	{
		2, {0x98, 0xCF},
		3, {0xE4, 0xBD, 0xB0},
		2, {0x70, 0x4F},
		2, {0xD0, 0xD1}
	},
	{
		2, {0x98, 0xD0},
		3, {0xE4, 0xBE, 0x91},
		2, {0x91, 0x4F},
		2, {0xD0, 0xD2}
	},
	{
		2, {0x98, 0xD1},
		3, {0xE4, 0xBD, 0xAF},
		2, {0x6F, 0x4F},
		2, {0xD0, 0xD3}
	},
	{
		2, {0x98, 0xD2},
		3, {0xE4, 0xBE, 0x86},
		2, {0x86, 0x4F},
		2, {0xD0, 0xD4}
	},
	{
		2, {0x98, 0xD3},
		3, {0xE4, 0xBE, 0x96},
		2, {0x96, 0x4F},
		2, {0xD0, 0xD5}
	},
	{
		2, {0x98, 0xD4},
		3, {0xE5, 0x84, 0x98},
		2, {0x18, 0x51},
		2, {0xD0, 0xD6}
	},
	{
		2, {0x98, 0xD5},
		3, {0xE4, 0xBF, 0x94},
		2, {0xD4, 0x4F},
		2, {0xD0, 0xD7}
	},
	{
		2, {0x98, 0xD6},
		3, {0xE4, 0xBF, 0x9F},
		2, {0xDF, 0x4F},
		2, {0xD0, 0xD8}
	},
	{
		2, {0x98, 0xD7},
		3, {0xE4, 0xBF, 0x8E},
		2, {0xCE, 0x4F},
		2, {0xD0, 0xD9}
	},
	{
		2, {0x98, 0xD8},
		3, {0xE4, 0xBF, 0x98},
		2, {0xD8, 0x4F},
		2, {0xD0, 0xDA}
	},
	{
		2, {0x98, 0xD9},
		3, {0xE4, 0xBF, 0x9B},
		2, {0xDB, 0x4F},
		2, {0xD0, 0xDB}
	},
	{
		2, {0x98, 0xDA},
		3, {0xE4, 0xBF, 0x91},
		2, {0xD1, 0x4F},
		2, {0xD0, 0xDC}
	},
	{
		2, {0x98, 0xDB},
		3, {0xE4, 0xBF, 0x9A},
		2, {0xDA, 0x4F},
		2, {0xD0, 0xDD}
	},
	{
		2, {0x98, 0xDC},
		3, {0xE4, 0xBF, 0x90},
		2, {0xD0, 0x4F},
		2, {0xD0, 0xDE}
	},
	{
		2, {0x98, 0xDD},
		3, {0xE4, 0xBF, 0xA4},
		2, {0xE4, 0x4F},
		2, {0xD0, 0xDF}
	},
	{
		2, {0x98, 0xDE},
		3, {0xE4, 0xBF, 0xA5},
		2, {0xE5, 0x4F},
		2, {0xD0, 0xE0}
	},
	{
		2, {0x98, 0xDF},
		3, {0xE5, 0x80, 0x9A},
		2, {0x1A, 0x50},
		2, {0xD0, 0xE1}
	},
	{
		2, {0x98, 0xE0},
		3, {0xE5, 0x80, 0xA8},
		2, {0x28, 0x50},
		2, {0xD0, 0xE2}
	},
	{
		2, {0x98, 0xE1},
		3, {0xE5, 0x80, 0x94},
		2, {0x14, 0x50},
		2, {0xD0, 0xE3}
	},
	{
		2, {0x98, 0xE2},
		3, {0xE5, 0x80, 0xAA},
		2, {0x2A, 0x50},
		2, {0xD0, 0xE4}
	},
	{
		2, {0x98, 0xE3},
		3, {0xE5, 0x80, 0xA5},
		2, {0x25, 0x50},
		2, {0xD0, 0xE5}
	},
	{
		2, {0x98, 0xE4},
		3, {0xE5, 0x80, 0x85},
		2, {0x05, 0x50},
		2, {0xD0, 0xE6}
	},
	{
		2, {0x98, 0xE5},
		3, {0xE4, 0xBC, 0x9C},
		2, {0x1C, 0x4F},
		2, {0xD0, 0xE7}
	},
	{
		2, {0x98, 0xE6},
		3, {0xE4, 0xBF, 0xB6},
		2, {0xF6, 0x4F},
		2, {0xD0, 0xE8}
	},
	{
		2, {0x98, 0xE7},
		3, {0xE5, 0x80, 0xA1},
		2, {0x21, 0x50},
		2, {0xD0, 0xE9}
	},
	{
		2, {0x98, 0xE8},
		3, {0xE5, 0x80, 0xA9},
		2, {0x29, 0x50},
		2, {0xD0, 0xEA}
	},
	{
		2, {0x98, 0xE9},
		3, {0xE5, 0x80, 0xAC},
		2, {0x2C, 0x50},
		2, {0xD0, 0xEB}
	},
	{
		2, {0x98, 0xEA},
		3, {0xE4, 0xBF, 0xBE},
		2, {0xFE, 0x4F},
		2, {0xD0, 0xEC}
	},
	{
		2, {0x98, 0xEB},
		3, {0xE4, 0xBF, 0xAF},
		2, {0xEF, 0x4F},
		2, {0xD0, 0xED}
	},
	{
		2, {0x98, 0xEC},
		3, {0xE5, 0x80, 0x91},
		2, {0x11, 0x50},
		2, {0xD0, 0xEE}
	},
	{
		2, {0x98, 0xED},
		3, {0xE5, 0x80, 0x86},
		2, {0x06, 0x50},
		2, {0xD0, 0xEF}
	},
	{
		2, {0x98, 0xEE},
		3, {0xE5, 0x81, 0x83},
		2, {0x43, 0x50},
		2, {0xD0, 0xF0}
	},
	{
		2, {0x98, 0xEF},
		3, {0xE5, 0x81, 0x87},
		2, {0x47, 0x50},
		2, {0xD0, 0xF1}
	},
	{
		2, {0x98, 0xF0},
		3, {0xE6, 0x9C, 0x83},
		2, {0x03, 0x67},
		2, {0xD0, 0xF2}
	},
	{
		2, {0x98, 0xF1},
		3, {0xE5, 0x81, 0x95},
		2, {0x55, 0x50},
		2, {0xD0, 0xF3}
	},
	{
		2, {0x98, 0xF2},
		3, {0xE5, 0x81, 0x90},
		2, {0x50, 0x50},
		2, {0xD0, 0xF4}
	},
	{
		2, {0x98, 0xF3},
		3, {0xE5, 0x81, 0x88},
		2, {0x48, 0x50},
		2, {0xD0, 0xF5}
	},
	{
		2, {0x98, 0xF4},
		3, {0xE5, 0x81, 0x9A},
		2, {0x5A, 0x50},
		2, {0xD0, 0xF6}
	},
	{
		2, {0x98, 0xF5},
		3, {0xE5, 0x81, 0x96},
		2, {0x56, 0x50},
		2, {0xD0, 0xF7}
	},
	{
		2, {0x98, 0xF6},
		3, {0xE5, 0x81, 0xAC},
		2, {0x6C, 0x50},
		2, {0xD0, 0xF8}
	},
	{
		2, {0x98, 0xF7},
		3, {0xE5, 0x81, 0xB8},
		2, {0x78, 0x50},
		2, {0xD0, 0xF9}
	},
	{
		2, {0x98, 0xF8},
		3, {0xE5, 0x82, 0x80},
		2, {0x80, 0x50},
		2, {0xD0, 0xFA}
	},
	{
		2, {0x98, 0xF9},
		3, {0xE5, 0x82, 0x9A},
		2, {0x9A, 0x50},
		2, {0xD0, 0xFB}
	},
	{
		2, {0x98, 0xFA},
		3, {0xE5, 0x82, 0x85},
		2, {0x85, 0x50},
		2, {0xD0, 0xFC}
	},
	{
		2, {0x98, 0xFB},
		3, {0xE5, 0x82, 0xB4},
		2, {0xB4, 0x50},
		2, {0xD0, 0xFD}
	},
	{
		2, {0x98, 0xFC},
		3, {0xE5, 0x82, 0xB2},
		2, {0xB2, 0x50},
		2, {0xD0, 0xFE}
	},
	{
		2, {0x99, 0x40},
		3, {0xE5, 0x83, 0x89},
		2, {0xC9, 0x50},
		2, {0xD1, 0xA1}
	},
	{
		2, {0x99, 0x41},
		3, {0xE5, 0x83, 0x8A},
		2, {0xCA, 0x50},
		2, {0xD1, 0xA2}
	},
	{
		2, {0x99, 0x42},
		3, {0xE5, 0x82, 0xB3},
		2, {0xB3, 0x50},
		2, {0xD1, 0xA3}
	},
	{
		2, {0x99, 0x43},
		3, {0xE5, 0x83, 0x82},
		2, {0xC2, 0x50},
		2, {0xD1, 0xA4}
	},
	{
		2, {0x99, 0x44},
		3, {0xE5, 0x83, 0x96},
		2, {0xD6, 0x50},
		2, {0xD1, 0xA5}
	},
	{
		2, {0x99, 0x45},
		3, {0xE5, 0x83, 0x9E},
		2, {0xDE, 0x50},
		2, {0xD1, 0xA6}
	},
	{
		2, {0x99, 0x46},
		3, {0xE5, 0x83, 0xA5},
		2, {0xE5, 0x50},
		2, {0xD1, 0xA7}
	},
	{
		2, {0x99, 0x47},
		3, {0xE5, 0x83, 0xAD},
		2, {0xED, 0x50},
		2, {0xD1, 0xA8}
	},
	{
		2, {0x99, 0x48},
		3, {0xE5, 0x83, 0xA3},
		2, {0xE3, 0x50},
		2, {0xD1, 0xA9}
	},
	{
		2, {0x99, 0x49},
		3, {0xE5, 0x83, 0xAE},
		2, {0xEE, 0x50},
		2, {0xD1, 0xAA}
	},
	{
		2, {0x99, 0x4A},
		3, {0xE5, 0x83, 0xB9},
		2, {0xF9, 0x50},
		2, {0xD1, 0xAB}
	},
	{
		2, {0x99, 0x4B},
		3, {0xE5, 0x83, 0xB5},
		2, {0xF5, 0x50},
		2, {0xD1, 0xAC}
	},
	{
		2, {0x99, 0x4C},
		3, {0xE5, 0x84, 0x89},
		2, {0x09, 0x51},
		2, {0xD1, 0xAD}
	},
	{
		2, {0x99, 0x4D},
		3, {0xE5, 0x84, 0x81},
		2, {0x01, 0x51},
		2, {0xD1, 0xAE}
	},
	{
		2, {0x99, 0x4E},
		3, {0xE5, 0x84, 0x82},
		2, {0x02, 0x51},
		2, {0xD1, 0xAF}
	},
	{
		2, {0x99, 0x4F},
		3, {0xE5, 0x84, 0x96},
		2, {0x16, 0x51},
		2, {0xD1, 0xB0}
	},
	{
		2, {0x99, 0x50},
		3, {0xE5, 0x84, 0x95},
		2, {0x15, 0x51},
		2, {0xD1, 0xB1}
	},
	{
		2, {0x99, 0x51},
		3, {0xE5, 0x84, 0x94},
		2, {0x14, 0x51},
		2, {0xD1, 0xB2}
	},
	{
		2, {0x99, 0x52},
		3, {0xE5, 0x84, 0x9A},
		2, {0x1A, 0x51},
		2, {0xD1, 0xB3}
	},
	{
		2, {0x99, 0x53},
		3, {0xE5, 0x84, 0xA1},
		2, {0x21, 0x51},
		2, {0xD1, 0xB4}
	},
	{
		2, {0x99, 0x54},
		3, {0xE5, 0x84, 0xBA},
		2, {0x3A, 0x51},
		2, {0xD1, 0xB5}
	},
	{
		2, {0x99, 0x55},
		3, {0xE5, 0x84, 0xB7},
		2, {0x37, 0x51},
		2, {0xD1, 0xB6}
	},
	{
		2, {0x99, 0x56},
		3, {0xE5, 0x84, 0xBC},
		2, {0x3C, 0x51},
		2, {0xD1, 0xB7}
	},
	{
		2, {0x99, 0x57},
		3, {0xE5, 0x84, 0xBB},
		2, {0x3B, 0x51},
		2, {0xD1, 0xB8}
	},
	{
		2, {0x99, 0x58},
		3, {0xE5, 0x84, 0xBF},
		2, {0x3F, 0x51},
		2, {0xD1, 0xB9}
	},
	{
		2, {0x99, 0x59},
		3, {0xE5, 0x85, 0x80},
		2, {0x40, 0x51},
		2, {0xD1, 0xBA}
	},
	{
		2, {0x99, 0x5A},
		3, {0xE5, 0x85, 0x92},
		2, {0x52, 0x51},
		2, {0xD1, 0xBB}
	},
	{
		2, {0x99, 0x5B},
		3, {0xE5, 0x85, 0x8C},
		2, {0x4C, 0x51},
		2, {0xD1, 0xBC}
	},
	{
		2, {0x99, 0x5C},
		3, {0xE5, 0x85, 0x94},
		2, {0x54, 0x51},
		2, {0xD1, 0xBD}
	},
	{
		2, {0x99, 0x5D},
		3, {0xE5, 0x85, 0xA2},
		2, {0x62, 0x51},
		2, {0xD1, 0xBE}
	},
	{
		2, {0x99, 0x5E},
		3, {0xE7, 0xAB, 0xB8},
		2, {0xF8, 0x7A},
		2, {0xD1, 0xBF}
	},
	{
		2, {0x99, 0x5F},
		3, {0xE5, 0x85, 0xA9},
		2, {0x69, 0x51},
		2, {0xD1, 0xC0}
	},
	{
		2, {0x99, 0x60},
		3, {0xE5, 0x85, 0xAA},
		2, {0x6A, 0x51},
		2, {0xD1, 0xC1}
	},
	{
		2, {0x99, 0x61},
		3, {0xE5, 0x85, 0xAE},
		2, {0x6E, 0x51},
		2, {0xD1, 0xC2}
	},
	{
		2, {0x99, 0x62},
		3, {0xE5, 0x86, 0x80},
		2, {0x80, 0x51},
		2, {0xD1, 0xC3}
	},
	{
		2, {0x99, 0x63},
		3, {0xE5, 0x86, 0x82},
		2, {0x82, 0x51},
		2, {0xD1, 0xC4}
	},
	{
		2, {0x99, 0x64},
		3, {0xE5, 0x9B, 0x98},
		2, {0xD8, 0x56},
		2, {0xD1, 0xC5}
	},
	{
		2, {0x99, 0x65},
		3, {0xE5, 0x86, 0x8C},
		2, {0x8C, 0x51},
		2, {0xD1, 0xC6}
	},
	{
		2, {0x99, 0x66},
		3, {0xE5, 0x86, 0x89},
		2, {0x89, 0x51},
		2, {0xD1, 0xC7}
	},
	{
		2, {0x99, 0x67},
		3, {0xE5, 0x86, 0x8F},
		2, {0x8F, 0x51},
		2, {0xD1, 0xC8}
	},
	{
		2, {0x99, 0x68},
		3, {0xE5, 0x86, 0x91},
		2, {0x91, 0x51},
		2, {0xD1, 0xC9}
	},
	{
		2, {0x99, 0x69},
		3, {0xE5, 0x86, 0x93},
		2, {0x93, 0x51},
		2, {0xD1, 0xCA}
	},
	{
		2, {0x99, 0x6A},
		3, {0xE5, 0x86, 0x95},
		2, {0x95, 0x51},
		2, {0xD1, 0xCB}
	},
	{
		2, {0x99, 0x6B},
		3, {0xE5, 0x86, 0x96},
		2, {0x96, 0x51},
		2, {0xD1, 0xCC}
	},
	{
		2, {0x99, 0x6C},
		3, {0xE5, 0x86, 0xA4},
		2, {0xA4, 0x51},
		2, {0xD1, 0xCD}
	},
	{
		2, {0x99, 0x6D},
		3, {0xE5, 0x86, 0xA6},
		2, {0xA6, 0x51},
		2, {0xD1, 0xCE}
	},
	{
		2, {0x99, 0x6E},
		3, {0xE5, 0x86, 0xA2},
		2, {0xA2, 0x51},
		2, {0xD1, 0xCF}
	},
	{
		2, {0x99, 0x6F},
		3, {0xE5, 0x86, 0xA9},
		2, {0xA9, 0x51},
		2, {0xD1, 0xD0}
	},
	{
		2, {0x99, 0x70},
		3, {0xE5, 0x86, 0xAA},
		2, {0xAA, 0x51},
		2, {0xD1, 0xD1}
	},
	{
		2, {0x99, 0x71},
		3, {0xE5, 0x86, 0xAB},
		2, {0xAB, 0x51},
		2, {0xD1, 0xD2}
	},
	{
		2, {0x99, 0x72},
		3, {0xE5, 0x86, 0xB3},
		2, {0xB3, 0x51},
		2, {0xD1, 0xD3}
	},
	{
		2, {0x99, 0x73},
		3, {0xE5, 0x86, 0xB1},
		2, {0xB1, 0x51},
		2, {0xD1, 0xD4}
	},
	{
		2, {0x99, 0x74},
		3, {0xE5, 0x86, 0xB2},
		2, {0xB2, 0x51},
		2, {0xD1, 0xD5}
	},
	{
		2, {0x99, 0x75},
		3, {0xE5, 0x86, 0xB0},
		2, {0xB0, 0x51},
		2, {0xD1, 0xD6}
	},
	{
		2, {0x99, 0x76},
		3, {0xE5, 0x86, 0xB5},
		2, {0xB5, 0x51},
		2, {0xD1, 0xD7}
	},
	{
		2, {0x99, 0x77},
		3, {0xE5, 0x86, 0xBD},
		2, {0xBD, 0x51},
		2, {0xD1, 0xD8}
	},
	{
		2, {0x99, 0x78},
		3, {0xE5, 0x87, 0x85},
		2, {0xC5, 0x51},
		2, {0xD1, 0xD9}
	},
	{
		2, {0x99, 0x79},
		3, {0xE5, 0x87, 0x89},
		2, {0xC9, 0x51},
		2, {0xD1, 0xDA}
	},
	{
		2, {0x99, 0x7A},
		3, {0xE5, 0x87, 0x9B},
		2, {0xDB, 0x51},
		2, {0xD1, 0xDB}
	},
	{
		2, {0x99, 0x7B},
		3, {0xE5, 0x87, 0xA0},
		2, {0xE0, 0x51},
		2, {0xD1, 0xDC}
	},
	{
		2, {0x99, 0x7C},
		3, {0xE8, 0x99, 0x95},
		2, {0x55, 0x86},
		2, {0xD1, 0xDD}
	},
	{
		2, {0x99, 0x7D},
		3, {0xE5, 0x87, 0xA9},
		2, {0xE9, 0x51},
		2, {0xD1, 0xDE}
	},
	{
		2, {0x99, 0x7E},
		3, {0xE5, 0x87, 0xAD},
		2, {0xED, 0x51},
		2, {0xD1, 0xDF}
	},
	{
		2, {0x99, 0x80},
		3, {0xE5, 0x87, 0xB0},
		2, {0xF0, 0x51},
		2, {0xD1, 0xE0}
	},
	{
		2, {0x99, 0x81},
		3, {0xE5, 0x87, 0xB5},
		2, {0xF5, 0x51},
		2, {0xD1, 0xE1}
	},
	{
		2, {0x99, 0x82},
		3, {0xE5, 0x87, 0xBE},
		2, {0xFE, 0x51},
		2, {0xD1, 0xE2}
	},
	{
		2, {0x99, 0x83},
		3, {0xE5, 0x88, 0x84},
		2, {0x04, 0x52},
		2, {0xD1, 0xE3}
	},
	{
		2, {0x99, 0x84},
		3, {0xE5, 0x88, 0x8B},
		2, {0x0B, 0x52},
		2, {0xD1, 0xE4}
	},
	{
		2, {0x99, 0x85},
		3, {0xE5, 0x88, 0x94},
		2, {0x14, 0x52},
		2, {0xD1, 0xE5}
	},
	{
		2, {0x99, 0x86},
		3, {0xE5, 0x88, 0x8E},
		2, {0x0E, 0x52},
		2, {0xD1, 0xE6}
	},
	{
		2, {0x99, 0x87},
		3, {0xE5, 0x88, 0xA7},
		2, {0x27, 0x52},
		2, {0xD1, 0xE7}
	},
	{
		2, {0x99, 0x88},
		3, {0xE5, 0x88, 0xAA},
		2, {0x2A, 0x52},
		2, {0xD1, 0xE8}
	},
	{
		2, {0x99, 0x89},
		3, {0xE5, 0x88, 0xAE},
		2, {0x2E, 0x52},
		2, {0xD1, 0xE9}
	},
	{
		2, {0x99, 0x8A},
		3, {0xE5, 0x88, 0xB3},
		2, {0x33, 0x52},
		2, {0xD1, 0xEA}
	},
	{
		2, {0x99, 0x8B},
		3, {0xE5, 0x88, 0xB9},
		2, {0x39, 0x52},
		2, {0xD1, 0xEB}
	},
	{
		2, {0x99, 0x8C},
		3, {0xE5, 0x89, 0x8F},
		2, {0x4F, 0x52},
		2, {0xD1, 0xEC}
	},
	{
		2, {0x99, 0x8D},
		3, {0xE5, 0x89, 0x84},
		2, {0x44, 0x52},
		2, {0xD1, 0xED}
	},
	{
		2, {0x99, 0x8E},
		3, {0xE5, 0x89, 0x8B},
		2, {0x4B, 0x52},
		2, {0xD1, 0xEE}
	},
	{
		2, {0x99, 0x8F},
		3, {0xE5, 0x89, 0x8C},
		2, {0x4C, 0x52},
		2, {0xD1, 0xEF}
	},
	{
		2, {0x99, 0x90},
		3, {0xE5, 0x89, 0x9E},
		2, {0x5E, 0x52},
		2, {0xD1, 0xF0}
	},
	{
		2, {0x99, 0x91},
		3, {0xE5, 0x89, 0x94},
		2, {0x54, 0x52},
		2, {0xD1, 0xF1}
	},
	{
		2, {0x99, 0x92},
		3, {0xE5, 0x89, 0xAA},
		2, {0x6A, 0x52},
		2, {0xD1, 0xF2}
	},
	{
		2, {0x99, 0x93},
		3, {0xE5, 0x89, 0xB4},
		2, {0x74, 0x52},
		2, {0xD1, 0xF3}
	},
	{
		2, {0x99, 0x94},
		3, {0xE5, 0x89, 0xA9},
		2, {0x69, 0x52},
		2, {0xD1, 0xF4}
	},
	{
		2, {0x99, 0x95},
		3, {0xE5, 0x89, 0xB3},
		2, {0x73, 0x52},
		2, {0xD1, 0xF5}
	},
	{
		2, {0x99, 0x96},
		3, {0xE5, 0x89, 0xBF},
		2, {0x7F, 0x52},
		2, {0xD1, 0xF6}
	},
	{
		2, {0x99, 0x97},
		3, {0xE5, 0x89, 0xBD},
		2, {0x7D, 0x52},
		2, {0xD1, 0xF7}
	},
	{
		2, {0x99, 0x98},
		3, {0xE5, 0x8A, 0x8D},
		2, {0x8D, 0x52},
		2, {0xD1, 0xF8}
	},
	{
		2, {0x99, 0x99},
		3, {0xE5, 0x8A, 0x94},
		2, {0x94, 0x52},
		2, {0xD1, 0xF9}
	},
	{
		2, {0x99, 0x9A},
		3, {0xE5, 0x8A, 0x92},
		2, {0x92, 0x52},
		2, {0xD1, 0xFA}
	},
	{
		2, {0x99, 0x9B},
		3, {0xE5, 0x89, 0xB1},
		2, {0x71, 0x52},
		2, {0xD1, 0xFB}
	},
	{
		2, {0x99, 0x9C},
		3, {0xE5, 0x8A, 0x88},
		2, {0x88, 0x52},
		2, {0xD1, 0xFC}
	},
	{
		2, {0x99, 0x9D},
		3, {0xE5, 0x8A, 0x91},
		2, {0x91, 0x52},
		2, {0xD1, 0xFD}
	},
	{
		2, {0x99, 0x9E},
		3, {0xE8, 0xBE, 0xA8},
		2, {0xA8, 0x8F},
		2, {0xD1, 0xFE}
	},
	{
		2, {0x99, 0x9F},
		3, {0xE8, 0xBE, 0xA7},
		2, {0xA7, 0x8F},
		2, {0xD2, 0xA1}
	},
	{
		2, {0x99, 0xA0},
		3, {0xE5, 0x8A, 0xAC},
		2, {0xAC, 0x52},
		2, {0xD2, 0xA2}
	},
	{
		2, {0x99, 0xA1},
		3, {0xE5, 0x8A, 0xAD},
		2, {0xAD, 0x52},
		2, {0xD2, 0xA3}
	},
	{
		2, {0x99, 0xA2},
		3, {0xE5, 0x8A, 0xBC},
		2, {0xBC, 0x52},
		2, {0xD2, 0xA4}
	},
	{
		2, {0x99, 0xA3},
		3, {0xE5, 0x8A, 0xB5},
		2, {0xB5, 0x52},
		2, {0xD2, 0xA5}
	},
	{
		2, {0x99, 0xA4},
		3, {0xE5, 0x8B, 0x81},
		2, {0xC1, 0x52},
		2, {0xD2, 0xA6}
	},
	{
		2, {0x99, 0xA5},
		3, {0xE5, 0x8B, 0x8D},
		2, {0xCD, 0x52},
		2, {0xD2, 0xA7}
	},
	{
		2, {0x99, 0xA6},
		3, {0xE5, 0x8B, 0x97},
		2, {0xD7, 0x52},
		2, {0xD2, 0xA8}
	},
	{
		2, {0x99, 0xA7},
		3, {0xE5, 0x8B, 0x9E},
		2, {0xDE, 0x52},
		2, {0xD2, 0xA9}
	},
	{
		2, {0x99, 0xA8},
		3, {0xE5, 0x8B, 0xA3},
		2, {0xE3, 0x52},
		2, {0xD2, 0xAA}
	},
	{
		2, {0x99, 0xA9},
		3, {0xE5, 0x8B, 0xA6},
		2, {0xE6, 0x52},
		2, {0xD2, 0xAB}
	},
	{
		2, {0x99, 0xAA},
		3, {0xE9, 0xA3, 0xAD},
		2, {0xED, 0x98},
		2, {0xD2, 0xAC}
	},
	{
		2, {0x99, 0xAB},
		3, {0xE5, 0x8B, 0xA0},
		2, {0xE0, 0x52},
		2, {0xD2, 0xAD}
	},
	{
		2, {0x99, 0xAC},
		3, {0xE5, 0x8B, 0xB3},
		2, {0xF3, 0x52},
		2, {0xD2, 0xAE}
	},
	{
		2, {0x99, 0xAD},
		3, {0xE5, 0x8B, 0xB5},
		2, {0xF5, 0x52},
		2, {0xD2, 0xAF}
	},
	{
		2, {0x99, 0xAE},
		3, {0xE5, 0x8B, 0xB8},
		2, {0xF8, 0x52},
		2, {0xD2, 0xB0}
	},
	{
		2, {0x99, 0xAF},
		3, {0xE5, 0x8B, 0xB9},
		2, {0xF9, 0x52},
		2, {0xD2, 0xB1}
	},
	{
		2, {0x99, 0xB0},
		3, {0xE5, 0x8C, 0x86},
		2, {0x06, 0x53},
		2, {0xD2, 0xB2}
	},
	{
		2, {0x99, 0xB1},
		3, {0xE5, 0x8C, 0x88},
		2, {0x08, 0x53},
		2, {0xD2, 0xB3}
	},
	{
		2, {0x99, 0xB2},
		3, {0xE7, 0x94, 0xB8},
		2, {0x38, 0x75},
		2, {0xD2, 0xB4}
	},
	{
		2, {0x99, 0xB3},
		3, {0xE5, 0x8C, 0x8D},
		2, {0x0D, 0x53},
		2, {0xD2, 0xB5}
	},
	{
		2, {0x99, 0xB4},
		3, {0xE5, 0x8C, 0x90},
		2, {0x10, 0x53},
		2, {0xD2, 0xB6}
	},
	{
		2, {0x99, 0xB5},
		3, {0xE5, 0x8C, 0x8F},
		2, {0x0F, 0x53},
		2, {0xD2, 0xB7}
	},
	{
		2, {0x99, 0xB6},
		3, {0xE5, 0x8C, 0x95},
		2, {0x15, 0x53},
		2, {0xD2, 0xB8}
	},
	{
		2, {0x99, 0xB7},
		3, {0xE5, 0x8C, 0x9A},
		2, {0x1A, 0x53},
		2, {0xD2, 0xB9}
	},
	{
		2, {0x99, 0xB8},
		3, {0xE5, 0x8C, 0xA3},
		2, {0x23, 0x53},
		2, {0xD2, 0xBA}
	},
	{
		2, {0x99, 0xB9},
		3, {0xE5, 0x8C, 0xAF},
		2, {0x2F, 0x53},
		2, {0xD2, 0xBB}
	},
	{
		2, {0x99, 0xBA},
		3, {0xE5, 0x8C, 0xB1},
		2, {0x31, 0x53},
		2, {0xD2, 0xBC}
	},
	{
		2, {0x99, 0xBB},
		3, {0xE5, 0x8C, 0xB3},
		2, {0x33, 0x53},
		2, {0xD2, 0xBD}
	},
	{
		2, {0x99, 0xBC},
		3, {0xE5, 0x8C, 0xB8},
		2, {0x38, 0x53},
		2, {0xD2, 0xBE}
	},
	{
		2, {0x99, 0xBD},
		3, {0xE5, 0x8D, 0x80},
		2, {0x40, 0x53},
		2, {0xD2, 0xBF}
	},
	{
		2, {0x99, 0xBE},
		3, {0xE5, 0x8D, 0x86},
		2, {0x46, 0x53},
		2, {0xD2, 0xC0}
	},
	{
		2, {0x99, 0xBF},
		3, {0xE5, 0x8D, 0x85},
		2, {0x45, 0x53},
		2, {0xD2, 0xC1}
	},
	{
		2, {0x99, 0xC0},
		3, {0xE4, 0xB8, 0x97},
		2, {0x17, 0x4E},
		2, {0xD2, 0xC2}
	},
	{
		2, {0x99, 0xC1},
		3, {0xE5, 0x8D, 0x89},
		2, {0x49, 0x53},
		2, {0xD2, 0xC3}
	},
	{
		2, {0x99, 0xC2},
		3, {0xE5, 0x8D, 0x8D},
		2, {0x4D, 0x53},
		2, {0xD2, 0xC4}
	},
	{
		2, {0x99, 0xC3},
		3, {0xE5, 0x87, 0x96},
		2, {0xD6, 0x51},
		2, {0xD2, 0xC5}
	},
	{
		2, {0x99, 0xC4},
		3, {0xE5, 0x8D, 0x9E},
		2, {0x5E, 0x53},
		2, {0xD2, 0xC6}
	},
	{
		2, {0x99, 0xC5},
		3, {0xE5, 0x8D, 0xA9},
		2, {0x69, 0x53},
		2, {0xD2, 0xC7}
	},
	{
		2, {0x99, 0xC6},
		3, {0xE5, 0x8D, 0xAE},
		2, {0x6E, 0x53},
		2, {0xD2, 0xC8}
	},
	{
		2, {0x99, 0xC7},
		3, {0xE5, 0xA4, 0x98},
		2, {0x18, 0x59},
		2, {0xD2, 0xC9}
	},
	{
		2, {0x99, 0xC8},
		3, {0xE5, 0x8D, 0xBB},
		2, {0x7B, 0x53},
		2, {0xD2, 0xCA}
	},
	{
		2, {0x99, 0xC9},
		3, {0xE5, 0x8D, 0xB7},
		2, {0x77, 0x53},
		2, {0xD2, 0xCB}
	},
	{
		2, {0x99, 0xCA},
		3, {0xE5, 0x8E, 0x82},
		2, {0x82, 0x53},
		2, {0xD2, 0xCC}
	},
	{
		2, {0x99, 0xCB},
		3, {0xE5, 0x8E, 0x96},
		2, {0x96, 0x53},
		2, {0xD2, 0xCD}
	},
	{
		2, {0x99, 0xCC},
		3, {0xE5, 0x8E, 0xA0},
		2, {0xA0, 0x53},
		2, {0xD2, 0xCE}
	},
	{
		2, {0x99, 0xCD},
		3, {0xE5, 0x8E, 0xA6},
		2, {0xA6, 0x53},
		2, {0xD2, 0xCF}
	},
	{
		2, {0x99, 0xCE},
		3, {0xE5, 0x8E, 0xA5},
		2, {0xA5, 0x53},
		2, {0xD2, 0xD0}
	},
	{
		2, {0x99, 0xCF},
		3, {0xE5, 0x8E, 0xAE},
		2, {0xAE, 0x53},
		2, {0xD2, 0xD1}
	},
	{
		2, {0x99, 0xD0},
		3, {0xE5, 0x8E, 0xB0},
		2, {0xB0, 0x53},
		2, {0xD2, 0xD2}
	},
	{
		2, {0x99, 0xD1},
		3, {0xE5, 0x8E, 0xB6},
		2, {0xB6, 0x53},
		2, {0xD2, 0xD3}
	},
	{
		2, {0x99, 0xD2},
		3, {0xE5, 0x8F, 0x83},
		2, {0xC3, 0x53},
		2, {0xD2, 0xD4}
	},
	{
		2, {0x99, 0xD3},
		3, {0xE7, 0xB0, 0x92},
		2, {0x12, 0x7C},
		2, {0xD2, 0xD5}
	},
	{
		2, {0x99, 0xD4},
		3, {0xE9, 0x9B, 0x99},
		2, {0xD9, 0x96},
		2, {0xD2, 0xD6}
	},
	{
		2, {0x99, 0xD5},
		3, {0xE5, 0x8F, 0x9F},
		2, {0xDF, 0x53},
		2, {0xD2, 0xD7}
	},
	{
		2, {0x99, 0xD6},
		3, {0xE6, 0x9B, 0xBC},
		2, {0xFC, 0x66},
		2, {0xD2, 0xD8}
	},
	{
		2, {0x99, 0xD7},
		3, {0xE7, 0x87, 0xAE},
		2, {0xEE, 0x71},
		2, {0xD2, 0xD9}
	},
	{
		2, {0x99, 0xD8},
		3, {0xE5, 0x8F, 0xAE},
		2, {0xEE, 0x53},
		2, {0xD2, 0xDA}
	},
	{
		2, {0x99, 0xD9},
		3, {0xE5, 0x8F, 0xA8},
		2, {0xE8, 0x53},
		2, {0xD2, 0xDB}
	},
	{
		2, {0x99, 0xDA},
		3, {0xE5, 0x8F, 0xAD},
		2, {0xED, 0x53},
		2, {0xD2, 0xDC}
	},
	{
		2, {0x99, 0xDB},
		3, {0xE5, 0x8F, 0xBA},
		2, {0xFA, 0x53},
		2, {0xD2, 0xDD}
	},
	{
		2, {0x99, 0xDC},
		3, {0xE5, 0x90, 0x81},
		2, {0x01, 0x54},
		2, {0xD2, 0xDE}
	},
	{
		2, {0x99, 0xDD},
		3, {0xE5, 0x90, 0xBD},
		2, {0x3D, 0x54},
		2, {0xD2, 0xDF}
	},
	{
		2, {0x99, 0xDE},
		3, {0xE5, 0x91, 0x80},
		2, {0x40, 0x54},
		2, {0xD2, 0xE0}
	},
	{
		2, {0x99, 0xDF},
		3, {0xE5, 0x90, 0xAC},
		2, {0x2C, 0x54},
		2, {0xD2, 0xE1}
	},
	{
		2, {0x99, 0xE0},
		3, {0xE5, 0x90, 0xAD},
		2, {0x2D, 0x54},
		2, {0xD2, 0xE2}
	},
	{
		2, {0x99, 0xE1},
		3, {0xE5, 0x90, 0xBC},
		2, {0x3C, 0x54},
		2, {0xD2, 0xE3}
	},
	{
		2, {0x99, 0xE2},
		3, {0xE5, 0x90, 0xAE},
		2, {0x2E, 0x54},
		2, {0xD2, 0xE4}
	},
	{
		2, {0x99, 0xE3},
		3, {0xE5, 0x90, 0xB6},
		2, {0x36, 0x54},
		2, {0xD2, 0xE5}
	},
	{
		2, {0x99, 0xE4},
		3, {0xE5, 0x90, 0xA9},
		2, {0x29, 0x54},
		2, {0xD2, 0xE6}
	},
	{
		2, {0x99, 0xE5},
		3, {0xE5, 0x90, 0x9D},
		2, {0x1D, 0x54},
		2, {0xD2, 0xE7}
	},
	{
		2, {0x99, 0xE6},
		3, {0xE5, 0x91, 0x8E},
		2, {0x4E, 0x54},
		2, {0xD2, 0xE8}
	},
	{
		2, {0x99, 0xE7},
		3, {0xE5, 0x92, 0x8F},
		2, {0x8F, 0x54},
		2, {0xD2, 0xE9}
	},
	{
		2, {0x99, 0xE8},
		3, {0xE5, 0x91, 0xB5},
		2, {0x75, 0x54},
		2, {0xD2, 0xEA}
	},
	{
		2, {0x99, 0xE9},
		3, {0xE5, 0x92, 0x8E},
		2, {0x8E, 0x54},
		2, {0xD2, 0xEB}
	},
	{
		2, {0x99, 0xEA},
		3, {0xE5, 0x91, 0x9F},
		2, {0x5F, 0x54},
		2, {0xD2, 0xEC}
	},
	{
		2, {0x99, 0xEB},
		3, {0xE5, 0x91, 0xB1},
		2, {0x71, 0x54},
		2, {0xD2, 0xED}
	},
	{
		2, {0x99, 0xEC},
		3, {0xE5, 0x91, 0xB7},
		2, {0x77, 0x54},
		2, {0xD2, 0xEE}
	},
	{
		2, {0x99, 0xED},
		3, {0xE5, 0x91, 0xB0},
		2, {0x70, 0x54},
		2, {0xD2, 0xEF}
	},
	{
		2, {0x99, 0xEE},
		3, {0xE5, 0x92, 0x92},
		2, {0x92, 0x54},
		2, {0xD2, 0xF0}
	},
	{
		2, {0x99, 0xEF},
		3, {0xE5, 0x91, 0xBB},
		2, {0x7B, 0x54},
		2, {0xD2, 0xF1}
	},
	{
		2, {0x99, 0xF0},
		3, {0xE5, 0x92, 0x80},
		2, {0x80, 0x54},
		2, {0xD2, 0xF2}
	},
	{
		2, {0x99, 0xF1},
		3, {0xE5, 0x91, 0xB6},
		2, {0x76, 0x54},
		2, {0xD2, 0xF3}
	},
	{
		2, {0x99, 0xF2},
		3, {0xE5, 0x92, 0x84},
		2, {0x84, 0x54},
		2, {0xD2, 0xF4}
	},
	{
		2, {0x99, 0xF3},
		3, {0xE5, 0x92, 0x90},
		2, {0x90, 0x54},
		2, {0xD2, 0xF5}
	},
	{
		2, {0x99, 0xF4},
		3, {0xE5, 0x92, 0x86},
		2, {0x86, 0x54},
		2, {0xD2, 0xF6}
	},
	{
		2, {0x99, 0xF5},
		3, {0xE5, 0x93, 0x87},
		2, {0xC7, 0x54},
		2, {0xD2, 0xF7}
	},
	{
		2, {0x99, 0xF6},
		3, {0xE5, 0x92, 0xA2},
		2, {0xA2, 0x54},
		2, {0xD2, 0xF8}
	},
	{
		2, {0x99, 0xF7},
		3, {0xE5, 0x92, 0xB8},
		2, {0xB8, 0x54},
		2, {0xD2, 0xF9}
	},
	{
		2, {0x99, 0xF8},
		3, {0xE5, 0x92, 0xA5},
		2, {0xA5, 0x54},
		2, {0xD2, 0xFA}
	},
	{
		2, {0x99, 0xF9},
		3, {0xE5, 0x92, 0xAC},
		2, {0xAC, 0x54},
		2, {0xD2, 0xFB}
	},
	{
		2, {0x99, 0xFA},
		3, {0xE5, 0x93, 0x84},
		2, {0xC4, 0x54},
		2, {0xD2, 0xFC}
	},
	{
		2, {0x99, 0xFB},
		3, {0xE5, 0x93, 0x88},
		2, {0xC8, 0x54},
		2, {0xD2, 0xFD}
	},
	{
		2, {0x99, 0xFC},
		3, {0xE5, 0x92, 0xA8},
		2, {0xA8, 0x54},
		2, {0xD2, 0xFE}
	},
	{
		2, {0x9A, 0x40},
		3, {0xE5, 0x92, 0xAB},
		2, {0xAB, 0x54},
		2, {0xD3, 0xA1}
	},
	{
		2, {0x9A, 0x41},
		3, {0xE5, 0x93, 0x82},
		2, {0xC2, 0x54},
		2, {0xD3, 0xA2}
	},
	{
		2, {0x9A, 0x42},
		3, {0xE5, 0x92, 0xA4},
		2, {0xA4, 0x54},
		2, {0xD3, 0xA3}
	},
	{
		2, {0x9A, 0x43},
		3, {0xE5, 0x92, 0xBE},
		2, {0xBE, 0x54},
		2, {0xD3, 0xA4}
	},
	{
		2, {0x9A, 0x44},
		3, {0xE5, 0x92, 0xBC},
		2, {0xBC, 0x54},
		2, {0xD3, 0xA5}
	},
	{
		2, {0x9A, 0x45},
		3, {0xE5, 0x93, 0x98},
		2, {0xD8, 0x54},
		2, {0xD3, 0xA6}
	},
	{
		2, {0x9A, 0x46},
		3, {0xE5, 0x93, 0xA5},
		2, {0xE5, 0x54},
		2, {0xD3, 0xA7}
	},
	{
		2, {0x9A, 0x47},
		3, {0xE5, 0x93, 0xA6},
		2, {0xE6, 0x54},
		2, {0xD3, 0xA8}
	},
	{
		2, {0x9A, 0x48},
		3, {0xE5, 0x94, 0x8F},
		2, {0x0F, 0x55},
		2, {0xD3, 0xA9}
	},
	{
		2, {0x9A, 0x49},
		3, {0xE5, 0x94, 0x94},
		2, {0x14, 0x55},
		2, {0xD3, 0xAA}
	},
	{
		2, {0x9A, 0x4A},
		3, {0xE5, 0x93, 0xBD},
		2, {0xFD, 0x54},
		2, {0xD3, 0xAB}
	},
	{
		2, {0x9A, 0x4B},
		3, {0xE5, 0x93, 0xAE},
		2, {0xEE, 0x54},
		2, {0xD3, 0xAC}
	},
	{
		2, {0x9A, 0x4C},
		3, {0xE5, 0x93, 0xAD},
		2, {0xED, 0x54},
		2, {0xD3, 0xAD}
	},
	{
		2, {0x9A, 0x4D},
		3, {0xE5, 0x93, 0xBA},
		2, {0xFA, 0x54},
		2, {0xD3, 0xAE}
	},
	{
		2, {0x9A, 0x4E},
		3, {0xE5, 0x93, 0xA2},
		2, {0xE2, 0x54},
		2, {0xD3, 0xAF}
	},
	{
		2, {0x9A, 0x4F},
		3, {0xE5, 0x94, 0xB9},
		2, {0x39, 0x55},
		2, {0xD3, 0xB0}
	},
	{
		2, {0x9A, 0x50},
		3, {0xE5, 0x95, 0x80},
		2, {0x40, 0x55},
		2, {0xD3, 0xB1}
	},
	{
		2, {0x9A, 0x51},
		3, {0xE5, 0x95, 0xA3},
		2, {0x63, 0x55},
		2, {0xD3, 0xB2}
	},
	{
		2, {0x9A, 0x52},
		3, {0xE5, 0x95, 0x8C},
		2, {0x4C, 0x55},
		2, {0xD3, 0xB3}
	},
	{
		2, {0x9A, 0x53},
		3, {0xE5, 0x94, 0xAE},
		2, {0x2E, 0x55},
		2, {0xD3, 0xB4}
	},
	{
		2, {0x9A, 0x54},
		3, {0xE5, 0x95, 0x9C},
		2, {0x5C, 0x55},
		2, {0xD3, 0xB5}
	},
	{
		2, {0x9A, 0x55},
		3, {0xE5, 0x95, 0x85},
		2, {0x45, 0x55},
		2, {0xD3, 0xB6}
	},
	{
		2, {0x9A, 0x56},
		3, {0xE5, 0x95, 0x96},
		2, {0x56, 0x55},
		2, {0xD3, 0xB7}
	},
	{
		2, {0x9A, 0x57},
		3, {0xE5, 0x95, 0x97},
		2, {0x57, 0x55},
		2, {0xD3, 0xB8}
	},
	{
		2, {0x9A, 0x58},
		3, {0xE5, 0x94, 0xB8},
		2, {0x38, 0x55},
		2, {0xD3, 0xB9}
	},
	{
		2, {0x9A, 0x59},
		3, {0xE5, 0x94, 0xB3},
		2, {0x33, 0x55},
		2, {0xD3, 0xBA}
	},
	{
		2, {0x9A, 0x5A},
		3, {0xE5, 0x95, 0x9D},
		2, {0x5D, 0x55},
		2, {0xD3, 0xBB}
	},
	{
		2, {0x9A, 0x5B},
		3, {0xE5, 0x96, 0x99},
		2, {0x99, 0x55},
		2, {0xD3, 0xBC}
	},
	{
		2, {0x9A, 0x5C},
		3, {0xE5, 0x96, 0x80},
		2, {0x80, 0x55},
		2, {0xD3, 0xBD}
	},
	{
		2, {0x9A, 0x5D},
		3, {0xE5, 0x92, 0xAF},
		2, {0xAF, 0x54},
		2, {0xD3, 0xBE}
	},
	{
		2, {0x9A, 0x5E},
		3, {0xE5, 0x96, 0x8A},
		2, {0x8A, 0x55},
		2, {0xD3, 0xBF}
	},
	{
		2, {0x9A, 0x5F},
		3, {0xE5, 0x96, 0x9F},
		2, {0x9F, 0x55},
		2, {0xD3, 0xC0}
	},
	{
		2, {0x9A, 0x60},
		3, {0xE5, 0x95, 0xBB},
		2, {0x7B, 0x55},
		2, {0xD3, 0xC1}
	},
	{
		2, {0x9A, 0x61},
		3, {0xE5, 0x95, 0xBE},
		2, {0x7E, 0x55},
		2, {0xD3, 0xC2}
	},
	{
		2, {0x9A, 0x62},
		3, {0xE5, 0x96, 0x98},
		2, {0x98, 0x55},
		2, {0xD3, 0xC3}
	},
	{
		2, {0x9A, 0x63},
		3, {0xE5, 0x96, 0x9E},
		2, {0x9E, 0x55},
		2, {0xD3, 0xC4}
	},
	{
		2, {0x9A, 0x64},
		3, {0xE5, 0x96, 0xAE},
		2, {0xAE, 0x55},
		2, {0xD3, 0xC5}
	},
	{
		2, {0x9A, 0x65},
		3, {0xE5, 0x95, 0xBC},
		2, {0x7C, 0x55},
		2, {0xD3, 0xC6}
	},
	{
		2, {0x9A, 0x66},
		3, {0xE5, 0x96, 0x83},
		2, {0x83, 0x55},
		2, {0xD3, 0xC7}
	},
	{
		2, {0x9A, 0x67},
		3, {0xE5, 0x96, 0xA9},
		2, {0xA9, 0x55},
		2, {0xD3, 0xC8}
	},
	{
		2, {0x9A, 0x68},
		3, {0xE5, 0x96, 0x87},
		2, {0x87, 0x55},
		2, {0xD3, 0xC9}
	},
	{
		2, {0x9A, 0x69},
		3, {0xE5, 0x96, 0xA8},
		2, {0xA8, 0x55},
		2, {0xD3, 0xCA}
	},
	{
		2, {0x9A, 0x6A},
		3, {0xE5, 0x97, 0x9A},
		2, {0xDA, 0x55},
		2, {0xD3, 0xCB}
	},
	{
		2, {0x9A, 0x6B},
		3, {0xE5, 0x97, 0x85},
		2, {0xC5, 0x55},
		2, {0xD3, 0xCC}
	},
	{
		2, {0x9A, 0x6C},
		3, {0xE5, 0x97, 0x9F},
		2, {0xDF, 0x55},
		2, {0xD3, 0xCD}
	},
	{
		2, {0x9A, 0x6D},
		3, {0xE5, 0x97, 0x84},
		2, {0xC4, 0x55},
		2, {0xD3, 0xCE}
	},
	{
		2, {0x9A, 0x6E},
		3, {0xE5, 0x97, 0x9C},
		2, {0xDC, 0x55},
		2, {0xD3, 0xCF}
	},
	{
		2, {0x9A, 0x6F},
		3, {0xE5, 0x97, 0xA4},
		2, {0xE4, 0x55},
		2, {0xD3, 0xD0}
	},
	{
		2, {0x9A, 0x70},
		3, {0xE5, 0x97, 0x94},
		2, {0xD4, 0x55},
		2, {0xD3, 0xD1}
	},
	{
		2, {0x9A, 0x71},
		3, {0xE5, 0x98, 0x94},
		2, {0x14, 0x56},
		2, {0xD3, 0xD2}
	},
	{
		2, {0x9A, 0x72},
		3, {0xE5, 0x97, 0xB7},
		2, {0xF7, 0x55},
		2, {0xD3, 0xD3}
	},
	{
		2, {0x9A, 0x73},
		3, {0xE5, 0x98, 0x96},
		2, {0x16, 0x56},
		2, {0xD3, 0xD4}
	},
	{
		2, {0x9A, 0x74},
		3, {0xE5, 0x97, 0xBE},
		2, {0xFE, 0x55},
		2, {0xD3, 0xD5}
	},
	{
		2, {0x9A, 0x75},
		3, {0xE5, 0x97, 0xBD},
		2, {0xFD, 0x55},
		2, {0xD3, 0xD6}
	},
	{
		2, {0x9A, 0x76},
		3, {0xE5, 0x98, 0x9B},
		2, {0x1B, 0x56},
		2, {0xD3, 0xD7}
	},
	{
		2, {0x9A, 0x77},
		3, {0xE5, 0x97, 0xB9},
		2, {0xF9, 0x55},
		2, {0xD3, 0xD8}
	},
	{
		2, {0x9A, 0x78},
		3, {0xE5, 0x99, 0x8E},
		2, {0x4E, 0x56},
		2, {0xD3, 0xD9}
	},
	{
		2, {0x9A, 0x79},
		3, {0xE5, 0x99, 0x90},
		2, {0x50, 0x56},
		2, {0xD3, 0xDA}
	},
	{
		2, {0x9A, 0x7A},
		3, {0xE7, 0x87, 0x9F},
		2, {0xDF, 0x71},
		2, {0xD3, 0xDB}
	},
	{
		2, {0x9A, 0x7B},
		3, {0xE5, 0x98, 0xB4},
		2, {0x34, 0x56},
		2, {0xD3, 0xDC}
	},
	{
		2, {0x9A, 0x7C},
		3, {0xE5, 0x98, 0xB6},
		2, {0x36, 0x56},
		2, {0xD3, 0xDD}
	},
	{
		2, {0x9A, 0x7D},
		3, {0xE5, 0x98, 0xB2},
		2, {0x32, 0x56},
		2, {0xD3, 0xDE}
	},
	{
		2, {0x9A, 0x7E},
		3, {0xE5, 0x98, 0xB8},
		2, {0x38, 0x56},
		2, {0xD3, 0xDF}
	},
	{
		2, {0x9A, 0x80},
		3, {0xE5, 0x99, 0xAB},
		2, {0x6B, 0x56},
		2, {0xD3, 0xE0}
	},
	{
		2, {0x9A, 0x81},
		3, {0xE5, 0x99, 0xA4},
		2, {0x64, 0x56},
		2, {0xD3, 0xE1}
	},
	{
		2, {0x9A, 0x82},
		3, {0xE5, 0x98, 0xAF},
		2, {0x2F, 0x56},
		2, {0xD3, 0xE2}
	},
	{
		2, {0x9A, 0x83},
		3, {0xE5, 0x99, 0xAC},
		2, {0x6C, 0x56},
		2, {0xD3, 0xE3}
	},
	{
		2, {0x9A, 0x84},
		3, {0xE5, 0x99, 0xAA},
		2, {0x6A, 0x56},
		2, {0xD3, 0xE4}
	},
	{
		2, {0x9A, 0x85},
		3, {0xE5, 0x9A, 0x86},
		2, {0x86, 0x56},
		2, {0xD3, 0xE5}
	},
	{
		2, {0x9A, 0x86},
		3, {0xE5, 0x9A, 0x80},
		2, {0x80, 0x56},
		2, {0xD3, 0xE6}
	},
	{
		2, {0x9A, 0x87},
		3, {0xE5, 0x9A, 0x8A},
		2, {0x8A, 0x56},
		2, {0xD3, 0xE7}
	},
	{
		2, {0x9A, 0x88},
		3, {0xE5, 0x9A, 0xA0},
		2, {0xA0, 0x56},
		2, {0xD3, 0xE8}
	},
	{
		2, {0x9A, 0x89},
		3, {0xE5, 0x9A, 0x94},
		2, {0x94, 0x56},
		2, {0xD3, 0xE9}
	},
	{
		2, {0x9A, 0x8A},
		3, {0xE5, 0x9A, 0x8F},
		2, {0x8F, 0x56},
		2, {0xD3, 0xEA}
	},
	{
		2, {0x9A, 0x8B},
		3, {0xE5, 0x9A, 0xA5},
		2, {0xA5, 0x56},
		2, {0xD3, 0xEB}
	},
	{
		2, {0x9A, 0x8C},
		3, {0xE5, 0x9A, 0xAE},
		2, {0xAE, 0x56},
		2, {0xD3, 0xEC}
	},
	{
		2, {0x9A, 0x8D},
		3, {0xE5, 0x9A, 0xB6},
		2, {0xB6, 0x56},
		2, {0xD3, 0xED}
	},
	{
		2, {0x9A, 0x8E},
		3, {0xE5, 0x9A, 0xB4},
		2, {0xB4, 0x56},
		2, {0xD3, 0xEE}
	},
	{
		2, {0x9A, 0x8F},
		3, {0xE5, 0x9B, 0x82},
		2, {0xC2, 0x56},
		2, {0xD3, 0xEF}
	},
	{
		2, {0x9A, 0x90},
		3, {0xE5, 0x9A, 0xBC},
		2, {0xBC, 0x56},
		2, {0xD3, 0xF0}
	},
	{
		2, {0x9A, 0x91},
		3, {0xE5, 0x9B, 0x81},
		2, {0xC1, 0x56},
		2, {0xD3, 0xF1}
	},
	{
		2, {0x9A, 0x92},
		3, {0xE5, 0x9B, 0x83},
		2, {0xC3, 0x56},
		2, {0xD3, 0xF2}
	},
	{
		2, {0x9A, 0x93},
		3, {0xE5, 0x9B, 0x80},
		2, {0xC0, 0x56},
		2, {0xD3, 0xF3}
	},
	{
		2, {0x9A, 0x94},
		3, {0xE5, 0x9B, 0x88},
		2, {0xC8, 0x56},
		2, {0xD3, 0xF4}
	},
	{
		2, {0x9A, 0x95},
		3, {0xE5, 0x9B, 0x8E},
		2, {0xCE, 0x56},
		2, {0xD3, 0xF5}
	},
	{
		2, {0x9A, 0x96},
		3, {0xE5, 0x9B, 0x91},
		2, {0xD1, 0x56},
		2, {0xD3, 0xF6}
	},
	{
		2, {0x9A, 0x97},
		3, {0xE5, 0x9B, 0x93},
		2, {0xD3, 0x56},
		2, {0xD3, 0xF7}
	},
	{
		2, {0x9A, 0x98},
		3, {0xE5, 0x9B, 0x97},
		2, {0xD7, 0x56},
		2, {0xD3, 0xF8}
	},
	{
		2, {0x9A, 0x99},
		3, {0xE5, 0x9B, 0xAE},
		2, {0xEE, 0x56},
		2, {0xD3, 0xF9}
	},
	{
		2, {0x9A, 0x9A},
		3, {0xE5, 0x9B, 0xB9},
		2, {0xF9, 0x56},
		2, {0xD3, 0xFA}
	},
	{
		2, {0x9A, 0x9B},
		3, {0xE5, 0x9C, 0x80},
		2, {0x00, 0x57},
		2, {0xD3, 0xFB}
	},
	{
		2, {0x9A, 0x9C},
		3, {0xE5, 0x9B, 0xBF},
		2, {0xFF, 0x56},
		2, {0xD3, 0xFC}
	},
	{
		2, {0x9A, 0x9D},
		3, {0xE5, 0x9C, 0x84},
		2, {0x04, 0x57},
		2, {0xD3, 0xFD}
	},
	{
		2, {0x9A, 0x9E},
		3, {0xE5, 0x9C, 0x89},
		2, {0x09, 0x57},
		2, {0xD3, 0xFE}
	},
	{
		2, {0x9A, 0x9F},
		3, {0xE5, 0x9C, 0x88},
		2, {0x08, 0x57},
		2, {0xD4, 0xA1}
	},
	{
		2, {0x9A, 0xA0},
		3, {0xE5, 0x9C, 0x8B},
		2, {0x0B, 0x57},
		2, {0xD4, 0xA2}
	},
	{
		2, {0x9A, 0xA1},
		3, {0xE5, 0x9C, 0x8D},
		2, {0x0D, 0x57},
		2, {0xD4, 0xA3}
	},
	{
		2, {0x9A, 0xA2},
		3, {0xE5, 0x9C, 0x93},
		2, {0x13, 0x57},
		2, {0xD4, 0xA4}
	},
	{
		2, {0x9A, 0xA3},
		3, {0xE5, 0x9C, 0x98},
		2, {0x18, 0x57},
		2, {0xD4, 0xA5}
	},
	{
		2, {0x9A, 0xA4},
		3, {0xE5, 0x9C, 0x96},
		2, {0x16, 0x57},
		2, {0xD4, 0xA6}
	},
	{
		2, {0x9A, 0xA5},
		3, {0xE5, 0x97, 0x87},
		2, {0xC7, 0x55},
		2, {0xD4, 0xA7}
	},
	{
		2, {0x9A, 0xA6},
		3, {0xE5, 0x9C, 0x9C},
		2, {0x1C, 0x57},
		2, {0xD4, 0xA8}
	},
	{
		2, {0x9A, 0xA7},
		3, {0xE5, 0x9C, 0xA6},
		2, {0x26, 0x57},
		2, {0xD4, 0xA9}
	},
	{
		2, {0x9A, 0xA8},
		3, {0xE5, 0x9C, 0xB7},
		2, {0x37, 0x57},
		2, {0xD4, 0xAA}
	},
	{
		2, {0x9A, 0xA9},
		3, {0xE5, 0x9C, 0xB8},
		2, {0x38, 0x57},
		2, {0xD4, 0xAB}
	},
	{
		2, {0x9A, 0xAA},
		3, {0xE5, 0x9D, 0x8E},
		2, {0x4E, 0x57},
		2, {0xD4, 0xAC}
	},
	{
		2, {0x9A, 0xAB},
		3, {0xE5, 0x9C, 0xBB},
		2, {0x3B, 0x57},
		2, {0xD4, 0xAD}
	},
	{
		2, {0x9A, 0xAC},
		3, {0xE5, 0x9D, 0x80},
		2, {0x40, 0x57},
		2, {0xD4, 0xAE}
	},
	{
		2, {0x9A, 0xAD},
		3, {0xE5, 0x9D, 0x8F},
		2, {0x4F, 0x57},
		2, {0xD4, 0xAF}
	},
	{
		2, {0x9A, 0xAE},
		3, {0xE5, 0x9D, 0xA9},
		2, {0x69, 0x57},
		2, {0xD4, 0xB0}
	},
	{
		2, {0x9A, 0xAF},
		3, {0xE5, 0x9F, 0x80},
		2, {0xC0, 0x57},
		2, {0xD4, 0xB1}
	},
	{
		2, {0x9A, 0xB0},
		3, {0xE5, 0x9E, 0x88},
		2, {0x88, 0x57},
		2, {0xD4, 0xB2}
	},
	{
		2, {0x9A, 0xB1},
		3, {0xE5, 0x9D, 0xA1},
		2, {0x61, 0x57},
		2, {0xD4, 0xB3}
	},
	{
		2, {0x9A, 0xB2},
		3, {0xE5, 0x9D, 0xBF},
		2, {0x7F, 0x57},
		2, {0xD4, 0xB4}
	},
	{
		2, {0x9A, 0xB3},
		3, {0xE5, 0x9E, 0x89},
		2, {0x89, 0x57},
		2, {0xD4, 0xB5}
	},
	{
		2, {0x9A, 0xB4},
		3, {0xE5, 0x9E, 0x93},
		2, {0x93, 0x57},
		2, {0xD4, 0xB6}
	},
	{
		2, {0x9A, 0xB5},
		3, {0xE5, 0x9E, 0xA0},
		2, {0xA0, 0x57},
		2, {0xD4, 0xB7}
	},
	{
		2, {0x9A, 0xB6},
		3, {0xE5, 0x9E, 0xB3},
		2, {0xB3, 0x57},
		2, {0xD4, 0xB8}
	},
	{
		2, {0x9A, 0xB7},
		3, {0xE5, 0x9E, 0xA4},
		2, {0xA4, 0x57},
		2, {0xD4, 0xB9}
	},
	{
		2, {0x9A, 0xB8},
		3, {0xE5, 0x9E, 0xAA},
		2, {0xAA, 0x57},
		2, {0xD4, 0xBA}
	},
	{
		2, {0x9A, 0xB9},
		3, {0xE5, 0x9E, 0xB0},
		2, {0xB0, 0x57},
		2, {0xD4, 0xBB}
	},
	{
		2, {0x9A, 0xBA},
		3, {0xE5, 0x9F, 0x83},
		2, {0xC3, 0x57},
		2, {0xD4, 0xBC}
	},
	{
		2, {0x9A, 0xBB},
		3, {0xE5, 0x9F, 0x86},
		2, {0xC6, 0x57},
		2, {0xD4, 0xBD}
	},
	{
		2, {0x9A, 0xBC},
		3, {0xE5, 0x9F, 0x94},
		2, {0xD4, 0x57},
		2, {0xD4, 0xBE}
	},
	{
		2, {0x9A, 0xBD},
		3, {0xE5, 0x9F, 0x92},
		2, {0xD2, 0x57},
		2, {0xD4, 0xBF}
	},
	{
		2, {0x9A, 0xBE},
		3, {0xE5, 0x9F, 0x93},
		2, {0xD3, 0x57},
		2, {0xD4, 0xC0}
	},
	{
		2, {0x9A, 0xBF},
		3, {0xE5, 0xA0, 0x8A},
		2, {0x0A, 0x58},
		2, {0xD4, 0xC1}
	},
	{
		2, {0x9A, 0xC0},
		3, {0xE5, 0x9F, 0x96},
		2, {0xD6, 0x57},
		2, {0xD4, 0xC2}
	},
	{
		2, {0x9A, 0xC1},
		3, {0xE5, 0x9F, 0xA3},
		2, {0xE3, 0x57},
		2, {0xD4, 0xC3}
	},
	{
		2, {0x9A, 0xC2},
		3, {0xE5, 0xA0, 0x8B},
		2, {0x0B, 0x58},
		2, {0xD4, 0xC4}
	},
	{
		2, {0x9A, 0xC3},
		3, {0xE5, 0xA0, 0x99},
		2, {0x19, 0x58},
		2, {0xD4, 0xC5}
	},
	{
		2, {0x9A, 0xC4},
		3, {0xE5, 0xA0, 0x9D},
		2, {0x1D, 0x58},
		2, {0xD4, 0xC6}
	},
	{
		2, {0x9A, 0xC5},
		3, {0xE5, 0xA1, 0xB2},
		2, {0x72, 0x58},
		2, {0xD4, 0xC7}
	},
	{
		2, {0x9A, 0xC6},
		3, {0xE5, 0xA0, 0xA1},
		2, {0x21, 0x58},
		2, {0xD4, 0xC8}
	},
	{
		2, {0x9A, 0xC7},
		3, {0xE5, 0xA1, 0xA2},
		2, {0x62, 0x58},
		2, {0xD4, 0xC9}
	},
	{
		2, {0x9A, 0xC8},
		3, {0xE5, 0xA1, 0x8B},
		2, {0x4B, 0x58},
		2, {0xD4, 0xCA}
	},
	{
		2, {0x9A, 0xC9},
		3, {0xE5, 0xA1, 0xB0},
		2, {0x70, 0x58},
		2, {0xD4, 0xCB}
	},
	{
		2, {0x9A, 0xCA},
		3, {0xE6, 0xAF, 0x80},
		2, {0xC0, 0x6B},
		2, {0xD4, 0xCC}
	},
	{
		2, {0x9A, 0xCB},
		3, {0xE5, 0xA1, 0x92},
		2, {0x52, 0x58},
		2, {0xD4, 0xCD}
	},
	{
		2, {0x9A, 0xCC},
		3, {0xE5, 0xA0, 0xBD},
		2, {0x3D, 0x58},
		2, {0xD4, 0xCE}
	},
	{
		2, {0x9A, 0xCD},
		3, {0xE5, 0xA1, 0xB9},
		2, {0x79, 0x58},
		2, {0xD4, 0xCF}
	},
	{
		2, {0x9A, 0xCE},
		3, {0xE5, 0xA2, 0x85},
		2, {0x85, 0x58},
		2, {0xD4, 0xD0}
	},
	{
		2, {0x9A, 0xCF},
		3, {0xE5, 0xA2, 0xB9},
		2, {0xB9, 0x58},
		2, {0xD4, 0xD1}
	},
	{
		2, {0x9A, 0xD0},
		3, {0xE5, 0xA2, 0x9F},
		2, {0x9F, 0x58},
		2, {0xD4, 0xD2}
	},
	{
		2, {0x9A, 0xD1},
		3, {0xE5, 0xA2, 0xAB},
		2, {0xAB, 0x58},
		2, {0xD4, 0xD3}
	},
	{
		2, {0x9A, 0xD2},
		3, {0xE5, 0xA2, 0xBA},
		2, {0xBA, 0x58},
		2, {0xD4, 0xD4}
	},
	{
		2, {0x9A, 0xD3},
		3, {0xE5, 0xA3, 0x9E},
		2, {0xDE, 0x58},
		2, {0xD4, 0xD5}
	},
	{
		2, {0x9A, 0xD4},
		3, {0xE5, 0xA2, 0xBB},
		2, {0xBB, 0x58},
		2, {0xD4, 0xD6}
	},
	{
		2, {0x9A, 0xD5},
		3, {0xE5, 0xA2, 0xB8},
		2, {0xB8, 0x58},
		2, {0xD4, 0xD7}
	},
	{
		2, {0x9A, 0xD6},
		3, {0xE5, 0xA2, 0xAE},
		2, {0xAE, 0x58},
		2, {0xD4, 0xD8}
	},
	{
		2, {0x9A, 0xD7},
		3, {0xE5, 0xA3, 0x85},
		2, {0xC5, 0x58},
		2, {0xD4, 0xD9}
	},
	{
		2, {0x9A, 0xD8},
		3, {0xE5, 0xA3, 0x93},
		2, {0xD3, 0x58},
		2, {0xD4, 0xDA}
	},
	{
		2, {0x9A, 0xD9},
		3, {0xE5, 0xA3, 0x91},
		2, {0xD1, 0x58},
		2, {0xD4, 0xDB}
	},
	{
		2, {0x9A, 0xDA},
		3, {0xE5, 0xA3, 0x97},
		2, {0xD7, 0x58},
		2, {0xD4, 0xDC}
	},
	{
		2, {0x9A, 0xDB},
		3, {0xE5, 0xA3, 0x99},
		2, {0xD9, 0x58},
		2, {0xD4, 0xDD}
	},
	{
		2, {0x9A, 0xDC},
		3, {0xE5, 0xA3, 0x98},
		2, {0xD8, 0x58},
		2, {0xD4, 0xDE}
	},
	{
		2, {0x9A, 0xDD},
		3, {0xE5, 0xA3, 0xA5},
		2, {0xE5, 0x58},
		2, {0xD4, 0xDF}
	},
	{
		2, {0x9A, 0xDE},
		3, {0xE5, 0xA3, 0x9C},
		2, {0xDC, 0x58},
		2, {0xD4, 0xE0}
	},
	{
		2, {0x9A, 0xDF},
		3, {0xE5, 0xA3, 0xA4},
		2, {0xE4, 0x58},
		2, {0xD4, 0xE1}
	},
	{
		2, {0x9A, 0xE0},
		3, {0xE5, 0xA3, 0x9F},
		2, {0xDF, 0x58},
		2, {0xD4, 0xE2}
	},
	{
		2, {0x9A, 0xE1},
		3, {0xE5, 0xA3, 0xAF},
		2, {0xEF, 0x58},
		2, {0xD4, 0xE3}
	},
	{
		2, {0x9A, 0xE2},
		3, {0xE5, 0xA3, 0xBA},
		2, {0xFA, 0x58},
		2, {0xD4, 0xE4}
	},
	{
		2, {0x9A, 0xE3},
		3, {0xE5, 0xA3, 0xB9},
		2, {0xF9, 0x58},
		2, {0xD4, 0xE5}
	},
	{
		2, {0x9A, 0xE4},
		3, {0xE5, 0xA3, 0xBB},
		2, {0xFB, 0x58},
		2, {0xD4, 0xE6}
	},
	{
		2, {0x9A, 0xE5},
		3, {0xE5, 0xA3, 0xBC},
		2, {0xFC, 0x58},
		2, {0xD4, 0xE7}
	},
	{
		2, {0x9A, 0xE6},
		3, {0xE5, 0xA3, 0xBD},
		2, {0xFD, 0x58},
		2, {0xD4, 0xE8}
	},
	{
		2, {0x9A, 0xE7},
		3, {0xE5, 0xA4, 0x82},
		2, {0x02, 0x59},
		2, {0xD4, 0xE9}
	},
	{
		2, {0x9A, 0xE8},
		3, {0xE5, 0xA4, 0x8A},
		2, {0x0A, 0x59},
		2, {0xD4, 0xEA}
	},
	{
		2, {0x9A, 0xE9},
		3, {0xE5, 0xA4, 0x90},
		2, {0x10, 0x59},
		2, {0xD4, 0xEB}
	},
	{
		2, {0x9A, 0xEA},
		3, {0xE5, 0xA4, 0x9B},
		2, {0x1B, 0x59},
		2, {0xD4, 0xEC}
	},
	{
		2, {0x9A, 0xEB},
		3, {0xE6, 0xA2, 0xA6},
		2, {0xA6, 0x68},
		2, {0xD4, 0xED}
	},
	{
		2, {0x9A, 0xEC},
		3, {0xE5, 0xA4, 0xA5},
		2, {0x25, 0x59},
		2, {0xD4, 0xEE}
	},
	{
		2, {0x9A, 0xED},
		3, {0xE5, 0xA4, 0xAC},
		2, {0x2C, 0x59},
		2, {0xD4, 0xEF}
	},
	{
		2, {0x9A, 0xEE},
		3, {0xE5, 0xA4, 0xAD},
		2, {0x2D, 0x59},
		2, {0xD4, 0xF0}
	},
	{
		2, {0x9A, 0xEF},
		3, {0xE5, 0xA4, 0xB2},
		2, {0x32, 0x59},
		2, {0xD4, 0xF1}
	},
	{
		2, {0x9A, 0xF0},
		3, {0xE5, 0xA4, 0xB8},
		2, {0x38, 0x59},
		2, {0xD4, 0xF2}
	},
	{
		2, {0x9A, 0xF1},
		3, {0xE5, 0xA4, 0xBE},
		2, {0x3E, 0x59},
		2, {0xD4, 0xF3}
	},
	{
		2, {0x9A, 0xF2},
		3, {0xE7, 0xAB, 0x92},
		2, {0xD2, 0x7A},
		2, {0xD4, 0xF4}
	},
	{
		2, {0x9A, 0xF3},
		3, {0xE5, 0xA5, 0x95},
		2, {0x55, 0x59},
		2, {0xD4, 0xF5}
	},
	{
		2, {0x9A, 0xF4},
		3, {0xE5, 0xA5, 0x90},
		2, {0x50, 0x59},
		2, {0xD4, 0xF6}
	},
	{
		2, {0x9A, 0xF5},
		3, {0xE5, 0xA5, 0x8E},
		2, {0x4E, 0x59},
		2, {0xD4, 0xF7}
	},
	{
		2, {0x9A, 0xF6},
		3, {0xE5, 0xA5, 0x9A},
		2, {0x5A, 0x59},
		2, {0xD4, 0xF8}
	},
	{
		2, {0x9A, 0xF7},
		3, {0xE5, 0xA5, 0x98},
		2, {0x58, 0x59},
		2, {0xD4, 0xF9}
	},
	{
		2, {0x9A, 0xF8},
		3, {0xE5, 0xA5, 0xA2},
		2, {0x62, 0x59},
		2, {0xD4, 0xFA}
	},
	{
		2, {0x9A, 0xF9},
		3, {0xE5, 0xA5, 0xA0},
		2, {0x60, 0x59},
		2, {0xD4, 0xFB}
	},
	{
		2, {0x9A, 0xFA},
		3, {0xE5, 0xA5, 0xA7},
		2, {0x67, 0x59},
		2, {0xD4, 0xFC}
	},
	{
		2, {0x9A, 0xFB},
		3, {0xE5, 0xA5, 0xAC},
		2, {0x6C, 0x59},
		2, {0xD4, 0xFD}
	},
	{
		2, {0x9A, 0xFC},
		3, {0xE5, 0xA5, 0xA9},
		2, {0x69, 0x59},
		2, {0xD4, 0xFE}
	},
	{
		2, {0x9B, 0x40},
		3, {0xE5, 0xA5, 0xB8},
		2, {0x78, 0x59},
		2, {0xD5, 0xA1}
	},
	{
		2, {0x9B, 0x41},
		3, {0xE5, 0xA6, 0x81},
		2, {0x81, 0x59},
		2, {0xD5, 0xA2}
	},
	{
		2, {0x9B, 0x42},
		3, {0xE5, 0xA6, 0x9D},
		2, {0x9D, 0x59},
		2, {0xD5, 0xA3}
	},
	{
		2, {0x9B, 0x43},
		3, {0xE4, 0xBD, 0x9E},
		2, {0x5E, 0x4F},
		2, {0xD5, 0xA4}
	},
	{
		2, {0x9B, 0x44},
		3, {0xE4, 0xBE, 0xAB},
		2, {0xAB, 0x4F},
		2, {0xD5, 0xA5}
	},
	{
		2, {0x9B, 0x45},
		3, {0xE5, 0xA6, 0xA3},
		2, {0xA3, 0x59},
		2, {0xD5, 0xA6}
	},
	{
		2, {0x9B, 0x46},
		3, {0xE5, 0xA6, 0xB2},
		2, {0xB2, 0x59},
		2, {0xD5, 0xA7}
	},
	{
		2, {0x9B, 0x47},
		3, {0xE5, 0xA7, 0x86},
		2, {0xC6, 0x59},
		2, {0xD5, 0xA8}
	},
	{
		2, {0x9B, 0x48},
		3, {0xE5, 0xA7, 0xA8},
		2, {0xE8, 0x59},
		2, {0xD5, 0xA9}
	},
	{
		2, {0x9B, 0x49},
		3, {0xE5, 0xA7, 0x9C},
		2, {0xDC, 0x59},
		2, {0xD5, 0xAA}
	},
	{
		2, {0x9B, 0x4A},
		3, {0xE5, 0xA6, 0x8D},
		2, {0x8D, 0x59},
		2, {0xD5, 0xAB}
	},
	{
		2, {0x9B, 0x4B},
		3, {0xE5, 0xA7, 0x99},
		2, {0xD9, 0x59},
		2, {0xD5, 0xAC}
	},
	{
		2, {0x9B, 0x4C},
		3, {0xE5, 0xA7, 0x9A},
		2, {0xDA, 0x59},
		2, {0xD5, 0xAD}
	},
	{
		2, {0x9B, 0x4D},
		3, {0xE5, 0xA8, 0xA5},
		2, {0x25, 0x5A},
		2, {0xD5, 0xAE}
	},
	{
		2, {0x9B, 0x4E},
		3, {0xE5, 0xA8, 0x9F},
		2, {0x1F, 0x5A},
		2, {0xD5, 0xAF}
	},
	{
		2, {0x9B, 0x4F},
		3, {0xE5, 0xA8, 0x91},
		2, {0x11, 0x5A},
		2, {0xD5, 0xB0}
	},
	{
		2, {0x9B, 0x50},
		3, {0xE5, 0xA8, 0x9C},
		2, {0x1C, 0x5A},
		2, {0xD5, 0xB1}
	},
	{
		2, {0x9B, 0x51},
		3, {0xE5, 0xA8, 0x89},
		2, {0x09, 0x5A},
		2, {0xD5, 0xB2}
	},
	{
		2, {0x9B, 0x52},
		3, {0xE5, 0xA8, 0x9A},
		2, {0x1A, 0x5A},
		2, {0xD5, 0xB3}
	},
	{
		2, {0x9B, 0x53},
		3, {0xE5, 0xA9, 0x80},
		2, {0x40, 0x5A},
		2, {0xD5, 0xB4}
	},
	{
		2, {0x9B, 0x54},
		3, {0xE5, 0xA9, 0xAC},
		2, {0x6C, 0x5A},
		2, {0xD5, 0xB5}
	},
	{
		2, {0x9B, 0x55},
		3, {0xE5, 0xA9, 0x89},
		2, {0x49, 0x5A},
		2, {0xD5, 0xB6}
	},
	{
		2, {0x9B, 0x56},
		3, {0xE5, 0xA8, 0xB5},
		2, {0x35, 0x5A},
		2, {0xD5, 0xB7}
	},
	{
		2, {0x9B, 0x57},
		3, {0xE5, 0xA8, 0xB6},
		2, {0x36, 0x5A},
		2, {0xD5, 0xB8}
	},
	{
		2, {0x9B, 0x58},
		3, {0xE5, 0xA9, 0xA2},
		2, {0x62, 0x5A},
		2, {0xD5, 0xB9}
	},
	{
		2, {0x9B, 0x59},
		3, {0xE5, 0xA9, 0xAA},
		2, {0x6A, 0x5A},
		2, {0xD5, 0xBA}
	},
	{
		2, {0x9B, 0x5A},
		3, {0xE5, 0xAA, 0x9A},
		2, {0x9A, 0x5A},
		2, {0xD5, 0xBB}
	},
	{
		2, {0x9B, 0x5B},
		3, {0xE5, 0xAA, 0xBC},
		2, {0xBC, 0x5A},
		2, {0xD5, 0xBC}
	},
	{
		2, {0x9B, 0x5C},
		3, {0xE5, 0xAA, 0xBE},
		2, {0xBE, 0x5A},
		2, {0xD5, 0xBD}
	},
	{
		2, {0x9B, 0x5D},
		3, {0xE5, 0xAB, 0x8B},
		2, {0xCB, 0x5A},
		2, {0xD5, 0xBE}
	},
	{
		2, {0x9B, 0x5E},
		3, {0xE5, 0xAB, 0x82},
		2, {0xC2, 0x5A},
		2, {0xD5, 0xBF}
	},
	{
		2, {0x9B, 0x5F},
		3, {0xE5, 0xAA, 0xBD},
		2, {0xBD, 0x5A},
		2, {0xD5, 0xC0}
	},
	{
		2, {0x9B, 0x60},
		3, {0xE5, 0xAB, 0xA3},
		2, {0xE3, 0x5A},
		2, {0xD5, 0xC1}
	},
	{
		2, {0x9B, 0x61},
		3, {0xE5, 0xAB, 0x97},
		2, {0xD7, 0x5A},
		2, {0xD5, 0xC2}
	},
	{
		2, {0x9B, 0x62},
		3, {0xE5, 0xAB, 0xA6},
		2, {0xE6, 0x5A},
		2, {0xD5, 0xC3}
	},
	{
		2, {0x9B, 0x63},
		3, {0xE5, 0xAB, 0xA9},
		2, {0xE9, 0x5A},
		2, {0xD5, 0xC4}
	},
	{
		2, {0x9B, 0x64},
		3, {0xE5, 0xAB, 0x96},
		2, {0xD6, 0x5A},
		2, {0xD5, 0xC5}
	},
	{
		2, {0x9B, 0x65},
		3, {0xE5, 0xAB, 0xBA},
		2, {0xFA, 0x5A},
		2, {0xD5, 0xC6}
	},
	{
		2, {0x9B, 0x66},
		3, {0xE5, 0xAB, 0xBB},
		2, {0xFB, 0x5A},
		2, {0xD5, 0xC7}
	},
	{
		2, {0x9B, 0x67},
		3, {0xE5, 0xAC, 0x8C},
		2, {0x0C, 0x5B},
		2, {0xD5, 0xC8}
	},
	{
		2, {0x9B, 0x68},
		3, {0xE5, 0xAC, 0x8B},
		2, {0x0B, 0x5B},
		2, {0xD5, 0xC9}
	},
	{
		2, {0x9B, 0x69},
		3, {0xE5, 0xAC, 0x96},
		2, {0x16, 0x5B},
		2, {0xD5, 0xCA}
	},
	{
		2, {0x9B, 0x6A},
		3, {0xE5, 0xAC, 0xB2},
		2, {0x32, 0x5B},
		2, {0xD5, 0xCB}
	},
	{
		2, {0x9B, 0x6B},
		3, {0xE5, 0xAB, 0x90},
		2, {0xD0, 0x5A},
		2, {0xD5, 0xCC}
	},
	{
		2, {0x9B, 0x6C},
		3, {0xE5, 0xAC, 0xAA},
		2, {0x2A, 0x5B},
		2, {0xD5, 0xCD}
	},
	{
		2, {0x9B, 0x6D},
		3, {0xE5, 0xAC, 0xB6},
		2, {0x36, 0x5B},
		2, {0xD5, 0xCE}
	},
	{
		2, {0x9B, 0x6E},
		3, {0xE5, 0xAC, 0xBE},
		2, {0x3E, 0x5B},
		2, {0xD5, 0xCF}
	},
	{
		2, {0x9B, 0x6F},
		3, {0xE5, 0xAD, 0x83},
		2, {0x43, 0x5B},
		2, {0xD5, 0xD0}
	},
	{
		2, {0x9B, 0x70},
		3, {0xE5, 0xAD, 0x85},
		2, {0x45, 0x5B},
		2, {0xD5, 0xD1}
	},
	{
		2, {0x9B, 0x71},
		3, {0xE5, 0xAD, 0x80},
		2, {0x40, 0x5B},
		2, {0xD5, 0xD2}
	},
	{
		2, {0x9B, 0x72},
		3, {0xE5, 0xAD, 0x91},
		2, {0x51, 0x5B},
		2, {0xD5, 0xD3}
	},
	{
		2, {0x9B, 0x73},
		3, {0xE5, 0xAD, 0x95},
		2, {0x55, 0x5B},
		2, {0xD5, 0xD4}
	},
	{
		2, {0x9B, 0x74},
		3, {0xE5, 0xAD, 0x9A},
		2, {0x5A, 0x5B},
		2, {0xD5, 0xD5}
	},
	{
		2, {0x9B, 0x75},
		3, {0xE5, 0xAD, 0x9B},
		2, {0x5B, 0x5B},
		2, {0xD5, 0xD6}
	},
	{
		2, {0x9B, 0x76},
		3, {0xE5, 0xAD, 0xA5},
		2, {0x65, 0x5B},
		2, {0xD5, 0xD7}
	},
	{
		2, {0x9B, 0x77},
		3, {0xE5, 0xAD, 0xA9},
		2, {0x69, 0x5B},
		2, {0xD5, 0xD8}
	},
	{
		2, {0x9B, 0x78},
		3, {0xE5, 0xAD, 0xB0},
		2, {0x70, 0x5B},
		2, {0xD5, 0xD9}
	},
	{
		2, {0x9B, 0x79},
		3, {0xE5, 0xAD, 0xB3},
		2, {0x73, 0x5B},
		2, {0xD5, 0xDA}
	},
	{
		2, {0x9B, 0x7A},
		3, {0xE5, 0xAD, 0xB5},
		2, {0x75, 0x5B},
		2, {0xD5, 0xDB}
	},
	{
		2, {0x9B, 0x7B},
		3, {0xE5, 0xAD, 0xB8},
		2, {0x78, 0x5B},
		2, {0xD5, 0xDC}
	},
	{
		2, {0x9B, 0x7C},
		3, {0xE6, 0x96, 0x88},
		2, {0x88, 0x65},
		2, {0xD5, 0xDD}
	},
	{
		2, {0x9B, 0x7D},
		3, {0xE5, 0xAD, 0xBA},
		2, {0x7A, 0x5B},
		2, {0xD5, 0xDE}
	},
	{
		2, {0x9B, 0x7E},
		3, {0xE5, 0xAE, 0x80},
		2, {0x80, 0x5B},
		2, {0xD5, 0xDF}
	},
	{
		2, {0x9B, 0x80},
		3, {0xE5, 0xAE, 0x83},
		2, {0x83, 0x5B},
		2, {0xD5, 0xE0}
	},
	{
		2, {0x9B, 0x81},
		3, {0xE5, 0xAE, 0xA6},
		2, {0xA6, 0x5B},
		2, {0xD5, 0xE1}
	},
	{
		2, {0x9B, 0x82},
		3, {0xE5, 0xAE, 0xB8},
		2, {0xB8, 0x5B},
		2, {0xD5, 0xE2}
	},
	{
		2, {0x9B, 0x83},
		3, {0xE5, 0xAF, 0x83},
		2, {0xC3, 0x5B},
		2, {0xD5, 0xE3}
	},
	{
		2, {0x9B, 0x84},
		3, {0xE5, 0xAF, 0x87},
		2, {0xC7, 0x5B},
		2, {0xD5, 0xE4}
	},
	{
		2, {0x9B, 0x85},
		3, {0xE5, 0xAF, 0x89},
		2, {0xC9, 0x5B},
		2, {0xD5, 0xE5}
	},
	{
		2, {0x9B, 0x86},
		3, {0xE5, 0xAF, 0x94},
		2, {0xD4, 0x5B},
		2, {0xD5, 0xE6}
	},
	{
		2, {0x9B, 0x87},
		3, {0xE5, 0xAF, 0x90},
		2, {0xD0, 0x5B},
		2, {0xD5, 0xE7}
	},
	{
		2, {0x9B, 0x88},
		3, {0xE5, 0xAF, 0xA4},
		2, {0xE4, 0x5B},
		2, {0xD5, 0xE8}
	},
	{
		2, {0x9B, 0x89},
		3, {0xE5, 0xAF, 0xA6},
		2, {0xE6, 0x5B},
		2, {0xD5, 0xE9}
	},
	{
		2, {0x9B, 0x8A},
		3, {0xE5, 0xAF, 0xA2},
		2, {0xE2, 0x5B},
		2, {0xD5, 0xEA}
	},
	{
		2, {0x9B, 0x8B},
		3, {0xE5, 0xAF, 0x9E},
		2, {0xDE, 0x5B},
		2, {0xD5, 0xEB}
	},
	{
		2, {0x9B, 0x8C},
		3, {0xE5, 0xAF, 0xA5},
		2, {0xE5, 0x5B},
		2, {0xD5, 0xEC}
	},
	{
		2, {0x9B, 0x8D},
		3, {0xE5, 0xAF, 0xAB},
		2, {0xEB, 0x5B},
		2, {0xD5, 0xED}
	},
	{
		2, {0x9B, 0x8E},
		3, {0xE5, 0xAF, 0xB0},
		2, {0xF0, 0x5B},
		2, {0xD5, 0xEE}
	},
	{
		2, {0x9B, 0x8F},
		3, {0xE5, 0xAF, 0xB6},
		2, {0xF6, 0x5B},
		2, {0xD5, 0xEF}
	},
	{
		2, {0x9B, 0x90},
		3, {0xE5, 0xAF, 0xB3},
		2, {0xF3, 0x5B},
		2, {0xD5, 0xF0}
	},
	{
		2, {0x9B, 0x91},
		3, {0xE5, 0xB0, 0x85},
		2, {0x05, 0x5C},
		2, {0xD5, 0xF1}
	},
	{
		2, {0x9B, 0x92},
		3, {0xE5, 0xB0, 0x87},
		2, {0x07, 0x5C},
		2, {0xD5, 0xF2}
	},
	{
		2, {0x9B, 0x93},
		3, {0xE5, 0xB0, 0x88},
		2, {0x08, 0x5C},
		2, {0xD5, 0xF3}
	},
	{
		2, {0x9B, 0x94},
		3, {0xE5, 0xB0, 0x8D},
		2, {0x0D, 0x5C},
		2, {0xD5, 0xF4}
	},
	{
		2, {0x9B, 0x95},
		3, {0xE5, 0xB0, 0x93},
		2, {0x13, 0x5C},
		2, {0xD5, 0xF5}
	},
	{
		2, {0x9B, 0x96},
		3, {0xE5, 0xB0, 0xA0},
		2, {0x20, 0x5C},
		2, {0xD5, 0xF6}
	},
	{
		2, {0x9B, 0x97},
		3, {0xE5, 0xB0, 0xA2},
		2, {0x22, 0x5C},
		2, {0xD5, 0xF7}
	},
	{
		2, {0x9B, 0x98},
		3, {0xE5, 0xB0, 0xA8},
		2, {0x28, 0x5C},
		2, {0xD5, 0xF8}
	},
	{
		2, {0x9B, 0x99},
		3, {0xE5, 0xB0, 0xB8},
		2, {0x38, 0x5C},
		2, {0xD5, 0xF9}
	},
	{
		2, {0x9B, 0x9A},
		3, {0xE5, 0xB0, 0xB9},
		2, {0x39, 0x5C},
		2, {0xD5, 0xFA}
	},
	{
		2, {0x9B, 0x9B},
		3, {0xE5, 0xB1, 0x81},
		2, {0x41, 0x5C},
		2, {0xD5, 0xFB}
	},
	{
		2, {0x9B, 0x9C},
		3, {0xE5, 0xB1, 0x86},
		2, {0x46, 0x5C},
		2, {0xD5, 0xFC}
	},
	{
		2, {0x9B, 0x9D},
		3, {0xE5, 0xB1, 0x8E},
		2, {0x4E, 0x5C},
		2, {0xD5, 0xFD}
	},
	{
		2, {0x9B, 0x9E},
		3, {0xE5, 0xB1, 0x93},
		2, {0x53, 0x5C},
		2, {0xD5, 0xFE}
	},
	{
		2, {0x9B, 0x9F},
		3, {0xE5, 0xB1, 0x90},
		2, {0x50, 0x5C},
		2, {0xD6, 0xA1}
	},
	{
		2, {0x9B, 0xA0},
		3, {0xE5, 0xB1, 0x8F},
		2, {0x4F, 0x5C},
		2, {0xD6, 0xA2}
	},
	{
		2, {0x9B, 0xA1},
		3, {0xE5, 0xAD, 0xB1},
		2, {0x71, 0x5B},
		2, {0xD6, 0xA3}
	},
	{
		2, {0x9B, 0xA2},
		3, {0xE5, 0xB1, 0xAC},
		2, {0x6C, 0x5C},
		2, {0xD6, 0xA4}
	},
	{
		2, {0x9B, 0xA3},
		3, {0xE5, 0xB1, 0xAE},
		2, {0x6E, 0x5C},
		2, {0xD6, 0xA5}
	},
	{
		2, {0x9B, 0xA4},
		3, {0xE4, 0xB9, 0xA2},
		2, {0x62, 0x4E},
		2, {0xD6, 0xA6}
	},
	{
		2, {0x9B, 0xA5},
		3, {0xE5, 0xB1, 0xB6},
		2, {0x76, 0x5C},
		2, {0xD6, 0xA7}
	},
	{
		2, {0x9B, 0xA6},
		3, {0xE5, 0xB1, 0xB9},
		2, {0x79, 0x5C},
		2, {0xD6, 0xA8}
	},
	{
		2, {0x9B, 0xA7},
		3, {0xE5, 0xB2, 0x8C},
		2, {0x8C, 0x5C},
		2, {0xD6, 0xA9}
	},
	{
		2, {0x9B, 0xA8},
		3, {0xE5, 0xB2, 0x91},
		2, {0x91, 0x5C},
		2, {0xD6, 0xAA}
	},
	{
		2, {0x9B, 0xA9},
		3, {0xE5, 0xB2, 0x94},
		2, {0x94, 0x5C},
		2, {0xD6, 0xAB}
	},
	{
		2, {0x9B, 0xAA},
		3, {0xE5, 0xA6, 0x9B},
		2, {0x9B, 0x59},
		2, {0xD6, 0xAC}
	},
	{
		2, {0x9B, 0xAB},
		3, {0xE5, 0xB2, 0xAB},
		2, {0xAB, 0x5C},
		2, {0xD6, 0xAD}
	},
	{
		2, {0x9B, 0xAC},
		3, {0xE5, 0xB2, 0xBB},
		2, {0xBB, 0x5C},
		2, {0xD6, 0xAE}
	},
	{
		2, {0x9B, 0xAD},
		3, {0xE5, 0xB2, 0xB6},
		2, {0xB6, 0x5C},
		2, {0xD6, 0xAF}
	},
	{
		2, {0x9B, 0xAE},
		3, {0xE5, 0xB2, 0xBC},
		2, {0xBC, 0x5C},
		2, {0xD6, 0xB0}
	},
	{
		2, {0x9B, 0xAF},
		3, {0xE5, 0xB2, 0xB7},
		2, {0xB7, 0x5C},
		2, {0xD6, 0xB1}
	},
	{
		2, {0x9B, 0xB0},
		3, {0xE5, 0xB3, 0x85},
		2, {0xC5, 0x5C},
		2, {0xD6, 0xB2}
	},
	{
		2, {0x9B, 0xB1},
		3, {0xE5, 0xB2, 0xBE},
		2, {0xBE, 0x5C},
		2, {0xD6, 0xB3}
	},
	{
		2, {0x9B, 0xB2},
		3, {0xE5, 0xB3, 0x87},
		2, {0xC7, 0x5C},
		2, {0xD6, 0xB4}
	},
	{
		2, {0x9B, 0xB3},
		3, {0xE5, 0xB3, 0x99},
		2, {0xD9, 0x5C},
		2, {0xD6, 0xB5}
	},
	{
		2, {0x9B, 0xB4},
		3, {0xE5, 0xB3, 0xA9},
		2, {0xE9, 0x5C},
		2, {0xD6, 0xB6}
	},
	{
		2, {0x9B, 0xB5},
		3, {0xE5, 0xB3, 0xBD},
		2, {0xFD, 0x5C},
		2, {0xD6, 0xB7}
	},
	{
		2, {0x9B, 0xB6},
		3, {0xE5, 0xB3, 0xBA},
		2, {0xFA, 0x5C},
		2, {0xD6, 0xB8}
	},
	{
		2, {0x9B, 0xB7},
		3, {0xE5, 0xB3, 0xAD},
		2, {0xED, 0x5C},
		2, {0xD6, 0xB9}
	},
	{
		2, {0x9B, 0xB8},
		3, {0xE5, 0xB6, 0x8C},
		2, {0x8C, 0x5D},
		2, {0xD6, 0xBA}
	},
	{
		2, {0x9B, 0xB9},
		3, {0xE5, 0xB3, 0xAA},
		2, {0xEA, 0x5C},
		2, {0xD6, 0xBB}
	},
	{
		2, {0x9B, 0xBA},
		3, {0xE5, 0xB4, 0x8B},
		2, {0x0B, 0x5D},
		2, {0xD6, 0xBC}
	},
	{
		2, {0x9B, 0xBB},
		3, {0xE5, 0xB4, 0x95},
		2, {0x15, 0x5D},
		2, {0xD6, 0xBD}
	},
	{
		2, {0x9B, 0xBC},
		3, {0xE5, 0xB4, 0x97},
		2, {0x17, 0x5D},
		2, {0xD6, 0xBE}
	},
	{
		2, {0x9B, 0xBD},
		3, {0xE5, 0xB5, 0x9C},
		2, {0x5C, 0x5D},
		2, {0xD6, 0xBF}
	},
	{
		2, {0x9B, 0xBE},
		3, {0xE5, 0xB4, 0x9F},
		2, {0x1F, 0x5D},
		2, {0xD6, 0xC0}
	},
	{
		2, {0x9B, 0xBF},
		3, {0xE5, 0xB4, 0x9B},
		2, {0x1B, 0x5D},
		2, {0xD6, 0xC1}
	},
	{
		2, {0x9B, 0xC0},
		3, {0xE5, 0xB4, 0x91},
		2, {0x11, 0x5D},
		2, {0xD6, 0xC2}
	},
	{
		2, {0x9B, 0xC1},
		3, {0xE5, 0xB4, 0x94},
		2, {0x14, 0x5D},
		2, {0xD6, 0xC3}
	},
	{
		2, {0x9B, 0xC2},
		3, {0xE5, 0xB4, 0xA2},
		2, {0x22, 0x5D},
		2, {0xD6, 0xC4}
	},
	{
		2, {0x9B, 0xC3},
		3, {0xE5, 0xB4, 0x9A},
		2, {0x1A, 0x5D},
		2, {0xD6, 0xC5}
	},
	{
		2, {0x9B, 0xC4},
		3, {0xE5, 0xB4, 0x99},
		2, {0x19, 0x5D},
		2, {0xD6, 0xC6}
	},
	{
		2, {0x9B, 0xC5},
		3, {0xE5, 0xB4, 0x98},
		2, {0x18, 0x5D},
		2, {0xD6, 0xC7}
	},
	{
		2, {0x9B, 0xC6},
		3, {0xE5, 0xB5, 0x8C},
		2, {0x4C, 0x5D},
		2, {0xD6, 0xC8}
	},
	{
		2, {0x9B, 0xC7},
		3, {0xE5, 0xB5, 0x92},
		2, {0x52, 0x5D},
		2, {0xD6, 0xC9}
	},
	{
		2, {0x9B, 0xC8},
		3, {0xE5, 0xB5, 0x8E},
		2, {0x4E, 0x5D},
		2, {0xD6, 0xCA}
	},
	{
		2, {0x9B, 0xC9},
		3, {0xE5, 0xB5, 0x8B},
		2, {0x4B, 0x5D},
		2, {0xD6, 0xCB}
	},
	{
		2, {0x9B, 0xCA},
		3, {0xE5, 0xB5, 0xAC},
		2, {0x6C, 0x5D},
		2, {0xD6, 0xCC}
	},
	{
		2, {0x9B, 0xCB},
		3, {0xE5, 0xB5, 0xB3},
		2, {0x73, 0x5D},
		2, {0xD6, 0xCD}
	},
	{
		2, {0x9B, 0xCC},
		3, {0xE5, 0xB5, 0xB6},
		2, {0x76, 0x5D},
		2, {0xD6, 0xCE}
	},
	{
		2, {0x9B, 0xCD},
		3, {0xE5, 0xB6, 0x87},
		2, {0x87, 0x5D},
		2, {0xD6, 0xCF}
	},
	{
		2, {0x9B, 0xCE},
		3, {0xE5, 0xB6, 0x84},
		2, {0x84, 0x5D},
		2, {0xD6, 0xD0}
	},
	{
		2, {0x9B, 0xCF},
		3, {0xE5, 0xB6, 0x82},
		2, {0x82, 0x5D},
		2, {0xD6, 0xD1}
	},
	{
		2, {0x9B, 0xD0},
		3, {0xE5, 0xB6, 0xA2},
		2, {0xA2, 0x5D},
		2, {0xD6, 0xD2}
	},
	{
		2, {0x9B, 0xD1},
		3, {0xE5, 0xB6, 0x9D},
		2, {0x9D, 0x5D},
		2, {0xD6, 0xD3}
	},
	{
		2, {0x9B, 0xD2},
		3, {0xE5, 0xB6, 0xAC},
		2, {0xAC, 0x5D},
		2, {0xD6, 0xD4}
	},
	{
		2, {0x9B, 0xD3},
		3, {0xE5, 0xB6, 0xAE},
		2, {0xAE, 0x5D},
		2, {0xD6, 0xD5}
	},
	{
		2, {0x9B, 0xD4},
		3, {0xE5, 0xB6, 0xBD},
		2, {0xBD, 0x5D},
		2, {0xD6, 0xD6}
	},
	{
		2, {0x9B, 0xD5},
		3, {0xE5, 0xB6, 0x90},
		2, {0x90, 0x5D},
		2, {0xD6, 0xD7}
	},
	{
		2, {0x9B, 0xD6},
		3, {0xE5, 0xB6, 0xB7},
		2, {0xB7, 0x5D},
		2, {0xD6, 0xD8}
	},
	{
		2, {0x9B, 0xD7},
		3, {0xE5, 0xB6, 0xBC},
		2, {0xBC, 0x5D},
		2, {0xD6, 0xD9}
	},
	{
		2, {0x9B, 0xD8},
		3, {0xE5, 0xB7, 0x89},
		2, {0xC9, 0x5D},
		2, {0xD6, 0xDA}
	},
	{
		2, {0x9B, 0xD9},
		3, {0xE5, 0xB7, 0x8D},
		2, {0xCD, 0x5D},
		2, {0xD6, 0xDB}
	},
	{
		2, {0x9B, 0xDA},
		3, {0xE5, 0xB7, 0x93},
		2, {0xD3, 0x5D},
		2, {0xD6, 0xDC}
	},
	{
		2, {0x9B, 0xDB},
		3, {0xE5, 0xB7, 0x92},
		2, {0xD2, 0x5D},
		2, {0xD6, 0xDD}
	},
	{
		2, {0x9B, 0xDC},
		3, {0xE5, 0xB7, 0x96},
		2, {0xD6, 0x5D},
		2, {0xD6, 0xDE}
	},
	{
		2, {0x9B, 0xDD},
		3, {0xE5, 0xB7, 0x9B},
		2, {0xDB, 0x5D},
		2, {0xD6, 0xDF}
	},
	{
		2, {0x9B, 0xDE},
		3, {0xE5, 0xB7, 0xAB},
		2, {0xEB, 0x5D},
		2, {0xD6, 0xE0}
	},
	{
		2, {0x9B, 0xDF},
		3, {0xE5, 0xB7, 0xB2},
		2, {0xF2, 0x5D},
		2, {0xD6, 0xE1}
	},
	{
		2, {0x9B, 0xE0},
		3, {0xE5, 0xB7, 0xB5},
		2, {0xF5, 0x5D},
		2, {0xD6, 0xE2}
	},
	{
		2, {0x9B, 0xE1},
		3, {0xE5, 0xB8, 0x8B},
		2, {0x0B, 0x5E},
		2, {0xD6, 0xE3}
	},
	{
		2, {0x9B, 0xE2},
		3, {0xE5, 0xB8, 0x9A},
		2, {0x1A, 0x5E},
		2, {0xD6, 0xE4}
	},
	{
		2, {0x9B, 0xE3},
		3, {0xE5, 0xB8, 0x99},
		2, {0x19, 0x5E},
		2, {0xD6, 0xE5}
	},
	{
		2, {0x9B, 0xE4},
		3, {0xE5, 0xB8, 0x91},
		2, {0x11, 0x5E},
		2, {0xD6, 0xE6}
	},
	{
		2, {0x9B, 0xE5},
		3, {0xE5, 0xB8, 0x9B},
		2, {0x1B, 0x5E},
		2, {0xD6, 0xE7}
	},
	{
		2, {0x9B, 0xE6},
		3, {0xE5, 0xB8, 0xB6},
		2, {0x36, 0x5E},
		2, {0xD6, 0xE8}
	},
	{
		2, {0x9B, 0xE7},
		3, {0xE5, 0xB8, 0xB7},
		2, {0x37, 0x5E},
		2, {0xD6, 0xE9}
	},
	{
		2, {0x9B, 0xE8},
		3, {0xE5, 0xB9, 0x84},
		2, {0x44, 0x5E},
		2, {0xD6, 0xEA}
	},
	{
		2, {0x9B, 0xE9},
		3, {0xE5, 0xB9, 0x83},
		2, {0x43, 0x5E},
		2, {0xD6, 0xEB}
	},
	{
		2, {0x9B, 0xEA},
		3, {0xE5, 0xB9, 0x80},
		2, {0x40, 0x5E},
		2, {0xD6, 0xEC}
	},
	{
		2, {0x9B, 0xEB},
		3, {0xE5, 0xB9, 0x8E},
		2, {0x4E, 0x5E},
		2, {0xD6, 0xED}
	},
	{
		2, {0x9B, 0xEC},
		3, {0xE5, 0xB9, 0x97},
		2, {0x57, 0x5E},
		2, {0xD6, 0xEE}
	},
	{
		2, {0x9B, 0xED},
		3, {0xE5, 0xB9, 0x94},
		2, {0x54, 0x5E},
		2, {0xD6, 0xEF}
	},
	{
		2, {0x9B, 0xEE},
		3, {0xE5, 0xB9, 0x9F},
		2, {0x5F, 0x5E},
		2, {0xD6, 0xF0}
	},
	{
		2, {0x9B, 0xEF},
		3, {0xE5, 0xB9, 0xA2},
		2, {0x62, 0x5E},
		2, {0xD6, 0xF1}
	},
	{
		2, {0x9B, 0xF0},
		3, {0xE5, 0xB9, 0xA4},
		2, {0x64, 0x5E},
		2, {0xD6, 0xF2}
	},
	{
		2, {0x9B, 0xF1},
		3, {0xE5, 0xB9, 0x87},
		2, {0x47, 0x5E},
		2, {0xD6, 0xF3}
	},
	{
		2, {0x9B, 0xF2},
		3, {0xE5, 0xB9, 0xB5},
		2, {0x75, 0x5E},
		2, {0xD6, 0xF4}
	},
	{
		2, {0x9B, 0xF3},
		3, {0xE5, 0xB9, 0xB6},
		2, {0x76, 0x5E},
		2, {0xD6, 0xF5}
	},
	{
		2, {0x9B, 0xF4},
		3, {0xE5, 0xB9, 0xBA},
		2, {0x7A, 0x5E},
		2, {0xD6, 0xF6}
	},
	{
		2, {0x9B, 0xF5},
		3, {0xE9, 0xBA, 0xBC},
		2, {0xBC, 0x9E},
		2, {0xD6, 0xF7}
	},
	{
		2, {0x9B, 0xF6},
		3, {0xE5, 0xB9, 0xBF},
		2, {0x7F, 0x5E},
		2, {0xD6, 0xF8}
	},
	{
		2, {0x9B, 0xF7},
		3, {0xE5, 0xBA, 0xA0},
		2, {0xA0, 0x5E},
		2, {0xD6, 0xF9}
	},
	{
		2, {0x9B, 0xF8},
		3, {0xE5, 0xBB, 0x81},
		2, {0xC1, 0x5E},
		2, {0xD6, 0xFA}
	},
	{
		2, {0x9B, 0xF9},
		3, {0xE5, 0xBB, 0x82},
		2, {0xC2, 0x5E},
		2, {0xD6, 0xFB}
	},
	{
		2, {0x9B, 0xFA},
		3, {0xE5, 0xBB, 0x88},
		2, {0xC8, 0x5E},
		2, {0xD6, 0xFC}
	},
	{
		2, {0x9B, 0xFB},
		3, {0xE5, 0xBB, 0x90},
		2, {0xD0, 0x5E},
		2, {0xD6, 0xFD}
	},
	{
		2, {0x9B, 0xFC},
		3, {0xE5, 0xBB, 0x8F},
		2, {0xCF, 0x5E},
		2, {0xD6, 0xFE}
	},
	{
		2, {0x9C, 0x40},
		3, {0xE5, 0xBB, 0x96},
		2, {0xD6, 0x5E},
		2, {0xD7, 0xA1}
	},
	{
		2, {0x9C, 0x41},
		3, {0xE5, 0xBB, 0xA3},
		2, {0xE3, 0x5E},
		2, {0xD7, 0xA2}
	},
	{
		2, {0x9C, 0x42},
		3, {0xE5, 0xBB, 0x9D},
		2, {0xDD, 0x5E},
		2, {0xD7, 0xA3}
	},
	{
		2, {0x9C, 0x43},
		3, {0xE5, 0xBB, 0x9A},
		2, {0xDA, 0x5E},
		2, {0xD7, 0xA4}
	},
	{
		2, {0x9C, 0x44},
		3, {0xE5, 0xBB, 0x9B},
		2, {0xDB, 0x5E},
		2, {0xD7, 0xA5}
	},
	{
		2, {0x9C, 0x45},
		3, {0xE5, 0xBB, 0xA2},
		2, {0xE2, 0x5E},
		2, {0xD7, 0xA6}
	},
	{
		2, {0x9C, 0x46},
		3, {0xE5, 0xBB, 0xA1},
		2, {0xE1, 0x5E},
		2, {0xD7, 0xA7}
	},
	{
		2, {0x9C, 0x47},
		3, {0xE5, 0xBB, 0xA8},
		2, {0xE8, 0x5E},
		2, {0xD7, 0xA8}
	},
	{
		2, {0x9C, 0x48},
		3, {0xE5, 0xBB, 0xA9},
		2, {0xE9, 0x5E},
		2, {0xD7, 0xA9}
	},
	{
		2, {0x9C, 0x49},
		3, {0xE5, 0xBB, 0xAC},
		2, {0xEC, 0x5E},
		2, {0xD7, 0xAA}
	},
	{
		2, {0x9C, 0x4A},
		3, {0xE5, 0xBB, 0xB1},
		2, {0xF1, 0x5E},
		2, {0xD7, 0xAB}
	},
	{
		2, {0x9C, 0x4B},
		3, {0xE5, 0xBB, 0xB3},
		2, {0xF3, 0x5E},
		2, {0xD7, 0xAC}
	},
	{
		2, {0x9C, 0x4C},
		3, {0xE5, 0xBB, 0xB0},
		2, {0xF0, 0x5E},
		2, {0xD7, 0xAD}
	},
	{
		2, {0x9C, 0x4D},
		3, {0xE5, 0xBB, 0xB4},
		2, {0xF4, 0x5E},
		2, {0xD7, 0xAE}
	},
	{
		2, {0x9C, 0x4E},
		3, {0xE5, 0xBB, 0xB8},
		2, {0xF8, 0x5E},
		2, {0xD7, 0xAF}
	},
	{
		2, {0x9C, 0x4F},
		3, {0xE5, 0xBB, 0xBE},
		2, {0xFE, 0x5E},
		2, {0xD7, 0xB0}
	},
	{
		2, {0x9C, 0x50},
		3, {0xE5, 0xBC, 0x83},
		2, {0x03, 0x5F},
		2, {0xD7, 0xB1}
	},
	{
		2, {0x9C, 0x51},
		3, {0xE5, 0xBC, 0x89},
		2, {0x09, 0x5F},
		2, {0xD7, 0xB2}
	},
	{
		2, {0x9C, 0x52},
		3, {0xE5, 0xBD, 0x9D},
		2, {0x5D, 0x5F},
		2, {0xD7, 0xB3}
	},
	{
		2, {0x9C, 0x53},
		3, {0xE5, 0xBD, 0x9C},
		2, {0x5C, 0x5F},
		2, {0xD7, 0xB4}
	},
	{
		2, {0x9C, 0x54},
		3, {0xE5, 0xBC, 0x8B},
		2, {0x0B, 0x5F},
		2, {0xD7, 0xB5}
	},
	{
		2, {0x9C, 0x55},
		3, {0xE5, 0xBC, 0x91},
		2, {0x11, 0x5F},
		2, {0xD7, 0xB6}
	},
	{
		2, {0x9C, 0x56},
		3, {0xE5, 0xBC, 0x96},
		2, {0x16, 0x5F},
		2, {0xD7, 0xB7}
	},
	{
		2, {0x9C, 0x57},
		3, {0xE5, 0xBC, 0xA9},
		2, {0x29, 0x5F},
		2, {0xD7, 0xB8}
	},
	{
		2, {0x9C, 0x58},
		3, {0xE5, 0xBC, 0xAD},
		2, {0x2D, 0x5F},
		2, {0xD7, 0xB9}
	},
	{
		2, {0x9C, 0x59},
		3, {0xE5, 0xBC, 0xB8},
		2, {0x38, 0x5F},
		2, {0xD7, 0xBA}
	},
	{
		2, {0x9C, 0x5A},
		3, {0xE5, 0xBD, 0x81},
		2, {0x41, 0x5F},
		2, {0xD7, 0xBB}
	},
	{
		2, {0x9C, 0x5B},
		3, {0xE5, 0xBD, 0x88},
		2, {0x48, 0x5F},
		2, {0xD7, 0xBC}
	},
	{
		2, {0x9C, 0x5C},
		3, {0xE5, 0xBD, 0x8C},
		2, {0x4C, 0x5F},
		2, {0xD7, 0xBD}
	},
	{
		2, {0x9C, 0x5D},
		3, {0xE5, 0xBD, 0x8E},
		2, {0x4E, 0x5F},
		2, {0xD7, 0xBE}
	},
	{
		2, {0x9C, 0x5E},
		3, {0xE5, 0xBC, 0xAF},
		2, {0x2F, 0x5F},
		2, {0xD7, 0xBF}
	},
	{
		2, {0x9C, 0x5F},
		3, {0xE5, 0xBD, 0x91},
		2, {0x51, 0x5F},
		2, {0xD7, 0xC0}
	},
	{
		2, {0x9C, 0x60},
		3, {0xE5, 0xBD, 0x96},
		2, {0x56, 0x5F},
		2, {0xD7, 0xC1}
	},
	{
		2, {0x9C, 0x61},
		3, {0xE5, 0xBD, 0x97},
		2, {0x57, 0x5F},
		2, {0xD7, 0xC2}
	},
	{
		2, {0x9C, 0x62},
		3, {0xE5, 0xBD, 0x99},
		2, {0x59, 0x5F},
		2, {0xD7, 0xC3}
	},
	{
		2, {0x9C, 0x63},
		3, {0xE5, 0xBD, 0xA1},
		2, {0x61, 0x5F},
		2, {0xD7, 0xC4}
	},
	{
		2, {0x9C, 0x64},
		3, {0xE5, 0xBD, 0xAD},
		2, {0x6D, 0x5F},
		2, {0xD7, 0xC5}
	},
	{
		2, {0x9C, 0x65},
		3, {0xE5, 0xBD, 0xB3},
		2, {0x73, 0x5F},
		2, {0xD7, 0xC6}
	},
	{
		2, {0x9C, 0x66},
		3, {0xE5, 0xBD, 0xB7},
		2, {0x77, 0x5F},
		2, {0xD7, 0xC7}
	},
	{
		2, {0x9C, 0x67},
		3, {0xE5, 0xBE, 0x83},
		2, {0x83, 0x5F},
		2, {0xD7, 0xC8}
	},
	{
		2, {0x9C, 0x68},
		3, {0xE5, 0xBE, 0x82},
		2, {0x82, 0x5F},
		2, {0xD7, 0xC9}
	},
	{
		2, {0x9C, 0x69},
		3, {0xE5, 0xBD, 0xBF},
		2, {0x7F, 0x5F},
		2, {0xD7, 0xCA}
	},
	{
		2, {0x9C, 0x6A},
		3, {0xE5, 0xBE, 0x8A},
		2, {0x8A, 0x5F},
		2, {0xD7, 0xCB}
	},
	{
		2, {0x9C, 0x6B},
		3, {0xE5, 0xBE, 0x88},
		2, {0x88, 0x5F},
		2, {0xD7, 0xCC}
	},
	{
		2, {0x9C, 0x6C},
		3, {0xE5, 0xBE, 0x91},
		2, {0x91, 0x5F},
		2, {0xD7, 0xCD}
	},
	{
		2, {0x9C, 0x6D},
		3, {0xE5, 0xBE, 0x87},
		2, {0x87, 0x5F},
		2, {0xD7, 0xCE}
	},
	{
		2, {0x9C, 0x6E},
		3, {0xE5, 0xBE, 0x9E},
		2, {0x9E, 0x5F},
		2, {0xD7, 0xCF}
	},
	{
		2, {0x9C, 0x6F},
		3, {0xE5, 0xBE, 0x99},
		2, {0x99, 0x5F},
		2, {0xD7, 0xD0}
	},
	{
		2, {0x9C, 0x70},
		3, {0xE5, 0xBE, 0x98},
		2, {0x98, 0x5F},
		2, {0xD7, 0xD1}
	},
	{
		2, {0x9C, 0x71},
		3, {0xE5, 0xBE, 0xA0},
		2, {0xA0, 0x5F},
		2, {0xD7, 0xD2}
	},
	{
		2, {0x9C, 0x72},
		3, {0xE5, 0xBE, 0xA8},
		2, {0xA8, 0x5F},
		2, {0xD7, 0xD3}
	},
	{
		2, {0x9C, 0x73},
		3, {0xE5, 0xBE, 0xAD},
		2, {0xAD, 0x5F},
		2, {0xD7, 0xD4}
	},
	{
		2, {0x9C, 0x74},
		3, {0xE5, 0xBE, 0xBC},
		2, {0xBC, 0x5F},
		2, {0xD7, 0xD5}
	},
	{
		2, {0x9C, 0x75},
		3, {0xE5, 0xBF, 0x96},
		2, {0xD6, 0x5F},
		2, {0xD7, 0xD6}
	},
	{
		2, {0x9C, 0x76},
		3, {0xE5, 0xBF, 0xBB},
		2, {0xFB, 0x5F},
		2, {0xD7, 0xD7}
	},
	{
		2, {0x9C, 0x77},
		3, {0xE5, 0xBF, 0xA4},
		2, {0xE4, 0x5F},
		2, {0xD7, 0xD8}
	},
	{
		2, {0x9C, 0x78},
		3, {0xE5, 0xBF, 0xB8},
		2, {0xF8, 0x5F},
		2, {0xD7, 0xD9}
	},
	{
		2, {0x9C, 0x79},
		3, {0xE5, 0xBF, 0xB1},
		2, {0xF1, 0x5F},
		2, {0xD7, 0xDA}
	},
	{
		2, {0x9C, 0x7A},
		3, {0xE5, 0xBF, 0x9D},
		2, {0xDD, 0x5F},
		2, {0xD7, 0xDB}
	},
	{
		2, {0x9C, 0x7B},
		3, {0xE6, 0x82, 0xB3},
		2, {0xB3, 0x60},
		2, {0xD7, 0xDC}
	},
	{
		2, {0x9C, 0x7C},
		3, {0xE5, 0xBF, 0xBF},
		2, {0xFF, 0x5F},
		2, {0xD7, 0xDD}
	},
	{
		2, {0x9C, 0x7D},
		3, {0xE6, 0x80, 0xA1},
		2, {0x21, 0x60},
		2, {0xD7, 0xDE}
	},
	{
		2, {0x9C, 0x7E},
		3, {0xE6, 0x81, 0xA0},
		2, {0x60, 0x60},
		2, {0xD7, 0xDF}
	},
	{
		2, {0x9C, 0x80},
		3, {0xE6, 0x80, 0x99},
		2, {0x19, 0x60},
		2, {0xD7, 0xE0}
	},
	{
		2, {0x9C, 0x81},
		3, {0xE6, 0x80, 0x90},
		2, {0x10, 0x60},
		2, {0xD7, 0xE1}
	},
	{
		2, {0x9C, 0x82},
		3, {0xE6, 0x80, 0xA9},
		2, {0x29, 0x60},
		2, {0xD7, 0xE2}
	},
	{
		2, {0x9C, 0x83},
		3, {0xE6, 0x80, 0x8E},
		2, {0x0E, 0x60},
		2, {0xD7, 0xE3}
	},
	{
		2, {0x9C, 0x84},
		3, {0xE6, 0x80, 0xB1},
		2, {0x31, 0x60},
		2, {0xD7, 0xE4}
	},
	{
		2, {0x9C, 0x85},
		3, {0xE6, 0x80, 0x9B},
		2, {0x1B, 0x60},
		2, {0xD7, 0xE5}
	},
	{
		2, {0x9C, 0x86},
		3, {0xE6, 0x80, 0x95},
		2, {0x15, 0x60},
		2, {0xD7, 0xE6}
	},
	{
		2, {0x9C, 0x87},
		3, {0xE6, 0x80, 0xAB},
		2, {0x2B, 0x60},
		2, {0xD7, 0xE7}
	},
	{
		2, {0x9C, 0x88},
		3, {0xE6, 0x80, 0xA6},
		2, {0x26, 0x60},
		2, {0xD7, 0xE8}
	},
	{
		2, {0x9C, 0x89},
		3, {0xE6, 0x80, 0x8F},
		2, {0x0F, 0x60},
		2, {0xD7, 0xE9}
	},
	{
		2, {0x9C, 0x8A},
		3, {0xE6, 0x80, 0xBA},
		2, {0x3A, 0x60},
		2, {0xD7, 0xEA}
	},
	{
		2, {0x9C, 0x8B},
		3, {0xE6, 0x81, 0x9A},
		2, {0x5A, 0x60},
		2, {0xD7, 0xEB}
	},
	{
		2, {0x9C, 0x8C},
		3, {0xE6, 0x81, 0x81},
		2, {0x41, 0x60},
		2, {0xD7, 0xEC}
	},
	{
		2, {0x9C, 0x8D},
		3, {0xE6, 0x81, 0xAA},
		2, {0x6A, 0x60},
		2, {0xD7, 0xED}
	},
	{
		2, {0x9C, 0x8E},
		3, {0xE6, 0x81, 0xB7},
		2, {0x77, 0x60},
		2, {0xD7, 0xEE}
	},
	{
		2, {0x9C, 0x8F},
		3, {0xE6, 0x81, 0x9F},
		2, {0x5F, 0x60},
		2, {0xD7, 0xEF}
	},
	{
		2, {0x9C, 0x90},
		3, {0xE6, 0x81, 0x8A},
		2, {0x4A, 0x60},
		2, {0xD7, 0xF0}
	},
	{
		2, {0x9C, 0x91},
		3, {0xE6, 0x81, 0x86},
		2, {0x46, 0x60},
		2, {0xD7, 0xF1}
	},
	{
		2, {0x9C, 0x92},
		3, {0xE6, 0x81, 0x8D},
		2, {0x4D, 0x60},
		2, {0xD7, 0xF2}
	},
	{
		2, {0x9C, 0x93},
		3, {0xE6, 0x81, 0xA3},
		2, {0x63, 0x60},
		2, {0xD7, 0xF3}
	},
	{
		2, {0x9C, 0x94},
		3, {0xE6, 0x81, 0x83},
		2, {0x43, 0x60},
		2, {0xD7, 0xF4}
	},
	{
		2, {0x9C, 0x95},
		3, {0xE6, 0x81, 0xA4},
		2, {0x64, 0x60},
		2, {0xD7, 0xF5}
	},
	{
		2, {0x9C, 0x96},
		3, {0xE6, 0x81, 0x82},
		2, {0x42, 0x60},
		2, {0xD7, 0xF6}
	},
	{
		2, {0x9C, 0x97},
		3, {0xE6, 0x81, 0xAC},
		2, {0x6C, 0x60},
		2, {0xD7, 0xF7}
	},
	{
		2, {0x9C, 0x98},
		3, {0xE6, 0x81, 0xAB},
		2, {0x6B, 0x60},
		2, {0xD7, 0xF8}
	},
	{
		2, {0x9C, 0x99},
		3, {0xE6, 0x81, 0x99},
		2, {0x59, 0x60},
		2, {0xD7, 0xF9}
	},
	{
		2, {0x9C, 0x9A},
		3, {0xE6, 0x82, 0x81},
		2, {0x81, 0x60},
		2, {0xD7, 0xFA}
	},
	{
		2, {0x9C, 0x9B},
		3, {0xE6, 0x82, 0x8D},
		2, {0x8D, 0x60},
		2, {0xD7, 0xFB}
	},
	{
		2, {0x9C, 0x9C},
		3, {0xE6, 0x83, 0xA7},
		2, {0xE7, 0x60},
		2, {0xD7, 0xFC}
	},
	{
		2, {0x9C, 0x9D},
		3, {0xE6, 0x82, 0x83},
		2, {0x83, 0x60},
		2, {0xD7, 0xFD}
	},
	{
		2, {0x9C, 0x9E},
		3, {0xE6, 0x82, 0x9A},
		2, {0x9A, 0x60},
		2, {0xD7, 0xFE}
	},
	{
		2, {0x9C, 0x9F},
		3, {0xE6, 0x82, 0x84},
		2, {0x84, 0x60},
		2, {0xD8, 0xA1}
	},
	{
		2, {0x9C, 0xA0},
		3, {0xE6, 0x82, 0x9B},
		2, {0x9B, 0x60},
		2, {0xD8, 0xA2}
	},
	{
		2, {0x9C, 0xA1},
		3, {0xE6, 0x82, 0x96},
		2, {0x96, 0x60},
		2, {0xD8, 0xA3}
	},
	{
		2, {0x9C, 0xA2},
		3, {0xE6, 0x82, 0x97},
		2, {0x97, 0x60},
		2, {0xD8, 0xA4}
	},
	{
		2, {0x9C, 0xA3},
		3, {0xE6, 0x82, 0x92},
		2, {0x92, 0x60},
		2, {0xD8, 0xA5}
	},
	{
		2, {0x9C, 0xA4},
		3, {0xE6, 0x82, 0xA7},
		2, {0xA7, 0x60},
		2, {0xD8, 0xA6}
	},
	{
		2, {0x9C, 0xA5},
		3, {0xE6, 0x82, 0x8B},
		2, {0x8B, 0x60},
		2, {0xD8, 0xA7}
	},
	{
		2, {0x9C, 0xA6},
		3, {0xE6, 0x83, 0xA1},
		2, {0xE1, 0x60},
		2, {0xD8, 0xA8}
	},
	{
		2, {0x9C, 0xA7},
		3, {0xE6, 0x82, 0xB8},
		2, {0xB8, 0x60},
		2, {0xD8, 0xA9}
	},
	{
		2, {0x9C, 0xA8},
		3, {0xE6, 0x83, 0xA0},
		2, {0xE0, 0x60},
		2, {0xD8, 0xAA}
	},
	{
		2, {0x9C, 0xA9},
		3, {0xE6, 0x83, 0x93},
		2, {0xD3, 0x60},
		2, {0xD8, 0xAB}
	},
	{
		2, {0x9C, 0xAA},
		3, {0xE6, 0x82, 0xB4},
		2, {0xB4, 0x60},
		2, {0xD8, 0xAC}
	},
	{
		2, {0x9C, 0xAB},
		3, {0xE5, 0xBF, 0xB0},
		2, {0xF0, 0x5F},
		2, {0xD8, 0xAD}
	},
	{
		2, {0x9C, 0xAC},
		3, {0xE6, 0x82, 0xBD},
		2, {0xBD, 0x60},
		2, {0xD8, 0xAE}
	},
	{
		2, {0x9C, 0xAD},
		3, {0xE6, 0x83, 0x86},
		2, {0xC6, 0x60},
		2, {0xD8, 0xAF}
	},
	{
		2, {0x9C, 0xAE},
		3, {0xE6, 0x82, 0xB5},
		2, {0xB5, 0x60},
		2, {0xD8, 0xB0}
	},
	{
		2, {0x9C, 0xAF},
		3, {0xE6, 0x83, 0x98},
		2, {0xD8, 0x60},
		2, {0xD8, 0xB1}
	},
	{
		2, {0x9C, 0xB0},
		3, {0xE6, 0x85, 0x8D},
		2, {0x4D, 0x61},
		2, {0xD8, 0xB2}
	},
	{
		2, {0x9C, 0xB1},
		3, {0xE6, 0x84, 0x95},
		2, {0x15, 0x61},
		2, {0xD8, 0xB3}
	},
	{
		2, {0x9C, 0xB2},
		3, {0xE6, 0x84, 0x86},
		2, {0x06, 0x61},
		2, {0xD8, 0xB4}
	},
	{
		2, {0x9C, 0xB3},
		3, {0xE6, 0x83, 0xB6},
		2, {0xF6, 0x60},
		2, {0xD8, 0xB5}
	},
	{
		2, {0x9C, 0xB4},
		3, {0xE6, 0x83, 0xB7},
		2, {0xF7, 0x60},
		2, {0xD8, 0xB6}
	},
	{
		2, {0x9C, 0xB5},
		3, {0xE6, 0x84, 0x80},
		2, {0x00, 0x61},
		2, {0xD8, 0xB7}
	},
	{
		2, {0x9C, 0xB6},
		3, {0xE6, 0x83, 0xB4},
		2, {0xF4, 0x60},
		2, {0xD8, 0xB8}
	},
	{
		2, {0x9C, 0xB7},
		3, {0xE6, 0x83, 0xBA},
		2, {0xFA, 0x60},
		2, {0xD8, 0xB9}
	},
	{
		2, {0x9C, 0xB8},
		3, {0xE6, 0x84, 0x83},
		2, {0x03, 0x61},
		2, {0xD8, 0xBA}
	},
	{
		2, {0x9C, 0xB9},
		3, {0xE6, 0x84, 0xA1},
		2, {0x21, 0x61},
		2, {0xD8, 0xBB}
	},
	{
		2, {0x9C, 0xBA},
		3, {0xE6, 0x83, 0xBB},
		2, {0xFB, 0x60},
		2, {0xD8, 0xBC}
	},
	{
		2, {0x9C, 0xBB},
		3, {0xE6, 0x83, 0xB1},
		2, {0xF1, 0x60},
		2, {0xD8, 0xBD}
	},
	{
		2, {0x9C, 0xBC},
		3, {0xE6, 0x84, 0x8D},
		2, {0x0D, 0x61},
		2, {0xD8, 0xBE}
	},
	{
		2, {0x9C, 0xBD},
		3, {0xE6, 0x84, 0x8E},
		2, {0x0E, 0x61},
		2, {0xD8, 0xBF}
	},
	{
		2, {0x9C, 0xBE},
		3, {0xE6, 0x85, 0x87},
		2, {0x47, 0x61},
		2, {0xD8, 0xC0}
	},
	{
		2, {0x9C, 0xBF},
		3, {0xE6, 0x84, 0xBE},
		2, {0x3E, 0x61},
		2, {0xD8, 0xC1}
	},
	{
		2, {0x9C, 0xC0},
		3, {0xE6, 0x84, 0xA8},
		2, {0x28, 0x61},
		2, {0xD8, 0xC2}
	},
	{
		2, {0x9C, 0xC1},
		3, {0xE6, 0x84, 0xA7},
		2, {0x27, 0x61},
		2, {0xD8, 0xC3}
	},
	{
		2, {0x9C, 0xC2},
		3, {0xE6, 0x85, 0x8A},
		2, {0x4A, 0x61},
		2, {0xD8, 0xC4}
	},
	{
		2, {0x9C, 0xC3},
		3, {0xE6, 0x84, 0xBF},
		2, {0x3F, 0x61},
		2, {0xD8, 0xC5}
	},
	{
		2, {0x9C, 0xC4},
		3, {0xE6, 0x84, 0xBC},
		2, {0x3C, 0x61},
		2, {0xD8, 0xC6}
	},
	{
		2, {0x9C, 0xC5},
		3, {0xE6, 0x84, 0xAC},
		2, {0x2C, 0x61},
		2, {0xD8, 0xC7}
	},
	{
		2, {0x9C, 0xC6},
		3, {0xE6, 0x84, 0xB4},
		2, {0x34, 0x61},
		2, {0xD8, 0xC8}
	},
	{
		2, {0x9C, 0xC7},
		3, {0xE6, 0x84, 0xBD},
		2, {0x3D, 0x61},
		2, {0xD8, 0xC9}
	},
	{
		2, {0x9C, 0xC8},
		3, {0xE6, 0x85, 0x82},
		2, {0x42, 0x61},
		2, {0xD8, 0xCA}
	},
	{
		2, {0x9C, 0xC9},
		3, {0xE6, 0x85, 0x84},
		2, {0x44, 0x61},
		2, {0xD8, 0xCB}
	},
	{
		2, {0x9C, 0xCA},
		3, {0xE6, 0x85, 0xB3},
		2, {0x73, 0x61},
		2, {0xD8, 0xCC}
	},
	{
		2, {0x9C, 0xCB},
		3, {0xE6, 0x85, 0xB7},
		2, {0x77, 0x61},
		2, {0xD8, 0xCD}
	},
	{
		2, {0x9C, 0xCC},
		3, {0xE6, 0x85, 0x98},
		2, {0x58, 0x61},
		2, {0xD8, 0xCE}
	},
	{
		2, {0x9C, 0xCD},
		3, {0xE6, 0x85, 0x99},
		2, {0x59, 0x61},
		2, {0xD8, 0xCF}
	},
	{
		2, {0x9C, 0xCE},
		3, {0xE6, 0x85, 0x9A},
		2, {0x5A, 0x61},
		2, {0xD8, 0xD0}
	},
	{
		2, {0x9C, 0xCF},
		3, {0xE6, 0x85, 0xAB},
		2, {0x6B, 0x61},
		2, {0xD8, 0xD1}
	},
	{
		2, {0x9C, 0xD0},
		3, {0xE6, 0x85, 0xB4},
		2, {0x74, 0x61},
		2, {0xD8, 0xD2}
	},
	{
		2, {0x9C, 0xD1},
		3, {0xE6, 0x85, 0xAF},
		2, {0x6F, 0x61},
		2, {0xD8, 0xD3}
	},
	{
		2, {0x9C, 0xD2},
		3, {0xE6, 0x85, 0xA5},
		2, {0x65, 0x61},
		2, {0xD8, 0xD4}
	},
	{
		2, {0x9C, 0xD3},
		3, {0xE6, 0x85, 0xB1},
		2, {0x71, 0x61},
		2, {0xD8, 0xD5}
	},
	{
		2, {0x9C, 0xD4},
		3, {0xE6, 0x85, 0x9F},
		2, {0x5F, 0x61},
		2, {0xD8, 0xD6}
	},
	{
		2, {0x9C, 0xD5},
		3, {0xE6, 0x85, 0x9D},
		2, {0x5D, 0x61},
		2, {0xD8, 0xD7}
	},
	{
		2, {0x9C, 0xD6},
		3, {0xE6, 0x85, 0x93},
		2, {0x53, 0x61},
		2, {0xD8, 0xD8}
	},
	{
		2, {0x9C, 0xD7},
		3, {0xE6, 0x85, 0xB5},
		2, {0x75, 0x61},
		2, {0xD8, 0xD9}
	},
	{
		2, {0x9C, 0xD8},
		3, {0xE6, 0x86, 0x99},
		2, {0x99, 0x61},
		2, {0xD8, 0xDA}
	},
	{
		2, {0x9C, 0xD9},
		3, {0xE6, 0x86, 0x96},
		2, {0x96, 0x61},
		2, {0xD8, 0xDB}
	},
	{
		2, {0x9C, 0xDA},
		3, {0xE6, 0x86, 0x87},
		2, {0x87, 0x61},
		2, {0xD8, 0xDC}
	},
	{
		2, {0x9C, 0xDB},
		3, {0xE6, 0x86, 0xAC},
		2, {0xAC, 0x61},
		2, {0xD8, 0xDD}
	},
	{
		2, {0x9C, 0xDC},
		3, {0xE6, 0x86, 0x94},
		2, {0x94, 0x61},
		2, {0xD8, 0xDE}
	},
	{
		2, {0x9C, 0xDD},
		3, {0xE6, 0x86, 0x9A},
		2, {0x9A, 0x61},
		2, {0xD8, 0xDF}
	},
	{
		2, {0x9C, 0xDE},
		3, {0xE6, 0x86, 0x8A},
		2, {0x8A, 0x61},
		2, {0xD8, 0xE0}
	},
	{
		2, {0x9C, 0xDF},
		3, {0xE6, 0x86, 0x91},
		2, {0x91, 0x61},
		2, {0xD8, 0xE1}
	},
	{
		2, {0x9C, 0xE0},
		3, {0xE6, 0x86, 0xAB},
		2, {0xAB, 0x61},
		2, {0xD8, 0xE2}
	},
	{
		2, {0x9C, 0xE1},
		3, {0xE6, 0x86, 0xAE},
		2, {0xAE, 0x61},
		2, {0xD8, 0xE3}
	},
	{
		2, {0x9C, 0xE2},
		3, {0xE6, 0x87, 0x8C},
		2, {0xCC, 0x61},
		2, {0xD8, 0xE4}
	},
	{
		2, {0x9C, 0xE3},
		3, {0xE6, 0x87, 0x8A},
		2, {0xCA, 0x61},
		2, {0xD8, 0xE5}
	},
	{
		2, {0x9C, 0xE4},
		3, {0xE6, 0x87, 0x89},
		2, {0xC9, 0x61},
		2, {0xD8, 0xE6}
	},
	{
		2, {0x9C, 0xE5},
		3, {0xE6, 0x87, 0xB7},
		2, {0xF7, 0x61},
		2, {0xD8, 0xE7}
	},
	{
		2, {0x9C, 0xE6},
		3, {0xE6, 0x87, 0x88},
		2, {0xC8, 0x61},
		2, {0xD8, 0xE8}
	},
	{
		2, {0x9C, 0xE7},
		3, {0xE6, 0x87, 0x83},
		2, {0xC3, 0x61},
		2, {0xD8, 0xE9}
	},
	{
		2, {0x9C, 0xE8},
		3, {0xE6, 0x87, 0x86},
		2, {0xC6, 0x61},
		2, {0xD8, 0xEA}
	},
	{
		2, {0x9C, 0xE9},
		3, {0xE6, 0x86, 0xBA},
		2, {0xBA, 0x61},
		2, {0xD8, 0xEB}
	},
	{
		2, {0x9C, 0xEA},
		3, {0xE6, 0x87, 0x8B},
		2, {0xCB, 0x61},
		2, {0xD8, 0xEC}
	},
	{
		2, {0x9C, 0xEB},
		3, {0xE7, 0xBD, 0xB9},
		2, {0x79, 0x7F},
		2, {0xD8, 0xED}
	},
	{
		2, {0x9C, 0xEC},
		3, {0xE6, 0x87, 0x8D},
		2, {0xCD, 0x61},
		2, {0xD8, 0xEE}
	},
	{
		2, {0x9C, 0xED},
		3, {0xE6, 0x87, 0xA6},
		2, {0xE6, 0x61},
		2, {0xD8, 0xEF}
	},
	{
		2, {0x9C, 0xEE},
		3, {0xE6, 0x87, 0xA3},
		2, {0xE3, 0x61},
		2, {0xD8, 0xF0}
	},
	{
		2, {0x9C, 0xEF},
		3, {0xE6, 0x87, 0xB6},
		2, {0xF6, 0x61},
		2, {0xD8, 0xF1}
	},
	{
		2, {0x9C, 0xF0},
		3, {0xE6, 0x87, 0xBA},
		2, {0xFA, 0x61},
		2, {0xD8, 0xF2}
	},
	{
		2, {0x9C, 0xF1},
		3, {0xE6, 0x87, 0xB4},
		2, {0xF4, 0x61},
		2, {0xD8, 0xF3}
	},
	{
		2, {0x9C, 0xF2},
		3, {0xE6, 0x87, 0xBF},
		2, {0xFF, 0x61},
		2, {0xD8, 0xF4}
	},
	{
		2, {0x9C, 0xF3},
		3, {0xE6, 0x87, 0xBD},
		2, {0xFD, 0x61},
		2, {0xD8, 0xF5}
	},
	{
		2, {0x9C, 0xF4},
		3, {0xE6, 0x87, 0xBC},
		2, {0xFC, 0x61},
		2, {0xD8, 0xF6}
	},
	{
		2, {0x9C, 0xF5},
		3, {0xE6, 0x87, 0xBE},
		2, {0xFE, 0x61},
		2, {0xD8, 0xF7}
	},
	{
		2, {0x9C, 0xF6},
		3, {0xE6, 0x88, 0x80},
		2, {0x00, 0x62},
		2, {0xD8, 0xF8}
	},
	{
		2, {0x9C, 0xF7},
		3, {0xE6, 0x88, 0x88},
		2, {0x08, 0x62},
		2, {0xD8, 0xF9}
	},
	{
		2, {0x9C, 0xF8},
		3, {0xE6, 0x88, 0x89},
		2, {0x09, 0x62},
		2, {0xD8, 0xFA}
	},
	{
		2, {0x9C, 0xF9},
		3, {0xE6, 0x88, 0x8D},
		2, {0x0D, 0x62},
		2, {0xD8, 0xFB}
	},
	{
		2, {0x9C, 0xFA},
		3, {0xE6, 0x88, 0x8C},
		2, {0x0C, 0x62},
		2, {0xD8, 0xFC}
	},
	{
		2, {0x9C, 0xFB},
		3, {0xE6, 0x88, 0x94},
		2, {0x14, 0x62},
		2, {0xD8, 0xFD}
	},
	{
		2, {0x9C, 0xFC},
		3, {0xE6, 0x88, 0x9B},
		2, {0x1B, 0x62},
		2, {0xD8, 0xFE}
	},
	{
		2, {0x9D, 0x40},
		3, {0xE6, 0x88, 0x9E},
		2, {0x1E, 0x62},
		2, {0xD9, 0xA1}
	},
	{
		2, {0x9D, 0x41},
		3, {0xE6, 0x88, 0xA1},
		2, {0x21, 0x62},
		2, {0xD9, 0xA2}
	},
	{
		2, {0x9D, 0x42},
		3, {0xE6, 0x88, 0xAA},
		2, {0x2A, 0x62},
		2, {0xD9, 0xA3}
	},
	{
		2, {0x9D, 0x43},
		3, {0xE6, 0x88, 0xAE},
		2, {0x2E, 0x62},
		2, {0xD9, 0xA4}
	},
	{
		2, {0x9D, 0x44},
		3, {0xE6, 0x88, 0xB0},
		2, {0x30, 0x62},
		2, {0xD9, 0xA5}
	},
	{
		2, {0x9D, 0x45},
		3, {0xE6, 0x88, 0xB2},
		2, {0x32, 0x62},
		2, {0xD9, 0xA6}
	},
	{
		2, {0x9D, 0x46},
		3, {0xE6, 0x88, 0xB3},
		2, {0x33, 0x62},
		2, {0xD9, 0xA7}
	},
	{
		2, {0x9D, 0x47},
		3, {0xE6, 0x89, 0x81},
		2, {0x41, 0x62},
		2, {0xD9, 0xA8}
	},
	{
		2, {0x9D, 0x48},
		3, {0xE6, 0x89, 0x8E},
		2, {0x4E, 0x62},
		2, {0xD9, 0xA9}
	},
	{
		2, {0x9D, 0x49},
		3, {0xE6, 0x89, 0x9E},
		2, {0x5E, 0x62},
		2, {0xD9, 0xAA}
	},
	{
		2, {0x9D, 0x4A},
		3, {0xE6, 0x89, 0xA3},
		2, {0x63, 0x62},
		2, {0xD9, 0xAB}
	},
	{
		2, {0x9D, 0x4B},
		3, {0xE6, 0x89, 0x9B},
		2, {0x5B, 0x62},
		2, {0xD9, 0xAC}
	},
	{
		2, {0x9D, 0x4C},
		3, {0xE6, 0x89, 0xA0},
		2, {0x60, 0x62},
		2, {0xD9, 0xAD}
	},
	{
		2, {0x9D, 0x4D},
		3, {0xE6, 0x89, 0xA8},
		2, {0x68, 0x62},
		2, {0xD9, 0xAE}
	},
	{
		2, {0x9D, 0x4E},
		3, {0xE6, 0x89, 0xBC},
		2, {0x7C, 0x62},
		2, {0xD9, 0xAF}
	},
	{
		2, {0x9D, 0x4F},
		3, {0xE6, 0x8A, 0x82},
		2, {0x82, 0x62},
		2, {0xD9, 0xB0}
	},
	{
		2, {0x9D, 0x50},
		3, {0xE6, 0x8A, 0x89},
		2, {0x89, 0x62},
		2, {0xD9, 0xB1}
	},
	{
		2, {0x9D, 0x51},
		3, {0xE6, 0x89, 0xBE},
		2, {0x7E, 0x62},
		2, {0xD9, 0xB2}
	},
	{
		2, {0x9D, 0x52},
		3, {0xE6, 0x8A, 0x92},
		2, {0x92, 0x62},
		2, {0xD9, 0xB3}
	},
	{
		2, {0x9D, 0x53},
		3, {0xE6, 0x8A, 0x93},
		2, {0x93, 0x62},
		2, {0xD9, 0xB4}
	},
	{
		2, {0x9D, 0x54},
		3, {0xE6, 0x8A, 0x96},
		2, {0x96, 0x62},
		2, {0xD9, 0xB5}
	},
	{
		2, {0x9D, 0x55},
		3, {0xE6, 0x8B, 0x94},
		2, {0xD4, 0x62},
		2, {0xD9, 0xB6}
	},
	{
		2, {0x9D, 0x56},
		3, {0xE6, 0x8A, 0x83},
		2, {0x83, 0x62},
		2, {0xD9, 0xB7}
	},
	{
		2, {0x9D, 0x57},
		3, {0xE6, 0x8A, 0x94},
		2, {0x94, 0x62},
		2, {0xD9, 0xB8}
	},
	{
		2, {0x9D, 0x58},
		3, {0xE6, 0x8B, 0x97},
		2, {0xD7, 0x62},
		2, {0xD9, 0xB9}
	},
	{
		2, {0x9D, 0x59},
		3, {0xE6, 0x8B, 0x91},
		2, {0xD1, 0x62},
		2, {0xD9, 0xBA}
	},
	{
		2, {0x9D, 0x5A},
		3, {0xE6, 0x8A, 0xBB},
		2, {0xBB, 0x62},
		2, {0xD9, 0xBB}
	},
	{
		2, {0x9D, 0x5B},
		3, {0xE6, 0x8B, 0x8F},
		2, {0xCF, 0x62},
		2, {0xD9, 0xBC}
	},
	{
		2, {0x9D, 0x5C},
		3, {0xE6, 0x8B, 0xBF},
		2, {0xFF, 0x62},
		2, {0xD9, 0xBD}
	},
	{
		2, {0x9D, 0x5D},
		3, {0xE6, 0x8B, 0x86},
		2, {0xC6, 0x62},
		2, {0xD9, 0xBE}
	},
	{
		2, {0x9D, 0x5E},
		3, {0xE6, 0x93, 0x94},
		2, {0xD4, 0x64},
		2, {0xD9, 0xBF}
	},
	{
		2, {0x9D, 0x5F},
		3, {0xE6, 0x8B, 0x88},
		2, {0xC8, 0x62},
		2, {0xD9, 0xC0}
	},
	{
		2, {0x9D, 0x60},
		3, {0xE6, 0x8B, 0x9C},
		2, {0xDC, 0x62},
		2, {0xD9, 0xC1}
	},
	{
		2, {0x9D, 0x61},
		3, {0xE6, 0x8B, 0x8C},
		2, {0xCC, 0x62},
		2, {0xD9, 0xC2}
	},
	{
		2, {0x9D, 0x62},
		3, {0xE6, 0x8B, 0x8A},
		2, {0xCA, 0x62},
		2, {0xD9, 0xC3}
	},
	{
		2, {0x9D, 0x63},
		3, {0xE6, 0x8B, 0x82},
		2, {0xC2, 0x62},
		2, {0xD9, 0xC4}
	},
	{
		2, {0x9D, 0x64},
		3, {0xE6, 0x8B, 0x87},
		2, {0xC7, 0x62},
		2, {0xD9, 0xC5}
	},
	{
		2, {0x9D, 0x65},
		3, {0xE6, 0x8A, 0x9B},
		2, {0x9B, 0x62},
		2, {0xD9, 0xC6}
	},
	{
		2, {0x9D, 0x66},
		3, {0xE6, 0x8B, 0x89},
		2, {0xC9, 0x62},
		2, {0xD9, 0xC7}
	},
	{
		2, {0x9D, 0x67},
		3, {0xE6, 0x8C, 0x8C},
		2, {0x0C, 0x63},
		2, {0xD9, 0xC8}
	},
	{
		2, {0x9D, 0x68},
		3, {0xE6, 0x8B, 0xAE},
		2, {0xEE, 0x62},
		2, {0xD9, 0xC9}
	},
	{
		2, {0x9D, 0x69},
		3, {0xE6, 0x8B, 0xB1},
		2, {0xF1, 0x62},
		2, {0xD9, 0xCA}
	},
	{
		2, {0x9D, 0x6A},
		3, {0xE6, 0x8C, 0xA7},
		2, {0x27, 0x63},
		2, {0xD9, 0xCB}
	},
	{
		2, {0x9D, 0x6B},
		3, {0xE6, 0x8C, 0x82},
		2, {0x02, 0x63},
		2, {0xD9, 0xCC}
	},
	{
		2, {0x9D, 0x6C},
		3, {0xE6, 0x8C, 0x88},
		2, {0x08, 0x63},
		2, {0xD9, 0xCD}
	},
	{
		2, {0x9D, 0x6D},
		3, {0xE6, 0x8B, 0xAF},
		2, {0xEF, 0x62},
		2, {0xD9, 0xCE}
	},
	{
		2, {0x9D, 0x6E},
		3, {0xE6, 0x8B, 0xB5},
		2, {0xF5, 0x62},
		2, {0xD9, 0xCF}
	},
	{
		2, {0x9D, 0x6F},
		3, {0xE6, 0x8D, 0x90},
		2, {0x50, 0x63},
		2, {0xD9, 0xD0}
	},
	{
		2, {0x9D, 0x70},
		3, {0xE6, 0x8C, 0xBE},
		2, {0x3E, 0x63},
		2, {0xD9, 0xD1}
	},
	{
		2, {0x9D, 0x71},
		3, {0xE6, 0x8D, 0x8D},
		2, {0x4D, 0x63},
		2, {0xD9, 0xD2}
	},
	{
		2, {0x9D, 0x72},
		3, {0xE6, 0x90, 0x9C},
		2, {0x1C, 0x64},
		2, {0xD9, 0xD3}
	},
	{
		2, {0x9D, 0x73},
		3, {0xE6, 0x8D, 0x8F},
		2, {0x4F, 0x63},
		2, {0xD9, 0xD4}
	},
	{
		2, {0x9D, 0x74},
		3, {0xE6, 0x8E, 0x96},
		2, {0x96, 0x63},
		2, {0xD9, 0xD5}
	},
	{
		2, {0x9D, 0x75},
		3, {0xE6, 0x8E, 0x8E},
		2, {0x8E, 0x63},
		2, {0xD9, 0xD6}
	},
	{
		2, {0x9D, 0x76},
		3, {0xE6, 0x8E, 0x80},
		2, {0x80, 0x63},
		2, {0xD9, 0xD7}
	},
	{
		2, {0x9D, 0x77},
		3, {0xE6, 0x8E, 0xAB},
		2, {0xAB, 0x63},
		2, {0xD9, 0xD8}
	},
	{
		2, {0x9D, 0x78},
		3, {0xE6, 0x8D, 0xB6},
		2, {0x76, 0x63},
		2, {0xD9, 0xD9}
	},
	{
		2, {0x9D, 0x79},
		3, {0xE6, 0x8E, 0xA3},
		2, {0xA3, 0x63},
		2, {0xD9, 0xDA}
	},
	{
		2, {0x9D, 0x7A},
		3, {0xE6, 0x8E, 0x8F},
		2, {0x8F, 0x63},
		2, {0xD9, 0xDB}
	},
	{
		2, {0x9D, 0x7B},
		3, {0xE6, 0x8E, 0x89},
		2, {0x89, 0x63},
		2, {0xD9, 0xDC}
	},
	{
		2, {0x9D, 0x7C},
		3, {0xE6, 0x8E, 0x9F},
		2, {0x9F, 0x63},
		2, {0xD9, 0xDD}
	},
	{
		2, {0x9D, 0x7D},
		3, {0xE6, 0x8E, 0xB5},
		2, {0xB5, 0x63},
		2, {0xD9, 0xDE}
	},
	{
		2, {0x9D, 0x7E},
		3, {0xE6, 0x8D, 0xAB},
		2, {0x6B, 0x63},
		2, {0xD9, 0xDF}
	},
	{
		2, {0x9D, 0x80},
		3, {0xE6, 0x8D, 0xA9},
		2, {0x69, 0x63},
		2, {0xD9, 0xE0}
	},
	{
		2, {0x9D, 0x81},
		3, {0xE6, 0x8E, 0xBE},
		2, {0xBE, 0x63},
		2, {0xD9, 0xE1}
	},
	{
		2, {0x9D, 0x82},
		3, {0xE6, 0x8F, 0xA9},
		2, {0xE9, 0x63},
		2, {0xD9, 0xE2}
	},
	{
		2, {0x9D, 0x83},
		3, {0xE6, 0x8F, 0x80},
		2, {0xC0, 0x63},
		2, {0xD9, 0xE3}
	},
	{
		2, {0x9D, 0x84},
		3, {0xE6, 0x8F, 0x86},
		2, {0xC6, 0x63},
		2, {0xD9, 0xE4}
	},
	{
		2, {0x9D, 0x85},
		3, {0xE6, 0x8F, 0xA3},
		2, {0xE3, 0x63},
		2, {0xD9, 0xE5}
	},
	{
		2, {0x9D, 0x86},
		3, {0xE6, 0x8F, 0x89},
		2, {0xC9, 0x63},
		2, {0xD9, 0xE6}
	},
	{
		2, {0x9D, 0x87},
		3, {0xE6, 0x8F, 0x92},
		2, {0xD2, 0x63},
		2, {0xD9, 0xE7}
	},
	{
		2, {0x9D, 0x88},
		3, {0xE6, 0x8F, 0xB6},
		2, {0xF6, 0x63},
		2, {0xD9, 0xE8}
	},
	{
		2, {0x9D, 0x89},
		3, {0xE6, 0x8F, 0x84},
		2, {0xC4, 0x63},
		2, {0xD9, 0xE9}
	},
	{
		2, {0x9D, 0x8A},
		3, {0xE6, 0x90, 0x96},
		2, {0x16, 0x64},
		2, {0xD9, 0xEA}
	},
	{
		2, {0x9D, 0x8B},
		3, {0xE6, 0x90, 0xB4},
		2, {0x34, 0x64},
		2, {0xD9, 0xEB}
	},
	{
		2, {0x9D, 0x8C},
		3, {0xE6, 0x90, 0x86},
		2, {0x06, 0x64},
		2, {0xD9, 0xEC}
	},
	{
		2, {0x9D, 0x8D},
		3, {0xE6, 0x90, 0x93},
		2, {0x13, 0x64},
		2, {0xD9, 0xED}
	},
	{
		2, {0x9D, 0x8E},
		3, {0xE6, 0x90, 0xA6},
		2, {0x26, 0x64},
		2, {0xD9, 0xEE}
	},
	{
		2, {0x9D, 0x8F},
		3, {0xE6, 0x90, 0xB6},
		2, {0x36, 0x64},
		2, {0xD9, 0xEF}
	},
	{
		2, {0x9D, 0x90},
		3, {0xE6, 0x94, 0x9D},
		2, {0x1D, 0x65},
		2, {0xD9, 0xF0}
	},
	{
		2, {0x9D, 0x91},
		3, {0xE6, 0x90, 0x97},
		2, {0x17, 0x64},
		2, {0xD9, 0xF1}
	},
	{
		2, {0x9D, 0x92},
		3, {0xE6, 0x90, 0xA8},
		2, {0x28, 0x64},
		2, {0xD9, 0xF2}
	},
	{
		2, {0x9D, 0x93},
		3, {0xE6, 0x90, 0x8F},
		2, {0x0F, 0x64},
		2, {0xD9, 0xF3}
	},
	{
		2, {0x9D, 0x94},
		3, {0xE6, 0x91, 0xA7},
		2, {0x67, 0x64},
		2, {0xD9, 0xF4}
	},
	{
		2, {0x9D, 0x95},
		3, {0xE6, 0x91, 0xAF},
		2, {0x6F, 0x64},
		2, {0xD9, 0xF5}
	},
	{
		2, {0x9D, 0x96},
		3, {0xE6, 0x91, 0xB6},
		2, {0x76, 0x64},
		2, {0xD9, 0xF6}
	},
	{
		2, {0x9D, 0x97},
		3, {0xE6, 0x91, 0x8E},
		2, {0x4E, 0x64},
		2, {0xD9, 0xF7}
	},
	{
		2, {0x9D, 0x98},
		3, {0xE6, 0x94, 0xAA},
		2, {0x2A, 0x65},
		2, {0xD9, 0xF8}
	},
	{
		2, {0x9D, 0x99},
		3, {0xE6, 0x92, 0x95},
		2, {0x95, 0x64},
		2, {0xD9, 0xF9}
	},
	{
		2, {0x9D, 0x9A},
		3, {0xE6, 0x92, 0x93},
		2, {0x93, 0x64},
		2, {0xD9, 0xFA}
	},
	{
		2, {0x9D, 0x9B},
		3, {0xE6, 0x92, 0xA5},
		2, {0xA5, 0x64},
		2, {0xD9, 0xFB}
	},
	{
		2, {0x9D, 0x9C},
		3, {0xE6, 0x92, 0xA9},
		2, {0xA9, 0x64},
		2, {0xD9, 0xFC}
	},
	{
		2, {0x9D, 0x9D},
		3, {0xE6, 0x92, 0x88},
		2, {0x88, 0x64},
		2, {0xD9, 0xFD}
	},
	{
		2, {0x9D, 0x9E},
		3, {0xE6, 0x92, 0xBC},
		2, {0xBC, 0x64},
		2, {0xD9, 0xFE}
	},
	{
		2, {0x9D, 0x9F},
		3, {0xE6, 0x93, 0x9A},
		2, {0xDA, 0x64},
		2, {0xDA, 0xA1}
	},
	{
		2, {0x9D, 0xA0},
		3, {0xE6, 0x93, 0x92},
		2, {0xD2, 0x64},
		2, {0xDA, 0xA2}
	},
	{
		2, {0x9D, 0xA1},
		3, {0xE6, 0x93, 0x85},
		2, {0xC5, 0x64},
		2, {0xDA, 0xA3}
	},
	{
		2, {0x9D, 0xA2},
		3, {0xE6, 0x93, 0x87},
		2, {0xC7, 0x64},
		2, {0xDA, 0xA4}
	},
	{
		2, {0x9D, 0xA3},
		3, {0xE6, 0x92, 0xBB},
		2, {0xBB, 0x64},
		2, {0xDA, 0xA5}
	},
	{
		2, {0x9D, 0xA4},
		3, {0xE6, 0x93, 0x98},
		2, {0xD8, 0x64},
		2, {0xDA, 0xA6}
	},
	{
		2, {0x9D, 0xA5},
		3, {0xE6, 0x93, 0x82},
		2, {0xC2, 0x64},
		2, {0xDA, 0xA7}
	},
	{
		2, {0x9D, 0xA6},
		3, {0xE6, 0x93, 0xB1},
		2, {0xF1, 0x64},
		2, {0xDA, 0xA8}
	},
	{
		2, {0x9D, 0xA7},
		3, {0xE6, 0x93, 0xA7},
		2, {0xE7, 0x64},
		2, {0xDA, 0xA9}
	},
	{
		2, {0x9D, 0xA8},
		3, {0xE8, 0x88, 0x89},
		2, {0x09, 0x82},
		2, {0xDA, 0xAA}
	},
	{
		2, {0x9D, 0xA9},
		3, {0xE6, 0x93, 0xA0},
		2, {0xE0, 0x64},
		2, {0xDA, 0xAB}
	},
	{
		2, {0x9D, 0xAA},
		3, {0xE6, 0x93, 0xA1},
		2, {0xE1, 0x64},
		2, {0xDA, 0xAC}
	},
	{
		2, {0x9D, 0xAB},
		3, {0xE6, 0x8A, 0xAC},
		2, {0xAC, 0x62},
		2, {0xDA, 0xAD}
	},
	{
		2, {0x9D, 0xAC},
		3, {0xE6, 0x93, 0xA3},
		2, {0xE3, 0x64},
		2, {0xDA, 0xAE}
	},
	{
		2, {0x9D, 0xAD},
		3, {0xE6, 0x93, 0xAF},
		2, {0xEF, 0x64},
		2, {0xDA, 0xAF}
	},
	{
		2, {0x9D, 0xAE},
		3, {0xE6, 0x94, 0xAC},
		2, {0x2C, 0x65},
		2, {0xDA, 0xB0}
	},
	{
		2, {0x9D, 0xAF},
		3, {0xE6, 0x93, 0xB6},
		2, {0xF6, 0x64},
		2, {0xDA, 0xB1}
	},
	{
		2, {0x9D, 0xB0},
		3, {0xE6, 0x93, 0xB4},
		2, {0xF4, 0x64},
		2, {0xDA, 0xB2}
	},
	{
		2, {0x9D, 0xB1},
		3, {0xE6, 0x93, 0xB2},
		2, {0xF2, 0x64},
		2, {0xDA, 0xB3}
	},
	{
		2, {0x9D, 0xB2},
		3, {0xE6, 0x93, 0xBA},
		2, {0xFA, 0x64},
		2, {0xDA, 0xB4}
	},
	{
		2, {0x9D, 0xB3},
		3, {0xE6, 0x94, 0x80},
		2, {0x00, 0x65},
		2, {0xDA, 0xB5}
	},
	{
		2, {0x9D, 0xB4},
		3, {0xE6, 0x93, 0xBD},
		2, {0xFD, 0x64},
		2, {0xDA, 0xB6}
	},
	{
		2, {0x9D, 0xB5},
		3, {0xE6, 0x94, 0x98},
		2, {0x18, 0x65},
		2, {0xDA, 0xB7}
	},
	{
		2, {0x9D, 0xB6},
		3, {0xE6, 0x94, 0x9C},
		2, {0x1C, 0x65},
		2, {0xDA, 0xB8}
	},
	{
		2, {0x9D, 0xB7},
		3, {0xE6, 0x94, 0x85},
		2, {0x05, 0x65},
		2, {0xDA, 0xB9}
	},
	{
		2, {0x9D, 0xB8},
		3, {0xE6, 0x94, 0xA4},
		2, {0x24, 0x65},
		2, {0xDA, 0xBA}
	},
	{
		2, {0x9D, 0xB9},
		3, {0xE6, 0x94, 0xA3},
		2, {0x23, 0x65},
		2, {0xDA, 0xBB}
	},
	{
		2, {0x9D, 0xBA},
		3, {0xE6, 0x94, 0xAB},
		2, {0x2B, 0x65},
		2, {0xDA, 0xBC}
	},
	{
		2, {0x9D, 0xBB},
		3, {0xE6, 0x94, 0xB4},
		2, {0x34, 0x65},
		2, {0xDA, 0xBD}
	},
	{
		2, {0x9D, 0xBC},
		3, {0xE6, 0x94, 0xB5},
		2, {0x35, 0x65},
		2, {0xDA, 0xBE}
	},
	{
		2, {0x9D, 0xBD},
		3, {0xE6, 0x94, 0xB7},
		2, {0x37, 0x65},
		2, {0xDA, 0xBF}
	},
	{
		2, {0x9D, 0xBE},
		3, {0xE6, 0x94, 0xB6},
		2, {0x36, 0x65},
		2, {0xDA, 0xC0}
	},
	{
		2, {0x9D, 0xBF},
		3, {0xE6, 0x94, 0xB8},
		2, {0x38, 0x65},
		2, {0xDA, 0xC1}
	},
	{
		2, {0x9D, 0xC0},
		3, {0xE7, 0x95, 0x8B},
		2, {0x4B, 0x75},
		2, {0xDA, 0xC2}
	},
	{
		2, {0x9D, 0xC1},
		3, {0xE6, 0x95, 0x88},
		2, {0x48, 0x65},
		2, {0xDA, 0xC3}
	},
	{
		2, {0x9D, 0xC2},
		3, {0xE6, 0x95, 0x96},
		2, {0x56, 0x65},
		2, {0xDA, 0xC4}
	},
	{
		2, {0x9D, 0xC3},
		3, {0xE6, 0x95, 0x95},
		2, {0x55, 0x65},
		2, {0xDA, 0xC5}
	},
	{
		2, {0x9D, 0xC4},
		3, {0xE6, 0x95, 0x8D},
		2, {0x4D, 0x65},
		2, {0xDA, 0xC6}
	},
	{
		2, {0x9D, 0xC5},
		3, {0xE6, 0x95, 0x98},
		2, {0x58, 0x65},
		2, {0xDA, 0xC7}
	},
	{
		2, {0x9D, 0xC6},
		3, {0xE6, 0x95, 0x9E},
		2, {0x5E, 0x65},
		2, {0xDA, 0xC8}
	},
	{
		2, {0x9D, 0xC7},
		3, {0xE6, 0x95, 0x9D},
		2, {0x5D, 0x65},
		2, {0xDA, 0xC9}
	},
	{
		2, {0x9D, 0xC8},
		3, {0xE6, 0x95, 0xB2},
		2, {0x72, 0x65},
		2, {0xDA, 0xCA}
	},
	{
		2, {0x9D, 0xC9},
		3, {0xE6, 0x95, 0xB8},
		2, {0x78, 0x65},
		2, {0xDA, 0xCB}
	},
	{
		2, {0x9D, 0xCA},
		3, {0xE6, 0x96, 0x82},
		2, {0x82, 0x65},
		2, {0xDA, 0xCC}
	},
	{
		2, {0x9D, 0xCB},
		3, {0xE6, 0x96, 0x83},
		2, {0x83, 0x65},
		2, {0xDA, 0xCD}
	},
	{
		2, {0x9D, 0xCC},
		3, {0xE8, 0xAE, 0x8A},
		2, {0x8A, 0x8B},
		2, {0xDA, 0xCE}
	},
	{
		2, {0x9D, 0xCD},
		3, {0xE6, 0x96, 0x9B},
		2, {0x9B, 0x65},
		2, {0xDA, 0xCF}
	},
	{
		2, {0x9D, 0xCE},
		3, {0xE6, 0x96, 0x9F},
		2, {0x9F, 0x65},
		2, {0xDA, 0xD0}
	},
	{
		2, {0x9D, 0xCF},
		3, {0xE6, 0x96, 0xAB},
		2, {0xAB, 0x65},
		2, {0xDA, 0xD1}
	},
	{
		2, {0x9D, 0xD0},
		3, {0xE6, 0x96, 0xB7},
		2, {0xB7, 0x65},
		2, {0xDA, 0xD2}
	},
	{
		2, {0x9D, 0xD1},
		3, {0xE6, 0x97, 0x83},
		2, {0xC3, 0x65},
		2, {0xDA, 0xD3}
	},
	{
		2, {0x9D, 0xD2},
		3, {0xE6, 0x97, 0x86},
		2, {0xC6, 0x65},
		2, {0xDA, 0xD4}
	},
	{
		2, {0x9D, 0xD3},
		3, {0xE6, 0x97, 0x81},
		2, {0xC1, 0x65},
		2, {0xDA, 0xD5}
	},
	{
		2, {0x9D, 0xD4},
		3, {0xE6, 0x97, 0x84},
		2, {0xC4, 0x65},
		2, {0xDA, 0xD6}
	},
	{
		2, {0x9D, 0xD5},
		3, {0xE6, 0x97, 0x8C},
		2, {0xCC, 0x65},
		2, {0xDA, 0xD7}
	},
	{
		2, {0x9D, 0xD6},
		3, {0xE6, 0x97, 0x92},
		2, {0xD2, 0x65},
		2, {0xDA, 0xD8}
	},
	{
		2, {0x9D, 0xD7},
		3, {0xE6, 0x97, 0x9B},
		2, {0xDB, 0x65},
		2, {0xDA, 0xD9}
	},
	{
		2, {0x9D, 0xD8},
		3, {0xE6, 0x97, 0x99},
		2, {0xD9, 0x65},
		2, {0xDA, 0xDA}
	},
	{
		2, {0x9D, 0xD9},
		3, {0xE6, 0x97, 0xA0},
		2, {0xE0, 0x65},
		2, {0xDA, 0xDB}
	},
	{
		2, {0x9D, 0xDA},
		3, {0xE6, 0x97, 0xA1},
		2, {0xE1, 0x65},
		2, {0xDA, 0xDC}
	},
	{
		2, {0x9D, 0xDB},
		3, {0xE6, 0x97, 0xB1},
		2, {0xF1, 0x65},
		2, {0xDA, 0xDD}
	},
	{
		2, {0x9D, 0xDC},
		3, {0xE6, 0x9D, 0xB2},
		2, {0x72, 0x67},
		2, {0xDA, 0xDE}
	},
	{
		2, {0x9D, 0xDD},
		3, {0xE6, 0x98, 0x8A},
		2, {0x0A, 0x66},
		2, {0xDA, 0xDF}
	},
	{
		2, {0x9D, 0xDE},
		3, {0xE6, 0x98, 0x83},
		2, {0x03, 0x66},
		2, {0xDA, 0xE0}
	},
	{
		2, {0x9D, 0xDF},
		3, {0xE6, 0x97, 0xBB},
		2, {0xFB, 0x65},
		2, {0xDA, 0xE1}
	},
	{
		2, {0x9D, 0xE0},
		3, {0xE6, 0x9D, 0xB3},
		2, {0x73, 0x67},
		2, {0xDA, 0xE2}
	},
	{
		2, {0x9D, 0xE1},
		3, {0xE6, 0x98, 0xB5},
		2, {0x35, 0x66},
		2, {0xDA, 0xE3}
	},
	{
		2, {0x9D, 0xE2},
		3, {0xE6, 0x98, 0xB6},
		2, {0x36, 0x66},
		2, {0xDA, 0xE4}
	},
	{
		2, {0x9D, 0xE3},
		3, {0xE6, 0x98, 0xB4},
		2, {0x34, 0x66},
		2, {0xDA, 0xE5}
	},
	{
		2, {0x9D, 0xE4},
		3, {0xE6, 0x98, 0x9C},
		2, {0x1C, 0x66},
		2, {0xDA, 0xE6}
	},
	{
		2, {0x9D, 0xE5},
		3, {0xE6, 0x99, 0x8F},
		2, {0x4F, 0x66},
		2, {0xDA, 0xE7}
	},
	{
		2, {0x9D, 0xE6},
		3, {0xE6, 0x99, 0x84},
		2, {0x44, 0x66},
		2, {0xDA, 0xE8}
	},
	{
		2, {0x9D, 0xE7},
		3, {0xE6, 0x99, 0x89},
		2, {0x49, 0x66},
		2, {0xDA, 0xE9}
	},
	{
		2, {0x9D, 0xE8},
		3, {0xE6, 0x99, 0x81},
		2, {0x41, 0x66},
		2, {0xDA, 0xEA}
	},
	{
		2, {0x9D, 0xE9},
		3, {0xE6, 0x99, 0x9E},
		2, {0x5E, 0x66},
		2, {0xDA, 0xEB}
	},
	{
		2, {0x9D, 0xEA},
		3, {0xE6, 0x99, 0x9D},
		2, {0x5D, 0x66},
		2, {0xDA, 0xEC}
	},
	{
		2, {0x9D, 0xEB},
		3, {0xE6, 0x99, 0xA4},
		2, {0x64, 0x66},
		2, {0xDA, 0xED}
	},
	{
		2, {0x9D, 0xEC},
		3, {0xE6, 0x99, 0xA7},
		2, {0x67, 0x66},
		2, {0xDA, 0xEE}
	},
	{
		2, {0x9D, 0xED},
		3, {0xE6, 0x99, 0xA8},
		2, {0x68, 0x66},
		2, {0xDA, 0xEF}
	},
	{
		2, {0x9D, 0xEE},
		3, {0xE6, 0x99, 0x9F},
		2, {0x5F, 0x66},
		2, {0xDA, 0xF0}
	},
	{
		2, {0x9D, 0xEF},
		3, {0xE6, 0x99, 0xA2},
		2, {0x62, 0x66},
		2, {0xDA, 0xF1}
	},
	{
		2, {0x9D, 0xF0},
		3, {0xE6, 0x99, 0xB0},
		2, {0x70, 0x66},
		2, {0xDA, 0xF2}
	},
	{
		2, {0x9D, 0xF1},
		3, {0xE6, 0x9A, 0x83},
		2, {0x83, 0x66},
		2, {0xDA, 0xF3}
	},
	{
		2, {0x9D, 0xF2},
		3, {0xE6, 0x9A, 0x88},
		2, {0x88, 0x66},
		2, {0xDA, 0xF4}
	},
	{
		2, {0x9D, 0xF3},
		3, {0xE6, 0x9A, 0x8E},
		2, {0x8E, 0x66},
		2, {0xDA, 0xF5}
	},
	{
		2, {0x9D, 0xF4},
		3, {0xE6, 0x9A, 0x89},
		2, {0x89, 0x66},
		2, {0xDA, 0xF6}
	},
	{
		2, {0x9D, 0xF5},
		3, {0xE6, 0x9A, 0x84},
		2, {0x84, 0x66},
		2, {0xDA, 0xF7}
	},
	{
		2, {0x9D, 0xF6},
		3, {0xE6, 0x9A, 0x98},
		2, {0x98, 0x66},
		2, {0xDA, 0xF8}
	},
	{
		2, {0x9D, 0xF7},
		3, {0xE6, 0x9A, 0x9D},
		2, {0x9D, 0x66},
		2, {0xDA, 0xF9}
	},
	{
		2, {0x9D, 0xF8},
		3, {0xE6, 0x9B, 0x81},
		2, {0xC1, 0x66},
		2, {0xDA, 0xFA}
	},
	{
		2, {0x9D, 0xF9},
		3, {0xE6, 0x9A, 0xB9},
		2, {0xB9, 0x66},
		2, {0xDA, 0xFB}
	},
	{
		2, {0x9D, 0xFA},
		3, {0xE6, 0x9B, 0x89},
		2, {0xC9, 0x66},
		2, {0xDA, 0xFC}
	},
	{
		2, {0x9D, 0xFB},
		3, {0xE6, 0x9A, 0xBE},
		2, {0xBE, 0x66},
		2, {0xDA, 0xFD}
	},
	{
		2, {0x9D, 0xFC},
		3, {0xE6, 0x9A, 0xBC},
		2, {0xBC, 0x66},
		2, {0xDA, 0xFE}
	},
	{
		2, {0x9E, 0x40},
		3, {0xE6, 0x9B, 0x84},
		2, {0xC4, 0x66},
		2, {0xDB, 0xA1}
	},
	{
		2, {0x9E, 0x41},
		3, {0xE6, 0x9A, 0xB8},
		2, {0xB8, 0x66},
		2, {0xDB, 0xA2}
	},
	{
		2, {0x9E, 0x42},
		3, {0xE6, 0x9B, 0x96},
		2, {0xD6, 0x66},
		2, {0xDB, 0xA3}
	},
	{
		2, {0x9E, 0x43},
		3, {0xE6, 0x9B, 0x9A},
		2, {0xDA, 0x66},
		2, {0xDB, 0xA4}
	},
	{
		2, {0x9E, 0x44},
		3, {0xE6, 0x9B, 0xA0},
		2, {0xE0, 0x66},
		2, {0xDB, 0xA5}
	},
	{
		2, {0x9E, 0x45},
		3, {0xE6, 0x98, 0xBF},
		2, {0x3F, 0x66},
		2, {0xDB, 0xA6}
	},
	{
		2, {0x9E, 0x46},
		3, {0xE6, 0x9B, 0xA6},
		2, {0xE6, 0x66},
		2, {0xDB, 0xA7}
	},
	{
		2, {0x9E, 0x47},
		3, {0xE6, 0x9B, 0xA9},
		2, {0xE9, 0x66},
		2, {0xDB, 0xA8}
	},
	{
		2, {0x9E, 0x48},
		3, {0xE6, 0x9B, 0xB0},
		2, {0xF0, 0x66},
		2, {0xDB, 0xA9}
	},
	{
		2, {0x9E, 0x49},
		3, {0xE6, 0x9B, 0xB5},
		2, {0xF5, 0x66},
		2, {0xDB, 0xAA}
	},
	{
		2, {0x9E, 0x4A},
		3, {0xE6, 0x9B, 0xB7},
		2, {0xF7, 0x66},
		2, {0xDB, 0xAB}
	},
	{
		2, {0x9E, 0x4B},
		3, {0xE6, 0x9C, 0x8F},
		2, {0x0F, 0x67},
		2, {0xDB, 0xAC}
	},
	{
		2, {0x9E, 0x4C},
		3, {0xE6, 0x9C, 0x96},
		2, {0x16, 0x67},
		2, {0xDB, 0xAD}
	},
	{
		2, {0x9E, 0x4D},
		3, {0xE6, 0x9C, 0x9E},
		2, {0x1E, 0x67},
		2, {0xDB, 0xAE}
	},
	{
		2, {0x9E, 0x4E},
		3, {0xE6, 0x9C, 0xA6},
		2, {0x26, 0x67},
		2, {0xDB, 0xAF}
	},
	{
		2, {0x9E, 0x4F},
		3, {0xE6, 0x9C, 0xA7},
		2, {0x27, 0x67},
		2, {0xDB, 0xB0}
	},
	{
		2, {0x9E, 0x50},
		3, {0xE9, 0x9C, 0xB8},
		2, {0x38, 0x97},
		2, {0xDB, 0xB1}
	},
	{
		2, {0x9E, 0x51},
		3, {0xE6, 0x9C, 0xAE},
		2, {0x2E, 0x67},
		2, {0xDB, 0xB2}
	},
	{
		2, {0x9E, 0x52},
		3, {0xE6, 0x9C, 0xBF},
		2, {0x3F, 0x67},
		2, {0xDB, 0xB3}
	},
	{
		2, {0x9E, 0x53},
		3, {0xE6, 0x9C, 0xB6},
		2, {0x36, 0x67},
		2, {0xDB, 0xB4}
	},
	{
		2, {0x9E, 0x54},
		3, {0xE6, 0x9D, 0x81},
		2, {0x41, 0x67},
		2, {0xDB, 0xB5}
	},
	{
		2, {0x9E, 0x55},
		3, {0xE6, 0x9C, 0xB8},
		2, {0x38, 0x67},
		2, {0xDB, 0xB6}
	},
	{
		2, {0x9E, 0x56},
		3, {0xE6, 0x9C, 0xB7},
		2, {0x37, 0x67},
		2, {0xDB, 0xB7}
	},
	{
		2, {0x9E, 0x57},
		3, {0xE6, 0x9D, 0x86},
		2, {0x46, 0x67},
		2, {0xDB, 0xB8}
	},
	{
		2, {0x9E, 0x58},
		3, {0xE6, 0x9D, 0x9E},
		2, {0x5E, 0x67},
		2, {0xDB, 0xB9}
	},
	{
		2, {0x9E, 0x59},
		3, {0xE6, 0x9D, 0xA0},
		2, {0x60, 0x67},
		2, {0xDB, 0xBA}
	},
	{
		2, {0x9E, 0x5A},
		3, {0xE6, 0x9D, 0x99},
		2, {0x59, 0x67},
		2, {0xDB, 0xBB}
	},
	{
		2, {0x9E, 0x5B},
		3, {0xE6, 0x9D, 0xA3},
		2, {0x63, 0x67},
		2, {0xDB, 0xBC}
	},
	{
		2, {0x9E, 0x5C},
		3, {0xE6, 0x9D, 0xA4},
		2, {0x64, 0x67},
		2, {0xDB, 0xBD}
	},
	{
		2, {0x9E, 0x5D},
		3, {0xE6, 0x9E, 0x89},
		2, {0x89, 0x67},
		2, {0xDB, 0xBE}
	},
	{
		2, {0x9E, 0x5E},
		3, {0xE6, 0x9D, 0xB0},
		2, {0x70, 0x67},
		2, {0xDB, 0xBF}
	},
	{
		2, {0x9E, 0x5F},
		3, {0xE6, 0x9E, 0xA9},
		2, {0xA9, 0x67},
		2, {0xDB, 0xC0}
	},
	{
		2, {0x9E, 0x60},
		3, {0xE6, 0x9D, 0xBC},
		2, {0x7C, 0x67},
		2, {0xDB, 0xC1}
	},
	{
		2, {0x9E, 0x61},
		3, {0xE6, 0x9D, 0xAA},
		2, {0x6A, 0x67},
		2, {0xDB, 0xC2}
	},
	{
		2, {0x9E, 0x62},
		3, {0xE6, 0x9E, 0x8C},
		2, {0x8C, 0x67},
		2, {0xDB, 0xC3}
	},
	{
		2, {0x9E, 0x63},
		3, {0xE6, 0x9E, 0x8B},
		2, {0x8B, 0x67},
		2, {0xDB, 0xC4}
	},
	{
		2, {0x9E, 0x64},
		3, {0xE6, 0x9E, 0xA6},
		2, {0xA6, 0x67},
		2, {0xDB, 0xC5}
	},
	{
		2, {0x9E, 0x65},
		3, {0xE6, 0x9E, 0xA1},
		2, {0xA1, 0x67},
		2, {0xDB, 0xC6}
	},
	{
		2, {0x9E, 0x66},
		3, {0xE6, 0x9E, 0x85},
		2, {0x85, 0x67},
		2, {0xDB, 0xC7}
	},
	{
		2, {0x9E, 0x67},
		3, {0xE6, 0x9E, 0xB7},
		2, {0xB7, 0x67},
		2, {0xDB, 0xC8}
	},
	{
		2, {0x9E, 0x68},
		3, {0xE6, 0x9F, 0xAF},
		2, {0xEF, 0x67},
		2, {0xDB, 0xC9}
	},
	{
		2, {0x9E, 0x69},
		3, {0xE6, 0x9E, 0xB4},
		2, {0xB4, 0x67},
		2, {0xDB, 0xCA}
	},
	{
		2, {0x9E, 0x6A},
		3, {0xE6, 0x9F, 0xAC},
		2, {0xEC, 0x67},
		2, {0xDB, 0xCB}
	},
	{
		2, {0x9E, 0x6B},
		3, {0xE6, 0x9E, 0xB3},
		2, {0xB3, 0x67},
		2, {0xDB, 0xCC}
	},
	{
		2, {0x9E, 0x6C},
		3, {0xE6, 0x9F, 0xA9},
		2, {0xE9, 0x67},
		2, {0xDB, 0xCD}
	},
	{
		2, {0x9E, 0x6D},
		3, {0xE6, 0x9E, 0xB8},
		2, {0xB8, 0x67},
		2, {0xDB, 0xCE}
	},
	{
		2, {0x9E, 0x6E},
		3, {0xE6, 0x9F, 0xA4},
		2, {0xE4, 0x67},
		2, {0xDB, 0xCF}
	},
	{
		2, {0x9E, 0x6F},
		3, {0xE6, 0x9F, 0x9E},
		2, {0xDE, 0x67},
		2, {0xDB, 0xD0}
	},
	{
		2, {0x9E, 0x70},
		3, {0xE6, 0x9F, 0x9D},
		2, {0xDD, 0x67},
		2, {0xDB, 0xD1}
	},
	{
		2, {0x9E, 0x71},
		3, {0xE6, 0x9F, 0xA2},
		2, {0xE2, 0x67},
		2, {0xDB, 0xD2}
	},
	{
		2, {0x9E, 0x72},
		3, {0xE6, 0x9F, 0xAE},
		2, {0xEE, 0x67},
		2, {0xDB, 0xD3}
	},
	{
		2, {0x9E, 0x73},
		3, {0xE6, 0x9E, 0xB9},
		2, {0xB9, 0x67},
		2, {0xDB, 0xD4}
	},
	{
		2, {0x9E, 0x74},
		3, {0xE6, 0x9F, 0x8E},
		2, {0xCE, 0x67},
		2, {0xDB, 0xD5}
	},
	{
		2, {0x9E, 0x75},
		3, {0xE6, 0x9F, 0x86},
		2, {0xC6, 0x67},
		2, {0xDB, 0xD6}
	},
	{
		2, {0x9E, 0x76},
		3, {0xE6, 0x9F, 0xA7},
		2, {0xE7, 0x67},
		2, {0xDB, 0xD7}
	},
	{
		2, {0x9E, 0x77},
		3, {0xE6, 0xAA, 0x9C},
		2, {0x9C, 0x6A},
		2, {0xDB, 0xD8}
	},
	{
		2, {0x9E, 0x78},
		3, {0xE6, 0xA0, 0x9E},
		2, {0x1E, 0x68},
		2, {0xDB, 0xD9}
	},
	{
		2, {0x9E, 0x79},
		3, {0xE6, 0xA1, 0x86},
		2, {0x46, 0x68},
		2, {0xDB, 0xDA}
	},
	{
		2, {0x9E, 0x7A},
		3, {0xE6, 0xA0, 0xA9},
		2, {0x29, 0x68},
		2, {0xDB, 0xDB}
	},
	{
		2, {0x9E, 0x7B},
		3, {0xE6, 0xA1, 0x80},
		2, {0x40, 0x68},
		2, {0xDB, 0xDC}
	},
	{
		2, {0x9E, 0x7C},
		3, {0xE6, 0xA1, 0x8D},
		2, {0x4D, 0x68},
		2, {0xDB, 0xDD}
	},
	{
		2, {0x9E, 0x7D},
		3, {0xE6, 0xA0, 0xB2},
		2, {0x32, 0x68},
		2, {0xDB, 0xDE}
	},
	{
		2, {0x9E, 0x7E},
		3, {0xE6, 0xA1, 0x8E},
		2, {0x4E, 0x68},
		2, {0xDB, 0xDF}
	},
	{
		2, {0x9E, 0x80},
		3, {0xE6, 0xA2, 0xB3},
		2, {0xB3, 0x68},
		2, {0xDB, 0xE0}
	},
	{
		2, {0x9E, 0x81},
		3, {0xE6, 0xA0, 0xAB},
		2, {0x2B, 0x68},
		2, {0xDB, 0xE1}
	},
	{
		2, {0x9E, 0x82},
		3, {0xE6, 0xA1, 0x99},
		2, {0x59, 0x68},
		2, {0xDB, 0xE2}
	},
	{
		2, {0x9E, 0x83},
		3, {0xE6, 0xA1, 0xA3},
		2, {0x63, 0x68},
		2, {0xDB, 0xE3}
	},
	{
		2, {0x9E, 0x84},
		3, {0xE6, 0xA1, 0xB7},
		2, {0x77, 0x68},
		2, {0xDB, 0xE4}
	},
	{
		2, {0x9E, 0x85},
		3, {0xE6, 0xA1, 0xBF},
		2, {0x7F, 0x68},
		2, {0xDB, 0xE5}
	},
	{
		2, {0x9E, 0x86},
		3, {0xE6, 0xA2, 0x9F},
		2, {0x9F, 0x68},
		2, {0xDB, 0xE6}
	},
	{
		2, {0x9E, 0x87},
		3, {0xE6, 0xA2, 0x8F},
		2, {0x8F, 0x68},
		2, {0xDB, 0xE7}
	},
	{
		2, {0x9E, 0x88},
		3, {0xE6, 0xA2, 0xAD},
		2, {0xAD, 0x68},
		2, {0xDB, 0xE8}
	},
	{
		2, {0x9E, 0x89},
		3, {0xE6, 0xA2, 0x94},
		2, {0x94, 0x68},
		2, {0xDB, 0xE9}
	},
	{
		2, {0x9E, 0x8A},
		3, {0xE6, 0xA2, 0x9D},
		2, {0x9D, 0x68},
		2, {0xDB, 0xEA}
	},
	{
		2, {0x9E, 0x8B},
		3, {0xE6, 0xA2, 0x9B},
		2, {0x9B, 0x68},
		2, {0xDB, 0xEB}
	},
	{
		2, {0x9E, 0x8C},
		3, {0xE6, 0xA2, 0x83},
		2, {0x83, 0x68},
		2, {0xDB, 0xEC}
	},
	{
		2, {0x9E, 0x8D},
		3, {0xE6, 0xAA, 0xAE},
		2, {0xAE, 0x6A},
		2, {0xDB, 0xED}
	},
	{
		2, {0x9E, 0x8E},
		3, {0xE6, 0xA2, 0xB9},
		2, {0xB9, 0x68},
		2, {0xDB, 0xEE}
	},
	{
		2, {0x9E, 0x8F},
		3, {0xE6, 0xA1, 0xB4},
		2, {0x74, 0x68},
		2, {0xDB, 0xEF}
	},
	{
		2, {0x9E, 0x90},
		3, {0xE6, 0xA2, 0xB5},
		2, {0xB5, 0x68},
		2, {0xDB, 0xF0}
	},
	{
		2, {0x9E, 0x91},
		3, {0xE6, 0xA2, 0xA0},
		2, {0xA0, 0x68},
		2, {0xDB, 0xF1}
	},
	{
		2, {0x9E, 0x92},
		3, {0xE6, 0xA2, 0xBA},
		2, {0xBA, 0x68},
		2, {0xDB, 0xF2}
	},
	{
		2, {0x9E, 0x93},
		3, {0xE6, 0xA4, 0x8F},
		2, {0x0F, 0x69},
		2, {0xDB, 0xF3}
	},
	{
		2, {0x9E, 0x94},
		3, {0xE6, 0xA2, 0x8D},
		2, {0x8D, 0x68},
		2, {0xDB, 0xF4}
	},
	{
		2, {0x9E, 0x95},
		3, {0xE6, 0xA1, 0xBE},
		2, {0x7E, 0x68},
		2, {0xDB, 0xF5}
	},
	{
		2, {0x9E, 0x96},
		3, {0xE6, 0xA4, 0x81},
		2, {0x01, 0x69},
		2, {0xDB, 0xF6}
	},
	{
		2, {0x9E, 0x97},
		3, {0xE6, 0xA3, 0x8A},
		2, {0xCA, 0x68},
		2, {0xDB, 0xF7}
	},
	{
		2, {0x9E, 0x98},
		3, {0xE6, 0xA4, 0x88},
		2, {0x08, 0x69},
		2, {0xDB, 0xF8}
	},
	{
		2, {0x9E, 0x99},
		3, {0xE6, 0xA3, 0x98},
		2, {0xD8, 0x68},
		2, {0xDB, 0xF9}
	},
	{
		2, {0x9E, 0x9A},
		3, {0xE6, 0xA4, 0xA2},
		2, {0x22, 0x69},
		2, {0xDB, 0xFA}
	},
	{
		2, {0x9E, 0x9B},
		3, {0xE6, 0xA4, 0xA6},
		2, {0x26, 0x69},
		2, {0xDB, 0xFB}
	},
	{
		2, {0x9E, 0x9C},
		3, {0xE6, 0xA3, 0xA1},
		2, {0xE1, 0x68},
		2, {0xDB, 0xFC}
	},
	{
		2, {0x9E, 0x9D},
		3, {0xE6, 0xA4, 0x8C},
		2, {0x0C, 0x69},
		2, {0xDB, 0xFD}
	},
	{
		2, {0x9E, 0x9E},
		3, {0xE6, 0xA3, 0x8D},
		2, {0xCD, 0x68},
		2, {0xDB, 0xFE}
	},
	{
		2, {0x9E, 0x9F},
		3, {0xE6, 0xA3, 0x94},
		2, {0xD4, 0x68},
		2, {0xDC, 0xA1}
	},
	{
		2, {0x9E, 0xA0},
		3, {0xE6, 0xA3, 0xA7},
		2, {0xE7, 0x68},
		2, {0xDC, 0xA2}
	},
	{
		2, {0x9E, 0xA1},
		3, {0xE6, 0xA3, 0x95},
		2, {0xD5, 0x68},
		2, {0xDC, 0xA3}
	},
	{
		2, {0x9E, 0xA2},
		3, {0xE6, 0xA4, 0xB6},
		2, {0x36, 0x69},
		2, {0xDC, 0xA4}
	},
	{
		2, {0x9E, 0xA3},
		3, {0xE6, 0xA4, 0x92},
		2, {0x12, 0x69},
		2, {0xDC, 0xA5}
	},
	{
		2, {0x9E, 0xA4},
		3, {0xE6, 0xA4, 0x84},
		2, {0x04, 0x69},
		2, {0xDC, 0xA6}
	},
	{
		2, {0x9E, 0xA5},
		3, {0xE6, 0xA3, 0x97},
		2, {0xD7, 0x68},
		2, {0xDC, 0xA7}
	},
	{
		2, {0x9E, 0xA6},
		3, {0xE6, 0xA3, 0xA3},
		2, {0xE3, 0x68},
		2, {0xDC, 0xA8}
	},
	{
		2, {0x9E, 0xA7},
		3, {0xE6, 0xA4, 0xA5},
		2, {0x25, 0x69},
		2, {0xDC, 0xA9}
	},
	{
		2, {0x9E, 0xA8},
		3, {0xE6, 0xA3, 0xB9},
		2, {0xF9, 0x68},
		2, {0xDC, 0xAA}
	},
	{
		2, {0x9E, 0xA9},
		3, {0xE6, 0xA3, 0xA0},
		2, {0xE0, 0x68},
		2, {0xDC, 0xAB}
	},
	{
		2, {0x9E, 0xAA},
		3, {0xE6, 0xA3, 0xAF},
		2, {0xEF, 0x68},
		2, {0xDC, 0xAC}
	},
	{
		2, {0x9E, 0xAB},
		3, {0xE6, 0xA4, 0xA8},
		2, {0x28, 0x69},
		2, {0xDC, 0xAD}
	},
	{
		2, {0x9E, 0xAC},
		3, {0xE6, 0xA4, 0xAA},
		2, {0x2A, 0x69},
		2, {0xDC, 0xAE}
	},
	{
		2, {0x9E, 0xAD},
		3, {0xE6, 0xA4, 0x9A},
		2, {0x1A, 0x69},
		2, {0xDC, 0xAF}
	},
	{
		2, {0x9E, 0xAE},
		3, {0xE6, 0xA4, 0xA3},
		2, {0x23, 0x69},
		2, {0xDC, 0xB0}
	},
	{
		2, {0x9E, 0xAF},
		3, {0xE6, 0xA4, 0xA1},
		2, {0x21, 0x69},
		2, {0xDC, 0xB1}
	},
	{
		2, {0x9E, 0xB0},
		3, {0xE6, 0xA3, 0x86},
		2, {0xC6, 0x68},
		2, {0xDC, 0xB2}
	},
	{
		2, {0x9E, 0xB1},
		3, {0xE6, 0xA5, 0xB9},
		2, {0x79, 0x69},
		2, {0xDC, 0xB3}
	},
	{
		2, {0x9E, 0xB2},
		3, {0xE6, 0xA5, 0xB7},
		2, {0x77, 0x69},
		2, {0xDC, 0xB4}
	},
	{
		2, {0x9E, 0xB3},
		3, {0xE6, 0xA5, 0x9C},
		2, {0x5C, 0x69},
		2, {0xDC, 0xB5}
	},
	{
		2, {0x9E, 0xB4},
		3, {0xE6, 0xA5, 0xB8},
		2, {0x78, 0x69},
		2, {0xDC, 0xB6}
	},
	{
		2, {0x9E, 0xB5},
		3, {0xE6, 0xA5, 0xAB},
		2, {0x6B, 0x69},
		2, {0xDC, 0xB7}
	},
	{
		2, {0x9E, 0xB6},
		3, {0xE6, 0xA5, 0x94},
		2, {0x54, 0x69},
		2, {0xDC, 0xB8}
	},
	{
		2, {0x9E, 0xB7},
		3, {0xE6, 0xA5, 0xBE},
		2, {0x7E, 0x69},
		2, {0xDC, 0xB9}
	},
	{
		2, {0x9E, 0xB8},
		3, {0xE6, 0xA5, 0xAE},
		2, {0x6E, 0x69},
		2, {0xDC, 0xBA}
	},
	{
		2, {0x9E, 0xB9},
		3, {0xE6, 0xA4, 0xB9},
		2, {0x39, 0x69},
		2, {0xDC, 0xBB}
	},
	{
		2, {0x9E, 0xBA},
		3, {0xE6, 0xA5, 0xB4},
		2, {0x74, 0x69},
		2, {0xDC, 0xBC}
	},
	{
		2, {0x9E, 0xBB},
		3, {0xE6, 0xA4, 0xBD},
		2, {0x3D, 0x69},
		2, {0xDC, 0xBD}
	},
	{
		2, {0x9E, 0xBC},
		3, {0xE6, 0xA5, 0x99},
		2, {0x59, 0x69},
		2, {0xDC, 0xBE}
	},
	{
		2, {0x9E, 0xBD},
		3, {0xE6, 0xA4, 0xB0},
		2, {0x30, 0x69},
		2, {0xDC, 0xBF}
	},
	{
		2, {0x9E, 0xBE},
		3, {0xE6, 0xA5, 0xA1},
		2, {0x61, 0x69},
		2, {0xDC, 0xC0}
	},
	{
		2, {0x9E, 0xBF},
		3, {0xE6, 0xA5, 0x9E},
		2, {0x5E, 0x69},
		2, {0xDC, 0xC1}
	},
	{
		2, {0x9E, 0xC0},
		3, {0xE6, 0xA5, 0x9D},
		2, {0x5D, 0x69},
		2, {0xDC, 0xC2}
	},
	{
		2, {0x9E, 0xC1},
		3, {0xE6, 0xA6, 0x81},
		2, {0x81, 0x69},
		2, {0xDC, 0xC3}
	},
	{
		2, {0x9E, 0xC2},
		3, {0xE6, 0xA5, 0xAA},
		2, {0x6A, 0x69},
		2, {0xDC, 0xC4}
	},
	{
		2, {0x9E, 0xC3},
		3, {0xE6, 0xA6, 0xB2},
		2, {0xB2, 0x69},
		2, {0xDC, 0xC5}
	},
	{
		2, {0x9E, 0xC4},
		3, {0xE6, 0xA6, 0xAE},
		2, {0xAE, 0x69},
		2, {0xDC, 0xC6}
	},
	{
		2, {0x9E, 0xC5},
		3, {0xE6, 0xA7, 0x90},
		2, {0xD0, 0x69},
		2, {0xDC, 0xC7}
	},
	{
		2, {0x9E, 0xC6},
		3, {0xE6, 0xA6, 0xBF},
		2, {0xBF, 0x69},
		2, {0xDC, 0xC8}
	},
	{
		2, {0x9E, 0xC7},
		3, {0xE6, 0xA7, 0x81},
		2, {0xC1, 0x69},
		2, {0xDC, 0xC9}
	},
	{
		2, {0x9E, 0xC8},
		3, {0xE6, 0xA7, 0x93},
		2, {0xD3, 0x69},
		2, {0xDC, 0xCA}
	},
	{
		2, {0x9E, 0xC9},
		3, {0xE6, 0xA6, 0xBE},
		2, {0xBE, 0x69},
		2, {0xDC, 0xCB}
	},
	{
		2, {0x9E, 0xCA},
		3, {0xE6, 0xA7, 0x8E},
		2, {0xCE, 0x69},
		2, {0xDC, 0xCC}
	},
	{
		2, {0x9E, 0xCB},
		3, {0xE5, 0xAF, 0xA8},
		2, {0xE8, 0x5B},
		2, {0xDC, 0xCD}
	},
	{
		2, {0x9E, 0xCC},
		3, {0xE6, 0xA7, 0x8A},
		2, {0xCA, 0x69},
		2, {0xDC, 0xCE}
	},
	{
		2, {0x9E, 0xCD},
		3, {0xE6, 0xA7, 0x9D},
		2, {0xDD, 0x69},
		2, {0xDC, 0xCF}
	},
	{
		2, {0x9E, 0xCE},
		3, {0xE6, 0xA6, 0xBB},
		2, {0xBB, 0x69},
		2, {0xDC, 0xD0}
	},
	{
		2, {0x9E, 0xCF},
		3, {0xE6, 0xA7, 0x83},
		2, {0xC3, 0x69},
		2, {0xDC, 0xD1}
	},
	{
		2, {0x9E, 0xD0},
		3, {0xE6, 0xA6, 0xA7},
		2, {0xA7, 0x69},
		2, {0xDC, 0xD2}
	},
	{
		2, {0x9E, 0xD1},
		3, {0xE6, 0xA8, 0xAE},
		2, {0x2E, 0x6A},
		2, {0xDC, 0xD3}
	},
	{
		2, {0x9E, 0xD2},
		3, {0xE6, 0xA6, 0x91},
		2, {0x91, 0x69},
		2, {0xDC, 0xD4}
	},
	{
		2, {0x9E, 0xD3},
		3, {0xE6, 0xA6, 0xA0},
		2, {0xA0, 0x69},
		2, {0xDC, 0xD5}
	},
	{
		2, {0x9E, 0xD4},
		3, {0xE6, 0xA6, 0x9C},
		2, {0x9C, 0x69},
		2, {0xDC, 0xD6}
	},
	{
		2, {0x9E, 0xD5},
		3, {0xE6, 0xA6, 0x95},
		2, {0x95, 0x69},
		2, {0xDC, 0xD7}
	},
	{
		2, {0x9E, 0xD6},
		3, {0xE6, 0xA6, 0xB4},
		2, {0xB4, 0x69},
		2, {0xDC, 0xD8}
	},
	{
		2, {0x9E, 0xD7},
		3, {0xE6, 0xA7, 0x9E},
		2, {0xDE, 0x69},
		2, {0xDC, 0xD9}
	},
	{
		2, {0x9E, 0xD8},
		3, {0xE6, 0xA7, 0xA8},
		2, {0xE8, 0x69},
		2, {0xDC, 0xDA}
	},
	{
		2, {0x9E, 0xD9},
		3, {0xE6, 0xA8, 0x82},
		2, {0x02, 0x6A},
		2, {0xDC, 0xDB}
	},
	{
		2, {0x9E, 0xDA},
		3, {0xE6, 0xA8, 0x9B},
		2, {0x1B, 0x6A},
		2, {0xDC, 0xDC}
	},
	{
		2, {0x9E, 0xDB},
		3, {0xE6, 0xA7, 0xBF},
		2, {0xFF, 0x69},
		2, {0xDC, 0xDD}
	},
	{
		2, {0x9E, 0xDC},
		3, {0xE6, 0xAC, 0x8A},
		2, {0x0A, 0x6B},
		2, {0xDC, 0xDE}
	},
	{
		2, {0x9E, 0xDD},
		3, {0xE6, 0xA7, 0xB9},
		2, {0xF9, 0x69},
		2, {0xDC, 0xDF}
	},
	{
		2, {0x9E, 0xDE},
		3, {0xE6, 0xA7, 0xB2},
		2, {0xF2, 0x69},
		2, {0xDC, 0xE0}
	},
	{
		2, {0x9E, 0xDF},
		3, {0xE6, 0xA7, 0xA7},
		2, {0xE7, 0x69},
		2, {0xDC, 0xE1}
	},
	{
		2, {0x9E, 0xE0},
		3, {0xE6, 0xA8, 0x85},
		2, {0x05, 0x6A},
		2, {0xDC, 0xE2}
	},
	{
		2, {0x9E, 0xE1},
		3, {0xE6, 0xA6, 0xB1},
		2, {0xB1, 0x69},
		2, {0xDC, 0xE3}
	},
	{
		2, {0x9E, 0xE2},
		3, {0xE6, 0xA8, 0x9E},
		2, {0x1E, 0x6A},
		2, {0xDC, 0xE4}
	},
	{
		2, {0x9E, 0xE3},
		3, {0xE6, 0xA7, 0xAD},
		2, {0xED, 0x69},
		2, {0xDC, 0xE5}
	},
	{
		2, {0x9E, 0xE4},
		3, {0xE6, 0xA8, 0x94},
		2, {0x14, 0x6A},
		2, {0xDC, 0xE6}
	},
	{
		2, {0x9E, 0xE5},
		3, {0xE6, 0xA7, 0xAB},
		2, {0xEB, 0x69},
		2, {0xDC, 0xE7}
	},
	{
		2, {0x9E, 0xE6},
		3, {0xE6, 0xA8, 0x8A},
		2, {0x0A, 0x6A},
		2, {0xDC, 0xE8}
	},
	{
		2, {0x9E, 0xE7},
		3, {0xE6, 0xA8, 0x92},
		2, {0x12, 0x6A},
		2, {0xDC, 0xE9}
	},
	{
		2, {0x9E, 0xE8},
		3, {0xE6, 0xAB, 0x81},
		2, {0xC1, 0x6A},
		2, {0xDC, 0xEA}
	},
	{
		2, {0x9E, 0xE9},
		3, {0xE6, 0xA8, 0xA3},
		2, {0x23, 0x6A},
		2, {0xDC, 0xEB}
	},
	{
		2, {0x9E, 0xEA},
		3, {0xE6, 0xA8, 0x93},
		2, {0x13, 0x6A},
		2, {0xDC, 0xEC}
	},
	{
		2, {0x9E, 0xEB},
		3, {0xE6, 0xA9, 0x84},
		2, {0x44, 0x6A},
		2, {0xDC, 0xED}
	},
	{
		2, {0x9E, 0xEC},
		3, {0xE6, 0xA8, 0x8C},
		2, {0x0C, 0x6A},
		2, {0xDC, 0xEE}
	},
	{
		2, {0x9E, 0xED},
		3, {0xE6, 0xA9, 0xB2},
		2, {0x72, 0x6A},
		2, {0xDC, 0xEF}
	},
	{
		2, {0x9E, 0xEE},
		3, {0xE6, 0xA8, 0xB6},
		2, {0x36, 0x6A},
		2, {0xDC, 0xF0}
	},
	{
		2, {0x9E, 0xEF},
		3, {0xE6, 0xA9, 0xB8},
		2, {0x78, 0x6A},
		2, {0xDC, 0xF1}
	},
	{
		2, {0x9E, 0xF0},
		3, {0xE6, 0xA9, 0x87},
		2, {0x47, 0x6A},
		2, {0xDC, 0xF2}
	},
	{
		2, {0x9E, 0xF1},
		3, {0xE6, 0xA9, 0xA2},
		2, {0x62, 0x6A},
		2, {0xDC, 0xF3}
	},
	{
		2, {0x9E, 0xF2},
		3, {0xE6, 0xA9, 0x99},
		2, {0x59, 0x6A},
		2, {0xDC, 0xF4}
	},
	{
		2, {0x9E, 0xF3},
		3, {0xE6, 0xA9, 0xA6},
		2, {0x66, 0x6A},
		2, {0xDC, 0xF5}
	},
	{
		2, {0x9E, 0xF4},
		3, {0xE6, 0xA9, 0x88},
		2, {0x48, 0x6A},
		2, {0xDC, 0xF6}
	},
	{
		2, {0x9E, 0xF5},
		3, {0xE6, 0xA8, 0xB8},
		2, {0x38, 0x6A},
		2, {0xDC, 0xF7}
	},
	{
		2, {0x9E, 0xF6},
		3, {0xE6, 0xA8, 0xA2},
		2, {0x22, 0x6A},
		2, {0xDC, 0xF8}
	},
	{
		2, {0x9E, 0xF7},
		3, {0xE6, 0xAA, 0x90},
		2, {0x90, 0x6A},
		2, {0xDC, 0xF9}
	},
	{
		2, {0x9E, 0xF8},
		3, {0xE6, 0xAA, 0x8D},
		2, {0x8D, 0x6A},
		2, {0xDC, 0xFA}
	},
	{
		2, {0x9E, 0xF9},
		3, {0xE6, 0xAA, 0xA0},
		2, {0xA0, 0x6A},
		2, {0xDC, 0xFB}
	},
	{
		2, {0x9E, 0xFA},
		3, {0xE6, 0xAA, 0x84},
		2, {0x84, 0x6A},
		2, {0xDC, 0xFC}
	},
	{
		2, {0x9E, 0xFB},
		3, {0xE6, 0xAA, 0xA2},
		2, {0xA2, 0x6A},
		2, {0xDC, 0xFD}
	},
	{
		2, {0x9E, 0xFC},
		3, {0xE6, 0xAA, 0xA3},
		2, {0xA3, 0x6A},
		2, {0xDC, 0xFE}
	},
	{
		2, {0x9F, 0x40},
		3, {0xE6, 0xAA, 0x97},
		2, {0x97, 0x6A},
		2, {0xDD, 0xA1}
	},
	{
		2, {0x9F, 0x41},
		3, {0xE8, 0x98, 0x97},
		2, {0x17, 0x86},
		2, {0xDD, 0xA2}
	},
	{
		2, {0x9F, 0x42},
		3, {0xE6, 0xAA, 0xBB},
		2, {0xBB, 0x6A},
		2, {0xDD, 0xA3}
	},
	{
		2, {0x9F, 0x43},
		3, {0xE6, 0xAB, 0x83},
		2, {0xC3, 0x6A},
		2, {0xDD, 0xA4}
	},
	{
		2, {0x9F, 0x44},
		3, {0xE6, 0xAB, 0x82},
		2, {0xC2, 0x6A},
		2, {0xDD, 0xA5}
	},
	{
		2, {0x9F, 0x45},
		3, {0xE6, 0xAA, 0xB8},
		2, {0xB8, 0x6A},
		2, {0xDD, 0xA6}
	},
	{
		2, {0x9F, 0x46},
		3, {0xE6, 0xAA, 0xB3},
		2, {0xB3, 0x6A},
		2, {0xDD, 0xA7}
	},
	{
		2, {0x9F, 0x47},
		3, {0xE6, 0xAA, 0xAC},
		2, {0xAC, 0x6A},
		2, {0xDD, 0xA8}
	},
	{
		2, {0x9F, 0x48},
		3, {0xE6, 0xAB, 0x9E},
		2, {0xDE, 0x6A},
		2, {0xDD, 0xA9}
	},
	{
		2, {0x9F, 0x49},
		3, {0xE6, 0xAB, 0x91},
		2, {0xD1, 0x6A},
		2, {0xDD, 0xAA}
	},
	{
		2, {0x9F, 0x4A},
		3, {0xE6, 0xAB, 0x9F},
		2, {0xDF, 0x6A},
		2, {0xDD, 0xAB}
	},
	{
		2, {0x9F, 0x4B},
		3, {0xE6, 0xAA, 0xAA},
		2, {0xAA, 0x6A},
		2, {0xDD, 0xAC}
	},
	{
		2, {0x9F, 0x4C},
		3, {0xE6, 0xAB, 0x9A},
		2, {0xDA, 0x6A},
		2, {0xDD, 0xAD}
	},
	{
		2, {0x9F, 0x4D},
		3, {0xE6, 0xAB, 0xAA},
		2, {0xEA, 0x6A},
		2, {0xDD, 0xAE}
	},
	{
		2, {0x9F, 0x4E},
		3, {0xE6, 0xAB, 0xBB},
		2, {0xFB, 0x6A},
		2, {0xDD, 0xAF}
	},
	{
		2, {0x9F, 0x4F},
		3, {0xE6, 0xAC, 0x85},
		2, {0x05, 0x6B},
		2, {0xDD, 0xB0}
	},
	{
		2, {0x9F, 0x50},
		3, {0xE8, 0x98, 0x96},
		2, {0x16, 0x86},
		2, {0xDD, 0xB1}
	},
	{
		2, {0x9F, 0x51},
		3, {0xE6, 0xAB, 0xBA},
		2, {0xFA, 0x6A},
		2, {0xDD, 0xB2}
	},
	{
		2, {0x9F, 0x52},
		3, {0xE6, 0xAC, 0x92},
		2, {0x12, 0x6B},
		2, {0xDD, 0xB3}
	},
	{
		2, {0x9F, 0x53},
		3, {0xE6, 0xAC, 0x96},
		2, {0x16, 0x6B},
		2, {0xDD, 0xB4}
	},
	{
		2, {0x9F, 0x54},
		3, {0xE9, 0xAC, 0xB1},
		2, {0x31, 0x9B},
		2, {0xDD, 0xB5}
	},
	{
		2, {0x9F, 0x55},
		3, {0xE6, 0xAC, 0x9F},
		2, {0x1F, 0x6B},
		2, {0xDD, 0xB6}
	},
	{
		2, {0x9F, 0x56},
		3, {0xE6, 0xAC, 0xB8},
		2, {0x38, 0x6B},
		2, {0xDD, 0xB7}
	},
	{
		2, {0x9F, 0x57},
		3, {0xE6, 0xAC, 0xB7},
		2, {0x37, 0x6B},
		2, {0xDD, 0xB8}
	},
	{
		2, {0x9F, 0x58},
		3, {0xE7, 0x9B, 0x9C},
		2, {0xDC, 0x76},
		2, {0xDD, 0xB9}
	},
	{
		2, {0x9F, 0x59},
		3, {0xE6, 0xAC, 0xB9},
		2, {0x39, 0x6B},
		2, {0xDD, 0xBA}
	},
	{
		2, {0x9F, 0x5A},
		3, {0xE9, 0xA3, 0xAE},
		2, {0xEE, 0x98},
		2, {0xDD, 0xBB}
	},
	{
		2, {0x9F, 0x5B},
		3, {0xE6, 0xAD, 0x87},
		2, {0x47, 0x6B},
		2, {0xDD, 0xBC}
	},
	{
		2, {0x9F, 0x5C},
		3, {0xE6, 0xAD, 0x83},
		2, {0x43, 0x6B},
		2, {0xDD, 0xBD}
	},
	{
		2, {0x9F, 0x5D},
		3, {0xE6, 0xAD, 0x89},
		2, {0x49, 0x6B},
		2, {0xDD, 0xBE}
	},
	{
		2, {0x9F, 0x5E},
		3, {0xE6, 0xAD, 0x90},
		2, {0x50, 0x6B},
		2, {0xDD, 0xBF}
	},
	{
		2, {0x9F, 0x5F},
		3, {0xE6, 0xAD, 0x99},
		2, {0x59, 0x6B},
		2, {0xDD, 0xC0}
	},
	{
		2, {0x9F, 0x60},
		3, {0xE6, 0xAD, 0x94},
		2, {0x54, 0x6B},
		2, {0xDD, 0xC1}
	},
	{
		2, {0x9F, 0x61},
		3, {0xE6, 0xAD, 0x9B},
		2, {0x5B, 0x6B},
		2, {0xDD, 0xC2}
	},
	{
		2, {0x9F, 0x62},
		3, {0xE6, 0xAD, 0x9F},
		2, {0x5F, 0x6B},
		2, {0xDD, 0xC3}
	},
	{
		2, {0x9F, 0x63},
		3, {0xE6, 0xAD, 0xA1},
		2, {0x61, 0x6B},
		2, {0xDD, 0xC4}
	},
	{
		2, {0x9F, 0x64},
		3, {0xE6, 0xAD, 0xB8},
		2, {0x78, 0x6B},
		2, {0xDD, 0xC5}
	},
	{
		2, {0x9F, 0x65},
		3, {0xE6, 0xAD, 0xB9},
		2, {0x79, 0x6B},
		2, {0xDD, 0xC6}
	},
	{
		2, {0x9F, 0x66},
		3, {0xE6, 0xAD, 0xBF},
		2, {0x7F, 0x6B},
		2, {0xDD, 0xC7}
	},
	{
		2, {0x9F, 0x67},
		3, {0xE6, 0xAE, 0x80},
		2, {0x80, 0x6B},
		2, {0xDD, 0xC8}
	},
	{
		2, {0x9F, 0x68},
		3, {0xE6, 0xAE, 0x84},
		2, {0x84, 0x6B},
		2, {0xDD, 0xC9}
	},
	{
		2, {0x9F, 0x69},
		3, {0xE6, 0xAE, 0x83},
		2, {0x83, 0x6B},
		2, {0xDD, 0xCA}
	},
	{
		2, {0x9F, 0x6A},
		3, {0xE6, 0xAE, 0x8D},
		2, {0x8D, 0x6B},
		2, {0xDD, 0xCB}
	},
	{
		2, {0x9F, 0x6B},
		3, {0xE6, 0xAE, 0x98},
		2, {0x98, 0x6B},
		2, {0xDD, 0xCC}
	},
	{
		2, {0x9F, 0x6C},
		3, {0xE6, 0xAE, 0x95},
		2, {0x95, 0x6B},
		2, {0xDD, 0xCD}
	},
	{
		2, {0x9F, 0x6D},
		3, {0xE6, 0xAE, 0x9E},
		2, {0x9E, 0x6B},
		2, {0xDD, 0xCE}
	},
	{
		2, {0x9F, 0x6E},
		3, {0xE6, 0xAE, 0xA4},
		2, {0xA4, 0x6B},
		2, {0xDD, 0xCF}
	},
	{
		2, {0x9F, 0x6F},
		3, {0xE6, 0xAE, 0xAA},
		2, {0xAA, 0x6B},
		2, {0xDD, 0xD0}
	},
	{
		2, {0x9F, 0x70},
		3, {0xE6, 0xAE, 0xAB},
		2, {0xAB, 0x6B},
		2, {0xDD, 0xD1}
	},
	{
		2, {0x9F, 0x71},
		3, {0xE6, 0xAE, 0xAF},
		2, {0xAF, 0x6B},
		2, {0xDD, 0xD2}
	},
	{
		2, {0x9F, 0x72},
		3, {0xE6, 0xAE, 0xB2},
		2, {0xB2, 0x6B},
		2, {0xDD, 0xD3}
	},
	{
		2, {0x9F, 0x73},
		3, {0xE6, 0xAE, 0xB1},
		2, {0xB1, 0x6B},
		2, {0xDD, 0xD4}
	},
	{
		2, {0x9F, 0x74},
		3, {0xE6, 0xAE, 0xB3},
		2, {0xB3, 0x6B},
		2, {0xDD, 0xD5}
	},
	{
		2, {0x9F, 0x75},
		3, {0xE6, 0xAE, 0xB7},
		2, {0xB7, 0x6B},
		2, {0xDD, 0xD6}
	},
	{
		2, {0x9F, 0x76},
		3, {0xE6, 0xAE, 0xBC},
		2, {0xBC, 0x6B},
		2, {0xDD, 0xD7}
	},
	{
		2, {0x9F, 0x77},
		3, {0xE6, 0xAF, 0x86},
		2, {0xC6, 0x6B},
		2, {0xDD, 0xD8}
	},
	{
		2, {0x9F, 0x78},
		3, {0xE6, 0xAF, 0x8B},
		2, {0xCB, 0x6B},
		2, {0xDD, 0xD9}
	},
	{
		2, {0x9F, 0x79},
		3, {0xE6, 0xAF, 0x93},
		2, {0xD3, 0x6B},
		2, {0xDD, 0xDA}
	},
	{
		2, {0x9F, 0x7A},
		3, {0xE6, 0xAF, 0x9F},
		2, {0xDF, 0x6B},
		2, {0xDD, 0xDB}
	},
	{
		2, {0x9F, 0x7B},
		3, {0xE6, 0xAF, 0xAC},
		2, {0xEC, 0x6B},
		2, {0xDD, 0xDC}
	},
	{
		2, {0x9F, 0x7C},
		3, {0xE6, 0xAF, 0xAB},
		2, {0xEB, 0x6B},
		2, {0xDD, 0xDD}
	},
	{
		2, {0x9F, 0x7D},
		3, {0xE6, 0xAF, 0xB3},
		2, {0xF3, 0x6B},
		2, {0xDD, 0xDE}
	},
	{
		2, {0x9F, 0x7E},
		3, {0xE6, 0xAF, 0xAF},
		2, {0xEF, 0x6B},
		2, {0xDD, 0xDF}
	},
	{
		2, {0x9F, 0x80},
		3, {0xE9, 0xBA, 0xBE},
		2, {0xBE, 0x9E},
		2, {0xDD, 0xE0}
	},
	{
		2, {0x9F, 0x81},
		3, {0xE6, 0xB0, 0x88},
		2, {0x08, 0x6C},
		2, {0xDD, 0xE1}
	},
	{
		2, {0x9F, 0x82},
		3, {0xE6, 0xB0, 0x93},
		2, {0x13, 0x6C},
		2, {0xDD, 0xE2}
	},
	{
		2, {0x9F, 0x83},
		3, {0xE6, 0xB0, 0x94},
		2, {0x14, 0x6C},
		2, {0xDD, 0xE3}
	},
	{
		2, {0x9F, 0x84},
		3, {0xE6, 0xB0, 0x9B},
		2, {0x1B, 0x6C},
		2, {0xDD, 0xE4}
	},
	{
		2, {0x9F, 0x85},
		3, {0xE6, 0xB0, 0xA4},
		2, {0x24, 0x6C},
		2, {0xDD, 0xE5}
	},
	{
		2, {0x9F, 0x86},
		3, {0xE6, 0xB0, 0xA3},
		2, {0x23, 0x6C},
		2, {0xDD, 0xE6}
	},
	{
		2, {0x9F, 0x87},
		3, {0xE6, 0xB1, 0x9E},
		2, {0x5E, 0x6C},
		2, {0xDD, 0xE7}
	},
	{
		2, {0x9F, 0x88},
		3, {0xE6, 0xB1, 0x95},
		2, {0x55, 0x6C},
		2, {0xDD, 0xE8}
	},
	{
		2, {0x9F, 0x89},
		3, {0xE6, 0xB1, 0xA2},
		2, {0x62, 0x6C},
		2, {0xDD, 0xE9}
	},
	{
		2, {0x9F, 0x8A},
		3, {0xE6, 0xB1, 0xAA},
		2, {0x6A, 0x6C},
		2, {0xDD, 0xEA}
	},
	{
		2, {0x9F, 0x8B},
		3, {0xE6, 0xB2, 0x82},
		2, {0x82, 0x6C},
		2, {0xDD, 0xEB}
	},
	{
		2, {0x9F, 0x8C},
		3, {0xE6, 0xB2, 0x8D},
		2, {0x8D, 0x6C},
		2, {0xDD, 0xEC}
	},
	{
		2, {0x9F, 0x8D},
		3, {0xE6, 0xB2, 0x9A},
		2, {0x9A, 0x6C},
		2, {0xDD, 0xED}
	},
	{
		2, {0x9F, 0x8E},
		3, {0xE6, 0xB2, 0x81},
		2, {0x81, 0x6C},
		2, {0xDD, 0xEE}
	},
	{
		2, {0x9F, 0x8F},
		3, {0xE6, 0xB2, 0x9B},
		2, {0x9B, 0x6C},
		2, {0xDD, 0xEF}
	},
	{
		2, {0x9F, 0x90},
		3, {0xE6, 0xB1, 0xBE},
		2, {0x7E, 0x6C},
		2, {0xDD, 0xF0}
	},
	{
		2, {0x9F, 0x91},
		3, {0xE6, 0xB1, 0xA8},
		2, {0x68, 0x6C},
		2, {0xDD, 0xF1}
	},
	{
		2, {0x9F, 0x92},
		3, {0xE6, 0xB1, 0xB3},
		2, {0x73, 0x6C},
		2, {0xDD, 0xF2}
	},
	{
		2, {0x9F, 0x93},
		3, {0xE6, 0xB2, 0x92},
		2, {0x92, 0x6C},
		2, {0xDD, 0xF3}
	},
	{
		2, {0x9F, 0x94},
		3, {0xE6, 0xB2, 0x90},
		2, {0x90, 0x6C},
		2, {0xDD, 0xF4}
	},
	{
		2, {0x9F, 0x95},
		3, {0xE6, 0xB3, 0x84},
		2, {0xC4, 0x6C},
		2, {0xDD, 0xF5}
	},
	{
		2, {0x9F, 0x96},
		3, {0xE6, 0xB3, 0xB1},
		2, {0xF1, 0x6C},
		2, {0xDD, 0xF6}
	},
	{
		2, {0x9F, 0x97},
		3, {0xE6, 0xB3, 0x93},
		2, {0xD3, 0x6C},
		2, {0xDD, 0xF7}
	},
	{
		2, {0x9F, 0x98},
		3, {0xE6, 0xB2, 0xBD},
		2, {0xBD, 0x6C},
		2, {0xDD, 0xF8}
	},
	{
		2, {0x9F, 0x99},
		3, {0xE6, 0xB3, 0x97},
		2, {0xD7, 0x6C},
		2, {0xDD, 0xF9}
	},
	{
		2, {0x9F, 0x9A},
		3, {0xE6, 0xB3, 0x85},
		2, {0xC5, 0x6C},
		2, {0xDD, 0xFA}
	},
	{
		2, {0x9F, 0x9B},
		3, {0xE6, 0xB3, 0x9D},
		2, {0xDD, 0x6C},
		2, {0xDD, 0xFB}
	},
	{
		2, {0x9F, 0x9C},
		3, {0xE6, 0xB2, 0xAE},
		2, {0xAE, 0x6C},
		2, {0xDD, 0xFC}
	},
	{
		2, {0x9F, 0x9D},
		3, {0xE6, 0xB2, 0xB1},
		2, {0xB1, 0x6C},
		2, {0xDD, 0xFD}
	},
	{
		2, {0x9F, 0x9E},
		3, {0xE6, 0xB2, 0xBE},
		2, {0xBE, 0x6C},
		2, {0xDD, 0xFE}
	},
	{
		2, {0x9F, 0x9F},
		3, {0xE6, 0xB2, 0xBA},
		2, {0xBA, 0x6C},
		2, {0xDE, 0xA1}
	},
	{
		2, {0x9F, 0xA0},
		3, {0xE6, 0xB3, 0x9B},
		2, {0xDB, 0x6C},
		2, {0xDE, 0xA2}
	},
	{
		2, {0x9F, 0xA1},
		3, {0xE6, 0xB3, 0xAF},
		2, {0xEF, 0x6C},
		2, {0xDE, 0xA3}
	},
	{
		2, {0x9F, 0xA2},
		3, {0xE6, 0xB3, 0x99},
		2, {0xD9, 0x6C},
		2, {0xDE, 0xA4}
	},
	{
		2, {0x9F, 0xA3},
		3, {0xE6, 0xB3, 0xAA},
		2, {0xEA, 0x6C},
		2, {0xDE, 0xA5}
	},
	{
		2, {0x9F, 0xA4},
		3, {0xE6, 0xB4, 0x9F},
		2, {0x1F, 0x6D},
		2, {0xDE, 0xA6}
	},
	{
		2, {0x9F, 0xA5},
		3, {0xE8, 0xA1, 0x8D},
		2, {0x4D, 0x88},
		2, {0xDE, 0xA7}
	},
	{
		2, {0x9F, 0xA6},
		3, {0xE6, 0xB4, 0xB6},
		2, {0x36, 0x6D},
		2, {0xDE, 0xA8}
	},
	{
		2, {0x9F, 0xA7},
		3, {0xE6, 0xB4, 0xAB},
		2, {0x2B, 0x6D},
		2, {0xDE, 0xA9}
	},
	{
		2, {0x9F, 0xA8},
		3, {0xE6, 0xB4, 0xBD},
		2, {0x3D, 0x6D},
		2, {0xDE, 0xAA}
	},
	{
		2, {0x9F, 0xA9},
		3, {0xE6, 0xB4, 0xB8},
		2, {0x38, 0x6D},
		2, {0xDE, 0xAB}
	},
	{
		2, {0x9F, 0xAA},
		3, {0xE6, 0xB4, 0x99},
		2, {0x19, 0x6D},
		2, {0xDE, 0xAC}
	},
	{
		2, {0x9F, 0xAB},
		3, {0xE6, 0xB4, 0xB5},
		2, {0x35, 0x6D},
		2, {0xDE, 0xAD}
	},
	{
		2, {0x9F, 0xAC},
		3, {0xE6, 0xB4, 0xB3},
		2, {0x33, 0x6D},
		2, {0xDE, 0xAE}
	},
	{
		2, {0x9F, 0xAD},
		3, {0xE6, 0xB4, 0x92},
		2, {0x12, 0x6D},
		2, {0xDE, 0xAF}
	},
	{
		2, {0x9F, 0xAE},
		3, {0xE6, 0xB4, 0x8C},
		2, {0x0C, 0x6D},
		2, {0xDE, 0xB0}
	},
	{
		2, {0x9F, 0xAF},
		3, {0xE6, 0xB5, 0xA3},
		2, {0x63, 0x6D},
		2, {0xDE, 0xB1}
	},
	{
		2, {0x9F, 0xB0},
		3, {0xE6, 0xB6, 0x93},
		2, {0x93, 0x6D},
		2, {0xDE, 0xB2}
	},
	{
		2, {0x9F, 0xB1},
		3, {0xE6, 0xB5, 0xA4},
		2, {0x64, 0x6D},
		2, {0xDE, 0xB3}
	},
	{
		2, {0x9F, 0xB2},
		3, {0xE6, 0xB5, 0x9A},
		2, {0x5A, 0x6D},
		2, {0xDE, 0xB4}
	},
	{
		2, {0x9F, 0xB3},
		3, {0xE6, 0xB5, 0xB9},
		2, {0x79, 0x6D},
		2, {0xDE, 0xB5}
	},
	{
		2, {0x9F, 0xB4},
		3, {0xE6, 0xB5, 0x99},
		2, {0x59, 0x6D},
		2, {0xDE, 0xB6}
	},
	{
		2, {0x9F, 0xB5},
		3, {0xE6, 0xB6, 0x8E},
		2, {0x8E, 0x6D},
		2, {0xDE, 0xB7}
	},
	{
		2, {0x9F, 0xB6},
		3, {0xE6, 0xB6, 0x95},
		2, {0x95, 0x6D},
		2, {0xDE, 0xB8}
	},
	{
		2, {0x9F, 0xB7},
		3, {0xE6, 0xBF, 0xA4},
		2, {0xE4, 0x6F},
		2, {0xDE, 0xB9}
	},
	{
		2, {0x9F, 0xB8},
		3, {0xE6, 0xB6, 0x85},
		2, {0x85, 0x6D},
		2, {0xDE, 0xBA}
	},
	{
		2, {0x9F, 0xB9},
		3, {0xE6, 0xB7, 0xB9},
		2, {0xF9, 0x6D},
		2, {0xDE, 0xBB}
	},
	{
		2, {0x9F, 0xBA},
		3, {0xE6, 0xB8, 0x95},
		2, {0x15, 0x6E},
		2, {0xDE, 0xBC}
	},
	{
		2, {0x9F, 0xBB},
		3, {0xE6, 0xB8, 0x8A},
		2, {0x0A, 0x6E},
		2, {0xDE, 0xBD}
	},
	{
		2, {0x9F, 0xBC},
		3, {0xE6, 0xB6, 0xB5},
		2, {0xB5, 0x6D},
		2, {0xDE, 0xBE}
	},
	{
		2, {0x9F, 0xBD},
		3, {0xE6, 0xB7, 0x87},
		2, {0xC7, 0x6D},
		2, {0xDE, 0xBF}
	},
	{
		2, {0x9F, 0xBE},
		3, {0xE6, 0xB7, 0xA6},
		2, {0xE6, 0x6D},
		2, {0xDE, 0xC0}
	},
	{
		2, {0x9F, 0xBF},
		3, {0xE6, 0xB6, 0xB8},
		2, {0xB8, 0x6D},
		2, {0xDE, 0xC1}
	},
	{
		2, {0x9F, 0xC0},
		3, {0xE6, 0xB7, 0x86},
		2, {0xC6, 0x6D},
		2, {0xDE, 0xC2}
	},
	{
		2, {0x9F, 0xC1},
		3, {0xE6, 0xB7, 0xAC},
		2, {0xEC, 0x6D},
		2, {0xDE, 0xC3}
	},
	{
		2, {0x9F, 0xC2},
		3, {0xE6, 0xB7, 0x9E},
		2, {0xDE, 0x6D},
		2, {0xDE, 0xC4}
	},
	{
		2, {0x9F, 0xC3},
		3, {0xE6, 0xB7, 0x8C},
		2, {0xCC, 0x6D},
		2, {0xDE, 0xC5}
	},
	{
		2, {0x9F, 0xC4},
		3, {0xE6, 0xB7, 0xA8},
		2, {0xE8, 0x6D},
		2, {0xDE, 0xC6}
	},
	{
		2, {0x9F, 0xC5},
		3, {0xE6, 0xB7, 0x92},
		2, {0xD2, 0x6D},
		2, {0xDE, 0xC7}
	},
	{
		2, {0x9F, 0xC6},
		3, {0xE6, 0xB7, 0x85},
		2, {0xC5, 0x6D},
		2, {0xDE, 0xC8}
	},
	{
		2, {0x9F, 0xC7},
		3, {0xE6, 0xB7, 0xBA},
		2, {0xFA, 0x6D},
		2, {0xDE, 0xC9}
	},
	{
		2, {0x9F, 0xC8},
		3, {0xE6, 0xB7, 0x99},
		2, {0xD9, 0x6D},
		2, {0xDE, 0xCA}
	},
	{
		2, {0x9F, 0xC9},
		3, {0xE6, 0xB7, 0xA4},
		2, {0xE4, 0x6D},
		2, {0xDE, 0xCB}
	},
	{
		2, {0x9F, 0xCA},
		3, {0xE6, 0xB7, 0x95},
		2, {0xD5, 0x6D},
		2, {0xDE, 0xCC}
	},
	{
		2, {0x9F, 0xCB},
		3, {0xE6, 0xB7, 0xAA},
		2, {0xEA, 0x6D},
		2, {0xDE, 0xCD}
	},
	{
		2, {0x9F, 0xCC},
		3, {0xE6, 0xB7, 0xAE},
		2, {0xEE, 0x6D},
		2, {0xDE, 0xCE}
	},
	{
		2, {0x9F, 0xCD},
		3, {0xE6, 0xB8, 0xAD},
		2, {0x2D, 0x6E},
		2, {0xDE, 0xCF}
	},
	{
		2, {0x9F, 0xCE},
		3, {0xE6, 0xB9, 0xAE},
		2, {0x6E, 0x6E},
		2, {0xDE, 0xD0}
	},
	{
		2, {0x9F, 0xCF},
		3, {0xE6, 0xB8, 0xAE},
		2, {0x2E, 0x6E},
		2, {0xDE, 0xD1}
	},
	{
		2, {0x9F, 0xD0},
		3, {0xE6, 0xB8, 0x99},
		2, {0x19, 0x6E},
		2, {0xDE, 0xD2}
	},
	{
		2, {0x9F, 0xD1},
		3, {0xE6, 0xB9, 0xB2},
		2, {0x72, 0x6E},
		2, {0xDE, 0xD3}
	},
	{
		2, {0x9F, 0xD2},
		3, {0xE6, 0xB9, 0x9F},
		2, {0x5F, 0x6E},
		2, {0xDE, 0xD4}
	},
	{
		2, {0x9F, 0xD3},
		3, {0xE6, 0xB8, 0xBE},
		2, {0x3E, 0x6E},
		2, {0xDE, 0xD5}
	},
	{
		2, {0x9F, 0xD4},
		3, {0xE6, 0xB8, 0xA3},
		2, {0x23, 0x6E},
		2, {0xDE, 0xD6}
	},
	{
		2, {0x9F, 0xD5},
		3, {0xE6, 0xB9, 0xAB},
		2, {0x6B, 0x6E},
		2, {0xDE, 0xD7}
	},
	{
		2, {0x9F, 0xD6},
		3, {0xE6, 0xB8, 0xAB},
		2, {0x2B, 0x6E},
		2, {0xDE, 0xD8}
	},
	{
		2, {0x9F, 0xD7},
		3, {0xE6, 0xB9, 0xB6},
		2, {0x76, 0x6E},
		2, {0xDE, 0xD9}
	},
	{
		2, {0x9F, 0xD8},
		3, {0xE6, 0xB9, 0x8D},
		2, {0x4D, 0x6E},
		2, {0xDE, 0xDA}
	},
	{
		2, {0x9F, 0xD9},
		3, {0xE6, 0xB8, 0x9F},
		2, {0x1F, 0x6E},
		2, {0xDE, 0xDB}
	},
	{
		2, {0x9F, 0xDA},
		3, {0xE6, 0xB9, 0x83},
		2, {0x43, 0x6E},
		2, {0xDE, 0xDC}
	},
	{
		2, {0x9F, 0xDB},
		3, {0xE6, 0xB8, 0xBA},
		2, {0x3A, 0x6E},
		2, {0xDE, 0xDD}
	},
	{
		2, {0x9F, 0xDC},
		3, {0xE6, 0xB9, 0x8E},
		2, {0x4E, 0x6E},
		2, {0xDE, 0xDE}
	},
	{
		2, {0x9F, 0xDD},
		3, {0xE6, 0xB8, 0xA4},
		2, {0x24, 0x6E},
		2, {0xDE, 0xDF}
	},
	{
		2, {0x9F, 0xDE},
		3, {0xE6, 0xBB, 0xBF},
		2, {0xFF, 0x6E},
		2, {0xDE, 0xE0}
	},
	{
		2, {0x9F, 0xDF},
		3, {0xE6, 0xB8, 0x9D},
		2, {0x1D, 0x6E},
		2, {0xDE, 0xE1}
	},
	{
		2, {0x9F, 0xE0},
		3, {0xE6, 0xB8, 0xB8},
		2, {0x38, 0x6E},
		2, {0xDE, 0xE2}
	},
	{
		2, {0x9F, 0xE1},
		3, {0xE6, 0xBA, 0x82},
		2, {0x82, 0x6E},
		2, {0xDE, 0xE3}
	},
	{
		2, {0x9F, 0xE2},
		3, {0xE6, 0xBA, 0xAA},
		2, {0xAA, 0x6E},
		2, {0xDE, 0xE4}
	},
	{
		2, {0x9F, 0xE3},
		3, {0xE6, 0xBA, 0x98},
		2, {0x98, 0x6E},
		2, {0xDE, 0xE5}
	},
	{
		2, {0x9F, 0xE4},
		3, {0xE6, 0xBB, 0x89},
		2, {0xC9, 0x6E},
		2, {0xDE, 0xE6}
	},
	{
		2, {0x9F, 0xE5},
		3, {0xE6, 0xBA, 0xB7},
		2, {0xB7, 0x6E},
		2, {0xDE, 0xE7}
	},
	{
		2, {0x9F, 0xE6},
		3, {0xE6, 0xBB, 0x93},
		2, {0xD3, 0x6E},
		2, {0xDE, 0xE8}
	},
	{
		2, {0x9F, 0xE7},
		3, {0xE6, 0xBA, 0xBD},
		2, {0xBD, 0x6E},
		2, {0xDE, 0xE9}
	},
	{
		2, {0x9F, 0xE8},
		3, {0xE6, 0xBA, 0xAF},
		2, {0xAF, 0x6E},
		2, {0xDE, 0xEA}
	},
	{
		2, {0x9F, 0xE9},
		3, {0xE6, 0xBB, 0x84},
		2, {0xC4, 0x6E},
		2, {0xDE, 0xEB}
	},
	{
		2, {0x9F, 0xEA},
		3, {0xE6, 0xBA, 0xB2},
		2, {0xB2, 0x6E},
		2, {0xDE, 0xEC}
	},
	{
		2, {0x9F, 0xEB},
		3, {0xE6, 0xBB, 0x94},
		2, {0xD4, 0x6E},
		2, {0xDE, 0xED}
	},
	{
		2, {0x9F, 0xEC},
		3, {0xE6, 0xBB, 0x95},
		2, {0xD5, 0x6E},
		2, {0xDE, 0xEE}
	},
	{
		2, {0x9F, 0xED},
		3, {0xE6, 0xBA, 0x8F},
		2, {0x8F, 0x6E},
		2, {0xDE, 0xEF}
	},
	{
		2, {0x9F, 0xEE},
		3, {0xE6, 0xBA, 0xA5},
		2, {0xA5, 0x6E},
		2, {0xDE, 0xF0}
	},
	{
		2, {0x9F, 0xEF},
		3, {0xE6, 0xBB, 0x82},
		2, {0xC2, 0x6E},
		2, {0xDE, 0xF1}
	},
	{
		2, {0x9F, 0xF0},
		3, {0xE6, 0xBA, 0x9F},
		2, {0x9F, 0x6E},
		2, {0xDE, 0xF2}
	},
	{
		2, {0x9F, 0xF1},
		3, {0xE6, 0xBD, 0x81},
		2, {0x41, 0x6F},
		2, {0xDE, 0xF3}
	},
	{
		2, {0x9F, 0xF2},
		3, {0xE6, 0xBC, 0x91},
		2, {0x11, 0x6F},
		2, {0xDE, 0xF4}
	},
	{
		2, {0x9F, 0xF3},
		3, {0xE7, 0x81, 0x8C},
		2, {0x4C, 0x70},
		2, {0xDE, 0xF5}
	},
	{
		2, {0x9F, 0xF4},
		3, {0xE6, 0xBB, 0xAC},
		2, {0xEC, 0x6E},
		2, {0xDE, 0xF6}
	},
	{
		2, {0x9F, 0xF5},
		3, {0xE6, 0xBB, 0xB8},
		2, {0xF8, 0x6E},
		2, {0xDE, 0xF7}
	},
	{
		2, {0x9F, 0xF6},
		3, {0xE6, 0xBB, 0xBE},
		2, {0xFE, 0x6E},
		2, {0xDE, 0xF8}
	},
	{
		2, {0x9F, 0xF7},
		3, {0xE6, 0xBC, 0xBF},
		2, {0x3F, 0x6F},
		2, {0xDE, 0xF9}
	},
	{
		2, {0x9F, 0xF8},
		3, {0xE6, 0xBB, 0xB2},
		2, {0xF2, 0x6E},
		2, {0xDE, 0xFA}
	},
	{
		2, {0x9F, 0xF9},
		3, {0xE6, 0xBC, 0xB1},
		2, {0x31, 0x6F},
		2, {0xDE, 0xFB}
	},
	{
		2, {0x9F, 0xFA},
		3, {0xE6, 0xBB, 0xAF},
		2, {0xEF, 0x6E},
		2, {0xDE, 0xFC}
	},
	{
		2, {0x9F, 0xFB},
		3, {0xE6, 0xBC, 0xB2},
		2, {0x32, 0x6F},
		2, {0xDE, 0xFD}
	},
	{
		2, {0x9F, 0xFC},
		3, {0xE6, 0xBB, 0x8C},
		2, {0xCC, 0x6E},
		2, {0xDE, 0xFE}
	},
	{
		2, {0xE0, 0x40},
		3, {0xE6, 0xBC, 0xBE},
		2, {0x3E, 0x6F},
		2, {0xDF, 0xA1}
	},
	{
		2, {0xE0, 0x41},
		3, {0xE6, 0xBC, 0x93},
		2, {0x13, 0x6F},
		2, {0xDF, 0xA2}
	},
	{
		2, {0xE0, 0x42},
		3, {0xE6, 0xBB, 0xB7},
		2, {0xF7, 0x6E},
		2, {0xDF, 0xA3}
	},
	{
		2, {0xE0, 0x43},
		3, {0xE6, 0xBE, 0x86},
		2, {0x86, 0x6F},
		2, {0xDF, 0xA4}
	},
	{
		2, {0xE0, 0x44},
		3, {0xE6, 0xBD, 0xBA},
		2, {0x7A, 0x6F},
		2, {0xDF, 0xA5}
	},
	{
		2, {0xE0, 0x45},
		3, {0xE6, 0xBD, 0xB8},
		2, {0x78, 0x6F},
		2, {0xDF, 0xA6}
	},
	{
		2, {0xE0, 0x46},
		3, {0xE6, 0xBE, 0x81},
		2, {0x81, 0x6F},
		2, {0xDF, 0xA7}
	},
	{
		2, {0xE0, 0x47},
		3, {0xE6, 0xBE, 0x80},
		2, {0x80, 0x6F},
		2, {0xDF, 0xA8}
	},
	{
		2, {0xE0, 0x48},
		3, {0xE6, 0xBD, 0xAF},
		2, {0x6F, 0x6F},
		2, {0xDF, 0xA9}
	},
	{
		2, {0xE0, 0x49},
		3, {0xE6, 0xBD, 0x9B},
		2, {0x5B, 0x6F},
		2, {0xDF, 0xAA}
	},
	{
		2, {0xE0, 0x4A},
		3, {0xE6, 0xBF, 0xB3},
		2, {0xF3, 0x6F},
		2, {0xDF, 0xAB}
	},
	{
		2, {0xE0, 0x4B},
		3, {0xE6, 0xBD, 0xAD},
		2, {0x6D, 0x6F},
		2, {0xDF, 0xAC}
	},
	{
		2, {0xE0, 0x4C},
		3, {0xE6, 0xBE, 0x82},
		2, {0x82, 0x6F},
		2, {0xDF, 0xAD}
	},
	{
		2, {0xE0, 0x4D},
		3, {0xE6, 0xBD, 0xBC},
		2, {0x7C, 0x6F},
		2, {0xDF, 0xAE}
	},
	{
		2, {0xE0, 0x4E},
		3, {0xE6, 0xBD, 0x98},
		2, {0x58, 0x6F},
		2, {0xDF, 0xAF}
	},
	{
		2, {0xE0, 0x4F},
		3, {0xE6, 0xBE, 0x8E},
		2, {0x8E, 0x6F},
		2, {0xDF, 0xB0}
	},
	{
		2, {0xE0, 0x50},
		3, {0xE6, 0xBE, 0x91},
		2, {0x91, 0x6F},
		2, {0xDF, 0xB1}
	},
	{
		2, {0xE0, 0x51},
		3, {0xE6, 0xBF, 0x82},
		2, {0xC2, 0x6F},
		2, {0xDF, 0xB2}
	},
	{
		2, {0xE0, 0x52},
		3, {0xE6, 0xBD, 0xA6},
		2, {0x66, 0x6F},
		2, {0xDF, 0xB3}
	},
	{
		2, {0xE0, 0x53},
		3, {0xE6, 0xBE, 0xB3},
		2, {0xB3, 0x6F},
		2, {0xDF, 0xB4}
	},
	{
		2, {0xE0, 0x54},
		3, {0xE6, 0xBE, 0xA3},
		2, {0xA3, 0x6F},
		2, {0xDF, 0xB5}
	},
	{
		2, {0xE0, 0x55},
		3, {0xE6, 0xBE, 0xA1},
		2, {0xA1, 0x6F},
		2, {0xDF, 0xB6}
	},
	{
		2, {0xE0, 0x56},
		3, {0xE6, 0xBE, 0xA4},
		2, {0xA4, 0x6F},
		2, {0xDF, 0xB7}
	},
	{
		2, {0xE0, 0x57},
		3, {0xE6, 0xBE, 0xB9},
		2, {0xB9, 0x6F},
		2, {0xDF, 0xB8}
	},
	{
		2, {0xE0, 0x58},
		3, {0xE6, 0xBF, 0x86},
		2, {0xC6, 0x6F},
		2, {0xDF, 0xB9}
	},
	{
		2, {0xE0, 0x59},
		3, {0xE6, 0xBE, 0xAA},
		2, {0xAA, 0x6F},
		2, {0xDF, 0xBA}
	},
	{
		2, {0xE0, 0x5A},
		3, {0xE6, 0xBF, 0x9F},
		2, {0xDF, 0x6F},
		2, {0xDF, 0xBB}
	},
	{
		2, {0xE0, 0x5B},
		3, {0xE6, 0xBF, 0x95},
		2, {0xD5, 0x6F},
		2, {0xDF, 0xBC}
	},
	{
		2, {0xE0, 0x5C},
		3, {0xE6, 0xBF, 0xAC},
		2, {0xEC, 0x6F},
		2, {0xDF, 0xBD}
	},
	{
		2, {0xE0, 0x5D},
		3, {0xE6, 0xBF, 0x94},
		2, {0xD4, 0x6F},
		2, {0xDF, 0xBE}
	},
	{
		2, {0xE0, 0x5E},
		3, {0xE6, 0xBF, 0x98},
		2, {0xD8, 0x6F},
		2, {0xDF, 0xBF}
	},
	{
		2, {0xE0, 0x5F},
		3, {0xE6, 0xBF, 0xB1},
		2, {0xF1, 0x6F},
		2, {0xDF, 0xC0}
	},
	{
		2, {0xE0, 0x60},
		3, {0xE6, 0xBF, 0xAE},
		2, {0xEE, 0x6F},
		2, {0xDF, 0xC1}
	},
	{
		2, {0xE0, 0x61},
		3, {0xE6, 0xBF, 0x9B},
		2, {0xDB, 0x6F},
		2, {0xDF, 0xC2}
	},
	{
		2, {0xE0, 0x62},
		3, {0xE7, 0x80, 0x89},
		2, {0x09, 0x70},
		2, {0xDF, 0xC3}
	},
	{
		2, {0xE0, 0x63},
		3, {0xE7, 0x80, 0x8B},
		2, {0x0B, 0x70},
		2, {0xDF, 0xC4}
	},
	{
		2, {0xE0, 0x64},
		3, {0xE6, 0xBF, 0xBA},
		2, {0xFA, 0x6F},
		2, {0xDF, 0xC5}
	},
	{
		2, {0xE0, 0x65},
		3, {0xE7, 0x80, 0x91},
		2, {0x11, 0x70},
		2, {0xDF, 0xC6}
	},
	{
		2, {0xE0, 0x66},
		3, {0xE7, 0x80, 0x81},
		2, {0x01, 0x70},
		2, {0xDF, 0xC7}
	},
	{
		2, {0xE0, 0x67},
		3, {0xE7, 0x80, 0x8F},
		2, {0x0F, 0x70},
		2, {0xDF, 0xC8}
	},
	{
		2, {0xE0, 0x68},
		3, {0xE6, 0xBF, 0xBE},
		2, {0xFE, 0x6F},
		2, {0xDF, 0xC9}
	},
	{
		2, {0xE0, 0x69},
		3, {0xE7, 0x80, 0x9B},
		2, {0x1B, 0x70},
		2, {0xDF, 0xCA}
	},
	{
		2, {0xE0, 0x6A},
		3, {0xE7, 0x80, 0x9A},
		2, {0x1A, 0x70},
		2, {0xDF, 0xCB}
	},
	{
		2, {0xE0, 0x6B},
		3, {0xE6, 0xBD, 0xB4},
		2, {0x74, 0x6F},
		2, {0xDF, 0xCC}
	},
	{
		2, {0xE0, 0x6C},
		3, {0xE7, 0x80, 0x9D},
		2, {0x1D, 0x70},
		2, {0xDF, 0xCD}
	},
	{
		2, {0xE0, 0x6D},
		3, {0xE7, 0x80, 0x98},
		2, {0x18, 0x70},
		2, {0xDF, 0xCE}
	},
	{
		2, {0xE0, 0x6E},
		3, {0xE7, 0x80, 0x9F},
		2, {0x1F, 0x70},
		2, {0xDF, 0xCF}
	},
	{
		2, {0xE0, 0x6F},
		3, {0xE7, 0x80, 0xB0},
		2, {0x30, 0x70},
		2, {0xDF, 0xD0}
	},
	{
		2, {0xE0, 0x70},
		3, {0xE7, 0x80, 0xBE},
		2, {0x3E, 0x70},
		2, {0xDF, 0xD1}
	},
	{
		2, {0xE0, 0x71},
		3, {0xE7, 0x80, 0xB2},
		2, {0x32, 0x70},
		2, {0xDF, 0xD2}
	},
	{
		2, {0xE0, 0x72},
		3, {0xE7, 0x81, 0x91},
		2, {0x51, 0x70},
		2, {0xDF, 0xD3}
	},
	{
		2, {0xE0, 0x73},
		3, {0xE7, 0x81, 0xA3},
		2, {0x63, 0x70},
		2, {0xDF, 0xD4}
	},
	{
		2, {0xE0, 0x74},
		3, {0xE7, 0x82, 0x99},
		2, {0x99, 0x70},
		2, {0xDF, 0xD5}
	},
	{
		2, {0xE0, 0x75},
		3, {0xE7, 0x82, 0x92},
		2, {0x92, 0x70},
		2, {0xDF, 0xD6}
	},
	{
		2, {0xE0, 0x76},
		3, {0xE7, 0x82, 0xAF},
		2, {0xAF, 0x70},
		2, {0xDF, 0xD7}
	},
	{
		2, {0xE0, 0x77},
		3, {0xE7, 0x83, 0xB1},
		2, {0xF1, 0x70},
		2, {0xDF, 0xD8}
	},
	{
		2, {0xE0, 0x78},
		3, {0xE7, 0x82, 0xAC},
		2, {0xAC, 0x70},
		2, {0xDF, 0xD9}
	},
	{
		2, {0xE0, 0x79},
		3, {0xE7, 0x82, 0xB8},
		2, {0xB8, 0x70},
		2, {0xDF, 0xDA}
	},
	{
		2, {0xE0, 0x7A},
		3, {0xE7, 0x82, 0xB3},
		2, {0xB3, 0x70},
		2, {0xDF, 0xDB}
	},
	{
		2, {0xE0, 0x7B},
		3, {0xE7, 0x82, 0xAE},
		2, {0xAE, 0x70},
		2, {0xDF, 0xDC}
	},
	{
		2, {0xE0, 0x7C},
		3, {0xE7, 0x83, 0x9F},
		2, {0xDF, 0x70},
		2, {0xDF, 0xDD}
	},
	{
		2, {0xE0, 0x7D},
		3, {0xE7, 0x83, 0x8B},
		2, {0xCB, 0x70},
		2, {0xDF, 0xDE}
	},
	{
		2, {0xE0, 0x7E},
		3, {0xE7, 0x83, 0x9D},
		2, {0xDD, 0x70},
		2, {0xDF, 0xDF}
	},
	{
		2, {0xE0, 0x80},
		3, {0xE7, 0x83, 0x99},
		2, {0xD9, 0x70},
		2, {0xDF, 0xE0}
	},
	{
		2, {0xE0, 0x81},
		3, {0xE7, 0x84, 0x89},
		2, {0x09, 0x71},
		2, {0xDF, 0xE1}
	},
	{
		2, {0xE0, 0x82},
		3, {0xE7, 0x83, 0xBD},
		2, {0xFD, 0x70},
		2, {0xDF, 0xE2}
	},
	{
		2, {0xE0, 0x83},
		3, {0xE7, 0x84, 0x9C},
		2, {0x1C, 0x71},
		2, {0xDF, 0xE3}
	},
	{
		2, {0xE0, 0x84},
		3, {0xE7, 0x84, 0x99},
		2, {0x19, 0x71},
		2, {0xDF, 0xE4}
	},
	{
		2, {0xE0, 0x85},
		3, {0xE7, 0x85, 0xA5},
		2, {0x65, 0x71},
		2, {0xDF, 0xE5}
	},
	{
		2, {0xE0, 0x86},
		3, {0xE7, 0x85, 0x95},
		2, {0x55, 0x71},
		2, {0xDF, 0xE6}
	},
	{
		2, {0xE0, 0x87},
		3, {0xE7, 0x86, 0x88},
		2, {0x88, 0x71},
		2, {0xDF, 0xE7}
	},
	{
		2, {0xE0, 0x88},
		3, {0xE7, 0x85, 0xA6},
		2, {0x66, 0x71},
		2, {0xDF, 0xE8}
	},
	{
		2, {0xE0, 0x89},
		3, {0xE7, 0x85, 0xA2},
		2, {0x62, 0x71},
		2, {0xDF, 0xE9}
	},
	{
		2, {0xE0, 0x8A},
		3, {0xE7, 0x85, 0x8C},
		2, {0x4C, 0x71},
		2, {0xDF, 0xEA}
	},
	{
		2, {0xE0, 0x8B},
		3, {0xE7, 0x85, 0x96},
		2, {0x56, 0x71},
		2, {0xDF, 0xEB}
	},
	{
		2, {0xE0, 0x8C},
		3, {0xE7, 0x85, 0xAC},
		2, {0x6C, 0x71},
		2, {0xDF, 0xEC}
	},
	{
		2, {0xE0, 0x8D},
		3, {0xE7, 0x86, 0x8F},
		2, {0x8F, 0x71},
		2, {0xDF, 0xED}
	},
	{
		2, {0xE0, 0x8E},
		3, {0xE7, 0x87, 0xBB},
		2, {0xFB, 0x71},
		2, {0xDF, 0xEE}
	},
	{
		2, {0xE0, 0x8F},
		3, {0xE7, 0x86, 0x84},
		2, {0x84, 0x71},
		2, {0xDF, 0xEF}
	},
	{
		2, {0xE0, 0x90},
		3, {0xE7, 0x86, 0x95},
		2, {0x95, 0x71},
		2, {0xDF, 0xF0}
	},
	{
		2, {0xE0, 0x91},
		3, {0xE7, 0x86, 0xA8},
		2, {0xA8, 0x71},
		2, {0xDF, 0xF1}
	},
	{
		2, {0xE0, 0x92},
		3, {0xE7, 0x86, 0xAC},
		2, {0xAC, 0x71},
		2, {0xDF, 0xF2}
	},
	{
		2, {0xE0, 0x93},
		3, {0xE7, 0x87, 0x97},
		2, {0xD7, 0x71},
		2, {0xDF, 0xF3}
	},
	{
		2, {0xE0, 0x94},
		3, {0xE7, 0x86, 0xB9},
		2, {0xB9, 0x71},
		2, {0xDF, 0xF4}
	},
	{
		2, {0xE0, 0x95},
		3, {0xE7, 0x86, 0xBE},
		2, {0xBE, 0x71},
		2, {0xDF, 0xF5}
	},
	{
		2, {0xE0, 0x96},
		3, {0xE7, 0x87, 0x92},
		2, {0xD2, 0x71},
		2, {0xDF, 0xF6}
	},
	{
		2, {0xE0, 0x97},
		3, {0xE7, 0x87, 0x89},
		2, {0xC9, 0x71},
		2, {0xDF, 0xF7}
	},
	{
		2, {0xE0, 0x98},
		3, {0xE7, 0x87, 0x94},
		2, {0xD4, 0x71},
		2, {0xDF, 0xF8}
	},
	{
		2, {0xE0, 0x99},
		3, {0xE7, 0x87, 0x8E},
		2, {0xCE, 0x71},
		2, {0xDF, 0xF9}
	},
	{
		2, {0xE0, 0x9A},
		3, {0xE7, 0x87, 0xA0},
		2, {0xE0, 0x71},
		2, {0xDF, 0xFA}
	},
	{
		2, {0xE0, 0x9B},
		3, {0xE7, 0x87, 0xAC},
		2, {0xEC, 0x71},
		2, {0xDF, 0xFB}
	},
	{
		2, {0xE0, 0x9C},
		3, {0xE7, 0x87, 0xA7},
		2, {0xE7, 0x71},
		2, {0xDF, 0xFC}
	},
	{
		2, {0xE0, 0x9D},
		3, {0xE7, 0x87, 0xB5},
		2, {0xF5, 0x71},
		2, {0xDF, 0xFD}
	},
	{
		2, {0xE0, 0x9E},
		3, {0xE7, 0x87, 0xBC},
		2, {0xFC, 0x71},
		2, {0xDF, 0xFE}
	},
	{
		2, {0xE0, 0x9F},
		3, {0xE7, 0x87, 0xB9},
		2, {0xF9, 0x71},
		2, {0xE0, 0xA1}
	},
	{
		2, {0xE0, 0xA0},
		3, {0xE7, 0x87, 0xBF},
		2, {0xFF, 0x71},
		2, {0xE0, 0xA2}
	},
	{
		2, {0xE0, 0xA1},
		3, {0xE7, 0x88, 0x8D},
		2, {0x0D, 0x72},
		2, {0xE0, 0xA3}
	},
	{
		2, {0xE0, 0xA2},
		3, {0xE7, 0x88, 0x90},
		2, {0x10, 0x72},
		2, {0xE0, 0xA4}
	},
	{
		2, {0xE0, 0xA3},
		3, {0xE7, 0x88, 0x9B},
		2, {0x1B, 0x72},
		2, {0xE0, 0xA5}
	},
	{
		2, {0xE0, 0xA4},
		3, {0xE7, 0x88, 0xA8},
		2, {0x28, 0x72},
		2, {0xE0, 0xA6}
	},
	{
		2, {0xE0, 0xA5},
		3, {0xE7, 0x88, 0xAD},
		2, {0x2D, 0x72},
		2, {0xE0, 0xA7}
	},
	{
		2, {0xE0, 0xA6},
		3, {0xE7, 0x88, 0xAC},
		2, {0x2C, 0x72},
		2, {0xE0, 0xA8}
	},
	{
		2, {0xE0, 0xA7},
		3, {0xE7, 0x88, 0xB0},
		2, {0x30, 0x72},
		2, {0xE0, 0xA9}
	},
	{
		2, {0xE0, 0xA8},
		3, {0xE7, 0x88, 0xB2},
		2, {0x32, 0x72},
		2, {0xE0, 0xAA}
	},
	{
		2, {0xE0, 0xA9},
		3, {0xE7, 0x88, 0xBB},
		2, {0x3B, 0x72},
		2, {0xE0, 0xAB}
	},
	{
		2, {0xE0, 0xAA},
		3, {0xE7, 0x88, 0xBC},
		2, {0x3C, 0x72},
		2, {0xE0, 0xAC}
	},
	{
		2, {0xE0, 0xAB},
		3, {0xE7, 0x88, 0xBF},
		2, {0x3F, 0x72},
		2, {0xE0, 0xAD}
	},
	{
		2, {0xE0, 0xAC},
		3, {0xE7, 0x89, 0x80},
		2, {0x40, 0x72},
		2, {0xE0, 0xAE}
	},
	{
		2, {0xE0, 0xAD},
		3, {0xE7, 0x89, 0x86},
		2, {0x46, 0x72},
		2, {0xE0, 0xAF}
	},
	{
		2, {0xE0, 0xAE},
		3, {0xE7, 0x89, 0x8B},
		2, {0x4B, 0x72},
		2, {0xE0, 0xB0}
	},
	{
		2, {0xE0, 0xAF},
		3, {0xE7, 0x89, 0x98},
		2, {0x58, 0x72},
		2, {0xE0, 0xB1}
	},
	{
		2, {0xE0, 0xB0},
		3, {0xE7, 0x89, 0xB4},
		2, {0x74, 0x72},
		2, {0xE0, 0xB2}
	},
	{
		2, {0xE0, 0xB1},
		3, {0xE7, 0x89, 0xBE},
		2, {0x7E, 0x72},
		2, {0xE0, 0xB3}
	},
	{
		2, {0xE0, 0xB2},
		3, {0xE7, 0x8A, 0x82},
		2, {0x82, 0x72},
		2, {0xE0, 0xB4}
	},
	{
		2, {0xE0, 0xB3},
		3, {0xE7, 0x8A, 0x81},
		2, {0x81, 0x72},
		2, {0xE0, 0xB5}
	},
	{
		2, {0xE0, 0xB4},
		3, {0xE7, 0x8A, 0x87},
		2, {0x87, 0x72},
		2, {0xE0, 0xB6}
	},
	{
		2, {0xE0, 0xB5},
		3, {0xE7, 0x8A, 0x92},
		2, {0x92, 0x72},
		2, {0xE0, 0xB7}
	},
	{
		2, {0xE0, 0xB6},
		3, {0xE7, 0x8A, 0x96},
		2, {0x96, 0x72},
		2, {0xE0, 0xB8}
	},
	{
		2, {0xE0, 0xB7},
		3, {0xE7, 0x8A, 0xA2},
		2, {0xA2, 0x72},
		2, {0xE0, 0xB9}
	},
	{
		2, {0xE0, 0xB8},
		3, {0xE7, 0x8A, 0xA7},
		2, {0xA7, 0x72},
		2, {0xE0, 0xBA}
	},
	{
		2, {0xE0, 0xB9},
		3, {0xE7, 0x8A, 0xB9},
		2, {0xB9, 0x72},
		2, {0xE0, 0xBB}
	},
	{
		2, {0xE0, 0xBA},
		3, {0xE7, 0x8A, 0xB2},
		2, {0xB2, 0x72},
		2, {0xE0, 0xBC}
	},
	{
		2, {0xE0, 0xBB},
		3, {0xE7, 0x8B, 0x83},
		2, {0xC3, 0x72},
		2, {0xE0, 0xBD}
	},
	{
		2, {0xE0, 0xBC},
		3, {0xE7, 0x8B, 0x86},
		2, {0xC6, 0x72},
		2, {0xE0, 0xBE}
	},
	{
		2, {0xE0, 0xBD},
		3, {0xE7, 0x8B, 0x84},
		2, {0xC4, 0x72},
		2, {0xE0, 0xBF}
	},
	{
		2, {0xE0, 0xBE},
		3, {0xE7, 0x8B, 0x8E},
		2, {0xCE, 0x72},
		2, {0xE0, 0xC0}
	},
	{
		2, {0xE0, 0xBF},
		3, {0xE7, 0x8B, 0x92},
		2, {0xD2, 0x72},
		2, {0xE0, 0xC1}
	},
	{
		2, {0xE0, 0xC0},
		3, {0xE7, 0x8B, 0xA2},
		2, {0xE2, 0x72},
		2, {0xE0, 0xC2}
	},
	{
		2, {0xE0, 0xC1},
		3, {0xE7, 0x8B, 0xA0},
		2, {0xE0, 0x72},
		2, {0xE0, 0xC3}
	},
	{
		2, {0xE0, 0xC2},
		3, {0xE7, 0x8B, 0xA1},
		2, {0xE1, 0x72},
		2, {0xE0, 0xC4}
	},
	{
		2, {0xE0, 0xC3},
		3, {0xE7, 0x8B, 0xB9},
		2, {0xF9, 0x72},
		2, {0xE0, 0xC5}
	},
	{
		2, {0xE0, 0xC4},
		3, {0xE7, 0x8B, 0xB7},
		2, {0xF7, 0x72},
		2, {0xE0, 0xC6}
	},
	{
		2, {0xE0, 0xC5},
		3, {0xE5, 0x80, 0x8F},
		2, {0x0F, 0x50},
		2, {0xE0, 0xC7}
	},
	{
		2, {0xE0, 0xC6},
		3, {0xE7, 0x8C, 0x97},
		2, {0x17, 0x73},
		2, {0xE0, 0xC8}
	},
	{
		2, {0xE0, 0xC7},
		3, {0xE7, 0x8C, 0x8A},
		2, {0x0A, 0x73},
		2, {0xE0, 0xC9}
	},
	{
		2, {0xE0, 0xC8},
		3, {0xE7, 0x8C, 0x9C},
		2, {0x1C, 0x73},
		2, {0xE0, 0xCA}
	},
	{
		2, {0xE0, 0xC9},
		3, {0xE7, 0x8C, 0x96},
		2, {0x16, 0x73},
		2, {0xE0, 0xCB}
	},
	{
		2, {0xE0, 0xCA},
		3, {0xE7, 0x8C, 0x9D},
		2, {0x1D, 0x73},
		2, {0xE0, 0xCC}
	},
	{
		2, {0xE0, 0xCB},
		3, {0xE7, 0x8C, 0xB4},
		2, {0x34, 0x73},
		2, {0xE0, 0xCD}
	},
	{
		2, {0xE0, 0xCC},
		3, {0xE7, 0x8C, 0xAF},
		2, {0x2F, 0x73},
		2, {0xE0, 0xCE}
	},
	{
		2, {0xE0, 0xCD},
		3, {0xE7, 0x8C, 0xA9},
		2, {0x29, 0x73},
		2, {0xE0, 0xCF}
	},
	{
		2, {0xE0, 0xCE},
		3, {0xE7, 0x8C, 0xA5},
		2, {0x25, 0x73},
		2, {0xE0, 0xD0}
	},
	{
		2, {0xE0, 0xCF},
		3, {0xE7, 0x8C, 0xBE},
		2, {0x3E, 0x73},
		2, {0xE0, 0xD1}
	},
	{
		2, {0xE0, 0xD0},
		3, {0xE7, 0x8D, 0x8E},
		2, {0x4E, 0x73},
		2, {0xE0, 0xD2}
	},
	{
		2, {0xE0, 0xD1},
		3, {0xE7, 0x8D, 0x8F},
		2, {0x4F, 0x73},
		2, {0xE0, 0xD3}
	},
	{
		2, {0xE0, 0xD2},
		3, {0xE9, 0xBB, 0x98},
		2, {0xD8, 0x9E},
		2, {0xE0, 0xD4}
	},
	{
		2, {0xE0, 0xD3},
		3, {0xE7, 0x8D, 0x97},
		2, {0x57, 0x73},
		2, {0xE0, 0xD5}
	},
	{
		2, {0xE0, 0xD4},
		3, {0xE7, 0x8D, 0xAA},
		2, {0x6A, 0x73},
		2, {0xE0, 0xD6}
	},
	{
		2, {0xE0, 0xD5},
		3, {0xE7, 0x8D, 0xA8},
		2, {0x68, 0x73},
		2, {0xE0, 0xD7}
	},
	{
		2, {0xE0, 0xD6},
		3, {0xE7, 0x8D, 0xB0},
		2, {0x70, 0x73},
		2, {0xE0, 0xD8}
	},
	{
		2, {0xE0, 0xD7},
		3, {0xE7, 0x8D, 0xB8},
		2, {0x78, 0x73},
		2, {0xE0, 0xD9}
	},
	{
		2, {0xE0, 0xD8},
		3, {0xE7, 0x8D, 0xB5},
		2, {0x75, 0x73},
		2, {0xE0, 0xDA}
	},
	{
		2, {0xE0, 0xD9},
		3, {0xE7, 0x8D, 0xBB},
		2, {0x7B, 0x73},
		2, {0xE0, 0xDB}
	},
	{
		2, {0xE0, 0xDA},
		3, {0xE7, 0x8D, 0xBA},
		2, {0x7A, 0x73},
		2, {0xE0, 0xDC}
	},
	{
		2, {0xE0, 0xDB},
		3, {0xE7, 0x8F, 0x88},
		2, {0xC8, 0x73},
		2, {0xE0, 0xDD}
	},
	{
		2, {0xE0, 0xDC},
		3, {0xE7, 0x8E, 0xB3},
		2, {0xB3, 0x73},
		2, {0xE0, 0xDE}
	},
	{
		2, {0xE0, 0xDD},
		3, {0xE7, 0x8F, 0x8E},
		2, {0xCE, 0x73},
		2, {0xE0, 0xDF}
	},
	{
		2, {0xE0, 0xDE},
		3, {0xE7, 0x8E, 0xBB},
		2, {0xBB, 0x73},
		2, {0xE0, 0xE0}
	},
	{
		2, {0xE0, 0xDF},
		3, {0xE7, 0x8F, 0x80},
		2, {0xC0, 0x73},
		2, {0xE0, 0xE1}
	},
	{
		2, {0xE0, 0xE0},
		3, {0xE7, 0x8F, 0xA5},
		2, {0xE5, 0x73},
		2, {0xE0, 0xE2}
	},
	{
		2, {0xE0, 0xE1},
		3, {0xE7, 0x8F, 0xAE},
		2, {0xEE, 0x73},
		2, {0xE0, 0xE3}
	},
	{
		2, {0xE0, 0xE2},
		3, {0xE7, 0x8F, 0x9E},
		2, {0xDE, 0x73},
		2, {0xE0, 0xE4}
	},
	{
		2, {0xE0, 0xE3},
		3, {0xE7, 0x92, 0xA2},
		2, {0xA2, 0x74},
		2, {0xE0, 0xE5}
	},
	{
		2, {0xE0, 0xE4},
		3, {0xE7, 0x90, 0x85},
		2, {0x05, 0x74},
		2, {0xE0, 0xE6}
	},
	{
		2, {0xE0, 0xE5},
		3, {0xE7, 0x91, 0xAF},
		2, {0x6F, 0x74},
		2, {0xE0, 0xE7}
	},
	{
		2, {0xE0, 0xE6},
		3, {0xE7, 0x90, 0xA5},
		2, {0x25, 0x74},
		2, {0xE0, 0xE8}
	},
	{
		2, {0xE0, 0xE7},
		3, {0xE7, 0x8F, 0xB8},
		2, {0xF8, 0x73},
		2, {0xE0, 0xE9}
	},
	{
		2, {0xE0, 0xE8},
		3, {0xE7, 0x90, 0xB2},
		2, {0x32, 0x74},
		2, {0xE0, 0xEA}
	},
	{
		2, {0xE0, 0xE9},
		3, {0xE7, 0x90, 0xBA},
		2, {0x3A, 0x74},
		2, {0xE0, 0xEB}
	},
	{
		2, {0xE0, 0xEA},
		3, {0xE7, 0x91, 0x95},
		2, {0x55, 0x74},
		2, {0xE0, 0xEC}
	},
	{
		2, {0xE0, 0xEB},
		3, {0xE7, 0x90, 0xBF},
		2, {0x3F, 0x74},
		2, {0xE0, 0xED}
	},
	{
		2, {0xE0, 0xEC},
		3, {0xE7, 0x91, 0x9F},
		2, {0x5F, 0x74},
		2, {0xE0, 0xEE}
	},
	{
		2, {0xE0, 0xED},
		3, {0xE7, 0x91, 0x99},
		2, {0x59, 0x74},
		2, {0xE0, 0xEF}
	},
	{
		2, {0xE0, 0xEE},
		3, {0xE7, 0x91, 0x81},
		2, {0x41, 0x74},
		2, {0xE0, 0xF0}
	},
	{
		2, {0xE0, 0xEF},
		3, {0xE7, 0x91, 0x9C},
		2, {0x5C, 0x74},
		2, {0xE0, 0xF1}
	},
	{
		2, {0xE0, 0xF0},
		3, {0xE7, 0x91, 0xA9},
		2, {0x69, 0x74},
		2, {0xE0, 0xF2}
	},
	{
		2, {0xE0, 0xF1},
		3, {0xE7, 0x91, 0xB0},
		2, {0x70, 0x74},
		2, {0xE0, 0xF3}
	},
	{
		2, {0xE0, 0xF2},
		3, {0xE7, 0x91, 0xA3},
		2, {0x63, 0x74},
		2, {0xE0, 0xF4}
	},
	{
		2, {0xE0, 0xF3},
		3, {0xE7, 0x91, 0xAA},
		2, {0x6A, 0x74},
		2, {0xE0, 0xF5}
	},
	{
		2, {0xE0, 0xF4},
		3, {0xE7, 0x91, 0xB6},
		2, {0x76, 0x74},
		2, {0xE0, 0xF6}
	},
	{
		2, {0xE0, 0xF5},
		3, {0xE7, 0x91, 0xBE},
		2, {0x7E, 0x74},
		2, {0xE0, 0xF7}
	},
	{
		2, {0xE0, 0xF6},
		3, {0xE7, 0x92, 0x8B},
		2, {0x8B, 0x74},
		2, {0xE0, 0xF8}
	},
	{
		2, {0xE0, 0xF7},
		3, {0xE7, 0x92, 0x9E},
		2, {0x9E, 0x74},
		2, {0xE0, 0xF9}
	},
	{
		2, {0xE0, 0xF8},
		3, {0xE7, 0x92, 0xA7},
		2, {0xA7, 0x74},
		2, {0xE0, 0xFA}
	},
	{
		2, {0xE0, 0xF9},
		3, {0xE7, 0x93, 0x8A},
		2, {0xCA, 0x74},
		2, {0xE0, 0xFB}
	},
	{
		2, {0xE0, 0xFA},
		3, {0xE7, 0x93, 0x8F},
		2, {0xCF, 0x74},
		2, {0xE0, 0xFC}
	},
	{
		2, {0xE0, 0xFB},
		3, {0xE7, 0x93, 0x94},
		2, {0xD4, 0x74},
		2, {0xE0, 0xFD}
	},
	{
		2, {0xE0, 0xFC},
		3, {0xE7, 0x8F, 0xB1},
		2, {0xF1, 0x73},
		2, {0xE0, 0xFE}
	},
	{
		2, {0xE1, 0x40},
		3, {0xE7, 0x93, 0xA0},
		2, {0xE0, 0x74},
		2, {0xE1, 0xA1}
	},
	{
		2, {0xE1, 0x41},
		3, {0xE7, 0x93, 0xA3},
		2, {0xE3, 0x74},
		2, {0xE1, 0xA2}
	},
	{
		2, {0xE1, 0x42},
		3, {0xE7, 0x93, 0xA7},
		2, {0xE7, 0x74},
		2, {0xE1, 0xA3}
	},
	{
		2, {0xE1, 0x43},
		3, {0xE7, 0x93, 0xA9},
		2, {0xE9, 0x74},
		2, {0xE1, 0xA4}
	},
	{
		2, {0xE1, 0x44},
		3, {0xE7, 0x93, 0xAE},
		2, {0xEE, 0x74},
		2, {0xE1, 0xA5}
	},
	{
		2, {0xE1, 0x45},
		3, {0xE7, 0x93, 0xB2},
		2, {0xF2, 0x74},
		2, {0xE1, 0xA6}
	},
	{
		2, {0xE1, 0x46},
		3, {0xE7, 0x93, 0xB0},
		2, {0xF0, 0x74},
		2, {0xE1, 0xA7}
	},
	{
		2, {0xE1, 0x47},
		3, {0xE7, 0x93, 0xB1},
		2, {0xF1, 0x74},
		2, {0xE1, 0xA8}
	},
	{
		2, {0xE1, 0x48},
		3, {0xE7, 0x93, 0xB8},
		2, {0xF8, 0x74},
		2, {0xE1, 0xA9}
	},
	{
		2, {0xE1, 0x49},
		3, {0xE7, 0x93, 0xB7},
		2, {0xF7, 0x74},
		2, {0xE1, 0xAA}
	},
	{
		2, {0xE1, 0x4A},
		3, {0xE7, 0x94, 0x84},
		2, {0x04, 0x75},
		2, {0xE1, 0xAB}
	},
	{
		2, {0xE1, 0x4B},
		3, {0xE7, 0x94, 0x83},
		2, {0x03, 0x75},
		2, {0xE1, 0xAC}
	},
	{
		2, {0xE1, 0x4C},
		3, {0xE7, 0x94, 0x85},
		2, {0x05, 0x75},
		2, {0xE1, 0xAD}
	},
	{
		2, {0xE1, 0x4D},
		3, {0xE7, 0x94, 0x8C},
		2, {0x0C, 0x75},
		2, {0xE1, 0xAE}
	},
	{
		2, {0xE1, 0x4E},
		3, {0xE7, 0x94, 0x8E},
		2, {0x0E, 0x75},
		2, {0xE1, 0xAF}
	},
	{
		2, {0xE1, 0x4F},
		3, {0xE7, 0x94, 0x8D},
		2, {0x0D, 0x75},
		2, {0xE1, 0xB0}
	},
	{
		2, {0xE1, 0x50},
		3, {0xE7, 0x94, 0x95},
		2, {0x15, 0x75},
		2, {0xE1, 0xB1}
	},
	{
		2, {0xE1, 0x51},
		3, {0xE7, 0x94, 0x93},
		2, {0x13, 0x75},
		2, {0xE1, 0xB2}
	},
	{
		2, {0xE1, 0x52},
		3, {0xE7, 0x94, 0x9E},
		2, {0x1E, 0x75},
		2, {0xE1, 0xB3}
	},
	{
		2, {0xE1, 0x53},
		3, {0xE7, 0x94, 0xA6},
		2, {0x26, 0x75},
		2, {0xE1, 0xB4}
	},
	{
		2, {0xE1, 0x54},
		3, {0xE7, 0x94, 0xAC},
		2, {0x2C, 0x75},
		2, {0xE1, 0xB5}
	},
	{
		2, {0xE1, 0x55},
		3, {0xE7, 0x94, 0xBC},
		2, {0x3C, 0x75},
		2, {0xE1, 0xB6}
	},
	{
		2, {0xE1, 0x56},
		3, {0xE7, 0x95, 0x84},
		2, {0x44, 0x75},
		2, {0xE1, 0xB7}
	},
	{
		2, {0xE1, 0x57},
		3, {0xE7, 0x95, 0x8D},
		2, {0x4D, 0x75},
		2, {0xE1, 0xB8}
	},
	{
		2, {0xE1, 0x58},
		3, {0xE7, 0x95, 0x8A},
		2, {0x4A, 0x75},
		2, {0xE1, 0xB9}
	},
	{
		2, {0xE1, 0x59},
		3, {0xE7, 0x95, 0x89},
		2, {0x49, 0x75},
		2, {0xE1, 0xBA}
	},
	{
		2, {0xE1, 0x5A},
		3, {0xE7, 0x95, 0x9B},
		2, {0x5B, 0x75},
		2, {0xE1, 0xBB}
	},
	{
		2, {0xE1, 0x5B},
		3, {0xE7, 0x95, 0x86},
		2, {0x46, 0x75},
		2, {0xE1, 0xBC}
	},
	{
		2, {0xE1, 0x5C},
		3, {0xE7, 0x95, 0x9A},
		2, {0x5A, 0x75},
		2, {0xE1, 0xBD}
	},
	{
		2, {0xE1, 0x5D},
		3, {0xE7, 0x95, 0xA9},
		2, {0x69, 0x75},
		2, {0xE1, 0xBE}
	},
	{
		2, {0xE1, 0x5E},
		3, {0xE7, 0x95, 0xA4},
		2, {0x64, 0x75},
		2, {0xE1, 0xBF}
	},
	{
		2, {0xE1, 0x5F},
		3, {0xE7, 0x95, 0xA7},
		2, {0x67, 0x75},
		2, {0xE1, 0xC0}
	},
	{
		2, {0xE1, 0x60},
		3, {0xE7, 0x95, 0xAB},
		2, {0x6B, 0x75},
		2, {0xE1, 0xC1}
	},
	{
		2, {0xE1, 0x61},
		3, {0xE7, 0x95, 0xAD},
		2, {0x6D, 0x75},
		2, {0xE1, 0xC2}
	},
	{
		2, {0xE1, 0x62},
		3, {0xE7, 0x95, 0xB8},
		2, {0x78, 0x75},
		2, {0xE1, 0xC3}
	},
	{
		2, {0xE1, 0x63},
		3, {0xE7, 0x95, 0xB6},
		2, {0x76, 0x75},
		2, {0xE1, 0xC4}
	},
	{
		2, {0xE1, 0x64},
		3, {0xE7, 0x96, 0x86},
		2, {0x86, 0x75},
		2, {0xE1, 0xC5}
	},
	{
		2, {0xE1, 0x65},
		3, {0xE7, 0x96, 0x87},
		2, {0x87, 0x75},
		2, {0xE1, 0xC6}
	},
	{
		2, {0xE1, 0x66},
		3, {0xE7, 0x95, 0xB4},
		2, {0x74, 0x75},
		2, {0xE1, 0xC7}
	},
	{
		2, {0xE1, 0x67},
		3, {0xE7, 0x96, 0x8A},
		2, {0x8A, 0x75},
		2, {0xE1, 0xC8}
	},
	{
		2, {0xE1, 0x68},
		3, {0xE7, 0x96, 0x89},
		2, {0x89, 0x75},
		2, {0xE1, 0xC9}
	},
	{
		2, {0xE1, 0x69},
		3, {0xE7, 0x96, 0x82},
		2, {0x82, 0x75},
		2, {0xE1, 0xCA}
	},
	{
		2, {0xE1, 0x6A},
		3, {0xE7, 0x96, 0x94},
		2, {0x94, 0x75},
		2, {0xE1, 0xCB}
	},
	{
		2, {0xE1, 0x6B},
		3, {0xE7, 0x96, 0x9A},
		2, {0x9A, 0x75},
		2, {0xE1, 0xCC}
	},
	{
		2, {0xE1, 0x6C},
		3, {0xE7, 0x96, 0x9D},
		2, {0x9D, 0x75},
		2, {0xE1, 0xCD}
	},
	{
		2, {0xE1, 0x6D},
		3, {0xE7, 0x96, 0xA5},
		2, {0xA5, 0x75},
		2, {0xE1, 0xCE}
	},
	{
		2, {0xE1, 0x6E},
		3, {0xE7, 0x96, 0xA3},
		2, {0xA3, 0x75},
		2, {0xE1, 0xCF}
	},
	{
		2, {0xE1, 0x6F},
		3, {0xE7, 0x97, 0x82},
		2, {0xC2, 0x75},
		2, {0xE1, 0xD0}
	},
	{
		2, {0xE1, 0x70},
		3, {0xE7, 0x96, 0xB3},
		2, {0xB3, 0x75},
		2, {0xE1, 0xD1}
	},
	{
		2, {0xE1, 0x71},
		3, {0xE7, 0x97, 0x83},
		2, {0xC3, 0x75},
		2, {0xE1, 0xD2}
	},
	{
		2, {0xE1, 0x72},
		3, {0xE7, 0x96, 0xB5},
		2, {0xB5, 0x75},
		2, {0xE1, 0xD3}
	},
	{
		2, {0xE1, 0x73},
		3, {0xE7, 0x96, 0xBD},
		2, {0xBD, 0x75},
		2, {0xE1, 0xD4}
	},
	{
		2, {0xE1, 0x74},
		3, {0xE7, 0x96, 0xB8},
		2, {0xB8, 0x75},
		2, {0xE1, 0xD5}
	},
	{
		2, {0xE1, 0x75},
		3, {0xE7, 0x96, 0xBC},
		2, {0xBC, 0x75},
		2, {0xE1, 0xD6}
	},
	{
		2, {0xE1, 0x76},
		3, {0xE7, 0x96, 0xB1},
		2, {0xB1, 0x75},
		2, {0xE1, 0xD7}
	},
	{
		2, {0xE1, 0x77},
		3, {0xE7, 0x97, 0x8D},
		2, {0xCD, 0x75},
		2, {0xE1, 0xD8}
	},
	{
		2, {0xE1, 0x78},
		3, {0xE7, 0x97, 0x8A},
		2, {0xCA, 0x75},
		2, {0xE1, 0xD9}
	},
	{
		2, {0xE1, 0x79},
		3, {0xE7, 0x97, 0x92},
		2, {0xD2, 0x75},
		2, {0xE1, 0xDA}
	},
	{
		2, {0xE1, 0x7A},
		3, {0xE7, 0x97, 0x99},
		2, {0xD9, 0x75},
		2, {0xE1, 0xDB}
	},
	{
		2, {0xE1, 0x7B},
		3, {0xE7, 0x97, 0xA3},
		2, {0xE3, 0x75},
		2, {0xE1, 0xDC}
	},
	{
		2, {0xE1, 0x7C},
		3, {0xE7, 0x97, 0x9E},
		2, {0xDE, 0x75},
		2, {0xE1, 0xDD}
	},
	{
		2, {0xE1, 0x7D},
		3, {0xE7, 0x97, 0xBE},
		2, {0xFE, 0x75},
		2, {0xE1, 0xDE}
	},
	{
		2, {0xE1, 0x7E},
		3, {0xE7, 0x97, 0xBF},
		2, {0xFF, 0x75},
		2, {0xE1, 0xDF}
	},
	{
		2, {0xE1, 0x80},
		3, {0xE7, 0x97, 0xBC},
		2, {0xFC, 0x75},
		2, {0xE1, 0xE0}
	},
	{
		2, {0xE1, 0x81},
		3, {0xE7, 0x98, 0x81},
		2, {0x01, 0x76},
		2, {0xE1, 0xE1}
	},
	{
		2, {0xE1, 0x82},
		3, {0xE7, 0x97, 0xB0},
		2, {0xF0, 0x75},
		2, {0xE1, 0xE2}
	},
	{
		2, {0xE1, 0x83},
		3, {0xE7, 0x97, 0xBA},
		2, {0xFA, 0x75},
		2, {0xE1, 0xE3}
	},
	{
		2, {0xE1, 0x84},
		3, {0xE7, 0x97, 0xB2},
		2, {0xF2, 0x75},
		2, {0xE1, 0xE4}
	},
	{
		2, {0xE1, 0x85},
		3, {0xE7, 0x97, 0xB3},
		2, {0xF3, 0x75},
		2, {0xE1, 0xE5}
	},
	{
		2, {0xE1, 0x86},
		3, {0xE7, 0x98, 0x8B},
		2, {0x0B, 0x76},
		2, {0xE1, 0xE6}
	},
	{
		2, {0xE1, 0x87},
		3, {0xE7, 0x98, 0x8D},
		2, {0x0D, 0x76},
		2, {0xE1, 0xE7}
	},
	{
		2, {0xE1, 0x88},
		3, {0xE7, 0x98, 0x89},
		2, {0x09, 0x76},
		2, {0xE1, 0xE8}
	},
	{
		2, {0xE1, 0x89},
		3, {0xE7, 0x98, 0x9F},
		2, {0x1F, 0x76},
		2, {0xE1, 0xE9}
	},
	{
		2, {0xE1, 0x8A},
		3, {0xE7, 0x98, 0xA7},
		2, {0x27, 0x76},
		2, {0xE1, 0xEA}
	},
	{
		2, {0xE1, 0x8B},
		3, {0xE7, 0x98, 0xA0},
		2, {0x20, 0x76},
		2, {0xE1, 0xEB}
	},
	{
		2, {0xE1, 0x8C},
		3, {0xE7, 0x98, 0xA1},
		2, {0x21, 0x76},
		2, {0xE1, 0xEC}
	},
	{
		2, {0xE1, 0x8D},
		3, {0xE7, 0x98, 0xA2},
		2, {0x22, 0x76},
		2, {0xE1, 0xED}
	},
	{
		2, {0xE1, 0x8E},
		3, {0xE7, 0x98, 0xA4},
		2, {0x24, 0x76},
		2, {0xE1, 0xEE}
	},
	{
		2, {0xE1, 0x8F},
		3, {0xE7, 0x98, 0xB4},
		2, {0x34, 0x76},
		2, {0xE1, 0xEF}
	},
	{
		2, {0xE1, 0x90},
		3, {0xE7, 0x98, 0xB0},
		2, {0x30, 0x76},
		2, {0xE1, 0xF0}
	},
	{
		2, {0xE1, 0x91},
		3, {0xE7, 0x98, 0xBB},
		2, {0x3B, 0x76},
		2, {0xE1, 0xF1}
	},
	{
		2, {0xE1, 0x92},
		3, {0xE7, 0x99, 0x87},
		2, {0x47, 0x76},
		2, {0xE1, 0xF2}
	},
	{
		2, {0xE1, 0x93},
		3, {0xE7, 0x99, 0x88},
		2, {0x48, 0x76},
		2, {0xE1, 0xF3}
	},
	{
		2, {0xE1, 0x94},
		3, {0xE7, 0x99, 0x86},
		2, {0x46, 0x76},
		2, {0xE1, 0xF4}
	},
	{
		2, {0xE1, 0x95},
		3, {0xE7, 0x99, 0x9C},
		2, {0x5C, 0x76},
		2, {0xE1, 0xF5}
	},
	{
		2, {0xE1, 0x96},
		3, {0xE7, 0x99, 0x98},
		2, {0x58, 0x76},
		2, {0xE1, 0xF6}
	},
	{
		2, {0xE1, 0x97},
		3, {0xE7, 0x99, 0xA1},
		2, {0x61, 0x76},
		2, {0xE1, 0xF7}
	},
	{
		2, {0xE1, 0x98},
		3, {0xE7, 0x99, 0xA2},
		2, {0x62, 0x76},
		2, {0xE1, 0xF8}
	},
	{
		2, {0xE1, 0x99},
		3, {0xE7, 0x99, 0xA8},
		2, {0x68, 0x76},
		2, {0xE1, 0xF9}
	},
	{
		2, {0xE1, 0x9A},
		3, {0xE7, 0x99, 0xA9},
		2, {0x69, 0x76},
		2, {0xE1, 0xFA}
	},
	{
		2, {0xE1, 0x9B},
		3, {0xE7, 0x99, 0xAA},
		2, {0x6A, 0x76},
		2, {0xE1, 0xFB}
	},
	{
		2, {0xE1, 0x9C},
		3, {0xE7, 0x99, 0xA7},
		2, {0x67, 0x76},
		2, {0xE1, 0xFC}
	},
	{
		2, {0xE1, 0x9D},
		3, {0xE7, 0x99, 0xAC},
		2, {0x6C, 0x76},
		2, {0xE1, 0xFD}
	},
	{
		2, {0xE1, 0x9E},
		3, {0xE7, 0x99, 0xB0},
		2, {0x70, 0x76},
		2, {0xE1, 0xFE}
	},
	{
		2, {0xE1, 0x9F},
		3, {0xE7, 0x99, 0xB2},
		2, {0x72, 0x76},
		2, {0xE2, 0xA1}
	},
	{
		2, {0xE1, 0xA0},
		3, {0xE7, 0x99, 0xB6},
		2, {0x76, 0x76},
		2, {0xE2, 0xA2}
	},
	{
		2, {0xE1, 0xA1},
		3, {0xE7, 0x99, 0xB8},
		2, {0x78, 0x76},
		2, {0xE2, 0xA3}
	},
	{
		2, {0xE1, 0xA2},
		3, {0xE7, 0x99, 0xBC},
		2, {0x7C, 0x76},
		2, {0xE2, 0xA4}
	},
	{
		2, {0xE1, 0xA3},
		3, {0xE7, 0x9A, 0x80},
		2, {0x80, 0x76},
		2, {0xE2, 0xA5}
	},
	{
		2, {0xE1, 0xA4},
		3, {0xE7, 0x9A, 0x83},
		2, {0x83, 0x76},
		2, {0xE2, 0xA6}
	},
	{
		2, {0xE1, 0xA5},
		3, {0xE7, 0x9A, 0x88},
		2, {0x88, 0x76},
		2, {0xE2, 0xA7}
	},
	{
		2, {0xE1, 0xA6},
		3, {0xE7, 0x9A, 0x8B},
		2, {0x8B, 0x76},
		2, {0xE2, 0xA8}
	},
	{
		2, {0xE1, 0xA7},
		3, {0xE7, 0x9A, 0x8E},
		2, {0x8E, 0x76},
		2, {0xE2, 0xA9}
	},
	{
		2, {0xE1, 0xA8},
		3, {0xE7, 0x9A, 0x96},
		2, {0x96, 0x76},
		2, {0xE2, 0xAA}
	},
	{
		2, {0xE1, 0xA9},
		3, {0xE7, 0x9A, 0x93},
		2, {0x93, 0x76},
		2, {0xE2, 0xAB}
	},
	{
		2, {0xE1, 0xAA},
		3, {0xE7, 0x9A, 0x99},
		2, {0x99, 0x76},
		2, {0xE2, 0xAC}
	},
	{
		2, {0xE1, 0xAB},
		3, {0xE7, 0x9A, 0x9A},
		2, {0x9A, 0x76},
		2, {0xE2, 0xAD}
	},
	{
		2, {0xE1, 0xAC},
		3, {0xE7, 0x9A, 0xB0},
		2, {0xB0, 0x76},
		2, {0xE2, 0xAE}
	},
	{
		2, {0xE1, 0xAD},
		3, {0xE7, 0x9A, 0xB4},
		2, {0xB4, 0x76},
		2, {0xE2, 0xAF}
	},
	{
		2, {0xE1, 0xAE},
		3, {0xE7, 0x9A, 0xB8},
		2, {0xB8, 0x76},
		2, {0xE2, 0xB0}
	},
	{
		2, {0xE1, 0xAF},
		3, {0xE7, 0x9A, 0xB9},
		2, {0xB9, 0x76},
		2, {0xE2, 0xB1}
	},
	{
		2, {0xE1, 0xB0},
		3, {0xE7, 0x9A, 0xBA},
		2, {0xBA, 0x76},
		2, {0xE2, 0xB2}
	},
	{
		2, {0xE1, 0xB1},
		3, {0xE7, 0x9B, 0x82},
		2, {0xC2, 0x76},
		2, {0xE2, 0xB3}
	},
	{
		2, {0xE1, 0xB2},
		3, {0xE7, 0x9B, 0x8D},
		2, {0xCD, 0x76},
		2, {0xE2, 0xB4}
	},
	{
		2, {0xE1, 0xB3},
		3, {0xE7, 0x9B, 0x96},
		2, {0xD6, 0x76},
		2, {0xE2, 0xB5}
	},
	{
		2, {0xE1, 0xB4},
		3, {0xE7, 0x9B, 0x92},
		2, {0xD2, 0x76},
		2, {0xE2, 0xB6}
	},
	{
		2, {0xE1, 0xB5},
		3, {0xE7, 0x9B, 0x9E},
		2, {0xDE, 0x76},
		2, {0xE2, 0xB7}
	},
	{
		2, {0xE1, 0xB6},
		3, {0xE7, 0x9B, 0xA1},
		2, {0xE1, 0x76},
		2, {0xE2, 0xB8}
	},
	{
		2, {0xE1, 0xB7},
		3, {0xE7, 0x9B, 0xA5},
		2, {0xE5, 0x76},
		2, {0xE2, 0xB9}
	},
	{
		2, {0xE1, 0xB8},
		3, {0xE7, 0x9B, 0xA7},
		2, {0xE7, 0x76},
		2, {0xE2, 0xBA}
	},
	{
		2, {0xE1, 0xB9},
		3, {0xE7, 0x9B, 0xAA},
		2, {0xEA, 0x76},
		2, {0xE2, 0xBB}
	},
	{
		2, {0xE1, 0xBA},
		3, {0xE8, 0x98, 0xAF},
		2, {0x2F, 0x86},
		2, {0xE2, 0xBC}
	},
	{
		2, {0xE1, 0xBB},
		3, {0xE7, 0x9B, 0xBB},
		2, {0xFB, 0x76},
		2, {0xE2, 0xBD}
	},
	{
		2, {0xE1, 0xBC},
		3, {0xE7, 0x9C, 0x88},
		2, {0x08, 0x77},
		2, {0xE2, 0xBE}
	},
	{
		2, {0xE1, 0xBD},
		3, {0xE7, 0x9C, 0x87},
		2, {0x07, 0x77},
		2, {0xE2, 0xBF}
	},
	{
		2, {0xE1, 0xBE},
		3, {0xE7, 0x9C, 0x84},
		2, {0x04, 0x77},
		2, {0xE2, 0xC0}
	},
	{
		2, {0xE1, 0xBF},
		3, {0xE7, 0x9C, 0xA9},
		2, {0x29, 0x77},
		2, {0xE2, 0xC1}
	},
	{
		2, {0xE1, 0xC0},
		3, {0xE7, 0x9C, 0xA4},
		2, {0x24, 0x77},
		2, {0xE2, 0xC2}
	},
	{
		2, {0xE1, 0xC1},
		3, {0xE7, 0x9C, 0x9E},
		2, {0x1E, 0x77},
		2, {0xE2, 0xC3}
	},
	{
		2, {0xE1, 0xC2},
		3, {0xE7, 0x9C, 0xA5},
		2, {0x25, 0x77},
		2, {0xE2, 0xC4}
	},
	{
		2, {0xE1, 0xC3},
		3, {0xE7, 0x9C, 0xA6},
		2, {0x26, 0x77},
		2, {0xE2, 0xC5}
	},
	{
		2, {0xE1, 0xC4},
		3, {0xE7, 0x9C, 0x9B},
		2, {0x1B, 0x77},
		2, {0xE2, 0xC6}
	},
	{
		2, {0xE1, 0xC5},
		3, {0xE7, 0x9C, 0xB7},
		2, {0x37, 0x77},
		2, {0xE2, 0xC7}
	},
	{
		2, {0xE1, 0xC6},
		3, {0xE7, 0x9C, 0xB8},
		2, {0x38, 0x77},
		2, {0xE2, 0xC8}
	},
	{
		2, {0xE1, 0xC7},
		3, {0xE7, 0x9D, 0x87},
		2, {0x47, 0x77},
		2, {0xE2, 0xC9}
	},
	{
		2, {0xE1, 0xC8},
		3, {0xE7, 0x9D, 0x9A},
		2, {0x5A, 0x77},
		2, {0xE2, 0xCA}
	},
	{
		2, {0xE1, 0xC9},
		3, {0xE7, 0x9D, 0xA8},
		2, {0x68, 0x77},
		2, {0xE2, 0xCB}
	},
	{
		2, {0xE1, 0xCA},
		3, {0xE7, 0x9D, 0xAB},
		2, {0x6B, 0x77},
		2, {0xE2, 0xCC}
	},
	{
		2, {0xE1, 0xCB},
		3, {0xE7, 0x9D, 0x9B},
		2, {0x5B, 0x77},
		2, {0xE2, 0xCD}
	},
	{
		2, {0xE1, 0xCC},
		3, {0xE7, 0x9D, 0xA5},
		2, {0x65, 0x77},
		2, {0xE2, 0xCE}
	},
	{
		2, {0xE1, 0xCD},
		3, {0xE7, 0x9D, 0xBF},
		2, {0x7F, 0x77},
		2, {0xE2, 0xCF}
	},
	{
		2, {0xE1, 0xCE},
		3, {0xE7, 0x9D, 0xBE},
		2, {0x7E, 0x77},
		2, {0xE2, 0xD0}
	},
	{
		2, {0xE1, 0xCF},
		3, {0xE7, 0x9D, 0xB9},
		2, {0x79, 0x77},
		2, {0xE2, 0xD1}
	},
	{
		2, {0xE1, 0xD0},
		3, {0xE7, 0x9E, 0x8E},
		2, {0x8E, 0x77},
		2, {0xE2, 0xD2}
	},
	{
		2, {0xE1, 0xD1},
		3, {0xE7, 0x9E, 0x8B},
		2, {0x8B, 0x77},
		2, {0xE2, 0xD3}
	},
	{
		2, {0xE1, 0xD2},
		3, {0xE7, 0x9E, 0x91},
		2, {0x91, 0x77},
		2, {0xE2, 0xD4}
	},
	{
		2, {0xE1, 0xD3},
		3, {0xE7, 0x9E, 0xA0},
		2, {0xA0, 0x77},
		2, {0xE2, 0xD5}
	},
	{
		2, {0xE1, 0xD4},
		3, {0xE7, 0x9E, 0x9E},
		2, {0x9E, 0x77},
		2, {0xE2, 0xD6}
	},
	{
		2, {0xE1, 0xD5},
		3, {0xE7, 0x9E, 0xB0},
		2, {0xB0, 0x77},
		2, {0xE2, 0xD7}
	},
	{
		2, {0xE1, 0xD6},
		3, {0xE7, 0x9E, 0xB6},
		2, {0xB6, 0x77},
		2, {0xE2, 0xD8}
	},
	{
		2, {0xE1, 0xD7},
		3, {0xE7, 0x9E, 0xB9},
		2, {0xB9, 0x77},
		2, {0xE2, 0xD9}
	},
	{
		2, {0xE1, 0xD8},
		3, {0xE7, 0x9E, 0xBF},
		2, {0xBF, 0x77},
		2, {0xE2, 0xDA}
	},
	{
		2, {0xE1, 0xD9},
		3, {0xE7, 0x9E, 0xBC},
		2, {0xBC, 0x77},
		2, {0xE2, 0xDB}
	},
	{
		2, {0xE1, 0xDA},
		3, {0xE7, 0x9E, 0xBD},
		2, {0xBD, 0x77},
		2, {0xE2, 0xDC}
	},
	{
		2, {0xE1, 0xDB},
		3, {0xE7, 0x9E, 0xBB},
		2, {0xBB, 0x77},
		2, {0xE2, 0xDD}
	},
	{
		2, {0xE1, 0xDC},
		3, {0xE7, 0x9F, 0x87},
		2, {0xC7, 0x77},
		2, {0xE2, 0xDE}
	},
	{
		2, {0xE1, 0xDD},
		3, {0xE7, 0x9F, 0x8D},
		2, {0xCD, 0x77},
		2, {0xE2, 0xDF}
	},
	{
		2, {0xE1, 0xDE},
		3, {0xE7, 0x9F, 0x97},
		2, {0xD7, 0x77},
		2, {0xE2, 0xE0}
	},
	{
		2, {0xE1, 0xDF},
		3, {0xE7, 0x9F, 0x9A},
		2, {0xDA, 0x77},
		2, {0xE2, 0xE1}
	},
	{
		2, {0xE1, 0xE0},
		3, {0xE7, 0x9F, 0x9C},
		2, {0xDC, 0x77},
		2, {0xE2, 0xE2}
	},
	{
		2, {0xE1, 0xE1},
		3, {0xE7, 0x9F, 0xA3},
		2, {0xE3, 0x77},
		2, {0xE2, 0xE3}
	},
	{
		2, {0xE1, 0xE2},
		3, {0xE7, 0x9F, 0xAE},
		2, {0xEE, 0x77},
		2, {0xE2, 0xE4}
	},
	{
		2, {0xE1, 0xE3},
		3, {0xE7, 0x9F, 0xBC},
		2, {0xFC, 0x77},
		2, {0xE2, 0xE5}
	},
	{
		2, {0xE1, 0xE4},
		3, {0xE7, 0xA0, 0x8C},
		2, {0x0C, 0x78},
		2, {0xE2, 0xE6}
	},
	{
		2, {0xE1, 0xE5},
		3, {0xE7, 0xA0, 0x92},
		2, {0x12, 0x78},
		2, {0xE2, 0xE7}
	},
	{
		2, {0xE1, 0xE6},
		3, {0xE7, 0xA4, 0xA6},
		2, {0x26, 0x79},
		2, {0xE2, 0xE8}
	},
	{
		2, {0xE1, 0xE7},
		3, {0xE7, 0xA0, 0xA0},
		2, {0x20, 0x78},
		2, {0xE2, 0xE9}
	},
	{
		2, {0xE1, 0xE8},
		3, {0xE7, 0xA4, 0xAA},
		2, {0x2A, 0x79},
		2, {0xE2, 0xEA}
	},
	{
		2, {0xE1, 0xE9},
		3, {0xE7, 0xA1, 0x85},
		2, {0x45, 0x78},
		2, {0xE2, 0xEB}
	},
	{
		2, {0xE1, 0xEA},
		3, {0xE7, 0xA2, 0x8E},
		2, {0x8E, 0x78},
		2, {0xE2, 0xEC}
	},
	{
		2, {0xE1, 0xEB},
		3, {0xE7, 0xA1, 0xB4},
		2, {0x74, 0x78},
		2, {0xE2, 0xED}
	},
	{
		2, {0xE1, 0xEC},
		3, {0xE7, 0xA2, 0x86},
		2, {0x86, 0x78},
		2, {0xE2, 0xEE}
	},
	{
		2, {0xE1, 0xED},
		3, {0xE7, 0xA1, 0xBC},
		2, {0x7C, 0x78},
		2, {0xE2, 0xEF}
	},
	{
		2, {0xE1, 0xEE},
		3, {0xE7, 0xA2, 0x9A},
		2, {0x9A, 0x78},
		2, {0xE2, 0xF0}
	},
	{
		2, {0xE1, 0xEF},
		3, {0xE7, 0xA2, 0x8C},
		2, {0x8C, 0x78},
		2, {0xE2, 0xF1}
	},
	{
		2, {0xE1, 0xF0},
		3, {0xE7, 0xA2, 0xA3},
		2, {0xA3, 0x78},
		2, {0xE2, 0xF2}
	},
	{
		2, {0xE1, 0xF1},
		3, {0xE7, 0xA2, 0xB5},
		2, {0xB5, 0x78},
		2, {0xE2, 0xF3}
	},
	{
		2, {0xE1, 0xF2},
		3, {0xE7, 0xA2, 0xAA},
		2, {0xAA, 0x78},
		2, {0xE2, 0xF4}
	},
	{
		2, {0xE1, 0xF3},
		3, {0xE7, 0xA2, 0xAF},
		2, {0xAF, 0x78},
		2, {0xE2, 0xF5}
	},
	{
		2, {0xE1, 0xF4},
		3, {0xE7, 0xA3, 0x91},
		2, {0xD1, 0x78},
		2, {0xE2, 0xF6}
	},
	{
		2, {0xE1, 0xF5},
		3, {0xE7, 0xA3, 0x86},
		2, {0xC6, 0x78},
		2, {0xE2, 0xF7}
	},
	{
		2, {0xE1, 0xF6},
		3, {0xE7, 0xA3, 0x8B},
		2, {0xCB, 0x78},
		2, {0xE2, 0xF8}
	},
	{
		2, {0xE1, 0xF7},
		3, {0xE7, 0xA3, 0x94},
		2, {0xD4, 0x78},
		2, {0xE2, 0xF9}
	},
	{
		2, {0xE1, 0xF8},
		3, {0xE7, 0xA2, 0xBE},
		2, {0xBE, 0x78},
		2, {0xE2, 0xFA}
	},
	{
		2, {0xE1, 0xF9},
		3, {0xE7, 0xA2, 0xBC},
		2, {0xBC, 0x78},
		2, {0xE2, 0xFB}
	},
	{
		2, {0xE1, 0xFA},
		3, {0xE7, 0xA3, 0x85},
		2, {0xC5, 0x78},
		2, {0xE2, 0xFC}
	},
	{
		2, {0xE1, 0xFB},
		3, {0xE7, 0xA3, 0x8A},
		2, {0xCA, 0x78},
		2, {0xE2, 0xFD}
	},
	{
		2, {0xE1, 0xFC},
		3, {0xE7, 0xA3, 0xAC},
		2, {0xEC, 0x78},
		2, {0xE2, 0xFE}
	},
	{
		2, {0xE2, 0x40},
		3, {0xE7, 0xA3, 0xA7},
		2, {0xE7, 0x78},
		2, {0xE3, 0xA1}
	},
	{
		2, {0xE2, 0x41},
		3, {0xE7, 0xA3, 0x9A},
		2, {0xDA, 0x78},
		2, {0xE3, 0xA2}
	},
	{
		2, {0xE2, 0x42},
		3, {0xE7, 0xA3, 0xBD},
		2, {0xFD, 0x78},
		2, {0xE3, 0xA3}
	},
	{
		2, {0xE2, 0x43},
		3, {0xE7, 0xA3, 0xB4},
		2, {0xF4, 0x78},
		2, {0xE3, 0xA4}
	},
	{
		2, {0xE2, 0x44},
		3, {0xE7, 0xA4, 0x87},
		2, {0x07, 0x79},
		2, {0xE3, 0xA5}
	},
	{
		2, {0xE2, 0x45},
		3, {0xE7, 0xA4, 0x92},
		2, {0x12, 0x79},
		2, {0xE3, 0xA6}
	},
	{
		2, {0xE2, 0x46},
		3, {0xE7, 0xA4, 0x91},
		2, {0x11, 0x79},
		2, {0xE3, 0xA7}
	},
	{
		2, {0xE2, 0x47},
		3, {0xE7, 0xA4, 0x99},
		2, {0x19, 0x79},
		2, {0xE3, 0xA8}
	},
	{
		2, {0xE2, 0x48},
		3, {0xE7, 0xA4, 0xAC},
		2, {0x2C, 0x79},
		2, {0xE3, 0xA9}
	},
	{
		2, {0xE2, 0x49},
		3, {0xE7, 0xA4, 0xAB},
		2, {0x2B, 0x79},
		2, {0xE3, 0xAA}
	},
	{
		2, {0xE2, 0x4A},
		3, {0xE7, 0xA5, 0x80},
		2, {0x40, 0x79},
		2, {0xE3, 0xAB}
	},
	{
		2, {0xE2, 0x4B},
		3, {0xE7, 0xA5, 0xA0},
		2, {0x60, 0x79},
		2, {0xE3, 0xAC}
	},
	{
		2, {0xE2, 0x4C},
		3, {0xE7, 0xA5, 0x97},
		2, {0x57, 0x79},
		2, {0xE3, 0xAD}
	},
	{
		2, {0xE2, 0x4D},
		3, {0xE7, 0xA5, 0x9F},
		2, {0x5F, 0x79},
		2, {0xE3, 0xAE}
	},
	{
		2, {0xE2, 0x4E},
		3, {0xE7, 0xA5, 0x9A},
		2, {0x5A, 0x79},
		2, {0xE3, 0xAF}
	},
	{
		2, {0xE2, 0x4F},
		3, {0xE7, 0xA5, 0x95},
		2, {0x55, 0x79},
		2, {0xE3, 0xB0}
	},
	{
		2, {0xE2, 0x50},
		3, {0xE7, 0xA5, 0x93},
		2, {0x53, 0x79},
		2, {0xE3, 0xB1}
	},
	{
		2, {0xE2, 0x51},
		3, {0xE7, 0xA5, 0xBA},
		2, {0x7A, 0x79},
		2, {0xE3, 0xB2}
	},
	{
		2, {0xE2, 0x52},
		3, {0xE7, 0xA5, 0xBF},
		2, {0x7F, 0x79},
		2, {0xE3, 0xB3}
	},
	{
		2, {0xE2, 0x53},
		3, {0xE7, 0xA6, 0x8A},
		2, {0x8A, 0x79},
		2, {0xE3, 0xB4}
	},
	{
		2, {0xE2, 0x54},
		3, {0xE7, 0xA6, 0x9D},
		2, {0x9D, 0x79},
		2, {0xE3, 0xB5}
	},
	{
		2, {0xE2, 0x55},
		3, {0xE7, 0xA6, 0xA7},
		2, {0xA7, 0x79},
		2, {0xE3, 0xB6}
	},
	{
		2, {0xE2, 0x56},
		3, {0xE9, 0xBD, 0x8B},
		2, {0x4B, 0x9F},
		2, {0xE3, 0xB7}
	},
	{
		2, {0xE2, 0x57},
		3, {0xE7, 0xA6, 0xAA},
		2, {0xAA, 0x79},
		2, {0xE3, 0xB8}
	},
	{
		2, {0xE2, 0x58},
		3, {0xE7, 0xA6, 0xAE},
		2, {0xAE, 0x79},
		2, {0xE3, 0xB9}
	},
	{
		2, {0xE2, 0x59},
		3, {0xE7, 0xA6, 0xB3},
		2, {0xB3, 0x79},
		2, {0xE3, 0xBA}
	},
	{
		2, {0xE2, 0x5A},
		3, {0xE7, 0xA6, 0xB9},
		2, {0xB9, 0x79},
		2, {0xE3, 0xBB}
	},
	{
		2, {0xE2, 0x5B},
		3, {0xE7, 0xA6, 0xBA},
		2, {0xBA, 0x79},
		2, {0xE3, 0xBC}
	},
	{
		2, {0xE2, 0x5C},
		3, {0xE7, 0xA7, 0x89},
		2, {0xC9, 0x79},
		2, {0xE3, 0xBD}
	},
	{
		2, {0xE2, 0x5D},
		3, {0xE7, 0xA7, 0x95},
		2, {0xD5, 0x79},
		2, {0xE3, 0xBE}
	},
	{
		2, {0xE2, 0x5E},
		3, {0xE7, 0xA7, 0xA7},
		2, {0xE7, 0x79},
		2, {0xE3, 0xBF}
	},
	{
		2, {0xE2, 0x5F},
		3, {0xE7, 0xA7, 0xAC},
		2, {0xEC, 0x79},
		2, {0xE3, 0xC0}
	},
	{
		2, {0xE2, 0x60},
		3, {0xE7, 0xA7, 0xA1},
		2, {0xE1, 0x79},
		2, {0xE3, 0xC1}
	},
	{
		2, {0xE2, 0x61},
		3, {0xE7, 0xA7, 0xA3},
		2, {0xE3, 0x79},
		2, {0xE3, 0xC2}
	},
	{
		2, {0xE2, 0x62},
		3, {0xE7, 0xA8, 0x88},
		2, {0x08, 0x7A},
		2, {0xE3, 0xC3}
	},
	{
		2, {0xE2, 0x63},
		3, {0xE7, 0xA8, 0x8D},
		2, {0x0D, 0x7A},
		2, {0xE3, 0xC4}
	},
	{
		2, {0xE2, 0x64},
		3, {0xE7, 0xA8, 0x98},
		2, {0x18, 0x7A},
		2, {0xE3, 0xC5}
	},
	{
		2, {0xE2, 0x65},
		3, {0xE7, 0xA8, 0x99},
		2, {0x19, 0x7A},
		2, {0xE3, 0xC6}
	},
	{
		2, {0xE2, 0x66},
		3, {0xE7, 0xA8, 0xA0},
		2, {0x20, 0x7A},
		2, {0xE3, 0xC7}
	},
	{
		2, {0xE2, 0x67},
		3, {0xE7, 0xA8, 0x9F},
		2, {0x1F, 0x7A},
		2, {0xE3, 0xC8}
	},
	{
		2, {0xE2, 0x68},
		3, {0xE7, 0xA6, 0x80},
		2, {0x80, 0x79},
		2, {0xE3, 0xC9}
	},
	{
		2, {0xE2, 0x69},
		3, {0xE7, 0xA8, 0xB1},
		2, {0x31, 0x7A},
		2, {0xE3, 0xCA}
	},
	{
		2, {0xE2, 0x6A},
		3, {0xE7, 0xA8, 0xBB},
		2, {0x3B, 0x7A},
		2, {0xE3, 0xCB}
	},
	{
		2, {0xE2, 0x6B},
		3, {0xE7, 0xA8, 0xBE},
		2, {0x3E, 0x7A},
		2, {0xE3, 0xCC}
	},
	{
		2, {0xE2, 0x6C},
		3, {0xE7, 0xA8, 0xB7},
		2, {0x37, 0x7A},
		2, {0xE3, 0xCD}
	},
	{
		2, {0xE2, 0x6D},
		3, {0xE7, 0xA9, 0x83},
		2, {0x43, 0x7A},
		2, {0xE3, 0xCE}
	},
	{
		2, {0xE2, 0x6E},
		3, {0xE7, 0xA9, 0x97},
		2, {0x57, 0x7A},
		2, {0xE3, 0xCF}
	},
	{
		2, {0xE2, 0x6F},
		3, {0xE7, 0xA9, 0x89},
		2, {0x49, 0x7A},
		2, {0xE3, 0xD0}
	},
	{
		2, {0xE2, 0x70},
		3, {0xE7, 0xA9, 0xA1},
		2, {0x61, 0x7A},
		2, {0xE3, 0xD1}
	},
	{
		2, {0xE2, 0x71},
		3, {0xE7, 0xA9, 0xA2},
		2, {0x62, 0x7A},
		2, {0xE3, 0xD2}
	},
	{
		2, {0xE2, 0x72},
		3, {0xE7, 0xA9, 0xA9},
		2, {0x69, 0x7A},
		2, {0xE3, 0xD3}
	},
	{
		2, {0xE2, 0x73},
		3, {0xE9, 0xBE, 0x9D},
		2, {0x9D, 0x9F},
		2, {0xE3, 0xD4}
	},
	{
		2, {0xE2, 0x74},
		3, {0xE7, 0xA9, 0xB0},
		2, {0x70, 0x7A},
		2, {0xE3, 0xD5}
	},
	{
		2, {0xE2, 0x75},
		3, {0xE7, 0xA9, 0xB9},
		2, {0x79, 0x7A},
		2, {0xE3, 0xD6}
	},
	{
		2, {0xE2, 0x76},
		3, {0xE7, 0xA9, 0xBD},
		2, {0x7D, 0x7A},
		2, {0xE3, 0xD7}
	},
	{
		2, {0xE2, 0x77},
		3, {0xE7, 0xAA, 0x88},
		2, {0x88, 0x7A},
		2, {0xE3, 0xD8}
	},
	{
		2, {0xE2, 0x78},
		3, {0xE7, 0xAA, 0x97},
		2, {0x97, 0x7A},
		2, {0xE3, 0xD9}
	},
	{
		2, {0xE2, 0x79},
		3, {0xE7, 0xAA, 0x95},
		2, {0x95, 0x7A},
		2, {0xE3, 0xDA}
	},
	{
		2, {0xE2, 0x7A},
		3, {0xE7, 0xAA, 0x98},
		2, {0x98, 0x7A},
		2, {0xE3, 0xDB}
	},
	{
		2, {0xE2, 0x7B},
		3, {0xE7, 0xAA, 0x96},
		2, {0x96, 0x7A},
		2, {0xE3, 0xDC}
	},
	{
		2, {0xE2, 0x7C},
		3, {0xE7, 0xAA, 0xA9},
		2, {0xA9, 0x7A},
		2, {0xE3, 0xDD}
	},
	{
		2, {0xE2, 0x7D},
		3, {0xE7, 0xAB, 0x88},
		2, {0xC8, 0x7A},
		2, {0xE3, 0xDE}
	},
	{
		2, {0xE2, 0x7E},
		3, {0xE7, 0xAA, 0xB0},
		2, {0xB0, 0x7A},
		2, {0xE3, 0xDF}
	},
	{
		2, {0xE2, 0x80},
		3, {0xE7, 0xAA, 0xB6},
		2, {0xB6, 0x7A},
		2, {0xE3, 0xE0}
	},
	{
		2, {0xE2, 0x81},
		3, {0xE7, 0xAB, 0x85},
		2, {0xC5, 0x7A},
		2, {0xE3, 0xE1}
	},
	{
		2, {0xE2, 0x82},
		3, {0xE7, 0xAB, 0x84},
		2, {0xC4, 0x7A},
		2, {0xE3, 0xE2}
	},
	{
		2, {0xE2, 0x83},
		3, {0xE7, 0xAA, 0xBF},
		2, {0xBF, 0x7A},
		2, {0xE3, 0xE3}
	},
	{
		2, {0xE2, 0x84},
		3, {0xE9, 0x82, 0x83},
		2, {0x83, 0x90},
		2, {0xE3, 0xE4}
	},
	{
		2, {0xE2, 0x85},
		3, {0xE7, 0xAB, 0x87},
		2, {0xC7, 0x7A},
		2, {0xE3, 0xE5}
	},
	{
		2, {0xE2, 0x86},
		3, {0xE7, 0xAB, 0x8A},
		2, {0xCA, 0x7A},
		2, {0xE3, 0xE6}
	},
	{
		2, {0xE2, 0x87},
		3, {0xE7, 0xAB, 0x8D},
		2, {0xCD, 0x7A},
		2, {0xE3, 0xE7}
	},
	{
		2, {0xE2, 0x88},
		3, {0xE7, 0xAB, 0x8F},
		2, {0xCF, 0x7A},
		2, {0xE3, 0xE8}
	},
	{
		2, {0xE2, 0x89},
		3, {0xE7, 0xAB, 0x95},
		2, {0xD5, 0x7A},
		2, {0xE3, 0xE9}
	},
	{
		2, {0xE2, 0x8A},
		3, {0xE7, 0xAB, 0x93},
		2, {0xD3, 0x7A},
		2, {0xE3, 0xEA}
	},
	{
		2, {0xE2, 0x8B},
		3, {0xE7, 0xAB, 0x99},
		2, {0xD9, 0x7A},
		2, {0xE3, 0xEB}
	},
	{
		2, {0xE2, 0x8C},
		3, {0xE7, 0xAB, 0x9A},
		2, {0xDA, 0x7A},
		2, {0xE3, 0xEC}
	},
	{
		2, {0xE2, 0x8D},
		3, {0xE7, 0xAB, 0x9D},
		2, {0xDD, 0x7A},
		2, {0xE3, 0xED}
	},
	{
		2, {0xE2, 0x8E},
		3, {0xE7, 0xAB, 0xA1},
		2, {0xE1, 0x7A},
		2, {0xE3, 0xEE}
	},
	{
		2, {0xE2, 0x8F},
		3, {0xE7, 0xAB, 0xA2},
		2, {0xE2, 0x7A},
		2, {0xE3, 0xEF}
	},
	{
		2, {0xE2, 0x90},
		3, {0xE7, 0xAB, 0xA6},
		2, {0xE6, 0x7A},
		2, {0xE3, 0xF0}
	},
	{
		2, {0xE2, 0x91},
		3, {0xE7, 0xAB, 0xAD},
		2, {0xED, 0x7A},
		2, {0xE3, 0xF1}
	},
	{
		2, {0xE2, 0x92},
		3, {0xE7, 0xAB, 0xB0},
		2, {0xF0, 0x7A},
		2, {0xE3, 0xF2}
	},
	{
		2, {0xE2, 0x93},
		3, {0xE7, 0xAC, 0x82},
		2, {0x02, 0x7B},
		2, {0xE3, 0xF3}
	},
	{
		2, {0xE2, 0x94},
		3, {0xE7, 0xAC, 0x8F},
		2, {0x0F, 0x7B},
		2, {0xE3, 0xF4}
	},
	{
		2, {0xE2, 0x95},
		3, {0xE7, 0xAC, 0x8A},
		2, {0x0A, 0x7B},
		2, {0xE3, 0xF5}
	},
	{
		2, {0xE2, 0x96},
		3, {0xE7, 0xAC, 0x86},
		2, {0x06, 0x7B},
		2, {0xE3, 0xF6}
	},
	{
		2, {0xE2, 0x97},
		3, {0xE7, 0xAC, 0xB3},
		2, {0x33, 0x7B},
		2, {0xE3, 0xF7}
	},
	{
		2, {0xE2, 0x98},
		3, {0xE7, 0xAC, 0x98},
		2, {0x18, 0x7B},
		2, {0xE3, 0xF8}
	},
	{
		2, {0xE2, 0x99},
		3, {0xE7, 0xAC, 0x99},
		2, {0x19, 0x7B},
		2, {0xE3, 0xF9}
	},
	{
		2, {0xE2, 0x9A},
		3, {0xE7, 0xAC, 0x9E},
		2, {0x1E, 0x7B},
		2, {0xE3, 0xFA}
	},
	{
		2, {0xE2, 0x9B},
		3, {0xE7, 0xAC, 0xB5},
		2, {0x35, 0x7B},
		2, {0xE3, 0xFB}
	},
	{
		2, {0xE2, 0x9C},
		3, {0xE7, 0xAC, 0xA8},
		2, {0x28, 0x7B},
		2, {0xE3, 0xFC}
	},
	{
		2, {0xE2, 0x9D},
		3, {0xE7, 0xAC, 0xB6},
		2, {0x36, 0x7B},
		2, {0xE3, 0xFD}
	},
	{
		2, {0xE2, 0x9E},
		3, {0xE7, 0xAD, 0x90},
		2, {0x50, 0x7B},
		2, {0xE3, 0xFE}
	},
	{
		2, {0xE2, 0x9F},
		3, {0xE7, 0xAD, 0xBA},
		2, {0x7A, 0x7B},
		2, {0xE4, 0xA1}
	},
	{
		2, {0xE2, 0xA0},
		3, {0xE7, 0xAC, 0x84},
		2, {0x04, 0x7B},
		2, {0xE4, 0xA2}
	},
	{
		2, {0xE2, 0xA1},
		3, {0xE7, 0xAD, 0x8D},
		2, {0x4D, 0x7B},
		2, {0xE4, 0xA3}
	},
	{
		2, {0xE2, 0xA2},
		3, {0xE7, 0xAC, 0x8B},
		2, {0x0B, 0x7B},
		2, {0xE4, 0xA4}
	},
	{
		2, {0xE2, 0xA3},
		3, {0xE7, 0xAD, 0x8C},
		2, {0x4C, 0x7B},
		2, {0xE4, 0xA5}
	},
	{
		2, {0xE2, 0xA4},
		3, {0xE7, 0xAD, 0x85},
		2, {0x45, 0x7B},
		2, {0xE4, 0xA6}
	},
	{
		2, {0xE2, 0xA5},
		3, {0xE7, 0xAD, 0xB5},
		2, {0x75, 0x7B},
		2, {0xE4, 0xA7}
	},
	{
		2, {0xE2, 0xA6},
		3, {0xE7, 0xAD, 0xA5},
		2, {0x65, 0x7B},
		2, {0xE4, 0xA8}
	},
	{
		2, {0xE2, 0xA7},
		3, {0xE7, 0xAD, 0xB4},
		2, {0x74, 0x7B},
		2, {0xE4, 0xA9}
	},
	{
		2, {0xE2, 0xA8},
		3, {0xE7, 0xAD, 0xA7},
		2, {0x67, 0x7B},
		2, {0xE4, 0xAA}
	},
	{
		2, {0xE2, 0xA9},
		3, {0xE7, 0xAD, 0xB0},
		2, {0x70, 0x7B},
		2, {0xE4, 0xAB}
	},
	{
		2, {0xE2, 0xAA},
		3, {0xE7, 0xAD, 0xB1},
		2, {0x71, 0x7B},
		2, {0xE4, 0xAC}
	},
	{
		2, {0xE2, 0xAB},
		3, {0xE7, 0xAD, 0xAC},
		2, {0x6C, 0x7B},
		2, {0xE4, 0xAD}
	},
	{
		2, {0xE2, 0xAC},
		3, {0xE7, 0xAD, 0xAE},
		2, {0x6E, 0x7B},
		2, {0xE4, 0xAE}
	},
	{
		2, {0xE2, 0xAD},
		3, {0xE7, 0xAE, 0x9D},
		2, {0x9D, 0x7B},
		2, {0xE4, 0xAF}
	},
	{
		2, {0xE2, 0xAE},
		3, {0xE7, 0xAE, 0x98},
		2, {0x98, 0x7B},
		2, {0xE4, 0xB0}
	},
	{
		2, {0xE2, 0xAF},
		3, {0xE7, 0xAE, 0x9F},
		2, {0x9F, 0x7B},
		2, {0xE4, 0xB1}
	},
	{
		2, {0xE2, 0xB0},
		3, {0xE7, 0xAE, 0x8D},
		2, {0x8D, 0x7B},
		2, {0xE4, 0xB2}
	},
	{
		2, {0xE2, 0xB1},
		3, {0xE7, 0xAE, 0x9C},
		2, {0x9C, 0x7B},
		2, {0xE4, 0xB3}
	},
	{
		2, {0xE2, 0xB2},
		3, {0xE7, 0xAE, 0x9A},
		2, {0x9A, 0x7B},
		2, {0xE4, 0xB4}
	},
	{
		2, {0xE2, 0xB3},
		3, {0xE7, 0xAE, 0x8B},
		2, {0x8B, 0x7B},
		2, {0xE4, 0xB5}
	},
	{
		2, {0xE2, 0xB4},
		3, {0xE7, 0xAE, 0x92},
		2, {0x92, 0x7B},
		2, {0xE4, 0xB6}
	},
	{
		2, {0xE2, 0xB5},
		3, {0xE7, 0xAE, 0x8F},
		2, {0x8F, 0x7B},
		2, {0xE4, 0xB7}
	},
	{
		2, {0xE2, 0xB6},
		3, {0xE7, 0xAD, 0x9D},
		2, {0x5D, 0x7B},
		2, {0xE4, 0xB8}
	},
	{
		2, {0xE2, 0xB7},
		3, {0xE7, 0xAE, 0x99},
		2, {0x99, 0x7B},
		2, {0xE4, 0xB9}
	},
	{
		2, {0xE2, 0xB8},
		3, {0xE7, 0xAF, 0x8B},
		2, {0xCB, 0x7B},
		2, {0xE4, 0xBA}
	},
	{
		2, {0xE2, 0xB9},
		3, {0xE7, 0xAF, 0x81},
		2, {0xC1, 0x7B},
		2, {0xE4, 0xBB}
	},
	{
		2, {0xE2, 0xBA},
		3, {0xE7, 0xAF, 0x8C},
		2, {0xCC, 0x7B},
		2, {0xE4, 0xBC}
	},
	{
		2, {0xE2, 0xBB},
		3, {0xE7, 0xAF, 0x8F},
		2, {0xCF, 0x7B},
		2, {0xE4, 0xBD}
	},
	{
		2, {0xE2, 0xBC},
		3, {0xE7, 0xAE, 0xB4},
		2, {0xB4, 0x7B},
		2, {0xE4, 0xBE}
	},
	{
		2, {0xE2, 0xBD},
		3, {0xE7, 0xAF, 0x86},
		2, {0xC6, 0x7B},
		2, {0xE4, 0xBF}
	},
	{
		2, {0xE2, 0xBE},
		3, {0xE7, 0xAF, 0x9D},
		2, {0xDD, 0x7B},
		2, {0xE4, 0xC0}
	},
	{
		2, {0xE2, 0xBF},
		3, {0xE7, 0xAF, 0xA9},
		2, {0xE9, 0x7B},
		2, {0xE4, 0xC1}
	},
	{
		2, {0xE2, 0xC0},
		3, {0xE7, 0xB0, 0x91},
		2, {0x11, 0x7C},
		2, {0xE4, 0xC2}
	},
	{
		2, {0xE2, 0xC1},
		3, {0xE7, 0xB0, 0x94},
		2, {0x14, 0x7C},
		2, {0xE4, 0xC3}
	},
	{
		2, {0xE2, 0xC2},
		3, {0xE7, 0xAF, 0xA6},
		2, {0xE6, 0x7B},
		2, {0xE4, 0xC4}
	},
	{
		2, {0xE2, 0xC3},
		3, {0xE7, 0xAF, 0xA5},
		2, {0xE5, 0x7B},
		2, {0xE4, 0xC5}
	},
	{
		2, {0xE2, 0xC4},
		3, {0xE7, 0xB1, 0xA0},
		2, {0x60, 0x7C},
		2, {0xE4, 0xC6}
	},
	{
		2, {0xE2, 0xC5},
		3, {0xE7, 0xB0, 0x80},
		2, {0x00, 0x7C},
		2, {0xE4, 0xC7}
	},
	{
		2, {0xE2, 0xC6},
		3, {0xE7, 0xB0, 0x87},
		2, {0x07, 0x7C},
		2, {0xE4, 0xC8}
	},
	{
		2, {0xE2, 0xC7},
		3, {0xE7, 0xB0, 0x93},
		2, {0x13, 0x7C},
		2, {0xE4, 0xC9}
	},
	{
		2, {0xE2, 0xC8},
		3, {0xE7, 0xAF, 0xB3},
		2, {0xF3, 0x7B},
		2, {0xE4, 0xCA}
	},
	{
		2, {0xE2, 0xC9},
		3, {0xE7, 0xAF, 0xB7},
		2, {0xF7, 0x7B},
		2, {0xE4, 0xCB}
	},
	{
		2, {0xE2, 0xCA},
		3, {0xE7, 0xB0, 0x97},
		2, {0x17, 0x7C},
		2, {0xE4, 0xCC}
	},
	{
		2, {0xE2, 0xCB},
		3, {0xE7, 0xB0, 0x8D},
		2, {0x0D, 0x7C},
		2, {0xE4, 0xCD}
	},
	{
		2, {0xE2, 0xCC},
		3, {0xE7, 0xAF, 0xB6},
		2, {0xF6, 0x7B},
		2, {0xE4, 0xCE}
	},
	{
		2, {0xE2, 0xCD},
		3, {0xE7, 0xB0, 0xA3},
		2, {0x23, 0x7C},
		2, {0xE4, 0xCF}
	},
	{
		2, {0xE2, 0xCE},
		3, {0xE7, 0xB0, 0xA7},
		2, {0x27, 0x7C},
		2, {0xE4, 0xD0}
	},
	{
		2, {0xE2, 0xCF},
		3, {0xE7, 0xB0, 0xAA},
		2, {0x2A, 0x7C},
		2, {0xE4, 0xD1}
	},
	{
		2, {0xE2, 0xD0},
		3, {0xE7, 0xB0, 0x9F},
		2, {0x1F, 0x7C},
		2, {0xE4, 0xD2}
	},
	{
		2, {0xE2, 0xD1},
		3, {0xE7, 0xB0, 0xB7},
		2, {0x37, 0x7C},
		2, {0xE4, 0xD3}
	},
	{
		2, {0xE2, 0xD2},
		3, {0xE7, 0xB0, 0xAB},
		2, {0x2B, 0x7C},
		2, {0xE4, 0xD4}
	},
	{
		2, {0xE2, 0xD3},
		3, {0xE7, 0xB0, 0xBD},
		2, {0x3D, 0x7C},
		2, {0xE4, 0xD5}
	},
	{
		2, {0xE2, 0xD4},
		3, {0xE7, 0xB1, 0x8C},
		2, {0x4C, 0x7C},
		2, {0xE4, 0xD6}
	},
	{
		2, {0xE2, 0xD5},
		3, {0xE7, 0xB1, 0x83},
		2, {0x43, 0x7C},
		2, {0xE4, 0xD7}
	},
	{
		2, {0xE2, 0xD6},
		3, {0xE7, 0xB1, 0x94},
		2, {0x54, 0x7C},
		2, {0xE4, 0xD8}
	},
	{
		2, {0xE2, 0xD7},
		3, {0xE7, 0xB1, 0x8F},
		2, {0x4F, 0x7C},
		2, {0xE4, 0xD9}
	},
	{
		2, {0xE2, 0xD8},
		3, {0xE7, 0xB1, 0x80},
		2, {0x40, 0x7C},
		2, {0xE4, 0xDA}
	},
	{
		2, {0xE2, 0xD9},
		3, {0xE7, 0xB1, 0x90},
		2, {0x50, 0x7C},
		2, {0xE4, 0xDB}
	},
	{
		2, {0xE2, 0xDA},
		3, {0xE7, 0xB1, 0x98},
		2, {0x58, 0x7C},
		2, {0xE4, 0xDC}
	},
	{
		2, {0xE2, 0xDB},
		3, {0xE7, 0xB1, 0x9F},
		2, {0x5F, 0x7C},
		2, {0xE4, 0xDD}
	},
	{
		2, {0xE2, 0xDC},
		3, {0xE7, 0xB1, 0xA4},
		2, {0x64, 0x7C},
		2, {0xE4, 0xDE}
	},
	{
		2, {0xE2, 0xDD},
		3, {0xE7, 0xB1, 0x96},
		2, {0x56, 0x7C},
		2, {0xE4, 0xDF}
	},
	{
		2, {0xE2, 0xDE},
		3, {0xE7, 0xB1, 0xA5},
		2, {0x65, 0x7C},
		2, {0xE4, 0xE0}
	},
	{
		2, {0xE2, 0xDF},
		3, {0xE7, 0xB1, 0xAC},
		2, {0x6C, 0x7C},
		2, {0xE4, 0xE1}
	},
	{
		2, {0xE2, 0xE0},
		3, {0xE7, 0xB1, 0xB5},
		2, {0x75, 0x7C},
		2, {0xE4, 0xE2}
	},
	{
		2, {0xE2, 0xE1},
		3, {0xE7, 0xB2, 0x83},
		2, {0x83, 0x7C},
		2, {0xE4, 0xE3}
	},
	{
		2, {0xE2, 0xE2},
		3, {0xE7, 0xB2, 0x90},
		2, {0x90, 0x7C},
		2, {0xE4, 0xE4}
	},
	{
		2, {0xE2, 0xE3},
		3, {0xE7, 0xB2, 0xA4},
		2, {0xA4, 0x7C},
		2, {0xE4, 0xE5}
	},
	{
		2, {0xE2, 0xE4},
		3, {0xE7, 0xB2, 0xAD},
		2, {0xAD, 0x7C},
		2, {0xE4, 0xE6}
	},
	{
		2, {0xE2, 0xE5},
		3, {0xE7, 0xB2, 0xA2},
		2, {0xA2, 0x7C},
		2, {0xE4, 0xE7}
	},
	{
		2, {0xE2, 0xE6},
		3, {0xE7, 0xB2, 0xAB},
		2, {0xAB, 0x7C},
		2, {0xE4, 0xE8}
	},
	{
		2, {0xE2, 0xE7},
		3, {0xE7, 0xB2, 0xA1},
		2, {0xA1, 0x7C},
		2, {0xE4, 0xE9}
	},
	{
		2, {0xE2, 0xE8},
		3, {0xE7, 0xB2, 0xA8},
		2, {0xA8, 0x7C},
		2, {0xE4, 0xEA}
	},
	{
		2, {0xE2, 0xE9},
		3, {0xE7, 0xB2, 0xB3},
		2, {0xB3, 0x7C},
		2, {0xE4, 0xEB}
	},
	{
		2, {0xE2, 0xEA},
		3, {0xE7, 0xB2, 0xB2},
		2, {0xB2, 0x7C},
		2, {0xE4, 0xEC}
	},
	{
		2, {0xE2, 0xEB},
		3, {0xE7, 0xB2, 0xB1},
		2, {0xB1, 0x7C},
		2, {0xE4, 0xED}
	},
	{
		2, {0xE2, 0xEC},
		3, {0xE7, 0xB2, 0xAE},
		2, {0xAE, 0x7C},
		2, {0xE4, 0xEE}
	},
	{
		2, {0xE2, 0xED},
		3, {0xE7, 0xB2, 0xB9},
		2, {0xB9, 0x7C},
		2, {0xE4, 0xEF}
	},
	{
		2, {0xE2, 0xEE},
		3, {0xE7, 0xB2, 0xBD},
		2, {0xBD, 0x7C},
		2, {0xE4, 0xF0}
	},
	{
		2, {0xE2, 0xEF},
		3, {0xE7, 0xB3, 0x80},
		2, {0xC0, 0x7C},
		2, {0xE4, 0xF1}
	},
	{
		2, {0xE2, 0xF0},
		3, {0xE7, 0xB3, 0x85},
		2, {0xC5, 0x7C},
		2, {0xE4, 0xF2}
	},
	{
		2, {0xE2, 0xF1},
		3, {0xE7, 0xB3, 0x82},
		2, {0xC2, 0x7C},
		2, {0xE4, 0xF3}
	},
	{
		2, {0xE2, 0xF2},
		3, {0xE7, 0xB3, 0x98},
		2, {0xD8, 0x7C},
		2, {0xE4, 0xF4}
	},
	{
		2, {0xE2, 0xF3},
		3, {0xE7, 0xB3, 0x92},
		2, {0xD2, 0x7C},
		2, {0xE4, 0xF5}
	},
	{
		2, {0xE2, 0xF4},
		3, {0xE7, 0xB3, 0x9C},
		2, {0xDC, 0x7C},
		2, {0xE4, 0xF6}
	},
	{
		2, {0xE2, 0xF5},
		3, {0xE7, 0xB3, 0xA2},
		2, {0xE2, 0x7C},
		2, {0xE4, 0xF7}
	},
	{
		2, {0xE2, 0xF6},
		3, {0xE9, 0xAC, 0xBB},
		2, {0x3B, 0x9B},
		2, {0xE4, 0xF8}
	},
	{
		2, {0xE2, 0xF7},
		3, {0xE7, 0xB3, 0xAF},
		2, {0xEF, 0x7C},
		2, {0xE4, 0xF9}
	},
	{
		2, {0xE2, 0xF8},
		3, {0xE7, 0xB3, 0xB2},
		2, {0xF2, 0x7C},
		2, {0xE4, 0xFA}
	},
	{
		2, {0xE2, 0xF9},
		3, {0xE7, 0xB3, 0xB4},
		2, {0xF4, 0x7C},
		2, {0xE4, 0xFB}
	},
	{
		2, {0xE2, 0xFA},
		3, {0xE7, 0xB3, 0xB6},
		2, {0xF6, 0x7C},
		2, {0xE4, 0xFC}
	},
	{
		2, {0xE2, 0xFB},
		3, {0xE7, 0xB3, 0xBA},
		2, {0xFA, 0x7C},
		2, {0xE4, 0xFD}
	},
	{
		2, {0xE2, 0xFC},
		3, {0xE7, 0xB4, 0x86},
		2, {0x06, 0x7D},
		2, {0xE4, 0xFE}
	},
	{
		2, {0xE3, 0x40},
		3, {0xE7, 0xB4, 0x82},
		2, {0x02, 0x7D},
		2, {0xE5, 0xA1}
	},
	{
		2, {0xE3, 0x41},
		3, {0xE7, 0xB4, 0x9C},
		2, {0x1C, 0x7D},
		2, {0xE5, 0xA2}
	},
	{
		2, {0xE3, 0x42},
		3, {0xE7, 0xB4, 0x95},
		2, {0x15, 0x7D},
		2, {0xE5, 0xA3}
	},
	{
		2, {0xE3, 0x43},
		3, {0xE7, 0xB4, 0x8A},
		2, {0x0A, 0x7D},
		2, {0xE5, 0xA4}
	},
	{
		2, {0xE3, 0x44},
		3, {0xE7, 0xB5, 0x85},
		2, {0x45, 0x7D},
		2, {0xE5, 0xA5}
	},
	{
		2, {0xE3, 0x45},
		3, {0xE7, 0xB5, 0x8B},
		2, {0x4B, 0x7D},
		2, {0xE5, 0xA6}
	},
	{
		2, {0xE3, 0x46},
		3, {0xE7, 0xB4, 0xAE},
		2, {0x2E, 0x7D},
		2, {0xE5, 0xA7}
	},
	{
		2, {0xE3, 0x47},
		3, {0xE7, 0xB4, 0xB2},
		2, {0x32, 0x7D},
		2, {0xE5, 0xA8}
	},
	{
		2, {0xE3, 0x48},
		3, {0xE7, 0xB4, 0xBF},
		2, {0x3F, 0x7D},
		2, {0xE5, 0xA9}
	},
	{
		2, {0xE3, 0x49},
		3, {0xE7, 0xB4, 0xB5},
		2, {0x35, 0x7D},
		2, {0xE5, 0xAA}
	},
	{
		2, {0xE3, 0x4A},
		3, {0xE7, 0xB5, 0x86},
		2, {0x46, 0x7D},
		2, {0xE5, 0xAB}
	},
	{
		2, {0xE3, 0x4B},
		3, {0xE7, 0xB5, 0xB3},
		2, {0x73, 0x7D},
		2, {0xE5, 0xAC}
	},
	{
		2, {0xE3, 0x4C},
		3, {0xE7, 0xB5, 0x96},
		2, {0x56, 0x7D},
		2, {0xE5, 0xAD}
	},
	{
		2, {0xE3, 0x4D},
		3, {0xE7, 0xB5, 0x8E},
		2, {0x4E, 0x7D},
		2, {0xE5, 0xAE}
	},
	{
		2, {0xE3, 0x4E},
		3, {0xE7, 0xB5, 0xB2},
		2, {0x72, 0x7D},
		2, {0xE5, 0xAF}
	},
	{
		2, {0xE3, 0x4F},
		3, {0xE7, 0xB5, 0xA8},
		2, {0x68, 0x7D},
		2, {0xE5, 0xB0}
	},
	{
		2, {0xE3, 0x50},
		3, {0xE7, 0xB5, 0xAE},
		2, {0x6E, 0x7D},
		2, {0xE5, 0xB1}
	},
	{
		2, {0xE3, 0x51},
		3, {0xE7, 0xB5, 0x8F},
		2, {0x4F, 0x7D},
		2, {0xE5, 0xB2}
	},
	{
		2, {0xE3, 0x52},
		3, {0xE7, 0xB5, 0xA3},
		2, {0x63, 0x7D},
		2, {0xE5, 0xB3}
	},
	{
		2, {0xE3, 0x53},
		3, {0xE7, 0xB6, 0x93},
		2, {0x93, 0x7D},
		2, {0xE5, 0xB4}
	},
	{
		2, {0xE3, 0x54},
		3, {0xE7, 0xB6, 0x89},
		2, {0x89, 0x7D},
		2, {0xE5, 0xB5}
	},
	{
		2, {0xE3, 0x55},
		3, {0xE7, 0xB5, 0x9B},
		2, {0x5B, 0x7D},
		2, {0xE5, 0xB6}
	},
	{
		2, {0xE3, 0x56},
		3, {0xE7, 0xB6, 0x8F},
		2, {0x8F, 0x7D},
		2, {0xE5, 0xB7}
	},
	{
		2, {0xE3, 0x57},
		3, {0xE7, 0xB5, 0xBD},
		2, {0x7D, 0x7D},
		2, {0xE5, 0xB8}
	},
	{
		2, {0xE3, 0x58},
		3, {0xE7, 0xB6, 0x9B},
		2, {0x9B, 0x7D},
		2, {0xE5, 0xB9}
	},
	{
		2, {0xE3, 0x59},
		3, {0xE7, 0xB6, 0xBA},
		2, {0xBA, 0x7D},
		2, {0xE5, 0xBA}
	},
	{
		2, {0xE3, 0x5A},
		3, {0xE7, 0xB6, 0xAE},
		2, {0xAE, 0x7D},
		2, {0xE5, 0xBB}
	},
	{
		2, {0xE3, 0x5B},
		3, {0xE7, 0xB6, 0xA3},
		2, {0xA3, 0x7D},
		2, {0xE5, 0xBC}
	},
	{
		2, {0xE3, 0x5C},
		3, {0xE7, 0xB6, 0xB5},
		2, {0xB5, 0x7D},
		2, {0xE5, 0xBD}
	},
	{
		2, {0xE3, 0x5D},
		3, {0xE7, 0xB7, 0x87},
		2, {0xC7, 0x7D},
		2, {0xE5, 0xBE}
	},
	{
		2, {0xE3, 0x5E},
		3, {0xE7, 0xB6, 0xBD},
		2, {0xBD, 0x7D},
		2, {0xE5, 0xBF}
	},
	{
		2, {0xE3, 0x5F},
		3, {0xE7, 0xB6, 0xAB},
		2, {0xAB, 0x7D},
		2, {0xE5, 0xC0}
	},
	{
		2, {0xE3, 0x60},
		3, {0xE7, 0xB8, 0xBD},
		2, {0x3D, 0x7E},
		2, {0xE5, 0xC1}
	},
	{
		2, {0xE3, 0x61},
		3, {0xE7, 0xB6, 0xA2},
		2, {0xA2, 0x7D},
		2, {0xE5, 0xC2}
	},
	{
		2, {0xE3, 0x62},
		3, {0xE7, 0xB6, 0xAF},
		2, {0xAF, 0x7D},
		2, {0xE5, 0xC3}
	},
	{
		2, {0xE3, 0x63},
		3, {0xE7, 0xB7, 0x9C},
		2, {0xDC, 0x7D},
		2, {0xE5, 0xC4}
	},
	{
		2, {0xE3, 0x64},
		3, {0xE7, 0xB6, 0xB8},
		2, {0xB8, 0x7D},
		2, {0xE5, 0xC5}
	},
	{
		2, {0xE3, 0x65},
		3, {0xE7, 0xB6, 0x9F},
		2, {0x9F, 0x7D},
		2, {0xE5, 0xC6}
	},
	{
		2, {0xE3, 0x66},
		3, {0xE7, 0xB6, 0xB0},
		2, {0xB0, 0x7D},
		2, {0xE5, 0xC7}
	},
	{
		2, {0xE3, 0x67},
		3, {0xE7, 0xB7, 0x98},
		2, {0xD8, 0x7D},
		2, {0xE5, 0xC8}
	},
	{
		2, {0xE3, 0x68},
		3, {0xE7, 0xB7, 0x9D},
		2, {0xDD, 0x7D},
		2, {0xE5, 0xC9}
	},
	{
		2, {0xE3, 0x69},
		3, {0xE7, 0xB7, 0xA4},
		2, {0xE4, 0x7D},
		2, {0xE5, 0xCA}
	},
	{
		2, {0xE3, 0x6A},
		3, {0xE7, 0xB7, 0x9E},
		2, {0xDE, 0x7D},
		2, {0xE5, 0xCB}
	},
	{
		2, {0xE3, 0x6B},
		3, {0xE7, 0xB7, 0xBB},
		2, {0xFB, 0x7D},
		2, {0xE5, 0xCC}
	},
	{
		2, {0xE3, 0x6C},
		3, {0xE7, 0xB7, 0xB2},
		2, {0xF2, 0x7D},
		2, {0xE5, 0xCD}
	},
	{
		2, {0xE3, 0x6D},
		3, {0xE7, 0xB7, 0xA1},
		2, {0xE1, 0x7D},
		2, {0xE5, 0xCE}
	},
	{
		2, {0xE3, 0x6E},
		3, {0xE7, 0xB8, 0x85},
		2, {0x05, 0x7E},
		2, {0xE5, 0xCF}
	},
	{
		2, {0xE3, 0x6F},
		3, {0xE7, 0xB8, 0x8A},
		2, {0x0A, 0x7E},
		2, {0xE5, 0xD0}
	},
	{
		2, {0xE3, 0x70},
		3, {0xE7, 0xB8, 0xA3},
		2, {0x23, 0x7E},
		2, {0xE5, 0xD1}
	},
	{
		2, {0xE3, 0x71},
		3, {0xE7, 0xB8, 0xA1},
		2, {0x21, 0x7E},
		2, {0xE5, 0xD2}
	},
	{
		2, {0xE3, 0x72},
		3, {0xE7, 0xB8, 0x92},
		2, {0x12, 0x7E},
		2, {0xE5, 0xD3}
	},
	{
		2, {0xE3, 0x73},
		3, {0xE7, 0xB8, 0xB1},
		2, {0x31, 0x7E},
		2, {0xE5, 0xD4}
	},
	{
		2, {0xE3, 0x74},
		3, {0xE7, 0xB8, 0x9F},
		2, {0x1F, 0x7E},
		2, {0xE5, 0xD5}
	},
	{
		2, {0xE3, 0x75},
		3, {0xE7, 0xB8, 0x89},
		2, {0x09, 0x7E},
		2, {0xE5, 0xD6}
	},
	{
		2, {0xE3, 0x76},
		3, {0xE7, 0xB8, 0x8B},
		2, {0x0B, 0x7E},
		2, {0xE5, 0xD7}
	},
	{
		2, {0xE3, 0x77},
		3, {0xE7, 0xB8, 0xA2},
		2, {0x22, 0x7E},
		2, {0xE5, 0xD8}
	},
	{
		2, {0xE3, 0x78},
		3, {0xE7, 0xB9, 0x86},
		2, {0x46, 0x7E},
		2, {0xE5, 0xD9}
	},
	{
		2, {0xE3, 0x79},
		3, {0xE7, 0xB9, 0xA6},
		2, {0x66, 0x7E},
		2, {0xE5, 0xDA}
	},
	{
		2, {0xE3, 0x7A},
		3, {0xE7, 0xB8, 0xBB},
		2, {0x3B, 0x7E},
		2, {0xE5, 0xDB}
	},
	{
		2, {0xE3, 0x7B},
		3, {0xE7, 0xB8, 0xB5},
		2, {0x35, 0x7E},
		2, {0xE5, 0xDC}
	},
	{
		2, {0xE3, 0x7C},
		3, {0xE7, 0xB8, 0xB9},
		2, {0x39, 0x7E},
		2, {0xE5, 0xDD}
	},
	{
		2, {0xE3, 0x7D},
		3, {0xE7, 0xB9, 0x83},
		2, {0x43, 0x7E},
		2, {0xE5, 0xDE}
	},
	{
		2, {0xE3, 0x7E},
		3, {0xE7, 0xB8, 0xB7},
		2, {0x37, 0x7E},
		2, {0xE5, 0xDF}
	},
	{
		2, {0xE3, 0x80},
		3, {0xE7, 0xB8, 0xB2},
		2, {0x32, 0x7E},
		2, {0xE5, 0xE0}
	},
	{
		2, {0xE3, 0x81},
		3, {0xE7, 0xB8, 0xBA},
		2, {0x3A, 0x7E},
		2, {0xE5, 0xE1}
	},
	{
		2, {0xE3, 0x82},
		3, {0xE7, 0xB9, 0xA7},
		2, {0x67, 0x7E},
		2, {0xE5, 0xE2}
	},
	{
		2, {0xE3, 0x83},
		3, {0xE7, 0xB9, 0x9D},
		2, {0x5D, 0x7E},
		2, {0xE5, 0xE3}
	},
	{
		2, {0xE3, 0x84},
		3, {0xE7, 0xB9, 0x96},
		2, {0x56, 0x7E},
		2, {0xE5, 0xE4}
	},
	{
		2, {0xE3, 0x85},
		3, {0xE7, 0xB9, 0x9E},
		2, {0x5E, 0x7E},
		2, {0xE5, 0xE5}
	},
	{
		2, {0xE3, 0x86},
		3, {0xE7, 0xB9, 0x99},
		2, {0x59, 0x7E},
		2, {0xE5, 0xE6}
	},
	{
		2, {0xE3, 0x87},
		3, {0xE7, 0xB9, 0x9A},
		2, {0x5A, 0x7E},
		2, {0xE5, 0xE7}
	},
	{
		2, {0xE3, 0x88},
		3, {0xE7, 0xB9, 0xB9},
		2, {0x79, 0x7E},
		2, {0xE5, 0xE8}
	},
	{
		2, {0xE3, 0x89},
		3, {0xE7, 0xB9, 0xAA},
		2, {0x6A, 0x7E},
		2, {0xE5, 0xE9}
	},
	{
		2, {0xE3, 0x8A},
		3, {0xE7, 0xB9, 0xA9},
		2, {0x69, 0x7E},
		2, {0xE5, 0xEA}
	},
	{
		2, {0xE3, 0x8B},
		3, {0xE7, 0xB9, 0xBC},
		2, {0x7C, 0x7E},
		2, {0xE5, 0xEB}
	},
	{
		2, {0xE3, 0x8C},
		3, {0xE7, 0xB9, 0xBB},
		2, {0x7B, 0x7E},
		2, {0xE5, 0xEC}
	},
	{
		2, {0xE3, 0x8D},
		3, {0xE7, 0xBA, 0x83},
		2, {0x83, 0x7E},
		2, {0xE5, 0xED}
	},
	{
		2, {0xE3, 0x8E},
		3, {0xE7, 0xB7, 0x95},
		2, {0xD5, 0x7D},
		2, {0xE5, 0xEE}
	},
	{
		2, {0xE3, 0x8F},
		3, {0xE7, 0xB9, 0xBD},
		2, {0x7D, 0x7E},
		2, {0xE5, 0xEF}
	},
	{
		2, {0xE3, 0x90},
		3, {0xE8, 0xBE, 0xAE},
		2, {0xAE, 0x8F},
		2, {0xE5, 0xF0}
	},
	{
		2, {0xE3, 0x91},
		3, {0xE7, 0xB9, 0xBF},
		2, {0x7F, 0x7E},
		2, {0xE5, 0xF1}
	},
	{
		2, {0xE3, 0x92},
		3, {0xE7, 0xBA, 0x88},
		2, {0x88, 0x7E},
		2, {0xE5, 0xF2}
	},
	{
		2, {0xE3, 0x93},
		3, {0xE7, 0xBA, 0x89},
		2, {0x89, 0x7E},
		2, {0xE5, 0xF3}
	},
	{
		2, {0xE3, 0x94},
		3, {0xE7, 0xBA, 0x8C},
		2, {0x8C, 0x7E},
		2, {0xE5, 0xF4}
	},
	{
		2, {0xE3, 0x95},
		3, {0xE7, 0xBA, 0x92},
		2, {0x92, 0x7E},
		2, {0xE5, 0xF5}
	},
	{
		2, {0xE3, 0x96},
		3, {0xE7, 0xBA, 0x90},
		2, {0x90, 0x7E},
		2, {0xE5, 0xF6}
	},
	{
		2, {0xE3, 0x97},
		3, {0xE7, 0xBA, 0x93},
		2, {0x93, 0x7E},
		2, {0xE5, 0xF7}
	},
	{
		2, {0xE3, 0x98},
		3, {0xE7, 0xBA, 0x94},
		2, {0x94, 0x7E},
		2, {0xE5, 0xF8}
	},
	{
		2, {0xE3, 0x99},
		3, {0xE7, 0xBA, 0x96},
		2, {0x96, 0x7E},
		2, {0xE5, 0xF9}
	},
	{
		2, {0xE3, 0x9A},
		3, {0xE7, 0xBA, 0x8E},
		2, {0x8E, 0x7E},
		2, {0xE5, 0xFA}
	},
	{
		2, {0xE3, 0x9B},
		3, {0xE7, 0xBA, 0x9B},
		2, {0x9B, 0x7E},
		2, {0xE5, 0xFB}
	},
	{
		2, {0xE3, 0x9C},
		3, {0xE7, 0xBA, 0x9C},
		2, {0x9C, 0x7E},
		2, {0xE5, 0xFC}
	},
	{
		2, {0xE3, 0x9D},
		3, {0xE7, 0xBC, 0xB8},
		2, {0x38, 0x7F},
		2, {0xE5, 0xFD}
	},
	{
		2, {0xE3, 0x9E},
		3, {0xE7, 0xBC, 0xBA},
		2, {0x3A, 0x7F},
		2, {0xE5, 0xFE}
	},
	{
		2, {0xE3, 0x9F},
		3, {0xE7, 0xBD, 0x85},
		2, {0x45, 0x7F},
		2, {0xE6, 0xA1}
	},
	{
		2, {0xE3, 0xA0},
		3, {0xE7, 0xBD, 0x8C},
		2, {0x4C, 0x7F},
		2, {0xE6, 0xA2}
	},
	{
		2, {0xE3, 0xA1},
		3, {0xE7, 0xBD, 0x8D},
		2, {0x4D, 0x7F},
		2, {0xE6, 0xA3}
	},
	{
		2, {0xE3, 0xA2},
		3, {0xE7, 0xBD, 0x8E},
		2, {0x4E, 0x7F},
		2, {0xE6, 0xA4}
	},
	{
		2, {0xE3, 0xA3},
		3, {0xE7, 0xBD, 0x90},
		2, {0x50, 0x7F},
		2, {0xE6, 0xA5}
	},
	{
		2, {0xE3, 0xA4},
		3, {0xE7, 0xBD, 0x91},
		2, {0x51, 0x7F},
		2, {0xE6, 0xA6}
	},
	{
		2, {0xE3, 0xA5},
		3, {0xE7, 0xBD, 0x95},
		2, {0x55, 0x7F},
		2, {0xE6, 0xA7}
	},
	{
		2, {0xE3, 0xA6},
		3, {0xE7, 0xBD, 0x94},
		2, {0x54, 0x7F},
		2, {0xE6, 0xA8}
	},
	{
		2, {0xE3, 0xA7},
		3, {0xE7, 0xBD, 0x98},
		2, {0x58, 0x7F},
		2, {0xE6, 0xA9}
	},
	{
		2, {0xE3, 0xA8},
		3, {0xE7, 0xBD, 0x9F},
		2, {0x5F, 0x7F},
		2, {0xE6, 0xAA}
	},
	{
		2, {0xE3, 0xA9},
		3, {0xE7, 0xBD, 0xA0},
		2, {0x60, 0x7F},
		2, {0xE6, 0xAB}
	},
	{
		2, {0xE3, 0xAA},
		3, {0xE7, 0xBD, 0xA8},
		2, {0x68, 0x7F},
		2, {0xE6, 0xAC}
	},
	{
		2, {0xE3, 0xAB},
		3, {0xE7, 0xBD, 0xA9},
		2, {0x69, 0x7F},
		2, {0xE6, 0xAD}
	},
	{
		2, {0xE3, 0xAC},
		3, {0xE7, 0xBD, 0xA7},
		2, {0x67, 0x7F},
		2, {0xE6, 0xAE}
	},
	{
		2, {0xE3, 0xAD},
		3, {0xE7, 0xBD, 0xB8},
		2, {0x78, 0x7F},
		2, {0xE6, 0xAF}
	},
	{
		2, {0xE3, 0xAE},
		3, {0xE7, 0xBE, 0x82},
		2, {0x82, 0x7F},
		2, {0xE6, 0xB0}
	},
	{
		2, {0xE3, 0xAF},
		3, {0xE7, 0xBE, 0x86},
		2, {0x86, 0x7F},
		2, {0xE6, 0xB1}
	},
	{
		2, {0xE3, 0xB0},
		3, {0xE7, 0xBE, 0x83},
		2, {0x83, 0x7F},
		2, {0xE6, 0xB2}
	},
	{
		2, {0xE3, 0xB1},
		3, {0xE7, 0xBE, 0x88},
		2, {0x88, 0x7F},
		2, {0xE6, 0xB3}
	},
	{
		2, {0xE3, 0xB2},
		3, {0xE7, 0xBE, 0x87},
		2, {0x87, 0x7F},
		2, {0xE6, 0xB4}
	},
	{
		2, {0xE3, 0xB3},
		3, {0xE7, 0xBE, 0x8C},
		2, {0x8C, 0x7F},
		2, {0xE6, 0xB5}
	},
	{
		2, {0xE3, 0xB4},
		3, {0xE7, 0xBE, 0x94},
		2, {0x94, 0x7F},
		2, {0xE6, 0xB6}
	},
	{
		2, {0xE3, 0xB5},
		3, {0xE7, 0xBE, 0x9E},
		2, {0x9E, 0x7F},
		2, {0xE6, 0xB7}
	},
	{
		2, {0xE3, 0xB6},
		3, {0xE7, 0xBE, 0x9D},
		2, {0x9D, 0x7F},
		2, {0xE6, 0xB8}
	},
	{
		2, {0xE3, 0xB7},
		3, {0xE7, 0xBE, 0x9A},
		2, {0x9A, 0x7F},
		2, {0xE6, 0xB9}
	},
	{
		2, {0xE3, 0xB8},
		3, {0xE7, 0xBE, 0xA3},
		2, {0xA3, 0x7F},
		2, {0xE6, 0xBA}
	},
	{
		2, {0xE3, 0xB9},
		3, {0xE7, 0xBE, 0xAF},
		2, {0xAF, 0x7F},
		2, {0xE6, 0xBB}
	},
	{
		2, {0xE3, 0xBA},
		3, {0xE7, 0xBE, 0xB2},
		2, {0xB2, 0x7F},
		2, {0xE6, 0xBC}
	},
	{
		2, {0xE3, 0xBB},
		3, {0xE7, 0xBE, 0xB9},
		2, {0xB9, 0x7F},
		2, {0xE6, 0xBD}
	},
	{
		2, {0xE3, 0xBC},
		3, {0xE7, 0xBE, 0xAE},
		2, {0xAE, 0x7F},
		2, {0xE6, 0xBE}
	},
	{
		2, {0xE3, 0xBD},
		3, {0xE7, 0xBE, 0xB6},
		2, {0xB6, 0x7F},
		2, {0xE6, 0xBF}
	},
	{
		2, {0xE3, 0xBE},
		3, {0xE7, 0xBE, 0xB8},
		2, {0xB8, 0x7F},
		2, {0xE6, 0xC0}
	},
	{
		2, {0xE3, 0xBF},
		3, {0xE8, 0xAD, 0xB1},
		2, {0x71, 0x8B},
		2, {0xE6, 0xC1}
	},
	{
		2, {0xE3, 0xC0},
		3, {0xE7, 0xBF, 0x85},
		2, {0xC5, 0x7F},
		2, {0xE6, 0xC2}
	},
	{
		2, {0xE3, 0xC1},
		3, {0xE7, 0xBF, 0x86},
		2, {0xC6, 0x7F},
		2, {0xE6, 0xC3}
	},
	{
		2, {0xE3, 0xC2},
		3, {0xE7, 0xBF, 0x8A},
		2, {0xCA, 0x7F},
		2, {0xE6, 0xC4}
	},
	{
		2, {0xE3, 0xC3},
		3, {0xE7, 0xBF, 0x95},
		2, {0xD5, 0x7F},
		2, {0xE6, 0xC5}
	},
	{
		2, {0xE3, 0xC4},
		3, {0xE7, 0xBF, 0x94},
		2, {0xD4, 0x7F},
		2, {0xE6, 0xC6}
	},
	{
		2, {0xE3, 0xC5},
		3, {0xE7, 0xBF, 0xA1},
		2, {0xE1, 0x7F},
		2, {0xE6, 0xC7}
	},
	{
		2, {0xE3, 0xC6},
		3, {0xE7, 0xBF, 0xA6},
		2, {0xE6, 0x7F},
		2, {0xE6, 0xC8}
	},
	{
		2, {0xE3, 0xC7},
		3, {0xE7, 0xBF, 0xA9},
		2, {0xE9, 0x7F},
		2, {0xE6, 0xC9}
	},
	{
		2, {0xE3, 0xC8},
		3, {0xE7, 0xBF, 0xB3},
		2, {0xF3, 0x7F},
		2, {0xE6, 0xCA}
	},
	{
		2, {0xE3, 0xC9},
		3, {0xE7, 0xBF, 0xB9},
		2, {0xF9, 0x7F},
		2, {0xE6, 0xCB}
	},
	{
		2, {0xE3, 0xCA},
		3, {0xE9, 0xA3, 0x9C},
		2, {0xDC, 0x98},
		2, {0xE6, 0xCC}
	},
	{
		2, {0xE3, 0xCB},
		3, {0xE8, 0x80, 0x86},
		2, {0x06, 0x80},
		2, {0xE6, 0xCD}
	},
	{
		2, {0xE3, 0xCC},
		3, {0xE8, 0x80, 0x84},
		2, {0x04, 0x80},
		2, {0xE6, 0xCE}
	},
	{
		2, {0xE3, 0xCD},
		3, {0xE8, 0x80, 0x8B},
		2, {0x0B, 0x80},
		2, {0xE6, 0xCF}
	},
	{
		2, {0xE3, 0xCE},
		3, {0xE8, 0x80, 0x92},
		2, {0x12, 0x80},
		2, {0xE6, 0xD0}
	},
	{
		2, {0xE3, 0xCF},
		3, {0xE8, 0x80, 0x98},
		2, {0x18, 0x80},
		2, {0xE6, 0xD1}
	},
	{
		2, {0xE3, 0xD0},
		3, {0xE8, 0x80, 0x99},
		2, {0x19, 0x80},
		2, {0xE6, 0xD2}
	},
	{
		2, {0xE3, 0xD1},
		3, {0xE8, 0x80, 0x9C},
		2, {0x1C, 0x80},
		2, {0xE6, 0xD3}
	},
	{
		2, {0xE3, 0xD2},
		3, {0xE8, 0x80, 0xA1},
		2, {0x21, 0x80},
		2, {0xE6, 0xD4}
	},
	{
		2, {0xE3, 0xD3},
		3, {0xE8, 0x80, 0xA8},
		2, {0x28, 0x80},
		2, {0xE6, 0xD5}
	},
	{
		2, {0xE3, 0xD4},
		3, {0xE8, 0x80, 0xBF},
		2, {0x3F, 0x80},
		2, {0xE6, 0xD6}
	},
	{
		2, {0xE3, 0xD5},
		3, {0xE8, 0x80, 0xBB},
		2, {0x3B, 0x80},
		2, {0xE6, 0xD7}
	},
	{
		2, {0xE3, 0xD6},
		3, {0xE8, 0x81, 0x8A},
		2, {0x4A, 0x80},
		2, {0xE6, 0xD8}
	},
	{
		2, {0xE3, 0xD7},
		3, {0xE8, 0x81, 0x86},
		2, {0x46, 0x80},
		2, {0xE6, 0xD9}
	},
	{
		2, {0xE3, 0xD8},
		3, {0xE8, 0x81, 0x92},
		2, {0x52, 0x80},
		2, {0xE6, 0xDA}
	},
	{
		2, {0xE3, 0xD9},
		3, {0xE8, 0x81, 0x98},
		2, {0x58, 0x80},
		2, {0xE6, 0xDB}
	},
	{
		2, {0xE3, 0xDA},
		3, {0xE8, 0x81, 0x9A},
		2, {0x5A, 0x80},
		2, {0xE6, 0xDC}
	},
	{
		2, {0xE3, 0xDB},
		3, {0xE8, 0x81, 0x9F},
		2, {0x5F, 0x80},
		2, {0xE6, 0xDD}
	},
	{
		2, {0xE3, 0xDC},
		3, {0xE8, 0x81, 0xA2},
		2, {0x62, 0x80},
		2, {0xE6, 0xDE}
	},
	{
		2, {0xE3, 0xDD},
		3, {0xE8, 0x81, 0xA8},
		2, {0x68, 0x80},
		2, {0xE6, 0xDF}
	},
	{
		2, {0xE3, 0xDE},
		3, {0xE8, 0x81, 0xB3},
		2, {0x73, 0x80},
		2, {0xE6, 0xE0}
	},
	{
		2, {0xE3, 0xDF},
		3, {0xE8, 0x81, 0xB2},
		2, {0x72, 0x80},
		2, {0xE6, 0xE1}
	},
	{
		2, {0xE3, 0xE0},
		3, {0xE8, 0x81, 0xB0},
		2, {0x70, 0x80},
		2, {0xE6, 0xE2}
	},
	{
		2, {0xE3, 0xE1},
		3, {0xE8, 0x81, 0xB6},
		2, {0x76, 0x80},
		2, {0xE6, 0xE3}
	},
	{
		2, {0xE3, 0xE2},
		3, {0xE8, 0x81, 0xB9},
		2, {0x79, 0x80},
		2, {0xE6, 0xE4}
	},
	{
		2, {0xE3, 0xE3},
		3, {0xE8, 0x81, 0xBD},
		2, {0x7D, 0x80},
		2, {0xE6, 0xE5}
	},
	{
		2, {0xE3, 0xE4},
		3, {0xE8, 0x81, 0xBF},
		2, {0x7F, 0x80},
		2, {0xE6, 0xE6}
	},
	{
		2, {0xE3, 0xE5},
		3, {0xE8, 0x82, 0x84},
		2, {0x84, 0x80},
		2, {0xE6, 0xE7}
	},
	{
		2, {0xE3, 0xE6},
		3, {0xE8, 0x82, 0x86},
		2, {0x86, 0x80},
		2, {0xE6, 0xE8}
	},
	{
		2, {0xE3, 0xE7},
		3, {0xE8, 0x82, 0x85},
		2, {0x85, 0x80},
		2, {0xE6, 0xE9}
	},
	{
		2, {0xE3, 0xE8},
		3, {0xE8, 0x82, 0x9B},
		2, {0x9B, 0x80},
		2, {0xE6, 0xEA}
	},
	{
		2, {0xE3, 0xE9},
		3, {0xE8, 0x82, 0x93},
		2, {0x93, 0x80},
		2, {0xE6, 0xEB}
	},
	{
		2, {0xE3, 0xEA},
		3, {0xE8, 0x82, 0x9A},
		2, {0x9A, 0x80},
		2, {0xE6, 0xEC}
	},
	{
		2, {0xE3, 0xEB},
		3, {0xE8, 0x82, 0xAD},
		2, {0xAD, 0x80},
		2, {0xE6, 0xED}
	},
	{
		2, {0xE3, 0xEC},
		3, {0xE5, 0x86, 0x90},
		2, {0x90, 0x51},
		2, {0xE6, 0xEE}
	},
	{
		2, {0xE3, 0xED},
		3, {0xE8, 0x82, 0xAC},
		2, {0xAC, 0x80},
		2, {0xE6, 0xEF}
	},
	{
		2, {0xE3, 0xEE},
		3, {0xE8, 0x83, 0x9B},
		2, {0xDB, 0x80},
		2, {0xE6, 0xF0}
	},
	{
		2, {0xE3, 0xEF},
		3, {0xE8, 0x83, 0xA5},
		2, {0xE5, 0x80},
		2, {0xE6, 0xF1}
	},
	{
		2, {0xE3, 0xF0},
		3, {0xE8, 0x83, 0x99},
		2, {0xD9, 0x80},
		2, {0xE6, 0xF2}
	},
	{
		2, {0xE3, 0xF1},
		3, {0xE8, 0x83, 0x9D},
		2, {0xDD, 0x80},
		2, {0xE6, 0xF3}
	},
	{
		2, {0xE3, 0xF2},
		3, {0xE8, 0x83, 0x84},
		2, {0xC4, 0x80},
		2, {0xE6, 0xF4}
	},
	{
		2, {0xE3, 0xF3},
		3, {0xE8, 0x83, 0x9A},
		2, {0xDA, 0x80},
		2, {0xE6, 0xF5}
	},
	{
		2, {0xE3, 0xF4},
		3, {0xE8, 0x83, 0x96},
		2, {0xD6, 0x80},
		2, {0xE6, 0xF6}
	},
	{
		2, {0xE3, 0xF5},
		3, {0xE8, 0x84, 0x89},
		2, {0x09, 0x81},
		2, {0xE6, 0xF7}
	},
	{
		2, {0xE3, 0xF6},
		3, {0xE8, 0x83, 0xAF},
		2, {0xEF, 0x80},
		2, {0xE6, 0xF8}
	},
	{
		2, {0xE3, 0xF7},
		3, {0xE8, 0x83, 0xB1},
		2, {0xF1, 0x80},
		2, {0xE6, 0xF9}
	},
	{
		2, {0xE3, 0xF8},
		3, {0xE8, 0x84, 0x9B},
		2, {0x1B, 0x81},
		2, {0xE6, 0xFA}
	},
	{
		2, {0xE3, 0xF9},
		3, {0xE8, 0x84, 0xA9},
		2, {0x29, 0x81},
		2, {0xE6, 0xFB}
	},
	{
		2, {0xE3, 0xFA},
		3, {0xE8, 0x84, 0xA3},
		2, {0x23, 0x81},
		2, {0xE6, 0xFC}
	},
	{
		2, {0xE3, 0xFB},
		3, {0xE8, 0x84, 0xAF},
		2, {0x2F, 0x81},
		2, {0xE6, 0xFD}
	},
	{
		2, {0xE3, 0xFC},
		3, {0xE8, 0x85, 0x8B},
		2, {0x4B, 0x81},
		2, {0xE6, 0xFE}
	},
	{
		2, {0xE4, 0x40},
		3, {0xE9, 0x9A, 0x8B},
		2, {0x8B, 0x96},
		2, {0xE7, 0xA1}
	},
	{
		2, {0xE4, 0x41},
		3, {0xE8, 0x85, 0x86},
		2, {0x46, 0x81},
		2, {0xE7, 0xA2}
	},
	{
		2, {0xE4, 0x42},
		3, {0xE8, 0x84, 0xBE},
		2, {0x3E, 0x81},
		2, {0xE7, 0xA3}
	},
	{
		2, {0xE4, 0x43},
		3, {0xE8, 0x85, 0x93},
		2, {0x53, 0x81},
		2, {0xE7, 0xA4}
	},
	{
		2, {0xE4, 0x44},
		3, {0xE8, 0x85, 0x91},
		2, {0x51, 0x81},
		2, {0xE7, 0xA5}
	},
	{
		2, {0xE4, 0x45},
		3, {0xE8, 0x83, 0xBC},
		2, {0xFC, 0x80},
		2, {0xE7, 0xA6}
	},
	{
		2, {0xE4, 0x46},
		3, {0xE8, 0x85, 0xB1},
		2, {0x71, 0x81},
		2, {0xE7, 0xA7}
	},
	{
		2, {0xE4, 0x47},
		3, {0xE8, 0x85, 0xAE},
		2, {0x6E, 0x81},
		2, {0xE7, 0xA8}
	},
	{
		2, {0xE4, 0x48},
		3, {0xE8, 0x85, 0xA5},
		2, {0x65, 0x81},
		2, {0xE7, 0xA9}
	},
	{
		2, {0xE4, 0x49},
		3, {0xE8, 0x85, 0xA6},
		2, {0x66, 0x81},
		2, {0xE7, 0xAA}
	},
	{
		2, {0xE4, 0x4A},
		3, {0xE8, 0x85, 0xB4},
		2, {0x74, 0x81},
		2, {0xE7, 0xAB}
	},
	{
		2, {0xE4, 0x4B},
		3, {0xE8, 0x86, 0x83},
		2, {0x83, 0x81},
		2, {0xE7, 0xAC}
	},
	{
		2, {0xE4, 0x4C},
		3, {0xE8, 0x86, 0x88},
		2, {0x88, 0x81},
		2, {0xE7, 0xAD}
	},
	{
		2, {0xE4, 0x4D},
		3, {0xE8, 0x86, 0x8A},
		2, {0x8A, 0x81},
		2, {0xE7, 0xAE}
	},
	{
		2, {0xE4, 0x4E},
		3, {0xE8, 0x86, 0x80},
		2, {0x80, 0x81},
		2, {0xE7, 0xAF}
	},
	{
		2, {0xE4, 0x4F},
		3, {0xE8, 0x86, 0x82},
		2, {0x82, 0x81},
		2, {0xE7, 0xB0}
	},
	{
		2, {0xE4, 0x50},
		3, {0xE8, 0x86, 0xA0},
		2, {0xA0, 0x81},
		2, {0xE7, 0xB1}
	},
	{
		2, {0xE4, 0x51},
		3, {0xE8, 0x86, 0x95},
		2, {0x95, 0x81},
		2, {0xE7, 0xB2}
	},
	{
		2, {0xE4, 0x52},
		3, {0xE8, 0x86, 0xA4},
		2, {0xA4, 0x81},
		2, {0xE7, 0xB3}
	},
	{
		2, {0xE4, 0x53},
		3, {0xE8, 0x86, 0xA3},
		2, {0xA3, 0x81},
		2, {0xE7, 0xB4}
	},
	{
		2, {0xE4, 0x54},
		3, {0xE8, 0x85, 0x9F},
		2, {0x5F, 0x81},
		2, {0xE7, 0xB5}
	},
	{
		2, {0xE4, 0x55},
		3, {0xE8, 0x86, 0x93},
		2, {0x93, 0x81},
		2, {0xE7, 0xB6}
	},
	{
		2, {0xE4, 0x56},
		3, {0xE8, 0x86, 0xA9},
		2, {0xA9, 0x81},
		2, {0xE7, 0xB7}
	},
	{
		2, {0xE4, 0x57},
		3, {0xE8, 0x86, 0xB0},
		2, {0xB0, 0x81},
		2, {0xE7, 0xB8}
	},
	{
		2, {0xE4, 0x58},
		3, {0xE8, 0x86, 0xB5},
		2, {0xB5, 0x81},
		2, {0xE7, 0xB9}
	},
	{
		2, {0xE4, 0x59},
		3, {0xE8, 0x86, 0xBE},
		2, {0xBE, 0x81},
		2, {0xE7, 0xBA}
	},
	{
		2, {0xE4, 0x5A},
		3, {0xE8, 0x86, 0xB8},
		2, {0xB8, 0x81},
		2, {0xE7, 0xBB}
	},
	{
		2, {0xE4, 0x5B},
		3, {0xE8, 0x86, 0xBD},
		2, {0xBD, 0x81},
		2, {0xE7, 0xBC}
	},
	{
		2, {0xE4, 0x5C},
		3, {0xE8, 0x87, 0x80},
		2, {0xC0, 0x81},
		2, {0xE7, 0xBD}
	},
	{
		2, {0xE4, 0x5D},
		3, {0xE8, 0x87, 0x82},
		2, {0xC2, 0x81},
		2, {0xE7, 0xBE}
	},
	{
		2, {0xE4, 0x5E},
		3, {0xE8, 0x86, 0xBA},
		2, {0xBA, 0x81},
		2, {0xE7, 0xBF}
	},
	{
		2, {0xE4, 0x5F},
		3, {0xE8, 0x87, 0x89},
		2, {0xC9, 0x81},
		2, {0xE7, 0xC0}
	},
	{
		2, {0xE4, 0x60},
		3, {0xE8, 0x87, 0x8D},
		2, {0xCD, 0x81},
		2, {0xE7, 0xC1}
	},
	{
		2, {0xE4, 0x61},
		3, {0xE8, 0x87, 0x91},
		2, {0xD1, 0x81},
		2, {0xE7, 0xC2}
	},
	{
		2, {0xE4, 0x62},
		3, {0xE8, 0x87, 0x99},
		2, {0xD9, 0x81},
		2, {0xE7, 0xC3}
	},
	{
		2, {0xE4, 0x63},
		3, {0xE8, 0x87, 0x98},
		2, {0xD8, 0x81},
		2, {0xE7, 0xC4}
	},
	{
		2, {0xE4, 0x64},
		3, {0xE8, 0x87, 0x88},
		2, {0xC8, 0x81},
		2, {0xE7, 0xC5}
	},
	{
		2, {0xE4, 0x65},
		3, {0xE8, 0x87, 0x9A},
		2, {0xDA, 0x81},
		2, {0xE7, 0xC6}
	},
	{
		2, {0xE4, 0x66},
		3, {0xE8, 0x87, 0x9F},
		2, {0xDF, 0x81},
		2, {0xE7, 0xC7}
	},
	{
		2, {0xE4, 0x67},
		3, {0xE8, 0x87, 0xA0},
		2, {0xE0, 0x81},
		2, {0xE7, 0xC8}
	},
	{
		2, {0xE4, 0x68},
		3, {0xE8, 0x87, 0xA7},
		2, {0xE7, 0x81},
		2, {0xE7, 0xC9}
	},
	{
		2, {0xE4, 0x69},
		3, {0xE8, 0x87, 0xBA},
		2, {0xFA, 0x81},
		2, {0xE7, 0xCA}
	},
	{
		2, {0xE4, 0x6A},
		3, {0xE8, 0x87, 0xBB},
		2, {0xFB, 0x81},
		2, {0xE7, 0xCB}
	},
	{
		2, {0xE4, 0x6B},
		3, {0xE8, 0x87, 0xBE},
		2, {0xFE, 0x81},
		2, {0xE7, 0xCC}
	},
	{
		2, {0xE4, 0x6C},
		3, {0xE8, 0x88, 0x81},
		2, {0x01, 0x82},
		2, {0xE7, 0xCD}
	},
	{
		2, {0xE4, 0x6D},
		3, {0xE8, 0x88, 0x82},
		2, {0x02, 0x82},
		2, {0xE7, 0xCE}
	},
	{
		2, {0xE4, 0x6E},
		3, {0xE8, 0x88, 0x85},
		2, {0x05, 0x82},
		2, {0xE7, 0xCF}
	},
	{
		2, {0xE4, 0x6F},
		3, {0xE8, 0x88, 0x87},
		2, {0x07, 0x82},
		2, {0xE7, 0xD0}
	},
	{
		2, {0xE4, 0x70},
		3, {0xE8, 0x88, 0x8A},
		2, {0x0A, 0x82},
		2, {0xE7, 0xD1}
	},
	{
		2, {0xE4, 0x71},
		3, {0xE8, 0x88, 0x8D},
		2, {0x0D, 0x82},
		2, {0xE7, 0xD2}
	},
	{
		2, {0xE4, 0x72},
		3, {0xE8, 0x88, 0x90},
		2, {0x10, 0x82},
		2, {0xE7, 0xD3}
	},
	{
		2, {0xE4, 0x73},
		3, {0xE8, 0x88, 0x96},
		2, {0x16, 0x82},
		2, {0xE7, 0xD4}
	},
	{
		2, {0xE4, 0x74},
		3, {0xE8, 0x88, 0xA9},
		2, {0x29, 0x82},
		2, {0xE7, 0xD5}
	},
	{
		2, {0xE4, 0x75},
		3, {0xE8, 0x88, 0xAB},
		2, {0x2B, 0x82},
		2, {0xE7, 0xD6}
	},
	{
		2, {0xE4, 0x76},
		3, {0xE8, 0x88, 0xB8},
		2, {0x38, 0x82},
		2, {0xE7, 0xD7}
	},
	{
		2, {0xE4, 0x77},
		3, {0xE8, 0x88, 0xB3},
		2, {0x33, 0x82},
		2, {0xE7, 0xD8}
	},
	{
		2, {0xE4, 0x78},
		3, {0xE8, 0x89, 0x80},
		2, {0x40, 0x82},
		2, {0xE7, 0xD9}
	},
	{
		2, {0xE4, 0x79},
		3, {0xE8, 0x89, 0x99},
		2, {0x59, 0x82},
		2, {0xE7, 0xDA}
	},
	{
		2, {0xE4, 0x7A},
		3, {0xE8, 0x89, 0x98},
		2, {0x58, 0x82},
		2, {0xE7, 0xDB}
	},
	{
		2, {0xE4, 0x7B},
		3, {0xE8, 0x89, 0x9D},
		2, {0x5D, 0x82},
		2, {0xE7, 0xDC}
	},
	{
		2, {0xE4, 0x7C},
		3, {0xE8, 0x89, 0x9A},
		2, {0x5A, 0x82},
		2, {0xE7, 0xDD}
	},
	{
		2, {0xE4, 0x7D},
		3, {0xE8, 0x89, 0x9F},
		2, {0x5F, 0x82},
		2, {0xE7, 0xDE}
	},
	{
		2, {0xE4, 0x7E},
		3, {0xE8, 0x89, 0xA4},
		2, {0x64, 0x82},
		2, {0xE7, 0xDF}
	},
	{
		2, {0xE4, 0x80},
		3, {0xE8, 0x89, 0xA2},
		2, {0x62, 0x82},
		2, {0xE7, 0xE0}
	},
	{
		2, {0xE4, 0x81},
		3, {0xE8, 0x89, 0xA8},
		2, {0x68, 0x82},
		2, {0xE7, 0xE1}
	},
	{
		2, {0xE4, 0x82},
		3, {0xE8, 0x89, 0xAA},
		2, {0x6A, 0x82},
		2, {0xE7, 0xE2}
	},
	{
		2, {0xE4, 0x83},
		3, {0xE8, 0x89, 0xAB},
		2, {0x6B, 0x82},
		2, {0xE7, 0xE3}
	},
	{
		2, {0xE4, 0x84},
		3, {0xE8, 0x88, 0xAE},
		2, {0x2E, 0x82},
		2, {0xE7, 0xE4}
	},
	{
		2, {0xE4, 0x85},
		3, {0xE8, 0x89, 0xB1},
		2, {0x71, 0x82},
		2, {0xE7, 0xE5}
	},
	{
		2, {0xE4, 0x86},
		3, {0xE8, 0x89, 0xB7},
		2, {0x77, 0x82},
		2, {0xE7, 0xE6}
	},
	{
		2, {0xE4, 0x87},
		3, {0xE8, 0x89, 0xB8},
		2, {0x78, 0x82},
		2, {0xE7, 0xE7}
	},
	{
		2, {0xE4, 0x88},
		3, {0xE8, 0x89, 0xBE},
		2, {0x7E, 0x82},
		2, {0xE7, 0xE8}
	},
	{
		2, {0xE4, 0x89},
		3, {0xE8, 0x8A, 0x8D},
		2, {0x8D, 0x82},
		2, {0xE7, 0xE9}
	},
	{
		2, {0xE4, 0x8A},
		3, {0xE8, 0x8A, 0x92},
		2, {0x92, 0x82},
		2, {0xE7, 0xEA}
	},
	{
		2, {0xE4, 0x8B},
		3, {0xE8, 0x8A, 0xAB},
		2, {0xAB, 0x82},
		2, {0xE7, 0xEB}
	},
	{
		2, {0xE4, 0x8C},
		3, {0xE8, 0x8A, 0x9F},
		2, {0x9F, 0x82},
		2, {0xE7, 0xEC}
	},
	{
		2, {0xE4, 0x8D},
		3, {0xE8, 0x8A, 0xBB},
		2, {0xBB, 0x82},
		2, {0xE7, 0xED}
	},
	{
		2, {0xE4, 0x8E},
		3, {0xE8, 0x8A, 0xAC},
		2, {0xAC, 0x82},
		2, {0xE7, 0xEE}
	},
	{
		2, {0xE4, 0x8F},
		3, {0xE8, 0x8B, 0xA1},
		2, {0xE1, 0x82},
		2, {0xE7, 0xEF}
	},
	{
		2, {0xE4, 0x90},
		3, {0xE8, 0x8B, 0xA3},
		2, {0xE3, 0x82},
		2, {0xE7, 0xF0}
	},
	{
		2, {0xE4, 0x91},
		3, {0xE8, 0x8B, 0x9F},
		2, {0xDF, 0x82},
		2, {0xE7, 0xF1}
	},
	{
		2, {0xE4, 0x92},
		3, {0xE8, 0x8B, 0x92},
		2, {0xD2, 0x82},
		2, {0xE7, 0xF2}
	},
	{
		2, {0xE4, 0x93},
		3, {0xE8, 0x8B, 0xB4},
		2, {0xF4, 0x82},
		2, {0xE7, 0xF3}
	},
	{
		2, {0xE4, 0x94},
		3, {0xE8, 0x8B, 0xB3},
		2, {0xF3, 0x82},
		2, {0xE7, 0xF4}
	},
	{
		2, {0xE4, 0x95},
		3, {0xE8, 0x8B, 0xBA},
		2, {0xFA, 0x82},
		2, {0xE7, 0xF5}
	},
	{
		2, {0xE4, 0x96},
		3, {0xE8, 0x8E, 0x93},
		2, {0x93, 0x83},
		2, {0xE7, 0xF6}
	},
	{
		2, {0xE4, 0x97},
		3, {0xE8, 0x8C, 0x83},
		2, {0x03, 0x83},
		2, {0xE7, 0xF7}
	},
	{
		2, {0xE4, 0x98},
		3, {0xE8, 0x8B, 0xBB},
		2, {0xFB, 0x82},
		2, {0xE7, 0xF8}
	},
	{
		2, {0xE4, 0x99},
		3, {0xE8, 0x8B, 0xB9},
		2, {0xF9, 0x82},
		2, {0xE7, 0xF9}
	},
	{
		2, {0xE4, 0x9A},
		3, {0xE8, 0x8B, 0x9E},
		2, {0xDE, 0x82},
		2, {0xE7, 0xFA}
	},
	{
		2, {0xE4, 0x9B},
		3, {0xE8, 0x8C, 0x86},
		2, {0x06, 0x83},
		2, {0xE7, 0xFB}
	},
	{
		2, {0xE4, 0x9C},
		3, {0xE8, 0x8B, 0x9C},
		2, {0xDC, 0x82},
		2, {0xE7, 0xFC}
	},
	{
		2, {0xE4, 0x9D},
		3, {0xE8, 0x8C, 0x89},
		2, {0x09, 0x83},
		2, {0xE7, 0xFD}
	},
	{
		2, {0xE4, 0x9E},
		3, {0xE8, 0x8B, 0x99},
		2, {0xD9, 0x82},
		2, {0xE7, 0xFE}
	},
	{
		2, {0xE4, 0x9F},
		3, {0xE8, 0x8C, 0xB5},
		2, {0x35, 0x83},
		2, {0xE8, 0xA1}
	},
	{
		2, {0xE4, 0xA0},
		3, {0xE8, 0x8C, 0xB4},
		2, {0x34, 0x83},
		2, {0xE8, 0xA2}
	},
	{
		2, {0xE4, 0xA1},
		3, {0xE8, 0x8C, 0x96},
		2, {0x16, 0x83},
		2, {0xE8, 0xA3}
	},
	{
		2, {0xE4, 0xA2},
		3, {0xE8, 0x8C, 0xB2},
		2, {0x32, 0x83},
		2, {0xE8, 0xA4}
	},
	{
		2, {0xE4, 0xA3},
		3, {0xE8, 0x8C, 0xB1},
		2, {0x31, 0x83},
		2, {0xE8, 0xA5}
	},
	{
		2, {0xE4, 0xA4},
		3, {0xE8, 0x8D, 0x80},
		2, {0x40, 0x83},
		2, {0xE8, 0xA6}
	},
	{
		2, {0xE4, 0xA5},
		3, {0xE8, 0x8C, 0xB9},
		2, {0x39, 0x83},
		2, {0xE8, 0xA7}
	},
	{
		2, {0xE4, 0xA6},
		3, {0xE8, 0x8D, 0x90},
		2, {0x50, 0x83},
		2, {0xE8, 0xA8}
	},
	{
		2, {0xE4, 0xA7},
		3, {0xE8, 0x8D, 0x85},
		2, {0x45, 0x83},
		2, {0xE8, 0xA9}
	},
	{
		2, {0xE4, 0xA8},
		3, {0xE8, 0x8C, 0xAF},
		2, {0x2F, 0x83},
		2, {0xE8, 0xAA}
	},
	{
		2, {0xE4, 0xA9},
		3, {0xE8, 0x8C, 0xAB},
		2, {0x2B, 0x83},
		2, {0xE8, 0xAB}
	},
	{
		2, {0xE4, 0xAA},
		3, {0xE8, 0x8C, 0x97},
		2, {0x17, 0x83},
		2, {0xE8, 0xAC}
	},
	{
		2, {0xE4, 0xAB},
		3, {0xE8, 0x8C, 0x98},
		2, {0x18, 0x83},
		2, {0xE8, 0xAD}
	},
	{
		2, {0xE4, 0xAC},
		3, {0xE8, 0x8E, 0x85},
		2, {0x85, 0x83},
		2, {0xE8, 0xAE}
	},
	{
		2, {0xE4, 0xAD},
		3, {0xE8, 0x8E, 0x9A},
		2, {0x9A, 0x83},
		2, {0xE8, 0xAF}
	},
	{
		2, {0xE4, 0xAE},
		3, {0xE8, 0x8E, 0xAA},
		2, {0xAA, 0x83},
		2, {0xE8, 0xB0}
	},
	{
		2, {0xE4, 0xAF},
		3, {0xE8, 0x8E, 0x9F},
		2, {0x9F, 0x83},
		2, {0xE8, 0xB1}
	},
	{
		2, {0xE4, 0xB0},
		3, {0xE8, 0x8E, 0xA2},
		2, {0xA2, 0x83},
		2, {0xE8, 0xB2}
	},
	{
		2, {0xE4, 0xB1},
		3, {0xE8, 0x8E, 0x96},
		2, {0x96, 0x83},
		2, {0xE8, 0xB3}
	},
	{
		2, {0xE4, 0xB2},
		3, {0xE8, 0x8C, 0xA3},
		2, {0x23, 0x83},
		2, {0xE8, 0xB4}
	},
	{
		2, {0xE4, 0xB3},
		3, {0xE8, 0x8E, 0x8E},
		2, {0x8E, 0x83},
		2, {0xE8, 0xB5}
	},
	{
		2, {0xE4, 0xB4},
		3, {0xE8, 0x8E, 0x87},
		2, {0x87, 0x83},
		2, {0xE8, 0xB6}
	},
	{
		2, {0xE4, 0xB5},
		3, {0xE8, 0x8E, 0x8A},
		2, {0x8A, 0x83},
		2, {0xE8, 0xB7}
	},
	{
		2, {0xE4, 0xB6},
		3, {0xE8, 0x8D, 0xBC},
		2, {0x7C, 0x83},
		2, {0xE8, 0xB8}
	},
	{
		2, {0xE4, 0xB7},
		3, {0xE8, 0x8E, 0xB5},
		2, {0xB5, 0x83},
		2, {0xE8, 0xB9}
	},
	{
		2, {0xE4, 0xB8},
		3, {0xE8, 0x8D, 0xB3},
		2, {0x73, 0x83},
		2, {0xE8, 0xBA}
	},
	{
		2, {0xE4, 0xB9},
		3, {0xE8, 0x8D, 0xB5},
		2, {0x75, 0x83},
		2, {0xE8, 0xBB}
	},
	{
		2, {0xE4, 0xBA},
		3, {0xE8, 0x8E, 0xA0},
		2, {0xA0, 0x83},
		2, {0xE8, 0xBC}
	},
	{
		2, {0xE4, 0xBB},
		3, {0xE8, 0x8E, 0x89},
		2, {0x89, 0x83},
		2, {0xE8, 0xBD}
	},
	{
		2, {0xE4, 0xBC},
		3, {0xE8, 0x8E, 0xA8},
		2, {0xA8, 0x83},
		2, {0xE8, 0xBE}
	},
	{
		2, {0xE4, 0xBD},
		3, {0xE8, 0x8F, 0xB4},
		2, {0xF4, 0x83},
		2, {0xE8, 0xBF}
	},
	{
		2, {0xE4, 0xBE},
		3, {0xE8, 0x90, 0x93},
		2, {0x13, 0x84},
		2, {0xE8, 0xC0}
	},
	{
		2, {0xE4, 0xBF},
		3, {0xE8, 0x8F, 0xAB},
		2, {0xEB, 0x83},
		2, {0xE8, 0xC1}
	},
	{
		2, {0xE4, 0xC0},
		3, {0xE8, 0x8F, 0x8E},
		2, {0xCE, 0x83},
		2, {0xE8, 0xC2}
	},
	{
		2, {0xE4, 0xC1},
		3, {0xE8, 0x8F, 0xBD},
		2, {0xFD, 0x83},
		2, {0xE8, 0xC3}
	},
	{
		2, {0xE4, 0xC2},
		3, {0xE8, 0x90, 0x83},
		2, {0x03, 0x84},
		2, {0xE8, 0xC4}
	},
	{
		2, {0xE4, 0xC3},
		3, {0xE8, 0x8F, 0x98},
		2, {0xD8, 0x83},
		2, {0xE8, 0xC5}
	},
	{
		2, {0xE4, 0xC4},
		3, {0xE8, 0x90, 0x8B},
		2, {0x0B, 0x84},
		2, {0xE8, 0xC6}
	},
	{
		2, {0xE4, 0xC5},
		3, {0xE8, 0x8F, 0x81},
		2, {0xC1, 0x83},
		2, {0xE8, 0xC7}
	},
	{
		2, {0xE4, 0xC6},
		3, {0xE8, 0x8F, 0xB7},
		2, {0xF7, 0x83},
		2, {0xE8, 0xC8}
	},
	{
		2, {0xE4, 0xC7},
		3, {0xE8, 0x90, 0x87},
		2, {0x07, 0x84},
		2, {0xE8, 0xC9}
	},
	{
		2, {0xE4, 0xC8},
		3, {0xE8, 0x8F, 0xA0},
		2, {0xE0, 0x83},
		2, {0xE8, 0xCA}
	},
	{
		2, {0xE4, 0xC9},
		3, {0xE8, 0x8F, 0xB2},
		2, {0xF2, 0x83},
		2, {0xE8, 0xCB}
	},
	{
		2, {0xE4, 0xCA},
		3, {0xE8, 0x90, 0x8D},
		2, {0x0D, 0x84},
		2, {0xE8, 0xCC}
	},
	{
		2, {0xE4, 0xCB},
		3, {0xE8, 0x90, 0xA2},
		2, {0x22, 0x84},
		2, {0xE8, 0xCD}
	},
	{
		2, {0xE4, 0xCC},
		3, {0xE8, 0x90, 0xA0},
		2, {0x20, 0x84},
		2, {0xE8, 0xCE}
	},
	{
		2, {0xE4, 0xCD},
		3, {0xE8, 0x8E, 0xBD},
		2, {0xBD, 0x83},
		2, {0xE8, 0xCF}
	},
	{
		2, {0xE4, 0xCE},
		3, {0xE8, 0x90, 0xB8},
		2, {0x38, 0x84},
		2, {0xE8, 0xD0}
	},
	{
		2, {0xE4, 0xCF},
		3, {0xE8, 0x94, 0x86},
		2, {0x06, 0x85},
		2, {0xE8, 0xD1}
	},
	{
		2, {0xE4, 0xD0},
		3, {0xE8, 0x8F, 0xBB},
		2, {0xFB, 0x83},
		2, {0xE8, 0xD2}
	},
	{
		2, {0xE4, 0xD1},
		3, {0xE8, 0x91, 0xAD},
		2, {0x6D, 0x84},
		2, {0xE8, 0xD3}
	},
	{
		2, {0xE4, 0xD2},
		3, {0xE8, 0x90, 0xAA},
		2, {0x2A, 0x84},
		2, {0xE8, 0xD4}
	},
	{
		2, {0xE4, 0xD3},
		3, {0xE8, 0x90, 0xBC},
		2, {0x3C, 0x84},
		2, {0xE8, 0xD5}
	},
	{
		2, {0xE4, 0xD4},
		3, {0xE8, 0x95, 0x9A},
		2, {0x5A, 0x85},
		2, {0xE8, 0xD6}
	},
	{
		2, {0xE4, 0xD5},
		3, {0xE8, 0x92, 0x84},
		2, {0x84, 0x84},
		2, {0xE8, 0xD7}
	},
	{
		2, {0xE4, 0xD6},
		3, {0xE8, 0x91, 0xB7},
		2, {0x77, 0x84},
		2, {0xE8, 0xD8}
	},
	{
		2, {0xE4, 0xD7},
		3, {0xE8, 0x91, 0xAB},
		2, {0x6B, 0x84},
		2, {0xE8, 0xD9}
	},
	{
		2, {0xE4, 0xD8},
		3, {0xE8, 0x92, 0xAD},
		2, {0xAD, 0x84},
		2, {0xE8, 0xDA}
	},
	{
		2, {0xE4, 0xD9},
		3, {0xE8, 0x91, 0xAE},
		2, {0x6E, 0x84},
		2, {0xE8, 0xDB}
	},
	{
		2, {0xE4, 0xDA},
		3, {0xE8, 0x92, 0x82},
		2, {0x82, 0x84},
		2, {0xE8, 0xDC}
	},
	{
		2, {0xE4, 0xDB},
		3, {0xE8, 0x91, 0xA9},
		2, {0x69, 0x84},
		2, {0xE8, 0xDD}
	},
	{
		2, {0xE4, 0xDC},
		3, {0xE8, 0x91, 0x86},
		2, {0x46, 0x84},
		2, {0xE8, 0xDE}
	},
	{
		2, {0xE4, 0xDD},
		3, {0xE8, 0x90, 0xAC},
		2, {0x2C, 0x84},
		2, {0xE8, 0xDF}
	},
	{
		2, {0xE4, 0xDE},
		3, {0xE8, 0x91, 0xAF},
		2, {0x6F, 0x84},
		2, {0xE8, 0xE0}
	},
	{
		2, {0xE4, 0xDF},
		3, {0xE8, 0x91, 0xB9},
		2, {0x79, 0x84},
		2, {0xE8, 0xE1}
	},
	{
		2, {0xE4, 0xE0},
		3, {0xE8, 0x90, 0xB5},
		2, {0x35, 0x84},
		2, {0xE8, 0xE2}
	},
	{
		2, {0xE4, 0xE1},
		3, {0xE8, 0x93, 0x8A},
		2, {0xCA, 0x84},
		2, {0xE8, 0xE3}
	},
	{
		2, {0xE4, 0xE2},
		3, {0xE8, 0x91, 0xA2},
		2, {0x62, 0x84},
		2, {0xE8, 0xE4}
	},
	{
		2, {0xE4, 0xE3},
		3, {0xE8, 0x92, 0xB9},
		2, {0xB9, 0x84},
		2, {0xE8, 0xE5}
	},
	{
		2, {0xE4, 0xE4},
		3, {0xE8, 0x92, 0xBF},
		2, {0xBF, 0x84},
		2, {0xE8, 0xE6}
	},
	{
		2, {0xE4, 0xE5},
		3, {0xE8, 0x92, 0x9F},
		2, {0x9F, 0x84},
		2, {0xE8, 0xE7}
	},
	{
		2, {0xE4, 0xE6},
		3, {0xE8, 0x93, 0x99},
		2, {0xD9, 0x84},
		2, {0xE8, 0xE8}
	},
	{
		2, {0xE4, 0xE7},
		3, {0xE8, 0x93, 0x8D},
		2, {0xCD, 0x84},
		2, {0xE8, 0xE9}
	},
	{
		2, {0xE4, 0xE8},
		3, {0xE8, 0x92, 0xBB},
		2, {0xBB, 0x84},
		2, {0xE8, 0xEA}
	},
	{
		2, {0xE4, 0xE9},
		3, {0xE8, 0x93, 0x9A},
		2, {0xDA, 0x84},
		2, {0xE8, 0xEB}
	},
	{
		2, {0xE4, 0xEA},
		3, {0xE8, 0x93, 0x90},
		2, {0xD0, 0x84},
		2, {0xE8, 0xEC}
	},
	{
		2, {0xE4, 0xEB},
		3, {0xE8, 0x93, 0x81},
		2, {0xC1, 0x84},
		2, {0xE8, 0xED}
	},
	{
		2, {0xE4, 0xEC},
		3, {0xE8, 0x93, 0x86},
		2, {0xC6, 0x84},
		2, {0xE8, 0xEE}
	},
	{
		2, {0xE4, 0xED},
		3, {0xE8, 0x93, 0x96},
		2, {0xD6, 0x84},
		2, {0xE8, 0xEF}
	},
	{
		2, {0xE4, 0xEE},
		3, {0xE8, 0x92, 0xA1},
		2, {0xA1, 0x84},
		2, {0xE8, 0xF0}
	},
	{
		2, {0xE4, 0xEF},
		3, {0xE8, 0x94, 0xA1},
		2, {0x21, 0x85},
		2, {0xE8, 0xF1}
	},
	{
		2, {0xE4, 0xF0},
		3, {0xE8, 0x93, 0xBF},
		2, {0xFF, 0x84},
		2, {0xE8, 0xF2}
	},
	{
		2, {0xE4, 0xF1},
		3, {0xE8, 0x93, 0xB4},
		2, {0xF4, 0x84},
		2, {0xE8, 0xF3}
	},
	{
		2, {0xE4, 0xF2},
		3, {0xE8, 0x94, 0x97},
		2, {0x17, 0x85},
		2, {0xE8, 0xF4}
	},
	{
		2, {0xE4, 0xF3},
		3, {0xE8, 0x94, 0x98},
		2, {0x18, 0x85},
		2, {0xE8, 0xF5}
	},
	{
		2, {0xE4, 0xF4},
		3, {0xE8, 0x94, 0xAC},
		2, {0x2C, 0x85},
		2, {0xE8, 0xF6}
	},
	{
		2, {0xE4, 0xF5},
		3, {0xE8, 0x94, 0x9F},
		2, {0x1F, 0x85},
		2, {0xE8, 0xF7}
	},
	{
		2, {0xE4, 0xF6},
		3, {0xE8, 0x94, 0x95},
		2, {0x15, 0x85},
		2, {0xE8, 0xF8}
	},
	{
		2, {0xE4, 0xF7},
		3, {0xE8, 0x94, 0x94},
		2, {0x14, 0x85},
		2, {0xE8, 0xF9}
	},
	{
		2, {0xE4, 0xF8},
		3, {0xE8, 0x93, 0xBC},
		2, {0xFC, 0x84},
		2, {0xE8, 0xFA}
	},
	{
		2, {0xE4, 0xF9},
		3, {0xE8, 0x95, 0x80},
		2, {0x40, 0x85},
		2, {0xE8, 0xFB}
	},
	{
		2, {0xE4, 0xFA},
		3, {0xE8, 0x95, 0xA3},
		2, {0x63, 0x85},
		2, {0xE8, 0xFC}
	},
	{
		2, {0xE4, 0xFB},
		3, {0xE8, 0x95, 0x98},
		2, {0x58, 0x85},
		2, {0xE8, 0xFD}
	},
	{
		2, {0xE4, 0xFC},
		3, {0xE8, 0x95, 0x88},
		2, {0x48, 0x85},
		2, {0xE8, 0xFE}
	},
	{
		2, {0xE5, 0x40},
		3, {0xE8, 0x95, 0x81},
		2, {0x41, 0x85},
		2, {0xE9, 0xA1}
	},
	{
		2, {0xE5, 0x41},
		3, {0xE8, 0x98, 0x82},
		2, {0x02, 0x86},
		2, {0xE9, 0xA2}
	},
	{
		2, {0xE5, 0x42},
		3, {0xE8, 0x95, 0x8B},
		2, {0x4B, 0x85},
		2, {0xE9, 0xA3}
	},
	{
		2, {0xE5, 0x43},
		3, {0xE8, 0x95, 0x95},
		2, {0x55, 0x85},
		2, {0xE9, 0xA4}
	},
	{
		2, {0xE5, 0x44},
		3, {0xE8, 0x96, 0x80},
		2, {0x80, 0x85},
		2, {0xE9, 0xA5}
	},
	{
		2, {0xE5, 0x45},
		3, {0xE8, 0x96, 0xA4},
		2, {0xA4, 0x85},
		2, {0xE9, 0xA6}
	},
	{
		2, {0xE5, 0x46},
		3, {0xE8, 0x96, 0x88},
		2, {0x88, 0x85},
		2, {0xE9, 0xA7}
	},
	{
		2, {0xE5, 0x47},
		3, {0xE8, 0x96, 0x91},
		2, {0x91, 0x85},
		2, {0xE9, 0xA8}
	},
	{
		2, {0xE5, 0x48},
		3, {0xE8, 0x96, 0x8A},
		2, {0x8A, 0x85},
		2, {0xE9, 0xA9}
	},
	{
		2, {0xE5, 0x49},
		3, {0xE8, 0x96, 0xA8},
		2, {0xA8, 0x85},
		2, {0xE9, 0xAA}
	},
	{
		2, {0xE5, 0x4A},
		3, {0xE8, 0x95, 0xAD},
		2, {0x6D, 0x85},
		2, {0xE9, 0xAB}
	},
	{
		2, {0xE5, 0x4B},
		3, {0xE8, 0x96, 0x94},
		2, {0x94, 0x85},
		2, {0xE9, 0xAC}
	},
	{
		2, {0xE5, 0x4C},
		3, {0xE8, 0x96, 0x9B},
		2, {0x9B, 0x85},
		2, {0xE9, 0xAD}
	},
	{
		2, {0xE5, 0x4D},
		3, {0xE8, 0x97, 0xAA},
		2, {0xEA, 0x85},
		2, {0xE9, 0xAE}
	},
	{
		2, {0xE5, 0x4E},
		3, {0xE8, 0x96, 0x87},
		2, {0x87, 0x85},
		2, {0xE9, 0xAF}
	},
	{
		2, {0xE5, 0x4F},
		3, {0xE8, 0x96, 0x9C},
		2, {0x9C, 0x85},
		2, {0xE9, 0xB0}
	},
	{
		2, {0xE5, 0x50},
		3, {0xE8, 0x95, 0xB7},
		2, {0x77, 0x85},
		2, {0xE9, 0xB1}
	},
	{
		2, {0xE5, 0x51},
		3, {0xE8, 0x95, 0xBE},
		2, {0x7E, 0x85},
		2, {0xE9, 0xB2}
	},
	{
		2, {0xE5, 0x52},
		3, {0xE8, 0x96, 0x90},
		2, {0x90, 0x85},
		2, {0xE9, 0xB3}
	},
	{
		2, {0xE5, 0x53},
		3, {0xE8, 0x97, 0x89},
		2, {0xC9, 0x85},
		2, {0xE9, 0xB4}
	},
	{
		2, {0xE5, 0x54},
		3, {0xE8, 0x96, 0xBA},
		2, {0xBA, 0x85},
		2, {0xE9, 0xB5}
	},
	{
		2, {0xE5, 0x55},
		3, {0xE8, 0x97, 0x8F},
		2, {0xCF, 0x85},
		2, {0xE9, 0xB6}
	},
	{
		2, {0xE5, 0x56},
		3, {0xE8, 0x96, 0xB9},
		2, {0xB9, 0x85},
		2, {0xE9, 0xB7}
	},
	{
		2, {0xE5, 0x57},
		3, {0xE8, 0x97, 0x90},
		2, {0xD0, 0x85},
		2, {0xE9, 0xB8}
	},
	{
		2, {0xE5, 0x58},
		3, {0xE8, 0x97, 0x95},
		2, {0xD5, 0x85},
		2, {0xE9, 0xB9}
	},
	{
		2, {0xE5, 0x59},
		3, {0xE8, 0x97, 0x9D},
		2, {0xDD, 0x85},
		2, {0xE9, 0xBA}
	},
	{
		2, {0xE5, 0x5A},
		3, {0xE8, 0x97, 0xA5},
		2, {0xE5, 0x85},
		2, {0xE9, 0xBB}
	},
	{
		2, {0xE5, 0x5B},
		3, {0xE8, 0x97, 0x9C},
		2, {0xDC, 0x85},
		2, {0xE9, 0xBC}
	},
	{
		2, {0xE5, 0x5C},
		3, {0xE8, 0x97, 0xB9},
		2, {0xF9, 0x85},
		2, {0xE9, 0xBD}
	},
	{
		2, {0xE5, 0x5D},
		3, {0xE8, 0x98, 0x8A},
		2, {0x0A, 0x86},
		2, {0xE9, 0xBE}
	},
	{
		2, {0xE5, 0x5E},
		3, {0xE8, 0x98, 0x93},
		2, {0x13, 0x86},
		2, {0xE9, 0xBF}
	},
	{
		2, {0xE5, 0x5F},
		3, {0xE8, 0x98, 0x8B},
		2, {0x0B, 0x86},
		2, {0xE9, 0xC0}
	},
	{
		2, {0xE5, 0x60},
		3, {0xE8, 0x97, 0xBE},
		2, {0xFE, 0x85},
		2, {0xE9, 0xC1}
	},
	{
		2, {0xE5, 0x61},
		3, {0xE8, 0x97, 0xBA},
		2, {0xFA, 0x85},
		2, {0xE9, 0xC2}
	},
	{
		2, {0xE5, 0x62},
		3, {0xE8, 0x98, 0x86},
		2, {0x06, 0x86},
		2, {0xE9, 0xC3}
	},
	{
		2, {0xE5, 0x63},
		3, {0xE8, 0x98, 0xA2},
		2, {0x22, 0x86},
		2, {0xE9, 0xC4}
	},
	{
		2, {0xE5, 0x64},
		3, {0xE8, 0x98, 0x9A},
		2, {0x1A, 0x86},
		2, {0xE9, 0xC5}
	},
	{
		2, {0xE5, 0x65},
		3, {0xE8, 0x98, 0xB0},
		2, {0x30, 0x86},
		2, {0xE9, 0xC6}
	},
	{
		2, {0xE5, 0x66},
		3, {0xE8, 0x98, 0xBF},
		2, {0x3F, 0x86},
		2, {0xE9, 0xC7}
	},
	{
		2, {0xE5, 0x67},
		3, {0xE8, 0x99, 0x8D},
		2, {0x4D, 0x86},
		2, {0xE9, 0xC8}
	},
	{
		2, {0xE5, 0x68},
		3, {0xE4, 0xB9, 0x95},
		2, {0x55, 0x4E},
		2, {0xE9, 0xC9}
	},
	{
		2, {0xE5, 0x69},
		3, {0xE8, 0x99, 0x94},
		2, {0x54, 0x86},
		2, {0xE9, 0xCA}
	},
	{
		2, {0xE5, 0x6A},
		3, {0xE8, 0x99, 0x9F},
		2, {0x5F, 0x86},
		2, {0xE9, 0xCB}
	},
	{
		2, {0xE5, 0x6B},
		3, {0xE8, 0x99, 0xA7},
		2, {0x67, 0x86},
		2, {0xE9, 0xCC}
	},
	{
		2, {0xE5, 0x6C},
		3, {0xE8, 0x99, 0xB1},
		2, {0x71, 0x86},
		2, {0xE9, 0xCD}
	},
	{
		2, {0xE5, 0x6D},
		3, {0xE8, 0x9A, 0x93},
		2, {0x93, 0x86},
		2, {0xE9, 0xCE}
	},
	{
		2, {0xE5, 0x6E},
		3, {0xE8, 0x9A, 0xA3},
		2, {0xA3, 0x86},
		2, {0xE9, 0xCF}
	},
	{
		2, {0xE5, 0x6F},
		3, {0xE8, 0x9A, 0xA9},
		2, {0xA9, 0x86},
		2, {0xE9, 0xD0}
	},
	{
		2, {0xE5, 0x70},
		3, {0xE8, 0x9A, 0xAA},
		2, {0xAA, 0x86},
		2, {0xE9, 0xD1}
	},
	{
		2, {0xE5, 0x71},
		3, {0xE8, 0x9A, 0x8B},
		2, {0x8B, 0x86},
		2, {0xE9, 0xD2}
	},
	{
		2, {0xE5, 0x72},
		3, {0xE8, 0x9A, 0x8C},
		2, {0x8C, 0x86},
		2, {0xE9, 0xD3}
	},
	{
		2, {0xE5, 0x73},
		3, {0xE8, 0x9A, 0xB6},
		2, {0xB6, 0x86},
		2, {0xE9, 0xD4}
	},
	{
		2, {0xE5, 0x74},
		3, {0xE8, 0x9A, 0xAF},
		2, {0xAF, 0x86},
		2, {0xE9, 0xD5}
	},
	{
		2, {0xE5, 0x75},
		3, {0xE8, 0x9B, 0x84},
		2, {0xC4, 0x86},
		2, {0xE9, 0xD6}
	},
	{
		2, {0xE5, 0x76},
		3, {0xE8, 0x9B, 0x86},
		2, {0xC6, 0x86},
		2, {0xE9, 0xD7}
	},
	{
		2, {0xE5, 0x77},
		3, {0xE8, 0x9A, 0xB0},
		2, {0xB0, 0x86},
		2, {0xE9, 0xD8}
	},
	{
		2, {0xE5, 0x78},
		3, {0xE8, 0x9B, 0x89},
		2, {0xC9, 0x86},
		2, {0xE9, 0xD9}
	},
	{
		2, {0xE5, 0x79},
		3, {0xE8, 0xA0, 0xA3},
		2, {0x23, 0x88},
		2, {0xE9, 0xDA}
	},
	{
		2, {0xE5, 0x7A},
		3, {0xE8, 0x9A, 0xAB},
		2, {0xAB, 0x86},
		2, {0xE9, 0xDB}
	},
	{
		2, {0xE5, 0x7B},
		3, {0xE8, 0x9B, 0x94},
		2, {0xD4, 0x86},
		2, {0xE9, 0xDC}
	},
	{
		2, {0xE5, 0x7C},
		3, {0xE8, 0x9B, 0x9E},
		2, {0xDE, 0x86},
		2, {0xE9, 0xDD}
	},
	{
		2, {0xE5, 0x7D},
		3, {0xE8, 0x9B, 0xA9},
		2, {0xE9, 0x86},
		2, {0xE9, 0xDE}
	},
	{
		2, {0xE5, 0x7E},
		3, {0xE8, 0x9B, 0xAC},
		2, {0xEC, 0x86},
		2, {0xE9, 0xDF}
	},
	{
		2, {0xE5, 0x80},
		3, {0xE8, 0x9B, 0x9F},
		2, {0xDF, 0x86},
		2, {0xE9, 0xE0}
	},
	{
		2, {0xE5, 0x81},
		3, {0xE8, 0x9B, 0x9B},
		2, {0xDB, 0x86},
		2, {0xE9, 0xE1}
	},
	{
		2, {0xE5, 0x82},
		3, {0xE8, 0x9B, 0xAF},
		2, {0xEF, 0x86},
		2, {0xE9, 0xE2}
	},
	{
		2, {0xE5, 0x83},
		3, {0xE8, 0x9C, 0x92},
		2, {0x12, 0x87},
		2, {0xE9, 0xE3}
	},
	{
		2, {0xE5, 0x84},
		3, {0xE8, 0x9C, 0x86},
		2, {0x06, 0x87},
		2, {0xE9, 0xE4}
	},
	{
		2, {0xE5, 0x85},
		3, {0xE8, 0x9C, 0x88},
		2, {0x08, 0x87},
		2, {0xE9, 0xE5}
	},
	{
		2, {0xE5, 0x86},
		3, {0xE8, 0x9C, 0x80},
		2, {0x00, 0x87},
		2, {0xE9, 0xE6}
	},
	{
		2, {0xE5, 0x87},
		3, {0xE8, 0x9C, 0x83},
		2, {0x03, 0x87},
		2, {0xE9, 0xE7}
	},
	{
		2, {0xE5, 0x88},
		3, {0xE8, 0x9B, 0xBB},
		2, {0xFB, 0x86},
		2, {0xE9, 0xE8}
	},
	{
		2, {0xE5, 0x89},
		3, {0xE8, 0x9C, 0x91},
		2, {0x11, 0x87},
		2, {0xE9, 0xE9}
	},
	{
		2, {0xE5, 0x8A},
		3, {0xE8, 0x9C, 0x89},
		2, {0x09, 0x87},
		2, {0xE9, 0xEA}
	},
	{
		2, {0xE5, 0x8B},
		3, {0xE8, 0x9C, 0x8D},
		2, {0x0D, 0x87},
		2, {0xE9, 0xEB}
	},
	{
		2, {0xE5, 0x8C},
		3, {0xE8, 0x9B, 0xB9},
		2, {0xF9, 0x86},
		2, {0xE9, 0xEC}
	},
	{
		2, {0xE5, 0x8D},
		3, {0xE8, 0x9C, 0x8A},
		2, {0x0A, 0x87},
		2, {0xE9, 0xED}
	},
	{
		2, {0xE5, 0x8E},
		3, {0xE8, 0x9C, 0xB4},
		2, {0x34, 0x87},
		2, {0xE9, 0xEE}
	},
	{
		2, {0xE5, 0x8F},
		3, {0xE8, 0x9C, 0xBF},
		2, {0x3F, 0x87},
		2, {0xE9, 0xEF}
	},
	{
		2, {0xE5, 0x90},
		3, {0xE8, 0x9C, 0xB7},
		2, {0x37, 0x87},
		2, {0xE9, 0xF0}
	},
	{
		2, {0xE5, 0x91},
		3, {0xE8, 0x9C, 0xBB},
		2, {0x3B, 0x87},
		2, {0xE9, 0xF1}
	},
	{
		2, {0xE5, 0x92},
		3, {0xE8, 0x9C, 0xA5},
		2, {0x25, 0x87},
		2, {0xE9, 0xF2}
	},
	{
		2, {0xE5, 0x93},
		3, {0xE8, 0x9C, 0xA9},
		2, {0x29, 0x87},
		2, {0xE9, 0xF3}
	},
	{
		2, {0xE5, 0x94},
		3, {0xE8, 0x9C, 0x9A},
		2, {0x1A, 0x87},
		2, {0xE9, 0xF4}
	},
	{
		2, {0xE5, 0x95},
		3, {0xE8, 0x9D, 0xA0},
		2, {0x60, 0x87},
		2, {0xE9, 0xF5}
	},
	{
		2, {0xE5, 0x96},
		3, {0xE8, 0x9D, 0x9F},
		2, {0x5F, 0x87},
		2, {0xE9, 0xF6}
	},
	{
		2, {0xE5, 0x97},
		3, {0xE8, 0x9D, 0xB8},
		2, {0x78, 0x87},
		2, {0xE9, 0xF7}
	},
	{
		2, {0xE5, 0x98},
		3, {0xE8, 0x9D, 0x8C},
		2, {0x4C, 0x87},
		2, {0xE9, 0xF8}
	},
	{
		2, {0xE5, 0x99},
		3, {0xE8, 0x9D, 0x8E},
		2, {0x4E, 0x87},
		2, {0xE9, 0xF9}
	},
	{
		2, {0xE5, 0x9A},
		3, {0xE8, 0x9D, 0xB4},
		2, {0x74, 0x87},
		2, {0xE9, 0xFA}
	},
	{
		2, {0xE5, 0x9B},
		3, {0xE8, 0x9D, 0x97},
		2, {0x57, 0x87},
		2, {0xE9, 0xFB}
	},
	{
		2, {0xE5, 0x9C},
		3, {0xE8, 0x9D, 0xA8},
		2, {0x68, 0x87},
		2, {0xE9, 0xFC}
	},
	{
		2, {0xE5, 0x9D},
		3, {0xE8, 0x9D, 0xAE},
		2, {0x6E, 0x87},
		2, {0xE9, 0xFD}
	},
	{
		2, {0xE5, 0x9E},
		3, {0xE8, 0x9D, 0x99},
		2, {0x59, 0x87},
		2, {0xE9, 0xFE}
	},
	{
		2, {0xE5, 0x9F},
		3, {0xE8, 0x9D, 0x93},
		2, {0x53, 0x87},
		2, {0xEA, 0xA1}
	},
	{
		2, {0xE5, 0xA0},
		3, {0xE8, 0x9D, 0xA3},
		2, {0x63, 0x87},
		2, {0xEA, 0xA2}
	},
	{
		2, {0xE5, 0xA1},
		3, {0xE8, 0x9D, 0xAA},
		2, {0x6A, 0x87},
		2, {0xEA, 0xA3}
	},
	{
		2, {0xE5, 0xA2},
		3, {0xE8, 0xA0, 0x85},
		2, {0x05, 0x88},
		2, {0xEA, 0xA4}
	},
	{
		2, {0xE5, 0xA3},
		3, {0xE8, 0x9E, 0xA2},
		2, {0xA2, 0x87},
		2, {0xEA, 0xA5}
	},
	{
		2, {0xE5, 0xA4},
		3, {0xE8, 0x9E, 0x9F},
		2, {0x9F, 0x87},
		2, {0xEA, 0xA6}
	},
	{
		2, {0xE5, 0xA5},
		3, {0xE8, 0x9E, 0x82},
		2, {0x82, 0x87},
		2, {0xEA, 0xA7}
	},
	{
		2, {0xE5, 0xA6},
		3, {0xE8, 0x9E, 0xAF},
		2, {0xAF, 0x87},
		2, {0xEA, 0xA8}
	},
	{
		2, {0xE5, 0xA7},
		3, {0xE8, 0x9F, 0x8B},
		2, {0xCB, 0x87},
		2, {0xEA, 0xA9}
	},
	{
		2, {0xE5, 0xA8},
		3, {0xE8, 0x9E, 0xBD},
		2, {0xBD, 0x87},
		2, {0xEA, 0xAA}
	},
	{
		2, {0xE5, 0xA9},
		3, {0xE8, 0x9F, 0x80},
		2, {0xC0, 0x87},
		2, {0xEA, 0xAB}
	},
	{
		2, {0xE5, 0xAA},
		3, {0xE8, 0x9F, 0x90},
		2, {0xD0, 0x87},
		2, {0xEA, 0xAC}
	},
	{
		2, {0xE5, 0xAB},
		3, {0xE9, 0x9B, 0x96},
		2, {0xD6, 0x96},
		2, {0xEA, 0xAD}
	},
	{
		2, {0xE5, 0xAC},
		3, {0xE8, 0x9E, 0xAB},
		2, {0xAB, 0x87},
		2, {0xEA, 0xAE}
	},
	{
		2, {0xE5, 0xAD},
		3, {0xE8, 0x9F, 0x84},
		2, {0xC4, 0x87},
		2, {0xEA, 0xAF}
	},
	{
		2, {0xE5, 0xAE},
		3, {0xE8, 0x9E, 0xB3},
		2, {0xB3, 0x87},
		2, {0xEA, 0xB0}
	},
	{
		2, {0xE5, 0xAF},
		3, {0xE8, 0x9F, 0x87},
		2, {0xC7, 0x87},
		2, {0xEA, 0xB1}
	},
	{
		2, {0xE5, 0xB0},
		3, {0xE8, 0x9F, 0x86},
		2, {0xC6, 0x87},
		2, {0xEA, 0xB2}
	},
	{
		2, {0xE5, 0xB1},
		3, {0xE8, 0x9E, 0xBB},
		2, {0xBB, 0x87},
		2, {0xEA, 0xB3}
	},
	{
		2, {0xE5, 0xB2},
		3, {0xE8, 0x9F, 0xAF},
		2, {0xEF, 0x87},
		2, {0xEA, 0xB4}
	},
	{
		2, {0xE5, 0xB3},
		3, {0xE8, 0x9F, 0xB2},
		2, {0xF2, 0x87},
		2, {0xEA, 0xB5}
	},
	{
		2, {0xE5, 0xB4},
		3, {0xE8, 0x9F, 0xA0},
		2, {0xE0, 0x87},
		2, {0xEA, 0xB6}
	},
	{
		2, {0xE5, 0xB5},
		3, {0xE8, 0xA0, 0x8F},
		2, {0x0F, 0x88},
		2, {0xEA, 0xB7}
	},
	{
		2, {0xE5, 0xB6},
		3, {0xE8, 0xA0, 0x8D},
		2, {0x0D, 0x88},
		2, {0xEA, 0xB8}
	},
	{
		2, {0xE5, 0xB7},
		3, {0xE8, 0x9F, 0xBE},
		2, {0xFE, 0x87},
		2, {0xEA, 0xB9}
	},
	{
		2, {0xE5, 0xB8},
		3, {0xE8, 0x9F, 0xB6},
		2, {0xF6, 0x87},
		2, {0xEA, 0xBA}
	},
	{
		2, {0xE5, 0xB9},
		3, {0xE8, 0x9F, 0xB7},
		2, {0xF7, 0x87},
		2, {0xEA, 0xBB}
	},
	{
		2, {0xE5, 0xBA},
		3, {0xE8, 0xA0, 0x8E},
		2, {0x0E, 0x88},
		2, {0xEA, 0xBC}
	},
	{
		2, {0xE5, 0xBB},
		3, {0xE8, 0x9F, 0x92},
		2, {0xD2, 0x87},
		2, {0xEA, 0xBD}
	},
	{
		2, {0xE5, 0xBC},
		3, {0xE8, 0xA0, 0x91},
		2, {0x11, 0x88},
		2, {0xEA, 0xBE}
	},
	{
		2, {0xE5, 0xBD},
		3, {0xE8, 0xA0, 0x96},
		2, {0x16, 0x88},
		2, {0xEA, 0xBF}
	},
	{
		2, {0xE5, 0xBE},
		3, {0xE8, 0xA0, 0x95},
		2, {0x15, 0x88},
		2, {0xEA, 0xC0}
	},
	{
		2, {0xE5, 0xBF},
		3, {0xE8, 0xA0, 0xA2},
		2, {0x22, 0x88},
		2, {0xEA, 0xC1}
	},
	{
		2, {0xE5, 0xC0},
		3, {0xE8, 0xA0, 0xA1},
		2, {0x21, 0x88},
		2, {0xEA, 0xC2}
	},
	{
		2, {0xE5, 0xC1},
		3, {0xE8, 0xA0, 0xB1},
		2, {0x31, 0x88},
		2, {0xEA, 0xC3}
	},
	{
		2, {0xE5, 0xC2},
		3, {0xE8, 0xA0, 0xB6},
		2, {0x36, 0x88},
		2, {0xEA, 0xC4}
	},
	{
		2, {0xE5, 0xC3},
		3, {0xE8, 0xA0, 0xB9},
		2, {0x39, 0x88},
		2, {0xEA, 0xC5}
	},
	{
		2, {0xE5, 0xC4},
		3, {0xE8, 0xA0, 0xA7},
		2, {0x27, 0x88},
		2, {0xEA, 0xC6}
	},
	{
		2, {0xE5, 0xC5},
		3, {0xE8, 0xA0, 0xBB},
		2, {0x3B, 0x88},
		2, {0xEA, 0xC7}
	},
	{
		2, {0xE5, 0xC6},
		3, {0xE8, 0xA1, 0x84},
		2, {0x44, 0x88},
		2, {0xEA, 0xC8}
	},
	{
		2, {0xE5, 0xC7},
		3, {0xE8, 0xA1, 0x82},
		2, {0x42, 0x88},
		2, {0xEA, 0xC9}
	},
	{
		2, {0xE5, 0xC8},
		3, {0xE8, 0xA1, 0x92},
		2, {0x52, 0x88},
		2, {0xEA, 0xCA}
	},
	{
		2, {0xE5, 0xC9},
		3, {0xE8, 0xA1, 0x99},
		2, {0x59, 0x88},
		2, {0xEA, 0xCB}
	},
	{
		2, {0xE5, 0xCA},
		3, {0xE8, 0xA1, 0x9E},
		2, {0x5E, 0x88},
		2, {0xEA, 0xCC}
	},
	{
		2, {0xE5, 0xCB},
		3, {0xE8, 0xA1, 0xA2},
		2, {0x62, 0x88},
		2, {0xEA, 0xCD}
	},
	{
		2, {0xE5, 0xCC},
		3, {0xE8, 0xA1, 0xAB},
		2, {0x6B, 0x88},
		2, {0xEA, 0xCE}
	},
	{
		2, {0xE5, 0xCD},
		3, {0xE8, 0xA2, 0x81},
		2, {0x81, 0x88},
		2, {0xEA, 0xCF}
	},
	{
		2, {0xE5, 0xCE},
		3, {0xE8, 0xA1, 0xBE},
		2, {0x7E, 0x88},
		2, {0xEA, 0xD0}
	},
	{
		2, {0xE5, 0xCF},
		3, {0xE8, 0xA2, 0x9E},
		2, {0x9E, 0x88},
		2, {0xEA, 0xD1}
	},
	{
		2, {0xE5, 0xD0},
		3, {0xE8, 0xA1, 0xB5},
		2, {0x75, 0x88},
		2, {0xEA, 0xD2}
	},
	{
		2, {0xE5, 0xD1},
		3, {0xE8, 0xA1, 0xBD},
		2, {0x7D, 0x88},
		2, {0xEA, 0xD3}
	},
	{
		2, {0xE5, 0xD2},
		3, {0xE8, 0xA2, 0xB5},
		2, {0xB5, 0x88},
		2, {0xEA, 0xD4}
	},
	{
		2, {0xE5, 0xD3},
		3, {0xE8, 0xA1, 0xB2},
		2, {0x72, 0x88},
		2, {0xEA, 0xD5}
	},
	{
		2, {0xE5, 0xD4},
		3, {0xE8, 0xA2, 0x82},
		2, {0x82, 0x88},
		2, {0xEA, 0xD6}
	},
	{
		2, {0xE5, 0xD5},
		3, {0xE8, 0xA2, 0x97},
		2, {0x97, 0x88},
		2, {0xEA, 0xD7}
	},
	{
		2, {0xE5, 0xD6},
		3, {0xE8, 0xA2, 0x92},
		2, {0x92, 0x88},
		2, {0xEA, 0xD8}
	},
	{
		2, {0xE5, 0xD7},
		3, {0xE8, 0xA2, 0xAE},
		2, {0xAE, 0x88},
		2, {0xEA, 0xD9}
	},
	{
		2, {0xE5, 0xD8},
		3, {0xE8, 0xA2, 0x99},
		2, {0x99, 0x88},
		2, {0xEA, 0xDA}
	},
	{
		2, {0xE5, 0xD9},
		3, {0xE8, 0xA2, 0xA2},
		2, {0xA2, 0x88},
		2, {0xEA, 0xDB}
	},
	{
		2, {0xE5, 0xDA},
		3, {0xE8, 0xA2, 0x8D},
		2, {0x8D, 0x88},
		2, {0xEA, 0xDC}
	},
	{
		2, {0xE5, 0xDB},
		3, {0xE8, 0xA2, 0xA4},
		2, {0xA4, 0x88},
		2, {0xEA, 0xDD}
	},
	{
		2, {0xE5, 0xDC},
		3, {0xE8, 0xA2, 0xB0},
		2, {0xB0, 0x88},
		2, {0xEA, 0xDE}
	},
	{
		2, {0xE5, 0xDD},
		3, {0xE8, 0xA2, 0xBF},
		2, {0xBF, 0x88},
		2, {0xEA, 0xDF}
	},
	{
		2, {0xE5, 0xDE},
		3, {0xE8, 0xA2, 0xB1},
		2, {0xB1, 0x88},
		2, {0xEA, 0xE0}
	},
	{
		2, {0xE5, 0xDF},
		3, {0xE8, 0xA3, 0x83},
		2, {0xC3, 0x88},
		2, {0xEA, 0xE1}
	},
	{
		2, {0xE5, 0xE0},
		3, {0xE8, 0xA3, 0x84},
		2, {0xC4, 0x88},
		2, {0xEA, 0xE2}
	},
	{
		2, {0xE5, 0xE1},
		3, {0xE8, 0xA3, 0x94},
		2, {0xD4, 0x88},
		2, {0xEA, 0xE3}
	},
	{
		2, {0xE5, 0xE2},
		3, {0xE8, 0xA3, 0x98},
		2, {0xD8, 0x88},
		2, {0xEA, 0xE4}
	},
	{
		2, {0xE5, 0xE3},
		3, {0xE8, 0xA3, 0x99},
		2, {0xD9, 0x88},
		2, {0xEA, 0xE5}
	},
	{
		2, {0xE5, 0xE4},
		3, {0xE8, 0xA3, 0x9D},
		2, {0xDD, 0x88},
		2, {0xEA, 0xE6}
	},
	{
		2, {0xE5, 0xE5},
		3, {0xE8, 0xA3, 0xB9},
		2, {0xF9, 0x88},
		2, {0xEA, 0xE7}
	},
	{
		2, {0xE5, 0xE6},
		3, {0xE8, 0xA4, 0x82},
		2, {0x02, 0x89},
		2, {0xEA, 0xE8}
	},
	{
		2, {0xE5, 0xE7},
		3, {0xE8, 0xA3, 0xBC},
		2, {0xFC, 0x88},
		2, {0xEA, 0xE9}
	},
	{
		2, {0xE5, 0xE8},
		3, {0xE8, 0xA3, 0xB4},
		2, {0xF4, 0x88},
		2, {0xEA, 0xEA}
	},
	{
		2, {0xE5, 0xE9},
		3, {0xE8, 0xA3, 0xA8},
		2, {0xE8, 0x88},
		2, {0xEA, 0xEB}
	},
	{
		2, {0xE5, 0xEA},
		3, {0xE8, 0xA3, 0xB2},
		2, {0xF2, 0x88},
		2, {0xEA, 0xEC}
	},
	{
		2, {0xE5, 0xEB},
		3, {0xE8, 0xA4, 0x84},
		2, {0x04, 0x89},
		2, {0xEA, 0xED}
	},
	{
		2, {0xE5, 0xEC},
		3, {0xE8, 0xA4, 0x8C},
		2, {0x0C, 0x89},
		2, {0xEA, 0xEE}
	},
	{
		2, {0xE5, 0xED},
		3, {0xE8, 0xA4, 0x8A},
		2, {0x0A, 0x89},
		2, {0xEA, 0xEF}
	},
	{
		2, {0xE5, 0xEE},
		3, {0xE8, 0xA4, 0x93},
		2, {0x13, 0x89},
		2, {0xEA, 0xF0}
	},
	{
		2, {0xE5, 0xEF},
		3, {0xE8, 0xA5, 0x83},
		2, {0x43, 0x89},
		2, {0xEA, 0xF1}
	},
	{
		2, {0xE5, 0xF0},
		3, {0xE8, 0xA4, 0x9E},
		2, {0x1E, 0x89},
		2, {0xEA, 0xF2}
	},
	{
		2, {0xE5, 0xF1},
		3, {0xE8, 0xA4, 0xA5},
		2, {0x25, 0x89},
		2, {0xEA, 0xF3}
	},
	{
		2, {0xE5, 0xF2},
		3, {0xE8, 0xA4, 0xAA},
		2, {0x2A, 0x89},
		2, {0xEA, 0xF4}
	},
	{
		2, {0xE5, 0xF3},
		3, {0xE8, 0xA4, 0xAB},
		2, {0x2B, 0x89},
		2, {0xEA, 0xF5}
	},
	{
		2, {0xE5, 0xF4},
		3, {0xE8, 0xA5, 0x81},
		2, {0x41, 0x89},
		2, {0xEA, 0xF6}
	},
	{
		2, {0xE5, 0xF5},
		3, {0xE8, 0xA5, 0x84},
		2, {0x44, 0x89},
		2, {0xEA, 0xF7}
	},
	{
		2, {0xE5, 0xF6},
		3, {0xE8, 0xA4, 0xBB},
		2, {0x3B, 0x89},
		2, {0xEA, 0xF8}
	},
	{
		2, {0xE5, 0xF7},
		3, {0xE8, 0xA4, 0xB6},
		2, {0x36, 0x89},
		2, {0xEA, 0xF9}
	},
	{
		2, {0xE5, 0xF8},
		3, {0xE8, 0xA4, 0xB8},
		2, {0x38, 0x89},
		2, {0xEA, 0xFA}
	},
	{
		2, {0xE5, 0xF9},
		3, {0xE8, 0xA5, 0x8C},
		2, {0x4C, 0x89},
		2, {0xEA, 0xFB}
	},
	{
		2, {0xE5, 0xFA},
		3, {0xE8, 0xA4, 0x9D},
		2, {0x1D, 0x89},
		2, {0xEA, 0xFC}
	},
	{
		2, {0xE5, 0xFB},
		3, {0xE8, 0xA5, 0xA0},
		2, {0x60, 0x89},
		2, {0xEA, 0xFD}
	},
	{
		2, {0xE5, 0xFC},
		3, {0xE8, 0xA5, 0x9E},
		2, {0x5E, 0x89},
		2, {0xEA, 0xFE}
	},
	{
		2, {0xE6, 0x40},
		3, {0xE8, 0xA5, 0xA6},
		2, {0x66, 0x89},
		2, {0xEB, 0xA1}
	},
	{
		2, {0xE6, 0x41},
		3, {0xE8, 0xA5, 0xA4},
		2, {0x64, 0x89},
		2, {0xEB, 0xA2}
	},
	{
		2, {0xE6, 0x42},
		3, {0xE8, 0xA5, 0xAD},
		2, {0x6D, 0x89},
		2, {0xEB, 0xA3}
	},
	{
		2, {0xE6, 0x43},
		3, {0xE8, 0xA5, 0xAA},
		2, {0x6A, 0x89},
		2, {0xEB, 0xA4}
	},
	{
		2, {0xE6, 0x44},
		3, {0xE8, 0xA5, 0xAF},
		2, {0x6F, 0x89},
		2, {0xEB, 0xA5}
	},
	{
		2, {0xE6, 0x45},
		3, {0xE8, 0xA5, 0xB4},
		2, {0x74, 0x89},
		2, {0xEB, 0xA6}
	},
	{
		2, {0xE6, 0x46},
		3, {0xE8, 0xA5, 0xB7},
		2, {0x77, 0x89},
		2, {0xEB, 0xA7}
	},
	{
		2, {0xE6, 0x47},
		3, {0xE8, 0xA5, 0xBE},
		2, {0x7E, 0x89},
		2, {0xEB, 0xA8}
	},
	{
		2, {0xE6, 0x48},
		3, {0xE8, 0xA6, 0x83},
		2, {0x83, 0x89},
		2, {0xEB, 0xA9}
	},
	{
		2, {0xE6, 0x49},
		3, {0xE8, 0xA6, 0x88},
		2, {0x88, 0x89},
		2, {0xEB, 0xAA}
	},
	{
		2, {0xE6, 0x4A},
		3, {0xE8, 0xA6, 0x8A},
		2, {0x8A, 0x89},
		2, {0xEB, 0xAB}
	},
	{
		2, {0xE6, 0x4B},
		3, {0xE8, 0xA6, 0x93},
		2, {0x93, 0x89},
		2, {0xEB, 0xAC}
	},
	{
		2, {0xE6, 0x4C},
		3, {0xE8, 0xA6, 0x98},
		2, {0x98, 0x89},
		2, {0xEB, 0xAD}
	},
	{
		2, {0xE6, 0x4D},
		3, {0xE8, 0xA6, 0xA1},
		2, {0xA1, 0x89},
		2, {0xEB, 0xAE}
	},
	{
		2, {0xE6, 0x4E},
		3, {0xE8, 0xA6, 0xA9},
		2, {0xA9, 0x89},
		2, {0xEB, 0xAF}
	},
	{
		2, {0xE6, 0x4F},
		3, {0xE8, 0xA6, 0xA6},
		2, {0xA6, 0x89},
		2, {0xEB, 0xB0}
	},
	{
		2, {0xE6, 0x50},
		3, {0xE8, 0xA6, 0xAC},
		2, {0xAC, 0x89},
		2, {0xEB, 0xB1}
	},
	{
		2, {0xE6, 0x51},
		3, {0xE8, 0xA6, 0xAF},
		2, {0xAF, 0x89},
		2, {0xEB, 0xB2}
	},
	{
		2, {0xE6, 0x52},
		3, {0xE8, 0xA6, 0xB2},
		2, {0xB2, 0x89},
		2, {0xEB, 0xB3}
	},
	{
		2, {0xE6, 0x53},
		3, {0xE8, 0xA6, 0xBA},
		2, {0xBA, 0x89},
		2, {0xEB, 0xB4}
	},
	{
		2, {0xE6, 0x54},
		3, {0xE8, 0xA6, 0xBD},
		2, {0xBD, 0x89},
		2, {0xEB, 0xB5}
	},
	{
		2, {0xE6, 0x55},
		3, {0xE8, 0xA6, 0xBF},
		2, {0xBF, 0x89},
		2, {0xEB, 0xB6}
	},
	{
		2, {0xE6, 0x56},
		3, {0xE8, 0xA7, 0x80},
		2, {0xC0, 0x89},
		2, {0xEB, 0xB7}
	},
	{
		2, {0xE6, 0x57},
		3, {0xE8, 0xA7, 0x9A},
		2, {0xDA, 0x89},
		2, {0xEB, 0xB8}
	},
	{
		2, {0xE6, 0x58},
		3, {0xE8, 0xA7, 0x9C},
		2, {0xDC, 0x89},
		2, {0xEB, 0xB9}
	},
	{
		2, {0xE6, 0x59},
		3, {0xE8, 0xA7, 0x9D},
		2, {0xDD, 0x89},
		2, {0xEB, 0xBA}
	},
	{
		2, {0xE6, 0x5A},
		3, {0xE8, 0xA7, 0xA7},
		2, {0xE7, 0x89},
		2, {0xEB, 0xBB}
	},
	{
		2, {0xE6, 0x5B},
		3, {0xE8, 0xA7, 0xB4},
		2, {0xF4, 0x89},
		2, {0xEB, 0xBC}
	},
	{
		2, {0xE6, 0x5C},
		3, {0xE8, 0xA7, 0xB8},
		2, {0xF8, 0x89},
		2, {0xEB, 0xBD}
	},
	{
		2, {0xE6, 0x5D},
		3, {0xE8, 0xA8, 0x83},
		2, {0x03, 0x8A},
		2, {0xEB, 0xBE}
	},
	{
		2, {0xE6, 0x5E},
		3, {0xE8, 0xA8, 0x96},
		2, {0x16, 0x8A},
		2, {0xEB, 0xBF}
	},
	{
		2, {0xE6, 0x5F},
		3, {0xE8, 0xA8, 0x90},
		2, {0x10, 0x8A},
		2, {0xEB, 0xC0}
	},
	{
		2, {0xE6, 0x60},
		3, {0xE8, 0xA8, 0x8C},
		2, {0x0C, 0x8A},
		2, {0xEB, 0xC1}
	},
	{
		2, {0xE6, 0x61},
		3, {0xE8, 0xA8, 0x9B},
		2, {0x1B, 0x8A},
		2, {0xEB, 0xC2}
	},
	{
		2, {0xE6, 0x62},
		3, {0xE8, 0xA8, 0x9D},
		2, {0x1D, 0x8A},
		2, {0xEB, 0xC3}
	},
	{
		2, {0xE6, 0x63},
		3, {0xE8, 0xA8, 0xA5},
		2, {0x25, 0x8A},
		2, {0xEB, 0xC4}
	},
	{
		2, {0xE6, 0x64},
		3, {0xE8, 0xA8, 0xB6},
		2, {0x36, 0x8A},
		2, {0xEB, 0xC5}
	},
	{
		2, {0xE6, 0x65},
		3, {0xE8, 0xA9, 0x81},
		2, {0x41, 0x8A},
		2, {0xEB, 0xC6}
	},
	{
		2, {0xE6, 0x66},
		3, {0xE8, 0xA9, 0x9B},
		2, {0x5B, 0x8A},
		2, {0xEB, 0xC7}
	},
	{
		2, {0xE6, 0x67},
		3, {0xE8, 0xA9, 0x92},
		2, {0x52, 0x8A},
		2, {0xEB, 0xC8}
	},
	{
		2, {0xE6, 0x68},
		3, {0xE8, 0xA9, 0x86},
		2, {0x46, 0x8A},
		2, {0xEB, 0xC9}
	},
	{
		2, {0xE6, 0x69},
		3, {0xE8, 0xA9, 0x88},
		2, {0x48, 0x8A},
		2, {0xEB, 0xCA}
	},
	{
		2, {0xE6, 0x6A},
		3, {0xE8, 0xA9, 0xBC},
		2, {0x7C, 0x8A},
		2, {0xEB, 0xCB}
	},
	{
		2, {0xE6, 0x6B},
		3, {0xE8, 0xA9, 0xAD},
		2, {0x6D, 0x8A},
		2, {0xEB, 0xCC}
	},
	{
		2, {0xE6, 0x6C},
		3, {0xE8, 0xA9, 0xAC},
		2, {0x6C, 0x8A},
		2, {0xEB, 0xCD}
	},
	{
		2, {0xE6, 0x6D},
		3, {0xE8, 0xA9, 0xA2},
		2, {0x62, 0x8A},
		2, {0xEB, 0xCE}
	},
	{
		2, {0xE6, 0x6E},
		3, {0xE8, 0xAA, 0x85},
		2, {0x85, 0x8A},
		2, {0xEB, 0xCF}
	},
	{
		2, {0xE6, 0x6F},
		3, {0xE8, 0xAA, 0x82},
		2, {0x82, 0x8A},
		2, {0xEB, 0xD0}
	},
	{
		2, {0xE6, 0x70},
		3, {0xE8, 0xAA, 0x84},
		2, {0x84, 0x8A},
		2, {0xEB, 0xD1}
	},
	{
		2, {0xE6, 0x71},
		3, {0xE8, 0xAA, 0xA8},
		2, {0xA8, 0x8A},
		2, {0xEB, 0xD2}
	},
	{
		2, {0xE6, 0x72},
		3, {0xE8, 0xAA, 0xA1},
		2, {0xA1, 0x8A},
		2, {0xEB, 0xD3}
	},
	{
		2, {0xE6, 0x73},
		3, {0xE8, 0xAA, 0x91},
		2, {0x91, 0x8A},
		2, {0xEB, 0xD4}
	},
	{
		2, {0xE6, 0x74},
		3, {0xE8, 0xAA, 0xA5},
		2, {0xA5, 0x8A},
		2, {0xEB, 0xD5}
	},
	{
		2, {0xE6, 0x75},
		3, {0xE8, 0xAA, 0xA6},
		2, {0xA6, 0x8A},
		2, {0xEB, 0xD6}
	},
	{
		2, {0xE6, 0x76},
		3, {0xE8, 0xAA, 0x9A},
		2, {0x9A, 0x8A},
		2, {0xEB, 0xD7}
	},
	{
		2, {0xE6, 0x77},
		3, {0xE8, 0xAA, 0xA3},
		2, {0xA3, 0x8A},
		2, {0xEB, 0xD8}
	},
	{
		2, {0xE6, 0x78},
		3, {0xE8, 0xAB, 0x84},
		2, {0xC4, 0x8A},
		2, {0xEB, 0xD9}
	},
	{
		2, {0xE6, 0x79},
		3, {0xE8, 0xAB, 0x8D},
		2, {0xCD, 0x8A},
		2, {0xEB, 0xDA}
	},
	{
		2, {0xE6, 0x7A},
		3, {0xE8, 0xAB, 0x82},
		2, {0xC2, 0x8A},
		2, {0xEB, 0xDB}
	},
	{
		2, {0xE6, 0x7B},
		3, {0xE8, 0xAB, 0x9A},
		2, {0xDA, 0x8A},
		2, {0xEB, 0xDC}
	},
	{
		2, {0xE6, 0x7C},
		3, {0xE8, 0xAB, 0xAB},
		2, {0xEB, 0x8A},
		2, {0xEB, 0xDD}
	},
	{
		2, {0xE6, 0x7D},
		3, {0xE8, 0xAB, 0xB3},
		2, {0xF3, 0x8A},
		2, {0xEB, 0xDE}
	},
	{
		2, {0xE6, 0x7E},
		3, {0xE8, 0xAB, 0xA7},
		2, {0xE7, 0x8A},
		2, {0xEB, 0xDF}
	},
	{
		2, {0xE6, 0x80},
		3, {0xE8, 0xAB, 0xA4},
		2, {0xE4, 0x8A},
		2, {0xEB, 0xE0}
	},
	{
		2, {0xE6, 0x81},
		3, {0xE8, 0xAB, 0xB1},
		2, {0xF1, 0x8A},
		2, {0xEB, 0xE1}
	},
	{
		2, {0xE6, 0x82},
		3, {0xE8, 0xAC, 0x94},
		2, {0x14, 0x8B},
		2, {0xEB, 0xE2}
	},
	{
		2, {0xE6, 0x83},
		3, {0xE8, 0xAB, 0xA0},
		2, {0xE0, 0x8A},
		2, {0xEB, 0xE3}
	},
	{
		2, {0xE6, 0x84},
		3, {0xE8, 0xAB, 0xA2},
		2, {0xE2, 0x8A},
		2, {0xEB, 0xE4}
	},
	{
		2, {0xE6, 0x85},
		3, {0xE8, 0xAB, 0xB7},
		2, {0xF7, 0x8A},
		2, {0xEB, 0xE5}
	},
	{
		2, {0xE6, 0x86},
		3, {0xE8, 0xAB, 0x9E},
		2, {0xDE, 0x8A},
		2, {0xEB, 0xE6}
	},
	{
		2, {0xE6, 0x87},
		3, {0xE8, 0xAB, 0x9B},
		2, {0xDB, 0x8A},
		2, {0xEB, 0xE7}
	},
	{
		2, {0xE6, 0x88},
		3, {0xE8, 0xAC, 0x8C},
		2, {0x0C, 0x8B},
		2, {0xEB, 0xE8}
	},
	{
		2, {0xE6, 0x89},
		3, {0xE8, 0xAC, 0x87},
		2, {0x07, 0x8B},
		2, {0xEB, 0xE9}
	},
	{
		2, {0xE6, 0x8A},
		3, {0xE8, 0xAC, 0x9A},
		2, {0x1A, 0x8B},
		2, {0xEB, 0xEA}
	},
	{
		2, {0xE6, 0x8B},
		3, {0xE8, 0xAB, 0xA1},
		2, {0xE1, 0x8A},
		2, {0xEB, 0xEB}
	},
	{
		2, {0xE6, 0x8C},
		3, {0xE8, 0xAC, 0x96},
		2, {0x16, 0x8B},
		2, {0xEB, 0xEC}
	},
	{
		2, {0xE6, 0x8D},
		3, {0xE8, 0xAC, 0x90},
		2, {0x10, 0x8B},
		2, {0xEB, 0xED}
	},
	{
		2, {0xE6, 0x8E},
		3, {0xE8, 0xAC, 0x97},
		2, {0x17, 0x8B},
		2, {0xEB, 0xEE}
	},
	{
		2, {0xE6, 0x8F},
		3, {0xE8, 0xAC, 0xA0},
		2, {0x20, 0x8B},
		2, {0xEB, 0xEF}
	},
	{
		2, {0xE6, 0x90},
		3, {0xE8, 0xAC, 0xB3},
		2, {0x33, 0x8B},
		2, {0xEB, 0xF0}
	},
	{
		2, {0xE6, 0x91},
		3, {0xE9, 0x9E, 0xAB},
		2, {0xAB, 0x97},
		2, {0xEB, 0xF1}
	},
	{
		2, {0xE6, 0x92},
		3, {0xE8, 0xAC, 0xA6},
		2, {0x26, 0x8B},
		2, {0xEB, 0xF2}
	},
	{
		2, {0xE6, 0x93},
		3, {0xE8, 0xAC, 0xAB},
		2, {0x2B, 0x8B},
		2, {0xEB, 0xF3}
	},
	{
		2, {0xE6, 0x94},
		3, {0xE8, 0xAC, 0xBE},
		2, {0x3E, 0x8B},
		2, {0xEB, 0xF4}
	},
	{
		2, {0xE6, 0x95},
		3, {0xE8, 0xAC, 0xA8},
		2, {0x28, 0x8B},
		2, {0xEB, 0xF5}
	},
	{
		2, {0xE6, 0x96},
		3, {0xE8, 0xAD, 0x81},
		2, {0x41, 0x8B},
		2, {0xEB, 0xF6}
	},
	{
		2, {0xE6, 0x97},
		3, {0xE8, 0xAD, 0x8C},
		2, {0x4C, 0x8B},
		2, {0xEB, 0xF7}
	},
	{
		2, {0xE6, 0x98},
		3, {0xE8, 0xAD, 0x8F},
		2, {0x4F, 0x8B},
		2, {0xEB, 0xF8}
	},
	{
		2, {0xE6, 0x99},
		3, {0xE8, 0xAD, 0x8E},
		2, {0x4E, 0x8B},
		2, {0xEB, 0xF9}
	},
	{
		2, {0xE6, 0x9A},
		3, {0xE8, 0xAD, 0x89},
		2, {0x49, 0x8B},
		2, {0xEB, 0xFA}
	},
	{
		2, {0xE6, 0x9B},
		3, {0xE8, 0xAD, 0x96},
		2, {0x56, 0x8B},
		2, {0xEB, 0xFB}
	},
	{
		2, {0xE6, 0x9C},
		3, {0xE8, 0xAD, 0x9B},
		2, {0x5B, 0x8B},
		2, {0xEB, 0xFC}
	},
	{
		2, {0xE6, 0x9D},
		3, {0xE8, 0xAD, 0x9A},
		2, {0x5A, 0x8B},
		2, {0xEB, 0xFD}
	},
	{
		2, {0xE6, 0x9E},
		3, {0xE8, 0xAD, 0xAB},
		2, {0x6B, 0x8B},
		2, {0xEB, 0xFE}
	},
	{
		2, {0xE6, 0x9F},
		3, {0xE8, 0xAD, 0x9F},
		2, {0x5F, 0x8B},
		2, {0xEC, 0xA1}
	},
	{
		2, {0xE6, 0xA0},
		3, {0xE8, 0xAD, 0xAC},
		2, {0x6C, 0x8B},
		2, {0xEC, 0xA2}
	},
	{
		2, {0xE6, 0xA1},
		3, {0xE8, 0xAD, 0xAF},
		2, {0x6F, 0x8B},
		2, {0xEC, 0xA3}
	},
	{
		2, {0xE6, 0xA2},
		3, {0xE8, 0xAD, 0xB4},
		2, {0x74, 0x8B},
		2, {0xEC, 0xA4}
	},
	{
		2, {0xE6, 0xA3},
		3, {0xE8, 0xAD, 0xBD},
		2, {0x7D, 0x8B},
		2, {0xEC, 0xA5}
	},
	{
		2, {0xE6, 0xA4},
		3, {0xE8, 0xAE, 0x80},
		2, {0x80, 0x8B},
		2, {0xEC, 0xA6}
	},
	{
		2, {0xE6, 0xA5},
		3, {0xE8, 0xAE, 0x8C},
		2, {0x8C, 0x8B},
		2, {0xEC, 0xA7}
	},
	{
		2, {0xE6, 0xA6},
		3, {0xE8, 0xAE, 0x8E},
		2, {0x8E, 0x8B},
		2, {0xEC, 0xA8}
	},
	{
		2, {0xE6, 0xA7},
		3, {0xE8, 0xAE, 0x92},
		2, {0x92, 0x8B},
		2, {0xEC, 0xA9}
	},
	{
		2, {0xE6, 0xA8},
		3, {0xE8, 0xAE, 0x93},
		2, {0x93, 0x8B},
		2, {0xEC, 0xAA}
	},
	{
		2, {0xE6, 0xA9},
		3, {0xE8, 0xAE, 0x96},
		2, {0x96, 0x8B},
		2, {0xEC, 0xAB}
	},
	{
		2, {0xE6, 0xAA},
		3, {0xE8, 0xAE, 0x99},
		2, {0x99, 0x8B},
		2, {0xEC, 0xAC}
	},
	{
		2, {0xE6, 0xAB},
		3, {0xE8, 0xAE, 0x9A},
		2, {0x9A, 0x8B},
		2, {0xEC, 0xAD}
	},
	{
		2, {0xE6, 0xAC},
		3, {0xE8, 0xB0, 0xBA},
		2, {0x3A, 0x8C},
		2, {0xEC, 0xAE}
	},
	{
		2, {0xE6, 0xAD},
		3, {0xE8, 0xB1, 0x81},
		2, {0x41, 0x8C},
		2, {0xEC, 0xAF}
	},
	{
		2, {0xE6, 0xAE},
		3, {0xE8, 0xB0, 0xBF},
		2, {0x3F, 0x8C},
		2, {0xEC, 0xB0}
	},
	{
		2, {0xE6, 0xAF},
		3, {0xE8, 0xB1, 0x88},
		2, {0x48, 0x8C},
		2, {0xEC, 0xB1}
	},
	{
		2, {0xE6, 0xB0},
		3, {0xE8, 0xB1, 0x8C},
		2, {0x4C, 0x8C},
		2, {0xEC, 0xB2}
	},
	{
		2, {0xE6, 0xB1},
		3, {0xE8, 0xB1, 0x8E},
		2, {0x4E, 0x8C},
		2, {0xEC, 0xB3}
	},
	{
		2, {0xE6, 0xB2},
		3, {0xE8, 0xB1, 0x90},
		2, {0x50, 0x8C},
		2, {0xEC, 0xB4}
	},
	{
		2, {0xE6, 0xB3},
		3, {0xE8, 0xB1, 0x95},
		2, {0x55, 0x8C},
		2, {0xEC, 0xB5}
	},
	{
		2, {0xE6, 0xB4},
		3, {0xE8, 0xB1, 0xA2},
		2, {0x62, 0x8C},
		2, {0xEC, 0xB6}
	},
	{
		2, {0xE6, 0xB5},
		3, {0xE8, 0xB1, 0xAC},
		2, {0x6C, 0x8C},
		2, {0xEC, 0xB7}
	},
	{
		2, {0xE6, 0xB6},
		3, {0xE8, 0xB1, 0xB8},
		2, {0x78, 0x8C},
		2, {0xEC, 0xB8}
	},
	{
		2, {0xE6, 0xB7},
		3, {0xE8, 0xB1, 0xBA},
		2, {0x7A, 0x8C},
		2, {0xEC, 0xB9}
	},
	{
		2, {0xE6, 0xB8},
		3, {0xE8, 0xB2, 0x82},
		2, {0x82, 0x8C},
		2, {0xEC, 0xBA}
	},
	{
		2, {0xE6, 0xB9},
		3, {0xE8, 0xB2, 0x89},
		2, {0x89, 0x8C},
		2, {0xEC, 0xBB}
	},
	{
		2, {0xE6, 0xBA},
		3, {0xE8, 0xB2, 0x85},
		2, {0x85, 0x8C},
		2, {0xEC, 0xBC}
	},
	{
		2, {0xE6, 0xBB},
		3, {0xE8, 0xB2, 0x8A},
		2, {0x8A, 0x8C},
		2, {0xEC, 0xBD}
	},
	{
		2, {0xE6, 0xBC},
		3, {0xE8, 0xB2, 0x8D},
		2, {0x8D, 0x8C},
		2, {0xEC, 0xBE}
	},
	{
		2, {0xE6, 0xBD},
		3, {0xE8, 0xB2, 0x8E},
		2, {0x8E, 0x8C},
		2, {0xEC, 0xBF}
	},
	{
		2, {0xE6, 0xBE},
		3, {0xE8, 0xB2, 0x94},
		2, {0x94, 0x8C},
		2, {0xEC, 0xC0}
	},
	{
		2, {0xE6, 0xBF},
		3, {0xE8, 0xB1, 0xBC},
		2, {0x7C, 0x8C},
		2, {0xEC, 0xC1}
	},
	{
		2, {0xE6, 0xC0},
		3, {0xE8, 0xB2, 0x98},
		2, {0x98, 0x8C},
		2, {0xEC, 0xC2}
	},
	{
		2, {0xE6, 0xC1},
		3, {0xE6, 0x88, 0x9D},
		2, {0x1D, 0x62},
		2, {0xEC, 0xC3}
	},
	{
		2, {0xE6, 0xC2},
		3, {0xE8, 0xB2, 0xAD},
		2, {0xAD, 0x8C},
		2, {0xEC, 0xC4}
	},
	{
		2, {0xE6, 0xC3},
		3, {0xE8, 0xB2, 0xAA},
		2, {0xAA, 0x8C},
		2, {0xEC, 0xC5}
	},
	{
		2, {0xE6, 0xC4},
		3, {0xE8, 0xB2, 0xBD},
		2, {0xBD, 0x8C},
		2, {0xEC, 0xC6}
	},
	{
		2, {0xE6, 0xC5},
		3, {0xE8, 0xB2, 0xB2},
		2, {0xB2, 0x8C},
		2, {0xEC, 0xC7}
	},
	{
		2, {0xE6, 0xC6},
		3, {0xE8, 0xB2, 0xB3},
		2, {0xB3, 0x8C},
		2, {0xEC, 0xC8}
	},
	{
		2, {0xE6, 0xC7},
		3, {0xE8, 0xB2, 0xAE},
		2, {0xAE, 0x8C},
		2, {0xEC, 0xC9}
	},
	{
		2, {0xE6, 0xC8},
		3, {0xE8, 0xB2, 0xB6},
		2, {0xB6, 0x8C},
		2, {0xEC, 0xCA}
	},
	{
		2, {0xE6, 0xC9},
		3, {0xE8, 0xB3, 0x88},
		2, {0xC8, 0x8C},
		2, {0xEC, 0xCB}
	},
	{
		2, {0xE6, 0xCA},
		3, {0xE8, 0xB3, 0x81},
		2, {0xC1, 0x8C},
		2, {0xEC, 0xCC}
	},
	{
		2, {0xE6, 0xCB},
		3, {0xE8, 0xB3, 0xA4},
		2, {0xE4, 0x8C},
		2, {0xEC, 0xCD}
	},
	{
		2, {0xE6, 0xCC},
		3, {0xE8, 0xB3, 0xA3},
		2, {0xE3, 0x8C},
		2, {0xEC, 0xCE}
	},
	{
		2, {0xE6, 0xCD},
		3, {0xE8, 0xB3, 0x9A},
		2, {0xDA, 0x8C},
		2, {0xEC, 0xCF}
	},
	{
		2, {0xE6, 0xCE},
		3, {0xE8, 0xB3, 0xBD},
		2, {0xFD, 0x8C},
		2, {0xEC, 0xD0}
	},
	{
		2, {0xE6, 0xCF},
		3, {0xE8, 0xB3, 0xBA},
		2, {0xFA, 0x8C},
		2, {0xEC, 0xD1}
	},
	{
		2, {0xE6, 0xD0},
		3, {0xE8, 0xB3, 0xBB},
		2, {0xFB, 0x8C},
		2, {0xEC, 0xD2}
	},
	{
		2, {0xE6, 0xD1},
		3, {0xE8, 0xB4, 0x84},
		2, {0x04, 0x8D},
		2, {0xEC, 0xD3}
	},
	{
		2, {0xE6, 0xD2},
		3, {0xE8, 0xB4, 0x85},
		2, {0x05, 0x8D},
		2, {0xEC, 0xD4}
	},
	{
		2, {0xE6, 0xD3},
		3, {0xE8, 0xB4, 0x8A},
		2, {0x0A, 0x8D},
		2, {0xEC, 0xD5}
	},
	{
		2, {0xE6, 0xD4},
		3, {0xE8, 0xB4, 0x87},
		2, {0x07, 0x8D},
		2, {0xEC, 0xD6}
	},
	{
		2, {0xE6, 0xD5},
		3, {0xE8, 0xB4, 0x8F},
		2, {0x0F, 0x8D},
		2, {0xEC, 0xD7}
	},
	{
		2, {0xE6, 0xD6},
		3, {0xE8, 0xB4, 0x8D},
		2, {0x0D, 0x8D},
		2, {0xEC, 0xD8}
	},
	{
		2, {0xE6, 0xD7},
		3, {0xE8, 0xB4, 0x90},
		2, {0x10, 0x8D},
		2, {0xEC, 0xD9}
	},
	{
		2, {0xE6, 0xD8},
		3, {0xE9, 0xBD, 0x8E},
		2, {0x4E, 0x9F},
		2, {0xEC, 0xDA}
	},
	{
		2, {0xE6, 0xD9},
		3, {0xE8, 0xB4, 0x93},
		2, {0x13, 0x8D},
		2, {0xEC, 0xDB}
	},
	{
		2, {0xE6, 0xDA},
		3, {0xE8, 0xB3, 0x8D},
		2, {0xCD, 0x8C},
		2, {0xEC, 0xDC}
	},
	{
		2, {0xE6, 0xDB},
		3, {0xE8, 0xB4, 0x94},
		2, {0x14, 0x8D},
		2, {0xEC, 0xDD}
	},
	{
		2, {0xE6, 0xDC},
		3, {0xE8, 0xB4, 0x96},
		2, {0x16, 0x8D},
		2, {0xEC, 0xDE}
	},
	{
		2, {0xE6, 0xDD},
		3, {0xE8, 0xB5, 0xA7},
		2, {0x67, 0x8D},
		2, {0xEC, 0xDF}
	},
	{
		2, {0xE6, 0xDE},
		3, {0xE8, 0xB5, 0xAD},
		2, {0x6D, 0x8D},
		2, {0xEC, 0xE0}
	},
	{
		2, {0xE6, 0xDF},
		3, {0xE8, 0xB5, 0xB1},
		2, {0x71, 0x8D},
		2, {0xEC, 0xE1}
	},
	{
		2, {0xE6, 0xE0},
		3, {0xE8, 0xB5, 0xB3},
		2, {0x73, 0x8D},
		2, {0xEC, 0xE2}
	},
	{
		2, {0xE6, 0xE1},
		3, {0xE8, 0xB6, 0x81},
		2, {0x81, 0x8D},
		2, {0xEC, 0xE3}
	},
	{
		2, {0xE6, 0xE2},
		3, {0xE8, 0xB6, 0x99},
		2, {0x99, 0x8D},
		2, {0xEC, 0xE4}
	},
	{
		2, {0xE6, 0xE3},
		3, {0xE8, 0xB7, 0x82},
		2, {0xC2, 0x8D},
		2, {0xEC, 0xE5}
	},
	{
		2, {0xE6, 0xE4},
		3, {0xE8, 0xB6, 0xBE},
		2, {0xBE, 0x8D},
		2, {0xEC, 0xE6}
	},
	{
		2, {0xE6, 0xE5},
		3, {0xE8, 0xB6, 0xBA},
		2, {0xBA, 0x8D},
		2, {0xEC, 0xE7}
	},
	{
		2, {0xE6, 0xE6},
		3, {0xE8, 0xB7, 0x8F},
		2, {0xCF, 0x8D},
		2, {0xEC, 0xE8}
	},
	{
		2, {0xE6, 0xE7},
		3, {0xE8, 0xB7, 0x9A},
		2, {0xDA, 0x8D},
		2, {0xEC, 0xE9}
	},
	{
		2, {0xE6, 0xE8},
		3, {0xE8, 0xB7, 0x96},
		2, {0xD6, 0x8D},
		2, {0xEC, 0xEA}
	},
	{
		2, {0xE6, 0xE9},
		3, {0xE8, 0xB7, 0x8C},
		2, {0xCC, 0x8D},
		2, {0xEC, 0xEB}
	},
	{
		2, {0xE6, 0xEA},
		3, {0xE8, 0xB7, 0x9B},
		2, {0xDB, 0x8D},
		2, {0xEC, 0xEC}
	},
	{
		2, {0xE6, 0xEB},
		3, {0xE8, 0xB7, 0x8B},
		2, {0xCB, 0x8D},
		2, {0xEC, 0xED}
	},
	{
		2, {0xE6, 0xEC},
		3, {0xE8, 0xB7, 0xAA},
		2, {0xEA, 0x8D},
		2, {0xEC, 0xEE}
	},
	{
		2, {0xE6, 0xED},
		3, {0xE8, 0xB7, 0xAB},
		2, {0xEB, 0x8D},
		2, {0xEC, 0xEF}
	},
	{
		2, {0xE6, 0xEE},
		3, {0xE8, 0xB7, 0x9F},
		2, {0xDF, 0x8D},
		2, {0xEC, 0xF0}
	},
	{
		2, {0xE6, 0xEF},
		3, {0xE8, 0xB7, 0xA3},
		2, {0xE3, 0x8D},
		2, {0xEC, 0xF1}
	},
	{
		2, {0xE6, 0xF0},
		3, {0xE8, 0xB7, 0xBC},
		2, {0xFC, 0x8D},
		2, {0xEC, 0xF2}
	},
	{
		2, {0xE6, 0xF1},
		3, {0xE8, 0xB8, 0x88},
		2, {0x08, 0x8E},
		2, {0xEC, 0xF3}
	},
	{
		2, {0xE6, 0xF2},
		3, {0xE8, 0xB8, 0x89},
		2, {0x09, 0x8E},
		2, {0xEC, 0xF4}
	},
	{
		2, {0xE6, 0xF3},
		3, {0xE8, 0xB7, 0xBF},
		2, {0xFF, 0x8D},
		2, {0xEC, 0xF5}
	},
	{
		2, {0xE6, 0xF4},
		3, {0xE8, 0xB8, 0x9D},
		2, {0x1D, 0x8E},
		2, {0xEC, 0xF6}
	},
	{
		2, {0xE6, 0xF5},
		3, {0xE8, 0xB8, 0x9E},
		2, {0x1E, 0x8E},
		2, {0xEC, 0xF7}
	},
	{
		2, {0xE6, 0xF6},
		3, {0xE8, 0xB8, 0x90},
		2, {0x10, 0x8E},
		2, {0xEC, 0xF8}
	},
	{
		2, {0xE6, 0xF7},
		3, {0xE8, 0xB8, 0x9F},
		2, {0x1F, 0x8E},
		2, {0xEC, 0xF9}
	},
	{
		2, {0xE6, 0xF8},
		3, {0xE8, 0xB9, 0x82},
		2, {0x42, 0x8E},
		2, {0xEC, 0xFA}
	},
	{
		2, {0xE6, 0xF9},
		3, {0xE8, 0xB8, 0xB5},
		2, {0x35, 0x8E},
		2, {0xEC, 0xFB}
	},
	{
		2, {0xE6, 0xFA},
		3, {0xE8, 0xB8, 0xB0},
		2, {0x30, 0x8E},
		2, {0xEC, 0xFC}
	},
	{
		2, {0xE6, 0xFB},
		3, {0xE8, 0xB8, 0xB4},
		2, {0x34, 0x8E},
		2, {0xEC, 0xFD}
	},
	{
		2, {0xE6, 0xFC},
		3, {0xE8, 0xB9, 0x8A},
		2, {0x4A, 0x8E},
		2, {0xEC, 0xFE}
	},
	{
		2, {0xE7, 0x40},
		3, {0xE8, 0xB9, 0x87},
		2, {0x47, 0x8E},
		2, {0xED, 0xA1}
	},
	{
		2, {0xE7, 0x41},
		3, {0xE8, 0xB9, 0x89},
		2, {0x49, 0x8E},
		2, {0xED, 0xA2}
	},
	{
		2, {0xE7, 0x42},
		3, {0xE8, 0xB9, 0x8C},
		2, {0x4C, 0x8E},
		2, {0xED, 0xA3}
	},
	{
		2, {0xE7, 0x43},
		3, {0xE8, 0xB9, 0x90},
		2, {0x50, 0x8E},
		2, {0xED, 0xA4}
	},
	{
		2, {0xE7, 0x44},
		3, {0xE8, 0xB9, 0x88},
		2, {0x48, 0x8E},
		2, {0xED, 0xA5}
	},
	{
		2, {0xE7, 0x45},
		3, {0xE8, 0xB9, 0x99},
		2, {0x59, 0x8E},
		2, {0xED, 0xA6}
	},
	{
		2, {0xE7, 0x46},
		3, {0xE8, 0xB9, 0xA4},
		2, {0x64, 0x8E},
		2, {0xED, 0xA7}
	},
	{
		2, {0xE7, 0x47},
		3, {0xE8, 0xB9, 0xA0},
		2, {0x60, 0x8E},
		2, {0xED, 0xA8}
	},
	{
		2, {0xE7, 0x48},
		3, {0xE8, 0xB8, 0xAA},
		2, {0x2A, 0x8E},
		2, {0xED, 0xA9}
	},
	{
		2, {0xE7, 0x49},
		3, {0xE8, 0xB9, 0xA3},
		2, {0x63, 0x8E},
		2, {0xED, 0xAA}
	},
	{
		2, {0xE7, 0x4A},
		3, {0xE8, 0xB9, 0x95},
		2, {0x55, 0x8E},
		2, {0xED, 0xAB}
	},
	{
		2, {0xE7, 0x4B},
		3, {0xE8, 0xB9, 0xB6},
		2, {0x76, 0x8E},
		2, {0xED, 0xAC}
	},
	{
		2, {0xE7, 0x4C},
		3, {0xE8, 0xB9, 0xB2},
		2, {0x72, 0x8E},
		2, {0xED, 0xAD}
	},
	{
		2, {0xE7, 0x4D},
		3, {0xE8, 0xB9, 0xBC},
		2, {0x7C, 0x8E},
		2, {0xED, 0xAE}
	},
	{
		2, {0xE7, 0x4E},
		3, {0xE8, 0xBA, 0x81},
		2, {0x81, 0x8E},
		2, {0xED, 0xAF}
	},
	{
		2, {0xE7, 0x4F},
		3, {0xE8, 0xBA, 0x87},
		2, {0x87, 0x8E},
		2, {0xED, 0xB0}
	},
	{
		2, {0xE7, 0x50},
		3, {0xE8, 0xBA, 0x85},
		2, {0x85, 0x8E},
		2, {0xED, 0xB1}
	},
	{
		2, {0xE7, 0x51},
		3, {0xE8, 0xBA, 0x84},
		2, {0x84, 0x8E},
		2, {0xED, 0xB2}
	},
	{
		2, {0xE7, 0x52},
		3, {0xE8, 0xBA, 0x8B},
		2, {0x8B, 0x8E},
		2, {0xED, 0xB3}
	},
	{
		2, {0xE7, 0x53},
		3, {0xE8, 0xBA, 0x8A},
		2, {0x8A, 0x8E},
		2, {0xED, 0xB4}
	},
	{
		2, {0xE7, 0x54},
		3, {0xE8, 0xBA, 0x93},
		2, {0x93, 0x8E},
		2, {0xED, 0xB5}
	},
	{
		2, {0xE7, 0x55},
		3, {0xE8, 0xBA, 0x91},
		2, {0x91, 0x8E},
		2, {0xED, 0xB6}
	},
	{
		2, {0xE7, 0x56},
		3, {0xE8, 0xBA, 0x94},
		2, {0x94, 0x8E},
		2, {0xED, 0xB7}
	},
	{
		2, {0xE7, 0x57},
		3, {0xE8, 0xBA, 0x99},
		2, {0x99, 0x8E},
		2, {0xED, 0xB8}
	},
	{
		2, {0xE7, 0x58},
		3, {0xE8, 0xBA, 0xAA},
		2, {0xAA, 0x8E},
		2, {0xED, 0xB9}
	},
	{
		2, {0xE7, 0x59},
		3, {0xE8, 0xBA, 0xA1},
		2, {0xA1, 0x8E},
		2, {0xED, 0xBA}
	},
	{
		2, {0xE7, 0x5A},
		3, {0xE8, 0xBA, 0xAC},
		2, {0xAC, 0x8E},
		2, {0xED, 0xBB}
	},
	{
		2, {0xE7, 0x5B},
		3, {0xE8, 0xBA, 0xB0},
		2, {0xB0, 0x8E},
		2, {0xED, 0xBC}
	},
	{
		2, {0xE7, 0x5C},
		3, {0xE8, 0xBB, 0x86},
		2, {0xC6, 0x8E},
		2, {0xED, 0xBD}
	},
	{
		2, {0xE7, 0x5D},
		3, {0xE8, 0xBA, 0xB1},
		2, {0xB1, 0x8E},
		2, {0xED, 0xBE}
	},
	{
		2, {0xE7, 0x5E},
		3, {0xE8, 0xBA, 0xBE},
		2, {0xBE, 0x8E},
		2, {0xED, 0xBF}
	},
	{
		2, {0xE7, 0x5F},
		3, {0xE8, 0xBB, 0x85},
		2, {0xC5, 0x8E},
		2, {0xED, 0xC0}
	},
	{
		2, {0xE7, 0x60},
		3, {0xE8, 0xBB, 0x88},
		2, {0xC8, 0x8E},
		2, {0xED, 0xC1}
	},
	{
		2, {0xE7, 0x61},
		3, {0xE8, 0xBB, 0x8B},
		2, {0xCB, 0x8E},
		2, {0xED, 0xC2}
	},
	{
		2, {0xE7, 0x62},
		3, {0xE8, 0xBB, 0x9B},
		2, {0xDB, 0x8E},
		2, {0xED, 0xC3}
	},
	{
		2, {0xE7, 0x63},
		3, {0xE8, 0xBB, 0xA3},
		2, {0xE3, 0x8E},
		2, {0xED, 0xC4}
	},
	{
		2, {0xE7, 0x64},
		3, {0xE8, 0xBB, 0xBC},
		2, {0xFC, 0x8E},
		2, {0xED, 0xC5}
	},
	{
		2, {0xE7, 0x65},
		3, {0xE8, 0xBB, 0xBB},
		2, {0xFB, 0x8E},
		2, {0xED, 0xC6}
	},
	{
		2, {0xE7, 0x66},
		3, {0xE8, 0xBB, 0xAB},
		2, {0xEB, 0x8E},
		2, {0xED, 0xC7}
	},
	{
		2, {0xE7, 0x67},
		3, {0xE8, 0xBB, 0xBE},
		2, {0xFE, 0x8E},
		2, {0xED, 0xC8}
	},
	{
		2, {0xE7, 0x68},
		3, {0xE8, 0xBC, 0x8A},
		2, {0x0A, 0x8F},
		2, {0xED, 0xC9}
	},
	{
		2, {0xE7, 0x69},
		3, {0xE8, 0xBC, 0x85},
		2, {0x05, 0x8F},
		2, {0xED, 0xCA}
	},
	{
		2, {0xE7, 0x6A},
		3, {0xE8, 0xBC, 0x95},
		2, {0x15, 0x8F},
		2, {0xED, 0xCB}
	},
	{
		2, {0xE7, 0x6B},
		3, {0xE8, 0xBC, 0x92},
		2, {0x12, 0x8F},
		2, {0xED, 0xCC}
	},
	{
		2, {0xE7, 0x6C},
		3, {0xE8, 0xBC, 0x99},
		2, {0x19, 0x8F},
		2, {0xED, 0xCD}
	},
	{
		2, {0xE7, 0x6D},
		3, {0xE8, 0xBC, 0x93},
		2, {0x13, 0x8F},
		2, {0xED, 0xCE}
	},
	{
		2, {0xE7, 0x6E},
		3, {0xE8, 0xBC, 0x9C},
		2, {0x1C, 0x8F},
		2, {0xED, 0xCF}
	},
	{
		2, {0xE7, 0x6F},
		3, {0xE8, 0xBC, 0x9F},
		2, {0x1F, 0x8F},
		2, {0xED, 0xD0}
	},
	{
		2, {0xE7, 0x70},
		3, {0xE8, 0xBC, 0x9B},
		2, {0x1B, 0x8F},
		2, {0xED, 0xD1}
	},
	{
		2, {0xE7, 0x71},
		3, {0xE8, 0xBC, 0x8C},
		2, {0x0C, 0x8F},
		2, {0xED, 0xD2}
	},
	{
		2, {0xE7, 0x72},
		3, {0xE8, 0xBC, 0xA6},
		2, {0x26, 0x8F},
		2, {0xED, 0xD3}
	},
	{
		2, {0xE7, 0x73},
		3, {0xE8, 0xBC, 0xB3},
		2, {0x33, 0x8F},
		2, {0xED, 0xD4}
	},
	{
		2, {0xE7, 0x74},
		3, {0xE8, 0xBC, 0xBB},
		2, {0x3B, 0x8F},
		2, {0xED, 0xD5}
	},
	{
		2, {0xE7, 0x75},
		3, {0xE8, 0xBC, 0xB9},
		2, {0x39, 0x8F},
		2, {0xED, 0xD6}
	},
	{
		2, {0xE7, 0x76},
		3, {0xE8, 0xBD, 0x85},
		2, {0x45, 0x8F},
		2, {0xED, 0xD7}
	},
	{
		2, {0xE7, 0x77},
		3, {0xE8, 0xBD, 0x82},
		2, {0x42, 0x8F},
		2, {0xED, 0xD8}
	},
	{
		2, {0xE7, 0x78},
		3, {0xE8, 0xBC, 0xBE},
		2, {0x3E, 0x8F},
		2, {0xED, 0xD9}
	},
	{
		2, {0xE7, 0x79},
		3, {0xE8, 0xBD, 0x8C},
		2, {0x4C, 0x8F},
		2, {0xED, 0xDA}
	},
	{
		2, {0xE7, 0x7A},
		3, {0xE8, 0xBD, 0x89},
		2, {0x49, 0x8F},
		2, {0xED, 0xDB}
	},
	{
		2, {0xE7, 0x7B},
		3, {0xE8, 0xBD, 0x86},
		2, {0x46, 0x8F},
		2, {0xED, 0xDC}
	},
	{
		2, {0xE7, 0x7C},
		3, {0xE8, 0xBD, 0x8E},
		2, {0x4E, 0x8F},
		2, {0xED, 0xDD}
	},
	{
		2, {0xE7, 0x7D},
		3, {0xE8, 0xBD, 0x97},
		2, {0x57, 0x8F},
		2, {0xED, 0xDE}
	},
	{
		2, {0xE7, 0x7E},
		3, {0xE8, 0xBD, 0x9C},
		2, {0x5C, 0x8F},
		2, {0xED, 0xDF}
	},
	{
		2, {0xE7, 0x80},
		3, {0xE8, 0xBD, 0xA2},
		2, {0x62, 0x8F},
		2, {0xED, 0xE0}
	},
	{
		2, {0xE7, 0x81},
		3, {0xE8, 0xBD, 0xA3},
		2, {0x63, 0x8F},
		2, {0xED, 0xE1}
	},
	{
		2, {0xE7, 0x82},
		3, {0xE8, 0xBD, 0xA4},
		2, {0x64, 0x8F},
		2, {0xED, 0xE2}
	},
	{
		2, {0xE7, 0x83},
		3, {0xE8, 0xBE, 0x9C},
		2, {0x9C, 0x8F},
		2, {0xED, 0xE3}
	},
	{
		2, {0xE7, 0x84},
		3, {0xE8, 0xBE, 0x9F},
		2, {0x9F, 0x8F},
		2, {0xED, 0xE4}
	},
	{
		2, {0xE7, 0x85},
		3, {0xE8, 0xBE, 0xA3},
		2, {0xA3, 0x8F},
		2, {0xED, 0xE5}
	},
	{
		2, {0xE7, 0x86},
		3, {0xE8, 0xBE, 0xAD},
		2, {0xAD, 0x8F},
		2, {0xED, 0xE6}
	},
	{
		2, {0xE7, 0x87},
		3, {0xE8, 0xBE, 0xAF},
		2, {0xAF, 0x8F},
		2, {0xED, 0xE7}
	},
	{
		2, {0xE7, 0x88},
		3, {0xE8, 0xBE, 0xB7},
		2, {0xB7, 0x8F},
		2, {0xED, 0xE8}
	},
	{
		2, {0xE7, 0x89},
		3, {0xE8, 0xBF, 0x9A},
		2, {0xDA, 0x8F},
		2, {0xED, 0xE9}
	},
	{
		2, {0xE7, 0x8A},
		3, {0xE8, 0xBF, 0xA5},
		2, {0xE5, 0x8F},
		2, {0xED, 0xEA}
	},
	{
		2, {0xE7, 0x8B},
		3, {0xE8, 0xBF, 0xA2},
		2, {0xE2, 0x8F},
		2, {0xED, 0xEB}
	},
	{
		2, {0xE7, 0x8C},
		3, {0xE8, 0xBF, 0xAA},
		2, {0xEA, 0x8F},
		2, {0xED, 0xEC}
	},
	{
		2, {0xE7, 0x8D},
		3, {0xE8, 0xBF, 0xAF},
		2, {0xEF, 0x8F},
		2, {0xED, 0xED}
	},
	{
		2, {0xE7, 0x8E},
		3, {0xE9, 0x82, 0x87},
		2, {0x87, 0x90},
		2, {0xED, 0xEE}
	},
	{
		2, {0xE7, 0x8F},
		3, {0xE8, 0xBF, 0xB4},
		2, {0xF4, 0x8F},
		2, {0xED, 0xEF}
	},
	{
		2, {0xE7, 0x90},
		3, {0xE9, 0x80, 0x85},
		2, {0x05, 0x90},
		2, {0xED, 0xF0}
	},
	{
		2, {0xE7, 0x91},
		3, {0xE8, 0xBF, 0xB9},
		2, {0xF9, 0x8F},
		2, {0xED, 0xF1}
	},
	{
		2, {0xE7, 0x92},
		3, {0xE8, 0xBF, 0xBA},
		2, {0xFA, 0x8F},
		2, {0xED, 0xF2}
	},
	{
		2, {0xE7, 0x93},
		3, {0xE9, 0x80, 0x91},
		2, {0x11, 0x90},
		2, {0xED, 0xF3}
	},
	{
		2, {0xE7, 0x94},
		3, {0xE9, 0x80, 0x95},
		2, {0x15, 0x90},
		2, {0xED, 0xF4}
	},
	{
		2, {0xE7, 0x95},
		3, {0xE9, 0x80, 0xA1},
		2, {0x21, 0x90},
		2, {0xED, 0xF5}
	},
	{
		2, {0xE7, 0x96},
		3, {0xE9, 0x80, 0x8D},
		2, {0x0D, 0x90},
		2, {0xED, 0xF6}
	},
	{
		2, {0xE7, 0x97},
		3, {0xE9, 0x80, 0x9E},
		2, {0x1E, 0x90},
		2, {0xED, 0xF7}
	},
	{
		2, {0xE7, 0x98},
		3, {0xE9, 0x80, 0x96},
		2, {0x16, 0x90},
		2, {0xED, 0xF8}
	},
	{
		2, {0xE7, 0x99},
		3, {0xE9, 0x80, 0x8B},
		2, {0x0B, 0x90},
		2, {0xED, 0xF9}
	},
	{
		2, {0xE7, 0x9A},
		3, {0xE9, 0x80, 0xA7},
		2, {0x27, 0x90},
		2, {0xED, 0xFA}
	},
	{
		2, {0xE7, 0x9B},
		3, {0xE9, 0x80, 0xB6},
		2, {0x36, 0x90},
		2, {0xED, 0xFB}
	},
	{
		2, {0xE7, 0x9C},
		3, {0xE9, 0x80, 0xB5},
		2, {0x35, 0x90},
		2, {0xED, 0xFC}
	},
	{
		2, {0xE7, 0x9D},
		3, {0xE9, 0x80, 0xB9},
		2, {0x39, 0x90},
		2, {0xED, 0xFD}
	},
	{
		2, {0xE7, 0x9E},
		3, {0xE8, 0xBF, 0xB8},
		2, {0xF8, 0x8F},
		2, {0xED, 0xFE}
	},
	{
		2, {0xE7, 0x9F},
		3, {0xE9, 0x81, 0x8F},
		2, {0x4F, 0x90},
		2, {0xEE, 0xA1}
	},
	{
		2, {0xE7, 0xA0},
		3, {0xE9, 0x81, 0x90},
		2, {0x50, 0x90},
		2, {0xEE, 0xA2}
	},
	{
		2, {0xE7, 0xA1},
		3, {0xE9, 0x81, 0x91},
		2, {0x51, 0x90},
		2, {0xEE, 0xA3}
	},
	{
		2, {0xE7, 0xA2},
		3, {0xE9, 0x81, 0x92},
		2, {0x52, 0x90},
		2, {0xEE, 0xA4}
	},
	{
		2, {0xE7, 0xA3},
		3, {0xE9, 0x80, 0x8E},
		2, {0x0E, 0x90},
		2, {0xEE, 0xA5}
	},
	{
		2, {0xE7, 0xA4},
		3, {0xE9, 0x81, 0x89},
		2, {0x49, 0x90},
		2, {0xEE, 0xA6}
	},
	{
		2, {0xE7, 0xA5},
		3, {0xE9, 0x80, 0xBE},
		2, {0x3E, 0x90},
		2, {0xEE, 0xA7}
	},
	{
		2, {0xE7, 0xA6},
		3, {0xE9, 0x81, 0x96},
		2, {0x56, 0x90},
		2, {0xEE, 0xA8}
	},
	{
		2, {0xE7, 0xA7},
		3, {0xE9, 0x81, 0x98},
		2, {0x58, 0x90},
		2, {0xEE, 0xA9}
	},
	{
		2, {0xE7, 0xA8},
		3, {0xE9, 0x81, 0x9E},
		2, {0x5E, 0x90},
		2, {0xEE, 0xAA}
	},
	{
		2, {0xE7, 0xA9},
		3, {0xE9, 0x81, 0xA8},
		2, {0x68, 0x90},
		2, {0xEE, 0xAB}
	},
	{
		2, {0xE7, 0xAA},
		3, {0xE9, 0x81, 0xAF},
		2, {0x6F, 0x90},
		2, {0xEE, 0xAC}
	},
	{
		2, {0xE7, 0xAB},
		3, {0xE9, 0x81, 0xB6},
		2, {0x76, 0x90},
		2, {0xEE, 0xAD}
	},
	{
		2, {0xE7, 0xAC},
		3, {0xE9, 0x9A, 0xA8},
		2, {0xA8, 0x96},
		2, {0xEE, 0xAE}
	},
	{
		2, {0xE7, 0xAD},
		3, {0xE9, 0x81, 0xB2},
		2, {0x72, 0x90},
		2, {0xEE, 0xAF}
	},
	{
		2, {0xE7, 0xAE},
		3, {0xE9, 0x82, 0x82},
		2, {0x82, 0x90},
		2, {0xEE, 0xB0}
	},
	{
		2, {0xE7, 0xAF},
		3, {0xE9, 0x81, 0xBD},
		2, {0x7D, 0x90},
		2, {0xEE, 0xB1}
	},
	{
		2, {0xE7, 0xB0},
		3, {0xE9, 0x82, 0x81},
		2, {0x81, 0x90},
		2, {0xEE, 0xB2}
	},
	{
		2, {0xE7, 0xB1},
		3, {0xE9, 0x82, 0x80},
		2, {0x80, 0x90},
		2, {0xEE, 0xB3}
	},
	{
		2, {0xE7, 0xB2},
		3, {0xE9, 0x82, 0x8A},
		2, {0x8A, 0x90},
		2, {0xEE, 0xB4}
	},
	{
		2, {0xE7, 0xB3},
		3, {0xE9, 0x82, 0x89},
		2, {0x89, 0x90},
		2, {0xEE, 0xB5}
	},
	{
		2, {0xE7, 0xB4},
		3, {0xE9, 0x82, 0x8F},
		2, {0x8F, 0x90},
		2, {0xEE, 0xB6}
	},
	{
		2, {0xE7, 0xB5},
		3, {0xE9, 0x82, 0xA8},
		2, {0xA8, 0x90},
		2, {0xEE, 0xB7}
	},
	{
		2, {0xE7, 0xB6},
		3, {0xE9, 0x82, 0xAF},
		2, {0xAF, 0x90},
		2, {0xEE, 0xB8}
	},
	{
		2, {0xE7, 0xB7},
		3, {0xE9, 0x82, 0xB1},
		2, {0xB1, 0x90},
		2, {0xEE, 0xB9}
	},
	{
		2, {0xE7, 0xB8},
		3, {0xE9, 0x82, 0xB5},
		2, {0xB5, 0x90},
		2, {0xEE, 0xBA}
	},
	{
		2, {0xE7, 0xB9},
		3, {0xE9, 0x83, 0xA2},
		2, {0xE2, 0x90},
		2, {0xEE, 0xBB}
	},
	{
		2, {0xE7, 0xBA},
		3, {0xE9, 0x83, 0xA4},
		2, {0xE4, 0x90},
		2, {0xEE, 0xBC}
	},
	{
		2, {0xE7, 0xBB},
		3, {0xE6, 0x89, 0x88},
		2, {0x48, 0x62},
		2, {0xEE, 0xBD}
	},
	{
		2, {0xE7, 0xBC},
		3, {0xE9, 0x83, 0x9B},
		2, {0xDB, 0x90},
		2, {0xEE, 0xBE}
	},
	{
		2, {0xE7, 0xBD},
		3, {0xE9, 0x84, 0x82},
		2, {0x02, 0x91},
		2, {0xEE, 0xBF}
	},
	{
		2, {0xE7, 0xBE},
		3, {0xE9, 0x84, 0x92},
		2, {0x12, 0x91},
		2, {0xEE, 0xC0}
	},
	{
		2, {0xE7, 0xBF},
		3, {0xE9, 0x84, 0x99},
		2, {0x19, 0x91},
		2, {0xEE, 0xC1}
	},
	{
		2, {0xE7, 0xC0},
		3, {0xE9, 0x84, 0xB2},
		2, {0x32, 0x91},
		2, {0xEE, 0xC2}
	},
	{
		2, {0xE7, 0xC1},
		3, {0xE9, 0x84, 0xB0},
		2, {0x30, 0x91},
		2, {0xEE, 0xC3}
	},
	{
		2, {0xE7, 0xC2},
		3, {0xE9, 0x85, 0x8A},
		2, {0x4A, 0x91},
		2, {0xEE, 0xC4}
	},
	{
		2, {0xE7, 0xC3},
		3, {0xE9, 0x85, 0x96},
		2, {0x56, 0x91},
		2, {0xEE, 0xC5}
	},
	{
		2, {0xE7, 0xC4},
		3, {0xE9, 0x85, 0x98},
		2, {0x58, 0x91},
		2, {0xEE, 0xC6}
	},
	{
		2, {0xE7, 0xC5},
		3, {0xE9, 0x85, 0xA3},
		2, {0x63, 0x91},
		2, {0xEE, 0xC7}
	},
	{
		2, {0xE7, 0xC6},
		3, {0xE9, 0x85, 0xA5},
		2, {0x65, 0x91},
		2, {0xEE, 0xC8}
	},
	{
		2, {0xE7, 0xC7},
		3, {0xE9, 0x85, 0xA9},
		2, {0x69, 0x91},
		2, {0xEE, 0xC9}
	},
	{
		2, {0xE7, 0xC8},
		3, {0xE9, 0x85, 0xB3},
		2, {0x73, 0x91},
		2, {0xEE, 0xCA}
	},
	{
		2, {0xE7, 0xC9},
		3, {0xE9, 0x85, 0xB2},
		2, {0x72, 0x91},
		2, {0xEE, 0xCB}
	},
	{
		2, {0xE7, 0xCA},
		3, {0xE9, 0x86, 0x8B},
		2, {0x8B, 0x91},
		2, {0xEE, 0xCC}
	},
	{
		2, {0xE7, 0xCB},
		3, {0xE9, 0x86, 0x89},
		2, {0x89, 0x91},
		2, {0xEE, 0xCD}
	},
	{
		2, {0xE7, 0xCC},
		3, {0xE9, 0x86, 0x82},
		2, {0x82, 0x91},
		2, {0xEE, 0xCE}
	},
	{
		2, {0xE7, 0xCD},
		3, {0xE9, 0x86, 0xA2},
		2, {0xA2, 0x91},
		2, {0xEE, 0xCF}
	},
	{
		2, {0xE7, 0xCE},
		3, {0xE9, 0x86, 0xAB},
		2, {0xAB, 0x91},
		2, {0xEE, 0xD0}
	},
	{
		2, {0xE7, 0xCF},
		3, {0xE9, 0x86, 0xAF},
		2, {0xAF, 0x91},
		2, {0xEE, 0xD1}
	},
	{
		2, {0xE7, 0xD0},
		3, {0xE9, 0x86, 0xAA},
		2, {0xAA, 0x91},
		2, {0xEE, 0xD2}
	},
	{
		2, {0xE7, 0xD1},
		3, {0xE9, 0x86, 0xB5},
		2, {0xB5, 0x91},
		2, {0xEE, 0xD3}
	},
	{
		2, {0xE7, 0xD2},
		3, {0xE9, 0x86, 0xB4},
		2, {0xB4, 0x91},
		2, {0xEE, 0xD4}
	},
	{
		2, {0xE7, 0xD3},
		3, {0xE9, 0x86, 0xBA},
		2, {0xBA, 0x91},
		2, {0xEE, 0xD5}
	},
	{
		2, {0xE7, 0xD4},
		3, {0xE9, 0x87, 0x80},
		2, {0xC0, 0x91},
		2, {0xEE, 0xD6}
	},
	{
		2, {0xE7, 0xD5},
		3, {0xE9, 0x87, 0x81},
		2, {0xC1, 0x91},
		2, {0xEE, 0xD7}
	},
	{
		2, {0xE7, 0xD6},
		3, {0xE9, 0x87, 0x89},
		2, {0xC9, 0x91},
		2, {0xEE, 0xD8}
	},
	{
		2, {0xE7, 0xD7},
		3, {0xE9, 0x87, 0x8B},
		2, {0xCB, 0x91},
		2, {0xEE, 0xD9}
	},
	{
		2, {0xE7, 0xD8},
		3, {0xE9, 0x87, 0x90},
		2, {0xD0, 0x91},
		2, {0xEE, 0xDA}
	},
	{
		2, {0xE7, 0xD9},
		3, {0xE9, 0x87, 0x96},
		2, {0xD6, 0x91},
		2, {0xEE, 0xDB}
	},
	{
		2, {0xE7, 0xDA},
		3, {0xE9, 0x87, 0x9F},
		2, {0xDF, 0x91},
		2, {0xEE, 0xDC}
	},
	{
		2, {0xE7, 0xDB},
		3, {0xE9, 0x87, 0xA1},
		2, {0xE1, 0x91},
		2, {0xEE, 0xDD}
	},
	{
		2, {0xE7, 0xDC},
		3, {0xE9, 0x87, 0x9B},
		2, {0xDB, 0x91},
		2, {0xEE, 0xDE}
	},
	{
		2, {0xE7, 0xDD},
		3, {0xE9, 0x87, 0xBC},
		2, {0xFC, 0x91},
		2, {0xEE, 0xDF}
	},
	{
		2, {0xE7, 0xDE},
		3, {0xE9, 0x87, 0xB5},
		2, {0xF5, 0x91},
		2, {0xEE, 0xE0}
	},
	{
		2, {0xE7, 0xDF},
		3, {0xE9, 0x87, 0xB6},
		2, {0xF6, 0x91},
		2, {0xEE, 0xE1}
	},
	{
		2, {0xE7, 0xE0},
		3, {0xE9, 0x88, 0x9E},
		2, {0x1E, 0x92},
		2, {0xEE, 0xE2}
	},
	{
		2, {0xE7, 0xE1},
		3, {0xE9, 0x87, 0xBF},
		2, {0xFF, 0x91},
		2, {0xEE, 0xE3}
	},
	{
		2, {0xE7, 0xE2},
		3, {0xE9, 0x88, 0x94},
		2, {0x14, 0x92},
		2, {0xEE, 0xE4}
	},
	{
		2, {0xE7, 0xE3},
		3, {0xE9, 0x88, 0xAC},
		2, {0x2C, 0x92},
		2, {0xEE, 0xE5}
	},
	{
		2, {0xE7, 0xE4},
		3, {0xE9, 0x88, 0x95},
		2, {0x15, 0x92},
		2, {0xEE, 0xE6}
	},
	{
		2, {0xE7, 0xE5},
		3, {0xE9, 0x88, 0x91},
		2, {0x11, 0x92},
		2, {0xEE, 0xE7}
	},
	{
		2, {0xE7, 0xE6},
		3, {0xE9, 0x89, 0x9E},
		2, {0x5E, 0x92},
		2, {0xEE, 0xE8}
	},
	{
		2, {0xE7, 0xE7},
		3, {0xE9, 0x89, 0x97},
		2, {0x57, 0x92},
		2, {0xEE, 0xE9}
	},
	{
		2, {0xE7, 0xE8},
		3, {0xE9, 0x89, 0x85},
		2, {0x45, 0x92},
		2, {0xEE, 0xEA}
	},
	{
		2, {0xE7, 0xE9},
		3, {0xE9, 0x89, 0x89},
		2, {0x49, 0x92},
		2, {0xEE, 0xEB}
	},
	{
		2, {0xE7, 0xEA},
		3, {0xE9, 0x89, 0xA4},
		2, {0x64, 0x92},
		2, {0xEE, 0xEC}
	},
	{
		2, {0xE7, 0xEB},
		3, {0xE9, 0x89, 0x88},
		2, {0x48, 0x92},
		2, {0xEE, 0xED}
	},
	{
		2, {0xE7, 0xEC},
		3, {0xE9, 0x8A, 0x95},
		2, {0x95, 0x92},
		2, {0xEE, 0xEE}
	},
	{
		2, {0xE7, 0xED},
		3, {0xE9, 0x88, 0xBF},
		2, {0x3F, 0x92},
		2, {0xEE, 0xEF}
	},
	{
		2, {0xE7, 0xEE},
		3, {0xE9, 0x89, 0x8B},
		2, {0x4B, 0x92},
		2, {0xEE, 0xF0}
	},
	{
		2, {0xE7, 0xEF},
		3, {0xE9, 0x89, 0x90},
		2, {0x50, 0x92},
		2, {0xEE, 0xF1}
	},
	{
		2, {0xE7, 0xF0},
		3, {0xE9, 0x8A, 0x9C},
		2, {0x9C, 0x92},
		2, {0xEE, 0xF2}
	},
	{
		2, {0xE7, 0xF1},
		3, {0xE9, 0x8A, 0x96},
		2, {0x96, 0x92},
		2, {0xEE, 0xF3}
	},
	{
		2, {0xE7, 0xF2},
		3, {0xE9, 0x8A, 0x93},
		2, {0x93, 0x92},
		2, {0xEE, 0xF4}
	},
	{
		2, {0xE7, 0xF3},
		3, {0xE9, 0x8A, 0x9B},
		2, {0x9B, 0x92},
		2, {0xEE, 0xF5}
	},
	{
		2, {0xE7, 0xF4},
		3, {0xE9, 0x89, 0x9A},
		2, {0x5A, 0x92},
		2, {0xEE, 0xF6}
	},
	{
		2, {0xE7, 0xF5},
		3, {0xE9, 0x8B, 0x8F},
		2, {0xCF, 0x92},
		2, {0xEE, 0xF7}
	},
	{
		2, {0xE7, 0xF6},
		3, {0xE9, 0x8A, 0xB9},
		2, {0xB9, 0x92},
		2, {0xEE, 0xF8}
	},
	{
		2, {0xE7, 0xF7},
		3, {0xE9, 0x8A, 0xB7},
		2, {0xB7, 0x92},
		2, {0xEE, 0xF9}
	},
	{
		2, {0xE7, 0xF8},
		3, {0xE9, 0x8B, 0xA9},
		2, {0xE9, 0x92},
		2, {0xEE, 0xFA}
	},
	{
		2, {0xE7, 0xF9},
		3, {0xE9, 0x8C, 0x8F},
		2, {0x0F, 0x93},
		2, {0xEE, 0xFB}
	},
	{
		2, {0xE7, 0xFA},
		3, {0xE9, 0x8B, 0xBA},
		2, {0xFA, 0x92},
		2, {0xEE, 0xFC}
	},
	{
		2, {0xE7, 0xFB},
		3, {0xE9, 0x8D, 0x84},
		2, {0x44, 0x93},
		2, {0xEE, 0xFD}
	},
	{
		2, {0xE7, 0xFC},
		3, {0xE9, 0x8C, 0xAE},
		2, {0x2E, 0x93},
		2, {0xEE, 0xFE}
	},
	{
		2, {0xE8, 0x40},
		3, {0xE9, 0x8C, 0x99},
		2, {0x19, 0x93},
		2, {0xEF, 0xA1}
	},
	{
		2, {0xE8, 0x41},
		3, {0xE9, 0x8C, 0xA2},
		2, {0x22, 0x93},
		2, {0xEF, 0xA2}
	},
	{
		2, {0xE8, 0x42},
		3, {0xE9, 0x8C, 0x9A},
		2, {0x1A, 0x93},
		2, {0xEF, 0xA3}
	},
	{
		2, {0xE8, 0x43},
		3, {0xE9, 0x8C, 0xA3},
		2, {0x23, 0x93},
		2, {0xEF, 0xA4}
	},
	{
		2, {0xE8, 0x44},
		3, {0xE9, 0x8C, 0xBA},
		2, {0x3A, 0x93},
		2, {0xEF, 0xA5}
	},
	{
		2, {0xE8, 0x45},
		3, {0xE9, 0x8C, 0xB5},
		2, {0x35, 0x93},
		2, {0xEF, 0xA6}
	},
	{
		2, {0xE8, 0x46},
		3, {0xE9, 0x8C, 0xBB},
		2, {0x3B, 0x93},
		2, {0xEF, 0xA7}
	},
	{
		2, {0xE8, 0x47},
		3, {0xE9, 0x8D, 0x9C},
		2, {0x5C, 0x93},
		2, {0xEF, 0xA8}
	},
	{
		2, {0xE8, 0x48},
		3, {0xE9, 0x8D, 0xA0},
		2, {0x60, 0x93},
		2, {0xEF, 0xA9}
	},
	{
		2, {0xE8, 0x49},
		3, {0xE9, 0x8D, 0xBC},
		2, {0x7C, 0x93},
		2, {0xEF, 0xAA}
	},
	{
		2, {0xE8, 0x4A},
		3, {0xE9, 0x8D, 0xAE},
		2, {0x6E, 0x93},
		2, {0xEF, 0xAB}
	},
	{
		2, {0xE8, 0x4B},
		3, {0xE9, 0x8D, 0x96},
		2, {0x56, 0x93},
		2, {0xEF, 0xAC}
	},
	{
		2, {0xE8, 0x4C},
		3, {0xE9, 0x8E, 0xB0},
		2, {0xB0, 0x93},
		2, {0xEF, 0xAD}
	},
	{
		2, {0xE8, 0x4D},
		3, {0xE9, 0x8E, 0xAC},
		2, {0xAC, 0x93},
		2, {0xEF, 0xAE}
	},
	{
		2, {0xE8, 0x4E},
		3, {0xE9, 0x8E, 0xAD},
		2, {0xAD, 0x93},
		2, {0xEF, 0xAF}
	},
	{
		2, {0xE8, 0x4F},
		3, {0xE9, 0x8E, 0x94},
		2, {0x94, 0x93},
		2, {0xEF, 0xB0}
	},
	{
		2, {0xE8, 0x50},
		3, {0xE9, 0x8E, 0xB9},
		2, {0xB9, 0x93},
		2, {0xEF, 0xB1}
	},
	{
		2, {0xE8, 0x51},
		3, {0xE9, 0x8F, 0x96},
		2, {0xD6, 0x93},
		2, {0xEF, 0xB2}
	},
	{
		2, {0xE8, 0x52},
		3, {0xE9, 0x8F, 0x97},
		2, {0xD7, 0x93},
		2, {0xEF, 0xB3}
	},
	{
		2, {0xE8, 0x53},
		3, {0xE9, 0x8F, 0xA8},
		2, {0xE8, 0x93},
		2, {0xEF, 0xB4}
	},
	{
		2, {0xE8, 0x54},
		3, {0xE9, 0x8F, 0xA5},
		2, {0xE5, 0x93},
		2, {0xEF, 0xB5}
	},
	{
		2, {0xE8, 0x55},
		3, {0xE9, 0x8F, 0x98},
		2, {0xD8, 0x93},
		2, {0xEF, 0xB6}
	},
	{
		2, {0xE8, 0x56},
		3, {0xE9, 0x8F, 0x83},
		2, {0xC3, 0x93},
		2, {0xEF, 0xB7}
	},
	{
		2, {0xE8, 0x57},
		3, {0xE9, 0x8F, 0x9D},
		2, {0xDD, 0x93},
		2, {0xEF, 0xB8}
	},
	{
		2, {0xE8, 0x58},
		3, {0xE9, 0x8F, 0x90},
		2, {0xD0, 0x93},
		2, {0xEF, 0xB9}
	},
	{
		2, {0xE8, 0x59},
		3, {0xE9, 0x8F, 0x88},
		2, {0xC8, 0x93},
		2, {0xEF, 0xBA}
	},
	{
		2, {0xE8, 0x5A},
		3, {0xE9, 0x8F, 0xA4},
		2, {0xE4, 0x93},
		2, {0xEF, 0xBB}
	},
	{
		2, {0xE8, 0x5B},
		3, {0xE9, 0x90, 0x9A},
		2, {0x1A, 0x94},
		2, {0xEF, 0xBC}
	},
	{
		2, {0xE8, 0x5C},
		3, {0xE9, 0x90, 0x94},
		2, {0x14, 0x94},
		2, {0xEF, 0xBD}
	},
	{
		2, {0xE8, 0x5D},
		3, {0xE9, 0x90, 0x93},
		2, {0x13, 0x94},
		2, {0xEF, 0xBE}
	},
	{
		2, {0xE8, 0x5E},
		3, {0xE9, 0x90, 0x83},
		2, {0x03, 0x94},
		2, {0xEF, 0xBF}
	},
	{
		2, {0xE8, 0x5F},
		3, {0xE9, 0x90, 0x87},
		2, {0x07, 0x94},
		2, {0xEF, 0xC0}
	},
	{
		2, {0xE8, 0x60},
		3, {0xE9, 0x90, 0x90},
		2, {0x10, 0x94},
		2, {0xEF, 0xC1}
	},
	{
		2, {0xE8, 0x61},
		3, {0xE9, 0x90, 0xB6},
		2, {0x36, 0x94},
		2, {0xEF, 0xC2}
	},
	{
		2, {0xE8, 0x62},
		3, {0xE9, 0x90, 0xAB},
		2, {0x2B, 0x94},
		2, {0xEF, 0xC3}
	},
	{
		2, {0xE8, 0x63},
		3, {0xE9, 0x90, 0xB5},
		2, {0x35, 0x94},
		2, {0xEF, 0xC4}
	},
	{
		2, {0xE8, 0x64},
		3, {0xE9, 0x90, 0xA1},
		2, {0x21, 0x94},
		2, {0xEF, 0xC5}
	},
	{
		2, {0xE8, 0x65},
		3, {0xE9, 0x90, 0xBA},
		2, {0x3A, 0x94},
		2, {0xEF, 0xC6}
	},
	{
		2, {0xE8, 0x66},
		3, {0xE9, 0x91, 0x81},
		2, {0x41, 0x94},
		2, {0xEF, 0xC7}
	},
	{
		2, {0xE8, 0x67},
		3, {0xE9, 0x91, 0x92},
		2, {0x52, 0x94},
		2, {0xEF, 0xC8}
	},
	{
		2, {0xE8, 0x68},
		3, {0xE9, 0x91, 0x84},
		2, {0x44, 0x94},
		2, {0xEF, 0xC9}
	},
	{
		2, {0xE8, 0x69},
		3, {0xE9, 0x91, 0x9B},
		2, {0x5B, 0x94},
		2, {0xEF, 0xCA}
	},
	{
		2, {0xE8, 0x6A},
		3, {0xE9, 0x91, 0xA0},
		2, {0x60, 0x94},
		2, {0xEF, 0xCB}
	},
	{
		2, {0xE8, 0x6B},
		3, {0xE9, 0x91, 0xA2},
		2, {0x62, 0x94},
		2, {0xEF, 0xCC}
	},
	{
		2, {0xE8, 0x6C},
		3, {0xE9, 0x91, 0x9E},
		2, {0x5E, 0x94},
		2, {0xEF, 0xCD}
	},
	{
		2, {0xE8, 0x6D},
		3, {0xE9, 0x91, 0xAA},
		2, {0x6A, 0x94},
		2, {0xEF, 0xCE}
	},
	{
		2, {0xE8, 0x6E},
		3, {0xE9, 0x88, 0xA9},
		2, {0x29, 0x92},
		2, {0xEF, 0xCF}
	},
	{
		2, {0xE8, 0x6F},
		3, {0xE9, 0x91, 0xB0},
		2, {0x70, 0x94},
		2, {0xEF, 0xD0}
	},
	{
		2, {0xE8, 0x70},
		3, {0xE9, 0x91, 0xB5},
		2, {0x75, 0x94},
		2, {0xEF, 0xD1}
	},
	{
		2, {0xE8, 0x71},
		3, {0xE9, 0x91, 0xB7},
		2, {0x77, 0x94},
		2, {0xEF, 0xD2}
	},
	{
		2, {0xE8, 0x72},
		3, {0xE9, 0x91, 0xBD},
		2, {0x7D, 0x94},
		2, {0xEF, 0xD3}
	},
	{
		2, {0xE8, 0x73},
		3, {0xE9, 0x91, 0x9A},
		2, {0x5A, 0x94},
		2, {0xEF, 0xD4}
	},
	{
		2, {0xE8, 0x74},
		3, {0xE9, 0x91, 0xBC},
		2, {0x7C, 0x94},
		2, {0xEF, 0xD5}
	},
	{
		2, {0xE8, 0x75},
		3, {0xE9, 0x91, 0xBE},
		2, {0x7E, 0x94},
		2, {0xEF, 0xD6}
	},
	{
		2, {0xE8, 0x76},
		3, {0xE9, 0x92, 0x81},
		2, {0x81, 0x94},
		2, {0xEF, 0xD7}
	},
	{
		2, {0xE8, 0x77},
		3, {0xE9, 0x91, 0xBF},
		2, {0x7F, 0x94},
		2, {0xEF, 0xD8}
	},
	{
		2, {0xE8, 0x78},
		3, {0xE9, 0x96, 0x82},
		2, {0x82, 0x95},
		2, {0xEF, 0xD9}
	},
	{
		2, {0xE8, 0x79},
		3, {0xE9, 0x96, 0x87},
		2, {0x87, 0x95},
		2, {0xEF, 0xDA}
	},
	{
		2, {0xE8, 0x7A},
		3, {0xE9, 0x96, 0x8A},
		2, {0x8A, 0x95},
		2, {0xEF, 0xDB}
	},
	{
		2, {0xE8, 0x7B},
		3, {0xE9, 0x96, 0x94},
		2, {0x94, 0x95},
		2, {0xEF, 0xDC}
	},
	{
		2, {0xE8, 0x7C},
		3, {0xE9, 0x96, 0x96},
		2, {0x96, 0x95},
		2, {0xEF, 0xDD}
	},
	{
		2, {0xE8, 0x7D},
		3, {0xE9, 0x96, 0x98},
		2, {0x98, 0x95},
		2, {0xEF, 0xDE}
	},
	{
		2, {0xE8, 0x7E},
		3, {0xE9, 0x96, 0x99},
		2, {0x99, 0x95},
		2, {0xEF, 0xDF}
	},
	{
		2, {0xE8, 0x80},
		3, {0xE9, 0x96, 0xA0},
		2, {0xA0, 0x95},
		2, {0xEF, 0xE0}
	},
	{
		2, {0xE8, 0x81},
		3, {0xE9, 0x96, 0xA8},
		2, {0xA8, 0x95},
		2, {0xEF, 0xE1}
	},
	{
		2, {0xE8, 0x82},
		3, {0xE9, 0x96, 0xA7},
		2, {0xA7, 0x95},
		2, {0xEF, 0xE2}
	},
	{
		2, {0xE8, 0x83},
		3, {0xE9, 0x96, 0xAD},
		2, {0xAD, 0x95},
		2, {0xEF, 0xE3}
	},
	{
		2, {0xE8, 0x84},
		3, {0xE9, 0x96, 0xBC},
		2, {0xBC, 0x95},
		2, {0xEF, 0xE4}
	},
	{
		2, {0xE8, 0x85},
		3, {0xE9, 0x96, 0xBB},
		2, {0xBB, 0x95},
		2, {0xEF, 0xE5}
	},
	{
		2, {0xE8, 0x86},
		3, {0xE9, 0x96, 0xB9},
		2, {0xB9, 0x95},
		2, {0xEF, 0xE6}
	},
	{
		2, {0xE8, 0x87},
		3, {0xE9, 0x96, 0xBE},
		2, {0xBE, 0x95},
		2, {0xEF, 0xE7}
	},
	{
		2, {0xE8, 0x88},
		3, {0xE9, 0x97, 0x8A},
		2, {0xCA, 0x95},
		2, {0xEF, 0xE8}
	},
	{
		2, {0xE8, 0x89},
		3, {0xE6, 0xBF, 0xB6},
		2, {0xF6, 0x6F},
		2, {0xEF, 0xE9}
	},
	{
		2, {0xE8, 0x8A},
		3, {0xE9, 0x97, 0x83},
		2, {0xC3, 0x95},
		2, {0xEF, 0xEA}
	},
	{
		2, {0xE8, 0x8B},
		3, {0xE9, 0x97, 0x8D},
		2, {0xCD, 0x95},
		2, {0xEF, 0xEB}
	},
	{
		2, {0xE8, 0x8C},
		3, {0xE9, 0x97, 0x8C},
		2, {0xCC, 0x95},
		2, {0xEF, 0xEC}
	},
	{
		2, {0xE8, 0x8D},
		3, {0xE9, 0x97, 0x95},
		2, {0xD5, 0x95},
		2, {0xEF, 0xED}
	},
	{
		2, {0xE8, 0x8E},
		3, {0xE9, 0x97, 0x94},
		2, {0xD4, 0x95},
		2, {0xEF, 0xEE}
	},
	{
		2, {0xE8, 0x8F},
		3, {0xE9, 0x97, 0x96},
		2, {0xD6, 0x95},
		2, {0xEF, 0xEF}
	},
	{
		2, {0xE8, 0x90},
		3, {0xE9, 0x97, 0x9C},
		2, {0xDC, 0x95},
		2, {0xEF, 0xF0}
	},
	{
		2, {0xE8, 0x91},
		3, {0xE9, 0x97, 0xA1},
		2, {0xE1, 0x95},
		2, {0xEF, 0xF1}
	},
	{
		2, {0xE8, 0x92},
		3, {0xE9, 0x97, 0xA5},
		2, {0xE5, 0x95},
		2, {0xEF, 0xF2}
	},
	{
		2, {0xE8, 0x93},
		3, {0xE9, 0x97, 0xA2},
		2, {0xE2, 0x95},
		2, {0xEF, 0xF3}
	},
	{
		2, {0xE8, 0x94},
		3, {0xE9, 0x98, 0xA1},
		2, {0x21, 0x96},
		2, {0xEF, 0xF4}
	},
	{
		2, {0xE8, 0x95},
		3, {0xE9, 0x98, 0xA8},
		2, {0x28, 0x96},
		2, {0xEF, 0xF5}
	},
	{
		2, {0xE8, 0x96},
		3, {0xE9, 0x98, 0xAE},
		2, {0x2E, 0x96},
		2, {0xEF, 0xF6}
	},
	{
		2, {0xE8, 0x97},
		3, {0xE9, 0x98, 0xAF},
		2, {0x2F, 0x96},
		2, {0xEF, 0xF7}
	},
	{
		2, {0xE8, 0x98},
		3, {0xE9, 0x99, 0x82},
		2, {0x42, 0x96},
		2, {0xEF, 0xF8}
	},
	{
		2, {0xE8, 0x99},
		3, {0xE9, 0x99, 0x8C},
		2, {0x4C, 0x96},
		2, {0xEF, 0xF9}
	},
	{
		2, {0xE8, 0x9A},
		3, {0xE9, 0x99, 0x8F},
		2, {0x4F, 0x96},
		2, {0xEF, 0xFA}
	},
	{
		2, {0xE8, 0x9B},
		3, {0xE9, 0x99, 0x8B},
		2, {0x4B, 0x96},
		2, {0xEF, 0xFB}
	},
	{
		2, {0xE8, 0x9C},
		3, {0xE9, 0x99, 0xB7},
		2, {0x77, 0x96},
		2, {0xEF, 0xFC}
	},
	{
		2, {0xE8, 0x9D},
		3, {0xE9, 0x99, 0x9C},
		2, {0x5C, 0x96},
		2, {0xEF, 0xFD}
	},
	{
		2, {0xE8, 0x9E},
		3, {0xE9, 0x99, 0x9E},
		2, {0x5E, 0x96},
		2, {0xEF, 0xFE}
	},
	{
		2, {0xE8, 0x9F},
		3, {0xE9, 0x99, 0x9D},
		2, {0x5D, 0x96},
		2, {0xF0, 0xA1}
	},
	{
		2, {0xE8, 0xA0},
		3, {0xE9, 0x99, 0x9F},
		2, {0x5F, 0x96},
		2, {0xF0, 0xA2}
	},
	{
		2, {0xE8, 0xA1},
		3, {0xE9, 0x99, 0xA6},
		2, {0x66, 0x96},
		2, {0xF0, 0xA3}
	},
	{
		2, {0xE8, 0xA2},
		3, {0xE9, 0x99, 0xB2},
		2, {0x72, 0x96},
		2, {0xF0, 0xA4}
	},
	{
		2, {0xE8, 0xA3},
		3, {0xE9, 0x99, 0xAC},
		2, {0x6C, 0x96},
		2, {0xF0, 0xA5}
	},
	{
		2, {0xE8, 0xA4},
		3, {0xE9, 0x9A, 0x8D},
		2, {0x8D, 0x96},
		2, {0xF0, 0xA6}
	},
	{
		2, {0xE8, 0xA5},
		3, {0xE9, 0x9A, 0x98},
		2, {0x98, 0x96},
		2, {0xF0, 0xA7}
	},
	{
		2, {0xE8, 0xA6},
		3, {0xE9, 0x9A, 0x95},
		2, {0x95, 0x96},
		2, {0xF0, 0xA8}
	},
	{
		2, {0xE8, 0xA7},
		3, {0xE9, 0x9A, 0x97},
		2, {0x97, 0x96},
		2, {0xF0, 0xA9}
	},
	{
		2, {0xE8, 0xA8},
		3, {0xE9, 0x9A, 0xAA},
		2, {0xAA, 0x96},
		2, {0xF0, 0xAA}
	},
	{
		2, {0xE8, 0xA9},
		3, {0xE9, 0x9A, 0xA7},
		2, {0xA7, 0x96},
		2, {0xF0, 0xAB}
	},
	{
		2, {0xE8, 0xAA},
		3, {0xE9, 0x9A, 0xB1},
		2, {0xB1, 0x96},
		2, {0xF0, 0xAC}
	},
	{
		2, {0xE8, 0xAB},
		3, {0xE9, 0x9A, 0xB2},
		2, {0xB2, 0x96},
		2, {0xF0, 0xAD}
	},
	{
		2, {0xE8, 0xAC},
		3, {0xE9, 0x9A, 0xB0},
		2, {0xB0, 0x96},
		2, {0xF0, 0xAE}
	},
	{
		2, {0xE8, 0xAD},
		3, {0xE9, 0x9A, 0xB4},
		2, {0xB4, 0x96},
		2, {0xF0, 0xAF}
	},
	{
		2, {0xE8, 0xAE},
		3, {0xE9, 0x9A, 0xB6},
		2, {0xB6, 0x96},
		2, {0xF0, 0xB0}
	},
	{
		2, {0xE8, 0xAF},
		3, {0xE9, 0x9A, 0xB8},
		2, {0xB8, 0x96},
		2, {0xF0, 0xB1}
	},
	{
		2, {0xE8, 0xB0},
		3, {0xE9, 0x9A, 0xB9},
		2, {0xB9, 0x96},
		2, {0xF0, 0xB2}
	},
	{
		2, {0xE8, 0xB1},
		3, {0xE9, 0x9B, 0x8E},
		2, {0xCE, 0x96},
		2, {0xF0, 0xB3}
	},
	{
		2, {0xE8, 0xB2},
		3, {0xE9, 0x9B, 0x8B},
		2, {0xCB, 0x96},
		2, {0xF0, 0xB4}
	},
	{
		2, {0xE8, 0xB3},
		3, {0xE9, 0x9B, 0x89},
		2, {0xC9, 0x96},
		2, {0xF0, 0xB5}
	},
	{
		2, {0xE8, 0xB4},
		3, {0xE9, 0x9B, 0x8D},
		2, {0xCD, 0x96},
		2, {0xF0, 0xB6}
	},
	{
		2, {0xE8, 0xB5},
		3, {0xE8, 0xA5, 0x8D},
		2, {0x4D, 0x89},
		2, {0xF0, 0xB7}
	},
	{
		2, {0xE8, 0xB6},
		3, {0xE9, 0x9B, 0x9C},
		2, {0xDC, 0x96},
		2, {0xF0, 0xB8}
	},
	{
		2, {0xE8, 0xB7},
		3, {0xE9, 0x9C, 0x8D},
		2, {0x0D, 0x97},
		2, {0xF0, 0xB9}
	},
	{
		2, {0xE8, 0xB8},
		3, {0xE9, 0x9B, 0x95},
		2, {0xD5, 0x96},
		2, {0xF0, 0xBA}
	},
	{
		2, {0xE8, 0xB9},
		3, {0xE9, 0x9B, 0xB9},
		2, {0xF9, 0x96},
		2, {0xF0, 0xBB}
	},
	{
		2, {0xE8, 0xBA},
		3, {0xE9, 0x9C, 0x84},
		2, {0x04, 0x97},
		2, {0xF0, 0xBC}
	},
	{
		2, {0xE8, 0xBB},
		3, {0xE9, 0x9C, 0x86},
		2, {0x06, 0x97},
		2, {0xF0, 0xBD}
	},
	{
		2, {0xE8, 0xBC},
		3, {0xE9, 0x9C, 0x88},
		2, {0x08, 0x97},
		2, {0xF0, 0xBE}
	},
	{
		2, {0xE8, 0xBD},
		3, {0xE9, 0x9C, 0x93},
		2, {0x13, 0x97},
		2, {0xF0, 0xBF}
	},
	{
		2, {0xE8, 0xBE},
		3, {0xE9, 0x9C, 0x8E},
		2, {0x0E, 0x97},
		2, {0xF0, 0xC0}
	},
	{
		2, {0xE8, 0xBF},
		3, {0xE9, 0x9C, 0x91},
		2, {0x11, 0x97},
		2, {0xF0, 0xC1}
	},
	{
		2, {0xE8, 0xC0},
		3, {0xE9, 0x9C, 0x8F},
		2, {0x0F, 0x97},
		2, {0xF0, 0xC2}
	},
	{
		2, {0xE8, 0xC1},
		3, {0xE9, 0x9C, 0x96},
		2, {0x16, 0x97},
		2, {0xF0, 0xC3}
	},
	{
		2, {0xE8, 0xC2},
		3, {0xE9, 0x9C, 0x99},
		2, {0x19, 0x97},
		2, {0xF0, 0xC4}
	},
	{
		2, {0xE8, 0xC3},
		3, {0xE9, 0x9C, 0xA4},
		2, {0x24, 0x97},
		2, {0xF0, 0xC5}
	},
	{
		2, {0xE8, 0xC4},
		3, {0xE9, 0x9C, 0xAA},
		2, {0x2A, 0x97},
		2, {0xF0, 0xC6}
	},
	{
		2, {0xE8, 0xC5},
		3, {0xE9, 0x9C, 0xB0},
		2, {0x30, 0x97},
		2, {0xF0, 0xC7}
	},
	{
		2, {0xE8, 0xC6},
		3, {0xE9, 0x9C, 0xB9},
		2, {0x39, 0x97},
		2, {0xF0, 0xC8}
	},
	{
		2, {0xE8, 0xC7},
		3, {0xE9, 0x9C, 0xBD},
		2, {0x3D, 0x97},
		2, {0xF0, 0xC9}
	},
	{
		2, {0xE8, 0xC8},
		3, {0xE9, 0x9C, 0xBE},
		2, {0x3E, 0x97},
		2, {0xF0, 0xCA}
	},
	{
		2, {0xE8, 0xC9},
		3, {0xE9, 0x9D, 0x84},
		2, {0x44, 0x97},
		2, {0xF0, 0xCB}
	},
	{
		2, {0xE8, 0xCA},
		3, {0xE9, 0x9D, 0x86},
		2, {0x46, 0x97},
		2, {0xF0, 0xCC}
	},
	{
		2, {0xE8, 0xCB},
		3, {0xE9, 0x9D, 0x88},
		2, {0x48, 0x97},
		2, {0xF0, 0xCD}
	},
	{
		2, {0xE8, 0xCC},
		3, {0xE9, 0x9D, 0x82},
		2, {0x42, 0x97},
		2, {0xF0, 0xCE}
	},
	{
		2, {0xE8, 0xCD},
		3, {0xE9, 0x9D, 0x89},
		2, {0x49, 0x97},
		2, {0xF0, 0xCF}
	},
	{
		2, {0xE8, 0xCE},
		3, {0xE9, 0x9D, 0x9C},
		2, {0x5C, 0x97},
		2, {0xF0, 0xD0}
	},
	{
		2, {0xE8, 0xCF},
		3, {0xE9, 0x9D, 0xA0},
		2, {0x60, 0x97},
		2, {0xF0, 0xD1}
	},
	{
		2, {0xE8, 0xD0},
		3, {0xE9, 0x9D, 0xA4},
		2, {0x64, 0x97},
		2, {0xF0, 0xD2}
	},
	{
		2, {0xE8, 0xD1},
		3, {0xE9, 0x9D, 0xA6},
		2, {0x66, 0x97},
		2, {0xF0, 0xD3}
	},
	{
		2, {0xE8, 0xD2},
		3, {0xE9, 0x9D, 0xA8},
		2, {0x68, 0x97},
		2, {0xF0, 0xD4}
	},
	{
		2, {0xE8, 0xD3},
		3, {0xE5, 0x8B, 0x92},
		2, {0xD2, 0x52},
		2, {0xF0, 0xD5}
	},
	{
		2, {0xE8, 0xD4},
		3, {0xE9, 0x9D, 0xAB},
		2, {0x6B, 0x97},
		2, {0xF0, 0xD6}
	},
	{
		2, {0xE8, 0xD5},
		3, {0xE9, 0x9D, 0xB1},
		2, {0x71, 0x97},
		2, {0xF0, 0xD7}
	},
	{
		2, {0xE8, 0xD6},
		3, {0xE9, 0x9D, 0xB9},
		2, {0x79, 0x97},
		2, {0xF0, 0xD8}
	},
	{
		2, {0xE8, 0xD7},
		3, {0xE9, 0x9E, 0x85},
		2, {0x85, 0x97},
		2, {0xF0, 0xD9}
	},
	{
		2, {0xE8, 0xD8},
		3, {0xE9, 0x9D, 0xBC},
		2, {0x7C, 0x97},
		2, {0xF0, 0xDA}
	},
	{
		2, {0xE8, 0xD9},
		3, {0xE9, 0x9E, 0x81},
		2, {0x81, 0x97},
		2, {0xF0, 0xDB}
	},
	{
		2, {0xE8, 0xDA},
		3, {0xE9, 0x9D, 0xBA},
		2, {0x7A, 0x97},
		2, {0xF0, 0xDC}
	},
	{
		2, {0xE8, 0xDB},
		3, {0xE9, 0x9E, 0x86},
		2, {0x86, 0x97},
		2, {0xF0, 0xDD}
	},
	{
		2, {0xE8, 0xDC},
		3, {0xE9, 0x9E, 0x8B},
		2, {0x8B, 0x97},
		2, {0xF0, 0xDE}
	},
	{
		2, {0xE8, 0xDD},
		3, {0xE9, 0x9E, 0x8F},
		2, {0x8F, 0x97},
		2, {0xF0, 0xDF}
	},
	{
		2, {0xE8, 0xDE},
		3, {0xE9, 0x9E, 0x90},
		2, {0x90, 0x97},
		2, {0xF0, 0xE0}
	},
	{
		2, {0xE8, 0xDF},
		3, {0xE9, 0x9E, 0x9C},
		2, {0x9C, 0x97},
		2, {0xF0, 0xE1}
	},
	{
		2, {0xE8, 0xE0},
		3, {0xE9, 0x9E, 0xA8},
		2, {0xA8, 0x97},
		2, {0xF0, 0xE2}
	},
	{
		2, {0xE8, 0xE1},
		3, {0xE9, 0x9E, 0xA6},
		2, {0xA6, 0x97},
		2, {0xF0, 0xE3}
	},
	{
		2, {0xE8, 0xE2},
		3, {0xE9, 0x9E, 0xA3},
		2, {0xA3, 0x97},
		2, {0xF0, 0xE4}
	},
	{
		2, {0xE8, 0xE3},
		3, {0xE9, 0x9E, 0xB3},
		2, {0xB3, 0x97},
		2, {0xF0, 0xE5}
	},
	{
		2, {0xE8, 0xE4},
		3, {0xE9, 0x9E, 0xB4},
		2, {0xB4, 0x97},
		2, {0xF0, 0xE6}
	},
	{
		2, {0xE8, 0xE5},
		3, {0xE9, 0x9F, 0x83},
		2, {0xC3, 0x97},
		2, {0xF0, 0xE7}
	},
	{
		2, {0xE8, 0xE6},
		3, {0xE9, 0x9F, 0x86},
		2, {0xC6, 0x97},
		2, {0xF0, 0xE8}
	},
	{
		2, {0xE8, 0xE7},
		3, {0xE9, 0x9F, 0x88},
		2, {0xC8, 0x97},
		2, {0xF0, 0xE9}
	},
	{
		2, {0xE8, 0xE8},
		3, {0xE9, 0x9F, 0x8B},
		2, {0xCB, 0x97},
		2, {0xF0, 0xEA}
	},
	{
		2, {0xE8, 0xE9},
		3, {0xE9, 0x9F, 0x9C},
		2, {0xDC, 0x97},
		2, {0xF0, 0xEB}
	},
	{
		2, {0xE8, 0xEA},
		3, {0xE9, 0x9F, 0xAD},
		2, {0xED, 0x97},
		2, {0xF0, 0xEC}
	},
	{
		2, {0xE8, 0xEB},
		3, {0xE9, 0xBD, 0x8F},
		2, {0x4F, 0x9F},
		2, {0xF0, 0xED}
	},
	{
		2, {0xE8, 0xEC},
		3, {0xE9, 0x9F, 0xB2},
		2, {0xF2, 0x97},
		2, {0xF0, 0xEE}
	},
	{
		2, {0xE8, 0xED},
		3, {0xE7, 0xAB, 0x9F},
		2, {0xDF, 0x7A},
		2, {0xF0, 0xEF}
	},
	{
		2, {0xE8, 0xEE},
		3, {0xE9, 0x9F, 0xB6},
		2, {0xF6, 0x97},
		2, {0xF0, 0xF0}
	},
	{
		2, {0xE8, 0xEF},
		3, {0xE9, 0x9F, 0xB5},
		2, {0xF5, 0x97},
		2, {0xF0, 0xF1}
	},
	{
		2, {0xE8, 0xF0},
		3, {0xE9, 0xA0, 0x8F},
		2, {0x0F, 0x98},
		2, {0xF0, 0xF2}
	},
	{
		2, {0xE8, 0xF1},
		3, {0xE9, 0xA0, 0x8C},
		2, {0x0C, 0x98},
		2, {0xF0, 0xF3}
	},
	{
		2, {0xE8, 0xF2},
		3, {0xE9, 0xA0, 0xB8},
		2, {0x38, 0x98},
		2, {0xF0, 0xF4}
	},
	{
		2, {0xE8, 0xF3},
		3, {0xE9, 0xA0, 0xA4},
		2, {0x24, 0x98},
		2, {0xF0, 0xF5}
	},
	{
		2, {0xE8, 0xF4},
		3, {0xE9, 0xA0, 0xA1},
		2, {0x21, 0x98},
		2, {0xF0, 0xF6}
	},
	{
		2, {0xE8, 0xF5},
		3, {0xE9, 0xA0, 0xB7},
		2, {0x37, 0x98},
		2, {0xF0, 0xF7}
	},
	{
		2, {0xE8, 0xF6},
		3, {0xE9, 0xA0, 0xBD},
		2, {0x3D, 0x98},
		2, {0xF0, 0xF8}
	},
	{
		2, {0xE8, 0xF7},
		3, {0xE9, 0xA1, 0x86},
		2, {0x46, 0x98},
		2, {0xF0, 0xF9}
	},
	{
		2, {0xE8, 0xF8},
		3, {0xE9, 0xA1, 0x8F},
		2, {0x4F, 0x98},
		2, {0xF0, 0xFA}
	},
	{
		2, {0xE8, 0xF9},
		3, {0xE9, 0xA1, 0x8B},
		2, {0x4B, 0x98},
		2, {0xF0, 0xFB}
	},
	{
		2, {0xE8, 0xFA},
		3, {0xE9, 0xA1, 0xAB},
		2, {0x6B, 0x98},
		2, {0xF0, 0xFC}
	},
	{
		2, {0xE8, 0xFB},
		3, {0xE9, 0xA1, 0xAF},
		2, {0x6F, 0x98},
		2, {0xF0, 0xFD}
	},
	{
		2, {0xE8, 0xFC},
		3, {0xE9, 0xA1, 0xB0},
		2, {0x70, 0x98},
		2, {0xF0, 0xFE}
	},
	{
		2, {0xE9, 0x40},
		3, {0xE9, 0xA1, 0xB1},
		2, {0x71, 0x98},
		2, {0xF1, 0xA1}
	},
	{
		2, {0xE9, 0x41},
		3, {0xE9, 0xA1, 0xB4},
		2, {0x74, 0x98},
		2, {0xF1, 0xA2}
	},
	{
		2, {0xE9, 0x42},
		3, {0xE9, 0xA1, 0xB3},
		2, {0x73, 0x98},
		2, {0xF1, 0xA3}
	},
	{
		2, {0xE9, 0x43},
		3, {0xE9, 0xA2, 0xAA},
		2, {0xAA, 0x98},
		2, {0xF1, 0xA4}
	},
	{
		2, {0xE9, 0x44},
		3, {0xE9, 0xA2, 0xAF},
		2, {0xAF, 0x98},
		2, {0xF1, 0xA5}
	},
	{
		2, {0xE9, 0x45},
		3, {0xE9, 0xA2, 0xB1},
		2, {0xB1, 0x98},
		2, {0xF1, 0xA6}
	},
	{
		2, {0xE9, 0x46},
		3, {0xE9, 0xA2, 0xB6},
		2, {0xB6, 0x98},
		2, {0xF1, 0xA7}
	},
	{
		2, {0xE9, 0x47},
		3, {0xE9, 0xA3, 0x84},
		2, {0xC4, 0x98},
		2, {0xF1, 0xA8}
	},
	{
		2, {0xE9, 0x48},
		3, {0xE9, 0xA3, 0x83},
		2, {0xC3, 0x98},
		2, {0xF1, 0xA9}
	},
	{
		2, {0xE9, 0x49},
		3, {0xE9, 0xA3, 0x86},
		2, {0xC6, 0x98},
		2, {0xF1, 0xAA}
	},
	{
		2, {0xE9, 0x4A},
		3, {0xE9, 0xA3, 0xA9},
		2, {0xE9, 0x98},
		2, {0xF1, 0xAB}
	},
	{
		2, {0xE9, 0x4B},
		3, {0xE9, 0xA3, 0xAB},
		2, {0xEB, 0x98},
		2, {0xF1, 0xAC}
	},
	{
		2, {0xE9, 0x4C},
		3, {0xE9, 0xA4, 0x83},
		2, {0x03, 0x99},
		2, {0xF1, 0xAD}
	},
	{
		2, {0xE9, 0x4D},
		3, {0xE9, 0xA4, 0x89},
		2, {0x09, 0x99},
		2, {0xF1, 0xAE}
	},
	{
		2, {0xE9, 0x4E},
		3, {0xE9, 0xA4, 0x92},
		2, {0x12, 0x99},
		2, {0xF1, 0xAF}
	},
	{
		2, {0xE9, 0x4F},
		3, {0xE9, 0xA4, 0x94},
		2, {0x14, 0x99},
		2, {0xF1, 0xB0}
	},
	{
		2, {0xE9, 0x50},
		3, {0xE9, 0xA4, 0x98},
		2, {0x18, 0x99},
		2, {0xF1, 0xB1}
	},
	{
		2, {0xE9, 0x51},
		3, {0xE9, 0xA4, 0xA1},
		2, {0x21, 0x99},
		2, {0xF1, 0xB2}
	},
	{
		2, {0xE9, 0x52},
		3, {0xE9, 0xA4, 0x9D},
		2, {0x1D, 0x99},
		2, {0xF1, 0xB3}
	},
	{
		2, {0xE9, 0x53},
		3, {0xE9, 0xA4, 0x9E},
		2, {0x1E, 0x99},
		2, {0xF1, 0xB4}
	},
	{
		2, {0xE9, 0x54},
		3, {0xE9, 0xA4, 0xA4},
		2, {0x24, 0x99},
		2, {0xF1, 0xB5}
	},
	{
		2, {0xE9, 0x55},
		3, {0xE9, 0xA4, 0xA0},
		2, {0x20, 0x99},
		2, {0xF1, 0xB6}
	},
	{
		2, {0xE9, 0x56},
		3, {0xE9, 0xA4, 0xAC},
		2, {0x2C, 0x99},
		2, {0xF1, 0xB7}
	},
	{
		2, {0xE9, 0x57},
		3, {0xE9, 0xA4, 0xAE},
		2, {0x2E, 0x99},
		2, {0xF1, 0xB8}
	},
	{
		2, {0xE9, 0x58},
		3, {0xE9, 0xA4, 0xBD},
		2, {0x3D, 0x99},
		2, {0xF1, 0xB9}
	},
	{
		2, {0xE9, 0x59},
		3, {0xE9, 0xA4, 0xBE},
		2, {0x3E, 0x99},
		2, {0xF1, 0xBA}
	},
	{
		2, {0xE9, 0x5A},
		3, {0xE9, 0xA5, 0x82},
		2, {0x42, 0x99},
		2, {0xF1, 0xBB}
	},
	{
		2, {0xE9, 0x5B},
		3, {0xE9, 0xA5, 0x89},
		2, {0x49, 0x99},
		2, {0xF1, 0xBC}
	},
	{
		2, {0xE9, 0x5C},
		3, {0xE9, 0xA5, 0x85},
		2, {0x45, 0x99},
		2, {0xF1, 0xBD}
	},
	{
		2, {0xE9, 0x5D},
		3, {0xE9, 0xA5, 0x90},
		2, {0x50, 0x99},
		2, {0xF1, 0xBE}
	},
	{
		2, {0xE9, 0x5E},
		3, {0xE9, 0xA5, 0x8B},
		2, {0x4B, 0x99},
		2, {0xF1, 0xBF}
	},
	{
		2, {0xE9, 0x5F},
		3, {0xE9, 0xA5, 0x91},
		2, {0x51, 0x99},
		2, {0xF1, 0xC0}
	},
	{
		2, {0xE9, 0x60},
		3, {0xE9, 0xA5, 0x92},
		2, {0x52, 0x99},
		2, {0xF1, 0xC1}
	},
	{
		2, {0xE9, 0x61},
		3, {0xE9, 0xA5, 0x8C},
		2, {0x4C, 0x99},
		2, {0xF1, 0xC2}
	},
	{
		2, {0xE9, 0x62},
		3, {0xE9, 0xA5, 0x95},
		2, {0x55, 0x99},
		2, {0xF1, 0xC3}
	},
	{
		2, {0xE9, 0x63},
		3, {0xE9, 0xA6, 0x97},
		2, {0x97, 0x99},
		2, {0xF1, 0xC4}
	},
	{
		2, {0xE9, 0x64},
		3, {0xE9, 0xA6, 0x98},
		2, {0x98, 0x99},
		2, {0xF1, 0xC5}
	},
	{
		2, {0xE9, 0x65},
		3, {0xE9, 0xA6, 0xA5},
		2, {0xA5, 0x99},
		2, {0xF1, 0xC6}
	},
	{
		2, {0xE9, 0x66},
		3, {0xE9, 0xA6, 0xAD},
		2, {0xAD, 0x99},
		2, {0xF1, 0xC7}
	},
	{
		2, {0xE9, 0x67},
		3, {0xE9, 0xA6, 0xAE},
		2, {0xAE, 0x99},
		2, {0xF1, 0xC8}
	},
	{
		2, {0xE9, 0x68},
		3, {0xE9, 0xA6, 0xBC},
		2, {0xBC, 0x99},
		2, {0xF1, 0xC9}
	},
	{
		2, {0xE9, 0x69},
		3, {0xE9, 0xA7, 0x9F},
		2, {0xDF, 0x99},
		2, {0xF1, 0xCA}
	},
	{
		2, {0xE9, 0x6A},
		3, {0xE9, 0xA7, 0x9B},
		2, {0xDB, 0x99},
		2, {0xF1, 0xCB}
	},
	{
		2, {0xE9, 0x6B},
		3, {0xE9, 0xA7, 0x9D},
		2, {0xDD, 0x99},
		2, {0xF1, 0xCC}
	},
	{
		2, {0xE9, 0x6C},
		3, {0xE9, 0xA7, 0x98},
		2, {0xD8, 0x99},
		2, {0xF1, 0xCD}
	},
	{
		2, {0xE9, 0x6D},
		3, {0xE9, 0xA7, 0x91},
		2, {0xD1, 0x99},
		2, {0xF1, 0xCE}
	},
	{
		2, {0xE9, 0x6E},
		3, {0xE9, 0xA7, 0xAD},
		2, {0xED, 0x99},
		2, {0xF1, 0xCF}
	},
	{
		2, {0xE9, 0x6F},
		3, {0xE9, 0xA7, 0xAE},
		2, {0xEE, 0x99},
		2, {0xF1, 0xD0}
	},
	{
		2, {0xE9, 0x70},
		3, {0xE9, 0xA7, 0xB1},
		2, {0xF1, 0x99},
		2, {0xF1, 0xD1}
	},
	{
		2, {0xE9, 0x71},
		3, {0xE9, 0xA7, 0xB2},
		2, {0xF2, 0x99},
		2, {0xF1, 0xD2}
	},
	{
		2, {0xE9, 0x72},
		3, {0xE9, 0xA7, 0xBB},
		2, {0xFB, 0x99},
		2, {0xF1, 0xD3}
	},
	{
		2, {0xE9, 0x73},
		3, {0xE9, 0xA7, 0xB8},
		2, {0xF8, 0x99},
		2, {0xF1, 0xD4}
	},
	{
		2, {0xE9, 0x74},
		3, {0xE9, 0xA8, 0x81},
		2, {0x01, 0x9A},
		2, {0xF1, 0xD5}
	},
	{
		2, {0xE9, 0x75},
		3, {0xE9, 0xA8, 0x8F},
		2, {0x0F, 0x9A},
		2, {0xF1, 0xD6}
	},
	{
		2, {0xE9, 0x76},
		3, {0xE9, 0xA8, 0x85},
		2, {0x05, 0x9A},
		2, {0xF1, 0xD7}
	},
	{
		2, {0xE9, 0x77},
		3, {0xE9, 0xA7, 0xA2},
		2, {0xE2, 0x99},
		2, {0xF1, 0xD8}
	},
	{
		2, {0xE9, 0x78},
		3, {0xE9, 0xA8, 0x99},
		2, {0x19, 0x9A},
		2, {0xF1, 0xD9}
	},
	{
		2, {0xE9, 0x79},
		3, {0xE9, 0xA8, 0xAB},
		2, {0x2B, 0x9A},
		2, {0xF1, 0xDA}
	},
	{
		2, {0xE9, 0x7A},
		3, {0xE9, 0xA8, 0xB7},
		2, {0x37, 0x9A},
		2, {0xF1, 0xDB}
	},
	{
		2, {0xE9, 0x7B},
		3, {0xE9, 0xA9, 0x85},
		2, {0x45, 0x9A},
		2, {0xF1, 0xDC}
	},
	{
		2, {0xE9, 0x7C},
		3, {0xE9, 0xA9, 0x82},
		2, {0x42, 0x9A},
		2, {0xF1, 0xDD}
	},
	{
		2, {0xE9, 0x7D},
		3, {0xE9, 0xA9, 0x80},
		2, {0x40, 0x9A},
		2, {0xF1, 0xDE}
	},
	{
		2, {0xE9, 0x7E},
		3, {0xE9, 0xA9, 0x83},
		2, {0x43, 0x9A},
		2, {0xF1, 0xDF}
	},
	{
		2, {0xE9, 0x80},
		3, {0xE9, 0xA8, 0xBE},
		2, {0x3E, 0x9A},
		2, {0xF1, 0xE0}
	},
	{
		2, {0xE9, 0x81},
		3, {0xE9, 0xA9, 0x95},
		2, {0x55, 0x9A},
		2, {0xF1, 0xE1}
	},
	{
		2, {0xE9, 0x82},
		3, {0xE9, 0xA9, 0x8D},
		2, {0x4D, 0x9A},
		2, {0xF1, 0xE2}
	},
	{
		2, {0xE9, 0x83},
		3, {0xE9, 0xA9, 0x9B},
		2, {0x5B, 0x9A},
		2, {0xF1, 0xE3}
	},
	{
		2, {0xE9, 0x84},
		3, {0xE9, 0xA9, 0x97},
		2, {0x57, 0x9A},
		2, {0xF1, 0xE4}
	},
	{
		2, {0xE9, 0x85},
		3, {0xE9, 0xA9, 0x9F},
		2, {0x5F, 0x9A},
		2, {0xF1, 0xE5}
	},
	{
		2, {0xE9, 0x86},
		3, {0xE9, 0xA9, 0xA2},
		2, {0x62, 0x9A},
		2, {0xF1, 0xE6}
	},
	{
		2, {0xE9, 0x87},
		3, {0xE9, 0xA9, 0xA5},
		2, {0x65, 0x9A},
		2, {0xF1, 0xE7}
	},
	{
		2, {0xE9, 0x88},
		3, {0xE9, 0xA9, 0xA4},
		2, {0x64, 0x9A},
		2, {0xF1, 0xE8}
	},
	{
		2, {0xE9, 0x89},
		3, {0xE9, 0xA9, 0xA9},
		2, {0x69, 0x9A},
		2, {0xF1, 0xE9}
	},
	{
		2, {0xE9, 0x8A},
		3, {0xE9, 0xA9, 0xAB},
		2, {0x6B, 0x9A},
		2, {0xF1, 0xEA}
	},
	{
		2, {0xE9, 0x8B},
		3, {0xE9, 0xA9, 0xAA},
		2, {0x6A, 0x9A},
		2, {0xF1, 0xEB}
	},
	{
		2, {0xE9, 0x8C},
		3, {0xE9, 0xAA, 0xAD},
		2, {0xAD, 0x9A},
		2, {0xF1, 0xEC}
	},
	{
		2, {0xE9, 0x8D},
		3, {0xE9, 0xAA, 0xB0},
		2, {0xB0, 0x9A},
		2, {0xF1, 0xED}
	},
	{
		2, {0xE9, 0x8E},
		3, {0xE9, 0xAA, 0xBC},
		2, {0xBC, 0x9A},
		2, {0xF1, 0xEE}
	},
	{
		2, {0xE9, 0x8F},
		3, {0xE9, 0xAB, 0x80},
		2, {0xC0, 0x9A},
		2, {0xF1, 0xEF}
	},
	{
		2, {0xE9, 0x90},
		3, {0xE9, 0xAB, 0x8F},
		2, {0xCF, 0x9A},
		2, {0xF1, 0xF0}
	},
	{
		2, {0xE9, 0x91},
		3, {0xE9, 0xAB, 0x91},
		2, {0xD1, 0x9A},
		2, {0xF1, 0xF1}
	},
	{
		2, {0xE9, 0x92},
		3, {0xE9, 0xAB, 0x93},
		2, {0xD3, 0x9A},
		2, {0xF1, 0xF2}
	},
	{
		2, {0xE9, 0x93},
		3, {0xE9, 0xAB, 0x94},
		2, {0xD4, 0x9A},
		2, {0xF1, 0xF3}
	},
	{
		2, {0xE9, 0x94},
		3, {0xE9, 0xAB, 0x9E},
		2, {0xDE, 0x9A},
		2, {0xF1, 0xF4}
	},
	{
		2, {0xE9, 0x95},
		3, {0xE9, 0xAB, 0x9F},
		2, {0xDF, 0x9A},
		2, {0xF1, 0xF5}
	},
	{
		2, {0xE9, 0x96},
		3, {0xE9, 0xAB, 0xA2},
		2, {0xE2, 0x9A},
		2, {0xF1, 0xF6}
	},
	{
		2, {0xE9, 0x97},
		3, {0xE9, 0xAB, 0xA3},
		2, {0xE3, 0x9A},
		2, {0xF1, 0xF7}
	},
	{
		2, {0xE9, 0x98},
		3, {0xE9, 0xAB, 0xA6},
		2, {0xE6, 0x9A},
		2, {0xF1, 0xF8}
	},
	{
		2, {0xE9, 0x99},
		3, {0xE9, 0xAB, 0xAF},
		2, {0xEF, 0x9A},
		2, {0xF1, 0xF9}
	},
	{
		2, {0xE9, 0x9A},
		3, {0xE9, 0xAB, 0xAB},
		2, {0xEB, 0x9A},
		2, {0xF1, 0xFA}
	},
	{
		2, {0xE9, 0x9B},
		3, {0xE9, 0xAB, 0xAE},
		2, {0xEE, 0x9A},
		2, {0xF1, 0xFB}
	},
	{
		2, {0xE9, 0x9C},
		3, {0xE9, 0xAB, 0xB4},
		2, {0xF4, 0x9A},
		2, {0xF1, 0xFC}
	},
	{
		2, {0xE9, 0x9D},
		3, {0xE9, 0xAB, 0xB1},
		2, {0xF1, 0x9A},
		2, {0xF1, 0xFD}
	},
	{
		2, {0xE9, 0x9E},
		3, {0xE9, 0xAB, 0xB7},
		2, {0xF7, 0x9A},
		2, {0xF1, 0xFE}
	},
	{
		2, {0xE9, 0x9F},
		3, {0xE9, 0xAB, 0xBB},
		2, {0xFB, 0x9A},
		2, {0xF2, 0xA1}
	},
	{
		2, {0xE9, 0xA0},
		3, {0xE9, 0xAC, 0x86},
		2, {0x06, 0x9B},
		2, {0xF2, 0xA2}
	},
	{
		2, {0xE9, 0xA1},
		3, {0xE9, 0xAC, 0x98},
		2, {0x18, 0x9B},
		2, {0xF2, 0xA3}
	},
	{
		2, {0xE9, 0xA2},
		3, {0xE9, 0xAC, 0x9A},
		2, {0x1A, 0x9B},
		2, {0xF2, 0xA4}
	},
	{
		2, {0xE9, 0xA3},
		3, {0xE9, 0xAC, 0x9F},
		2, {0x1F, 0x9B},
		2, {0xF2, 0xA5}
	},
	{
		2, {0xE9, 0xA4},
		3, {0xE9, 0xAC, 0xA2},
		2, {0x22, 0x9B},
		2, {0xF2, 0xA6}
	},
	{
		2, {0xE9, 0xA5},
		3, {0xE9, 0xAC, 0xA3},
		2, {0x23, 0x9B},
		2, {0xF2, 0xA7}
	},
	{
		2, {0xE9, 0xA6},
		3, {0xE9, 0xAC, 0xA5},
		2, {0x25, 0x9B},
		2, {0xF2, 0xA8}
	},
	{
		2, {0xE9, 0xA7},
		3, {0xE9, 0xAC, 0xA7},
		2, {0x27, 0x9B},
		2, {0xF2, 0xA9}
	},
	{
		2, {0xE9, 0xA8},
		3, {0xE9, 0xAC, 0xA8},
		2, {0x28, 0x9B},
		2, {0xF2, 0xAA}
	},
	{
		2, {0xE9, 0xA9},
		3, {0xE9, 0xAC, 0xA9},
		2, {0x29, 0x9B},
		2, {0xF2, 0xAB}
	},
	{
		2, {0xE9, 0xAA},
		3, {0xE9, 0xAC, 0xAA},
		2, {0x2A, 0x9B},
		2, {0xF2, 0xAC}
	},
	{
		2, {0xE9, 0xAB},
		3, {0xE9, 0xAC, 0xAE},
		2, {0x2E, 0x9B},
		2, {0xF2, 0xAD}
	},
	{
		2, {0xE9, 0xAC},
		3, {0xE9, 0xAC, 0xAF},
		2, {0x2F, 0x9B},
		2, {0xF2, 0xAE}
	},
	{
		2, {0xE9, 0xAD},
		3, {0xE9, 0xAC, 0xB2},
		2, {0x32, 0x9B},
		2, {0xF2, 0xAF}
	},
	{
		2, {0xE9, 0xAE},
		3, {0xE9, 0xAD, 0x84},
		2, {0x44, 0x9B},
		2, {0xF2, 0xB0}
	},
	{
		2, {0xE9, 0xAF},
		3, {0xE9, 0xAD, 0x83},
		2, {0x43, 0x9B},
		2, {0xF2, 0xB1}
	},
	{
		2, {0xE9, 0xB0},
		3, {0xE9, 0xAD, 0x8F},
		2, {0x4F, 0x9B},
		2, {0xF2, 0xB2}
	},
	{
		2, {0xE9, 0xB1},
		3, {0xE9, 0xAD, 0x8D},
		2, {0x4D, 0x9B},
		2, {0xF2, 0xB3}
	},
	{
		2, {0xE9, 0xB2},
		3, {0xE9, 0xAD, 0x8E},
		2, {0x4E, 0x9B},
		2, {0xF2, 0xB4}
	},
	{
		2, {0xE9, 0xB3},
		3, {0xE9, 0xAD, 0x91},
		2, {0x51, 0x9B},
		2, {0xF2, 0xB5}
	},
	{
		2, {0xE9, 0xB4},
		3, {0xE9, 0xAD, 0x98},
		2, {0x58, 0x9B},
		2, {0xF2, 0xB6}
	},
	{
		2, {0xE9, 0xB5},
		3, {0xE9, 0xAD, 0xB4},
		2, {0x74, 0x9B},
		2, {0xF2, 0xB7}
	},
	{
		2, {0xE9, 0xB6},
		3, {0xE9, 0xAE, 0x93},
		2, {0x93, 0x9B},
		2, {0xF2, 0xB8}
	},
	{
		2, {0xE9, 0xB7},
		3, {0xE9, 0xAE, 0x83},
		2, {0x83, 0x9B},
		2, {0xF2, 0xB9}
	},
	{
		2, {0xE9, 0xB8},
		3, {0xE9, 0xAE, 0x91},
		2, {0x91, 0x9B},
		2, {0xF2, 0xBA}
	},
	{
		2, {0xE9, 0xB9},
		3, {0xE9, 0xAE, 0x96},
		2, {0x96, 0x9B},
		2, {0xF2, 0xBB}
	},
	{
		2, {0xE9, 0xBA},
		3, {0xE9, 0xAE, 0x97},
		2, {0x97, 0x9B},
		2, {0xF2, 0xBC}
	},
	{
		2, {0xE9, 0xBB},
		3, {0xE9, 0xAE, 0x9F},
		2, {0x9F, 0x9B},
		2, {0xF2, 0xBD}
	},
	{
		2, {0xE9, 0xBC},
		3, {0xE9, 0xAE, 0xA0},
		2, {0xA0, 0x9B},
		2, {0xF2, 0xBE}
	},
	{
		2, {0xE9, 0xBD},
		3, {0xE9, 0xAE, 0xA8},
		2, {0xA8, 0x9B},
		2, {0xF2, 0xBF}
	},
	{
		2, {0xE9, 0xBE},
		3, {0xE9, 0xAE, 0xB4},
		2, {0xB4, 0x9B},
		2, {0xF2, 0xC0}
	},
	{
		2, {0xE9, 0xBF},
		3, {0xE9, 0xAF, 0x80},
		2, {0xC0, 0x9B},
		2, {0xF2, 0xC1}
	},
	{
		2, {0xE9, 0xC0},
		3, {0xE9, 0xAF, 0x8A},
		2, {0xCA, 0x9B},
		2, {0xF2, 0xC2}
	},
	{
		2, {0xE9, 0xC1},
		3, {0xE9, 0xAE, 0xB9},
		2, {0xB9, 0x9B},
		2, {0xF2, 0xC3}
	},
	{
		2, {0xE9, 0xC2},
		3, {0xE9, 0xAF, 0x86},
		2, {0xC6, 0x9B},
		2, {0xF2, 0xC4}
	},
	{
		2, {0xE9, 0xC3},
		3, {0xE9, 0xAF, 0x8F},
		2, {0xCF, 0x9B},
		2, {0xF2, 0xC5}
	},
	{
		2, {0xE9, 0xC4},
		3, {0xE9, 0xAF, 0x91},
		2, {0xD1, 0x9B},
		2, {0xF2, 0xC6}
	},
	{
		2, {0xE9, 0xC5},
		3, {0xE9, 0xAF, 0x92},
		2, {0xD2, 0x9B},
		2, {0xF2, 0xC7}
	},
	{
		2, {0xE9, 0xC6},
		3, {0xE9, 0xAF, 0xA3},
		2, {0xE3, 0x9B},
		2, {0xF2, 0xC8}
	},
	{
		2, {0xE9, 0xC7},
		3, {0xE9, 0xAF, 0xA2},
		2, {0xE2, 0x9B},
		2, {0xF2, 0xC9}
	},
	{
		2, {0xE9, 0xC8},
		3, {0xE9, 0xAF, 0xA4},
		2, {0xE4, 0x9B},
		2, {0xF2, 0xCA}
	},
	{
		2, {0xE9, 0xC9},
		3, {0xE9, 0xAF, 0x94},
		2, {0xD4, 0x9B},
		2, {0xF2, 0xCB}
	},
	{
		2, {0xE9, 0xCA},
		3, {0xE9, 0xAF, 0xA1},
		2, {0xE1, 0x9B},
		2, {0xF2, 0xCC}
	},
	{
		2, {0xE9, 0xCB},
		3, {0xE9, 0xB0, 0xBA},
		2, {0x3A, 0x9C},
		2, {0xF2, 0xCD}
	},
	{
		2, {0xE9, 0xCC},
		3, {0xE9, 0xAF, 0xB2},
		2, {0xF2, 0x9B},
		2, {0xF2, 0xCE}
	},
	{
		2, {0xE9, 0xCD},
		3, {0xE9, 0xAF, 0xB1},
		2, {0xF1, 0x9B},
		2, {0xF2, 0xCF}
	},
	{
		2, {0xE9, 0xCE},
		3, {0xE9, 0xAF, 0xB0},
		2, {0xF0, 0x9B},
		2, {0xF2, 0xD0}
	},
	{
		2, {0xE9, 0xCF},
		3, {0xE9, 0xB0, 0x95},
		2, {0x15, 0x9C},
		2, {0xF2, 0xD1}
	},
	{
		2, {0xE9, 0xD0},
		3, {0xE9, 0xB0, 0x94},
		2, {0x14, 0x9C},
		2, {0xF2, 0xD2}
	},
	{
		2, {0xE9, 0xD1},
		3, {0xE9, 0xB0, 0x89},
		2, {0x09, 0x9C},
		2, {0xF2, 0xD3}
	},
	{
		2, {0xE9, 0xD2},
		3, {0xE9, 0xB0, 0x93},
		2, {0x13, 0x9C},
		2, {0xF2, 0xD4}
	},
	{
		2, {0xE9, 0xD3},
		3, {0xE9, 0xB0, 0x8C},
		2, {0x0C, 0x9C},
		2, {0xF2, 0xD5}
	},
	{
		2, {0xE9, 0xD4},
		3, {0xE9, 0xB0, 0x86},
		2, {0x06, 0x9C},
		2, {0xF2, 0xD6}
	},
	{
		2, {0xE9, 0xD5},
		3, {0xE9, 0xB0, 0x88},
		2, {0x08, 0x9C},
		2, {0xF2, 0xD7}
	},
	{
		2, {0xE9, 0xD6},
		3, {0xE9, 0xB0, 0x92},
		2, {0x12, 0x9C},
		2, {0xF2, 0xD8}
	},
	{
		2, {0xE9, 0xD7},
		3, {0xE9, 0xB0, 0x8A},
		2, {0x0A, 0x9C},
		2, {0xF2, 0xD9}
	},
	{
		2, {0xE9, 0xD8},
		3, {0xE9, 0xB0, 0x84},
		2, {0x04, 0x9C},
		2, {0xF2, 0xDA}
	},
	{
		2, {0xE9, 0xD9},
		3, {0xE9, 0xB0, 0xAE},
		2, {0x2E, 0x9C},
		2, {0xF2, 0xDB}
	},
	{
		2, {0xE9, 0xDA},
		3, {0xE9, 0xB0, 0x9B},
		2, {0x1B, 0x9C},
		2, {0xF2, 0xDC}
	},
	{
		2, {0xE9, 0xDB},
		3, {0xE9, 0xB0, 0xA5},
		2, {0x25, 0x9C},
		2, {0xF2, 0xDD}
	},
	{
		2, {0xE9, 0xDC},
		3, {0xE9, 0xB0, 0xA4},
		2, {0x24, 0x9C},
		2, {0xF2, 0xDE}
	},
	{
		2, {0xE9, 0xDD},
		3, {0xE9, 0xB0, 0xA1},
		2, {0x21, 0x9C},
		2, {0xF2, 0xDF}
	},
	{
		2, {0xE9, 0xDE},
		3, {0xE9, 0xB0, 0xB0},
		2, {0x30, 0x9C},
		2, {0xF2, 0xE0}
	},
	{
		2, {0xE9, 0xDF},
		3, {0xE9, 0xB1, 0x87},
		2, {0x47, 0x9C},
		2, {0xF2, 0xE1}
	},
	{
		2, {0xE9, 0xE0},
		3, {0xE9, 0xB0, 0xB2},
		2, {0x32, 0x9C},
		2, {0xF2, 0xE2}
	},
	{
		2, {0xE9, 0xE1},
		3, {0xE9, 0xB1, 0x86},
		2, {0x46, 0x9C},
		2, {0xF2, 0xE3}
	},
	{
		2, {0xE9, 0xE2},
		3, {0xE9, 0xB0, 0xBE},
		2, {0x3E, 0x9C},
		2, {0xF2, 0xE4}
	},
	{
		2, {0xE9, 0xE3},
		3, {0xE9, 0xB1, 0x9A},
		2, {0x5A, 0x9C},
		2, {0xF2, 0xE5}
	},
	{
		2, {0xE9, 0xE4},
		3, {0xE9, 0xB1, 0xA0},
		2, {0x60, 0x9C},
		2, {0xF2, 0xE6}
	},
	{
		2, {0xE9, 0xE5},
		3, {0xE9, 0xB1, 0xA7},
		2, {0x67, 0x9C},
		2, {0xF2, 0xE7}
	},
	{
		2, {0xE9, 0xE6},
		3, {0xE9, 0xB1, 0xB6},
		2, {0x76, 0x9C},
		2, {0xF2, 0xE8}
	},
	{
		2, {0xE9, 0xE7},
		3, {0xE9, 0xB1, 0xB8},
		2, {0x78, 0x9C},
		2, {0xF2, 0xE9}
	},
	{
		2, {0xE9, 0xE8},
		3, {0xE9, 0xB3, 0xA7},
		2, {0xE7, 0x9C},
		2, {0xF2, 0xEA}
	},
	{
		2, {0xE9, 0xE9},
		3, {0xE9, 0xB3, 0xAC},
		2, {0xEC, 0x9C},
		2, {0xF2, 0xEB}
	},
	{
		2, {0xE9, 0xEA},
		3, {0xE9, 0xB3, 0xB0},
		2, {0xF0, 0x9C},
		2, {0xF2, 0xEC}
	},
	{
		2, {0xE9, 0xEB},
		3, {0xE9, 0xB4, 0x89},
		2, {0x09, 0x9D},
		2, {0xF2, 0xED}
	},
	{
		2, {0xE9, 0xEC},
		3, {0xE9, 0xB4, 0x88},
		2, {0x08, 0x9D},
		2, {0xF2, 0xEE}
	},
	{
		2, {0xE9, 0xED},
		3, {0xE9, 0xB3, 0xAB},
		2, {0xEB, 0x9C},
		2, {0xF2, 0xEF}
	},
	{
		2, {0xE9, 0xEE},
		3, {0xE9, 0xB4, 0x83},
		2, {0x03, 0x9D},
		2, {0xF2, 0xF0}
	},
	{
		2, {0xE9, 0xEF},
		3, {0xE9, 0xB4, 0x86},
		2, {0x06, 0x9D},
		2, {0xF2, 0xF1}
	},
	{
		2, {0xE9, 0xF0},
		3, {0xE9, 0xB4, 0xAA},
		2, {0x2A, 0x9D},
		2, {0xF2, 0xF2}
	},
	{
		2, {0xE9, 0xF1},
		3, {0xE9, 0xB4, 0xA6},
		2, {0x26, 0x9D},
		2, {0xF2, 0xF3}
	},
	{
		2, {0xE9, 0xF2},
		3, {0xE9, 0xB6, 0xAF},
		2, {0xAF, 0x9D},
		2, {0xF2, 0xF4}
	},
	{
		2, {0xE9, 0xF3},
		3, {0xE9, 0xB4, 0xA3},
		2, {0x23, 0x9D},
		2, {0xF2, 0xF5}
	},
	{
		2, {0xE9, 0xF4},
		3, {0xE9, 0xB4, 0x9F},
		2, {0x1F, 0x9D},
		2, {0xF2, 0xF6}
	},
	{
		2, {0xE9, 0xF5},
		3, {0xE9, 0xB5, 0x84},
		2, {0x44, 0x9D},
		2, {0xF2, 0xF7}
	},
	{
		2, {0xE9, 0xF6},
		3, {0xE9, 0xB4, 0x95},
		2, {0x15, 0x9D},
		2, {0xF2, 0xF8}
	},
	{
		2, {0xE9, 0xF7},
		3, {0xE9, 0xB4, 0x92},
		2, {0x12, 0x9D},
		2, {0xF2, 0xF9}
	},
	{
		2, {0xE9, 0xF8},
		3, {0xE9, 0xB5, 0x81},
		2, {0x41, 0x9D},
		2, {0xF2, 0xFA}
	},
	{
		2, {0xE9, 0xF9},
		3, {0xE9, 0xB4, 0xBF},
		2, {0x3F, 0x9D},
		2, {0xF2, 0xFB}
	},
	{
		2, {0xE9, 0xFA},
		3, {0xE9, 0xB4, 0xBE},
		2, {0x3E, 0x9D},
		2, {0xF2, 0xFC}
	},
	{
		2, {0xE9, 0xFB},
		3, {0xE9, 0xB5, 0x86},
		2, {0x46, 0x9D},
		2, {0xF2, 0xFD}
	},
	{
		2, {0xE9, 0xFC},
		3, {0xE9, 0xB5, 0x88},
		2, {0x48, 0x9D},
		2, {0xF2, 0xFE}
	},
	{
		2, {0xEA, 0x40},
		3, {0xE9, 0xB5, 0x9D},
		2, {0x5D, 0x9D},
		2, {0xF3, 0xA1}
	},
	{
		2, {0xEA, 0x41},
		3, {0xE9, 0xB5, 0x9E},
		2, {0x5E, 0x9D},
		2, {0xF3, 0xA2}
	},
	{
		2, {0xEA, 0x42},
		3, {0xE9, 0xB5, 0xA4},
		2, {0x64, 0x9D},
		2, {0xF3, 0xA3}
	},
	{
		2, {0xEA, 0x43},
		3, {0xE9, 0xB5, 0x91},
		2, {0x51, 0x9D},
		2, {0xF3, 0xA4}
	},
	{
		2, {0xEA, 0x44},
		3, {0xE9, 0xB5, 0x90},
		2, {0x50, 0x9D},
		2, {0xF3, 0xA5}
	},
	{
		2, {0xEA, 0x45},
		3, {0xE9, 0xB5, 0x99},
		2, {0x59, 0x9D},
		2, {0xF3, 0xA6}
	},
	{
		2, {0xEA, 0x46},
		3, {0xE9, 0xB5, 0xB2},
		2, {0x72, 0x9D},
		2, {0xF3, 0xA7}
	},
	{
		2, {0xEA, 0x47},
		3, {0xE9, 0xB6, 0x89},
		2, {0x89, 0x9D},
		2, {0xF3, 0xA8}
	},
	{
		2, {0xEA, 0x48},
		3, {0xE9, 0xB6, 0x87},
		2, {0x87, 0x9D},
		2, {0xF3, 0xA9}
	},
	{
		2, {0xEA, 0x49},
		3, {0xE9, 0xB6, 0xAB},
		2, {0xAB, 0x9D},
		2, {0xF3, 0xAA}
	},
	{
		2, {0xEA, 0x4A},
		3, {0xE9, 0xB5, 0xAF},
		2, {0x6F, 0x9D},
		2, {0xF3, 0xAB}
	},
	{
		2, {0xEA, 0x4B},
		3, {0xE9, 0xB5, 0xBA},
		2, {0x7A, 0x9D},
		2, {0xF3, 0xAC}
	},
	{
		2, {0xEA, 0x4C},
		3, {0xE9, 0xB6, 0x9A},
		2, {0x9A, 0x9D},
		2, {0xF3, 0xAD}
	},
	{
		2, {0xEA, 0x4D},
		3, {0xE9, 0xB6, 0xA4},
		2, {0xA4, 0x9D},
		2, {0xF3, 0xAE}
	},
	{
		2, {0xEA, 0x4E},
		3, {0xE9, 0xB6, 0xA9},
		2, {0xA9, 0x9D},
		2, {0xF3, 0xAF}
	},
	{
		2, {0xEA, 0x4F},
		3, {0xE9, 0xB6, 0xB2},
		2, {0xB2, 0x9D},
		2, {0xF3, 0xB0}
	},
	{
		2, {0xEA, 0x50},
		3, {0xE9, 0xB7, 0x84},
		2, {0xC4, 0x9D},
		2, {0xF3, 0xB1}
	},
	{
		2, {0xEA, 0x51},
		3, {0xE9, 0xB7, 0x81},
		2, {0xC1, 0x9D},
		2, {0xF3, 0xB2}
	},
	{
		2, {0xEA, 0x52},
		3, {0xE9, 0xB6, 0xBB},
		2, {0xBB, 0x9D},
		2, {0xF3, 0xB3}
	},
	{
		2, {0xEA, 0x53},
		3, {0xE9, 0xB6, 0xB8},
		2, {0xB8, 0x9D},
		2, {0xF3, 0xB4}
	},
	{
		2, {0xEA, 0x54},
		3, {0xE9, 0xB6, 0xBA},
		2, {0xBA, 0x9D},
		2, {0xF3, 0xB5}
	},
	{
		2, {0xEA, 0x55},
		3, {0xE9, 0xB7, 0x86},
		2, {0xC6, 0x9D},
		2, {0xF3, 0xB6}
	},
	{
		2, {0xEA, 0x56},
		3, {0xE9, 0xB7, 0x8F},
		2, {0xCF, 0x9D},
		2, {0xF3, 0xB7}
	},
	{
		2, {0xEA, 0x57},
		3, {0xE9, 0xB7, 0x82},
		2, {0xC2, 0x9D},
		2, {0xF3, 0xB8}
	},
	{
		2, {0xEA, 0x58},
		3, {0xE9, 0xB7, 0x99},
		2, {0xD9, 0x9D},
		2, {0xF3, 0xB9}
	},
	{
		2, {0xEA, 0x59},
		3, {0xE9, 0xB7, 0x93},
		2, {0xD3, 0x9D},
		2, {0xF3, 0xBA}
	},
	{
		2, {0xEA, 0x5A},
		3, {0xE9, 0xB7, 0xB8},
		2, {0xF8, 0x9D},
		2, {0xF3, 0xBB}
	},
	{
		2, {0xEA, 0x5B},
		3, {0xE9, 0xB7, 0xA6},
		2, {0xE6, 0x9D},
		2, {0xF3, 0xBC}
	},
	{
		2, {0xEA, 0x5C},
		3, {0xE9, 0xB7, 0xAD},
		2, {0xED, 0x9D},
		2, {0xF3, 0xBD}
	},
	{
		2, {0xEA, 0x5D},
		3, {0xE9, 0xB7, 0xAF},
		2, {0xEF, 0x9D},
		2, {0xF3, 0xBE}
	},
	{
		2, {0xEA, 0x5E},
		3, {0xE9, 0xB7, 0xBD},
		2, {0xFD, 0x9D},
		2, {0xF3, 0xBF}
	},
	{
		2, {0xEA, 0x5F},
		3, {0xE9, 0xB8, 0x9A},
		2, {0x1A, 0x9E},
		2, {0xF3, 0xC0}
	},
	{
		2, {0xEA, 0x60},
		3, {0xE9, 0xB8, 0x9B},
		2, {0x1B, 0x9E},
		2, {0xF3, 0xC1}
	},
	{
		2, {0xEA, 0x61},
		3, {0xE9, 0xB8, 0x9E},
		2, {0x1E, 0x9E},
		2, {0xF3, 0xC2}
	},
	{
		2, {0xEA, 0x62},
		3, {0xE9, 0xB9, 0xB5},
		2, {0x75, 0x9E},
		2, {0xF3, 0xC3}
	},
	{
		2, {0xEA, 0x63},
		3, {0xE9, 0xB9, 0xB9},
		2, {0x79, 0x9E},
		2, {0xF3, 0xC4}
	},
	{
		2, {0xEA, 0x64},
		3, {0xE9, 0xB9, 0xBD},
		2, {0x7D, 0x9E},
		2, {0xF3, 0xC5}
	},
	{
		2, {0xEA, 0x65},
		3, {0xE9, 0xBA, 0x81},
		2, {0x81, 0x9E},
		2, {0xF3, 0xC6}
	},
	{
		2, {0xEA, 0x66},
		3, {0xE9, 0xBA, 0x88},
		2, {0x88, 0x9E},
		2, {0xF3, 0xC7}
	},
	{
		2, {0xEA, 0x67},
		3, {0xE9, 0xBA, 0x8B},
		2, {0x8B, 0x9E},
		2, {0xF3, 0xC8}
	},
	{
		2, {0xEA, 0x68},
		3, {0xE9, 0xBA, 0x8C},
		2, {0x8C, 0x9E},
		2, {0xF3, 0xC9}
	},
	{
		2, {0xEA, 0x69},
		3, {0xE9, 0xBA, 0x92},
		2, {0x92, 0x9E},
		2, {0xF3, 0xCA}
	},
	{
		2, {0xEA, 0x6A},
		3, {0xE9, 0xBA, 0x95},
		2, {0x95, 0x9E},
		2, {0xF3, 0xCB}
	},
	{
		2, {0xEA, 0x6B},
		3, {0xE9, 0xBA, 0x91},
		2, {0x91, 0x9E},
		2, {0xF3, 0xCC}
	},
	{
		2, {0xEA, 0x6C},
		3, {0xE9, 0xBA, 0x9D},
		2, {0x9D, 0x9E},
		2, {0xF3, 0xCD}
	},
	{
		2, {0xEA, 0x6D},
		3, {0xE9, 0xBA, 0xA5},
		2, {0xA5, 0x9E},
		2, {0xF3, 0xCE}
	},
	{
		2, {0xEA, 0x6E},
		3, {0xE9, 0xBA, 0xA9},
		2, {0xA9, 0x9E},
		2, {0xF3, 0xCF}
	},
	{
		2, {0xEA, 0x6F},
		3, {0xE9, 0xBA, 0xB8},
		2, {0xB8, 0x9E},
		2, {0xF3, 0xD0}
	},
	{
		2, {0xEA, 0x70},
		3, {0xE9, 0xBA, 0xAA},
		2, {0xAA, 0x9E},
		2, {0xF3, 0xD1}
	},
	{
		2, {0xEA, 0x71},
		3, {0xE9, 0xBA, 0xAD},
		2, {0xAD, 0x9E},
		2, {0xF3, 0xD2}
	},
	{
		2, {0xEA, 0x72},
		3, {0xE9, 0x9D, 0xA1},
		2, {0x61, 0x97},
		2, {0xF3, 0xD3}
	},
	{
		2, {0xEA, 0x73},
		3, {0xE9, 0xBB, 0x8C},
		2, {0xCC, 0x9E},
		2, {0xF3, 0xD4}
	},
	{
		2, {0xEA, 0x74},
		3, {0xE9, 0xBB, 0x8E},
		2, {0xCE, 0x9E},
		2, {0xF3, 0xD5}
	},
	{
		2, {0xEA, 0x75},
		3, {0xE9, 0xBB, 0x8F},
		2, {0xCF, 0x9E},
		2, {0xF3, 0xD6}
	},
	{
		2, {0xEA, 0x76},
		3, {0xE9, 0xBB, 0x90},
		2, {0xD0, 0x9E},
		2, {0xF3, 0xD7}
	},
	{
		2, {0xEA, 0x77},
		3, {0xE9, 0xBB, 0x94},
		2, {0xD4, 0x9E},
		2, {0xF3, 0xD8}
	},
	{
		2, {0xEA, 0x78},
		3, {0xE9, 0xBB, 0x9C},
		2, {0xDC, 0x9E},
		2, {0xF3, 0xD9}
	},
	{
		2, {0xEA, 0x79},
		3, {0xE9, 0xBB, 0x9E},
		2, {0xDE, 0x9E},
		2, {0xF3, 0xDA}
	},
	{
		2, {0xEA, 0x7A},
		3, {0xE9, 0xBB, 0x9D},
		2, {0xDD, 0x9E},
		2, {0xF3, 0xDB}
	},
	{
		2, {0xEA, 0x7B},
		3, {0xE9, 0xBB, 0xA0},
		2, {0xE0, 0x9E},
		2, {0xF3, 0xDC}
	},
	{
		2, {0xEA, 0x7C},
		3, {0xE9, 0xBB, 0xA5},
		2, {0xE5, 0x9E},
		2, {0xF3, 0xDD}
	},
	{
		2, {0xEA, 0x7D},
		3, {0xE9, 0xBB, 0xA8},
		2, {0xE8, 0x9E},
		2, {0xF3, 0xDE}
	},
	{
		2, {0xEA, 0x7E},
		3, {0xE9, 0xBB, 0xAF},
		2, {0xEF, 0x9E},
		2, {0xF3, 0xDF}
	},
	{
		2, {0xEA, 0x80},
		3, {0xE9, 0xBB, 0xB4},
		2, {0xF4, 0x9E},
		2, {0xF3, 0xE0}
	},
	{
		2, {0xEA, 0x81},
		3, {0xE9, 0xBB, 0xB6},
		2, {0xF6, 0x9E},
		2, {0xF3, 0xE1}
	},
	{
		2, {0xEA, 0x82},
		3, {0xE9, 0xBB, 0xB7},
		2, {0xF7, 0x9E},
		2, {0xF3, 0xE2}
	},
	{
		2, {0xEA, 0x83},
		3, {0xE9, 0xBB, 0xB9},
		2, {0xF9, 0x9E},
		2, {0xF3, 0xE3}
	},
	{
		2, {0xEA, 0x84},
		3, {0xE9, 0xBB, 0xBB},
		2, {0xFB, 0x9E},
		2, {0xF3, 0xE4}
	},
	{
		2, {0xEA, 0x85},
		3, {0xE9, 0xBB, 0xBC},
		2, {0xFC, 0x9E},
		2, {0xF3, 0xE5}
	},
	{
		2, {0xEA, 0x86},
		3, {0xE9, 0xBB, 0xBD},
		2, {0xFD, 0x9E},
		2, {0xF3, 0xE6}
	},
	{
		2, {0xEA, 0x87},
		3, {0xE9, 0xBC, 0x87},
		2, {0x07, 0x9F},
		2, {0xF3, 0xE7}
	},
	{
		2, {0xEA, 0x88},
		3, {0xE9, 0xBC, 0x88},
		2, {0x08, 0x9F},
		2, {0xF3, 0xE8}
	},
	{
		2, {0xEA, 0x89},
		3, {0xE7, 0x9A, 0xB7},
		2, {0xB7, 0x76},
		2, {0xF3, 0xE9}
	},
	{
		2, {0xEA, 0x8A},
		3, {0xE9, 0xBC, 0x95},
		2, {0x15, 0x9F},
		2, {0xF3, 0xEA}
	},
	{
		2, {0xEA, 0x8B},
		3, {0xE9, 0xBC, 0xA1},
		2, {0x21, 0x9F},
		2, {0xF3, 0xEB}
	},
	{
		2, {0xEA, 0x8C},
		3, {0xE9, 0xBC, 0xAC},
		2, {0x2C, 0x9F},
		2, {0xF3, 0xEC}
	},
	{
		2, {0xEA, 0x8D},
		3, {0xE9, 0xBC, 0xBE},
		2, {0x3E, 0x9F},
		2, {0xF3, 0xED}
	},
	{
		2, {0xEA, 0x8E},
		3, {0xE9, 0xBD, 0x8A},
		2, {0x4A, 0x9F},
		2, {0xF3, 0xEE}
	},
	{
		2, {0xEA, 0x8F},
		3, {0xE9, 0xBD, 0x92},
		2, {0x52, 0x9F},
		2, {0xF3, 0xEF}
	},
	{
		2, {0xEA, 0x90},
		3, {0xE9, 0xBD, 0x94},
		2, {0x54, 0x9F},
		2, {0xF3, 0xF0}
	},
	{
		2, {0xEA, 0x91},
		3, {0xE9, 0xBD, 0xA3},
		2, {0x63, 0x9F},
		2, {0xF3, 0xF1}
	},
	{
		2, {0xEA, 0x92},
		3, {0xE9, 0xBD, 0x9F},
		2, {0x5F, 0x9F},
		2, {0xF3, 0xF2}
	},
	{
		2, {0xEA, 0x93},
		3, {0xE9, 0xBD, 0xA0},
		2, {0x60, 0x9F},
		2, {0xF3, 0xF3}
	},
	{
		2, {0xEA, 0x94},
		3, {0xE9, 0xBD, 0xA1},
		2, {0x61, 0x9F},
		2, {0xF3, 0xF4}
	},
	{
		2, {0xEA, 0x95},
		3, {0xE9, 0xBD, 0xA6},
		2, {0x66, 0x9F},
		2, {0xF3, 0xF5}
	},
	{
		2, {0xEA, 0x96},
		3, {0xE9, 0xBD, 0xA7},
		2, {0x67, 0x9F},
		2, {0xF3, 0xF6}
	},
	{
		2, {0xEA, 0x97},
		3, {0xE9, 0xBD, 0xAC},
		2, {0x6C, 0x9F},
		2, {0xF3, 0xF7}
	},
	{
		2, {0xEA, 0x98},
		3, {0xE9, 0xBD, 0xAA},
		2, {0x6A, 0x9F},
		2, {0xF3, 0xF8}
	},
	{
		2, {0xEA, 0x99},
		3, {0xE9, 0xBD, 0xB7},
		2, {0x77, 0x9F},
		2, {0xF3, 0xF9}
	},
	{
		2, {0xEA, 0x9A},
		3, {0xE9, 0xBD, 0xB2},
		2, {0x72, 0x9F},
		2, {0xF3, 0xFA}
	},
	{
		2, {0xEA, 0x9B},
		3, {0xE9, 0xBD, 0xB6},
		2, {0x76, 0x9F},
		2, {0xF3, 0xFB}
	},
	{
		2, {0xEA, 0x9C},
		3, {0xE9, 0xBE, 0x95},
		2, {0x95, 0x9F},
		2, {0xF3, 0xFC}
	},
	{
		2, {0xEA, 0x9D},
		3, {0xE9, 0xBE, 0x9C},
		2, {0x9C, 0x9F},
		2, {0xF3, 0xFD}
	},
	{
		2, {0xEA, 0x9E},
		3, {0xE9, 0xBE, 0xA0},
		2, {0xA0, 0x9F},
		2, {0xF3, 0xFE}
	},
	{
		2, {0xEA, 0x9F},
		3, {0xE5, 0xA0, 0xAF},
		2, {0x2F, 0x58},
		2, {0xF4, 0xA1}
	},
	{
		2, {0xEA, 0xA0},
		3, {0xE6, 0xA7, 0x87},
		2, {0xC7, 0x69},
		2, {0xF4, 0xA2}
	},
	{
		2, {0xEA, 0xA1},
		3, {0xE9, 0x81, 0x99},
		2, {0x59, 0x90},
		2, {0xF4, 0xA3}
	},
	{
		2, {0xEA, 0xA2},
		3, {0xE7, 0x91, 0xA4},
		2, {0x64, 0x74},
		2, {0xF4, 0xA4}
	},
	{
		2, {0xEA, 0xA3},
		3, {0xE5, 0x87, 0x9C},
		2, {0xDC, 0x51},
		2, {0xF4, 0xA5}
	},
	{
		2, {0xEA, 0xA4},
		3, {0xE7, 0x86, 0x99},
		2, {0x99, 0x71},
		2, {0xF4, 0xA6}
	},
	{
		2, {0xEA, 0xA5},
		3, {0xE5, 0x99, 0x93},
		2, {0x53, 0x56},
		0, {0x00}
	},
	{
		2, {0xEA, 0xA6},
		3, {0xE5, 0xB7, 0xA2},
		2, {0xE2, 0x5D},
		0, {0x00}
	},
	{
		2, {0xEA, 0xA7},
		3, {0xE5, 0xB8, 0x94},
		2, {0x14, 0x5E},
		3, {0x8F, 0xBC, 0xA8}
	},
	{
		2, {0xEA, 0xA8},
		3, {0xE5, 0xB8, 0x98},
		2, {0x18, 0x5E},
		3, {0x8F, 0xBC, 0xAA}
	},
	{
		2, {0xEA, 0xA9},
		3, {0xE5, 0xB9, 0x98},
		2, {0x58, 0x5E},
		3, {0x8F, 0xBC, 0xB7}
	},
	{
		2, {0xEA, 0xAA},
		3, {0xE5, 0xB9, 0x9E},
		2, {0x5E, 0x5E},
		3, {0x8F, 0xBC, 0xBA}
	},
	{
		2, {0xEA, 0xAB},
		3, {0xE5, 0xBA, 0xBE},
		2, {0xBE, 0x5E},
		3, {0x8F, 0xBC, 0xCE}
	},
	{
		2, {0xEA, 0xAC},
		3, {0xEF, 0xA4, 0xA8},
		2, {0x28, 0xF9},
		0, {0x00}
	},
	{
		2, {0xEA, 0xAD},
		3, {0xE5, 0xBB, 0x8B},
		2, {0xCB, 0x5E},
		3, {0x8F, 0xBC, 0xD2}
	},
	{
		2, {0xEA, 0xAE},
		3, {0xE5, 0xBB, 0xB9},
		2, {0xF9, 0x5E},
		0, {0x00}
	},
	{
		2, {0xEA, 0xAF},
		3, {0xE5, 0xBC, 0x80},
		2, {0x00, 0x5F},
		0, {0x00}
	},
	{
		2, {0xEA, 0xB0},
		3, {0xE5, 0xBC, 0x82},
		2, {0x02, 0x5F},
		3, {0x8F, 0xBC, 0xDC}
	},
	{
		2, {0xEA, 0xB1},
		3, {0xE5, 0xBC, 0x87},
		2, {0x07, 0x5F},
		3, {0x8F, 0xBC, 0xDE}
	},
	{
		2, {0xEA, 0xB2},
		3, {0xE5, 0xBC, 0x9D},
		2, {0x1D, 0x5F},
		3, {0x8F, 0xBC, 0xE3}
	},
	{
		2, {0xEA, 0xB3},
		3, {0xE5, 0xBC, 0xA3},
		2, {0x23, 0x5F},
		3, {0x8F, 0xBC, 0xE6}
	},
	{
		2, {0xEA, 0xB4},
		3, {0xE5, 0xBC, 0xB4},
		2, {0x34, 0x5F},
		3, {0x8F, 0xBC, 0xED}
	},
	{
		2, {0xEA, 0xB5},
		3, {0xE5, 0xBC, 0xB6},
		2, {0x36, 0x5F},
		3, {0x8F, 0xBC, 0xEE}
	},
	{
		2, {0xEA, 0xB6},
		3, {0xE5, 0xBC, 0xBD},
		2, {0x3D, 0x5F},
		3, {0x8F, 0xBC, 0xF0}
	},
	{
		2, {0xEA, 0xB7},
		3, {0xE5, 0xBD, 0x80},
		2, {0x40, 0x5F},
		3, {0x8F, 0xBC, 0xF2}
	},
	{
		2, {0xEA, 0xB8},
		3, {0xE5, 0xBD, 0x85},
		2, {0x45, 0x5F},
		3, {0x8F, 0xBC, 0xF4}
	},
	{
		2, {0xEA, 0xB9},
		3, {0xE5, 0xBD, 0x94},
		2, {0x54, 0x5F},
		3, {0x8F, 0xBC, 0xF8}
	},
	{
		2, {0xEA, 0xBA},
		3, {0xE5, 0xBD, 0x98},
		2, {0x58, 0x5F},
		3, {0x8F, 0xBC, 0xF9}
	},
	{
		2, {0xEA, 0xBB},
		3, {0xE5, 0xBD, 0xA4},
		2, {0x64, 0x5F},
		3, {0x8F, 0xBC, 0xFD}
	},
	{
		2, {0xEA, 0xBC},
		3, {0xE5, 0xBD, 0xA7},
		2, {0x67, 0x5F},
		3, {0x8F, 0xBC, 0xFE}
	},
	{
		2, {0xEA, 0xBD},
		3, {0xE5, 0xBD, 0xBD},
		2, {0x7D, 0x5F},
		3, {0x8F, 0xBD, 0xA7}
	},
	{
		2, {0xEA, 0xBE},
		3, {0xE5, 0xBE, 0x89},
		2, {0x89, 0x5F},
		3, {0x8F, 0xBD, 0xA9}
	},
	{
		2, {0xEA, 0xBF},
		3, {0xE5, 0xBE, 0x9C},
		2, {0x9C, 0x5F},
		3, {0x8F, 0xBD, 0xAD}
	},
	{
		2, {0xEA, 0xC0},
		3, {0xE5, 0xBE, 0xA7},
		2, {0xA7, 0x5F},
		3, {0x8F, 0xBD, 0xB0}
	},
	{
		2, {0xEA, 0xC1},
		3, {0xE5, 0xBE, 0xAF},
		2, {0xAF, 0x5F},
		3, {0x8F, 0xBD, 0xB4}
	},
	{
		2, {0xEA, 0xC2},
		3, {0xE5, 0xBE, 0xB5},
		2, {0xB5, 0x5F},
		0, {0x00}
	},
	{
		2, {0xEA, 0xC3},
		3, {0xE5, 0xBE, 0xB7},
		2, {0xB7, 0x5F},
		0, {0x00}
	},
	{
		2, {0xEA, 0xC4},
		3, {0xE5, 0xBF, 0x89},
		2, {0xC9, 0x5F},
		3, {0x8F, 0xBD, 0xBB}
	},
	{
		2, {0xEA, 0xC5},
		3, {0xE5, 0xBF, 0x9E},
		2, {0xDE, 0x5F},
		3, {0x8F, 0xBD, 0xC2}
	},
	{
		2, {0xEA, 0xC6},
		3, {0xE5, 0xBF, 0xA1},
		2, {0xE1, 0x5F},
		3, {0x8F, 0xBD, 0xC3}
	},
	{
		2, {0xEA, 0xC7},
		3, {0xE5, 0xBF, 0xA9},
		2, {0xE9, 0x5F},
		3, {0x8F, 0xBD, 0xC6}
	},
	{
		2, {0xEA, 0xC8},
		3, {0xE6, 0x80, 0x8D},
		2, {0x0D, 0x60},
		3, {0x8F, 0xBD, 0xD3}
	},
	{
		2, {0xEA, 0xC9},
		3, {0xE6, 0x80, 0x94},
		2, {0x14, 0x60},
		3, {0x8F, 0xBD, 0xD5}
	},
	{
		2, {0xEA, 0xCA},
		3, {0xE6, 0x80, 0x98},
		2, {0x18, 0x60},
		3, {0x8F, 0xBD, 0xD7}
	},
	{
		2, {0xEA, 0xCB},
		3, {0xE6, 0x80, 0xB3},
		2, {0x33, 0x60},
		3, {0x8F, 0xBD, 0xDC}
	},
	{
		2, {0xEA, 0xCC},
		3, {0xE6, 0x80, 0xB5},
		2, {0x35, 0x60},
		3, {0x8F, 0xBD, 0xDD}
	},
	{
		2, {0xEA, 0xCD},
		3, {0xE6, 0x81, 0x87},
		2, {0x47, 0x60},
		3, {0x8F, 0xBD, 0xDF}
	},
	{
		2, {0xEA, 0xCE},
		3, {0xEF, 0xA8, 0xBD},
		2, {0x3D, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEA, 0xCF},
		3, {0xE6, 0x82, 0x9D},
		2, {0x9D, 0x60},
		3, {0x8F, 0xBD, 0xF6}
	},
	{
		2, {0xEA, 0xD0},
		3, {0xE6, 0x82, 0x9E},
		2, {0x9E, 0x60},
		3, {0x8F, 0xBD, 0xF7}
	},
	{
		2, {0xEA, 0xD1},
		3, {0xE6, 0x83, 0x8B},
		2, {0xCB, 0x60},
		3, {0x8F, 0xBE, 0xA8}
	},
	{
		2, {0xEA, 0xD2},
		3, {0xE6, 0x83, 0x94},
		2, {0xD4, 0x60},
		3, {0x8F, 0xBE, 0xAB}
	},
	{
		2, {0xEA, 0xD3},
		3, {0xE6, 0x83, 0x95},
		2, {0xD5, 0x60},
		3, {0x8F, 0xBE, 0xAC}
	},
	{
		2, {0xEA, 0xD4},
		3, {0xE6, 0x83, 0x9D},
		2, {0xDD, 0x60},
		3, {0x8F, 0xBE, 0xAF}
	},
	{
		2, {0xEA, 0xD5},
		3, {0xE6, 0x83, 0xB8},
		2, {0xF8, 0x60},
		3, {0x8F, 0xBE, 0xB5}
	},
	{
		2, {0xEA, 0xD6},
		3, {0xE6, 0x84, 0x9C},
		2, {0x1C, 0x61},
		3, {0x8F, 0xBE, 0xC4}
	},
	{
		2, {0xEA, 0xD7},
		3, {0xE6, 0x84, 0xAB},
		2, {0x2B, 0x61},
		3, {0x8F, 0xBE, 0xC8}
	},
	{
		2, {0xEA, 0xD8},
		3, {0xE6, 0x84, 0xB0},
		2, {0x30, 0x61},
		3, {0x8F, 0xBE, 0xC9}
	},
	{
		2, {0xEA, 0xD9},
		3, {0xE6, 0x84, 0xB7},
		2, {0x37, 0x61},
		3, {0x8F, 0xBE, 0xCD}
	},
	{
		2, {0xEA, 0xDA},
		3, {0xEF, 0xA8, 0xBE},
		2, {0x3E, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEA, 0xDB},
		3, {0xE6, 0x86, 0x8D},
		2, {0x8D, 0x61},
		3, {0x8F, 0xBE, 0xE0}
	},
	{
		2, {0xEA, 0xDC},
		3, {0xEF, 0xA8, 0xBF},
		2, {0x3F, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEA, 0xDD},
		3, {0xE6, 0x86, 0xBC},
		2, {0xBC, 0x61},
		3, {0x8F, 0xBE, 0xEF}
	},
	{
		2, {0xEA, 0xDE},
		3, {0xE6, 0x86, 0xB9},
		2, {0xB9, 0x61},
		3, {0x8F, 0xBE, 0xEE}
	},
	{
		2, {0xEA, 0xDF},
		3, {0xEF, 0xA9, 0x80},
		2, {0x40, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEA, 0xE0},
		3, {0xE6, 0x88, 0xA2},
		2, {0x22, 0x62},
		3, {0x8F, 0xBF, 0xAC}
	},
	{
		2, {0xEA, 0xE1},
		3, {0xE6, 0x88, 0xBE},
		2, {0x3E, 0x62},
		0, {0x00}
	},
	{
		2, {0xEA, 0xE2},
		3, {0xE6, 0x89, 0x83},
		2, {0x43, 0x62},
		3, {0x8F, 0xBF, 0xB4}
	},
	{
		2, {0xEA, 0xE3},
		3, {0xE6, 0x89, 0x96},
		2, {0x56, 0x62},
		3, {0x8F, 0xBF, 0xBC}
	},
	{
		2, {0xEA, 0xE4},
		3, {0xE6, 0x89, 0x9A},
		2, {0x5A, 0x62},
		3, {0x8F, 0xBF, 0xBD}
	},
	{
		2, {0xEA, 0xE5},
		3, {0xE6, 0x89, 0xAF},
		2, {0x6F, 0x62},
		3, {0x8F, 0xBF, 0xC1}
	},
	{
		2, {0xEA, 0xE6},
		3, {0xE6, 0x8A, 0x85},
		2, {0x85, 0x62},
		0, {0x00}
	},
	{
		2, {0xEA, 0xE7},
		3, {0xE6, 0x8B, 0x84},
		2, {0xC4, 0x62},
		3, {0x8F, 0xBF, 0xD1}
	},
	{
		2, {0xEA, 0xE8},
		3, {0xE6, 0x8B, 0x96},
		2, {0xD6, 0x62},
		3, {0x8F, 0xBF, 0xD4}
	},
	{
		2, {0xEA, 0xE9},
		3, {0xE6, 0x8B, 0xBC},
		2, {0xFC, 0x62},
		3, {0x8F, 0xBF, 0xD9}
	},
	{
		2, {0xEA, 0xEA},
		3, {0xE6, 0x8C, 0x8A},
		2, {0x0A, 0x63},
		3, {0x8F, 0xBF, 0xDD}
	},
	{
		2, {0xEA, 0xEB},
		3, {0xE6, 0x8C, 0x98},
		2, {0x18, 0x63},
		3, {0x8F, 0xBF, 0xE3}
	},
	{
		2, {0xEA, 0xEC},
		3, {0xE6, 0x8C, 0xB9},
		2, {0x39, 0x63},
		3, {0x8F, 0xBF, 0xE9}
	},
	{
		2, {0xEA, 0xED},
		3, {0xE6, 0x8D, 0x83},
		2, {0x43, 0x63},
		3, {0x8F, 0xBF, 0xED}
	},
	{
		2, {0xEA, 0xEE},
		3, {0xE6, 0x8D, 0xA5},
		2, {0x65, 0x63},
		3, {0x8F, 0xBF, 0xF8}
	},
	{
		2, {0xEA, 0xEF},
		3, {0xE6, 0x8D, 0xBC},
		2, {0x7C, 0x63},
		3, {0x8F, 0xC0, 0xA2}
	},
	{
		2, {0xEA, 0xF0},
		3, {0xE6, 0x8F, 0xA5},
		2, {0xE5, 0x63},
		3, {0x8F, 0xC0, 0xBF}
	},
	{
		2, {0xEA, 0xF1},
		3, {0xE6, 0x8F, 0xAD},
		2, {0xED, 0x63},
		0, {0x00}
	},
	{
		2, {0xEA, 0xF2},
		3, {0xE6, 0x8F, 0xB5},
		2, {0xF5, 0x63},
		3, {0x8F, 0xC0, 0xC4}
	},
	{
		2, {0xEA, 0xF3},
		3, {0xE6, 0x90, 0x90},
		2, {0x10, 0x64},
		3, {0x8F, 0xC0, 0xC9}
	},
	{
		2, {0xEA, 0xF4},
		3, {0xE6, 0x90, 0x94},
		2, {0x14, 0x64},
		3, {0x8F, 0xC0, 0xCB}
	},
	{
		2, {0xEA, 0xF5},
		3, {0xE6, 0x90, 0xA2},
		2, {0x22, 0x64},
		3, {0x8F, 0xC0, 0xCF}
	},
	{
		2, {0xEA, 0xF6},
		3, {0xE6, 0x91, 0xB9},
		2, {0x79, 0x64},
		0, {0x00}
	},
	{
		2, {0xEA, 0xF7},
		3, {0xE6, 0x91, 0x91},
		2, {0x51, 0x64},
		3, {0x8F, 0xC0, 0xDB}
	},
	{
		2, {0xEA, 0xF8},
		3, {0xE6, 0x91, 0xA0},
		2, {0x60, 0x64},
		3, {0x8F, 0xC0, 0xE4}
	},
	{
		2, {0xEA, 0xF9},
		3, {0xE6, 0x91, 0xAD},
		2, {0x6D, 0x64},
		3, {0x8F, 0xC0, 0xE7}
	},
	{
		2, {0xEA, 0xFA},
		3, {0xE6, 0x93, 0x8E},
		2, {0xCE, 0x64},
		3, {0x8F, 0xC1, 0xA6}
	},
	{
		2, {0xEA, 0xFB},
		3, {0xE6, 0x92, 0xBE},
		2, {0xBE, 0x64},
		3, {0x8F, 0xC0, 0xFD}
	},
	{
		2, {0xEA, 0xFC},
		3, {0xE6, 0x92, 0xBF},
		2, {0xBF, 0x64},
		3, {0x8F, 0xC0, 0xFE}
	},
	{
		2, {0xEB, 0x40},
		3, {0xE6, 0x93, 0x84},
		2, {0xC4, 0x64},
		3, {0x8F, 0xC1, 0xA1}
	},
	{
		2, {0xEB, 0x41},
		3, {0xE6, 0x93, 0x8A},
		2, {0xCA, 0x64},
		3, {0x8F, 0xC1, 0xA3}
	},
	{
		2, {0xEB, 0x42},
		3, {0xE6, 0x93, 0x90},
		2, {0xD0, 0x64},
		3, {0x8F, 0xC1, 0xA7}
	},
	{
		2, {0xEB, 0x43},
		3, {0xE6, 0x93, 0xB7},
		2, {0xF7, 0x64},
		3, {0x8F, 0xC1, 0xB2}
	},
	{
		2, {0xEB, 0x44},
		3, {0xE6, 0x93, 0xBB},
		2, {0xFB, 0x64},
		3, {0x8F, 0xC1, 0xB3}
	},
	{
		2, {0xEB, 0x45},
		3, {0xE6, 0x94, 0xA2},
		2, {0x22, 0x65},
		3, {0x8F, 0xC1, 0xC2}
	},
	{
		2, {0xEB, 0x46},
		3, {0xE6, 0x94, 0xA9},
		2, {0x29, 0x65},
		3, {0x8F, 0xC1, 0xC4}
	},
	{
		2, {0xEB, 0x47},
		3, {0xEF, 0xA9, 0x81},
		2, {0x41, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEB, 0x48},
		3, {0xE6, 0x95, 0xA7},
		2, {0x67, 0x65},
		3, {0x8F, 0xC1, 0xD2}
	},
	{
		2, {0xEB, 0x49},
		3, {0xE6, 0x96, 0x9D},
		2, {0x9D, 0x65},
		3, {0x8F, 0xC1, 0xDC}
	},
	{
		2, {0xEB, 0x4A},
		3, {0xEF, 0xA9, 0x82},
		2, {0x42, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEB, 0x4B},
		3, {0xE6, 0x98, 0x80},
		2, {0x00, 0x66},
		3, {0x8F, 0xC1, 0xF5}
	},
	{
		2, {0xEB, 0x4C},
		3, {0xE6, 0x98, 0x89},
		2, {0x09, 0x66},
		3, {0x8F, 0xC1, 0xF8}
	},
	{
		2, {0xEB, 0x4D},
		3, {0xE6, 0x98, 0x95},
		2, {0x15, 0x66},
		3, {0x8F, 0xC1, 0xFC}
	},
	{
		2, {0xEB, 0x4E},
		3, {0xE6, 0x98, 0x9E},
		2, {0x1E, 0x66},
		3, {0x8F, 0xC2, 0xA1}
	},
	{
		2, {0xEB, 0x4F},
		3, {0xE6, 0x98, 0xBA},
		2, {0x3A, 0x66},
		0, {0x00}
	},
	{
		2, {0xEB, 0x50},
		3, {0xE6, 0x98, 0xA2},
		2, {0x22, 0x66},
		3, {0x8F, 0xC2, 0xA3}
	},
	{
		2, {0xEB, 0x51},
		3, {0xE6, 0x98, 0xA4},
		2, {0x24, 0x66},
		3, {0x8F, 0xC2, 0xA5}
	},
	{
		2, {0xEB, 0x52},
		3, {0xE6, 0x98, 0xAB},
		2, {0x2B, 0x66},
		3, {0x8F, 0xC2, 0xA9}
	},
	{
		2, {0xEB, 0x53},
		3, {0xE6, 0x98, 0xB0},
		2, {0x30, 0x66},
		3, {0x8F, 0xC2, 0xAC}
	},
	{
		2, {0xEB, 0x54},
		3, {0xE6, 0x98, 0xB1},
		2, {0x31, 0x66},
		3, {0x8F, 0xC2, 0xAD}
	},
	{
		2, {0xEB, 0x55},
		3, {0xE6, 0x98, 0xB3},
		2, {0x33, 0x66},
		3, {0x8F, 0xC2, 0xAE}
	},
	{
		2, {0xEB, 0x56},
		3, {0xE6, 0x9B, 0xBB},
		2, {0xFB, 0x66},
		3, {0x8F, 0xC2, 0xBF}
	},
	{
		2, {0xEB, 0x57},
		3, {0xE6, 0x99, 0x88},
		2, {0x48, 0x66},
		0, {0x00}
	},
	{
		2, {0xEB, 0x58},
		3, {0xE6, 0x99, 0x8C},
		2, {0x4C, 0x66},
		3, {0x8F, 0xC2, 0xB5}
	},
	{
		2, {0xEB, 0x59},
		4, {0xF0, 0xA3, 0x87, 0x84},
		4, {0x4C, 0xD8, 0xC4, 0xDD},
		0, {0x00}
	},
	{
		2, {0xEB, 0x5A},
		3, {0xE6, 0x99, 0x99},
		2, {0x59, 0x66},
		3, {0x8F, 0xC2, 0xBA}
	},
	{
		2, {0xEB, 0x5B},
		3, {0xE6, 0x99, 0x9A},
		2, {0x5A, 0x66},
		0, {0x00}
	},
	{
		2, {0xEB, 0x5C},
		3, {0xE6, 0x99, 0xA1},
		2, {0x61, 0x66},
		3, {0x8F, 0xC2, 0xBE}
	},
	{
		2, {0xEB, 0x5D},
		3, {0xE6, 0x99, 0xA5},
		2, {0x65, 0x66},
		0, {0x00}
	},
	{
		2, {0xEB, 0x5E},
		3, {0xE6, 0x99, 0xB3},
		2, {0x73, 0x66},
		3, {0x8F, 0xC2, 0xC4}
	},
	{
		2, {0xEB, 0x5F},
		3, {0xE6, 0x99, 0xB7},
		2, {0x77, 0x66},
		3, {0x8F, 0xC2, 0xC7}
	},
	{
		2, {0xEB, 0x60},
		3, {0xE6, 0x99, 0xB8},
		2, {0x78, 0x66},
		3, {0x8F, 0xC2, 0xC8}
	},
	{
		2, {0xEB, 0x61},
		3, {0xE6, 0x9A, 0x8D},
		2, {0x8D, 0x66},
		3, {0x8F, 0xC2, 0xCF}
	},
	{
		2, {0xEB, 0x62},
		3, {0xEF, 0xA9, 0x83},
		2, {0x43, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEB, 0x63},
		3, {0xE6, 0x9A, 0xA0},
		2, {0xA0, 0x66},
		3, {0x8F, 0xC2, 0xD7}
	},
	{
		2, {0xEB, 0x64},
		3, {0xE6, 0x9A, 0xB2},
		2, {0xB2, 0x66},
		3, {0x8F, 0xC2, 0xDB}
	},
	{
		2, {0xEB, 0x65},
		3, {0xE6, 0x9A, 0xBB},
		2, {0xBB, 0x66},
		3, {0x8F, 0xC2, 0xDD}
	},
	{
		2, {0xEB, 0x66},
		3, {0xE6, 0x9B, 0x86},
		2, {0xC6, 0x66},
		0, {0x00}
	},
	{
		2, {0xEB, 0x67},
		3, {0xE6, 0x9B, 0x88},
		2, {0xC8, 0x66},
		3, {0x8F, 0xC2, 0xE2}
	},
	{
		2, {0xEB, 0x68},
		3, {0xE3, 0xAC, 0xA2},
		2, {0x22, 0x3B},
		0, {0x00}
	},
	{
		2, {0xEB, 0x69},
		3, {0xE6, 0x9B, 0x9B},
		2, {0xDB, 0x66},
		3, {0x8F, 0xC2, 0xE7}
	},
	{
		2, {0xEB, 0x6A},
		3, {0xE6, 0x9B, 0xA8},
		2, {0xE8, 0x66},
		3, {0x8F, 0xC2, 0xE9}
	},
	{
		2, {0xEB, 0x6B},
		3, {0xE6, 0x9B, 0xBA},
		2, {0xFA, 0x66},
		3, {0x8F, 0xC2, 0xED}
	},
	{
		2, {0xEB, 0x6C},
		3, {0xE6, 0x9C, 0x93},
		2, {0x13, 0x67},
		3, {0x8F, 0xC2, 0xF1}
	},
	{
		2, {0xEB, 0x6D},
		3, {0xEF, 0xA4, 0xA9},
		2, {0x29, 0xF9},
		0, {0x00}
	},
	{
		2, {0xEB, 0x6E},
		3, {0xE6, 0x9C, 0xB3},
		2, {0x33, 0x67},
		3, {0x8F, 0xC2, 0xF6}
	},
	{
		2, {0xEB, 0x6F},
		3, {0xE6, 0x9D, 0xA6},
		2, {0x66, 0x67},
		3, {0x8F, 0xC3, 0xA1}
	},
	{
		2, {0xEB, 0x70},
		3, {0xE6, 0x9D, 0x87},
		2, {0x47, 0x67},
		3, {0x8F, 0xC2, 0xF9}
	},
	{
		2, {0xEB, 0x71},
		3, {0xE6, 0x9D, 0x88},
		2, {0x48, 0x67},
		3, {0x8F, 0xC2, 0xFA}
	},
	{
		2, {0xEB, 0x72},
		3, {0xE6, 0x9D, 0xBB},
		2, {0x7B, 0x67},
		3, {0x8F, 0xC3, 0xA6}
	},
	{
		2, {0xEB, 0x73},
		3, {0xE6, 0x9E, 0x81},
		2, {0x81, 0x67},
		3, {0x8F, 0xC3, 0xA7}
	},
	{
		2, {0xEB, 0x74},
		3, {0xE6, 0x9E, 0x93},
		2, {0x93, 0x67},
		3, {0x8F, 0xC3, 0xAC}
	},
	{
		2, {0xEB, 0x75},
		3, {0xE6, 0x9E, 0x98},
		2, {0x98, 0x67},
		3, {0x8F, 0xC3, 0xAE}
	},
	{
		2, {0xEB, 0x76},
		3, {0xE6, 0x9E, 0x9B},
		2, {0x9B, 0x67},
		3, {0x8F, 0xC3, 0xB0}
	},
	{
		2, {0xEB, 0x77},
		3, {0xE6, 0x9E, 0xBB},
		2, {0xBB, 0x67},
		3, {0x8F, 0xC3, 0xB5}
	},
	{
		2, {0xEB, 0x78},
		3, {0xE6, 0x9F, 0xB9},
		2, {0xF9, 0x67},
		3, {0x8F, 0xC3, 0xB8}
	},
	{
		2, {0xEB, 0x79},
		3, {0xE6, 0x9F, 0x80},
		2, {0xC0, 0x67},
		3, {0x8F, 0xC3, 0xB9}
	},
	{
		2, {0xEB, 0x7A},
		3, {0xE6, 0x9F, 0x97},
		2, {0xD7, 0x67},
		3, {0x8F, 0xC3, 0xC0}
	},
	{
		2, {0xEB, 0x7B},
		3, {0xE6, 0x9F, 0xBC},
		2, {0xFC, 0x67},
		0, {0x00}
	},
	{
		2, {0xEB, 0x7C},
		3, {0xE6, 0xA0, 0x81},
		2, {0x01, 0x68},
		0, {0x00}
	},
	{
		2, {0xEB, 0x7D},
		3, {0xE6, 0xA1, 0x92},
		2, {0x52, 0x68},
		3, {0x8F, 0xC3, 0xC9}
	},
	{
		2, {0xEB, 0x7E},
		3, {0xE6, 0xA0, 0x9D},
		2, {0x1D, 0x68},
		3, {0x8F, 0xC3, 0xCC}
	},
	{
		2, {0xEB, 0x80},
		3, {0xE6, 0xA0, 0xAC},
		2, {0x2C, 0x68},
		3, {0x8F, 0xC3, 0xD0}
	},
	{
		2, {0xEB, 0x81},
		3, {0xE6, 0xA0, 0xB1},
		2, {0x31, 0x68},
		3, {0x8F, 0xC3, 0xD4}
	},
	{
		2, {0xEB, 0x82},
		3, {0xE6, 0xA1, 0x9B},
		2, {0x5B, 0x68},
		3, {0x8F, 0xC3, 0xDF}
	},
	{
		2, {0xEB, 0x83},
		3, {0xE6, 0xA1, 0xB2},
		2, {0x72, 0x68},
		3, {0x8F, 0xC3, 0xE5}
	},
	{
		2, {0xEB, 0x84},
		3, {0xE6, 0xA1, 0xB5},
		2, {0x75, 0x68},
		3, {0x8F, 0xC3, 0xE6}
	},
	{
		2, {0xEB, 0x85},
		3, {0xEF, 0xA9, 0x84},
		2, {0x44, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEB, 0x86},
		3, {0xE6, 0xA2, 0xA3},
		2, {0xA3, 0x68},
		3, {0x8F, 0xC3, 0xF4}
	},
	{
		2, {0xEB, 0x87},
		3, {0xE6, 0xA2, 0xA5},
		2, {0xA5, 0x68},
		3, {0x8F, 0xC3, 0xF5}
	},
	{
		2, {0xEB, 0x88},
		3, {0xE6, 0xA2, 0xB2},
		2, {0xB2, 0x68},
		3, {0x8F, 0xC3, 0xF9}
	},
	{
		2, {0xEB, 0x89},
		3, {0xE6, 0xA3, 0x88},
		2, {0xC8, 0x68},
		3, {0x8F, 0xC3, 0xFC}
	},
	{
		2, {0xEB, 0x8A},
		3, {0xE6, 0xA3, 0x90},
		2, {0xD0, 0x68},
		3, {0x8F, 0xC4, 0xA1}
	},
	{
		2, {0xEB, 0x8B},
		3, {0xE6, 0xA3, 0xA8},
		2, {0xE8, 0x68},
		3, {0x8F, 0xC4, 0xA9}
	},
	{
		2, {0xEB, 0x8C},
		3, {0xE6, 0xA3, 0xAD},
		2, {0xED, 0x68},
		3, {0x8F, 0xC4, 0xAD}
	},
	{
		2, {0xEB, 0x8D},
		3, {0xE6, 0xA3, 0xB0},
		2, {0xF0, 0x68},
		3, {0x8F, 0xC4, 0xAE}
	},
	{
		2, {0xEB, 0x8E},
		3, {0xE6, 0xA3, 0xB1},
		2, {0xF1, 0x68},
		3, {0x8F, 0xC4, 0xAF}
	},
	{
		2, {0xEB, 0x8F},
		3, {0xE6, 0xA3, 0xBC},
		2, {0xFC, 0x68},
		3, {0x8F, 0xC4, 0xB3}
	},
	{
		2, {0xEB, 0x90},
		3, {0xE6, 0xA4, 0x8A},
		2, {0x0A, 0x69},
		3, {0x8F, 0xC4, 0xB7}
	},
	{
		2, {0xEB, 0x91},
		3, {0xE6, 0xA5, 0x89},
		2, {0x49, 0x69},
		3, {0x8F, 0xC4, 0xC4}
	},
	{
		2, {0xEB, 0x92},
		4, {0xF0, 0xA3, 0x97, 0x84},
		4, {0x4D, 0xD8, 0xC4, 0xDD},
		0, {0x00}
	},
	{
		2, {0xEB, 0x93},
		3, {0xE6, 0xA4, 0xB5},
		2, {0x35, 0x69},
		3, {0x8F, 0xC4, 0xBF}
	},
	{
		2, {0xEB, 0x94},
		3, {0xE6, 0xA5, 0x82},
		2, {0x42, 0x69},
		3, {0x8F, 0xC4, 0xC2}
	},
	{
		2, {0xEB, 0x95},
		3, {0xE6, 0xA5, 0x97},
		2, {0x57, 0x69},
		3, {0x8F, 0xC4, 0xC6}
	},
	{
		2, {0xEB, 0x96},
		3, {0xE6, 0xA5, 0xA3},
		2, {0x63, 0x69},
		3, {0x8F, 0xC4, 0xC8}
	},
	{
		2, {0xEB, 0x97},
		3, {0xE6, 0xA5, 0xA4},
		2, {0x64, 0x69},
		3, {0x8F, 0xC4, 0xC9}
	},
	{
		2, {0xEB, 0x98},
		3, {0xE6, 0xA5, 0xA8},
		2, {0x68, 0x69},
		3, {0x8F, 0xC4, 0xCC}
	},
	{
		2, {0xEB, 0x99},
		3, {0xE6, 0xA6, 0x80},
		2, {0x80, 0x69},
		3, {0x8F, 0xC4, 0xD5}
	},
	{
		2, {0xEB, 0x9A},
		3, {0xEF, 0xA8, 0x94},
		2, {0x14, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEB, 0x9B},
		3, {0xE6, 0xA6, 0xA5},
		2, {0xA5, 0x69},
		3, {0x8F, 0xC4, 0xDB}
	},
	{
		2, {0xEB, 0x9C},
		3, {0xE6, 0xA6, 0xAD},
		2, {0xAD, 0x69},
		3, {0x8F, 0xC4, 0xDF}
	},
	{
		2, {0xEB, 0x9D},
		3, {0xE6, 0xA7, 0x8F},
		2, {0xCF, 0x69},
		0, {0x00}
	},
	{
		2, {0xEB, 0x9E},
		3, {0xE3, 0xAE, 0xB6},
		2, {0xB6, 0x3B},
		0, {0x00}
	},
	{
		2, {0xEB, 0x9F},
		3, {0xE3, 0xAF, 0x83},
		2, {0xC3, 0x3B},
		0, {0x00}
	},
	{
		2, {0xEB, 0xA0},
		3, {0xE6, 0xA7, 0xA2},
		2, {0xE2, 0x69},
		3, {0x8F, 0xC4, 0xEA}
	},
	{
		2, {0xEB, 0xA1},
		3, {0xE6, 0xA7, 0xA9},
		2, {0xE9, 0x69},
		0, {0x00}
	},
	{
		2, {0xEB, 0xA2},
		3, {0xE6, 0xA7, 0xAA},
		2, {0xEA, 0x69},
		0, {0x00}
	},
	{
		2, {0xEB, 0xA3},
		3, {0xE6, 0xA7, 0xB5},
		2, {0xF5, 0x69},
		3, {0x8F, 0xC4, 0xF0}
	},
	{
		2, {0xEB, 0xA4},
		3, {0xE6, 0xA7, 0xB6},
		2, {0xF6, 0x69},
		0, {0x00}
	},
	{
		2, {0xEB, 0xA5},
		3, {0xE6, 0xA8, 0x8F},
		2, {0x0F, 0x6A},
		3, {0x8F, 0xC4, 0xF5}
	},
	{
		2, {0xEB, 0xA6},
		3, {0xE6, 0xA8, 0x95},
		2, {0x15, 0x6A},
		3, {0x8F, 0xC4, 0xF7}
	},
	{
		2, {0xEB, 0xA7},
		4, {0xF0, 0xA3, 0x9C, 0xBF},
		4, {0x4D, 0xD8, 0x3F, 0xDF},
		0, {0x00}
	},
	{
		2, {0xEB, 0xA8},
		3, {0xE6, 0xA8, 0xBB},
		2, {0x3B, 0x6A},
		3, {0x8F, 0xC5, 0xA3}
	},
	{
		2, {0xEB, 0xA9},
		3, {0xE6, 0xA8, 0xBE},
		2, {0x3E, 0x6A},
		3, {0x8F, 0xC5, 0xA4}
	},
	{
		2, {0xEB, 0xAA},
		3, {0xE6, 0xA9, 0x85},
		2, {0x45, 0x6A},
		3, {0x8F, 0xC5, 0xA6}
	},
	{
		2, {0xEB, 0xAB},
		3, {0xE6, 0xA9, 0x90},
		2, {0x50, 0x6A},
		3, {0x8F, 0xC5, 0xAB}
	},
	{
		2, {0xEB, 0xAC},
		3, {0xE6, 0xA9, 0x96},
		2, {0x56, 0x6A},
		3, {0x8F, 0xC5, 0xAF}
	},
	{
		2, {0xEB, 0xAD},
		3, {0xE6, 0xA9, 0x9B},
		2, {0x5B, 0x6A},
		3, {0x8F, 0xC5, 0xB0}
	},
	{
		2, {0xEB, 0xAE},
		3, {0xE6, 0xA9, 0xAB},
		2, {0x6B, 0x6A},
		0, {0x00}
	},
	{
		2, {0xEB, 0xAF},
		3, {0xE6, 0xA9, 0xB3},
		2, {0x73, 0x6A},
		3, {0x8F, 0xC5, 0xB5}
	},
	{
		2, {0xEB, 0xB0},
		4, {0xF0, 0xA3, 0x9D, 0xA3},
		4, {0x4D, 0xD8, 0x63, 0xDF},
		0, {0x00}
	},
	{
		2, {0xEB, 0xB1},
		3, {0xE6, 0xAA, 0x89},
		2, {0x89, 0x6A},
		3, {0x8F, 0xC5, 0xBB}
	},
	{
		2, {0xEB, 0xB2},
		3, {0xE6, 0xAA, 0x94},
		2, {0x94, 0x6A},
		0, {0x00}
	},
	{
		2, {0xEB, 0xB3},
		3, {0xE6, 0xAA, 0x9D},
		2, {0x9D, 0x6A},
		3, {0x8F, 0xC5, 0xBF}
	},
	{
		2, {0xEB, 0xB4},
		3, {0xE6, 0xAA, 0x9E},
		2, {0x9E, 0x6A},
		3, {0x8F, 0xC5, 0xC0}
	},
	{
		2, {0xEB, 0xB5},
		3, {0xE6, 0xAA, 0xA5},
		2, {0xA5, 0x6A},
		3, {0x8F, 0xC5, 0xC2}
	},
	{
		2, {0xEB, 0xB6},
		3, {0xE6, 0xAB, 0xA4},
		2, {0xE4, 0x6A},
		3, {0x8F, 0xC5, 0xD5}
	},
	{
		2, {0xEB, 0xB7},
		3, {0xE6, 0xAB, 0xA7},
		2, {0xE7, 0x6A},
		3, {0x8F, 0xC5, 0xD6}
	},
	{
		2, {0xEB, 0xB8},
		3, {0xE3, 0xB0, 0x8F},
		2, {0x0F, 0x3C},
		0, {0x00}
	},
	{
		2, {0xEB, 0xB9},
		3, {0xEF, 0xA4, 0x9D},
		2, {0x1D, 0xF9},
		0, {0x00}
	},
	{
		2, {0xEB, 0xBA},
		3, {0xE6, 0xAC, 0x9B},
		2, {0x1B, 0x6B},
		3, {0x8F, 0xC5, 0xE6}
	},
	{
		2, {0xEB, 0xBB},
		3, {0xE6, 0xAC, 0x9E},
		2, {0x1E, 0x6B},
		3, {0x8F, 0xC5, 0xE7}
	},
	{
		2, {0xEB, 0xBC},
		3, {0xE6, 0xAC, 0xAC},
		2, {0x2C, 0x6B},
		3, {0x8F, 0xC5, 0xEB}
	},
	{
		2, {0xEB, 0xBD},
		3, {0xE6, 0xAC, 0xB5},
		2, {0x35, 0x6B},
		3, {0x8F, 0xC5, 0xED}
	},
	{
		2, {0xEB, 0xBE},
		3, {0xE6, 0xAD, 0x86},
		2, {0x46, 0x6B},
		3, {0x8F, 0xC5, 0xF1}
	},
	{
		2, {0xEB, 0xBF},
		3, {0xE6, 0xAD, 0x96},
		2, {0x56, 0x6B},
		3, {0x8F, 0xC5, 0xF5}
	},
	{
		2, {0xEB, 0xC0},
		3, {0xE6, 0xAD, 0xA0},
		2, {0x60, 0x6B},
		3, {0x8F, 0xC5, 0xF8}
	},
	{
		2, {0xEB, 0xC1},
		3, {0xE6, 0xAD, 0xA5},
		2, {0x65, 0x6B},
		0, {0x00}
	},
	{
		2, {0xEB, 0xC2},
		3, {0xE6, 0xAD, 0xA7},
		2, {0x67, 0x6B},
		3, {0x8F, 0xC5, 0xF9}
	},
	{
		2, {0xEB, 0xC3},
		3, {0xE6, 0xAD, 0xB7},
		2, {0x77, 0x6B},
		0, {0x00}
	},
	{
		2, {0xEB, 0xC4},
		3, {0xE6, 0xAE, 0x82},
		2, {0x82, 0x6B},
		3, {0x8F, 0xC6, 0xA2}
	},
	{
		2, {0xEB, 0xC5},
		3, {0xE6, 0xAE, 0xA9},
		2, {0xA9, 0x6B},
		3, {0x8F, 0xC6, 0xAB}
	},
	{
		2, {0xEB, 0xC6},
		3, {0xE6, 0xAE, 0xAD},
		2, {0xAD, 0x6B},
		3, {0x8F, 0xC6, 0xAD}
	},
	{
		2, {0xEB, 0xC7},
		3, {0xEF, 0xA5, 0xB0},
		2, {0x70, 0xF9},
		0, {0x00}
	},
	{
		2, {0xEB, 0xC8},
		3, {0xE6, 0xAF, 0x8F},
		2, {0xCF, 0x6B},
		0, {0x00}
	},
	{
		2, {0xEB, 0xC9},
		3, {0xE6, 0xAF, 0x96},
		2, {0xD6, 0x6B},
		3, {0x8F, 0xC6, 0xB8}
	},
	{
		2, {0xEB, 0xCA},
		3, {0xE6, 0xAF, 0x97},
		2, {0xD7, 0x6B},
		0, {0x00}
	},
	{
		2, {0xEB, 0xCB},
		3, {0xE6, 0xAF, 0xBF},
		2, {0xFF, 0x6B},
		3, {0x8F, 0xC6, 0xC2}
	},
	{
		2, {0xEB, 0xCC},
		3, {0xE6, 0xB0, 0x85},
		2, {0x05, 0x6C},
		3, {0x8F, 0xC6, 0xC5}
	},
	{
		2, {0xEB, 0xCD},
		3, {0xE6, 0xB0, 0x90},
		2, {0x10, 0x6C},
		3, {0x8F, 0xC6, 0xC9}
	},
	{
		2, {0xEB, 0xCE},
		3, {0xE6, 0xB0, 0xB3},
		2, {0x33, 0x6C},
		3, {0x8F, 0xC6, 0xD2}
	},
	{
		2, {0xEB, 0xCF},
		3, {0xE6, 0xB1, 0x99},
		2, {0x59, 0x6C},
		3, {0x8F, 0xC6, 0xDE}
	},
	{
		2, {0xEB, 0xD0},
		3, {0xE6, 0xB1, 0x9C},
		2, {0x5C, 0x6C},
		3, {0x8F, 0xC6, 0xE0}
	},
	{
		2, {0xEB, 0xD1},
		3, {0xE6, 0xB2, 0xAA},
		2, {0xAA, 0x6C},
		0, {0x00}
	},
	{
		2, {0xEB, 0xD2},
		3, {0xE6, 0xB1, 0xB4},
		2, {0x74, 0x6C},
		3, {0x8F, 0xC6, 0xE4}
	},
	{
		2, {0xEB, 0xD3},
		3, {0xE6, 0xB1, 0xB6},
		2, {0x76, 0x6C},
		3, {0x8F, 0xC6, 0xE5}
	},
	{
		2, {0xEB, 0xD4},
		3, {0xE6, 0xB2, 0x85},
		2, {0x85, 0x6C},
		3, {0x8F, 0xC6, 0xE9}
	},
	{
		2, {0xEB, 0xD5},
		3, {0xE6, 0xB2, 0x86},
		2, {0x86, 0x6C},
		3, {0x8F, 0xC6, 0xEA}
	},
	{
		2, {0xEB, 0xD6},
		3, {0xE6, 0xB2, 0x98},
		2, {0x98, 0x6C},
		3, {0x8F, 0xC6, 0xF0}
	},
	{
		2, {0xEB, 0xD7},
		3, {0xE6, 0xB2, 0x9C},
		2, {0x9C, 0x6C},
		3, {0x8F, 0xC6, 0xF1}
	},
	{
		2, {0xEB, 0xD8},
		3, {0xE6, 0xB3, 0xBB},
		2, {0xFB, 0x6C},
		0, {0x00}
	},
	{
		2, {0xEB, 0xD9},
		3, {0xE6, 0xB3, 0x86},
		2, {0xC6, 0x6C},
		3, {0x8F, 0xC6, 0xF7}
	},
	{
		2, {0xEB, 0xDA},
		3, {0xE6, 0xB3, 0x94},
		2, {0xD4, 0x6C},
		3, {0x8F, 0xC6, 0xFD}
	},
	{
		2, {0xEB, 0xDB},
		3, {0xE6, 0xB3, 0xA0},
		2, {0xE0, 0x6C},
		3, {0x8F, 0xC7, 0xA3}
	},
	{
		2, {0xEB, 0xDC},
		3, {0xE6, 0xB3, 0xAB},
		2, {0xEB, 0x6C},
		3, {0x8F, 0xC7, 0xA6}
	},
	{
		2, {0xEB, 0xDD},
		3, {0xE6, 0xB3, 0xAE},
		2, {0xEE, 0x6C},
		3, {0x8F, 0xC7, 0xA8}
	},
	{
		2, {0xEB, 0xDE},
		4, {0xF0, 0xA3, 0xB3, 0xBE},
		4, {0x4F, 0xD8, 0xFE, 0xDC},
		0, {0x00}
	},
	{
		2, {0xEB, 0xDF},
		3, {0xE6, 0xB4, 0x84},
		2, {0x04, 0x6D},
		3, {0x8F, 0xC7, 0xAB}
	},
	{
		2, {0xEB, 0xE0},
		3, {0xE6, 0xB4, 0x8E},
		2, {0x0E, 0x6D},
		3, {0x8F, 0xC7, 0xAE}
	},
	{
		2, {0xEB, 0xE1},
		3, {0xE6, 0xB4, 0xAE},
		2, {0x2E, 0x6D},
		3, {0x8F, 0xC7, 0xB7}
	},
	{
		2, {0xEB, 0xE2},
		3, {0xE6, 0xB4, 0xB1},
		2, {0x31, 0x6D},
		3, {0x8F, 0xC7, 0xB9}
	},
	{
		2, {0xEB, 0xE3},
		3, {0xE6, 0xB4, 0xB9},
		2, {0x39, 0x6D},
		3, {0x8F, 0xC7, 0xBA}
	},
	{
		2, {0xEB, 0xE4},
		3, {0xE6, 0xB4, 0xBF},
		2, {0x3F, 0x6D},
		3, {0x8F, 0xC7, 0xBC}
	},
	{
		2, {0xEB, 0xE5},
		3, {0xE6, 0xB5, 0x98},
		2, {0x58, 0x6D},
		0, {0x00}
	},
	{
		2, {0xEB, 0xE6},
		3, {0xE6, 0xB5, 0xA5},
		2, {0x65, 0x6D},
		3, {0x8F, 0xC7, 0xC1}
	},
	{
		2, {0xEB, 0xE7},
		3, {0xEF, 0xA9, 0x85},
		2, {0x45, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEB, 0xE8},
		3, {0xE6, 0xB6, 0x82},
		2, {0x82, 0x6D},
		3, {0x8F, 0xC7, 0xC6}
	},
	{
		2, {0xEB, 0xE9},
		3, {0xE6, 0xB6, 0x87},
		2, {0x87, 0x6D},
		3, {0x8F, 0xC7, 0xC7}
	},
	{
		2, {0xEB, 0xEA},
		3, {0xE6, 0xB6, 0x89},
		2, {0x89, 0x6D},
		0, {0x00}
	},
	{
		2, {0xEB, 0xEB},
		3, {0xE6, 0xB6, 0x94},
		2, {0x94, 0x6D},
		3, {0x8F, 0xC7, 0xCA}
	},
	{
		2, {0xEB, 0xEC},
		3, {0xE6, 0xB6, 0xAA},
		2, {0xAA, 0x6D},
		3, {0x8F, 0xC7, 0xCE}
	},
	{
		2, {0xEB, 0xED},
		3, {0xE6, 0xB6, 0xAC},
		2, {0xAC, 0x6D},
		3, {0x8F, 0xC7, 0xCF}
	},
	{
		2, {0xEB, 0xEE},
		3, {0xE6, 0xB6, 0xBF},
		2, {0xBF, 0x6D},
		3, {0x8F, 0xC7, 0xD4}
	},
	{
		2, {0xEB, 0xEF},
		3, {0xE6, 0xB7, 0x84},
		2, {0xC4, 0x6D},
		3, {0x8F, 0xC7, 0xD5}
	},
	{
		2, {0xEB, 0xF0},
		3, {0xE6, 0xB7, 0x96},
		2, {0xD6, 0x6D},
		3, {0x8F, 0xC7, 0xDA}
	},
	{
		2, {0xEB, 0xF1},
		3, {0xE6, 0xB7, 0x9A},
		2, {0xDA, 0x6D},
		0, {0x00}
	},
	{
		2, {0xEB, 0xF2},
		3, {0xE6, 0xB7, 0x9B},
		2, {0xDB, 0x6D},
		3, {0x8F, 0xC7, 0xDB}
	},
	{
		2, {0xEB, 0xF3},
		3, {0xE6, 0xB7, 0x9D},
		2, {0xDD, 0x6D},
		3, {0x8F, 0xC7, 0xDC}
	},
	{
		2, {0xEB, 0xF4},
		3, {0xE6, 0xB7, 0xBC},
		2, {0xFC, 0x6D},
		3, {0x8F, 0xC7, 0xE6}
	},
	{
		2, {0xEB, 0xF5},
		3, {0xEF, 0xA9, 0x86},
		2, {0x46, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEB, 0xF6},
		3, {0xE6, 0xB8, 0xB4},
		2, {0x34, 0x6E},
		0, {0x00}
	},
	{
		2, {0xEB, 0xF7},
		3, {0xE6, 0xB9, 0x84},
		2, {0x44, 0x6E},
		3, {0x8F, 0xC7, 0xF1}
	},
	{
		2, {0xEB, 0xF8},
		3, {0xE6, 0xB9, 0x9C},
		2, {0x5C, 0x6E},
		3, {0x8F, 0xC7, 0xFC}
	},
	{
		2, {0xEB, 0xF9},
		3, {0xE6, 0xB9, 0x9E},
		2, {0x5E, 0x6E},
		3, {0x8F, 0xC7, 0xFE}
	},
	{
		2, {0xEB, 0xFA},
		3, {0xE6, 0xBA, 0xAB},
		2, {0xAB, 0x6E},
		0, {0x00}
	},
	{
		2, {0xEB, 0xFB},
		3, {0xE6, 0xBA, 0xB1},
		2, {0xB1, 0x6E},
		3, {0x8F, 0xC8, 0xAE}
	},
	{
		2, {0xEB, 0xFC},
		3, {0xE6, 0xBB, 0x81},
		2, {0xC1, 0x6E},
		3, {0x8F, 0xC8, 0xB3}
	},
	{
		2, {0xEC, 0x40},
		3, {0xE6, 0xBB, 0x87},
		2, {0xC7, 0x6E},
		3, {0x8F, 0xC8, 0xB5}
	},
	{
		2, {0xEC, 0x41},
		3, {0xE6, 0xBB, 0x8E},
		2, {0xCE, 0x6E},
		3, {0x8F, 0xC8, 0xB9}
	},
	{
		2, {0xEC, 0x42},
		3, {0xE6, 0xBC, 0x90},
		2, {0x10, 0x6F},
		0, {0x00}
	},
	{
		2, {0xEC, 0x43},
		3, {0xE6, 0xBC, 0x9A},
		2, {0x1A, 0x6F},
		3, {0x8F, 0xC8, 0xC8}
	},
	{
		2, {0xEC, 0x44},
		3, {0xEF, 0xA9, 0x87},
		2, {0x47, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEC, 0x45},
		3, {0xE6, 0xBC, 0xAA},
		2, {0x2A, 0x6F},
		3, {0x8F, 0xC8, 0xCC}
	},
	{
		2, {0xEC, 0x46},
		3, {0xE6, 0xBC, 0xAF},
		2, {0x2F, 0x6F},
		3, {0x8F, 0xC8, 0xCD}
	},
	{
		2, {0xEC, 0x47},
		3, {0xE6, 0xBC, 0xB3},
		2, {0x33, 0x6F},
		3, {0x8F, 0xC8, 0xCF}
	},
	{
		2, {0xEC, 0x48},
		3, {0xE6, 0xBD, 0x91},
		2, {0x51, 0x6F},
		3, {0x8F, 0xC8, 0xD5}
	},
	{
		2, {0xEC, 0x49},
		3, {0xE6, 0xBD, 0x99},
		2, {0x59, 0x6F},
		3, {0x8F, 0xC8, 0xD9}
	},
	{
		2, {0xEC, 0x4A},
		3, {0xE6, 0xBD, 0x9E},
		2, {0x5E, 0x6F},
		3, {0x8F, 0xC8, 0xDC}
	},
	{
		2, {0xEC, 0x4B},
		3, {0xE6, 0xBD, 0xA1},
		2, {0x61, 0x6F},
		3, {0x8F, 0xC8, 0xDD}
	},
	{
		2, {0xEC, 0x4C},
		3, {0xE6, 0xBD, 0xA2},
		2, {0x62, 0x6F},
		3, {0x8F, 0xC8, 0xDE}
	},
	{
		2, {0xEC, 0x4D},
		3, {0xE6, 0xBD, 0xBE},
		2, {0x7E, 0x6F},
		3, {0x8F, 0xC8, 0xE2}
	},
	{
		2, {0xEC, 0x4E},
		3, {0xE6, 0xBE, 0x88},
		2, {0x88, 0x6F},
		3, {0x8F, 0xC8, 0xE5}
	},
	{
		2, {0xEC, 0x4F},
		3, {0xE6, 0xBE, 0x8C},
		2, {0x8C, 0x6F},
		3, {0x8F, 0xC8, 0xE7}
	},
	{
		2, {0xEC, 0x50},
		3, {0xE6, 0xBE, 0x8D},
		2, {0x8D, 0x6F},
		3, {0x8F, 0xC8, 0xE8}
	},
	{
		2, {0xEC, 0x51},
		3, {0xE6, 0xBE, 0x94},
		2, {0x94, 0x6F},
		3, {0x8F, 0xC8, 0xEC}
	},
	{
		2, {0xEC, 0x52},
		3, {0xE6, 0xBE, 0xA0},
		2, {0xA0, 0x6F},
		3, {0x8F, 0xC8, 0xF0}
	},
	{
		2, {0xEC, 0x53},
		3, {0xE6, 0xBE, 0xA7},
		2, {0xA7, 0x6F},
		3, {0x8F, 0xC8, 0xF3}
	},
	{
		2, {0xEC, 0x54},
		3, {0xE6, 0xBE, 0xB6},
		2, {0xB6, 0x6F},
		3, {0x8F, 0xC8, 0xF9}
	},
	{
		2, {0xEC, 0x55},
		3, {0xE6, 0xBE, 0xBC},
		2, {0xBC, 0x6F},
		3, {0x8F, 0xC8, 0xFA}
	},
	{
		2, {0xEC, 0x56},
		3, {0xE6, 0xBF, 0x87},
		2, {0xC7, 0x6F},
		3, {0x8F, 0xC8, 0xFC}
	},
	{
		2, {0xEC, 0x57},
		3, {0xE6, 0xBF, 0x8A},
		2, {0xCA, 0x6F},
		3, {0x8F, 0xC8, 0xFE}
	},
	{
		2, {0xEC, 0x58},
		3, {0xE6, 0xBF, 0xB9},
		2, {0xF9, 0x6F},
		3, {0x8F, 0xC9, 0xA7}
	},
	{
		2, {0xEC, 0x59},
		3, {0xE6, 0xBF, 0xB0},
		2, {0xF0, 0x6F},
		3, {0x8F, 0xC9, 0xA5}
	},
	{
		2, {0xEC, 0x5A},
		3, {0xE6, 0xBF, 0xB5},
		2, {0xF5, 0x6F},
		3, {0x8F, 0xC9, 0xA6}
	},
	{
		2, {0xEC, 0x5B},
		3, {0xE7, 0x80, 0x85},
		2, {0x05, 0x70},
		3, {0x8F, 0xC9, 0xAB}
	},
	{
		2, {0xEC, 0x5C},
		3, {0xE7, 0x80, 0x86},
		2, {0x06, 0x70},
		3, {0x8F, 0xC9, 0xAC}
	},
	{
		2, {0xEC, 0x5D},
		3, {0xE7, 0x80, 0xA8},
		2, {0x28, 0x70},
		0, {0x00}
	},
	{
		2, {0xEC, 0x5E},
		3, {0xE7, 0x81, 0x8A},
		2, {0x4A, 0x70},
		3, {0x8F, 0xC9, 0xBB}
	},
	{
		2, {0xEC, 0x5F},
		3, {0xE7, 0x81, 0x9D},
		2, {0x5D, 0x70},
		3, {0x8F, 0xC9, 0xBF}
	},
	{
		2, {0xEC, 0x60},
		3, {0xE7, 0x81, 0x9E},
		2, {0x5E, 0x70},
		3, {0x8F, 0xC9, 0xC0}
	},
	{
		2, {0xEC, 0x61},
		3, {0xE7, 0x81, 0x8E},
		2, {0x4E, 0x70},
		3, {0x8F, 0xC9, 0xC1}
	},
	{
		2, {0xEC, 0x62},
		3, {0xE7, 0x81, 0xA4},
		2, {0x64, 0x70},
		3, {0x8F, 0xC9, 0xC2}
	},
	{
		2, {0xEC, 0x63},
		3, {0xE7, 0x81, 0xB5},
		2, {0x75, 0x70},
		3, {0x8F, 0xC9, 0xC6}
	},
	{
		2, {0xEC, 0x64},
		3, {0xE7, 0x82, 0x85},
		2, {0x85, 0x70},
		3, {0x8F, 0xC9, 0xCA}
	},
	{
		2, {0xEC, 0x65},
		3, {0xE7, 0x82, 0xA4},
		2, {0xA4, 0x70},
		3, {0x8F, 0xC9, 0xD2}
	},
	{
		2, {0xEC, 0x66},
		3, {0xE7, 0x82, 0xAB},
		2, {0xAB, 0x70},
		3, {0x8F, 0xC9, 0xD3}
	},
	{
		2, {0xEC, 0x67},
		3, {0xE7, 0x82, 0xB7},
		2, {0xB7, 0x70},
		3, {0x8F, 0xC9, 0xD7}
	},
	{
		2, {0xEC, 0x68},
		3, {0xE7, 0x83, 0x94},
		2, {0xD4, 0x70},
		3, {0x8F, 0xC9, 0xDB}
	},
	{
		2, {0xEC, 0x69},
		3, {0xE7, 0x83, 0x98},
		2, {0xD8, 0x70},
		3, {0x8F, 0xC9, 0xDE}
	},
	{
		2, {0xEC, 0x6A},
		3, {0xE7, 0x83, 0xA4},
		2, {0xE4, 0x70},
		3, {0x8F, 0xC9, 0xE0}
	},
	{
		2, {0xEC, 0x6B},
		3, {0xE7, 0x84, 0x8F},
		2, {0x0F, 0x71},
		3, {0x8F, 0xC9, 0xE9}
	},
	{
		2, {0xEC, 0x6C},
		3, {0xE7, 0x84, 0xAB},
		2, {0x2B, 0x71},
		3, {0x8F, 0xC9, 0xEC}
	},
	{
		2, {0xEC, 0x6D},
		3, {0xE7, 0x84, 0x9E},
		2, {0x1E, 0x71},
		3, {0x8F, 0xC9, 0xEA}
	},
	{
		2, {0xEC, 0x6E},
		3, {0xE7, 0x84, 0xA0},
		2, {0x20, 0x71},
		3, {0x8F, 0xC9, 0xEB}
	},
	{
		2, {0xEC, 0x6F},
		3, {0xE7, 0x84, 0xAE},
		2, {0x2E, 0x71},
		0, {0x00}
	},
	{
		2, {0xEC, 0x70},
		3, {0xE7, 0x84, 0xB0},
		2, {0x30, 0x71},
		3, {0x8F, 0xC9, 0xEF}
	},
	{
		2, {0xEC, 0x71},
		3, {0xE7, 0x85, 0x86},
		2, {0x46, 0x71},
		3, {0x8F, 0xC9, 0xF4}
	},
	{
		2, {0xEC, 0x72},
		3, {0xE7, 0x85, 0x87},
		2, {0x47, 0x71},
		3, {0x8F, 0xC9, 0xF5}
	},
	{
		2, {0xEC, 0x73},
		3, {0xE7, 0x85, 0x91},
		2, {0x51, 0x71},
		0, {0x00}
	},
	{
		2, {0xEC, 0x74},
		3, {0xEF, 0xA9, 0x88},
		2, {0x48, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEC, 0x75},
		3, {0xE7, 0x85, 0x92},
		2, {0x52, 0x71},
		3, {0x8F, 0xC9, 0xF9}
	},
	{
		2, {0xEC, 0x76},
		3, {0xE7, 0x85, 0x9C},
		2, {0x5C, 0x71},
		3, {0x8F, 0xC9, 0xFC}
	},
	{
		2, {0xEC, 0x77},
		3, {0xE7, 0x85, 0xA0},
		2, {0x60, 0x71},
		3, {0x8F, 0xC9, 0xFE}
	},
	{
		2, {0xEC, 0x78},
		3, {0xE7, 0x85, 0xA8},
		2, {0x68, 0x71},
		3, {0x8F, 0xCA, 0xA1}
	},
	{
		2, {0xEC, 0x79},
		3, {0xEF, 0xA8, 0x95},
		2, {0x15, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEC, 0x7A},
		3, {0xE7, 0x86, 0x85},
		2, {0x85, 0x71},
		3, {0x8F, 0xCA, 0xA4}
	},
	{
		2, {0xEC, 0x7B},
		3, {0xE7, 0x86, 0x87},
		2, {0x87, 0x71},
		3, {0x8F, 0xCA, 0xA5}
	},
	{
		2, {0xEC, 0x7C},
		3, {0xE7, 0x86, 0x92},
		2, {0x92, 0x71},
		3, {0x8F, 0xCA, 0xA7}
	},
	{
		2, {0xEC, 0x7D},
		3, {0xE7, 0x87, 0x81},
		2, {0xC1, 0x71},
		3, {0x8F, 0xCA, 0xB3}
	},
	{
		2, {0xEC, 0x7E},
		3, {0xE7, 0x86, 0xBA},
		2, {0xBA, 0x71},
		3, {0x8F, 0xCA, 0xB0}
	},
	{
		2, {0xEC, 0x80},
		3, {0xE7, 0x87, 0x84},
		2, {0xC4, 0x71},
		3, {0x8F, 0xCA, 0xB4}
	},
	{
		2, {0xEC, 0x81},
		3, {0xE7, 0x87, 0xBE},
		2, {0xFE, 0x71},
		3, {0x8F, 0xCA, 0xBD}
	},
	{
		2, {0xEC, 0x82},
		3, {0xE7, 0x88, 0x80},
		2, {0x00, 0x72},
		3, {0x8F, 0xCA, 0xBE}
	},
	{
		2, {0xEC, 0x83},
		3, {0xE7, 0x88, 0x95},
		2, {0x15, 0x72},
		0, {0x00}
	},
	{
		2, {0xEC, 0x84},
		3, {0xE7, 0x89, 0x95},
		2, {0x55, 0x72},
		3, {0x8F, 0xCA, 0xD5}
	},
	{
		2, {0xEC, 0x85},
		3, {0xE7, 0x89, 0x96},
		2, {0x56, 0x72},
		3, {0x8F, 0xCA, 0xD6}
	},
	{
		2, {0xEC, 0x86},
		3, {0xE3, 0xB8, 0xBF},
		2, {0x3F, 0x3E},
		0, {0x00}
	},
	{
		2, {0xEC, 0x87},
		3, {0xE7, 0x8A, 0x8D},
		2, {0x8D, 0x72},
		3, {0x8F, 0xCA, 0xE8}
	},
	{
		2, {0xEC, 0x88},
		3, {0xE7, 0x8A, 0x9B},
		2, {0x9B, 0x72},
		3, {0x8F, 0xCA, 0xEB}
	},
	{
		2, {0xEC, 0x89},
		3, {0xE7, 0x8A, 0xBE},
		2, {0xBE, 0x72},
		3, {0x8F, 0xCA, 0xF1}
	},
	{
		2, {0xEC, 0x8A},
		3, {0xE7, 0x8B, 0x80},
		2, {0xC0, 0x72},
		0, {0x00}
	},
	{
		2, {0xEC, 0x8B},
		3, {0xE7, 0x8B, 0xBB},
		2, {0xFB, 0x72},
		3, {0x8F, 0xCA, 0xFE}
	},
	{
		2, {0xEC, 0x8C},
		4, {0xF0, 0xA4, 0x9F, 0xB1},
		4, {0x51, 0xD8, 0xF1, 0xDF},
		0, {0x00}
	},
	{
		2, {0xEC, 0x8D},
		3, {0xE7, 0x8C, 0xA7},
		2, {0x27, 0x73},
		3, {0x8F, 0xCB, 0xAF}
	},
	{
		2, {0xEC, 0x8E},
		3, {0xE7, 0x8C, 0xA8},
		2, {0x28, 0x73},
		3, {0x8F, 0xCB, 0xB0}
	},
	{
		2, {0xEC, 0x8F},
		3, {0xEF, 0xA8, 0x96},
		2, {0x16, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEC, 0x90},
		3, {0xE7, 0x8D, 0x90},
		2, {0x50, 0x73},
		3, {0x8F, 0xCB, 0xBA}
	},
	{
		2, {0xEC, 0x91},
		3, {0xE7, 0x8D, 0xA6},
		2, {0x66, 0x73},
		3, {0x8F, 0xCB, 0xC2}
	},
	{
		2, {0xEC, 0x92},
		3, {0xE7, 0x8D, 0xBC},
		2, {0x7C, 0x73},
		3, {0x8F, 0xCB, 0xCC}
	},
	{
		2, {0xEC, 0x93},
		3, {0xE7, 0x8E, 0x95},
		2, {0x95, 0x73},
		3, {0x8F, 0xCB, 0xD5}
	},
	{
		2, {0xEC, 0x94},
		3, {0xE7, 0x8E, 0x9F},
		2, {0x9F, 0x73},
		3, {0x8F, 0xCB, 0xDA}
	},
	{
		2, {0xEC, 0x95},
		3, {0xE7, 0x8E, 0xA0},
		2, {0xA0, 0x73},
		3, {0x8F, 0xCB, 0xDB}
	},
	{
		2, {0xEC, 0x96},
		3, {0xE7, 0x8E, 0xA2},
		2, {0xA2, 0x73},
		3, {0x8F, 0xCB, 0xDC}
	},
	{
		2, {0xEC, 0x97},
		3, {0xE7, 0x8E, 0xA6},
		2, {0xA6, 0x73},
		3, {0x8F, 0xCB, 0xDE}
	},
	{
		2, {0xEC, 0x98},
		3, {0xE7, 0x8E, 0xAB},
		2, {0xAB, 0x73},
		3, {0x8F, 0xCB, 0xE0}
	},
	{
		2, {0xEC, 0x99},
		3, {0xE7, 0x8F, 0x89},
		2, {0xC9, 0x73},
		3, {0x8F, 0xCB, 0xEA}
	},
	{
		2, {0xEC, 0x9A},
		3, {0xE7, 0x8F, 0x8F},
		2, {0xCF, 0x73},
		3, {0x8F, 0xCB, 0xED}
	},
	{
		2, {0xEC, 0x9B},
		3, {0xE7, 0x8F, 0x96},
		2, {0xD6, 0x73},
		3, {0x8F, 0xCB, 0xF0}
	},
	{
		2, {0xEC, 0x9C},
		3, {0xE7, 0x8F, 0x99},
		2, {0xD9, 0x73},
		3, {0x8F, 0xCB, 0xF1}
	},
	{
		2, {0xEC, 0x9D},
		3, {0xE7, 0x8F, 0xA3},
		2, {0xE3, 0x73},
		3, {0x8F, 0xCB, 0xF4}
	},
	{
		2, {0xEC, 0x9E},
		3, {0xE7, 0x8F, 0xA9},
		2, {0xE9, 0x73},
		3, {0x8F, 0xCB, 0xF7}
	},
	{
		2, {0xEC, 0x9F},
		3, {0xE7, 0x90, 0x87},
		2, {0x07, 0x74},
		3, {0x8F, 0xCC, 0xA5}
	},
	{
		2, {0xEC, 0xA0},
		3, {0xE7, 0x90, 0x8A},
		2, {0x0A, 0x74},
		3, {0x8F, 0xCC, 0xA6}
	},
	{
		2, {0xEC, 0xA1},
		3, {0xE7, 0x90, 0x9A},
		2, {0x1A, 0x74},
		3, {0x8F, 0xCC, 0xA8}
	},
	{
		2, {0xEC, 0xA2},
		3, {0xE7, 0x90, 0x9B},
		2, {0x1B, 0x74},
		3, {0x8F, 0xCC, 0xA9}
	},
	{
		2, {0xEC, 0xA3},
		3, {0xEF, 0xA9, 0x8A},
		2, {0x4A, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEC, 0xA4},
		3, {0xE7, 0x90, 0xA6},
		2, {0x26, 0x74},
		3, {0x8F, 0xCC, 0xAB}
	},
	{
		2, {0xEC, 0xA5},
		3, {0xE7, 0x90, 0xA8},
		2, {0x28, 0x74},
		3, {0x8F, 0xCC, 0xAC}
	},
	{
		2, {0xEC, 0xA6},
		3, {0xE7, 0x90, 0xAA},
		2, {0x2A, 0x74},
		3, {0x8F, 0xCC, 0xAE}
	},
	{
		2, {0xEC, 0xA7},
		3, {0xE7, 0x90, 0xAB},
		2, {0x2B, 0x74},
		3, {0x8F, 0xCC, 0xAF}
	},
	{
		2, {0xEC, 0xA8},
		3, {0xE7, 0x90, 0xAC},
		2, {0x2C, 0x74},
		3, {0x8F, 0xCC, 0xB0}
	},
	{
		2, {0xEC, 0xA9},
		3, {0xE7, 0x90, 0xAE},
		2, {0x2E, 0x74},
		3, {0x8F, 0xCC, 0xB2}
	},
	{
		2, {0xEC, 0xAA},
		3, {0xE7, 0x90, 0xAF},
		2, {0x2F, 0x74},
		3, {0x8F, 0xCC, 0xB3}
	},
	{
		2, {0xEC, 0xAB},
		3, {0xE7, 0x90, 0xB0},
		2, {0x30, 0x74},
		3, {0x8F, 0xCC, 0xB4}
	},
	{
		2, {0xEC, 0xAC},
		3, {0xE7, 0x91, 0x84},
		2, {0x44, 0x74},
		3, {0x8F, 0xCC, 0xB9}
	},
	{
		2, {0xEC, 0xAD},
		3, {0xE7, 0x91, 0x86},
		2, {0x46, 0x74},
		3, {0x8F, 0xCC, 0xBA}
	},
	{
		2, {0xEC, 0xAE},
		3, {0xE7, 0x91, 0x87},
		2, {0x47, 0x74},
		3, {0x8F, 0xCC, 0xBB}
	},
	{
		2, {0xEC, 0xAF},
		3, {0xE7, 0x91, 0x8B},
		2, {0x4B, 0x74},
		3, {0x8F, 0xCC, 0xBC}
	},
	{
		2, {0xEC, 0xB0},
		3, {0xE7, 0x91, 0x97},
		2, {0x57, 0x74},
		3, {0x8F, 0xCC, 0xC0}
	},
	{
		2, {0xEC, 0xB1},
		3, {0xE7, 0x91, 0xA2},
		2, {0x62, 0x74},
		3, {0x8F, 0xCC, 0xC2}
	},
	{
		2, {0xEC, 0xB2},
		3, {0xE7, 0x91, 0xAB},
		2, {0x6B, 0x74},
		3, {0x8F, 0xCC, 0xC6}
	},
	{
		2, {0xEC, 0xB3},
		3, {0xE7, 0x91, 0xAD},
		2, {0x6D, 0x74},
		3, {0x8F, 0xCC, 0xC7}
	},
	{
		2, {0xEC, 0xB4},
		3, {0xE7, 0x92, 0x86},
		2, {0x86, 0x74},
		3, {0x8F, 0xCC, 0xCE}
	},
	{
		2, {0xEC, 0xB5},
		3, {0xE7, 0x92, 0x87},
		2, {0x87, 0x74},
		3, {0x8F, 0xCC, 0xCF}
	},
	{
		2, {0xEC, 0xB6},
		3, {0xE7, 0x92, 0x89},
		2, {0x89, 0x74},
		3, {0x8F, 0xCC, 0xD0}
	},
	{
		2, {0xEC, 0xB7},
		3, {0xE7, 0x92, 0x98},
		2, {0x98, 0x74},
		3, {0x8F, 0xCC, 0xD5}
	},
	{
		2, {0xEC, 0xB8},
		3, {0xE7, 0x92, 0x9C},
		2, {0x9C, 0x74},
		3, {0x8F, 0xCC, 0xD8}
	},
	{
		2, {0xEC, 0xB9},
		3, {0xE7, 0x92, 0x9F},
		2, {0x9F, 0x74},
		3, {0x8F, 0xCC, 0xD9}
	},
	{
		2, {0xEC, 0xBA},
		3, {0xE7, 0x92, 0xA3},
		2, {0xA3, 0x74},
		3, {0x8F, 0xCC, 0xDC}
	},
	{
		2, {0xEC, 0xBB},
		3, {0xE7, 0x92, 0x90},
		2, {0x90, 0x74},
		3, {0x8F, 0xCC, 0xD2}
	},
	{
		2, {0xEC, 0xBC},
		3, {0xE7, 0x92, 0xA6},
		2, {0xA6, 0x74},
		3, {0x8F, 0xCC, 0xDD}
	},
	{
		2, {0xEC, 0xBD},
		3, {0xE7, 0x92, 0xA8},
		2, {0xA8, 0x74},
		3, {0x8F, 0xCC, 0xDE}
	},
	{
		2, {0xEC, 0xBE},
		3, {0xE7, 0x92, 0xA9},
		2, {0xA9, 0x74},
		3, {0x8F, 0xCC, 0xDF}
	},
	{
		2, {0xEC, 0xBF},
		3, {0xE7, 0x92, 0xB5},
		2, {0xB5, 0x74},
		3, {0x8F, 0xCC, 0xE6}
	},
	{
		2, {0xEC, 0xC0},
		3, {0xE7, 0x92, 0xBF},
		2, {0xBF, 0x74},
		3, {0x8F, 0xCC, 0xE9}
	},
	{
		2, {0xEC, 0xC1},
		3, {0xE7, 0x93, 0x88},
		2, {0xC8, 0x74},
		3, {0x8F, 0xCC, 0xEA}
	},
	{
		2, {0xEC, 0xC2},
		3, {0xE7, 0x93, 0x89},
		2, {0xC9, 0x74},
		3, {0x8F, 0xCC, 0xEB}
	},
	{
		2, {0xEC, 0xC3},
		3, {0xE7, 0x93, 0x9A},
		2, {0xDA, 0x74},
		3, {0x8F, 0xCC, 0xF0}
	},
	{
		2, {0xEC, 0xC4},
		3, {0xE7, 0x93, 0xBF},
		2, {0xFF, 0x74},
		3, {0x8F, 0xCC, 0xFD}
	},
	{
		2, {0xEC, 0xC5},
		3, {0xE7, 0x94, 0x81},
		2, {0x01, 0x75},
		0, {0x00}
	},
	{
		2, {0xEC, 0xC6},
		3, {0xE7, 0x94, 0x97},
		2, {0x17, 0x75},
		3, {0x8F, 0xCD, 0xA3}
	},
	{
		2, {0xEC, 0xC7},
		3, {0xE7, 0x94, 0xAF},
		2, {0x2F, 0x75},
		3, {0x8F, 0xCD, 0xAA}
	},
	{
		2, {0xEC, 0xC8},
		3, {0xE7, 0x95, 0xAF},
		2, {0x6F, 0x75},
		3, {0x8F, 0xCD, 0xBB}
	},
	{
		2, {0xEC, 0xC9},
		3, {0xE7, 0x95, 0xB9},
		2, {0x79, 0x75},
		3, {0x8F, 0xCD, 0xBD}
	},
	{
		2, {0xEC, 0xCA},
		3, {0xE7, 0x96, 0x92},
		2, {0x92, 0x75},
		3, {0x8F, 0xCD, 0xC6}
	},
	{
		2, {0xEC, 0xCB},
		3, {0xE3, 0xBD, 0xB2},
		2, {0x72, 0x3F},
		0, {0x00}
	},
	{
		2, {0xEC, 0xCC},
		3, {0xE7, 0x97, 0x8E},
		2, {0xCE, 0x75},
		3, {0x8F, 0xCD, 0xD5}
	},
	{
		2, {0xEC, 0xCD},
		3, {0xE7, 0x97, 0xA4},
		2, {0xE4, 0x75},
		3, {0x8F, 0xCD, 0xDC}
	},
	{
		2, {0xEC, 0xCE},
		3, {0xE7, 0x98, 0x80},
		2, {0x00, 0x76},
		3, {0x8F, 0xCD, 0xE3}
	},
	{
		2, {0xEC, 0xCF},
		3, {0xE7, 0x98, 0x82},
		2, {0x02, 0x76},
		3, {0x8F, 0xCD, 0xE4}
	},
	{
		2, {0xEC, 0xD0},
		3, {0xE7, 0x98, 0x88},
		2, {0x08, 0x76},
		3, {0x8F, 0xCD, 0xE8}
	},
	{
		2, {0xEC, 0xD1},
		3, {0xE7, 0x98, 0x95},
		2, {0x15, 0x76},
		3, {0x8F, 0xCD, 0xEE}
	},
	{
		2, {0xEC, 0xD2},
		3, {0xE7, 0x98, 0x96},
		2, {0x16, 0x76},
		3, {0x8F, 0xCD, 0xEF}
	},
	{
		2, {0xEC, 0xD3},
		3, {0xE7, 0x98, 0x99},
		2, {0x19, 0x76},
		3, {0x8F, 0xCD, 0xF0}
	},
	{
		2, {0xEC, 0xD4},
		3, {0xE7, 0x98, 0x9E},
		2, {0x1E, 0x76},
		3, {0x8F, 0xCD, 0xF4}
	},
	{
		2, {0xEC, 0xD5},
		3, {0xE7, 0x98, 0xAD},
		2, {0x2D, 0x76},
		3, {0x8F, 0xCD, 0xF9}
	},
	{
		2, {0xEC, 0xD6},
		3, {0xE7, 0x98, 0xB5},
		2, {0x35, 0x76},
		3, {0x8F, 0xCD, 0xFC}
	},
	{
		2, {0xEC, 0xD7},
		3, {0xE7, 0x99, 0x83},
		2, {0x43, 0x76},
		3, {0x8F, 0xCE, 0xA6}
	},
	{
		2, {0xEC, 0xD8},
		3, {0xE7, 0x99, 0x8B},
		2, {0x4B, 0x76},
		3, {0x8F, 0xCE, 0xAA}
	},
	{
		2, {0xEC, 0xD9},
		3, {0xE7, 0x99, 0xA4},
		2, {0x64, 0x76},
		3, {0x8F, 0xCE, 0xAE}
	},
	{
		2, {0xEC, 0xDA},
		3, {0xE7, 0x99, 0xA5},
		2, {0x65, 0x76},
		3, {0x8F, 0xCE, 0xAF}
	},
	{
		2, {0xEC, 0xDB},
		3, {0xE7, 0x99, 0xAD},
		2, {0x6D, 0x76},
		3, {0x8F, 0xCE, 0xB0}
	},
	{
		2, {0xEC, 0xDC},
		3, {0xE7, 0x99, 0xAF},
		2, {0x6F, 0x76},
		3, {0x8F, 0xCE, 0xB2}
	},
	{
		2, {0xEC, 0xDD},
		3, {0xE7, 0x99, 0xB1},
		2, {0x71, 0x76},
		3, {0x8F, 0xCE, 0xB3}
	},
	{
		2, {0xEC, 0xDE},
		3, {0xE7, 0x9A, 0x81},
		2, {0x81, 0x76},
		3, {0x8F, 0xCE, 0xB5}
	},
	{
		2, {0xEC, 0xDF},
		3, {0xE7, 0x9A, 0x9B},
		2, {0x9B, 0x76},
		3, {0x8F, 0xCE, 0xBA}
	},
	{
		2, {0xEC, 0xE0},
		3, {0xE7, 0x9A, 0x9D},
		2, {0x9D, 0x76},
		3, {0x8F, 0xCE, 0xBC}
	},
	{
		2, {0xEC, 0xE1},
		3, {0xE7, 0x9A, 0x9E},
		2, {0x9E, 0x76},
		0, {0x00}
	},
	{
		2, {0xEC, 0xE2},
		3, {0xE7, 0x9A, 0xA6},
		2, {0xA6, 0x76},
		3, {0x8F, 0xCE, 0xC3}
	},
	{
		2, {0xEC, 0xE3},
		3, {0xE7, 0x9A, 0xAA},
		2, {0xAA, 0x76},
		3, {0x8F, 0xCE, 0xC6}
	},
	{
		2, {0xEC, 0xE4},
		3, {0xE7, 0x9A, 0xB6},
		2, {0xB6, 0x76},
		0, {0x00}
	},
	{
		2, {0xEC, 0xE5},
		3, {0xE7, 0x9B, 0x85},
		2, {0xC5, 0x76},
		3, {0x8F, 0xCE, 0xCA}
	},
	{
		2, {0xEC, 0xE6},
		3, {0xE7, 0x9B, 0x8C},
		2, {0xCC, 0x76},
		3, {0x8F, 0xCE, 0xCD}
	},
	{
		2, {0xEC, 0xE7},
		3, {0xE7, 0x9B, 0x8E},
		2, {0xCE, 0x76},
		3, {0x8F, 0xCE, 0xCE}
	},
	{
		2, {0xEC, 0xE8},
		3, {0xE7, 0x9B, 0x94},
		2, {0xD4, 0x76},
		3, {0x8F, 0xCE, 0xCF}
	},
	{
		2, {0xEC, 0xE9},
		3, {0xE7, 0x9B, 0xA6},
		2, {0xE6, 0x76},
		3, {0x8F, 0xCE, 0xD2}
	},
	{
		2, {0xEC, 0xEA},
		3, {0xE7, 0x9B, 0xB1},
		2, {0xF1, 0x76},
		3, {0x8F, 0xCE, 0xD6}
	},
	{
		2, {0xEC, 0xEB},
		3, {0xE7, 0x9B, 0xBC},
		2, {0xFC, 0x76},
		3, {0x8F, 0xCE, 0xD9}
	},
	{
		2, {0xEC, 0xEC},
		3, {0xE7, 0x9C, 0x8A},
		2, {0x0A, 0x77},
		3, {0x8F, 0xCE, 0xDC}
	},
	{
		2, {0xEC, 0xED},
		3, {0xE7, 0x9C, 0x99},
		2, {0x19, 0x77},
		3, {0x8F, 0xCE, 0xE2}
	},
	{
		2, {0xEC, 0xEE},
		3, {0xE7, 0x9C, 0xB4},
		2, {0x34, 0x77},
		3, {0x8F, 0xCE, 0xEA}
	},
	{
		2, {0xEC, 0xEF},
		3, {0xE7, 0x9C, 0xB6},
		2, {0x36, 0x77},
		3, {0x8F, 0xCE, 0xEC}
	},
	{
		2, {0xEC, 0xF0},
		3, {0xE7, 0x9D, 0x86},
		2, {0x46, 0x77},
		3, {0x8F, 0xCE, 0xF2}
	},
	{
		2, {0xEC, 0xF1},
		3, {0xE7, 0x9D, 0x8D},
		2, {0x4D, 0x77},
		3, {0x8F, 0xCE, 0xF4}
	},
	{
		2, {0xEC, 0xF2},
		3, {0xE7, 0x9D, 0x8E},
		2, {0x4E, 0x77},
		3, {0x8F, 0xCE, 0xF5}
	},
	{
		2, {0xEC, 0xF3},
		3, {0xE7, 0x9D, 0x9C},
		2, {0x5C, 0x77},
		3, {0x8F, 0xCE, 0xFA}
	},
	{
		2, {0xEC, 0xF4},
		3, {0xE7, 0x9D, 0x9F},
		2, {0x5F, 0x77},
		3, {0x8F, 0xCE, 0xFC}
	},
	{
		2, {0xEC, 0xF5},
		3, {0xE7, 0x9D, 0xA2},
		2, {0x62, 0x77},
		3, {0x8F, 0xCE, 0xFE}
	},
	{
		2, {0xEC, 0xF6},
		3, {0xE7, 0x9D, 0xBA},
		2, {0x7A, 0x77},
		3, {0x8F, 0xCF, 0xA9}
	},
	{
		2, {0xEC, 0xF7},
		3, {0xE7, 0x9E, 0x80},
		2, {0x80, 0x77},
		3, {0x8F, 0xCF, 0xAB}
	},
	{
		2, {0xEC, 0xF8},
		3, {0xE7, 0x9E, 0x94},
		2, {0x94, 0x77},
		3, {0x8F, 0xCF, 0xAF}
	},
	{
		2, {0xEC, 0xF9},
		3, {0xE7, 0x9E, 0xAA},
		2, {0xAA, 0x77},
		3, {0x8F, 0xCF, 0xB6}
	},
	{
		2, {0xEC, 0xFA},
		3, {0xE7, 0x9F, 0xA0},
		2, {0xE0, 0x77},
		3, {0x8F, 0xCF, 0xC4}
	},
	{
		2, {0xEC, 0xFB},
		3, {0xE7, 0xA0, 0xAD},
		2, {0x2D, 0x78},
		3, {0x8F, 0xCF, 0xD8}
	},
	{
		2, {0xEC, 0xFC},
		4, {0xF0, 0xA5, 0x92, 0x8E},
		4, {0x55, 0xD8, 0x8E, 0xDC},
		0, {0x00}
	},
	{
		2, {0xED, 0x40},
		3, {0xE7, 0xA1, 0x83},
		2, {0x43, 0x78},
		3, {0x8F, 0xCF, 0xDD}
	},
	{
		2, {0xED, 0x41},
		3, {0xE7, 0xA1, 0x8E},
		2, {0x4E, 0x78},
		3, {0x8F, 0xCF, 0xE2}
	},
	{
		2, {0xED, 0x42},
		3, {0xE7, 0xA1, 0x8F},
		2, {0x4F, 0x78},
		0, {0x00}
	},
	{
		2, {0xED, 0x43},
		3, {0xE7, 0xA1, 0x91},
		2, {0x51, 0x78},
		0, {0x00}
	},
	{
		2, {0xED, 0x44},
		3, {0xE7, 0xA1, 0xA8},
		2, {0x68, 0x78},
		3, {0x8F, 0xCF, 0xEA}
	},
	{
		2, {0xED, 0x45},
		3, {0xE7, 0xA1, 0xAE},
		2, {0x6E, 0x78},
		3, {0x8F, 0xCF, 0xEC}
	},
	{
		2, {0xED, 0x46},
		3, {0xEF, 0xA9, 0x8B},
		2, {0x4B, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x47},
		3, {0xE7, 0xA2, 0xB0},
		2, {0xB0, 0x78},
		3, {0x8F, 0xCF, 0xFB}
	},
	{
		2, {0xED, 0x48},
		4, {0xF0, 0xA5, 0x94, 0x8E},
		4, {0x55, 0xD8, 0x0E, 0xDD},
		0, {0x00}
	},
	{
		2, {0xED, 0x49},
		3, {0xE7, 0xA2, 0xAD},
		2, {0xAD, 0x78},
		3, {0x8F, 0xCF, 0xFA}
	},
	{
		2, {0xED, 0x4A},
		3, {0xE7, 0xA3, 0xA4},
		2, {0xE4, 0x78},
		3, {0x8F, 0xD0, 0xAD}
	},
	{
		2, {0xED, 0x4B},
		3, {0xE7, 0xA3, 0xB2},
		2, {0xF2, 0x78},
		3, {0x8F, 0xD0, 0xB4}
	},
	{
		2, {0xED, 0x4C},
		3, {0xE7, 0xA4, 0x80},
		2, {0x00, 0x79},
		3, {0x8F, 0xD0, 0xB6}
	},
	{
		2, {0xED, 0x4D},
		3, {0xE7, 0xA3, 0xB7},
		2, {0xF7, 0x78},
		3, {0x8F, 0xD0, 0xB8}
	},
	{
		2, {0xED, 0x4E},
		3, {0xE7, 0xA4, 0x9C},
		2, {0x1C, 0x79},
		3, {0x8F, 0xD0, 0xC0}
	},
	{
		2, {0xED, 0x4F},
		3, {0xE7, 0xA4, 0xAE},
		2, {0x2E, 0x79},
		0, {0x00}
	},
	{
		2, {0xED, 0x50},
		3, {0xE7, 0xA4, 0xB1},
		2, {0x31, 0x79},
		3, {0x8F, 0xD0, 0xC8}
	},
	{
		2, {0xED, 0x51},
		3, {0xE7, 0xA4, 0xB4},
		2, {0x34, 0x79},
		3, {0x8F, 0xD0, 0xC9}
	},
	{
		2, {0xED, 0x52},
		3, {0xEF, 0xA9, 0x8C},
		2, {0x4C, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x53},
		3, {0xEF, 0xA9, 0x8D},
		2, {0x4D, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x54},
		3, {0xE7, 0xA5, 0x85},
		2, {0x45, 0x79},
		3, {0x8F, 0xD0, 0xCF}
	},
	{
		2, {0xED, 0x55},
		3, {0xE7, 0xA5, 0x86},
		2, {0x46, 0x79},
		3, {0x8F, 0xD0, 0xD0}
	},
	{
		2, {0xED, 0x56},
		3, {0xEF, 0xA9, 0x8E},
		2, {0x4E, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x57},
		3, {0xEF, 0xA9, 0x8F},
		2, {0x4F, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x58},
		3, {0xEF, 0xA9, 0x90},
		2, {0x50, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x59},
		3, {0xE7, 0xA5, 0x9C},
		2, {0x5C, 0x79},
		3, {0x8F, 0xD0, 0xD8}
	},
	{
		2, {0xED, 0x5A},
		3, {0xEF, 0xA9, 0x91},
		2, {0x51, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x5B},
		3, {0xEF, 0xA8, 0x99},
		2, {0x19, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x5C},
		3, {0xEF, 0xA8, 0x9A},
		2, {0x1A, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x5D},
		3, {0xE7, 0xA5, 0xB9},
		2, {0x79, 0x79},
		3, {0x8F, 0xD0, 0xDD}
	},
	{
		2, {0xED, 0x5E},
		3, {0xEF, 0xA9, 0x92},
		2, {0x52, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x5F},
		3, {0xEF, 0xA9, 0x93},
		2, {0x53, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x60},
		3, {0xEF, 0xA8, 0x9B},
		2, {0x1B, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x61},
		3, {0xE7, 0xA6, 0x98},
		2, {0x98, 0x79},
		3, {0x8F, 0xD0, 0xE8}
	},
	{
		2, {0xED, 0x62},
		3, {0xE7, 0xA6, 0xB1},
		2, {0xB1, 0x79},
		3, {0x8F, 0xD0, 0xF0}
	},
	{
		2, {0xED, 0x63},
		3, {0xE7, 0xA6, 0xB8},
		2, {0xB8, 0x79},
		3, {0x8F, 0xD0, 0xF2}
	},
	{
		2, {0xED, 0x64},
		3, {0xE7, 0xA7, 0x88},
		2, {0xC8, 0x79},
		3, {0x8F, 0xD0, 0xF7}
	},
	{
		2, {0xED, 0x65},
		3, {0xE7, 0xA7, 0x8A},
		2, {0xCA, 0x79},
		3, {0x8F, 0xD0, 0xF8}
	},
	{
		2, {0xED, 0x66},
		4, {0xF0, 0xA5, 0x9D, 0xB1},
		4, {0x55, 0xD8, 0x71, 0xDF},
		0, {0x00}
	},
	{
		2, {0xED, 0x67},
		3, {0xE7, 0xA7, 0x94},
		2, {0xD4, 0x79},
		3, {0x8F, 0xD0, 0xFA}
	},
	{
		2, {0xED, 0x68},
		3, {0xE7, 0xA7, 0x9E},
		2, {0xDE, 0x79},
		3, {0x8F, 0xD0, 0xFE}
	},
	{
		2, {0xED, 0x69},
		3, {0xE7, 0xA7, 0xAB},
		2, {0xEB, 0x79},
		3, {0x8F, 0xD1, 0xA5}
	},
	{
		2, {0xED, 0x6A},
		3, {0xE7, 0xA7, 0xAD},
		2, {0xED, 0x79},
		3, {0x8F, 0xD1, 0xA6}
	},
	{
		2, {0xED, 0x6B},
		3, {0xE7, 0xA8, 0x83},
		2, {0x03, 0x7A},
		3, {0x8F, 0xD1, 0xAB}
	},
	{
		2, {0xED, 0x6C},
		3, {0xEF, 0xA9, 0x94},
		2, {0x54, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x6D},
		3, {0xE7, 0xA8, 0xB9},
		2, {0x39, 0x7A},
		3, {0x8F, 0xD1, 0xBD}
	},
	{
		2, {0xED, 0x6E},
		3, {0xE7, 0xA9, 0x9D},
		2, {0x5D, 0x7A},
		3, {0x8F, 0xD1, 0xC8}
	},
	{
		2, {0xED, 0x6F},
		3, {0xE7, 0xA9, 0xAD},
		2, {0x6D, 0x7A},
		3, {0x8F, 0xD1, 0xCE}
	},
	{
		2, {0xED, 0x70},
		3, {0xEF, 0xA9, 0x95},
		2, {0x55, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x71},
		3, {0xE7, 0xAA, 0x85},
		2, {0x85, 0x7A},
		3, {0x8F, 0xD1, 0xD4}
	},
	{
		2, {0xED, 0x72},
		3, {0xE7, 0xAA, 0xA0},
		2, {0xA0, 0x7A},
		3, {0x8F, 0xD1, 0xDC}
	},
	{
		2, {0xED, 0x73},
		4, {0xF0, 0xA5, 0xA7, 0x84},
		4, {0x56, 0xD8, 0xC4, 0xDD},
		0, {0x00}
	},
	{
		2, {0xED, 0x74},
		3, {0xE7, 0xAA, 0xB3},
		2, {0xB3, 0x7A},
		3, {0x8F, 0xD1, 0xDF}
	},
	{
		2, {0xED, 0x75},
		3, {0xE7, 0xAA, 0xBB},
		2, {0xBB, 0x7A},
		3, {0x8F, 0xD1, 0xE2}
	},
	{
		2, {0xED, 0x76},
		3, {0xE7, 0xAB, 0x8E},
		2, {0xCE, 0x7A},
		3, {0x8F, 0xD1, 0xE7}
	},
	{
		2, {0xED, 0x77},
		3, {0xE7, 0xAB, 0xAB},
		2, {0xEB, 0x7A},
		3, {0x8F, 0xD1, 0xEC}
	},
	{
		2, {0xED, 0x78},
		3, {0xE7, 0xAB, 0xBD},
		2, {0xFD, 0x7A},
		3, {0x8F, 0xD1, 0xF1}
	},
	{
		2, {0xED, 0x79},
		3, {0xE7, 0xAC, 0x92},
		2, {0x12, 0x7B},
		0, {0x00}
	},
	{
		2, {0xED, 0x7A},
		3, {0xE7, 0xAC, 0xAD},
		2, {0x2D, 0x7B},
		3, {0x8F, 0xD1, 0xFB}
	},
	{
		2, {0xED, 0x7B},
		3, {0xE7, 0xAC, 0xBB},
		2, {0x3B, 0x7B},
		0, {0x00}
	},
	{
		2, {0xED, 0x7C},
		3, {0xE7, 0xAD, 0x87},
		2, {0x47, 0x7B},
		3, {0x8F, 0xD2, 0xA7}
	},
	{
		2, {0xED, 0x7D},
		3, {0xE7, 0xAD, 0x8E},
		2, {0x4E, 0x7B},
		3, {0x8F, 0xD2, 0xA8}
	},
	{
		2, {0xED, 0x7E},
		3, {0xE7, 0xAD, 0xA0},
		2, {0x60, 0x7B},
		3, {0x8F, 0xD2, 0xAA}
	},
	{
		2, {0xED, 0x80},
		3, {0xE7, 0xAD, 0xAD},
		2, {0x6D, 0x7B},
		3, {0x8F, 0xD2, 0xAF}
	},
	{
		2, {0xED, 0x81},
		3, {0xE7, 0xAD, 0xAF},
		2, {0x6F, 0x7B},
		3, {0x8F, 0xD2, 0xB0}
	},
	{
		2, {0xED, 0x82},
		3, {0xE7, 0xAD, 0xB2},
		2, {0x72, 0x7B},
		3, {0x8F, 0xD2, 0xB1}
	},
	{
		2, {0xED, 0x83},
		3, {0xE7, 0xAE, 0x9E},
		2, {0x9E, 0x7B},
		3, {0x8F, 0xD2, 0xBB}
	},
	{
		2, {0xED, 0x84},
		3, {0xEF, 0xA9, 0x96},
		2, {0x56, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0x85},
		3, {0xE7, 0xAF, 0x97},
		2, {0xD7, 0x7B},
		3, {0x8F, 0xD2, 0xCE}
	},
	{
		2, {0xED, 0x86},
		3, {0xE7, 0xAF, 0x99},
		2, {0xD9, 0x7B},
		3, {0x8F, 0xD2, 0xCF}
	},
	{
		2, {0xED, 0x87},
		3, {0xE7, 0xB0, 0x81},
		2, {0x01, 0x7C},
		3, {0x8F, 0xD2, 0xDC}
	},
	{
		2, {0xED, 0x88},
		3, {0xE7, 0xB0, 0xB1},
		2, {0x31, 0x7C},
		3, {0x8F, 0xD2, 0xED}
	},
	{
		2, {0xED, 0x89},
		3, {0xE7, 0xB0, 0x9E},
		2, {0x1E, 0x7C},
		0, {0x00}
	},
	{
		2, {0xED, 0x8A},
		3, {0xE7, 0xB0, 0xA0},
		2, {0x20, 0x7C},
		3, {0x8F, 0xD2, 0xE8}
	},
	{
		2, {0xED, 0x8B},
		3, {0xE7, 0xB0, 0xB3},
		2, {0x33, 0x7C},
		3, {0x8F, 0xD2, 0xEE}
	},
	{
		2, {0xED, 0x8C},
		3, {0xE7, 0xB0, 0xB6},
		2, {0x36, 0x7C},
		3, {0x8F, 0xD2, 0xF0}
	},
	{
		2, {0xED, 0x8D},
		3, {0xE4, 0x89, 0xA4},
		2, {0x64, 0x42},
		0, {0x00}
	},
	{
		2, {0xED, 0x8E},
		4, {0xF0, 0xA5, 0xB6, 0xA1},
		4, {0x57, 0xD8, 0xA1, 0xDD},
		0, {0x00}
	},
	{
		2, {0xED, 0x8F},
		3, {0xE7, 0xB1, 0x99},
		2, {0x59, 0x7C},
		3, {0x8F, 0xD2, 0xF9}
	},
	{
		2, {0xED, 0x90},
		3, {0xE7, 0xB1, 0xAD},
		2, {0x6D, 0x7C},
		3, {0x8F, 0xD3, 0xA5}
	},
	{
		2, {0xED, 0x91},
		3, {0xE7, 0xB1, 0xB9},
		2, {0x79, 0x7C},
		3, {0x8F, 0xD3, 0xA9}
	},
	{
		2, {0xED, 0x92},
		3, {0xE7, 0xB2, 0x8F},
		2, {0x8F, 0x7C},
		3, {0x8F, 0xD3, 0xAE}
	},
	{
		2, {0xED, 0x93},
		3, {0xE7, 0xB2, 0x94},
		2, {0x94, 0x7C},
		3, {0x8F, 0xD3, 0xAF}
	},
	{
		2, {0xED, 0x94},
		3, {0xE7, 0xB2, 0xA0},
		2, {0xA0, 0x7C},
		3, {0x8F, 0xD3, 0xB1}
	},
	{
		2, {0xED, 0x95},
		3, {0xE7, 0xB2, 0xBC},
		2, {0xBC, 0x7C},
		3, {0x8F, 0xD3, 0xB8}
	},
	{
		2, {0xED, 0x96},
		3, {0xE7, 0xB3, 0x95},
		2, {0xD5, 0x7C},
		3, {0x8F, 0xD3, 0xC2}
	},
	{
		2, {0xED, 0x97},
		3, {0xE7, 0xB3, 0x99},
		2, {0xD9, 0x7C},
		3, {0x8F, 0xD3, 0xC4}
	},
	{
		2, {0xED, 0x98},
		3, {0xE7, 0xB3, 0x9D},
		2, {0xDD, 0x7C},
		3, {0x8F, 0xD3, 0xC6}
	},
	{
		2, {0xED, 0x99},
		3, {0xE7, 0xB4, 0x87},
		2, {0x07, 0x7D},
		3, {0x8F, 0xD3, 0xCC}
	},
	{
		2, {0xED, 0x9A},
		3, {0xE7, 0xB4, 0x88},
		2, {0x08, 0x7D},
		3, {0x8F, 0xD3, 0xCD}
	},
	{
		2, {0xED, 0x9B},
		3, {0xE7, 0xB4, 0x93},
		2, {0x13, 0x7D},
		3, {0x8F, 0xD3, 0xD2}
	},
	{
		2, {0xED, 0x9C},
		3, {0xE7, 0xB4, 0x9D},
		2, {0x1D, 0x7D},
		3, {0x8F, 0xD3, 0xD4}
	},
	{
		2, {0xED, 0x9D},
		3, {0xE7, 0xB4, 0xA3},
		2, {0x23, 0x7D},
		3, {0x8F, 0xD3, 0xD6}
	},
	{
		2, {0xED, 0x9E},
		3, {0xE7, 0xB4, 0xB1},
		2, {0x31, 0x7D},
		3, {0x8F, 0xD3, 0xDA}
	},
	{
		2, {0xED, 0x9F},
		3, {0xE7, 0xB5, 0x81},
		2, {0x41, 0x7D},
		3, {0x8F, 0xD3, 0xDF}
	},
	{
		2, {0xED, 0xA0},
		3, {0xE7, 0xB5, 0x88},
		2, {0x48, 0x7D},
		3, {0x8F, 0xD3, 0xE1}
	},
	{
		2, {0xED, 0xA1},
		3, {0xE7, 0xB5, 0x93},
		2, {0x53, 0x7D},
		3, {0x8F, 0xD3, 0xE4}
	},
	{
		2, {0xED, 0xA2},
		3, {0xE7, 0xB5, 0x9C},
		2, {0x5C, 0x7D},
		3, {0x8F, 0xD3, 0xE8}
	},
	{
		2, {0xED, 0xA3},
		3, {0xE7, 0xB5, 0xBA},
		2, {0x7A, 0x7D},
		3, {0x8F, 0xD3, 0xEF}
	},
	{
		2, {0xED, 0xA4},
		3, {0xE7, 0xB6, 0x83},
		2, {0x83, 0x7D},
		3, {0x8F, 0xD3, 0xF4}
	},
	{
		2, {0xED, 0xA5},
		3, {0xE7, 0xB6, 0x8B},
		2, {0x8B, 0x7D},
		3, {0x8F, 0xD3, 0xF8}
	},
	{
		2, {0xED, 0xA6},
		3, {0xE7, 0xB6, 0xA0},
		2, {0xA0, 0x7D},
		0, {0x00}
	},
	{
		2, {0xED, 0xA7},
		3, {0xE7, 0xB6, 0xA6},
		2, {0xA6, 0x7D},
		3, {0x8F, 0xD4, 0xA2}
	},
	{
		2, {0xED, 0xA8},
		3, {0xE7, 0xB7, 0x82},
		2, {0xC2, 0x7D},
		3, {0x8F, 0xD4, 0xA9}
	},
	{
		2, {0xED, 0xA9},
		3, {0xE7, 0xB7, 0x8C},
		2, {0xCC, 0x7D},
		3, {0x8F, 0xD4, 0xAE}
	},
	{
		2, {0xED, 0xAA},
		3, {0xE7, 0xB7, 0x96},
		2, {0xD6, 0x7D},
		0, {0x00}
	},
	{
		2, {0xED, 0xAB},
		3, {0xE7, 0xB7, 0xA3},
		2, {0xE3, 0x7D},
		0, {0x00}
	},
	{
		2, {0xED, 0xAC},
		3, {0xEF, 0xA9, 0x97},
		2, {0x57, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0xAD},
		3, {0xE7, 0xB8, 0xA8},
		2, {0x28, 0x7E},
		3, {0x8F, 0xD4, 0xC8}
	},
	{
		2, {0xED, 0xAE},
		3, {0xE7, 0xB8, 0x88},
		2, {0x08, 0x7E},
		3, {0x8F, 0xD4, 0xBF}
	},
	{
		2, {0xED, 0xAF},
		3, {0xE7, 0xB8, 0x91},
		2, {0x11, 0x7E},
		3, {0x8F, 0xD4, 0xC1}
	},
	{
		2, {0xED, 0xB0},
		3, {0xE7, 0xB8, 0x95},
		2, {0x15, 0x7E},
		3, {0x8F, 0xD4, 0xC2}
	},
	{
		2, {0xED, 0xB1},
		3, {0xEF, 0xA9, 0x99},
		2, {0x59, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0xB2},
		3, {0xE7, 0xB9, 0x87},
		2, {0x47, 0x7E},
		3, {0x8F, 0xD4, 0xD1}
	},
	{
		2, {0xED, 0xB3},
		3, {0xE7, 0xB9, 0x92},
		2, {0x52, 0x7E},
		3, {0x8F, 0xD4, 0xD4}
	},
	{
		2, {0xED, 0xB4},
		3, {0xE7, 0xB9, 0xA1},
		2, {0x61, 0x7E},
		3, {0x8F, 0xD4, 0xD7}
	},
	{
		2, {0xED, 0xB5},
		3, {0xE7, 0xBA, 0x8A},
		2, {0x8A, 0x7E},
		3, {0x8F, 0xD4, 0xE3}
	},
	{
		2, {0xED, 0xB6},
		3, {0xE7, 0xBA, 0x8D},
		2, {0x8D, 0x7E},
		3, {0x8F, 0xD4, 0xE4}
	},
	{
		2, {0xED, 0xB7},
		3, {0xE7, 0xBD, 0x87},
		2, {0x47, 0x7F},
		3, {0x8F, 0xD4, 0xF2}
	},
	{
		2, {0xED, 0xB8},
		3, {0xEF, 0xA9, 0x9A},
		2, {0x5A, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0xB9},
		3, {0xE7, 0xBE, 0x91},
		2, {0x91, 0x7F},
		3, {0x8F, 0xD5, 0xAA}
	},
	{
		2, {0xED, 0xBA},
		3, {0xE7, 0xBE, 0x97},
		2, {0x97, 0x7F},
		3, {0x8F, 0xD5, 0xAC}
	},
	{
		2, {0xED, 0xBB},
		3, {0xE7, 0xBE, 0xBF},
		2, {0xBF, 0x7F},
		3, {0x8F, 0xD5, 0xB5}
	},
	{
		2, {0xED, 0xBC},
		3, {0xE7, 0xBF, 0x8E},
		2, {0xCE, 0x7F},
		3, {0x8F, 0xD5, 0xB9}
	},
	{
		2, {0xED, 0xBD},
		3, {0xE7, 0xBF, 0x9B},
		2, {0xDB, 0x7F},
		3, {0x8F, 0xD5, 0xBB}
	},
	{
		2, {0xED, 0xBE},
		3, {0xE7, 0xBF, 0x9F},
		2, {0xDF, 0x7F},
		3, {0x8F, 0xD5, 0xBC}
	},
	{
		2, {0xED, 0xBF},
		3, {0xE7, 0xBF, 0xAC},
		2, {0xEC, 0x7F},
		3, {0x8F, 0xD5, 0xC0}
	},
	{
		2, {0xED, 0xC0},
		3, {0xE7, 0xBF, 0xAE},
		2, {0xEE, 0x7F},
		3, {0x8F, 0xD5, 0xC1}
	},
	{
		2, {0xED, 0xC1},
		3, {0xE7, 0xBF, 0xBA},
		2, {0xFA, 0x7F},
		3, {0x8F, 0xD5, 0xC4}
	},
	{
		2, {0xED, 0xC2},
		3, {0xEF, 0xA9, 0x9B},
		2, {0x5B, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0xC3},
		3, {0xE8, 0x80, 0x94},
		2, {0x14, 0x80},
		3, {0x8F, 0xD5, 0xD0}
	},
	{
		2, {0xED, 0xC4},
		3, {0xE8, 0x80, 0xA6},
		2, {0x26, 0x80},
		3, {0x8F, 0xD5, 0xD7}
	},
	{
		2, {0xED, 0xC5},
		3, {0xE8, 0x80, 0xB5},
		2, {0x35, 0x80},
		3, {0x8F, 0xD5, 0xDC}
	},
	{
		2, {0xED, 0xC6},
		3, {0xE8, 0x80, 0xB7},
		2, {0x37, 0x80},
		3, {0x8F, 0xD5, 0xDD}
	},
	{
		2, {0xED, 0xC7},
		3, {0xE8, 0x80, 0xBC},
		2, {0x3C, 0x80},
		3, {0x8F, 0xD5, 0xE0}
	},
	{
		2, {0xED, 0xC8},
		3, {0xE8, 0x83, 0x8A},
		2, {0xCA, 0x80},
		0, {0x00}
	},
	{
		2, {0xED, 0xC9},
		3, {0xE8, 0x83, 0x97},
		2, {0xD7, 0x80},
		3, {0x8F, 0xD5, 0xFA}
	},
	{
		2, {0xED, 0xCA},
		3, {0xE8, 0x83, 0xA0},
		2, {0xE0, 0x80},
		3, {0x8F, 0xD5, 0xFC}
	},
	{
		2, {0xED, 0xCB},
		3, {0xE8, 0x83, 0xB3},
		2, {0xF3, 0x80},
		3, {0x8F, 0xD6, 0xA3}
	},
	{
		2, {0xED, 0xCC},
		3, {0xE8, 0x84, 0x98},
		2, {0x18, 0x81},
		3, {0x8F, 0xD6, 0xAC}
	},
	{
		2, {0xED, 0xCD},
		3, {0xE8, 0x85, 0x8A},
		2, {0x4A, 0x81},
		3, {0x8F, 0xD6, 0xB9}
	},
	{
		2, {0xED, 0xCE},
		3, {0xE8, 0x85, 0xA0},
		2, {0x60, 0x81},
		3, {0x8F, 0xD6, 0xBD}
	},
	{
		2, {0xED, 0xCF},
		3, {0xE8, 0x85, 0xA7},
		2, {0x67, 0x81},
		3, {0x8F, 0xD6, 0xBF}
	},
	{
		2, {0xED, 0xD0},
		3, {0xE8, 0x85, 0xA8},
		2, {0x68, 0x81},
		3, {0x8F, 0xD6, 0xC0}
	},
	{
		2, {0xED, 0xD1},
		3, {0xE8, 0x85, 0xAD},
		2, {0x6D, 0x81},
		3, {0x8F, 0xD6, 0xC2}
	},
	{
		2, {0xED, 0xD2},
		3, {0xE8, 0x86, 0xBB},
		2, {0xBB, 0x81},
		3, {0x8F, 0xD6, 0xD4}
	},
	{
		2, {0xED, 0xD3},
		3, {0xE8, 0x87, 0x8A},
		2, {0xCA, 0x81},
		3, {0x8F, 0xD6, 0xD8}
	},
	{
		2, {0xED, 0xD4},
		3, {0xE8, 0x87, 0x8F},
		2, {0xCF, 0x81},
		3, {0x8F, 0xD6, 0xDA}
	},
	{
		2, {0xED, 0xD5},
		3, {0xE8, 0x87, 0x97},
		2, {0xD7, 0x81},
		3, {0x8F, 0xD6, 0xDC}
	},
	{
		2, {0xED, 0xD6},
		3, {0xEF, 0xA9, 0x9C},
		2, {0x5C, 0xFA},
		0, {0x00}
	},
	{
		2, {0xED, 0xD7},
		3, {0xE4, 0x91, 0x93},
		2, {0x53, 0x44},
		0, {0x00}
	},
	{
		2, {0xED, 0xD8},
		3, {0xE4, 0x91, 0x9B},
		2, {0x5B, 0x44},
		0, {0x00}
	},
	{
		2, {0xED, 0xD9},
		3, {0xE8, 0x89, 0xA0},
		2, {0x60, 0x82},
		3, {0x8F, 0xD7, 0xA7}
	},
	{
		2, {0xED, 0xDA},
		3, {0xE8, 0x89, 0xB4},
		2, {0x74, 0x82},
		3, {0x8F, 0xD7, 0xAB}
	},
	{
		2, {0xED, 0xDB},
		4, {0xF0, 0xA6, 0xAB, 0xBF},
		4, {0x5A, 0xD8, 0xFF, 0xDE},
		0, {0x00}
	},
	{
		2, {0xED, 0xDC},
		3, {0xE8, 0x8A, 0x8E},
		2, {0x8E, 0x82},
		3, {0x8F, 0xD7, 0xB6}
	},
	{
		2, {0xED, 0xDD},
		3, {0xE8, 0x8A, 0xA1},
		2, {0xA1, 0x82},
		3, {0x8F, 0xD7, 0xBE}
	},
	{
		2, {0xED, 0xDE},
		3, {0xE8, 0x8A, 0xA3},
		2, {0xA3, 0x82},
		3, {0x8F, 0xD7, 0xBF}
	},
	{
		2, {0xED, 0xDF},
		3, {0xE8, 0x8A, 0xA4},
		2, {0xA4, 0x82},
		3, {0x8F, 0xD7, 0xC0}
	},
	{
		2, {0xED, 0xE0},
		3, {0xE8, 0x8A, 0xA9},
		2, {0xA9, 0x82},
		3, {0x8F, 0xD7, 0xC3}
	},
	{
		2, {0xED, 0xE1},
		3, {0xE8, 0x8A, 0xAE},
		2, {0xAE, 0x82},
		3, {0x8F, 0xD7, 0xC5}
	},
	{
		2, {0xED, 0xE2},
		3, {0xE8, 0x8A, 0xB7},
		2, {0xB7, 0x82},
		3, {0x8F, 0xD7, 0xC9}
	},
	{
		2, {0xED, 0xE3},
		3, {0xE8, 0x8A, 0xBE},
		2, {0xBE, 0x82},
		3, {0x8F, 0xD7, 0xCC}
	},
	{
		2, {0xED, 0xE4},
		3, {0xE8, 0x8A, 0xBF},
		2, {0xBF, 0x82},
		3, {0x8F, 0xD7, 0xCD}
	},
	{
		2, {0xED, 0xE5},
		3, {0xE8, 0x8B, 0x86},
		2, {0xC6, 0x82},
		3, {0x8F, 0xD7, 0xCE}
	},
	{
		2, {0xED, 0xE6},
		3, {0xE8, 0x8B, 0x95},
		2, {0xD5, 0x82},
		3, {0x8F, 0xD7, 0xD0}
	},
	{
		2, {0xED, 0xE7},
		3, {0xE8, 0x8B, 0xBD},
		2, {0xFD, 0x82},
		3, {0x8F, 0xD7, 0xDB}
	},
	{
		2, {0xED, 0xE8},
		3, {0xE8, 0x8B, 0xBE},
		2, {0xFE, 0x82},
		3, {0x8F, 0xD7, 0xDC}
	},
	{
		2, {0xED, 0xE9},
		3, {0xE8, 0x8C, 0x80},
		2, {0x00, 0x83},
		3, {0x8F, 0xD7, 0xDD}
	},
	{
		2, {0xED, 0xEA},
		3, {0xE8, 0x8C, 0x81},
		2, {0x01, 0x83},
		3, {0x8F, 0xD7, 0xDE}
	},
	{
		2, {0xED, 0xEB},
		3, {0xE8, 0x8D, 0xA2},
		2, {0x62, 0x83},
		0, {0x00}
	},
	{
		2, {0xED, 0xEC},
		3, {0xE8, 0x8C, 0xA2},
		2, {0x22, 0x83},
		3, {0x8F, 0xD7, 0xE9}
	},
	{
		2, {0xED, 0xED},
		3, {0xE8, 0x8C, 0xAD},
		2, {0x2D, 0x83},
		3, {0x8F, 0xD7, 0xEB}
	},
	{
		2, {0xED, 0xEE},
		3, {0xE8, 0x8C, 0xBA},
		2, {0x3A, 0x83},
		3, {0x8F, 0xD7, 0xF0}
	},
	{
		2, {0xED, 0xEF},
		3, {0xE8, 0x8D, 0x83},
		2, {0x43, 0x83},
		3, {0x8F, 0xD7, 0xF4}
	},
	{
		2, {0xED, 0xF0},
		3, {0xE8, 0x8D, 0x87},
		2, {0x47, 0x83},
		3, {0x8F, 0xD7, 0xF6}
	},
	{
		2, {0xED, 0xF1},
		3, {0xE8, 0x8D, 0x91},
		2, {0x51, 0x83},
		3, {0x8F, 0xD7, 0xF9}
	},
	{
		2, {0xED, 0xF2},
		3, {0xE8, 0x8D, 0x95},
		2, {0x55, 0x83},
		3, {0x8F, 0xD7, 0xFA}
	},
	{
		2, {0xED, 0xF3},
		3, {0xE8, 0x8D, 0xBD},
		2, {0x7D, 0x83},
		3, {0x8F, 0xD8, 0xA1}
	},
	{
		2, {0xED, 0xF4},
		3, {0xE8, 0x8E, 0x86},
		2, {0x86, 0x83},
		3, {0x8F, 0xD8, 0xA6}
	},
	{
		2, {0xED, 0xF5},
		3, {0xE8, 0x8E, 0x92},
		2, {0x92, 0x83},
		3, {0x8F, 0xD8, 0xA8}
	},
	{
		2, {0xED, 0xF6},
		3, {0xE8, 0x8E, 0x98},
		2, {0x98, 0x83},
		3, {0x8F, 0xD8, 0xAB}
	},
	{
		2, {0xED, 0xF7},
		3, {0xE8, 0x8E, 0xA7},
		2, {0xA7, 0x83},
		3, {0x8F, 0xD8, 0xB1}
	},
	{
		2, {0xED, 0xF8},
		3, {0xE8, 0x8E, 0xA9},
		2, {0xA9, 0x83},
		3, {0x8F, 0xD8, 0xB2}
	},
	{
		2, {0xED, 0xF9},
		3, {0xE8, 0x8E, 0xBF},
		2, {0xBF, 0x83},
		3, {0x8F, 0xD8, 0xB5}
	},
	{
		2, {0xED, 0xFA},
		3, {0xE8, 0x8F, 0x80},
		2, {0xC0, 0x83},
		3, {0x8F, 0xD8, 0xB6}
	},
	{
		2, {0xED, 0xFB},
		3, {0xE8, 0x8F, 0x87},
		2, {0xC7, 0x83},
		3, {0x8F, 0xD8, 0xB7}
	},
	{
		2, {0xED, 0xFC},
		3, {0xE8, 0x8F, 0x8F},
		2, {0xCF, 0x83},
		3, {0x8F, 0xD8, 0xB9}
	},
	{
		2, {0xEE, 0x40},
		3, {0xE8, 0x8F, 0x91},
		2, {0xD1, 0x83},
		3, {0x8F, 0xD8, 0xBB}
	},
	{
		2, {0xEE, 0x41},
		3, {0xE8, 0x8F, 0xA1},
		2, {0xE1, 0x83},
		0, {0x00}
	},
	{
		2, {0xEE, 0x42},
		3, {0xE8, 0x8F, 0xAA},
		2, {0xEA, 0x83},
		3, {0x8F, 0xD8, 0xC0}
	},
	{
		2, {0xEE, 0x43},
		3, {0xE8, 0x90, 0x81},
		2, {0x01, 0x84},
		3, {0x8F, 0xD8, 0xC5}
	},
	{
		2, {0xEE, 0x44},
		3, {0xE8, 0x90, 0x86},
		2, {0x06, 0x84},
		3, {0x8F, 0xD8, 0xC6}
	},
	{
		2, {0xEE, 0x45},
		3, {0xE8, 0x90, 0x8A},
		2, {0x0A, 0x84},
		3, {0x8F, 0xD8, 0xC7}
	},
	{
		2, {0xEE, 0x46},
		3, {0xEF, 0xA9, 0x9F},
		2, {0x5F, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0x47},
		3, {0xE8, 0x91, 0x88},
		2, {0x48, 0x84},
		3, {0x8F, 0xD8, 0xD1}
	},
	{
		2, {0xEE, 0x48},
		3, {0xE8, 0x91, 0x9F},
		2, {0x5F, 0x84},
		0, {0x00}
	},
	{
		2, {0xEE, 0x49},
		3, {0xE8, 0x91, 0xB0},
		2, {0x70, 0x84},
		3, {0x8F, 0xD8, 0xE1}
	},
	{
		2, {0xEE, 0x4A},
		3, {0xE8, 0x91, 0xB3},
		2, {0x73, 0x84},
		3, {0x8F, 0xD8, 0xE2}
	},
	{
		2, {0xEE, 0x4B},
		3, {0xE8, 0x92, 0x85},
		2, {0x85, 0x84},
		3, {0x8F, 0xD8, 0xE9}
	},
	{
		2, {0xEE, 0x4C},
		3, {0xE8, 0x92, 0x9E},
		2, {0x9E, 0x84},
		3, {0x8F, 0xD8, 0xED}
	},
	{
		2, {0xEE, 0x4D},
		3, {0xE8, 0x92, 0xAF},
		2, {0xAF, 0x84},
		3, {0x8F, 0xD8, 0xF2}
	},
	{
		2, {0xEE, 0x4E},
		3, {0xE8, 0x92, 0xB4},
		2, {0xB4, 0x84},
		3, {0x8F, 0xD8, 0xF4}
	},
	{
		2, {0xEE, 0x4F},
		3, {0xE8, 0x92, 0xBA},
		2, {0xBA, 0x84},
		3, {0x8F, 0xD8, 0xF5}
	},
	{
		2, {0xEE, 0x50},
		3, {0xE8, 0x93, 0x80},
		2, {0xC0, 0x84},
		3, {0x8F, 0xD8, 0xF8}
	},
	{
		2, {0xEE, 0x51},
		3, {0xE8, 0x93, 0x82},
		2, {0xC2, 0x84},
		3, {0x8F, 0xD8, 0xF9}
	},
	{
		2, {0xEE, 0x52},
		4, {0xF0, 0xA6, 0xB9, 0x80},
		4, {0x5B, 0xD8, 0x40, 0xDE},
		0, {0x00}
	},
	{
		2, {0xEE, 0x53},
		3, {0xE8, 0x94, 0xB2},
		2, {0x32, 0x85},
		3, {0x8F, 0xD9, 0xA9}
	},
	{
		2, {0xEE, 0x54},
		3, {0xE8, 0x94, 0x9E},
		2, {0x1E, 0x85},
		3, {0x8F, 0xD9, 0xB4}
	},
	{
		2, {0xEE, 0x55},
		3, {0xE8, 0x94, 0xA3},
		2, {0x23, 0x85},
		3, {0x8F, 0xD9, 0xB6}
	},
	{
		2, {0xEE, 0x56},
		3, {0xE8, 0x94, 0xAF},
		2, {0x2F, 0x85},
		3, {0x8F, 0xD9, 0xBC}
	},
	{
		2, {0xEE, 0x57},
		3, {0xE8, 0x95, 0x99},
		2, {0x59, 0x85},
		3, {0x8F, 0xD9, 0xC8}
	},
	{
		2, {0xEE, 0x58},
		3, {0xE8, 0x95, 0xA4},
		2, {0x64, 0x85},
		3, {0x8F, 0xD9, 0xD0}
	},
	{
		2, {0xEE, 0x59},
		3, {0xEF, 0xA8, 0x9F},
		2, {0x1F, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0x5A},
		3, {0xE8, 0x96, 0xAD},
		2, {0xAD, 0x85},
		3, {0x8F, 0xD9, 0xF4}
	},
	{
		2, {0xEE, 0x5B},
		3, {0xE8, 0x95, 0xBA},
		2, {0x7A, 0x85},
		3, {0x8F, 0xD9, 0xD4}
	},
	{
		2, {0xEE, 0x5C},
		3, {0xE8, 0x96, 0x8C},
		2, {0x8C, 0x85},
		3, {0x8F, 0xD9, 0xDD}
	},
	{
		2, {0xEE, 0x5D},
		3, {0xE8, 0x96, 0x8F},
		2, {0x8F, 0x85},
		3, {0x8F, 0xD9, 0xDE}
	},
	{
		2, {0xEE, 0x5E},
		3, {0xE8, 0x96, 0xA2},
		2, {0xA2, 0x85},
		3, {0x8F, 0xD9, 0xE4}
	},
	{
		2, {0xEE, 0x5F},
		3, {0xE8, 0x96, 0xB0},
		2, {0xB0, 0x85},
		0, {0x00}
	},
	{
		2, {0xEE, 0x60},
		3, {0xE8, 0x97, 0x8B},
		2, {0xCB, 0x85},
		3, {0x8F, 0xD9, 0xF2}
	},
	{
		2, {0xEE, 0x61},
		3, {0xE8, 0x97, 0x8E},
		2, {0xCE, 0x85},
		3, {0x8F, 0xD9, 0xF3}
	},
	{
		2, {0xEE, 0x62},
		3, {0xE8, 0x97, 0xAD},
		2, {0xED, 0x85},
		3, {0x8F, 0xD9, 0xFB}
	},
	{
		2, {0xEE, 0x63},
		3, {0xE8, 0x98, 0x92},
		2, {0x12, 0x86},
		3, {0x8F, 0xDA, 0xA9}
	},
	{
		2, {0xEE, 0x64},
		3, {0xE8, 0x97, 0xBF},
		2, {0xFF, 0x85},
		3, {0x8F, 0xDA, 0xA1}
	},
	{
		2, {0xEE, 0x65},
		3, {0xE8, 0x98, 0x84},
		2, {0x04, 0x86},
		3, {0x8F, 0xDA, 0xA3}
	},
	{
		2, {0xEE, 0x66},
		3, {0xE8, 0x98, 0x85},
		2, {0x05, 0x86},
		3, {0x8F, 0xDA, 0xA4}
	},
	{
		2, {0xEE, 0x67},
		3, {0xE8, 0x98, 0x90},
		2, {0x10, 0x86},
		3, {0x8F, 0xDA, 0xA7}
	},
	{
		2, {0xEE, 0x68},
		4, {0xF0, 0xA7, 0x83, 0xB4},
		4, {0x5C, 0xD8, 0xF4, 0xDC},
		0, {0x00}
	},
	{
		2, {0xEE, 0x69},
		3, {0xE8, 0x98, 0x98},
		2, {0x18, 0x86},
		3, {0x8F, 0xDA, 0xAA}
	},
	{
		2, {0xEE, 0x6A},
		3, {0xE8, 0x98, 0xA9},
		2, {0x29, 0x86},
		3, {0x8F, 0xDA, 0xB0}
	},
	{
		2, {0xEE, 0x6B},
		3, {0xE8, 0x98, 0xB8},
		2, {0x38, 0x86},
		3, {0x8F, 0xDA, 0xB2}
	},
	{
		2, {0xEE, 0x6C},
		3, {0xE8, 0x99, 0x97},
		2, {0x57, 0x86},
		3, {0x8F, 0xDA, 0xBC}
	},
	{
		2, {0xEE, 0x6D},
		3, {0xE8, 0x99, 0x9B},
		2, {0x5B, 0x86},
		0, {0x00}
	},
	{
		2, {0xEE, 0x6E},
		3, {0xEF, 0xA4, 0xB6},
		2, {0x36, 0xF9},
		0, {0x00}
	},
	{
		2, {0xEE, 0x6F},
		3, {0xE8, 0x99, 0xA2},
		2, {0x62, 0x86},
		3, {0x8F, 0xDA, 0xC2}
	},
	{
		2, {0xEE, 0x70},
		3, {0xE4, 0x96, 0x9D},
		2, {0x9D, 0x45},
		0, {0x00}
	},
	{
		2, {0xEE, 0x71},
		3, {0xE8, 0x99, 0xAC},
		2, {0x6C, 0x86},
		3, {0x8F, 0xDA, 0xC6}
	},
	{
		2, {0xEE, 0x72},
		3, {0xE8, 0x99, 0xB5},
		2, {0x75, 0x86},
		3, {0x8F, 0xDA, 0xC8}
	},
	{
		2, {0xEE, 0x73},
		3, {0xE8, 0x9A, 0x98},
		2, {0x98, 0x86},
		3, {0x8F, 0xDA, 0xCF}
	},
	{
		2, {0xEE, 0x74},
		3, {0xE8, 0x9A, 0xB8},
		2, {0xB8, 0x86},
		3, {0x8F, 0xDA, 0xDC}
	},
	{
		2, {0xEE, 0x75},
		3, {0xE8, 0x9B, 0xBA},
		2, {0xFA, 0x86},
		3, {0x8F, 0xDA, 0xEE}
	},
	{
		2, {0xEE, 0x76},
		3, {0xE8, 0x9B, 0xBC},
		2, {0xFC, 0x86},
		3, {0x8F, 0xDA, 0xEF}
	},
	{
		2, {0xEE, 0x77},
		3, {0xE8, 0x9B, 0xBD},
		2, {0xFD, 0x86},
		3, {0x8F, 0xDA, 0xF0}
	},
	{
		2, {0xEE, 0x78},
		3, {0xE8, 0x9C, 0x8B},
		2, {0x0B, 0x87},
		3, {0x8F, 0xDA, 0xF4}
	},
	{
		2, {0xEE, 0x79},
		3, {0xE8, 0x9D, 0xB1},
		2, {0x71, 0x87},
		3, {0x8F, 0xDB, 0xB5}
	},
	{
		2, {0xEE, 0x7A},
		3, {0xE8, 0x9E, 0x87},
		2, {0x87, 0x87},
		3, {0x8F, 0xDB, 0xBC}
	},
	{
		2, {0xEE, 0x7B},
		3, {0xE8, 0x9E, 0x88},
		2, {0x88, 0x87},
		3, {0x8F, 0xDB, 0xBD}
	},
	{
		2, {0xEE, 0x7C},
		3, {0xE8, 0x9E, 0xAC},
		2, {0xAC, 0x87},
		3, {0x8F, 0xDB, 0xCB}
	},
	{
		2, {0xEE, 0x7D},
		3, {0xE8, 0x9E, 0xAD},
		2, {0xAD, 0x87},
		3, {0x8F, 0xDB, 0xCC}
	},
	{
		2, {0xEE, 0x7E},
		3, {0xE8, 0x9E, 0xB5},
		2, {0xB5, 0x87},
		3, {0x8F, 0xDB, 0xCF}
	},
	{
		2, {0xEE, 0x80},
		3, {0xE4, 0x97, 0xAA},
		2, {0xEA, 0x45},
		0, {0x00}
	},
	{
		2, {0xEE, 0x81},
		3, {0xE8, 0x9F, 0x96},
		2, {0xD6, 0x87},
		3, {0x8F, 0xDB, 0xD8}
	},
	{
		2, {0xEE, 0x82},
		3, {0xE8, 0x9F, 0xAC},
		2, {0xEC, 0x87},
		0, {0x00}
	},
	{
		2, {0xEE, 0x83},
		3, {0xE8, 0xA0, 0x86},
		2, {0x06, 0x88},
		3, {0x8F, 0xDB, 0xEA}
	},
	{
		2, {0xEE, 0x84},
		3, {0xE8, 0xA0, 0x8A},
		2, {0x0A, 0x88},
		3, {0x8F, 0xDB, 0xEC}
	},
	{
		2, {0xEE, 0x85},
		3, {0xE8, 0xA0, 0x90},
		2, {0x10, 0x88},
		3, {0x8F, 0xDB, 0xEE}
	},
	{
		2, {0xEE, 0x86},
		3, {0xE8, 0xA0, 0x94},
		2, {0x14, 0x88},
		3, {0x8F, 0xDB, 0xF2}
	},
	{
		2, {0xEE, 0x87},
		3, {0xE8, 0xA0, 0x9F},
		2, {0x1F, 0x88},
		3, {0x8F, 0xDB, 0xF8}
	},
	{
		2, {0xEE, 0x88},
		3, {0xE8, 0xA2, 0x98},
		2, {0x98, 0x88},
		3, {0x8F, 0xDC, 0xB9}
	},
	{
		2, {0xEE, 0x89},
		3, {0xE8, 0xA2, 0xAA},
		2, {0xAA, 0x88},
		3, {0x8F, 0xDC, 0xC0}
	},
	{
		2, {0xEE, 0x8A},
		3, {0xE8, 0xA3, 0x8A},
		2, {0xCA, 0x88},
		3, {0x8F, 0xDC, 0xC5}
	},
	{
		2, {0xEE, 0x8B},
		3, {0xE8, 0xA3, 0x8E},
		2, {0xCE, 0x88},
		3, {0x8F, 0xDC, 0xC9}
	},
	{
		2, {0xEE, 0x8C},
		4, {0xF0, 0xA7, 0x9A, 0x84},
		4, {0x5D, 0xD8, 0x84, 0xDE},
		0, {0x00}
	},
	{
		2, {0xEE, 0x8D},
		3, {0xE8, 0xA3, 0xB5},
		2, {0xF5, 0x88},
		3, {0x8F, 0xDC, 0xD3}
	},
	{
		2, {0xEE, 0x8E},
		3, {0xE8, 0xA4, 0x9C},
		2, {0x1C, 0x89},
		3, {0x8F, 0xDC, 0xDF}
	},
	{
		2, {0xEE, 0x8F},
		3, {0xEF, 0xA9, 0xA0},
		2, {0x60, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0x90},
		3, {0xE8, 0xA4, 0x98},
		2, {0x18, 0x89},
		3, {0x8F, 0xDC, 0xDC}
	},
	{
		2, {0xEE, 0x91},
		3, {0xE8, 0xA4, 0x99},
		2, {0x19, 0x89},
		3, {0x8F, 0xDC, 0xDD}
	},
	{
		2, {0xEE, 0x92},
		3, {0xE8, 0xA4, 0x9A},
		2, {0x1A, 0x89},
		3, {0x8F, 0xDC, 0xDE}
	},
	{
		2, {0xEE, 0x93},
		3, {0xE8, 0xA4, 0xA7},
		2, {0x27, 0x89},
		3, {0x8F, 0xDC, 0xE2}
	},
	{
		2, {0xEE, 0x94},
		3, {0xE8, 0xA4, 0xB0},
		2, {0x30, 0x89},
		3, {0x8F, 0xDC, 0xE4}
	},
	{
		2, {0xEE, 0x95},
		3, {0xE8, 0xA4, 0xB2},
		2, {0x32, 0x89},
		3, {0x8F, 0xDC, 0xE6}
	},
	{
		2, {0xEE, 0x96},
		3, {0xE8, 0xA4, 0xB9},
		2, {0x39, 0x89},
		3, {0x8F, 0xDC, 0xE8}
	},
	{
		2, {0xEE, 0x97},
		3, {0xE8, 0xA5, 0x80},
		2, {0x40, 0x89},
		3, {0x8F, 0xDC, 0xEB}
	},
	{
		2, {0xEE, 0x98},
		3, {0xE8, 0xA6, 0x94},
		2, {0x94, 0x89},
		3, {0x8F, 0xDD, 0xA7}
	},
	{
		2, {0xEE, 0x99},
		3, {0xEF, 0xA9, 0xA1},
		2, {0x61, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0x9A},
		3, {0xE8, 0xA7, 0x94},
		2, {0xD4, 0x89},
		3, {0x8F, 0xDD, 0xB4}
	},
	{
		2, {0xEE, 0x9B},
		3, {0xE8, 0xA7, 0xA5},
		2, {0xE5, 0x89},
		3, {0x8F, 0xDD, 0xB9}
	},
	{
		2, {0xEE, 0x9C},
		3, {0xE8, 0xA7, 0xB6},
		2, {0xF6, 0x89},
		3, {0x8F, 0xDD, 0xBF}
	},
	{
		2, {0xEE, 0x9D},
		3, {0xE8, 0xA8, 0x92},
		2, {0x12, 0x8A},
		3, {0x8F, 0xDD, 0xC8}
	},
	{
		2, {0xEE, 0x9E},
		3, {0xE8, 0xA8, 0x95},
		2, {0x15, 0x8A},
		3, {0x8F, 0xDD, 0xCA}
	},
	{
		2, {0xEE, 0x9F},
		3, {0xE8, 0xA8, 0xA2},
		2, {0x22, 0x8A},
		3, {0x8F, 0xDD, 0xCD}
	},
	{
		2, {0xEE, 0xA0},
		3, {0xE8, 0xA8, 0xB7},
		2, {0x37, 0x8A},
		3, {0x8F, 0xDD, 0xD4}
	},
	{
		2, {0xEE, 0xA1},
		3, {0xE8, 0xA9, 0x87},
		2, {0x47, 0x8A},
		3, {0x8F, 0xDD, 0xDA}
	},
	{
		2, {0xEE, 0xA2},
		3, {0xE8, 0xA9, 0x8E},
		2, {0x4E, 0x8A},
		3, {0x8F, 0xDD, 0xDD}
	},
	{
		2, {0xEE, 0xA3},
		3, {0xE8, 0xA9, 0x9D},
		2, {0x5D, 0x8A},
		3, {0x8F, 0xDD, 0xE3}
	},
	{
		2, {0xEE, 0xA4},
		3, {0xE8, 0xA9, 0xA1},
		2, {0x61, 0x8A},
		3, {0x8F, 0xDD, 0xE4}
	},
	{
		2, {0xEE, 0xA5},
		3, {0xE8, 0xA9, 0xB5},
		2, {0x75, 0x8A},
		3, {0x8F, 0xDD, 0xE7}
	},
	{
		2, {0xEE, 0xA6},
		3, {0xE8, 0xA9, 0xB9},
		2, {0x79, 0x8A},
		3, {0x8F, 0xDD, 0xEA}
	},
	{
		2, {0xEE, 0xA7},
		3, {0xE8, 0xAA, 0xA7},
		2, {0xA7, 0x8A},
		3, {0x8F, 0xDD, 0xFA}
	},
	{
		2, {0xEE, 0xA8},
		3, {0xE8, 0xAB, 0x90},
		2, {0xD0, 0x8A},
		0, {0x00}
	},
	{
		2, {0xEE, 0xA9},
		3, {0xE8, 0xAB, 0x9F},
		2, {0xDF, 0x8A},
		3, {0x8F, 0xDE, 0xB0}
	},
	{
		2, {0xEE, 0xAA},
		3, {0xE8, 0xAB, 0xB4},
		2, {0xF4, 0x8A},
		3, {0x8F, 0xDE, 0xB3}
	},
	{
		2, {0xEE, 0xAB},
		3, {0xE8, 0xAB, 0xB6},
		2, {0xF6, 0x8A},
		3, {0x8F, 0xDE, 0xB5}
	},
	{
		2, {0xEE, 0xAC},
		3, {0xEF, 0xA8, 0xA2},
		2, {0x22, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0xAD},
		3, {0xEF, 0xA9, 0xA2},
		2, {0x62, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0xAE},
		3, {0xEF, 0xA9, 0xA3},
		2, {0x63, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0xAF},
		3, {0xE8, 0xAD, 0x86},
		2, {0x46, 0x8B},
		3, {0x8F, 0xDE, 0xC8}
	},
	{
		2, {0xEE, 0xB0},
		3, {0xE8, 0xAD, 0x94},
		2, {0x54, 0x8B},
		3, {0x8F, 0xDE, 0xCC}
	},
	{
		2, {0xEE, 0xB1},
		3, {0xE8, 0xAD, 0x99},
		2, {0x59, 0x8B},
		3, {0x8F, 0xDE, 0xCD}
	},
	{
		2, {0xEE, 0xB2},
		3, {0xE8, 0xAD, 0xA9},
		2, {0x69, 0x8B},
		0, {0x00}
	},
	{
		2, {0xEE, 0xB3},
		3, {0xE8, 0xAE, 0x9D},
		2, {0x9D, 0x8B},
		0, {0x00}
	},
	{
		2, {0xEE, 0xB4},
		3, {0xE8, 0xB1, 0x89},
		2, {0x49, 0x8C},
		3, {0x8F, 0xDE, 0xE8}
	},
	{
		2, {0xEE, 0xB5},
		3, {0xE8, 0xB1, 0xA8},
		2, {0x68, 0x8C},
		3, {0x8F, 0xDE, 0xF6}
	},
	{
		2, {0xEE, 0xB6},
		3, {0xEF, 0xA9, 0xA4},
		2, {0x64, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0xB7},
		3, {0xE8, 0xB3, 0xA1},
		2, {0xE1, 0x8C},
		3, {0x8F, 0xDF, 0xB5}
	},
	{
		2, {0xEE, 0xB8},
		3, {0xE8, 0xB3, 0xB4},
		2, {0xF4, 0x8C},
		0, {0x00}
	},
	{
		2, {0xEE, 0xB9},
		3, {0xE8, 0xB3, 0xB8},
		2, {0xF8, 0x8C},
		3, {0x8F, 0xDF, 0xBD}
	},
	{
		2, {0xEE, 0xBA},
		3, {0xE8, 0xB3, 0xBE},
		2, {0xFE, 0x8C},
		3, {0x8F, 0xDF, 0xBE}
	},
	{
		2, {0xEE, 0xBB},
		3, {0xEF, 0xA9, 0xA5},
		2, {0x65, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0xBC},
		3, {0xE8, 0xB4, 0x92},
		2, {0x12, 0x8D},
		3, {0x8F, 0xDF, 0xC3}
	},
	{
		2, {0xEE, 0xBD},
		3, {0xE8, 0xB4, 0x9B},
		2, {0x1B, 0x8D},
		3, {0x8F, 0xDF, 0xC5}
	},
	{
		2, {0xEE, 0xBE},
		3, {0xE8, 0xB6, 0xAF},
		2, {0xAF, 0x8D},
		3, {0x8F, 0xDF, 0xD8}
	},
	{
		2, {0xEE, 0xBF},
		3, {0xE8, 0xB7, 0x8E},
		2, {0xCE, 0x8D},
		3, {0x8F, 0xDF, 0xE4}
	},
	{
		2, {0xEE, 0xC0},
		3, {0xE8, 0xB7, 0x91},
		2, {0xD1, 0x8D},
		3, {0x8F, 0xDF, 0xE5}
	},
	{
		2, {0xEE, 0xC1},
		3, {0xE8, 0xB7, 0x97},
		2, {0xD7, 0x8D},
		3, {0x8F, 0xDF, 0xE8}
	},
	{
		2, {0xEE, 0xC2},
		3, {0xE8, 0xB8, 0xA0},
		2, {0x20, 0x8E},
		3, {0x8F, 0xDF, 0xFC}
	},
	{
		2, {0xEE, 0xC3},
		3, {0xE8, 0xB8, 0xA3},
		2, {0x23, 0x8E},
		3, {0x8F, 0xE0, 0xA1}
	},
	{
		2, {0xEE, 0xC4},
		3, {0xE8, 0xB8, 0xBD},
		2, {0x3D, 0x8E},
		3, {0x8F, 0xE0, 0xAA}
	},
	{
		2, {0xEE, 0xC5},
		3, {0xE8, 0xB9, 0xB0},
		2, {0x70, 0x8E},
		3, {0x8F, 0xE0, 0xBC}
	},
	{
		2, {0xEE, 0xC6},
		3, {0xE8, 0xB9, 0xBB},
		2, {0x7B, 0x8E},
		3, {0x8F, 0xE0, 0xC0}
	},
	{
		2, {0xEE, 0xC7},
		4, {0xF0, 0xA8, 0x89, 0xB7},
		4, {0x60, 0xD8, 0x77, 0xDE},
		0, {0x00}
	},
	{
		2, {0xEE, 0xC8},
		3, {0xE8, 0xBB, 0x80},
		2, {0xC0, 0x8E},
		3, {0x8F, 0xE0, 0xD4}
	},
	{
		2, {0xEE, 0xC9},
		3, {0xE4, 0xA1, 0x84},
		2, {0x44, 0x48},
		0, {0x00}
	},
	{
		2, {0xEE, 0xCA},
		3, {0xE8, 0xBB, 0xBA},
		2, {0xFA, 0x8E},
		3, {0x8F, 0xE0, 0xE3}
	},
	{
		2, {0xEE, 0xCB},
		3, {0xE8, 0xBC, 0x9E},
		2, {0x1E, 0x8F},
		3, {0x8F, 0xE0, 0xEE}
	},
	{
		2, {0xEE, 0xCC},
		3, {0xE8, 0xBC, 0xAD},
		2, {0x2D, 0x8F},
		3, {0x8F, 0xE0, 0xF6}
	},
	{
		2, {0xEE, 0xCD},
		3, {0xE8, 0xBC, 0xB6},
		2, {0x36, 0x8F},
		3, {0x8F, 0xE0, 0xFA}
	},
	{
		2, {0xEE, 0xCE},
		3, {0xE8, 0xBD, 0x94},
		2, {0x54, 0x8F},
		3, {0x8F, 0xE1, 0xA7}
	},
	{
		2, {0xEE, 0xCF},
		4, {0xF0, 0xA8, 0x8F, 0x8D},
		4, {0x60, 0xD8, 0xCD, 0xDF},
		0, {0x00}
	},
	{
		2, {0xEE, 0xD0},
		3, {0xE8, 0xBE, 0xA6},
		2, {0xA6, 0x8F},
		3, {0x8F, 0xE1, 0xB2}
	},
	{
		2, {0xEE, 0xD1},
		3, {0xE8, 0xBE, 0xB5},
		2, {0xB5, 0x8F},
		3, {0x8F, 0xE1, 0xB3}
	},
	{
		2, {0xEE, 0xD2},
		3, {0xE8, 0xBF, 0xA4},
		2, {0xE4, 0x8F},
		3, {0x8F, 0xE1, 0xC3}
	},
	{
		2, {0xEE, 0xD3},
		3, {0xE8, 0xBF, 0xA8},
		2, {0xE8, 0x8F},
		3, {0x8F, 0xE1, 0xC4}
	},
	{
		2, {0xEE, 0xD4},
		3, {0xE8, 0xBF, 0xAE},
		2, {0xEE, 0x8F},
		3, {0x8F, 0xE1, 0xC5}
	},
	{
		2, {0xEE, 0xD5},
		3, {0xE9, 0x80, 0x88},
		2, {0x08, 0x90},
		3, {0x8F, 0xE1, 0xCD}
	},
	{
		2, {0xEE, 0xD6},
		3, {0xE9, 0x80, 0xAD},
		2, {0x2D, 0x90},
		3, {0x8F, 0xE1, 0xD6}
	},
	{
		2, {0xEE, 0xD7},
		3, {0xEF, 0xA9, 0xA7},
		2, {0x67, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0xD8},
		3, {0xE9, 0x82, 0x88},
		2, {0x88, 0x90},
		3, {0x8F, 0xE1, 0xE8}
	},
	{
		2, {0xEE, 0xD9},
		3, {0xE9, 0x82, 0x95},
		2, {0x95, 0x90},
		3, {0x8F, 0xE1, 0xED}
	},
	{
		2, {0xEE, 0xDA},
		3, {0xE9, 0x82, 0x97},
		2, {0x97, 0x90},
		3, {0x8F, 0xE1, 0xEE}
	},
	{
		2, {0xEE, 0xDB},
		3, {0xE9, 0x82, 0x99},
		2, {0x99, 0x90},
		3, {0x8F, 0xE1, 0xF0}
	},
	{
		2, {0xEE, 0xDC},
		3, {0xE9, 0x82, 0x9B},
		2, {0x9B, 0x90},
		3, {0x8F, 0xE1, 0xF1}
	},
	{
		2, {0xEE, 0xDD},
		3, {0xE9, 0x82, 0xA2},
		2, {0xA2, 0x90},
		3, {0x8F, 0xE1, 0xF4}
	},
	{
		2, {0xEE, 0xDE},
		3, {0xE9, 0x82, 0xB3},
		2, {0xB3, 0x90},
		3, {0x8F, 0xE1, 0xF8}
	},
	{
		2, {0xEE, 0xDF},
		3, {0xE9, 0x82, 0xBE},
		2, {0xBE, 0x90},
		3, {0x8F, 0xE1, 0xFD}
	},
	{
		2, {0xEE, 0xE0},
		3, {0xE9, 0x83, 0x84},
		2, {0xC4, 0x90},
		3, {0x8F, 0xE2, 0xA1}
	},
	{
		2, {0xEE, 0xE1},
		3, {0xE9, 0x83, 0x85},
		2, {0xC5, 0x90},
		3, {0x8F, 0xE2, 0xA2}
	},
	{
		2, {0xEE, 0xE2},
		3, {0xE9, 0x83, 0x87},
		2, {0xC7, 0x90},
		3, {0x8F, 0xE2, 0xA3}
	},
	{
		2, {0xEE, 0xE3},
		3, {0xE9, 0x83, 0x97},
		2, {0xD7, 0x90},
		3, {0x8F, 0xE2, 0xA6}
	},
	{
		2, {0xEE, 0xE4},
		3, {0xE9, 0x83, 0x9D},
		2, {0xDD, 0x90},
		3, {0x8F, 0xE2, 0xAA}
	},
	{
		2, {0xEE, 0xE5},
		3, {0xE9, 0x83, 0x9E},
		2, {0xDE, 0x90},
		0, {0x00}
	},
	{
		2, {0xEE, 0xE6},
		3, {0xE9, 0x83, 0xAF},
		2, {0xEF, 0x90},
		3, {0x8F, 0xE2, 0xB0}
	},
	{
		2, {0xEE, 0xE7},
		3, {0xE9, 0x83, 0xB4},
		2, {0xF4, 0x90},
		3, {0x8F, 0xE2, 0xB2}
	},
	{
		2, {0xEE, 0xE8},
		3, {0xEF, 0xA8, 0xA6},
		2, {0x26, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEE, 0xE9},
		3, {0xE9, 0x84, 0x94},
		2, {0x14, 0x91},
		3, {0x8F, 0xE2, 0xBC}
	},
	{
		2, {0xEE, 0xEA},
		3, {0xE9, 0x84, 0x95},
		2, {0x15, 0x91},
		0, {0x00}
	},
	{
		2, {0xEE, 0xEB},
		3, {0xE9, 0x84, 0x96},
		2, {0x16, 0x91},
		3, {0x8F, 0xE2, 0xBD}
	},
	{
		2, {0xEE, 0xEC},
		3, {0xE9, 0x84, 0xA2},
		2, {0x22, 0x91},
		3, {0x8F, 0xE2, 0xC5}
	},
	{
		2, {0xEE, 0xED},
		3, {0xE9, 0x84, 0xA3},
		2, {0x23, 0x91},
		3, {0x8F, 0xE2, 0xC6}
	},
	{
		2, {0xEE, 0xEE},
		3, {0xE9, 0x84, 0xA7},
		2, {0x27, 0x91},
		3, {0x8F, 0xE2, 0xC7}
	},
	{
		2, {0xEE, 0xEF},
		3, {0xE9, 0x84, 0xAF},
		2, {0x2F, 0x91},
		3, {0x8F, 0xE2, 0xCA}
	},
	{
		2, {0xEE, 0xF0},
		3, {0xE9, 0x84, 0xB1},
		2, {0x31, 0x91},
		3, {0x8F, 0xE2, 0xCB}
	},
	{
		2, {0xEE, 0xF1},
		3, {0xE9, 0x84, 0xB4},
		2, {0x34, 0x91},
		3, {0x8F, 0xE2, 0xCC}
	},
	{
		2, {0xEE, 0xF2},
		3, {0xE9, 0x84, 0xBD},
		2, {0x3D, 0x91},
		3, {0x8F, 0xE2, 0xD2}
	},
	{
		2, {0xEE, 0xF3},
		3, {0xE9, 0x85, 0x88},
		2, {0x48, 0x91},
		3, {0x8F, 0xE2, 0xD5}
	},
	{
		2, {0xEE, 0xF4},
		3, {0xE9, 0x85, 0x9B},
		2, {0x5B, 0x91},
		3, {0x8F, 0xE2, 0xDB}
	},
	{
		2, {0xEE, 0xF5},
		3, {0xE9, 0x86, 0x83},
		2, {0x83, 0x91},
		3, {0x8F, 0xE2, 0xE5}
	},
	{
		2, {0xEE, 0xF6},
		3, {0xE9, 0x86, 0x9E},
		2, {0x9E, 0x91},
		3, {0x8F, 0xE2, 0xEF}
	},
	{
		2, {0xEE, 0xF7},
		3, {0xE9, 0x86, 0xAC},
		2, {0xAC, 0x91},
		3, {0x8F, 0xE2, 0xF3}
	},
	{
		2, {0xEE, 0xF8},
		3, {0xE9, 0x86, 0xB1},
		2, {0xB1, 0x91},
		3, {0x8F, 0xE2, 0xF7}
	},
	{
		2, {0xEE, 0xF9},
		3, {0xE9, 0x86, 0xBC},
		2, {0xBC, 0x91},
		3, {0x8F, 0xE2, 0xFC}
	},
	{
		2, {0xEE, 0xFA},
		3, {0xE9, 0x87, 0x97},
		2, {0xD7, 0x91},
		3, {0x8F, 0xE3, 0xA6}
	},
	{
		2, {0xEE, 0xFB},
		3, {0xE9, 0x87, 0xBB},
		2, {0xFB, 0x91},
		3, {0x8F, 0xE3, 0xB6}
	},
	{
		2, {0xEE, 0xFC},
		3, {0xE9, 0x87, 0xA4},
		2, {0xE4, 0x91},
		3, {0x8F, 0xE3, 0xAA}
	},
	{
		2, {0xEF, 0x40},
		3, {0xE9, 0x87, 0xA5},
		2, {0xE5, 0x91},
		3, {0x8F, 0xE3, 0xAB}
	},
	{
		2, {0xEF, 0x41},
		3, {0xE9, 0x87, 0xAD},
		2, {0xED, 0x91},
		3, {0x8F, 0xE3, 0xAF}
	},
	{
		2, {0xEF, 0x42},
		3, {0xE9, 0x87, 0xB1},
		2, {0xF1, 0x91},
		3, {0x8F, 0xE3, 0xB3}
	},
	{
		2, {0xEF, 0x43},
		3, {0xE9, 0x88, 0x87},
		2, {0x07, 0x92},
		3, {0x8F, 0xE3, 0xBD}
	},
	{
		2, {0xEF, 0x44},
		3, {0xE9, 0x88, 0x90},
		2, {0x10, 0x92},
		3, {0x8F, 0xE3, 0xC1}
	},
	{
		2, {0xEF, 0x45},
		3, {0xE9, 0x88, 0xB8},
		2, {0x38, 0x92},
		3, {0x8F, 0xE3, 0xD3}
	},
	{
		2, {0xEF, 0x46},
		3, {0xE9, 0x88, 0xB9},
		2, {0x39, 0x92},
		3, {0x8F, 0xE3, 0xD4}
	},
	{
		2, {0xEF, 0x47},
		3, {0xE9, 0x88, 0xBA},
		2, {0x3A, 0x92},
		3, {0x8F, 0xE3, 0xD5}
	},
	{
		2, {0xEF, 0x48},
		3, {0xE9, 0x88, 0xBC},
		2, {0x3C, 0x92},
		3, {0x8F, 0xE3, 0xD6}
	},
	{
		2, {0xEF, 0x49},
		3, {0xE9, 0x89, 0x80},
		2, {0x40, 0x92},
		3, {0x8F, 0xE3, 0xD8}
	},
	{
		2, {0xEF, 0x4A},
		3, {0xE9, 0x89, 0x83},
		2, {0x43, 0x92},
		3, {0x8F, 0xE3, 0xDA}
	},
	{
		2, {0xEF, 0x4B},
		3, {0xE9, 0x89, 0x8F},
		2, {0x4F, 0x92},
		3, {0x8F, 0xE3, 0xE0}
	},
	{
		2, {0xEF, 0x4C},
		3, {0xE9, 0x89, 0xB8},
		2, {0x78, 0x92},
		3, {0x8F, 0xE3, 0xF2}
	},
	{
		2, {0xEF, 0x4D},
		3, {0xE9, 0x8A, 0x88},
		2, {0x88, 0x92},
		3, {0x8F, 0xE3, 0xF8}
	},
	{
		2, {0xEF, 0x4E},
		3, {0xE9, 0x8B, 0x82},
		2, {0xC2, 0x92},
		3, {0x8F, 0xE4, 0xB4}
	},
	{
		2, {0xEF, 0x4F},
		3, {0xE9, 0x8B, 0x8B},
		2, {0xCB, 0x92},
		3, {0x8F, 0xE4, 0xBA}
	},
	{
		2, {0xEF, 0x50},
		3, {0xE9, 0x8B, 0x8C},
		2, {0xCC, 0x92},
		3, {0x8F, 0xE4, 0xBB}
	},
	{
		2, {0xEF, 0x51},
		3, {0xE9, 0x8B, 0x93},
		2, {0xD3, 0x92},
		3, {0x8F, 0xE4, 0xBF}
	},
	{
		2, {0xEF, 0x52},
		3, {0xE9, 0x8B, 0xA0},
		2, {0xE0, 0x92},
		3, {0x8F, 0xE4, 0xC7}
	},
	{
		2, {0xEF, 0x53},
		3, {0xE9, 0x8B, 0xBF},
		2, {0xFF, 0x92},
		3, {0x8F, 0xE4, 0xD2}
	},
	{
		2, {0xEF, 0x54},
		3, {0xE9, 0x8C, 0x84},
		2, {0x04, 0x93},
		0, {0x00}
	},
	{
		2, {0xEF, 0x55},
		3, {0xE9, 0x8C, 0x9F},
		2, {0x1F, 0x93},
		3, {0x8F, 0xE4, 0xDD}
	},
	{
		2, {0xEF, 0x56},
		3, {0xE9, 0x8C, 0xA1},
		2, {0x21, 0x93},
		3, {0x8F, 0xE4, 0xDE}
	},
	{
		2, {0xEF, 0x57},
		3, {0xE9, 0x8C, 0xA5},
		2, {0x25, 0x93},
		3, {0x8F, 0xE4, 0xE0}
	},
	{
		2, {0xEF, 0x58},
		3, {0xE9, 0x8D, 0x88},
		2, {0x48, 0x93},
		3, {0x8F, 0xE4, 0xE9}
	},
	{
		2, {0xEF, 0x59},
		3, {0xE9, 0x8D, 0x89},
		2, {0x49, 0x93},
		3, {0x8F, 0xE4, 0xEA}
	},
	{
		2, {0xEF, 0x5A},
		3, {0xE9, 0x8D, 0x8A},
		2, {0x4A, 0x93},
		0, {0x00}
	},
	{
		2, {0xEF, 0x5B},
		3, {0xE9, 0x8D, 0xA4},
		2, {0x64, 0x93},
		3, {0x8F, 0xE4, 0xF3}
	},
	{
		2, {0xEF, 0x5C},
		3, {0xE9, 0x8D, 0xA5},
		2, {0x65, 0x93},
		3, {0x8F, 0xE4, 0xF4}
	},
	{
		2, {0xEF, 0x5D},
		3, {0xE9, 0x8D, 0xAA},
		2, {0x6A, 0x93},
		3, {0x8F, 0xE4, 0xF7}
	},
	{
		2, {0xEF, 0x5E},
		3, {0xE9, 0x8D, 0xB0},
		2, {0x70, 0x93},
		3, {0x8F, 0xE4, 0xFA}
	},
	{
		2, {0xEF, 0x5F},
		3, {0xE9, 0x8E, 0x9B},
		2, {0x9B, 0x93},
		3, {0x8F, 0xE5, 0xAF}
	},
	{
		2, {0xEF, 0x60},
		3, {0xE9, 0x8E, 0xA3},
		2, {0xA3, 0x93},
		3, {0x8F, 0xE5, 0xB2}
	},
	{
		2, {0xEF, 0x61},
		3, {0xE9, 0x8E, 0xBA},
		2, {0xBA, 0x93},
		3, {0x8F, 0xE5, 0xBA}
	},
	{
		2, {0xEF, 0x62},
		3, {0xE9, 0x8F, 0x86},
		2, {0xC6, 0x93},
		3, {0x8F, 0xE5, 0xBF}
	},
	{
		2, {0xEF, 0x63},
		3, {0xE9, 0x8F, 0x9E},
		2, {0xDE, 0x93},
		3, {0x8F, 0xE5, 0xC9}
	},
	{
		2, {0xEF, 0x64},
		3, {0xE9, 0x8F, 0x9F},
		2, {0xDF, 0x93},
		3, {0x8F, 0xE5, 0xCA}
	},
	{
		2, {0xEF, 0x65},
		3, {0xE9, 0x90, 0x84},
		2, {0x04, 0x94},
		3, {0x8F, 0xE5, 0xD6}
	},
	{
		2, {0xEF, 0x66},
		3, {0xE9, 0x8F, 0xBD},
		2, {0xFD, 0x93},
		3, {0x8F, 0xE5, 0xD3}
	},
	{
		2, {0xEF, 0x67},
		3, {0xE9, 0x90, 0xB3},
		2, {0x33, 0x94},
		3, {0x8F, 0xE5, 0xE4}
	},
	{
		2, {0xEF, 0x68},
		3, {0xE9, 0x91, 0x8A},
		2, {0x4A, 0x94},
		3, {0x8F, 0xE5, 0xEC}
	},
	{
		2, {0xEF, 0x69},
		3, {0xE9, 0x91, 0xA3},
		2, {0x63, 0x94},
		3, {0x8F, 0xE5, 0xF3}
	},
	{
		2, {0xEF, 0x6A},
		3, {0xE9, 0x91, 0xAB},
		2, {0x6B, 0x94},
		3, {0x8F, 0xE5, 0xF5}
	},
	{
		2, {0xEF, 0x6B},
		3, {0xE9, 0x91, 0xB1},
		2, {0x71, 0x94},
		3, {0x8F, 0xE5, 0xF9}
	},
	{
		2, {0xEF, 0x6C},
		3, {0xE9, 0x91, 0xB2},
		2, {0x72, 0x94},
		3, {0x8F, 0xE5, 0xFA}
	},
	{
		2, {0xEF, 0x6D},
		3, {0xE9, 0x96, 0x8E},
		2, {0x8E, 0x95},
		3, {0x8F, 0xE6, 0xA6}
	},
	{
		2, {0xEF, 0x6E},
		3, {0xE9, 0x96, 0x9F},
		2, {0x9F, 0x95},
		3, {0x8F, 0xE6, 0xA9}
	},
	{
		2, {0xEF, 0x6F},
		3, {0xE9, 0x96, 0xA6},
		2, {0xA6, 0x95},
		3, {0x8F, 0xE6, 0xAB}
	},
	{
		2, {0xEF, 0x70},
		3, {0xE9, 0x96, 0xA9},
		2, {0xA9, 0x95},
		3, {0x8F, 0xE6, 0xAC}
	},
	{
		2, {0xEF, 0x71},
		3, {0xE9, 0x96, 0xAC},
		2, {0xAC, 0x95},
		3, {0x8F, 0xE6, 0xAE}
	},
	{
		2, {0xEF, 0x72},
		3, {0xE9, 0x96, 0xB6},
		2, {0xB6, 0x95},
		3, {0x8F, 0xE6, 0xB0}
	},
	{
		2, {0xEF, 0x73},
		3, {0xE9, 0x96, 0xBD},
		2, {0xBD, 0x95},
		3, {0x8F, 0xE6, 0xB2}
	},
	{
		2, {0xEF, 0x74},
		3, {0xE9, 0x97, 0x8B},
		2, {0xCB, 0x95},
		3, {0x8F, 0xE6, 0xB7}
	},
	{
		2, {0xEF, 0x75},
		3, {0xE9, 0x97, 0x90},
		2, {0xD0, 0x95},
		3, {0x8F, 0xE6, 0xB8}
	},
	{
		2, {0xEF, 0x76},
		3, {0xE9, 0x97, 0x93},
		2, {0xD3, 0x95},
		3, {0x8F, 0xE6, 0xBB}
	},
	{
		2, {0xEF, 0x77},
		3, {0xE4, 0xA6, 0xB0},
		2, {0xB0, 0x49},
		0, {0x00}
	},
	{
		2, {0xEF, 0x78},
		3, {0xE9, 0x97, 0x9A},
		2, {0xDA, 0x95},
		3, {0x8F, 0xE6, 0xBD}
	},
	{
		2, {0xEF, 0x79},
		3, {0xE9, 0x97, 0x9E},
		2, {0xDE, 0x95},
		3, {0x8F, 0xE6, 0xBF}
	},
	{
		2, {0xEF, 0x7A},
		3, {0xE9, 0x99, 0x98},
		2, {0x58, 0x96},
		3, {0x8F, 0xE6, 0xD8}
	},
	{
		2, {0xEF, 0x7B},
		3, {0xE9, 0x9A, 0x84},
		2, {0x84, 0x96},
		3, {0x8F, 0xE6, 0xE3}
	},
	{
		2, {0xEF, 0x7C},
		3, {0xEF, 0xA7, 0x9C},
		2, {0xDC, 0xF9},
		0, {0x00}
	},
	{
		2, {0xEF, 0x7D},
		3, {0xE9, 0x9A, 0x9D},
		2, {0x9D, 0x96},
		3, {0x8F, 0xE6, 0xE8}
	},
	{
		2, {0xEF, 0x7E},
		3, {0xE9, 0x9A, 0xA4},
		2, {0xA4, 0x96},
		3, {0x8F, 0xE6, 0xEA}
	},
	{
		2, {0xEF, 0x80},
		3, {0xE9, 0x9A, 0xA5},
		2, {0xA5, 0x96},
		3, {0x8F, 0xE6, 0xEB}
	},
	{
		2, {0xEF, 0x81},
		3, {0xE9, 0x9B, 0x92},
		2, {0xD2, 0x96},
		3, {0x8F, 0xE6, 0xF3}
	},
	{
		2, {0xEF, 0x82},
		3, {0xE9, 0x9B, 0x9E},
		2, {0xDE, 0x96},
		3, {0x8F, 0xE6, 0xF8}
	},
	{
		2, {0xEF, 0x83},
		3, {0xEF, 0xA9, 0xA8},
		2, {0x68, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEF, 0x84},
		3, {0xE9, 0x9B, 0xA9},
		2, {0xE9, 0x96},
		3, {0x8F, 0xE6, 0xFA}
	},
	{
		2, {0xEF, 0x85},
		3, {0xE9, 0x9B, 0xAF},
		2, {0xEF, 0x96},
		3, {0x8F, 0xE6, 0xFB}
	},
	{
		2, {0xEF, 0x86},
		3, {0xE9, 0x9C, 0xB3},
		2, {0x33, 0x97},
		3, {0x8F, 0xE7, 0xAC}
	},
	{
		2, {0xEF, 0x87},
		3, {0xE9, 0x9C, 0xBB},
		2, {0x3B, 0x97},
		0, {0x00}
	},
	{
		2, {0xEF, 0x88},
		3, {0xE9, 0x9D, 0x8D},
		2, {0x4D, 0x97},
		0, {0x00}
	},
	{
		2, {0xEF, 0x89},
		3, {0xE9, 0x9D, 0x8E},
		2, {0x4E, 0x97},
		3, {0x8F, 0xE7, 0xB0}
	},
	{
		2, {0xEF, 0x8A},
		3, {0xE9, 0x9D, 0x8F},
		2, {0x4F, 0x97},
		3, {0x8F, 0xE7, 0xB1}
	},
	{
		2, {0xEF, 0x8B},
		3, {0xE9, 0x9D, 0x9A},
		2, {0x5A, 0x97},
		3, {0x8F, 0xE7, 0xB5}
	},
	{
		2, {0xEF, 0x8C},
		3, {0xE9, 0x9D, 0xAE},
		2, {0x6E, 0x97},
		3, {0x8F, 0xE7, 0xBA}
	},
	{
		2, {0xEF, 0x8D},
		3, {0xE9, 0x9D, 0xB3},
		2, {0x73, 0x97},
		3, {0x8F, 0xE7, 0xBB}
	},
	{
		2, {0xEF, 0x8E},
		3, {0xE9, 0x9E, 0x95},
		2, {0x95, 0x97},
		3, {0x8F, 0xE7, 0xC4}
	},
	{
		2, {0xEF, 0x8F},
		3, {0xE9, 0x9E, 0xAE},
		2, {0xAE, 0x97},
		3, {0x8F, 0xE7, 0xCD}
	},
	{
		2, {0xEF, 0x90},
		3, {0xE9, 0x9E, 0xBA},
		2, {0xBA, 0x97},
		3, {0x8F, 0xE7, 0xD4}
	},
	{
		2, {0xEF, 0x91},
		3, {0xE9, 0x9F, 0x81},
		2, {0xC1, 0x97},
		3, {0x8F, 0xE7, 0xD8}
	},
	{
		2, {0xEF, 0x92},
		3, {0xE9, 0x9F, 0x89},
		2, {0xC9, 0x97},
		3, {0x8F, 0xE7, 0xDC}
	},
	{
		2, {0xEF, 0x93},
		3, {0xE9, 0x9F, 0x9E},
		2, {0xDE, 0x97},
		3, {0x8F, 0xE7, 0xE8}
	},
	{
		2, {0xEF, 0x94},
		3, {0xE9, 0x9F, 0x9B},
		2, {0xDB, 0x97},
		3, {0x8F, 0xE7, 0xEA}
	},
	{
		2, {0xEF, 0x95},
		3, {0xE9, 0x9F, 0xB4},
		2, {0xF4, 0x97},
		3, {0x8F, 0xE7, 0xEF}
	},
	{
		2, {0xEF, 0x96},
		3, {0xEF, 0xA9, 0xA9},
		2, {0x69, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEF, 0x97},
		3, {0xE9, 0xA0, 0x8A},
		2, {0x0A, 0x98},
		3, {0x8F, 0xE7, 0xF4}
	},
	{
		2, {0xEF, 0x98},
		3, {0xE9, 0xA0, 0x9E},
		2, {0x1E, 0x98},
		3, {0x8F, 0xE7, 0xFB}
	},
	{
		2, {0xEF, 0x99},
		3, {0xE9, 0xA0, 0xAB},
		2, {0x2B, 0x98},
		3, {0x8F, 0xE8, 0xA1}
	},
	{
		2, {0xEF, 0x9A},
		3, {0xE9, 0xA0, 0xB0},
		2, {0x30, 0x98},
		3, {0x8F, 0xE8, 0xA4}
	},
	{
		2, {0xEF, 0x9B},
		3, {0xEF, 0xA9, 0xAA},
		2, {0x6A, 0xFA},
		0, {0x00}
	},
	{
		2, {0xEF, 0x9C},
		3, {0xE9, 0xA1, 0x92},
		2, {0x52, 0x98},
		3, {0x8F, 0xE8, 0xAE}
	},
	{
		2, {0xEF, 0x9D},
		3, {0xE9, 0xA1, 0x93},
		2, {0x53, 0x98},
		3, {0x8F, 0xE8, 0xAF}
	},
	{
		2, {0xEF, 0x9E},
		3, {0xE9, 0xA1, 0x96},
		2, {0x56, 0x98},
		3, {0x8F, 0xE8, 0xB0}
	},
	{
		2, {0xEF, 0x9F},
		3, {0xE9, 0xA1, 0x97},
		2, {0x57, 0x98},
		3, {0x8F, 0xE8, 0xB1}
	},
	{
		2, {0xEF, 0xA0},
		3, {0xE9, 0xA1, 0x99},
		2, {0x59, 0x98},
		3, {0x8F, 0xE8, 0xB2}
	},
	{
		2, {0xEF, 0xA1},
		3, {0xE9, 0xA1, 0x9A},
		2, {0x5A, 0x98},
		3, {0x8F, 0xE8, 0xB3}
	},
	{
		2, {0xEF, 0xA2},
		3, {0xEF, 0xA7, 0x90},
		2, {0xD0, 0xF9},
		0, {0x00}
	},
	{
		2, {0xEF, 0xA3},
		3, {0xE9, 0xA1, 0xA5},
		2, {0x65, 0x98},
		3, {0x8F, 0xE8, 0xB6}
	},
	{
		2, {0xEF, 0xA4},
		3, {0xE9, 0xA1, 0xAC},
		2, {0x6C, 0x98},
		3, {0x8F, 0xE8, 0xB9}
	},
	{
		2, {0xEF, 0xA5},
		3, {0xE9, 0xA2, 0xBA},
		2, {0xBA, 0x98},
		3, {0x8F, 0xE8, 0xC1}
	},
	{
		2, {0xEF, 0xA6},
		3, {0xE9, 0xA3, 0x88},
		2, {0xC8, 0x98},
		3, {0x8F, 0xE8, 0xC6}
	},
	{
		2, {0xEF, 0xA7},
		3, {0xE9, 0xA3, 0xA7},
		2, {0xE7, 0x98},
		3, {0x8F, 0xE8, 0xCC}
	},
	{
		2, {0xEF, 0xA8},
		3, {0xE9, 0xA5, 0x98},
		2, {0x58, 0x99},
		3, {0x8F, 0xE8, 0xF0}
	},
	{
		2, {0xEF, 0xA9},
		3, {0xE9, 0xA6, 0x9E},
		2, {0x9E, 0x99},
		0, {0x00}
	},
	{
		2, {0xEF, 0xAA},
		3, {0xE9, 0xA8, 0x82},
		2, {0x02, 0x9A},
		3, {0x8F, 0xE9, 0xB7}
	},
	{
		2, {0xEF, 0xAB},
		3, {0xE9, 0xA8, 0x83},
		2, {0x03, 0x9A},
		3, {0x8F, 0xE9, 0xB8}
	},
	{
		2, {0xEF, 0xAC},
		3, {0xE9, 0xA8, 0xA4},
		2, {0x24, 0x9A},
		3, {0x8F, 0xE9, 0xC3}
	},
	{
		2, {0xEF, 0xAD},
		3, {0xE9, 0xA8, 0xAD},
		2, {0x2D, 0x9A},
		3, {0x8F, 0xE9, 0xC5}
	},
	{
		2, {0xEF, 0xAE},
		3, {0xE9, 0xA8, 0xAE},
		2, {0x2E, 0x9A},
		3, {0x8F, 0xE9, 0xC6}
	},
	{
		2, {0xEF, 0xAF},
		3, {0xE9, 0xA8, 0xB8},
		2, {0x38, 0x9A},
		3, {0x8F, 0xE9, 0xCA}
	},
	{
		2, {0xEF, 0xB0},
		3, {0xE9, 0xA9, 0x8A},
		2, {0x4A, 0x9A},
		3, {0x8F, 0xE9, 0xCE}
	},
	{
		2, {0xEF, 0xB1},
		3, {0xE9, 0xA9, 0x8E},
		2, {0x4E, 0x9A},
		3, {0x8F, 0xE9, 0xD1}
	},
	{
		2, {0xEF, 0xB2},
		3, {0xE9, 0xA9, 0x92},
		2, {0x52, 0x9A},
		0, {0x00}
	},
	{
		2, {0xEF, 0xB3},
		3, {0xE9, 0xAA, 0xB6},
		2, {0xB6, 0x9A},
		3, {0x8F, 0xE9, 0xDD}
	},
	{
		2, {0xEF, 0xB4},
		3, {0xE9, 0xAB, 0x81},
		2, {0xC1, 0x9A},
		3, {0x8F, 0xE9, 0xE2}
	},
	{
		2, {0xEF, 0xB5},
		3, {0xE9, 0xAB, 0x83},
		2, {0xC3, 0x9A},
		3, {0x8F, 0xE9, 0xE3}
	},
	{
		2, {0xEF, 0xB6},
		3, {0xE9, 0xAB, 0x8E},
		2, {0xCE, 0x9A},
		3, {0x8F, 0xE9, 0xE6}
	},
	{
		2, {0xEF, 0xB7},
		3, {0xE9, 0xAB, 0x96},
		2, {0xD6, 0x9A},
		3, {0x8F, 0xE9, 0xEA}
	},
	{
		2, {0xEF, 0xB8},
		3, {0xE9, 0xAB, 0xB9},
		2, {0xF9, 0x9A},
		3, {0x8F, 0xE9, 0xF7}
	},
	{
		2, {0xEF, 0xB9},
		3, {0xE9, 0xAC, 0x82},
		2, {0x02, 0x9B},
		3, {0x8F, 0xE9, 0xFD}
	},
	{
		2, {0xEF, 0xBA},
		3, {0xE9, 0xAC, 0x88},
		2, {0x08, 0x9B},
		3, {0x8F, 0xEA, 0xA3}
	},
	{
		2, {0xEF, 0xBB},
		3, {0xE9, 0xAC, 0xA0},
		2, {0x20, 0x9B},
		3, {0x8F, 0xEA, 0xAF}
	},
	{
		2, {0xEF, 0xBC},
		3, {0xE4, 0xB0, 0x97},
		2, {0x17, 0x4C},
		0, {0x00}
	},
	{
		2, {0xEF, 0xBD},
		3, {0xE9, 0xAC, 0xAD},
		2, {0x2D, 0x9B},
		3, {0x8F, 0xEA, 0xB2}
	},
	{
		2, {0xEF, 0xBE},
		3, {0xE9, 0xAD, 0x9E},
		2, {0x5E, 0x9B},
		3, {0x8F, 0xEA, 0xC1}
	},
	{
		2, {0xEF, 0xBF},
		3, {0xE9, 0xAD, 0xB9},
		2, {0x79, 0x9B},
		3, {0x8F, 0xEA, 0xD0}
	},
	{
		2, {0xEF, 0xC0},
		3, {0xE9, 0xAD, 0xA6},
		2, {0x66, 0x9B},
		3, {0x8F, 0xEA, 0xC5}
	},
	{
		2, {0xEF, 0xC1},
		3, {0xE9, 0xAD, 0xB2},
		2, {0x72, 0x9B},
		0, {0x00}
	},
	{
		2, {0xEF, 0xC2},
		3, {0xE9, 0xAD, 0xB5},
		2, {0x75, 0x9B},
		3, {0x8F, 0xEA, 0xCD}
	},
	{
		2, {0xEF, 0xC3},
		3, {0xE9, 0xAE, 0x84},
		2, {0x84, 0x9B},
		3, {0x8F, 0xEA, 0xD3}
	},
	{
		2, {0xEF, 0xC4},
		3, {0xE9, 0xAE, 0x8A},
		2, {0x8A, 0x9B},
		3, {0x8F, 0xEA, 0xD8}
	},
	{
		2, {0xEF, 0xC5},
		3, {0xE9, 0xAE, 0x8F},
		2, {0x8F, 0x9B},
		3, {0x8F, 0xEA, 0xDB}
	},
	{
		2, {0xEF, 0xC6},
		3, {0xE9, 0xAE, 0x9E},
		2, {0x9E, 0x9B},
		3, {0x8F, 0xEA, 0xE0}
	},
	{
		2, {0xEF, 0xC7},
		3, {0xE9, 0xAE, 0xA7},
		2, {0xA7, 0x9B},
		3, {0x8F, 0xEA, 0xE2}
	},
	{
		2, {0xEF, 0xC8},
		3, {0xE9, 0xAF, 0x81},
		2, {0xC1, 0x9B},
		3, {0x8F, 0xEA, 0xEE}
	},
	{
		2, {0xEF, 0xC9},
		3, {0xE9, 0xAF, 0x8E},
		2, {0xCE, 0x9B},
		3, {0x8F, 0xEA, 0xF1}
	},
	{
		2, {0xEF, 0xCA},
		3, {0xE9, 0xAF, 0xA5},
		2, {0xE5, 0x9B},
		3, {0x8F, 0xEA, 0xF7}
	},
	{
		2, {0xEF, 0xCB},
		3, {0xE9, 0xAF, 0xB8},
		2, {0xF8, 0x9B},
		3, {0x8F, 0xEA, 0xFE}
	},
	{
		2, {0xEF, 0xCC},
		3, {0xE9, 0xAF, 0xBD},
		2, {0xFD, 0x9B},
		3, {0x8F, 0xEB, 0xA3}
	},
	{
		2, {0xEF, 0xCD},
		3, {0xE9, 0xB0, 0x80},
		2, {0x00, 0x9C},
		3, {0x8F, 0xEB, 0xA5}
	},
	{
		2, {0xEF, 0xCE},
		3, {0xE9, 0xB0, 0xA3},
		2, {0x23, 0x9C},
		3, {0x8F, 0xEB, 0xB1}
	},
	{
		2, {0xEF, 0xCF},
		3, {0xE9, 0xB1, 0x81},
		2, {0x41, 0x9C},
		3, {0x8F, 0xEB, 0xBC}
	},
	{
		2, {0xEF, 0xD0},
		3, {0xE9, 0xB1, 0x8F},
		2, {0x4F, 0x9C},
		3, {0x8F, 0xEB, 0xC3}
	},
	{
		2, {0xEF, 0xD1},
		3, {0xE9, 0xB1, 0x90},
		2, {0x50, 0x9C},
		3, {0x8F, 0xEB, 0xC4}
	},
	{
		2, {0xEF, 0xD2},
		3, {0xE9, 0xB1, 0x93},
		2, {0x53, 0x9C},
		3, {0x8F, 0xEB, 0xC5}
	},
	{
		2, {0xEF, 0xD3},
		3, {0xE9, 0xB1, 0xA3},
		2, {0x63, 0x9C},
		3, {0x8F, 0xEB, 0xCD}
	},
	{
		2, {0xEF, 0xD4},
		3, {0xE9, 0xB1, 0xA5},
		2, {0x65, 0x9C},
		0, {0x00}
	},
	{
		2, {0xEF, 0xD5},
		3, {0xE9, 0xB1, 0xB7},
		2, {0x77, 0x9C},
		3, {0x8F, 0xEB, 0xD7}
	},
	{
		2, {0xEF, 0xD6},
		3, {0xE9, 0xB4, 0x9D},
		2, {0x1D, 0x9D},
		3, {0x8F, 0xEB, 0xE3}
	},
	{
		2, {0xEF, 0xD7},
		3, {0xE9, 0xB4, 0x9E},
		2, {0x1E, 0x9D},
		3, {0x8F, 0xEB, 0xE4}
	},
	{
		2, {0xEF, 0xD8},
		3, {0xE9, 0xB5, 0x83},
		2, {0x43, 0x9D},
		3, {0x8F, 0xEB, 0xEF}
	},
	{
		2, {0xEF, 0xD9},
		3, {0xE9, 0xB5, 0x87},
		2, {0x47, 0x9D},
		3, {0x8F, 0xEB, 0xF0}
	},
	{
		2, {0xEF, 0xDA},
		3, {0xE9, 0xB5, 0x92},
		2, {0x52, 0x9D},
		0, {0x00}
	},
	{
		2, {0xEF, 0xDB},
		3, {0xE9, 0xB5, 0xA3},
		2, {0x63, 0x9D},
		3, {0x8F, 0xEB, 0xF5}
	},
	{
		2, {0xEF, 0xDC},
		3, {0xE9, 0xB5, 0xB0},
		2, {0x70, 0x9D},
		3, {0x8F, 0xEB, 0xFB}
	},
	{
		2, {0xEF, 0xDD},
		3, {0xE9, 0xB5, 0xBC},
		2, {0x7C, 0x9D},
		3, {0x8F, 0xEC, 0xA1}
	},
	{
		2, {0xEF, 0xDE},
		3, {0xE9, 0xB6, 0x8A},
		2, {0x8A, 0x9D},
		3, {0x8F, 0xEC, 0xA6}
	},
	{
		2, {0xEF, 0xDF},
		3, {0xE9, 0xB6, 0x96},
		2, {0x96, 0x9D},
		3, {0x8F, 0xEC, 0xAC}
	},
	{
		2, {0xEF, 0xE0},
		3, {0xE9, 0xB7, 0x80},
		2, {0xC0, 0x9D},
		0, {0x00}
	},
	{
		2, {0xEF, 0xE1},
		3, {0xE9, 0xB6, 0xAC},
		2, {0xAC, 0x9D},
		3, {0x8F, 0xEC, 0xB1}
	},
	{
		2, {0xEF, 0xE2},
		3, {0xE9, 0xB6, 0xBC},
		2, {0xBC, 0x9D},
		3, {0x8F, 0xEC, 0xB6}
	},
	{
		2, {0xEF, 0xE3},
		3, {0xE9, 0xB7, 0x97},
		2, {0xD7, 0x9D},
		3, {0x8F, 0xEC, 0xBF}
	},
	{
		2, {0xEF, 0xE4},
		4, {0xF0, 0xAA, 0x86, 0x90},
		4, {0x68, 0xD8, 0x90, 0xDD},
		0, {0x00}
	},
	{
		2, {0xEF, 0xE5},
		3, {0xE9, 0xB7, 0xA7},
		2, {0xE7, 0x9D},
		3, {0x8F, 0xEC, 0xC5}
	},
	{
		2, {0xEF, 0xE6},
		3, {0xE9, 0xB8, 0x87},
		2, {0x07, 0x9E},
		3, {0x8F, 0xEC, 0xCF}
	},
	{
		2, {0xEF, 0xE7},
		3, {0xE9, 0xB8, 0x95},
		2, {0x15, 0x9E},
		3, {0x8F, 0xEC, 0xD4}
	},
	{
		2, {0xEF, 0xE8},
		3, {0xE9, 0xB9, 0xBC},
		2, {0x7C, 0x9E},
		3, {0x8F, 0xEC, 0xDB}
	},
	{
		2, {0xEF, 0xE9},
		3, {0xE9, 0xBA, 0x9E},
		2, {0x9E, 0x9E},
		3, {0x8F, 0xEC, 0xE7}
	},
	{
		2, {0xEF, 0xEA},
		3, {0xE9, 0xBA, 0xA4},
		2, {0xA4, 0x9E},
		3, {0x8F, 0xEC, 0xE8}
	},
	{
		2, {0xEF, 0xEB},
		3, {0xE9, 0xBA, 0xAC},
		2, {0xAC, 0x9E},
		3, {0x8F, 0xEC, 0xEA}
	},
	{
		2, {0xEF, 0xEC},
		3, {0xE9, 0xBA, 0xAF},
		2, {0xAF, 0x9E},
		3, {0x8F, 0xEC, 0xEC}
	},
	{
		2, {0xEF, 0xED},
		3, {0xE9, 0xBA, 0xB4},
		2, {0xB4, 0x9E},
		3, {0x8F, 0xEC, 0xEF}
	},
	{
		2, {0xEF, 0xEE},
		3, {0xE9, 0xBA, 0xB5},
		2, {0xB5, 0x9E},
		3, {0x8F, 0xEC, 0xF0}
	},
	{
		2, {0xEF, 0xEF},
		3, {0xE9, 0xBB, 0x83},
		2, {0xC3, 0x9E},
		0, {0x00}
	},
	{
		2, {0xEF, 0xF0},
		3, {0xE9, 0xBB, 0x91},
		2, {0xD1, 0x9E},
		0, {0x00}
	},
	{
		2, {0xEF, 0xF1},
		3, {0xE9, 0xBC, 0x90},
		2, {0x10, 0x9F},
		3, {0x8F, 0xED, 0xA7}
	},
	{
		2, {0xEF, 0xF2},
		3, {0xE9, 0xBC, 0xB9},
		2, {0x39, 0x9F},
		3, {0x8F, 0xED, 0xBA}
	},
	{
		2, {0xEF, 0xF3},
		3, {0xE9, 0xBD, 0x97},
		2, {0x57, 0x9F},
		3, {0x8F, 0xED, 0xC8}
	},
	{
		2, {0xEF, 0xF4},
		3, {0xE9, 0xBE, 0x90},
		2, {0x90, 0x9F},
		3, {0x8F, 0xED, 0xD9}
	},
	{
		2, {0xEF, 0xF5},
		3, {0xE9, 0xBE, 0x94},
		2, {0x94, 0x9F},
		3, {0x8F, 0xED, 0xDC}
	},
	{
		2, {0xEF, 0xF6},
		3, {0xE9, 0xBE, 0x97},
		2, {0x97, 0x9F},
		3, {0x8F, 0xED, 0xDE}
	},
	{
		2, {0xEF, 0xF7},
		3, {0xE9, 0xBE, 0xA2},
		2, {0xA2, 0x9F},
		3, {0x8F, 0xED, 0xE1}
	},
	{
		2, {0xEF, 0xF8},
		3, {0xE5, 0xA7, 0xB8},
		2, {0xF8, 0x59},
		0, {0x00}
	},
	{
		2, {0xEF, 0xF9},
		3, {0xE5, 0xB1, 0x9B},
		2, {0x5B, 0x5C},
		0, {0x00}
	},
	{
		2, {0xEF, 0xFA},
		3, {0xE5, 0xB9, 0xB7},
		2, {0x77, 0x5E},
		0, {0x00}
	},
	{
		2, {0xEF, 0xFB},
		3, {0xE7, 0x98, 0xA6},
		2, {0x26, 0x76},
		3, {0x8F, 0xCD, 0xF7}
	},
	{
		2, {0xEF, 0xFC},
		3, {0xE7, 0xB9, 0xAB},
		2, {0x6B, 0x7E},
		3, {0x8F, 0xD4, 0xDA}
	},
};

int JaCode_DetectType(const unsigned int uiLength, const unsigned char* pucBinary) {
	int iRet = -1;
	unsigned int uiLocate;
	unsigned char aucData[3];
	unsigned char bBinary = 0;
	unsigned char bJa = 0;
	unsigned int uiSJIS = 0;
	unsigned int uiUTF8 = 0;
	unsigned int uiEUC = 0;

	if(uiLength == 0 || pucBinary == NULL) {
		return iRet;
	}

	/* BOM */
	for(uiLocate = 0; uiLocate < uiLength - 2; uiLocate++) {
		if(pucBinary[uiLocate] == JACODE_BOM_UTF8[0] && pucBinary[uiLocate + 1] == JACODE_BOM_UTF8[1] && pucBinary[uiLocate + 2] == JACODE_BOM_UTF8[2]) {
			iRet = 2;
			return iRet;
		}
		if((pucBinary[uiLocate] == JACODE_BOM_UTF16_LE[0] && pucBinary[uiLocate + 1] == JACODE_BOM_UTF16_LE[1])) {
			iRet = 3;
			return iRet;
		}
		if(uiLocate == uiLength - 3) {
			if((pucBinary[uiLocate + 1] == JACODE_BOM_UTF16_LE[0] && pucBinary[uiLocate + 2] == JACODE_BOM_UTF16_LE[1])) {
				iRet = 3;
				return iRet;
			}
		}
	}

	/* RAW UTF-16 */
	for(uiLocate = 0; uiLocate < uiLength; uiLocate++) {
		aucData[0] = pucBinary[uiLocate];
		if(aucData[0] <= 0x06 || aucData[0] == 0x7F || aucData[0] == 0xFF) {
			bBinary = 1;
			if(aucData[0] == 0x00 && uiLocate < uiLength - 1 && pucBinary[uiLocate + 1] <= 0x7F) {
				iRet = 3;
				return iRet;
			}
		}
	}
	if(bBinary) {
		return iRet;
	}

	/* ASCII */
	for(uiLocate = 0; uiLocate < uiLength; uiLocate++) {
		aucData[0] = pucBinary[uiLocate];
		if(aucData[0] == 0x1B || aucData[0] >= 0x80) {
			bJa = 1;
			break;
		}
	}
	if(!bJa) {
		iRet = 0;
		return iRet;
	}

	/* SJIS */
	for(uiLocate = 0; uiLocate < uiLength - 1; uiLocate++) {
		aucData[0] = pucBinary[uiLocate    ];
		aucData[1] = pucBinary[uiLocate + 1];
		if(((0x81 <= aucData[0] && aucData[0] <= 0x9F) || (0xE0 <= aucData[0] && aucData[0] <= 0xFC)) &&
			((0x40 <= aucData[1] && aucData[1] <= 0x7E) || (0x80 <= aucData[1] && aucData[1] <= 0xFC))) {
			uiSJIS += 2;
			uiLocate++;
		}
	}

	/* EUC */
	for(uiLocate = 0; uiLocate < uiLength - 1; uiLocate++) {
		aucData[0] = pucBinary[uiLocate    ];
		aucData[1] = pucBinary[uiLocate + 1];
		if(((0xA1 <= aucData[0] && aucData[0] <= 0xFE) && (0xA1 <= aucData[1] && aucData[1] <= 0xFE)) ||
			(aucData[0] == 0x8E && (0xA1 <= aucData[1] && aucData[1] <= 0xDF))) {
			uiEUC += 2;
			uiLocate++;
		} else if(uiLocate < uiLength - 2) {
			aucData[2] = pucBinary[uiLocate + 2];
			if(aucData[0] == 0x8F && (0xA1 <= aucData[1] && aucData[1] <= 0xFE) &&
				(0xA1 <= aucData[2] && aucData[2] <= 0xFE)) {
				uiEUC += 3;
				uiLocate += 2;
			}
		}
	}

	/* UTF-8 */
	for(uiLocate = 0; uiLocate < uiLength - 1; uiLocate++) {
		aucData[0] = pucBinary[uiLocate    ];
		aucData[1] = pucBinary[uiLocate + 1];
		if((0xC0 <= aucData[0] && aucData[0] <= 0xDF) && (0x80 <= aucData[1] && aucData[1] <= 0xBF)) {
			uiUTF8 += 2;
			uiLocate++;
		} else if(uiLocate < uiLength - 2) {
			aucData[2] = pucBinary[uiLocate + 2];
			if((0xE0 <= aucData[0] && aucData[0] <= 0xEF) && (0x80 <= aucData[1] && aucData[1] <= 0xBF) &&
				(0x80 <= aucData[2] && aucData[2] <= 0xBF)) {
				uiUTF8 += 3;
				uiLocate += 2;
			}
		}
	}

	if(uiSJIS > uiUTF8 && uiSJIS > uiEUC) {
		iRet = 1;
	} else if(uiUTF8 > uiSJIS && uiUTF8 > uiEUC) {
		iRet = 2;
	} else if(uiEUC > uiSJIS && uiEUC > uiUTF8) {
		iRet = 4;
	}

	return iRet;
}

unsigned int JaCode_SearchCode(const unsigned char ucCodeType, const unsigned char* pucCode) {
	unsigned char ucCode;
	unsigned char ucCodeSize;
	unsigned char bLoop = 1;
	unsigned char bMatch;
	unsigned char bBOM = 0;
	unsigned int uiSize = 0;
	unsigned int uiCode;
	unsigned int uiCount;
	unsigned int uiLocate = 0;

	if(ucCodeType > 4 || pucCode == NULL) {
		return 0;
	}

	if(ucCodeType != 3) {
		if(pucCode[0] == '\0') {
			return 0;
		}

		if(ucCodeType == 0) {
			return 1;
		}
	}

	if(pucCode[0] != 0x1B && pucCode[0] < 0x80) {
		return 1;
	}

	switch(ucCodeType) {
	case 2:
		if(pucCode[0] == JACODE_BOM_UTF8[0] && pucCode[1] == JACODE_BOM_UTF8[1] && pucCode[2] == JACODE_BOM_UTF8[2]) {
			uiLocate = 3;
		}
		break;
	case 3:
		if(pucCode[0] == JACODE_BOM_UTF16_LE[0] && pucCode[1] == JACODE_BOM_UTF16_LE[1]) {
			uiLocate = 2;
		}
		break;
	}

	for(uiCode = 0; uiCode < sizeof(tJaCodes) / sizeof(JaCode_t); uiCode++) {
		switch(ucCodeType) {
		case 1:
			FLASH_BYTEREAD(&ucCode, tJaCodes[uiCode].aucSJIS, 0);
			if(ucCode == pucCode[0]) {
				bMatch = 1;
				FLASH_BYTEREAD(&ucCodeSize, &tJaCodes[uiCode].ucSJISSize, 0);
				for(uiCount = 1; uiCount < ucCodeSize; uiCount++) {
					FLASH_BYTEREAD(&ucCode, tJaCodes[uiCode].aucSJIS, uiCount);
					if(ucCode != pucCode[uiCount]) {
						bMatch = 0;
						break;
					}
				}
				if(bMatch) {
					uiSize = ucCodeSize;
					bLoop = 0;
				}
			}
			break;
		case 2:
			FLASH_BYTEREAD(&ucCode, tJaCodes[uiCode].aucUTF8, 0);
			if(ucCode == pucCode[uiLocate]) {
				bMatch = 1;
				FLASH_BYTEREAD(&ucCodeSize, &tJaCodes[uiCode].ucUTF8Size, 0);
				for(uiCount = 1; uiCount < ucCodeSize; uiCount++) {
					FLASH_BYTEREAD(&ucCode, tJaCodes[uiCode].aucUTF8, uiCount);
					if(ucCode != pucCode[uiCount]) {
						bMatch = 0;
						break;
					}
				}
				if(bMatch) {
					uiSize = uiLocate + ucCodeSize;
					bLoop = 0;
				}
			}
			break;
		case 3:
			FLASH_BYTEREAD(&ucCode, tJaCodes[uiCode].aucUTF16, 0);
			if(ucCode == pucCode[uiLocate]) {
				bMatch = 1;
				FLASH_BYTEREAD(&ucCodeSize, &tJaCodes[uiCode].ucUTF16Size, 0);
				for(uiCount = 1; uiCount < ucCodeSize; uiCount++) {
					FLASH_BYTEREAD(&ucCode, tJaCodes[uiCode].aucUTF16, uiCount);
					if(ucCode != pucCode[uiCount]) {
						bMatch = 0;
						break;
					}
				}
				if(bMatch) {
					uiSize = uiLocate + ucCodeSize;
					bLoop = 0;
				}
			}
			break;
		case 4:
			FLASH_BYTEREAD(&ucCode, tJaCodes[uiCode].aucEUC, 0);
			if(ucCode == pucCode[0]) {
				bMatch = 1;
				FLASH_BYTEREAD(&ucCodeSize, &tJaCodes[uiCode].ucEUCSize, 0);
				for(uiCount = 1; uiCount < ucCodeSize; uiCount++) {
					FLASH_BYTEREAD(&ucCode, tJaCodes[uiCode].aucEUC, uiCount);
					if(ucCode != pucCode[uiCount]) {
						bMatch = 0;
						break;
					}
				}
				if(bMatch) {
					uiSize = ucCodeSize;
					bLoop = 0;
				}
			}
			break;
		}
		if(!bLoop) {
			break;
		}
	}

	return uiSize;
}

unsigned int JaCode_ConvertCode(unsigned int* puiOutCodeSize, unsigned char* pucOutCode, const unsigned char ucInCodeType, const unsigned char* pucInCode, const unsigned char ucOutCodeType) {
	unsigned int uiRet = 0;
	unsigned char aucCode[6];
	unsigned char bLoop = 1;
	unsigned char bMatch;
	unsigned char ucInSize, ucOutSize;
	unsigned int uiCode;
	unsigned int uiCount;
	unsigned int uiLocate = 0;

	if(puiOutCodeSize == NULL || pucOutCode == NULL || ucInCodeType > 4 || pucInCode == NULL || ucInCodeType > 4) {
		return uiRet;
	}

	if(ucInCodeType != 3) {
		if(pucInCode[0] == '\0') {
			*puiOutCodeSize = 0;
			uiRet = 0;
			return uiRet;
		}

		if(pucInCode[0] != 0x1B && pucInCode[0] < 0x80) {
			pucOutCode[0] = pucInCode[0];
			*puiOutCodeSize = 1;
			uiRet = 1;
			return uiRet;
		}
	}

	switch(ucInCodeType) {
	case 2:
		if(pucInCode[0] == JACODE_BOM_UTF8[0] && pucInCode[1] == JACODE_BOM_UTF8[1] && pucInCode[2] == JACODE_BOM_UTF8[2]) {
			uiLocate = 3;
		}
		break;
	case 3:
		if(pucInCode[0] == JACODE_BOM_UTF16_LE[0] && pucInCode[1] == JACODE_BOM_UTF16_LE[1]) {
			uiLocate = 2;
		}
		break;
	}

	for(uiCode = 0; uiCode < sizeof(tJaCodes) / sizeof(JaCode_t); uiCode++) {
		switch(ucInCodeType) {
		case 0:
			FLASH_BYTEREAD(&(aucCode[0]), tJaCodes[uiCode].aucSJIS, 0);
			if(pucInCode[0] == aucCode[0]) {
				bMatch = 1;
				FLASH_BYTEREAD(&ucInSize, (&tJaCodes[uiCode].ucSJISSize), 0);
				if(ucInSize == 1) {
					switch(ucOutCodeType) {
					case 1:
						pucOutCode[0] = aucCode[0];
						*puiOutCodeSize = 1;
						uiRet = 1;
						break;
					case 2:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucUTF8Size), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucUTF8, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = 2;
						}
						break;
					case 3:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucUTF16Size), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucUTF16, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = 2;
						}
						break;
					case 4:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucEUCSize), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucEUC, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = 2;
						}
						break;
					}
					bLoop = 0;
				} else {
					pucOutCode[0] = pucInCode[0];
					*puiOutCodeSize = 1;
					uiRet = 1;
					bLoop = 0;
				}
			}
			break;

		case 1:
			FLASH_BYTEREAD(&(aucCode[0]), tJaCodes[uiCode].aucSJIS, 0);
			if(pucInCode[0] == aucCode[0]) {
				bMatch = 1;
				FLASH_BYTEREAD(&ucInSize, (&tJaCodes[uiCode].ucSJISSize), 0);
				for(uiCount = 1; uiCount < ucInSize; uiCount++) {
					FLASH_BYTEREAD(&(aucCode[uiCount]), tJaCodes[uiCode].aucSJIS, uiCount);
					if(pucInCode[uiCount] != aucCode[uiCount]) {
						bMatch = 0;
						break;
					}
				}
				if(bMatch) {
					switch(ucOutCodeType) {
					case 1:
						pucOutCode[0] = aucCode[0];
						pucOutCode[1] = aucCode[1];
						*puiOutCodeSize = 2;
						uiRet = 2;
						break;
					case 2:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucUTF8Size), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucUTF8, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = 2;
						}
						break;
					case 3:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucUTF16Size), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucUTF16, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = 2;
						}
						break;
					case 4:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucEUCSize), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucEUC, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = 2;
						}
						break;
					}
					bLoop = 0;
				}
			}
			break;

		case 2:
			FLASH_BYTEREAD(&(aucCode[0]), tJaCodes[uiCode].aucUTF8, 0);
			if(pucInCode[uiLocate] == aucCode[0]) {
				bMatch = 1;
				FLASH_BYTEREAD(&ucInSize, (&tJaCodes[uiCode].ucUTF8Size), 0);
				for(uiCount = 1; uiCount < ucInSize; uiCount++) {
					FLASH_BYTEREAD(&(aucCode[uiCount]), tJaCodes[uiCode].aucUTF8, uiCount);
					if(pucInCode[uiLocate + uiCount] != aucCode[uiCount]) {
						bMatch = 0;
						break;
					}
				}
				if(bMatch) {
					switch(ucOutCodeType) {
					case 1:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucSJISSize), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucSJIS, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = uiLocate + ucInSize;
						}
						break;
					case 2:
						for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
							pucOutCode[uiCount] = aucCode[uiCount];
						}
						*puiOutCodeSize = ucOutSize;
						uiRet = uiLocate + ucInSize;
						break;
					case 3:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucUTF16Size), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucUTF16, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = uiLocate + ucInSize;
						}
						break;
					case 4:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucEUCSize), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucEUC, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = uiLocate + ucInSize;
						}
						break;
					}
					bLoop = 0;
				}
			}
			break;
				
		case 3:
			FLASH_BYTEREAD(&(aucCode[0]), tJaCodes[uiCode].aucUTF16, 0);
			if(pucInCode[uiLocate] == aucCode[0]) {
				bMatch = 1;
				FLASH_BYTEREAD(&ucInSize, (&tJaCodes[uiCode].ucUTF16Size), 0);
				for(uiCount = 1; uiCount < ucInSize; uiCount++) {
					FLASH_BYTEREAD(&(aucCode[uiLocate + uiCount]), tJaCodes[uiCode].aucUTF16, uiCount);
					if(pucInCode[uiCount] != aucCode[uiCount]) {
						bMatch = 0;
						break;
					}
				}
				if(bMatch) {
					switch(ucOutCodeType) {
					case 1:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucSJISSize), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucSJIS, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = uiLocate + ucInSize;
						}
						break;
					case 2:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucUTF8Size), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucUTF8, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = uiLocate + ucInSize;
						}
						break;
					case 3:
						for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
							pucOutCode[uiCount] = aucCode[uiCount];
						}
						*puiOutCodeSize = ucOutSize;
						uiRet = uiLocate + ucInSize;
						break;
					case 4:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucEUCSize), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucEUC, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = uiLocate + ucInSize;
						}
						break;
					}
					bLoop = 0;
				}
			}
			break;
				
		case 4:
			FLASH_BYTEREAD(&(aucCode[0]), tJaCodes[uiCode].aucUTF16, 0);
			if(pucInCode[0] == aucCode[0]) {
				bMatch = 1;
				FLASH_BYTEREAD(&ucInSize, (&tJaCodes[uiCode].ucEUCSize), 0);
				for(uiCount = 1; uiCount < ucInSize; uiCount++) {
					FLASH_BYTEREAD(&(aucCode[uiCount]), tJaCodes[uiCode].aucEUC, uiCount);
					if(pucInCode[uiCount] != aucCode[uiCount]) {
						bMatch = 0;
						break;
					}
				}
				if(bMatch) {
					switch(ucOutCodeType) {
					case 1:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucSJISSize), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucSJIS, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = ucInSize;
						}
						break;
					case 2:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucUTF8Size), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucUTF8, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = ucInSize;
						}
						break;
					case 3:
						FLASH_BYTEREAD(&ucOutSize, (&tJaCodes[uiCode].ucUTF16Size), 0);
						if(ucOutSize == 0) {
							*puiOutCodeSize = 0;
						} else {
							for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
								FLASH_BYTEREAD(&(pucOutCode[uiCount]), tJaCodes[uiCode].aucUTF16, uiCount);
							}
							*puiOutCodeSize = ucOutSize;
							uiRet = ucInSize;
						}
						break;
					case 4:
						for(uiCount = 0; uiCount < ucOutSize; uiCount++) {
							pucOutCode[uiCount] = aucCode[uiCount];
						}
						*puiOutCodeSize = ucOutSize;
						uiRet = ucInSize;
						break;
					}
					bLoop = 0;
				}
			}
			break;
		}

		if(!bLoop) {
			break;
		}
	}

	return uiRet;
}

unsigned int JaCode_GetStringSize(const unsigned char ucStringType, const unsigned char* pucString) {
	unsigned int uiLocate = 0;
	unsigned int uiCodeSize;

	if(ucStringType > 4 || pucString == NULL) {
		return 0;
	}

	while(1) {
		if(ucStringType == 0) {
			if(pucString[uiLocate] == '\0') {
				break;
			}
			uiCodeSize = 1;
		} else {
			uiCodeSize = JaCode_SearchCode(ucStringType, &pucString[uiLocate]);
			if(uiCodeSize == 0) {
				if(pucString[uiLocate] == '\0') {
					break;
				}
				uiCodeSize = 1;
			}
		}
		uiLocate += uiCodeSize;
	}

	return uiLocate;
}

unsigned int JaCode_GetCharCount(const unsigned char ucStringType, const unsigned char* pucString) {
	unsigned int uiLocate = 0;
	unsigned int uiCharCount = 0;
	unsigned int uiCodeSize;

	if(ucStringType > 4 || pucString == NULL) {
		return 0;
	}

	while(1) {
		if(ucStringType == 0) {
			if(pucString[uiLocate] == '\0') {
				break;
			}
			uiLocate++;
			uiCharCount++;
		} else {
			uiCodeSize = JaCode_SearchCode(ucStringType, &pucString[uiLocate]);
			if(uiCodeSize == 0) {
				if(pucString[uiLocate] == '\0') {
					break;
				}
				uiCodeSize = 1;
			}
			uiLocate += uiCodeSize;
			uiCharCount++;
		}
	}

	return uiCharCount;
}

unsigned int JaCode_GetConvertStringSize(unsigned int* puiOutStringSize, const unsigned char ucInStringType, const unsigned char* pucInString, const unsigned char ucOutStringType) {
	unsigned int uiLocate = 0;
	unsigned int uiOutStringSize = 0;
	unsigned int uiInCodeSize, uiOutCodeSize;
	unsigned char aucCode[6];

	if(puiOutStringSize == NULL || ucInStringType > 4 || pucInString == NULL || ucOutStringType > 4) {
		return 0;
	}

	while(1) {
		if(ucOutStringType == 0) {
			if(pucInString[uiLocate] == '\0') {
				break;
			}
			uiLocate++;
			uiOutStringSize++;
		} else {
			uiInCodeSize = JaCode_ConvertCode(&uiOutCodeSize, aucCode, ucInStringType, &pucInString[uiLocate], ucOutStringType);
			if(uiInCodeSize == 0) {
				if(pucInString[uiLocate] == '\0') {
					break;
				}
				uiInCodeSize = 1;
				uiOutCodeSize = 1;
			}
			uiLocate += uiInCodeSize;
			uiOutStringSize += uiOutCodeSize;
		}
	}
	*puiOutStringSize = uiOutStringSize;

	return uiLocate;
}

unsigned int JaCode_ConvertString(unsigned int* puiOutStringSize, unsigned char* pucOutString, const unsigned char ucInStringType, const unsigned char* pucInString, const unsigned char ucOutStringType) {
	unsigned int uiInLocate = 0;
	unsigned int uiOutLocate = 0;
	unsigned int uiInCodeSize, uiOutCodeSize;

	if(puiOutStringSize == NULL || pucOutString == NULL || ucInStringType > 4 || pucInString == NULL || ucOutStringType > 4) {
		return 0;
	}

	while(1) {
		if(ucOutStringType == 0) {
			if(pucInString[uiInLocate] == '\0') {
				break;
			}
			pucOutString[uiOutLocate] = pucInString[uiInLocate];
			uiInLocate++;
			uiOutLocate++;
		} else {
			uiInCodeSize = JaCode_ConvertCode(&uiOutCodeSize, &pucOutString[uiOutLocate], ucInStringType, &pucInString[uiInLocate], ucOutStringType);
			if(uiInCodeSize == 0) {
				if(pucInString[uiInLocate] == '\0') {
					break;
				}
				uiInCodeSize = 1;
				uiOutCodeSize = 1;
			}
			uiInLocate += uiInCodeSize;
			uiOutLocate += uiOutCodeSize;
		}
	}
	pucOutString[uiOutLocate] = '\0';
	*puiOutStringSize = uiOutLocate;

	return uiInLocate;
}

unsigned int JaCode_AddBOMToString(unsigned char* pucOutString, const unsigned char ucInStringType, const unsigned char* pucInString) {
	unsigned int uiLocate = 0;
	unsigned int uiStringSize;

	if(pucOutString == NULL || ucInStringType > 4 || pucInString == NULL) {
		return 0;
	}

	switch(ucInStringType) {
	case 2:
		pucOutString[0] = JACODE_BOM_UTF8[0];
		pucOutString[1] = JACODE_BOM_UTF8[1];
		pucOutString[2] = JACODE_BOM_UTF8[2];
		uiLocate = 3;
		break;
	case 3:
		pucOutString[1] = JACODE_BOM_UTF16_LE[0];
		pucOutString[2] = JACODE_BOM_UTF16_LE[1];
		uiLocate = 2;
		break;
	}

	JaCode_ConvertString(&uiStringSize, &pucOutString[uiLocate], ucInStringType, pucInString, ucInStringType);

	return uiStringSize;
}


